@@ -2,7 +2,8 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015-2016 Mario Luzeiro <mrluzeiro@ua.pt>
- * Copyright (C) 2015-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2023 CERN
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,24 +24,25 @@
  */
 
 #include "render_3d_opengl.h"
-#include "opengl_utils.h"
 #include <board.h>
 #include <footprint.h>
 #include "../../3d_math.h"
+#include "convert_basic_shapes_to_polygon.h"
+#include <lset.h>
 #include <trigo.h>
 #include <project.h>
-#include <profile.h>        // To use GetRunningMicroSecs or another profiling utility
+#include <core/profile.h>        // To use GetRunningMicroSecs or another profiling utility
 #include <fp_lib_table.h>
-#include <eda_3d_canvas.h>
 #include <eda_3d_viewer_frame.h>
+#include <project_pcb.h>
 
 
-void RENDER_3D_OPENGL::addObjectTriangles( const FILLED_CIRCLE_2D* aFilledCircle,
+void RENDER_3D_OPENGL::addObjectTriangles( const FILLED_CIRCLE_2D* aCircle,
                                            TRIANGLE_DISPLAY_LIST* aDstLayer, float aZtop,
                                            float aZbot )
 {
-    const SFVEC2F& center = aFilledCircle->GetCenter();
-    const float radius = aFilledCircle->GetRadius() * 2.0f; // Double because the render triangle
+    const SFVEC2F& center = aCircle->GetCenter();
+    const float radius = aCircle->GetRadius() * 2.0f; // Double because the render triangle
 
     // This is a small adjustment to the circle texture
     const float texture_factor = ( 8.0f / (float) SIZE_OF_CIRCLE_TEXTURE ) + 1.0f;
@@ -302,7 +304,7 @@ OPENGL_RENDER_LIST* RENDER_3D_OPENGL::generateHoles( const LIST_OBJECT2D& aListH
 
 OPENGL_RENDER_LIST* RENDER_3D_OPENGL::generateLayerList( const BVH_CONTAINER_2D* aContainer,
                                                          const SHAPE_POLY_SET* aPolyList,
-                                                         PCB_LAYER_ID aLayerId,
+                                                         PCB_LAYER_ID aLayer,
                                                          const BVH_CONTAINER_2D* aThroughHoles )
 {
     if( aContainer == nullptr )
@@ -316,7 +318,7 @@ OPENGL_RENDER_LIST* RENDER_3D_OPENGL::generateLayerList( const BVH_CONTAINER_2D*
     float layer_z_bot = 0.0f;
     float layer_z_top = 0.0f;
 
-    getLayerZPos( aLayerId, layer_z_top, layer_z_bot );
+    getLayerZPos( aLayer, layer_z_top, layer_z_bot );
 
     // Calculate an estimation for the nr of triangles based on the nr of objects
     unsigned int nrTrianglesEstimation = listObject2d.size() * 8;
@@ -375,12 +377,12 @@ OPENGL_RENDER_LIST* RENDER_3D_OPENGL::generateLayerList( const BVH_CONTAINER_2D*
 }
 
 
-OPENGL_RENDER_LIST* RENDER_3D_OPENGL::generateEmptyLayerList( PCB_LAYER_ID aLayerId )
+OPENGL_RENDER_LIST* RENDER_3D_OPENGL::generateEmptyLayerList( PCB_LAYER_ID aLayer )
 {
     float layer_z_bot = 0.0f;
     float layer_z_top = 0.0f;
 
-    getLayerZPos( aLayerId, layer_z_top, layer_z_bot );
+    getLayerZPos( aLayer, layer_z_top, layer_z_bot );
 
     TRIANGLE_DISPLAY_LIST* layerTriangles = new TRIANGLE_DISPLAY_LIST( 1 );
 
@@ -453,7 +455,7 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
 
     OBJECT_2D_STATS::Instance().ResetStats();
 
-    unsigned stats_startReloadTime = GetRunningMicroSecs();
+    int64_t stats_startReloadTime = GetRunningMicroSecs();
 
     m_boardAdapter.InitSettings( aStatusReporter, aWarningReporter );
 
@@ -464,28 +466,22 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
         aStatusReporter->Report( _( "Load OpenGL: board" ) );
 
     // Create Board
-    m_board = createBoard( m_boardAdapter.GetBoardPoly(), &m_boardAdapter.GetThroughHoleIds() );
+    m_board = createBoard( m_boardAdapter.GetBoardPoly(), &m_boardAdapter.GetTH_IDs() );
 
-    if( m_boardAdapter.m_Cfg->m_Render.realistic )
-    {
-        m_antiBoardPolys.RemoveAllContours();
-        m_antiBoardPolys.NewOutline();
-        m_antiBoardPolys.Append( VECTOR2I( -INT_MAX/2, -INT_MAX/2 ) );
-        m_antiBoardPolys.Append( VECTOR2I(  INT_MAX/2, -INT_MAX/2 ) );
-        m_antiBoardPolys.Append( VECTOR2I(  INT_MAX/2,  INT_MAX/2 ) );
-        m_antiBoardPolys.Append( VECTOR2I( -INT_MAX/2,  INT_MAX/2 ) );
-        m_antiBoardPolys.Outline( 0 ).SetClosed( true );
+    m_antiBoardPolys.RemoveAllContours();
+    m_antiBoardPolys.NewOutline();
+    m_antiBoardPolys.Append( VECTOR2I( -INT_MAX/2, -INT_MAX/2 ) );
+    m_antiBoardPolys.Append( VECTOR2I(  INT_MAX/2, -INT_MAX/2 ) );
+    m_antiBoardPolys.Append( VECTOR2I(  INT_MAX/2,  INT_MAX/2 ) );
+    m_antiBoardPolys.Append( VECTOR2I( -INT_MAX/2,  INT_MAX/2 ) );
+    m_antiBoardPolys.Outline( 0 ).SetClosed( true );
 
-        m_antiBoardPolys.BooleanSubtract( m_boardAdapter.GetBoardPoly(),
-                                          SHAPE_POLY_SET::PM_STRICTLY_SIMPLE );
-        m_antiBoard = createBoard( m_antiBoardPolys );
-    }
+    m_antiBoardPolys.BooleanSubtract( m_boardAdapter.GetBoardPoly() );
+    m_antiBoard = createBoard( m_antiBoardPolys );
 
     SHAPE_POLY_SET board_poly_with_holes = m_boardAdapter.GetBoardPoly().CloneDropTriangulation();
-    board_poly_with_holes.BooleanSubtract( m_boardAdapter.GetThroughHoleOdPolys(),
-                                           SHAPE_POLY_SET::PM_FAST );
-    board_poly_with_holes.BooleanSubtract( m_boardAdapter.GetOuterNonPlatedThroughHolePoly(),
-                                           SHAPE_POLY_SET::PM_FAST );
+    board_poly_with_holes.BooleanSubtract( m_boardAdapter.GetTH_ODPolys() );
+    board_poly_with_holes.BooleanSubtract( m_boardAdapter.GetNPTH_ODPolys() );
 
     m_boardWithHoles = createBoard( board_poly_with_holes );
 
@@ -496,28 +492,21 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
     if( aStatusReporter )
         aStatusReporter->Report( _( "Load OpenGL: holes and vias" ) );
 
-    SHAPE_POLY_SET outerPolyTHT = m_boardAdapter.GetThroughHoleOdPolys().CloneDropTriangulation();
+    SHAPE_POLY_SET outerPolyTHT = m_boardAdapter.GetTH_ODPolys().CloneDropTriangulation();
 
-    if( m_boardAdapter.m_Cfg->m_Render.realistic )
+    outerPolyTHT.BooleanIntersection( m_boardAdapter.GetBoardPoly() );
+
+    m_outerThroughHoles = generateHoles( m_boardAdapter.GetTH_ODs().GetList(), outerPolyTHT,
+                                         1.0f, 0.0f, false, &m_boardAdapter.GetTH_IDs() );
+
+    m_outerViaThroughHoles = generateHoles( m_boardAdapter.GetViaTH_ODs().GetList(),
+                                            m_boardAdapter.GetViaTH_ODPolys(), 1.0f, 0.0f, false );
+
+    if( m_boardAdapter.m_Cfg->m_Render.clip_silk_on_via_annuli )
     {
-        outerPolyTHT.BooleanIntersection( m_boardAdapter.GetBoardPoly(),
-                                          SHAPE_POLY_SET::PM_STRICTLY_SIMPLE );
-    }
-
-    m_outerThroughHoles = generateHoles( m_boardAdapter.GetThroughHoleOds().GetList(),
-                                         outerPolyTHT, 1.0f, 0.0f, false,
-                                         &m_boardAdapter.GetThroughHoleIds() );
-
-    m_outerViaThroughHoles = generateHoles(
-            m_boardAdapter.GetThroughHoleViaOds().GetList(),
-            m_boardAdapter.GetThroughHoleViaOdPolys(), 1.0f, 0.0f, false );
-
-    if( m_boardAdapter.m_Cfg->m_Render.clip_silk_on_via_annulus &&
-        m_boardAdapter.m_Cfg->m_Render.realistic )
-    {
-        m_outerThroughHoleRings = generateHoles(
-                m_boardAdapter.GetThroughHoleAnnularRings().GetList(),
-                m_boardAdapter.GetThroughHoleAnnularRingPolys(), 1.0f, 0.0f, false );
+        m_outerThroughHoleRings = generateHoles( m_boardAdapter.GetViaAnnuli().GetList(),
+                                                 m_boardAdapter.GetViaAnnuliPolys(),
+                                                 1.0f, 0.0f, false );
     }
 
     const MAP_POLY& innerMapHoles = m_boardAdapter.GetHoleIdPolysMap();
@@ -532,28 +521,20 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
         float layer_z_bot = 0.0f;
         float layer_z_top = 0.0f;
 
-        for( const std::pair<const PCB_LAYER_ID, SHAPE_POLY_SET*>& ii : outerMapHoles )
+        for( const auto& [ layer, poly ] : outerMapHoles )
         {
-            const PCB_LAYER_ID      layer_id  = ii.first;
-            const SHAPE_POLY_SET*   poly      = ii.second;
-            const BVH_CONTAINER_2D* container = map_holes.at( layer_id );
+            getLayerZPos( layer, layer_z_top, layer_z_bot );
 
-            getLayerZPos( layer_id, layer_z_top, layer_z_bot );
-
-            m_outerLayerHoles[layer_id] = generateHoles( container->GetList(), *poly,
-                                                         layer_z_top, layer_z_bot, false );
+            m_outerLayerHoles[layer] = generateHoles( map_holes.at( layer )->GetList(), *poly,
+                                                      layer_z_top, layer_z_bot, false );
         }
 
-        for( const std::pair<const PCB_LAYER_ID, SHAPE_POLY_SET*>& ii : innerMapHoles )
+        for( const auto& [ layer, poly ] : innerMapHoles )
         {
-            const PCB_LAYER_ID      layer_id  = ii.first;
-            const SHAPE_POLY_SET*   poly      = ii.second;
-            const BVH_CONTAINER_2D* container = map_holes.at( layer_id );
+            getLayerZPos( layer, layer_z_top, layer_z_bot );
 
-            getLayerZPos( layer_id, layer_z_top, layer_z_bot );
-
-            m_innerLayerHoles[layer_id] = generateHoles( container->GetList(), *poly,
-                                                         layer_z_top, layer_z_bot, false );
+            m_innerLayerHoles[layer] = generateHoles( map_holes.at( layer )->GetList(), *poly,
+                                                      layer_z_top, layer_z_bot, false );
         }
     }
 
@@ -564,107 +545,96 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
     if( aStatusReporter )
         aStatusReporter->Report( _( "Load OpenGL: layers" ) );
 
-    const MAP_POLY& map_poly = m_boardAdapter.GetPolyMap();
+    std::bitset<LAYER_3D_END> visibilityFlags = m_boardAdapter.GetVisibleLayers();
+    const MAP_POLY&           map_poly = m_boardAdapter.GetPolyMap();
+    wxString                  msg;
 
-    for( const std::pair<const PCB_LAYER_ID, BVH_CONTAINER_2D*>& ii : m_boardAdapter.GetLayerMap() )
+    for( const auto& [ layer, container2d ] : m_boardAdapter.GetLayerMap() )
     {
-        const PCB_LAYER_ID layer_id = ii.first;
-
-        if( !m_boardAdapter.Is3dLayerEnabled( layer_id ) )
+        if( !m_boardAdapter.Is3dLayerEnabled( layer, visibilityFlags ) )
             continue;
 
         if( aStatusReporter )
         {
-            aStatusReporter->Report( wxString::Format( _( "Load OpenGL layer %d" ),
-                                                       (int) layer_id ) );
+            msg = m_boardAdapter.GetBoard()->GetLayerName( layer );
+            aStatusReporter->Report( wxString::Format( _( "Load OpenGL layer %s" ), msg ) );
         }
 
-        const BVH_CONTAINER_2D* container2d = ii.second;
-
         SHAPE_POLY_SET polyListSubtracted;
         SHAPE_POLY_SET* polyList = nullptr;
 
         // Load the vertical (Z axis) component of shapes
 
-        if( map_poly.find( layer_id ) != map_poly.end() )
+        if( m_boardAdapter.m_Cfg->m_Render.opengl_copper_thickness )
         {
-            polyListSubtracted = *map_poly.at( layer_id );;
-
-            if( m_boardAdapter.m_Cfg->m_Render.realistic )
+            if( map_poly.find( layer ) != map_poly.end() )
             {
-                polyListSubtracted.BooleanIntersection( m_boardAdapter.GetBoardPoly(),
-                                                        SHAPE_POLY_SET::PM_FAST );
+                polyListSubtracted = *map_poly.at( layer );
 
-                if( layer_id != B_Mask && layer_id != F_Mask )
+                if( LSET::PhysicalLayersMask().test( layer ) )
                 {
-                    polyListSubtracted.BooleanSubtract( m_boardAdapter.GetThroughHoleOdPolys(),
-                                                        SHAPE_POLY_SET::PM_FAST );
-                    polyListSubtracted.BooleanSubtract( m_boardAdapter.GetOuterNonPlatedThroughHolePoly(),
-                                                        SHAPE_POLY_SET::PM_FAST );
+                    polyListSubtracted.BooleanIntersection( m_boardAdapter.GetBoardPoly() );
+                }
+
+                if( layer != B_Mask && layer != F_Mask )
+                {
+                    polyListSubtracted.BooleanSubtract( m_boardAdapter.GetTH_ODPolys() );
+                    polyListSubtracted.BooleanSubtract( m_boardAdapter.GetNPTH_ODPolys() );
                 }
 
                 if( m_boardAdapter.m_Cfg->m_Render.subtract_mask_from_silk )
                 {
-                    if( layer_id == B_SilkS && map_poly.find( B_Mask ) != map_poly.end() )
+                    if( layer == B_SilkS && map_poly.find( B_Mask ) != map_poly.end() )
                     {
-                        polyListSubtracted.BooleanSubtract( *map_poly.at( B_Mask ),
-                                                            SHAPE_POLY_SET::PM_FAST );
+                        polyListSubtracted.BooleanSubtract( *map_poly.at( B_Mask ) );
                     }
-                    else if( layer_id == F_SilkS && map_poly.find( F_Mask ) != map_poly.end() )
+                    else if( layer == F_SilkS && map_poly.find( F_Mask ) != map_poly.end() )
                     {
-                        polyListSubtracted.BooleanSubtract( *map_poly.at( F_Mask ),
-                                                            SHAPE_POLY_SET::PM_FAST );
+                        polyListSubtracted.BooleanSubtract( *map_poly.at( F_Mask ) );
                     }
                 }
-            }
 
-            polyList = &polyListSubtracted;
+                polyList = &polyListSubtracted;
+            }
         }
 
-        OPENGL_RENDER_LIST* oglList = generateLayerList( container2d, polyList, layer_id,
-                                                         &m_boardAdapter.GetThroughHoleIds() );
+        OPENGL_RENDER_LIST* oglList = generateLayerList( container2d, polyList, layer,
+                                                         &m_boardAdapter.GetTH_IDs() );
 
         if( oglList != nullptr )
-            m_layers[layer_id] = oglList;
-
+            m_layers[layer] = oglList;
     }
 
-    if( m_boardAdapter.m_Cfg->m_Render.renderPlatedPadsAsPlated
-            && m_boardAdapter.m_Cfg->m_Render.realistic )
+    if( m_boardAdapter.m_Cfg->m_Render.DifferentiatePlatedCopper() )
     {
-        const SHAPE_POLY_SET* frontPlatedPadPolys = m_boardAdapter.GetFrontPlatedPadPolys();
-        const SHAPE_POLY_SET* backPlatedPadPolys = m_boardAdapter.GetBackPlatedPadPolys();
+        const SHAPE_POLY_SET* frontPlatedPadAndGraphicPolys =
+                m_boardAdapter.GetFrontPlatedPadAndGraphicPolys();
+        const SHAPE_POLY_SET* backPlatedPadAndGraphicPolys =
+                m_boardAdapter.GetBackPlatedPadAndGraphicPolys();
 
-        if( frontPlatedPadPolys )
+        if( frontPlatedPadAndGraphicPolys )
         {
-            SHAPE_POLY_SET polySubtracted = frontPlatedPadPolys->CloneDropTriangulation();
-            polySubtracted.BooleanIntersection( m_boardAdapter.GetBoardPoly(),
-                                                SHAPE_POLY_SET::PM_FAST );
-            polySubtracted.BooleanSubtract( m_boardAdapter.GetThroughHoleOdPolys(),
-                                            SHAPE_POLY_SET::PM_FAST );
-            polySubtracted.BooleanSubtract( m_boardAdapter.GetOuterNonPlatedThroughHolePoly(),
-                                            SHAPE_POLY_SET::PM_FAST );
+            SHAPE_POLY_SET poly = frontPlatedPadAndGraphicPolys->CloneDropTriangulation();
+            poly.BooleanIntersection( m_boardAdapter.GetBoardPoly() );
+            poly.BooleanSubtract( m_boardAdapter.GetTH_ODPolys() );
+            poly.BooleanSubtract( m_boardAdapter.GetNPTH_ODPolys() );
 
-            m_platedPadsFront = generateLayerList( m_boardAdapter.GetPlatedPadsFront(),
-                                                   &polySubtracted, F_Cu );
+            m_platedPadsFront = generateLayerList( m_boardAdapter.GetPlatedPadsFront(), &poly,
+                                                   F_Cu );
 
             // An entry for F_Cu must exist in m_layers or we'll never look at m_platedPadsFront
             if( m_layers.count( F_Cu ) == 0 )
                 m_layers[F_Cu] = generateEmptyLayerList( F_Cu );
         }
 
-        if( backPlatedPadPolys )
+        if( backPlatedPadAndGraphicPolys )
         {
-            SHAPE_POLY_SET polySubtracted = backPlatedPadPolys->CloneDropTriangulation();
-            polySubtracted.BooleanIntersection( m_boardAdapter.GetBoardPoly(),
-                                                SHAPE_POLY_SET::PM_FAST );
-            polySubtracted.BooleanSubtract( m_boardAdapter.GetThroughHoleOdPolys(),
-                                            SHAPE_POLY_SET::PM_FAST );
-            polySubtracted.BooleanSubtract( m_boardAdapter.GetOuterNonPlatedThroughHolePoly(),
-                                            SHAPE_POLY_SET::PM_FAST );
+            SHAPE_POLY_SET poly = backPlatedPadAndGraphicPolys->CloneDropTriangulation();
+            poly.BooleanIntersection( m_boardAdapter.GetBoardPoly() );
+            poly.BooleanSubtract( m_boardAdapter.GetTH_ODPolys() );
+            poly.BooleanSubtract( m_boardAdapter.GetNPTH_ODPolys() );
 
-            m_platedPadsBack = generateLayerList( m_boardAdapter.GetPlatedPadsBack(),
-                                                  &polySubtracted, B_Cu );
+            m_platedPadsBack = generateLayerList( m_boardAdapter.GetPlatedPadsBack(), &poly, B_Cu );
 
             // An entry for B_Cu must exist in m_layers or we'll never look at m_platedPadsBack
             if( m_layers.count( B_Cu ) == 0 )
@@ -672,6 +642,15 @@ void RENDER_3D_OPENGL::reload( REPORTER* aStatusReporter, REPORTER* aWarningRepo
         }
     }
 
+    if( m_boardAdapter.m_Cfg->m_Render.show_off_board_silk )
+    {
+        if( const BVH_CONTAINER_2D* padsFront = m_boardAdapter.GetOffboardPadsFront() )
+            m_offboardPadsFront = generateLayerList( padsFront, nullptr, F_Cu );
+
+        if( const BVH_CONTAINER_2D* padsBack = m_boardAdapter.GetOffboardPadsBack() )
+            m_offboardPadsBack = generateLayerList( padsBack, nullptr, B_Cu );
+    }
+
     // Load 3D models
     if( aStatusReporter )
         aStatusReporter->Report( _( "Loading 3D models..." ) );
@@ -775,6 +754,9 @@ void RENDER_3D_OPENGL::generateViasAndPads()
             {
                 const PCB_VIA* via = static_cast<const PCB_VIA*>( track );
 
+                if( via->GetViaType() == VIATYPE::THROUGH )
+                    continue;   // handle with pad holes so castellation is taken into account
+
                 const float holediameter = via->GetDrillValue() * m_boardAdapter.BiuTo3dUnits();
                 const int nrSegments = m_boardAdapter.GetCircleSegmentCount( via->GetDrillValue() );
                 const float hole_inner_radius = holediameter / 2.0f;
@@ -804,14 +786,33 @@ void RENDER_3D_OPENGL::generateViasAndPads()
     }
 
 
-    if( m_boardAdapter.GetHoleCount() > 0 )
+    if( m_boardAdapter.GetHoleCount() > 0 || m_boardAdapter.GetViaCount() > 0 )
     {
-        SHAPE_POLY_SET tht_outer_holes_poly; // Stores the outer poly of the copper holes (the pad)
-        SHAPE_POLY_SET tht_inner_holes_poly; // Stores the inner poly of the copper holes (the hole)
+        SHAPE_POLY_SET tht_outer_holes_poly; // Stores the outer poly of the copper holes
+        SHAPE_POLY_SET tht_inner_holes_poly; // Stores the inner poly of the copper holes
 
         tht_outer_holes_poly.RemoveAllContours();
         tht_inner_holes_poly.RemoveAllContours();
 
+        // Insert through-via holes (vertical cylinders)
+        for( const PCB_TRACK* track : m_boardAdapter.GetBoard()->Tracks() )
+        {
+            if( track->Type() == PCB_VIA_T )
+            {
+                const PCB_VIA* via = static_cast<const PCB_VIA*>( track );
+
+                if( via->GetViaType() == VIATYPE::THROUGH )
+                {
+                    TransformCircleToPolygon( tht_outer_holes_poly, via->GetPosition(),
+                                              via->GetDrill() / 2 + platingThickness,
+                                              ARC_HIGH_DEF, ERROR_INSIDE );
+
+                    TransformCircleToPolygon( tht_inner_holes_poly, via->GetPosition(),
+                                              via->GetDrill() / 2, ARC_HIGH_DEF, ERROR_INSIDE );
+                }
+            }
+        }
+
         // Insert pads holes (vertical cylinders)
         for( const FOOTPRINT* footprint : m_boardAdapter.GetBoard()->Footprints() )
         {
@@ -834,10 +835,9 @@ void RENDER_3D_OPENGL::generateViasAndPads()
         }
 
         // Subtract the holes
-        tht_outer_holes_poly.BooleanSubtract( tht_inner_holes_poly, SHAPE_POLY_SET::PM_FAST );
+        tht_outer_holes_poly.BooleanSubtract( tht_inner_holes_poly );
 
-        if( m_boardAdapter.m_Cfg->m_Render.realistic )
-            tht_outer_holes_poly.BooleanSubtract( m_antiBoardPolys, SHAPE_POLY_SET::PM_FAST );
+        tht_outer_holes_poly.BooleanSubtract( m_antiBoardPolys );
 
         CONTAINER_2D holesContainer;
 
@@ -894,16 +894,16 @@ void RENDER_3D_OPENGL::Load3dModelsIfNeeded()
     if( m_3dModelMap.size() > 0 )
         return;
 
-    wxFrame* frame = dynamic_cast<EDA_3D_VIEWER_FRAME*>( m_canvas->GetParent() );
-
-    if( frame )
+    if( wxFrame* frame = dynamic_cast<wxFrame*>( m_canvas->GetParent() ) )
     {
         STATUSBAR_REPORTER activityReporter( frame->GetStatusBar(),
                                              (int) EDA_3D_VIEWER_STATUSBAR::ACTIVITY );
         load3dModels( &activityReporter );
     }
     else
+    {
         load3dModels( nullptr );
+    }
 }
 
 
@@ -912,6 +912,10 @@ void RENDER_3D_OPENGL::load3dModels( REPORTER* aStatusReporter )
     if( !m_boardAdapter.GetBoard() )
         return;
 
+    // Building the 3D models late crashes on recent versions of macOS
+    // Unclear the exact mechanism, but as a workaround, just build them
+    // all the time.  See https://gitlab.com/kicad/code/kicad/-/issues/17198
+#ifndef __WXMAC__
     if( !m_boardAdapter.m_IsPreviewer
           && !m_boardAdapter.m_Cfg->m_Render.show_footprints_normal
           && !m_boardAdapter.m_Cfg->m_Render.show_footprints_insert
@@ -919,6 +923,7 @@ void RENDER_3D_OPENGL::load3dModels( REPORTER* aStatusReporter )
     {
         return;
     }
+#endif
 
     // Go for all footprints
     for( const FOOTPRINT* footprint : m_boardAdapter.GetBoard()->Footprints() )
@@ -932,8 +937,8 @@ void RENDER_3D_OPENGL::load3dModels( REPORTER* aStatusReporter )
             {
                 // FindRow() can throw an exception
                 const FP_LIB_TABLE_ROW* fpRow =
-                        m_boardAdapter.GetBoard()->GetProject()->PcbFootprintLibs()->FindRow(
-                                libraryName, false );
+                    PROJECT_PCB::PcbFootprintLibs( m_boardAdapter.GetBoard()->GetProject() )
+                            ->FindRow( libraryName, false );
 
                 if( fpRow )
                     footprintBasePath = fpRow->GetFullURI( true );
@@ -963,7 +968,9 @@ void RENDER_3D_OPENGL::load3dModels( REPORTER* aStatusReporter )
                 {
                     // It is not present, try get it from cache
                     const S3DMODEL* modelPtr =
-                            m_boardAdapter.Get3dCacheManager()->GetModel( fp_model.m_Filename, footprintBasePath );
+                            m_boardAdapter.Get3dCacheManager()->GetModel( fp_model.m_Filename,
+                                                                          footprintBasePath,
+                                                                          footprint );
 
                     // only add it if the return is not NULL
                     if( modelPtr )
