@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2016 Mario Luzeiro <mrluzeiro@ua.pt>
  * Copyright (C) 2015 Cirilo Bernardo <cirilo.bernardo@gmail.com>
- * Copyright (C) 2015-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -34,6 +34,7 @@
 #include <3d_canvas/eda_3d_canvas.h>
 #include <3d_viewer_id.h>
 #include <3d_rendering/track_ball.h>
+#include <wx/event.h>
 
 // Define min and max parameter values
 #define MAX_SCALE          10000.0
@@ -43,9 +44,8 @@
 #define SCALE_INCREMENT_FINE       0.02
 #define SCALE_INCREMENT            0.1
 
-#define ROTATION_INCREMENT 90            // in degrees, for spin button command
-#define ROTATION_INCREMENT_WHEEL 90      // in degrees, for mouse wheel command
-#define ROTATION_INCREMENT_WHEEL_FINE 1  // in degrees, for mouse wheel command
+#define ROTATION_INCREMENT 90      // in degrees
+#define ROTATION_INCREMENT_FINE 1  // in degrees
 
 #define OFFSET_INCREMENT_MM        0.5
 #define OFFSET_INCREMENT_MM_FINE   0.1
@@ -53,6 +53,7 @@
 #define OFFSET_INCREMENT_MIL       25.0
 #define OFFSET_INCREMENT_MIL_FINE  5.0
 
+wxDECLARE_EVENT( wxCUSTOM_PANEL_SHOWN_EVENT, wxCommandEvent );
 
 // Declared classes to create pointers
 class WX_INFOBAR;
@@ -61,8 +62,11 @@ class FILENAME_RESOLVER;
 class BOARD;
 class BOARD_ADAPTER;
 class FOOTPRINT;
+class NL_FOOTPRINT_PROPERTIES_PLUGIN;
 
-class PANEL_PREVIEW_3D_MODEL: public EDA_3D_BOARD_HOLDER, public TOOLS_HOLDER, public PANEL_PREVIEW_3D_MODEL_BASE
+#define PANEL_PREVIEW_3D_MODEL_ID  wxID_HIGHEST + 1244
+
+class PANEL_PREVIEW_3D_MODEL: public TOOLS_HOLDER, public PANEL_PREVIEW_3D_MODEL_BASE
 {
 public:
     PANEL_PREVIEW_3D_MODEL( wxWindow* aParent, PCB_BASE_FRAME* aFrame, FOOTPRINT* aFootprint,
@@ -78,8 +82,8 @@ public:
 
     wxWindow* GetToolCanvas() const override { return m_previewPane; }
 
-    BOARD_ADAPTER& GetAdapter() override { return m_boardAdapter; }
-    CAMERA& GetCurrentCamera() override { return m_currentCamera; }
+    BOARD_ADAPTER& GetAdapter() { return m_boardAdapter; }
+    CAMERA& GetCurrentCamera() { return m_currentCamera; }
 
     /**
      * Set the currently selected index in the model list so that the scale/rotation/offset
@@ -93,6 +97,12 @@ public:
      */
     void UpdateDummyFootprint( bool aRelaodRequired = true );
 
+    /**
+     * Get the dummy footprint that is used for previewing the 3D model.
+     * We use this to hold the temporary 3D model shapes.
+     */
+    FOOTPRINT* GetDummyFootprint() const { return m_dummyFootprint; }
+
 private:
     /**
      * Load 3D relevant settings from the user configuration
@@ -135,13 +145,12 @@ private:
 
     void onOpacitySlider( wxCommandEvent& event ) override;
 
-    void updateBoardThickness( wxCommandEvent& event ) override;
-
     void doIncrementScale( wxSpinEvent& aEvent, double aSign );
     void doIncrementRotation( wxSpinEvent& aEvent, double aSign );
     void doIncrementOffset( wxSpinEvent& aEvent, double aSign );
 
     void onUnitsChanged( wxCommandEvent& aEvent );
+    void onPanelShownEvent( wxCommandEvent& aEvent );
 
     wxString formatScaleValue( double aValue );
     wxString formatRotationValue( double aValue );
@@ -153,19 +162,24 @@ private:
         m_previewPane->Refresh();
     }
 
+    // turn ON or OFF options to show the board body. If OFF, solder paste, soldermask
+    // and board body are hidden, to allows a good view of the 3D model and its pads.
+    // Useful for 3D model placement
+	void setBodyStyleView( wxCommandEvent& event ) override;
+
 	void View3DLeft( wxCommandEvent& event ) override
     {
-        m_previewPane->SetView3D( ID_VIEW3D_LEFT );
+        m_previewPane->SetView3D( VIEW3D_TYPE::VIEW3D_LEFT );
     }
 
 	void View3DFront( wxCommandEvent& event ) override
     {
-        m_previewPane->SetView3D( ID_VIEW3D_FRONT );
+        m_previewPane->SetView3D( VIEW3D_TYPE::VIEW3D_FRONT );
     }
 
 	void View3DTop( wxCommandEvent& event ) override
     {
-        m_previewPane->SetView3D( ID_VIEW3D_TOP );
+        m_previewPane->SetView3D( VIEW3D_TYPE::VIEW3D_TOP );
     }
 
 	void View3DUpdate( wxCommandEvent& event ) override
@@ -174,22 +188,27 @@ private:
         m_previewPane->Refresh();
     }
 
+    void View3DSettings( wxCommandEvent& event ) override;
+
 	void View3DRight( wxCommandEvent& event ) override
     {
-        m_previewPane->SetView3D( ID_VIEW3D_RIGHT );
+        m_previewPane->SetView3D( VIEW3D_TYPE::VIEW3D_RIGHT );
     }
 
 	void View3DBack( wxCommandEvent& event ) override
     {
-        m_previewPane->SetView3D( ID_VIEW3D_BACK );
+        m_previewPane->SetView3D( VIEW3D_TYPE::VIEW3D_BACK );
     }
 
 	void View3DBottom( wxCommandEvent& event ) override
     {
-        m_previewPane->SetView3D( ID_VIEW3D_BOTTOM );
+        m_previewPane->SetView3D( VIEW3D_TYPE::VIEW3D_BOTTOM );
     }
 
+    void onModify();
+
 private:
+    PCB_BASE_FRAME*          m_parentFrame;
     EDA_3D_CANVAS*           m_previewPane;
     WX_INFOBAR*              m_infobar;
     BOARD_ADAPTER            m_boardAdapter;
@@ -200,10 +219,14 @@ private:
     FOOTPRINT*               m_dummyFootprint;
 
     std::vector<FP_3DMODEL>* m_parentModelList;
-    int                      m_selected;   /// Index into m_parentInfoList
+    int                      m_selected;            /// Index into m_parentInfoList
 
-    UNIT_BINDER              m_boardThickness;
     EDA_UNITS                m_userUnits;
+    bool                     m_bodyStyleShowAll;    /// true if the board body is show
+    /// The 3d viewer Render initial settings (must be saved and restored)
+    EDA_3D_VIEWER_SETTINGS::RENDER_SETTINGS m_initialRender;
+
+    std::unique_ptr<NL_FOOTPRINT_PROPERTIES_PLUGIN>     m_spaceMouse;
 };
 
 #endif  // PANEL_PREVIEW_3D_MODEL_H
