@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2021-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -22,26 +22,31 @@
 #include <bitmaps.h>
 #include <base_screen.h>
 #include <common.h>     // ExpandEnvVarSubstitutions
-#include <confirm.h>
 #include <core/arraydim.h>
 #include <dialogs/dialog_page_settings.h>
 #include <eda_draw_frame.h>
 #include <eda_item.h>
+#include <embedded_files.h>
+#include <filename_resolver.h>
 #include <gr_basic.h>
 #include <kiface_base.h>
 #include <macros.h>
-#include <math/util.h>      // for KiROUND, Clamp
+#include <math/util.h>      // for KiROUND
 #include <project.h>
 #include <tool/actions.h>
 #include <tool/tool_manager.h>
 #include <wildcards_and_files_ext.h>
 #include <drawing_sheet/ds_data_model.h>
 #include <drawing_sheet/ds_painter.h>
+#include <string_utils.h>
 #include <widgets/std_bitmap_button.h>
+#include <widgets/filedlg_open_embed_file.h>
 #include <wx/valgen.h>
 #include <wx/tokenzr.h>
 #include <wx/filedlg.h>
 #include <wx/dcmemory.h>
+#include <wx/msgdlg.h>
+#include <confirm.h>
 
 #define MAX_PAGE_EXAMPLE_SIZE 200
 
@@ -73,19 +78,21 @@ static const wxString pageFmts[] =
                                     // to be recognized in code
 };
 
-DIALOG_PAGES_SETTINGS::DIALOG_PAGES_SETTINGS( EDA_DRAW_FRAME* aParent, double aIuPerMils,
-                                              const wxSize& aMaxUserSizeMils ) :
+DIALOG_PAGES_SETTINGS::DIALOG_PAGES_SETTINGS( EDA_DRAW_FRAME* aParent,
+                                              EMBEDDED_FILES* aEmbeddedFiles, double aIuPerMils,
+                                              const VECTOR2D& aMaxUserSizeMils ) :
         DIALOG_PAGES_SETTINGS_BASE( aParent ),
         m_parent( aParent ),
         m_screen( m_parent->GetScreen() ),
         m_initialized( false ),
         m_pageBitmap( nullptr ),
         m_iuPerMils( aIuPerMils ),
+        m_embeddedFiles( aEmbeddedFiles ),
         m_customSizeX( aParent, m_userSizeXLabel, m_userSizeXCtrl, m_userSizeXUnits ),
         m_customSizeY( aParent, m_userSizeYLabel, m_userSizeYCtrl, m_userSizeYUnits )
 {
     m_projectPath = Prj().GetProjectPath();
-    m_browseButton->SetBitmap( KiBitmap( BITMAPS::small_folder ) );
+    m_browseButton->SetBitmap( KiBitmapBundle( BITMAPS::small_folder ) );
 
     m_maxPageSizeMils = aMaxUserSizeMils;
     m_tb = m_parent->GetTitleBlock();
@@ -112,6 +119,10 @@ DIALOG_PAGES_SETTINGS::DIALOG_PAGES_SETTINGS( EDA_DRAW_FRAME* aParent, double aI
         m_staticTextTitleBlock->SetLabel( _( "Title Block" ) );
     }
 
+    m_filenameResolver = new FILENAME_RESOLVER;
+    m_filenameResolver->SetProject( &Prj() );
+    m_filenameResolver->SetProgramBase( &Pgm() );
+
     SetupStandardButtons();
 
     Centre();
@@ -150,13 +161,13 @@ bool DIALOG_PAGES_SETTINGS::TransferDataToWindow()
 
     if( m_customFmt )
     {
-        m_customSizeX.SetValue( m_pageInfo.GetWidthMils() * m_iuPerMils );
-        m_customSizeY.SetValue( m_pageInfo.GetHeightMils() * m_iuPerMils );
+        m_customSizeX.SetDoubleValue( m_pageInfo.GetWidthMils() * m_iuPerMils );
+        m_customSizeY.SetDoubleValue( m_pageInfo.GetHeightMils() * m_iuPerMils );
     }
     else
     {
-        m_customSizeX.SetValue( m_pageInfo.GetCustomWidthMils() * m_iuPerMils );
-        m_customSizeY.SetValue( m_pageInfo.GetCustomHeightMils() * m_iuPerMils );
+        m_customSizeX.SetDoubleValue( m_pageInfo.GetCustomWidthMils() * m_iuPerMils );
+        m_customSizeY.SetDoubleValue( m_pageInfo.GetCustomHeightMils() * m_iuPerMils );
     }
 
     m_TextRevision->SetValue( m_tb.GetRevision() );
@@ -223,7 +234,7 @@ bool DIALOG_PAGES_SETTINGS::TransferDataFromWindow()
         m_screen->SetContentModified();
 
         if( LocalPrjConfigChanged() )
-            m_parent->SaveProjectSettings();
+            m_parent->OnModify();
 
         // Call the post processing (if any) after changes
         m_parent->OnPageSettingsChange();
@@ -459,24 +470,25 @@ void DIALOG_PAGES_SETTINGS::OnDateApplyClick( wxCommandEvent& event )
 
 bool DIALOG_PAGES_SETTINGS::SavePageSettings()
 {
-    bool success = false;
-
+    bool     success = false;
+    wxString msg;
     wxString fileName = GetWksFileName();
 
     if( fileName != BASE_SCREEN::m_DrawingSheetFileName )
     {
-        wxString fullFileName = DS_DATA_MODEL::ResolvePath( fileName, m_projectPath );
-
-        if( !fullFileName.IsEmpty() && !wxFileExists( fullFileName ) )
-        {
-            wxString msg;
-            msg.Printf( _( "Drawing sheet file '%s' not found." ), fullFileName );
-            wxMessageBox( msg );
-            return false;
-        }
+        wxString fullFileName = m_filenameResolver->ResolvePath( fileName, m_projectPath,
+                                                                 m_embeddedFiles );
 
         BASE_SCREEN::m_DrawingSheetFileName = fileName;
-        DS_DATA_MODEL::GetTheInstance().LoadDrawingSheet( fullFileName );
+
+        if( !DS_DATA_MODEL::GetTheInstance().LoadDrawingSheet( fullFileName, &msg ) )
+        {
+            DisplayErrorMessage( this,
+                                 wxString::Format( _( "Error loading drawing sheet '%s'." ),
+                                                   fullFileName ),
+                                 msg );
+        }
+
         m_localPrjConfigChanged = true;
     }
 
@@ -591,8 +603,10 @@ void DIALOG_PAGES_SETTINGS::UpdateDrawingSheetExample()
 {
     int lyWidth, lyHeight;
 
-    wxSize clamped_layout_size( Clamp( MIN_PAGE_SIZE_MILS, m_layout_size.x, m_maxPageSizeMils.x ),
-                                Clamp( MIN_PAGE_SIZE_MILS, m_layout_size.y, m_maxPageSizeMils.y ) );
+    VECTOR2D clamped_layout_size( std::clamp( m_layout_size.x, (double) MIN_PAGE_SIZE_MILS,
+                                              m_maxPageSizeMils.x ),
+                                  std::clamp( m_layout_size.y, (double) MIN_PAGE_SIZE_MILS,
+                                              m_maxPageSizeMils.y ) );
 
     double lyRatio = clamped_layout_size.x < clamped_layout_size.y ?
                         (double) clamped_layout_size.y / clamped_layout_size.x :
@@ -665,8 +679,7 @@ void DIALOG_PAGES_SETTINGS::UpdateDrawingSheetExample()
 
             if( m_parent->IsType( FRAME_SCH )
                 || m_parent->IsType( FRAME_SCH_SYMBOL_EDITOR )
-                || m_parent->IsType( FRAME_SCH_VIEWER )
-                || m_parent->IsType( FRAME_SCH_VIEWER_MODAL ) )
+                || m_parent->IsType( FRAME_SCH_VIEWER ) )
             {
                 COLOR4D color = renderSettings.GetLayerColor( LAYER_SCHEMATIC_DRAWINGSHEET );
                 renderSettings.SetLayerColor( LAYER_DRAWINGSHEET, color );
@@ -674,9 +687,9 @@ void DIALOG_PAGES_SETTINGS::UpdateDrawingSheetExample()
 
             GRFilledRect( &memDC, VECTOR2I( 0, 0 ), m_layout_size, 0, bgColor, bgColor );
 
-            PrintDrawingSheet( &renderSettings, pageDUMMY, emptyString, emptyString, m_tb,
-                               nullptr, m_screen->GetPageCount(), m_screen->GetPageNumber(), 1,
-                               &Prj(), wxEmptyString, m_screen->GetVirtualPageNumber() == 1 );
+            PrintDrawingSheet( &renderSettings, pageDUMMY, emptyString, emptyString, emptyString,
+                               m_tb, nullptr, m_screen->GetPageCount(), m_screen->GetPageNumber(),
+                               1, &Prj(), wxEmptyString, m_screen->GetVirtualPageNumber() == 1 );
 
             memDC.SelectObject( wxNullBitmap );
             m_PageLayoutExampleBitmap->SetBitmap( *m_pageBitmap );
@@ -744,8 +757,8 @@ void DIALOG_PAGES_SETTINGS::GetPageLayoutInfoFromDialog()
 
         wxASSERT( i != arrayDim(papers) );   // dialog UI match the above list?
 
-        VECTOR2I sz = pageInfo.GetSizeMils();
-        m_layout_size = wxSize( sz.x, sz.y );
+        VECTOR2D sz = pageInfo.GetSizeMils();
+        m_layout_size = VECTOR2D( sz.x, sz.y );
 
         // swap sizes to match orientation
         bool isPortrait = (bool) m_orientationComboBox->GetSelection();
@@ -753,7 +766,7 @@ void DIALOG_PAGES_SETTINGS::GetPageLayoutInfoFromDialog()
         if( ( isPortrait  && m_layout_size.x >= m_layout_size.y ) ||
             ( !isPortrait && m_layout_size.x <  m_layout_size.y ) )
         {
-            m_layout_size.Set( m_layout_size.y, m_layout_size.x );
+            std::swap( m_layout_size.x, m_layout_size.y );
         }
     }
 }
@@ -761,21 +774,22 @@ void DIALOG_PAGES_SETTINGS::GetPageLayoutInfoFromDialog()
 
 void DIALOG_PAGES_SETTINGS::GetCustomSizeMilsFromDialog()
 {
-    double customSizeX = (double) m_customSizeX.GetValue() / m_iuPerMils;
-    double customSizeY = (double) m_customSizeY.GetValue() / m_iuPerMils;
+    double customSizeX = (double) m_customSizeX.GetDoubleValue() / m_iuPerMils;
+    double customSizeY = (double) m_customSizeY.GetDoubleValue() / m_iuPerMils;
 
-    // Prepare to painless double -> int conversion.
-    customSizeX = Clamp( double( INT_MIN ), customSizeX, double( INT_MAX ) );
-    customSizeY = Clamp( double( INT_MIN ), customSizeY, double( INT_MAX ) );
-    m_layout_size = wxSize( KiROUND( customSizeX ), KiROUND( customSizeY ) );
+    // Ensure layout size can be converted to int coordinates later
+    customSizeX = std::clamp( customSizeX, double( INT_MIN ), double( INT_MAX ) );
+    customSizeY = std::clamp( customSizeY, double( INT_MIN ), double( INT_MAX ) );
+    m_layout_size = VECTOR2D( customSizeX, customSizeY );
 }
 
 
 void DIALOG_PAGES_SETTINGS::OnWksFileSelection( wxCommandEvent& event )
 {
     wxFileName fn = GetWksFileName();
-    wxString name = fn.GetFullName();
-    wxString path;
+    wxString   name = fn.GetFullName();
+    wxString   path;
+    wxString   msg;
 
     if( fn.IsAbsolute() )
     {
@@ -785,15 +799,20 @@ void DIALOG_PAGES_SETTINGS::OnWksFileSelection( wxCommandEvent& event )
     {
         wxFileName expanded( ExpandEnvVarSubstitutions( GetWksFileName(), &m_parentFrame->Prj() ) );
 
-         if( expanded.IsAbsolute() )
+        if( expanded.IsAbsolute() )
             path = expanded.GetPath();
         else
             path = m_projectPath;
     }
 
     // Display a file picker dialog
-    wxFileDialog fileDialog( this, _( "Select Drawing Sheet File" ), path, name,
-                             DrawingSheetFileWildcard(), wxFD_DEFAULT_STYLE|wxFD_FILE_MUST_EXIST );
+    FILEDLG_OPEN_EMBED_FILE customize;
+    wxFileDialog fileDialog( this, _( "Drawing Sheet File" ), path, name,
+                             FILEEXT::DrawingSheetFileWildcard(),
+                             wxFD_DEFAULT_STYLE | wxFD_FILE_MUST_EXIST );
+
+    if( m_embeddedFiles )
+        fileDialog.SetCustomizeHook( customize );
 
     if( fileDialog.ShowModal() != wxID_OK )
         return;
@@ -801,9 +820,16 @@ void DIALOG_PAGES_SETTINGS::OnWksFileSelection( wxCommandEvent& event )
     wxString fileName = fileDialog.GetPath();
     wxString shortFileName;
 
-    // Try to use a project-relative path first:
-    if( !m_projectPath.IsEmpty() && fileName.StartsWith( m_projectPath ) )
+    if( m_embeddedFiles && customize.GetEmbed() )
     {
+        fn.Assign( fileName );
+        EMBEDDED_FILES::EMBEDDED_FILE* result = m_embeddedFiles->AddFile( fn, false );
+        shortFileName = result->GetLink();
+        fileName = m_embeddedFiles->GetTemporaryFileName( result->name ).GetFullPath();
+    }
+    else if( !m_projectPath.IsEmpty() && fileName.StartsWith( m_projectPath ) )
+    {
+        // Try to use a project-relative path
         fn = wxFileName( fileName );
         fn.MakeRelativeTo( m_projectPath );
         shortFileName = fn.GetFullPath();
@@ -816,15 +842,21 @@ void DIALOG_PAGES_SETTINGS::OnWksFileSelection( wxCommandEvent& event )
 
     std::unique_ptr<DS_DATA_MODEL> ws = std::make_unique<DS_DATA_MODEL>();
 
-    if( ws->LoadDrawingSheet( fileName ) )
+    if( !ws->LoadDrawingSheet( fileName, &msg ) )
     {
-        delete m_drawingSheet;
-
-        m_drawingSheet = ws.release();
-
-        SetWksFileName( shortFileName );
-
-        GetPageLayoutInfoFromDialog();
-        UpdateDrawingSheetExample();
+        DisplayErrorMessage( this,
+                             wxString::Format( _( "Error loading drawing sheet '%s'.\n%s" ),
+                                               fileName ),
+                             msg );
+        return;
     }
+
+    delete m_drawingSheet;
+
+    m_drawingSheet = ws.release();
+
+    SetWksFileName( shortFileName );
+
+    GetPageLayoutInfoFromDialog();
+    UpdateDrawingSheetExample();
 }
