@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2004-2009 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2009 Dick Hollenbeck, dick@softplc.com
- * Copyright (C) 2009-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,27 +24,31 @@
  */
 
 #include <algorithm>
+#include <limits>
 
 #include <pgm_base.h>
 #include <eda_draw_frame.h>
 #include <bitmaps.h>
 #include <netclass.h>
-#include <confirm.h>
+#include <gal/painter.h>
 #include <grid_tricks.h>
 #include <dialogs/panel_setup_netclasses.h>
 #include <tool/tool_manager.h>
+#include <pcb_painter.h>
 #include <string_utils.h>
+#include <view/view.h>
 #include <widgets/grid_color_swatch_helpers.h>
 #include <widgets/grid_icon_text_helpers.h>
 #include <widgets/wx_html_report_box.h>
 #include <widgets/wx_panel.h>
 #include <widgets/std_bitmap_button.h>
-#include <wx/treebook.h>
 #include <project/net_settings.h>
+#include <confirm.h>
 
 
 // columns of netclasses grid
-enum {
+enum
+{
     GRID_NAME = 0,
 
     GRID_FIRST_PCBNEW,
@@ -56,6 +60,7 @@ enum {
     GRID_uVIADRILL,
     GRID_DIFF_PAIR_WIDTH,
     GRID_DIFF_PAIR_GAP,
+    GRID_PCB_COLOR,
 
     GRID_FIRST_EESCHEMA,
     GRID_WIREWIDTH = GRID_FIRST_EESCHEMA,
@@ -70,16 +75,16 @@ std::vector<BITMAPS> g_lineStyleIcons;
 wxArrayString        g_lineStyleNames;
 
 
-PANEL_SETUP_NETCLASSES::PANEL_SETUP_NETCLASSES( PAGED_DIALOG* aParent, EDA_DRAW_FRAME* aFrame,
+PANEL_SETUP_NETCLASSES::PANEL_SETUP_NETCLASSES( wxWindow* aParentWindow, EDA_DRAW_FRAME* aFrame,
                                                 std::shared_ptr<NET_SETTINGS> aNetSettings,
                                                 const std::set<wxString>& aNetNames,
                                                 bool aIsEEschema ) :
-        PANEL_SETUP_NETCLASSES_BASE( aParent->GetTreebook() ),
+        PANEL_SETUP_NETCLASSES_BASE( aParentWindow ),
         m_frame( aFrame ),
-        m_parent( aParent ),
         m_isEEschema( aIsEEschema ),
-        m_netSettings( aNetSettings ),
+        m_netSettings( std::move( aNetSettings ) ),
         m_netNames( aNetNames ),
+        m_lastCheckedTicker( 0 ),
         m_hoveredCol( -1 ),
         m_lastNetclassGridWidth( -1 )
 {
@@ -87,6 +92,8 @@ PANEL_SETUP_NETCLASSES::PANEL_SETUP_NETCLASSES( PAGED_DIALOG* aParent, EDA_DRAW_
     g_lineStyleIcons.clear();
     g_lineStyleNames.clear();
 
+    g_lineStyleIcons.push_back( BITMAPS::stroke_none );
+    g_lineStyleNames.push_back( _( "<Not defined>" ) );
     g_lineStyleIcons.push_back( BITMAPS::stroke_solid );
     g_lineStyleNames.push_back( _( "Solid" ) );
     g_lineStyleIcons.push_back( BITMAPS::stroke_dash );
@@ -109,9 +116,14 @@ PANEL_SETUP_NETCLASSES::PANEL_SETUP_NETCLASSES( PAGED_DIALOG* aParent, EDA_DRAW_
     // Prevent Size events from firing before we are ready
     Freeze();
     m_netclassGrid->BeginBatch();
+    m_netclassGrid->SetUseNativeColLabels();
     m_assignmentGrid->BeginBatch();
+    m_assignmentGrid->SetUseNativeColLabels();
+
+    m_splitter->SetMinimumPaneSize( FromDIP( m_splitter->GetMinimumPaneSize() ) );
+
+    wxASSERT( m_netclassGrid->GetNumberCols() == GRID_END );
 
-    m_originalColWidths = new int[ m_netclassGrid->GetNumberCols() ];
     // Calculate a min best size to handle longest usual numeric values:
     int const min_best_width = m_netclassGrid->GetTextExtent( "555,555555 mils" ).x;
 
@@ -121,60 +133,60 @@ PANEL_SETUP_NETCLASSES::PANEL_SETUP_NETCLASSES( PAGED_DIALOG* aParent, EDA_DRAW_
         // as this initial width is sometimes strange depending on the language (wxGrid bug?)
         int const min_width =  m_netclassGrid->GetVisibleWidth( i, true, true );
 
-        int const weighted_min_best_width =
-                        ( i == GRID_LINESTYLE )
-                        ? min_best_width * 3 / 2
-                        : min_best_width;
-
-        m_netclassGrid->SetColMinimalWidth( i, min_width );
+        int const weighted_min_best_width = ( i == GRID_LINESTYLE ) ? min_best_width * 3 / 2
+                                                                    : min_best_width;
 
         // We use a "best size" >= min_best_width
         m_originalColWidths[ i ] = std::max( min_width, weighted_min_best_width );
         m_netclassGrid->SetColSize( i, m_originalColWidths[ i ] );
-    }
 
-    for( int i = GRID_FIRST_PCBNEW; i < GRID_END; ++i )
-    {
         if( i >= GRID_FIRST_EESCHEMA )
-        {
             m_netclassGrid->SetUnitsProvider( m_schUnitsProvider.get(), i );
-
-            if( !m_isEEschema )
-            {
-                m_netclassGrid->HideCol( i );
-                m_originalColWidths[ i ] = 0;
-            }
-        }
         else
-        {
             m_netclassGrid->SetUnitsProvider( m_pcbUnitsProvider.get(), i );
-
-            if( m_isEEschema )
-            {
-                m_netclassGrid->HideCol( i );
-                m_originalColWidths[ i ] = 0;
-            }
-        }
     }
 
+    COMMON_SETTINGS* cfg = Pgm().GetCommonSettings();
+
+    if( m_isEEschema )
+        m_netclassGrid->ShowHideColumns( cfg->m_NetclassPanel.eeschema_visible_columns );
+    else
+        m_netclassGrid->ShowHideColumns( cfg->m_NetclassPanel.pcbnew_visible_columns );
+
+    m_shownColumns = m_netclassGrid->GetShownColumns();
+
     wxGridCellAttr* attr = new wxGridCellAttr;
-    attr->SetRenderer( new GRID_CELL_COLOR_RENDERER( aParent ) );
-    attr->SetEditor( new GRID_CELL_COLOR_SELECTOR( aParent, m_netclassGrid ) );
+    attr->SetRenderer( new GRID_CELL_COLOR_RENDERER( PAGED_DIALOG::GetDialog( this ) ) );
+    attr->SetEditor( new GRID_CELL_COLOR_SELECTOR( PAGED_DIALOG::GetDialog( this ),
+                                                   m_netclassGrid ) );
     m_netclassGrid->SetColAttr( GRID_SCHEMATIC_COLOR, attr );
 
+    attr = new wxGridCellAttr;
+    attr->SetRenderer( new GRID_CELL_COLOR_RENDERER( PAGED_DIALOG::GetDialog( this ) ) );
+    attr->SetEditor( new GRID_CELL_COLOR_SELECTOR( PAGED_DIALOG::GetDialog( this ),
+                                                   m_netclassGrid ) );
+    m_netclassGrid->SetColAttr( GRID_PCB_COLOR, attr );
+
     attr = new wxGridCellAttr;
     attr->SetRenderer( new GRID_CELL_ICON_TEXT_RENDERER( g_lineStyleIcons, g_lineStyleNames ) );
     attr->SetEditor( new GRID_CELL_ICON_TEXT_POPUP( g_lineStyleIcons, g_lineStyleNames ) );
     m_netclassGrid->SetColAttr( GRID_LINESTYLE, attr );
 
     if( m_isEEschema )
-        m_colorDefaultHelpText->SetFont( KIUI::GetInfoFont( this ).Italic() );
+    {
+        m_importColorsButton->Hide();
+    }
     else
-        m_colorDefaultHelpText->Hide();
+    {
+        m_colorDefaultHelpText->SetLabel(
+                _( "Set color to transparent to use layer default color." ) );
+        m_colorDefaultHelpText->GetParent()->Layout();
+    }
+
+    m_colorDefaultHelpText->SetFont( KIUI::GetInfoFont( this ).Italic() );
 
     m_netclassGrid->SetAutoEvalCols( { GRID_WIREWIDTH,
                                        GRID_BUSWIDTH,
-
                                        GRID_CLEARANCE,
                                        GRID_TRACKSIZE,
                                        GRID_VIASIZE,
@@ -199,14 +211,16 @@ PANEL_SETUP_NETCLASSES::PANEL_SETUP_NETCLASSES( PAGED_DIALOG* aParent, EDA_DRAW_
     m_netclassGrid->SetSelectionMode( wxGrid::wxGridSelectRows );
     m_assignmentGrid->SetSelectionMode( wxGrid::wxGridSelectRows );
 
-    COMMON_SETTINGS* cfg = Pgm().GetCommonSettings();
     m_splitter->SetSashPosition( cfg->m_NetclassPanel.sash_pos );
 
-    m_addButton->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
-    m_removeButton->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
+    m_addButton->SetBitmap( KiBitmapBundle( BITMAPS::small_plus ) );
+    m_removeButton->SetBitmap( KiBitmapBundle( BITMAPS::small_trash ) );
 
-    m_addAssignmentButton->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
-    m_removeAssignmentButton->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
+    m_addAssignmentButton->SetBitmap( KiBitmapBundle( BITMAPS::small_plus ) );
+    m_removeAssignmentButton->SetBitmap( KiBitmapBundle( BITMAPS::small_trash ) );
+
+    m_moveUpButton->SetBitmap( KiBitmapBundle( BITMAPS::small_up ) );
+    m_moveDownButton->SetBitmap( KiBitmapBundle( BITMAPS::small_down ) );
 
     // wxFormBuilder doesn't include this event...
     m_netclassGrid->Connect( wxEVT_GRID_CELL_CHANGING,
@@ -218,12 +232,27 @@ PANEL_SETUP_NETCLASSES::PANEL_SETUP_NETCLASSES( PAGED_DIALOG* aParent, EDA_DRAW_
                                                    &PANEL_SETUP_NETCLASSES::OnNetclassGridMouseEvent,
                                                    this );
 
-    m_frame->Bind( UNITS_CHANGED, &PANEL_SETUP_NETCLASSES::onUnitsChanged, this );
+    m_frame->Bind( EDA_EVT_UNITS_CHANGED, &PANEL_SETUP_NETCLASSES::onUnitsChanged, this );
 
     m_netclassGrid->EndBatch();
     m_assignmentGrid->EndBatch();
     Thaw();
 
+    Bind( wxEVT_IDLE,
+          [this]( wxIdleEvent& aEvent )
+          {
+              // Careful of consuming CPU in an idle event handler.  Check the ticker first to
+              // see if there's even a possibility of the netclasses having changed.
+              if( m_frame->Prj().GetNetclassesTicker() > m_lastCheckedTicker )
+              {
+                  wxWindow* dialog = wxGetTopLevelParent( this );
+                  wxWindow* topLevelFocus = wxGetTopLevelParent( wxWindow::FindFocus() );
+
+                  if( topLevelFocus == dialog && m_lastLoaded != m_netSettings->GetNetclasses() )
+                      checkReload();
+              }
+          } );
+
     m_matchingNets->SetFont( KIUI::GetInfoFont( this ) );
 }
 
@@ -233,7 +262,10 @@ PANEL_SETUP_NETCLASSES::~PANEL_SETUP_NETCLASSES()
     COMMON_SETTINGS* cfg = Pgm().GetCommonSettings();
     cfg->m_NetclassPanel.sash_pos = m_splitter->GetSashPosition();
 
-    delete [] m_originalColWidths;
+    if( m_isEEschema )
+        cfg->m_NetclassPanel.eeschema_visible_columns = m_netclassGrid->GetShownColumnsAsString();
+    else
+        cfg->m_NetclassPanel.pcbnew_visible_columns = m_netclassGrid->GetShownColumnsAsString();
 
     // Delete the GRID_TRICKS.
     m_netclassGrid->PopEventHandler( true );
@@ -243,7 +275,152 @@ PANEL_SETUP_NETCLASSES::~PANEL_SETUP_NETCLASSES()
                                 wxGridEventHandler( PANEL_SETUP_NETCLASSES::OnNetclassGridCellChanging ),
                                 nullptr, this );
 
-    m_frame->Unbind( UNITS_CHANGED, &PANEL_SETUP_NETCLASSES::onUnitsChanged, this );
+    m_frame->Unbind( EDA_EVT_UNITS_CHANGED, &PANEL_SETUP_NETCLASSES::onUnitsChanged, this );
+}
+
+
+void PANEL_SETUP_NETCLASSES::loadNetclasses()
+{
+    auto netclassToGridRow =
+            [&]( int aRow, const NETCLASS* nc )
+            {
+                m_netclassGrid->SetCellValue( aRow, GRID_NAME, nc->GetName() );
+
+                m_netclassGrid->SetOptionalUnitValue( aRow, GRID_WIREWIDTH,
+                                                      nc->GetWireWidthOpt() );
+                m_netclassGrid->SetOptionalUnitValue( aRow, GRID_BUSWIDTH,
+                                                      nc->GetBusWidthOpt() );
+
+                wxString colorAsString = nc->GetSchematicColor().ToCSSString();
+                m_netclassGrid->SetCellValue( aRow, GRID_SCHEMATIC_COLOR, colorAsString );
+
+                if( nc->HasLineStyle() )
+                {
+                    int lineStyleIdx = std::max( 0, nc->GetLineStyle() );
+
+                    if( lineStyleIdx >= (int) g_lineStyleNames.size() + 1 )
+                        lineStyleIdx = 0;
+
+                    m_netclassGrid->SetCellValue( aRow, GRID_LINESTYLE,
+                                                  g_lineStyleNames[lineStyleIdx + 1] );
+                }
+                else
+                {
+                    // <Not defined> line style in list.
+                    m_netclassGrid->SetCellValue( aRow, GRID_LINESTYLE,
+                                                  g_lineStyleNames[0] );
+                }
+
+                m_netclassGrid->SetOptionalUnitValue( aRow, GRID_CLEARANCE, nc->GetClearanceOpt() );
+                m_netclassGrid->SetOptionalUnitValue( aRow, GRID_TRACKSIZE,
+                                                      nc->GetTrackWidthOpt() );
+                m_netclassGrid->SetOptionalUnitValue( aRow, GRID_VIASIZE, nc->GetViaDiameterOpt() );
+                m_netclassGrid->SetOptionalUnitValue( aRow, GRID_VIADRILL, nc->GetViaDrillOpt() );
+                m_netclassGrid->SetOptionalUnitValue( aRow, GRID_uVIASIZE,
+                                                      nc->GetuViaDiameterOpt() );
+                m_netclassGrid->SetOptionalUnitValue( aRow, GRID_uVIADRILL, nc->GetuViaDrillOpt() );
+                m_netclassGrid->SetOptionalUnitValue( aRow, GRID_DIFF_PAIR_WIDTH,
+                                                      nc->GetDiffPairWidthOpt() );
+                m_netclassGrid->SetOptionalUnitValue( aRow, GRID_DIFF_PAIR_GAP,
+                                                      nc->GetDiffPairGapOpt() );
+
+                colorAsString = nc->GetPcbColor().ToCSSString();
+                m_netclassGrid->SetCellValue( aRow, GRID_PCB_COLOR, colorAsString );
+
+                if( nc->IsDefault() )
+                {
+                    m_netclassGrid->SetReadOnly( aRow, GRID_NAME );
+                    m_netclassGrid->SetReadOnly( aRow, GRID_PCB_COLOR );
+                    m_netclassGrid->SetReadOnly( aRow, GRID_SCHEMATIC_COLOR );
+                    m_netclassGrid->SetReadOnly( aRow, GRID_LINESTYLE );
+                }
+
+                setNetclassRowNullableEditors( aRow, nc->IsDefault() );
+            };
+
+    // Get the netclasses sorted by priority
+    std::vector<const NETCLASS*> netclasses;
+    netclasses.reserve( m_netSettings->GetNetclasses().size() );
+
+    for( const auto& [name, netclass] : m_netSettings->GetNetclasses() )
+        netclasses.push_back( netclass.get() );
+
+    std::sort( netclasses.begin(), netclasses.end(),
+               []( const NETCLASS* nc1, const NETCLASS* nc2 )
+               {
+                   return nc1->GetPriority() < nc2->GetPriority();
+               } );
+
+    // Enter user-defined netclasses
+    m_netclassGrid->ClearRows();
+    m_netclassGrid->AppendRows( static_cast<int>( netclasses.size() ) );
+
+    int row = 0;
+
+    for( const NETCLASS* nc : netclasses )
+        netclassToGridRow( row++, nc );
+
+    // Enter the Default netclass.
+    m_netclassGrid->AppendRows( 1 );
+    netclassToGridRow( row, m_netSettings->GetDefaultNetclass().get() );
+
+    m_assignmentGrid->ClearRows();
+    m_assignmentGrid->AppendRows( m_netSettings->GetNetclassPatternAssignments().size() );
+
+    row = 0;
+
+    for( const auto& [matcher, netclassName] : m_netSettings->GetNetclassPatternAssignments() )
+    {
+        m_assignmentGrid->SetCellValue( row, 0, matcher->GetPattern() );
+        m_assignmentGrid->SetCellValue( row, 1, netclassName );
+        row++;
+    }
+}
+
+
+void PANEL_SETUP_NETCLASSES::setNetclassRowNullableEditors( int aRowId, bool aIsDefault )
+{
+    // Set nullable editors
+    auto setCellEditor = [this, aRowId, aIsDefault]( int aCol )
+    {
+        GRID_CELL_MARK_AS_NULLABLE* cellEditor;
+
+        if( aIsDefault )
+            cellEditor = new GRID_CELL_MARK_AS_NULLABLE( false );
+        else
+            cellEditor = new GRID_CELL_MARK_AS_NULLABLE( true );
+
+        wxGridCellAttr* attr = m_netclassGrid->GetOrCreateCellAttr( aRowId, aCol );
+        attr->SetEditor( cellEditor );
+        attr->DecRef();
+    };
+
+    setCellEditor( GRID_WIREWIDTH );
+    setCellEditor( GRID_BUSWIDTH );
+    setCellEditor( GRID_CLEARANCE );
+    setCellEditor( GRID_TRACKSIZE );
+    setCellEditor( GRID_VIASIZE );
+    setCellEditor( GRID_VIADRILL );
+    setCellEditor( GRID_VIADRILL );
+    setCellEditor( GRID_uVIASIZE );
+    setCellEditor( GRID_uVIADRILL );
+    setCellEditor( GRID_DIFF_PAIR_WIDTH );
+    setCellEditor( GRID_DIFF_PAIR_GAP );
+}
+
+
+void PANEL_SETUP_NETCLASSES::checkReload()
+{
+    // MUST update the ticker before calling IsOK (or we'll end up re-entering through the idle
+    // event until we crash the stack).
+    m_lastCheckedTicker = m_frame->Prj().GetTextVarsTicker();
+
+    if( IsOK( m_parent, _( "The netclasses have been changed outside the Setup dialog.\n"
+                           "Do you wish to reload them?" ) ) )
+    {
+        m_lastLoaded = m_netSettings->GetNetclasses();
+        loadNetclasses();
+    }
 }
 
 
@@ -269,71 +446,16 @@ void PANEL_SETUP_NETCLASSES::onUnitsChanged( wxCommandEvent& aEvent )
 
 bool PANEL_SETUP_NETCLASSES::TransferDataToWindow()
 {
-    int row = 0;
+    m_lastLoaded = m_netSettings->GetNetclasses();
+    m_lastCheckedTicker = m_frame->Prj().GetNetclassesTicker();
 
-    auto netclassToGridRow =
-            [&]( int aRow, const std::shared_ptr<NETCLASS>& nc )
-            {
-                m_netclassGrid->SetCellValue( aRow, GRID_NAME, nc->GetName() );
-
-                m_netclassGrid->SetUnitValue( aRow, GRID_WIREWIDTH, nc->GetWireWidth() );
-                m_netclassGrid->SetUnitValue( aRow, GRID_BUSWIDTH, nc->GetBusWidth() );
-
-                wxString colorAsString = nc->GetSchematicColor().ToCSSString();
-                m_netclassGrid->SetCellValue( aRow, GRID_SCHEMATIC_COLOR, colorAsString );
-
-                int lineStyleIdx = std::max( 0, nc->GetLineStyle() );
-
-                if( lineStyleIdx >= (int) g_lineStyleNames.size() )
-                    lineStyleIdx = 0;
-
-                m_netclassGrid->SetCellValue( aRow, GRID_LINESTYLE, g_lineStyleNames[ lineStyleIdx ] );
-
-                m_netclassGrid->SetUnitValue( aRow, GRID_CLEARANCE, nc->GetClearance() );
-                m_netclassGrid->SetUnitValue( aRow, GRID_TRACKSIZE, nc->GetTrackWidth() );
-                m_netclassGrid->SetUnitValue( aRow, GRID_VIASIZE, nc->GetViaDiameter() );
-                m_netclassGrid->SetUnitValue( aRow, GRID_VIADRILL, nc->GetViaDrill() );
-                m_netclassGrid->SetUnitValue( aRow, GRID_uVIASIZE, nc->GetuViaDiameter() );
-                m_netclassGrid->SetUnitValue( aRow, GRID_uVIADRILL, nc->GetuViaDrill() );
-                m_netclassGrid->SetUnitValue( aRow, GRID_DIFF_PAIR_WIDTH, nc->GetDiffPairWidth() );
-                m_netclassGrid->SetUnitValue( aRow, GRID_DIFF_PAIR_GAP, nc->GetDiffPairGap() );
-            };
-
-    m_netclassGrid->ClearRows();
-
-    // enter the Default NETCLASS.
-    m_netclassGrid->AppendRows( 1 );
-    netclassToGridRow( row++, m_netSettings->m_DefaultNetClass );
-
-    // make the Default NETCLASS name read-only
-    wxGridCellAttr* cellAttr = m_netclassGrid->GetOrCreateCellAttr( 0, GRID_NAME );
-    cellAttr->SetReadOnly();
-    cellAttr->DecRef();
-
-    // enter other netclasses
-    m_netclassGrid->AppendRows( (int) m_netSettings->m_NetClasses.size() );
-
-    for( const auto& [ name, netclass ] : m_netSettings->m_NetClasses )
-        netclassToGridRow( row++, netclass );
-
-    m_assignmentGrid->ClearRows();
-    m_assignmentGrid->AppendRows( m_netSettings->m_NetClassPatternAssignments.size() );
-    row = 0;
-
-    for( const auto& [ matcher, netclassName ] :  m_netSettings->m_NetClassPatternAssignments )
-    {
-        m_assignmentGrid->SetCellValue( row, 0, matcher->GetPattern() );
-        m_assignmentGrid->SetCellValue( row, 1, netclassName );
-        row++;
-    }
+    loadNetclasses();
+    AdjustAssignmentGridColumns( GetSize().x * 3 / 5 );
 
     return true;
 }
 
 
-/*
- * Populates drop-downs with the list of net classes
- */
 void PANEL_SETUP_NETCLASSES::rebuildNetclassDropdowns()
 {
     m_assignmentGrid->CommitPendingChanges( true );
@@ -359,58 +481,79 @@ bool PANEL_SETUP_NETCLASSES::TransferDataFromWindow()
     if( !Validate() )
         return false;
 
-    int row = 0;
-
     auto gridRowToNetclass =
             [&]( int aRow, const std::shared_ptr<NETCLASS>& nc )
             {
+                if( nc->IsDefault() )
+                    nc->SetPriority( std::numeric_limits<int>::max() );
+                else
+                    nc->SetPriority( aRow );
+
                 nc->SetName( m_netclassGrid->GetCellValue( aRow, GRID_NAME ) );
 
-                nc->SetWireWidth( m_netclassGrid->GetUnitValue( aRow, GRID_WIREWIDTH ) );
-                nc->SetBusWidth( m_netclassGrid->GetUnitValue( aRow, GRID_BUSWIDTH ) );
-
-                wxString color = m_netclassGrid->GetCellValue( aRow, GRID_SCHEMATIC_COLOR );
-                nc->SetSchematicColor( wxColour( color ) );
+                nc->SetWireWidth( m_netclassGrid->GetOptionalUnitValue( aRow, GRID_WIREWIDTH ) );
+                nc->SetBusWidth( m_netclassGrid->GetOptionalUnitValue( aRow, GRID_BUSWIDTH ) );
 
                 wxString lineStyle = m_netclassGrid->GetCellValue( aRow, GRID_LINESTYLE );
-                nc->SetLineStyle( g_lineStyleNames.Index( lineStyle ) );
-                wxASSERT_MSG( nc->GetLineStyle() >= 0, "Line style name not found." );
+                int      lineIdx = g_lineStyleNames.Index( lineStyle );
 
-                nc->SetClearance( m_netclassGrid->GetUnitValue( aRow, GRID_CLEARANCE ) );
-                nc->SetTrackWidth( m_netclassGrid->GetUnitValue( aRow, GRID_TRACKSIZE ) );
-                nc->SetViaDiameter( m_netclassGrid->GetUnitValue( aRow, GRID_VIASIZE ) );
-                nc->SetViaDrill( m_netclassGrid->GetUnitValue( aRow, GRID_VIADRILL ) );
-                nc->SetuViaDiameter( m_netclassGrid->GetUnitValue( aRow, GRID_uVIASIZE ) );
-                nc->SetuViaDrill( m_netclassGrid->GetUnitValue( aRow, GRID_uVIADRILL ) );
-                nc->SetDiffPairWidth( m_netclassGrid->GetUnitValue( aRow, GRID_DIFF_PAIR_WIDTH ) );
-                nc->SetDiffPairGap( m_netclassGrid->GetUnitValue( aRow, GRID_DIFF_PAIR_GAP ) );
+                if( lineIdx == 0 )
+                    nc->SetLineStyle( std::optional<int>() );
+                else
+                    nc->SetLineStyle( lineIdx - 1 );
+
+                wxASSERT_MSG( lineIdx >= 0, "Line style name not found." );
+
+                nc->SetClearance( m_netclassGrid->GetOptionalUnitValue( aRow, GRID_CLEARANCE ) );
+                nc->SetTrackWidth( m_netclassGrid->GetOptionalUnitValue( aRow, GRID_TRACKSIZE ) );
+                nc->SetViaDiameter( m_netclassGrid->GetOptionalUnitValue( aRow, GRID_VIASIZE ) );
+                nc->SetViaDrill( m_netclassGrid->GetOptionalUnitValue( aRow, GRID_VIADRILL ) );
+                nc->SetuViaDiameter( m_netclassGrid->GetOptionalUnitValue( aRow, GRID_uVIASIZE ) );
+                nc->SetuViaDrill( m_netclassGrid->GetOptionalUnitValue( aRow, GRID_uVIADRILL ) );
+                nc->SetDiffPairWidth(
+                        m_netclassGrid->GetOptionalUnitValue( aRow, GRID_DIFF_PAIR_WIDTH ) );
+                nc->SetDiffPairGap( m_netclassGrid->GetOptionalUnitValue( aRow,
+                                                                          GRID_DIFF_PAIR_GAP ) );
+
+                if( !nc->IsDefault() )
+                {
+                    wxString color = m_netclassGrid->GetCellValue( aRow, GRID_PCB_COLOR );
+                    KIGFX::COLOR4D newPcbColor( color );
+
+                    if( newPcbColor != KIGFX::COLOR4D::UNSPECIFIED )
+                        nc->SetPcbColor( newPcbColor );
+
+                    color = m_netclassGrid->GetCellValue( aRow, GRID_SCHEMATIC_COLOR );
+                    KIGFX::COLOR4D newSchematicColor( color );
+
+                    if( newSchematicColor != KIGFX::COLOR4D::UNSPECIFIED )
+                        nc->SetSchematicColor( newSchematicColor );
+                }
             };
 
-    m_netSettings->m_NetClasses.clear();
+    m_netSettings->ClearNetclasses();
 
     // Copy the default NetClass:
-    gridRowToNetclass( row++, m_netSettings->m_DefaultNetClass );
+    gridRowToNetclass( m_netclassGrid->GetNumberRows() - 1, m_netSettings->GetDefaultNetclass() );
 
     // Copy other NetClasses:
-    for( row = 1; row < m_netclassGrid->GetNumberRows(); ++row )
+    for( int row = 0; row < m_netclassGrid->GetNumberRows() - 1; ++row )
     {
-        auto nc = std::make_shared<NETCLASS>( m_netclassGrid->GetCellValue( row, GRID_NAME ) );
+        auto nc =
+                std::make_shared<NETCLASS>( m_netclassGrid->GetCellValue( row, GRID_NAME ), false );
         gridRowToNetclass( row, nc );
-        m_netSettings->m_NetClasses[ nc->GetName() ] = nc;
+        m_netSettings->SetNetclass( nc->GetName(), nc );
     }
 
-    m_netSettings->m_NetClassPatternAssignments.clear();
+    m_netSettings->ClearNetclassPatternAssignments();
+    m_netSettings->ClearAllCaches();
 
-    for( row = 0; row < m_assignmentGrid->GetNumberRows(); ++row )
+    for( int row = 0; row < m_assignmentGrid->GetNumberRows(); ++row )
     {
         wxString pattern = m_assignmentGrid->GetCellValue( row, 0 );
         wxString netclass = m_assignmentGrid->GetCellValue( row, 1 );
 
-        m_netSettings->m_NetClassPatternAssignments.push_back(
-                {
-                    std::make_unique<EDA_COMBINED_MATCHER>( pattern, CTX_NETCLASS ),
-                    netclass
-                } );
+        m_netSettings->SetNetclassPatternAssignment( pattern, netclass );
     }
 
     return true;
@@ -428,7 +571,7 @@ bool PANEL_SETUP_NETCLASSES::validateNetclassName( int aRow, const wxString& aNa
     if( tmp.IsEmpty() )
     {
         wxString msg =  _( "Netclass must have a name." );
-        m_parent->SetError( msg, this, m_netclassGrid, aRow, GRID_NAME );
+        PAGED_DIALOG::GetDialog( this )->SetError( msg, this, m_netclassGrid, aRow, GRID_NAME );
         return false;
     }
 
@@ -437,7 +580,8 @@ bool PANEL_SETUP_NETCLASSES::validateNetclassName( int aRow, const wxString& aNa
         if( ii != aRow && m_netclassGrid->GetCellValue( ii, GRID_NAME ).CmpNoCase( tmp ) == 0 )
         {
             wxString msg = _( "Netclass name already in use." );
-            m_parent->SetError( msg, this, m_netclassGrid, focusFirst ? aRow : ii, GRID_NAME );
+            PAGED_DIALOG::GetDialog( this )->SetError( msg, this, m_netclassGrid,
+                                                       focusFirst ? aRow : ii, GRID_NAME );
             return false;
         }
     }
@@ -509,6 +653,7 @@ void PANEL_SETUP_NETCLASSES::OnNetclassGridMouseEvent( wxMouseEvent& aEvent )
         case GRID_BUSWIDTH:         tip = _( "Bus wire thickness" );            break;
         case GRID_SCHEMATIC_COLOR:  tip = _( "Schematic wire color" );          break;
         case GRID_LINESTYLE:        tip = _( "Schematic wire line style" );     break;
+        case GRID_PCB_COLOR:        tip = _( "PCB netclass color" );            break;
         }
 
         m_netclassGrid->GetGridColLabelWindow()->UnsetToolTip();
@@ -529,15 +674,19 @@ void PANEL_SETUP_NETCLASSES::OnAddNetclassClick( wxCommandEvent& event )
     if( !m_netclassGrid->CommitPendingChanges() )
         return;
 
-    int row = m_netclassGrid->GetNumberRows();
-    m_netclassGrid->AppendRows();
+    m_netclassGrid->InsertRows();
 
-    // Copy values of the default class:
-    for( int col = 1; col < m_netclassGrid->GetNumberCols(); col++ )
-        m_netclassGrid->SetCellValue( row, col, m_netclassGrid->GetCellValue( 0, col ) );
+    // Set defaults where required
+    wxString colorAsString = KIGFX::COLOR4D::UNSPECIFIED.ToCSSString();
+    m_netclassGrid->SetCellValue( 0, GRID_PCB_COLOR, colorAsString );
+    m_netclassGrid->SetCellValue( 0, GRID_SCHEMATIC_COLOR, colorAsString );
+    m_netclassGrid->SetCellValue( 0, GRID_LINESTYLE, g_lineStyleNames[0] );
 
-    m_netclassGrid->MakeCellVisible( row, 0 );
-    m_netclassGrid->SetGridCursor( row, 0 );
+    // Set the row nullable editors
+    setNetclassRowNullableEditors( 0, false );
+
+    m_netclassGrid->MakeCellVisible( 0, 0 );
+    m_netclassGrid->SetGridCursor( 0, 0 );
 
     m_netclassGrid->EnableCellEditControl( true );
     m_netclassGrid->ShowCellEditControl();
@@ -557,9 +706,11 @@ void PANEL_SETUP_NETCLASSES::OnRemoveNetclassClick( wxCommandEvent& event )
     {
         return;
     }
-    else if( curRow == 0 )
+    else if( curRow == m_netclassGrid->GetNumberRows() - 1 )
     {
-        DisplayErrorMessage( this, _( "The default net class is required." ) );
+        wxWindow* topLevelParent = wxGetTopLevelParent( this );
+
+        DisplayErrorMessage( topLevelParent, _( "The default net class is required." ) );
         return;
     }
 
@@ -592,8 +743,11 @@ void PANEL_SETUP_NETCLASSES::AdjustNetclassGridColumns( int aWidth )
 
         for( int i = 1; i < m_netclassGrid->GetNumberCols(); i++ )
         {
-            m_netclassGrid->SetColSize( i, m_originalColWidths[ i ] );
-            aWidth -= m_originalColWidths[ i ];
+            if( m_netclassGrid->GetColSize( i ) > 0 )
+            {
+                m_netclassGrid->SetColSize( i, m_originalColWidths[ i ] );
+                aWidth -= m_originalColWidths[ i ];
+            }
         }
 
         m_netclassGrid->SetColSize( 0, std::max( aWidth - 2, m_originalColWidths[ 0 ] ) );
@@ -617,7 +771,7 @@ void PANEL_SETUP_NETCLASSES::OnAddAssignmentClick( wxCommandEvent& event )
     int row = m_assignmentGrid->GetNumberRows();
     m_assignmentGrid->AppendRows();
 
-    m_assignmentGrid->SetCellValue( row, 1, m_netSettings->m_DefaultNetClass->GetName() );
+    m_assignmentGrid->SetCellValue( row, 1, m_netSettings->GetDefaultNetclass()->GetName() );
 
     m_assignmentGrid->MakeCellVisible( row, 0 );
     m_assignmentGrid->SetGridCursor( row, 0 );
@@ -647,21 +801,38 @@ void PANEL_SETUP_NETCLASSES::OnRemoveAssignmentClick( wxCommandEvent& event )
 }
 
 
+void PANEL_SETUP_NETCLASSES::OnImportColorsClick( wxCommandEvent& event )
+{
+    const std::map<wxString, std::shared_ptr<NETCLASS>>& netclasses =
+            m_netSettings->GetNetclasses();
+
+    for( int row = 0; row < m_netclassGrid->GetNumberRows() - 1; ++row )
+    {
+        wxString netclassName = m_netclassGrid->GetCellValue( row, GRID_NAME );
+
+        if( netclasses.find( netclassName ) != netclasses.end() )
+        {
+            const KIGFX::COLOR4D ncColor = netclasses.at( netclassName )->GetSchematicColor();
+            m_netclassGrid->SetCellValue( row, GRID_PCB_COLOR, ncColor.ToCSSString() );
+        }
+    }
+}
+
+
 void PANEL_SETUP_NETCLASSES::AdjustAssignmentGridColumns( int aWidth )
 {
     // Account for scroll bars
     aWidth -= ( m_assignmentGrid->GetSize().x - m_assignmentGrid->GetClientSize().x );
 
-    // Set className column width to original className width from netclasses grid
-    int classNameWidth = m_originalColWidths[ 0 ];
-    m_assignmentGrid->SetColSize( 1, m_originalColWidths[ 0 ] );
+    int classNameWidth = 160;
+    m_assignmentGrid->SetColSize( 1, classNameWidth );
     m_assignmentGrid->SetColSize( 0, std::max( aWidth - classNameWidth, classNameWidth ) );
 }
 
 
 void PANEL_SETUP_NETCLASSES::OnSizeAssignmentGrid( wxSizeEvent& event )
 {
-    AdjustAssignmentGridColumns( event.GetSize().GetX());
+    AdjustAssignmentGridColumns( event.GetSize().GetX() );
 
     event.Skip();
 }
@@ -675,6 +846,12 @@ void PANEL_SETUP_NETCLASSES::OnUpdateUI( wxUpdateUIEvent& event )
         m_netclassesDirty = false;
     }
 
+    if( m_shownColumns != m_netclassGrid->GetShownColumns() )
+    {
+        AdjustNetclassGridColumns( GetSize().x - 1 );
+        m_shownColumns = m_netclassGrid->GetShownColumns();
+    }
+
     if( m_assignmentGrid->GetNumberRows() == 0 )
         return;
 
@@ -708,10 +885,7 @@ void PANEL_SETUP_NETCLASSES::OnUpdateUI( wxUpdateUIEvent& event )
 
             for( const wxString& net : m_netNames )
             {
-                int matches;
-                int offset;
-
-                if( matcher.Find( net, matches, offset ) && offset == 0 )
+                if( matcher.StartsWith( net ) )
                     m_matchingNets->Report( net );
             }
         }
@@ -756,7 +930,85 @@ void PANEL_SETUP_NETCLASSES::ImportSettingsFrom( const std::shared_ptr<NET_SETTI
     m_netclassGrid->ForceRefresh();
     m_assignmentGrid->ForceRefresh();
 
-    m_netSettings = savedSettings;
+    m_netSettings = std::move( savedSettings );
 }
 
 
+void PANEL_SETUP_NETCLASSES::OnMoveNetclassUpClick( wxCommandEvent& event )
+{
+    if( !m_netclassGrid->CommitPendingChanges() )
+        return;
+
+    // Work out which rows are selected
+    std::vector<int> selectedRows;
+
+    for( int i = 0; i < m_netclassGrid->GetNumberRows(); ++i )
+    {
+        if( m_netclassGrid->IsInSelection( i, 0 ) )
+            selectedRows.push_back( i );
+    }
+
+    // Only move one row at a time
+    if( selectedRows.size() != 1 )
+        return;
+
+    // Can't move the first netclass, nor move the Default netclass
+    if( selectedRows[0] == 0 || selectedRows[0] == ( m_netclassGrid->GetNumberRows() - 1 ) )
+        return;
+
+    int newRowId = selectedRows[0] - 1;
+    m_netclassGrid->InsertRows( newRowId );
+
+    for( int col = 0; col < m_netclassGrid->GetNumberCols(); col++ )
+        m_netclassGrid->SetCellValue( newRowId, col,
+                                      m_netclassGrid->GetCellValue( newRowId + 2, col ) );
+
+    // Set the row nullable editors
+    setNetclassRowNullableEditors( newRowId, false );
+
+    m_netclassGrid->DeleteRows( newRowId + 2, 1 );
+    m_netclassGrid->MakeCellVisible( newRowId, 0 );
+    m_netclassGrid->SetGridCursor( newRowId, 0 );
+
+    m_netclassesDirty = true;
+}
+
+
+void PANEL_SETUP_NETCLASSES::OnMoveNetclassDownClick( wxCommandEvent& event )
+{
+    if( !m_netclassGrid->CommitPendingChanges() )
+        return;
+
+    // Work out which rows are selected
+    std::vector<int> selectedRows;
+
+    for( int i = 0; i < m_netclassGrid->GetNumberRows(); ++i )
+    {
+        if( m_netclassGrid->IsInSelection( i, 0 ) )
+            selectedRows.push_back( i );
+    }
+
+    // Only move one row at a time
+    if( selectedRows.size() != 1 )
+        return;
+
+    // Can't move the last row down, nor move the Default netclass
+    if( selectedRows[0] == ( m_netclassGrid->GetNumberRows() - 2 )
+        || selectedRows[0] == ( m_netclassGrid->GetNumberRows() - 1 ) )
+        return;
+
+    int newRowId = selectedRows[0] + 2;
+    m_netclassGrid->InsertRows( newRowId );
+
+    for( int col = 0; col < m_netclassGrid->GetNumberCols(); col++ )
+    {
+        m_netclassGrid->SetCellValue( newRowId, col,
+                                      m_netclassGrid->GetCellValue( newRowId - 2, col ) );
+    }
+
+    m_netclassGrid->DeleteRows( newRowId - 2, 1 );
+    m_netclassGrid->MakeCellVisible( newRowId - 1, 0 );
+    m_netclassGrid->SetGridCursor( newRowId - 1, 0 );
+
+    m_netclassesDirty = true;
+}
