@@ -3,7 +3,8 @@
  *
  * Copyright (C) 2017 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2013 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2023 CERN (www.cern.ch)
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,22 +24,26 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <eda_base_frame.h>
+
+#include <advanced_config.h>
 #include <bitmaps.h>
 #include <bitmap_store.h>
 #include <dialog_shim.h>
+#include <dialogs/git/panel_git_repos.h>
 #include <dialogs/panel_common_settings.h>
 #include <dialogs/panel_mouse_settings.h>
 #include <dialogs/panel_data_collection.h>
+#include <dialogs/panel_plugin_settings.h>
 #include <eda_dde.h>
 #include <file_history.h>
 #include <id.h>
 #include <kiface_base.h>
-#include <menus_helpers.h>
 #include <hotkeys_basic.h>
 #include <panel_hotkeys_editor.h>
 #include <paths.h>
 #include <confirm.h>
-#include <panel_pcm_settings.h>
+#include <panel_packages_and_updates.h>
 #include <pgm_base.h>
 #include <settings/app_settings.h>
 #include <settings/common_settings.h>
@@ -53,51 +58,58 @@
 #include <tool/tool_dispatcher.h>
 #include <trace_helpers.h>
 #include <widgets/paged_dialog.h>
+#include <widgets/wx_busy_indicator.h>
 #include <widgets/wx_infobar.h>
 #include <widgets/wx_aui_art_providers.h>
+#include <widgets/wx_grid.h>
+#include <widgets/wx_treebook.h>
 #include <wx/app.h>
 #include <wx/config.h>
 #include <wx/display.h>
 #include <wx/stdpaths.h>
 #include <wx/string.h>
-#include <wx/treebook.h>
+#include <wx/msgdlg.h>
 #include <kiplatform/app.h>
+#include <kiplatform/io.h>
 #include <kiplatform/ui.h>
 
 #include <functional>
 #include <kiface_ids.h>
 
-wxDEFINE_EVENT( UNITS_CHANGED, wxCommandEvent );
+#ifdef KICAD_IPC_API
+#include <api/api_server.h>
+#endif
 
 
 // Minimum window size
-static const wxSize minSize( FRAME_T aFrameType )
+static const wxSize minSizeLookup( FRAME_T aFrameType, wxWindow* aWindow )
 {
     switch( aFrameType )
     {
     case KICAD_MAIN_FRAME_T:
-        return wxSize( 406, 354 );
+        return wxWindow::FromDIP( wxSize( 406, 354 ), aWindow );
 
     default:
-        return wxSize( 500, 400 );
+        return wxWindow::FromDIP( wxSize( 500, 400 ), aWindow );
     }
 }
 
 
-static const wxSize defaultSize( FRAME_T aFrameType )
+static const wxSize defaultSize( FRAME_T aFrameType, wxWindow* aWindow )
 {
     switch( aFrameType )
     {
     case KICAD_MAIN_FRAME_T:
-        return wxSize( 850, 540 );
+        return wxWindow::FromDIP( wxSize( 850, 540 ), aWindow );
 
     default:
-        return wxSize( 1280, 720 );
+        return wxWindow::FromDIP( wxSize( 1280, 720 ), aWindow );
     }
 }
 
 
 BEGIN_EVENT_TABLE( EDA_BASE_FRAME, wxFrame )
+    // These event table entries are needed to handle events from the mac application menu
     EVT_MENU( wxID_ABOUT, EDA_BASE_FRAME::OnKicadAbout )
     EVT_MENU( wxID_PREFERENCES, EDA_BASE_FRAME::OnPreferences )
 
@@ -129,7 +141,7 @@ void EDA_BASE_FRAME::commonInit( FRAME_T aFrameType )
     m_autoSaveTimer     = new wxTimer( this, ID_AUTO_SAVE_TIMER );
     m_autoSaveRequired  = false;
     m_mruPath           = PATHS::GetDefaultUserProjectsPath();
-    m_frameSize         = defaultSize( aFrameType );
+    m_frameSize         = defaultSize( aFrameType, this );
     m_displayIndex      = -1;
 
     m_auimgr.SetArtProvider( new WX_AUI_DOCK_ART() );
@@ -137,7 +149,8 @@ void EDA_BASE_FRAME::commonInit( FRAME_T aFrameType )
     m_settingsManager = &Pgm().GetSettingsManager();
 
     // Set a reasonable minimal size for the frame
-    SetSizeHints( minSize( aFrameType ).x, minSize( aFrameType ).y, -1, -1, -1, -1 );
+    wxSize minSize = minSizeLookup( aFrameType, this );
+    SetSizeHints( minSize.x, minSize.y, -1, -1, -1, -1 );
 
     // Store dimensions of the user area of the main window.
     GetClientSize( &m_frameSize.x, &m_frameSize.y );
@@ -166,15 +179,28 @@ EDA_BASE_FRAME::EDA_BASE_FRAME( wxWindow* aParent, FRAME_T aFrameType, const wxS
 }
 
 
+wxWindow* findQuasiModalDialog( wxWindow* aParent )
+{
+    for( wxWindow* child : aParent->GetChildren() )
+    {
+        if( DIALOG_SHIM* dlg = dynamic_cast<DIALOG_SHIM*>( child ) )
+        {
+            if( dlg->IsQuasiModal() )
+                return dlg;
+
+            if( wxWindow* nestedDlg = findQuasiModalDialog( child ) )
+                return nestedDlg;
+        }
+    }
+
+    return nullptr;
+}
+
+
 wxWindow* EDA_BASE_FRAME::findQuasiModalDialog()
 {
-    for( wxWindow* iter : GetChildren() )
-    {
-        DIALOG_SHIM* dlg = dynamic_cast<DIALOG_SHIM*>( iter );
-
-        if( dlg && dlg->IsQuasiModal() )
-            return dlg;
-    }
+    if( wxWindow* dlg = ::findQuasiModalDialog( this ) )
+        return dlg;
 
     // FIXME: CvPcb is currently implemented on top of KIWAY_PLAYER rather than DIALOG_SHIM,
     // so we have to look for it separately.
@@ -245,6 +271,10 @@ void EDA_BASE_FRAME::windowClosing( wxCloseEvent& event )
 
 EDA_BASE_FRAME::~EDA_BASE_FRAME()
 {
+    Disconnect( ID_AUTO_SAVE_TIMER, wxEVT_TIMER,
+                wxTimerEventHandler( EDA_BASE_FRAME::onAutoSaveTimer ) );
+    Disconnect( wxEVT_CLOSE_WINDOW, wxCloseEventHandler( EDA_BASE_FRAME::windowClosing ) );
+
     delete m_autoSaveTimer;
     delete m_fileHistory;
 
@@ -266,6 +296,7 @@ bool EDA_BASE_FRAME::ProcessEvent( wxEvent& aEvent )
     if( !IsEnabled() && IsActive() )
     {
         wxWindow* dlg = findQuasiModalDialog();
+
         if( dlg )
             dlg->Raise();
     }
@@ -277,7 +308,7 @@ bool EDA_BASE_FRAME::ProcessEvent( wxEvent& aEvent )
     if( Pgm().m_Quitting )
         return true;
 
-    if( !m_isClosing && m_supportsAutoSave && IsShown() && IsActive()
+    if( !m_isClosing && m_supportsAutoSave && IsShownOnScreen() && IsActive()
             && m_autoSavePending != isAutoSaveRequired()
             && GetAutoSaveInterval() > 0 )
     {
@@ -392,6 +423,25 @@ void EDA_BASE_FRAME::HandleUpdateUIEvent( wxUpdateUIEvent& aEvent, EDA_BASE_FRAM
         return;
     }
 
+    if( showRes && aEvent.GetId() == ACTIONS::undo.GetUIId() )
+    {
+        wxString msg = _( "Undo" );
+
+        if( enableRes )
+            msg += wxS( " " ) + aFrame->GetUndoActionDescription();
+
+        aEvent.SetText( msg );
+    }
+    else if( showRes && aEvent.GetId() == ACTIONS::redo.GetUIId() )
+    {
+        wxString msg = _( "Redo" );
+
+        if( enableRes )
+            msg += wxS( " " ) + aFrame->GetRedoActionDescription();
+
+        aEvent.SetText( msg );
+    }
+
     if( isCut || isCopy || isPaste )
     {
         wxWindow*    focus = wxWindow::FindFocus();
@@ -403,27 +453,15 @@ void EDA_BASE_FRAME::HandleUpdateUIEvent( wxUpdateUIEvent& aEvent, EDA_BASE_FRAM
             enableRes = true;
         else if( textEntry && isPaste && textEntry->CanPaste() )
             enableRes = true;
+        else if( dynamic_cast<WX_GRID*>( focus ) )
+            enableRes = false;  // Must disable menu in order to get command as CharHook event
     }
 
     aEvent.Enable( enableRes );
     aEvent.Show( showRes );
 
-    // wxWidgets 3.1.5+ includes a field in the event that says if the event supports being
-    // checked, since wxMenuItems don't want to be checked unless they actually are checkable
-#if wxCHECK_VERSION( 3, 1, 5 )
     if( aEvent.IsCheckable() )
         aEvent.Check( checkRes );
-#else
-    bool canCheck = true;
-
-    // wxMenuItems don't want to be checked unless they actually are checkable, so we have to
-    // check to see if they can be and can't just universally apply a check in this event.
-    if( wxMenu* menu = dynamic_cast<wxMenu*>( aEvent.GetEventObject() ) )
-        canCheck = menu->FindItem( aEvent.GetId() )->IsCheckable();
-
-    if( canCheck )
-        aEvent.Check( checkRes );
-#endif
 }
 
 
@@ -431,10 +469,10 @@ void EDA_BASE_FRAME::setupUIConditions()
 {
     // Setup the conditions to check a language menu item
     auto isCurrentLang =
-        [] ( const SELECTION& aSel, int aLangIdentifier )
-        {
-            return Pgm().GetSelectedLanguageIdentifier() == aLangIdentifier;
-        };
+            [] ( const SELECTION& aSel, int aLangIdentifier )
+            {
+                return Pgm().GetSelectedLanguageIdentifier() == aLangIdentifier;
+            };
 
     for( unsigned ii = 0;  LanguagesList[ii].m_KI_Lang_Identifier != 0; ii++ )
     {
@@ -456,13 +494,30 @@ void EDA_BASE_FRAME::ReCreateMenuBar()
      * ensure that they do not occur within the same event handling call stack.
      */
 
-    CallAfter( [=]()
+    CallAfter( [this]()
                {
-                   doReCreateMenuBar();
+                   if( !m_isClosing )
+                       doReCreateMenuBar();
                } );
 }
 
 
+void EDA_BASE_FRAME::SetMenuBar( wxMenuBar* menu_bar )
+{
+    wxFrame::SetMenuBar( menu_bar );
+
+    // Move Help menu back to end of menubar
+
+    int pos = GetMenuBar()->FindMenu( _( "&Help" ) + wxS( " " ) );
+
+    if( pos != wxNOT_FOUND )
+    {
+        wxMenu* helpMenu = GetMenuBar()->Remove( pos );
+        GetMenuBar()->Append( helpMenu, _( "&Help" ) + wxS( " " ) );
+    }
+}
+
+
 void EDA_BASE_FRAME::AddStandardHelpMenu( wxMenuBar* aMenuBar )
 {
     COMMON_CONTROL* commonControl = m_toolManager->GetTool<COMMON_CONTROL>();
@@ -476,10 +531,15 @@ void EDA_BASE_FRAME::AddStandardHelpMenu( wxMenuBar* aMenuBar )
     helpMenu->Add( ACTIONS::reportBug );
 
     helpMenu->AppendSeparator();
-    helpMenu->Add( _( "&About KiCad" ), "", wxID_ABOUT, BITMAPS::about );
+    helpMenu->Add( ACTIONS::about );
 
     // Trailing space keeps OSX from hijacking our menu (and disabling everything in it).
     aMenuBar->Append( helpMenu, _( "&Help" ) + wxS( " " ) );
+
+    // This change is only needed on macOS, and creates a bug on Windows
+    #ifdef __WXMAC__
+    helpMenu->wxMenu::SetTitle( _( "&Help" ) + wxS( " " ) );
+    #endif
 }
 
 
@@ -495,12 +555,21 @@ void EDA_BASE_FRAME::ShowChangedLanguage()
 }
 
 
-void EDA_BASE_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVarsChanged )
+void EDA_BASE_FRAME::CommonSettingsChanged( int aFlags )
 {
-    TOOLS_HOLDER::CommonSettingsChanged( aEnvVarsChanged, aTextVarsChanged );
+    TOOLS_HOLDER::CommonSettingsChanged( aFlags );
 
     COMMON_SETTINGS* settings = Pgm().GetCommonSettings();
 
+#ifdef KICAD_IPC_API
+    bool running = Pgm().GetApiServer().Running();
+
+    if( running && !settings->m_Api.enable_server )
+        Pgm().GetApiServer().Stop();
+    else if( !running && settings->m_Api.enable_server )
+        Pgm().GetApiServer().Start();
+#endif
+
     if( m_fileHistory )
     {
         int historySize = settings->m_System.file_history_size;
@@ -579,9 +648,11 @@ void EDA_BASE_FRAME::LoadWindowState( const WINDOW_STATE& aState )
                 m_framePos.x, m_framePos.y, m_frameSize.x, m_frameSize.y );
 
     // Ensure minimum size is set if the stored config was zero-initialized
-    if( m_frameSize.x < minSize( m_ident ).x || m_frameSize.y < minSize( m_ident ).y )
+    wxSize minSize = minSizeLookup( m_ident, this );
+
+    if( m_frameSize.x < minSize.x || m_frameSize.y < minSize.y )
     {
-        m_frameSize = defaultSize( m_ident );
+        m_frameSize = defaultSize( m_ident, this );
         wasDefault  = true;
 
         wxLogTrace( traceDisplayLocation, wxS( "Using minimum size (%d, %d)" ),
@@ -668,7 +739,8 @@ void EDA_BASE_FRAME::ensureWindowIsOnScreen()
     wxSize    size       = GetWindowSize();
 
     wxLogTrace( traceDisplayLocation,
-                wxS( "ensureWindowIsOnScreen: clientArea (%d, %d) w %d h %d" ), clientSize.x, clientSize.y,
+                wxS( "ensureWindowIsOnScreen: clientArea (%d, %d) w %d h %d" ),
+                clientSize.x, clientSize.y,
                 clientSize.width, clientSize.height );
 
     if( pos.y < clientSize.y )
@@ -682,8 +754,8 @@ void EDA_BASE_FRAME::ensureWindowIsOnScreen()
     if( pos.x < clientSize.x )
     {
         wxLogTrace( traceDisplayLocation,
-                    wxS( "ensureWindowIsOnScreen: x pos %d is off the client rect, setting to %d" ), pos.x,
-                    clientSize.x );
+                    wxS( "ensureWindowIsOnScreen: x pos %d is off the client rect, setting to %d" ),
+                    pos.x, clientSize.x );
         pos.x = clientSize.x;
     }
 
@@ -691,8 +763,8 @@ void EDA_BASE_FRAME::ensureWindowIsOnScreen()
     {
         int newWidth = clientSize.width - ( pos.x - clientSize.x );
         wxLogTrace( traceDisplayLocation,
-                    wxS( "ensureWindowIsOnScreen: effective width %d above available %d, setting to %d" ),
-                    pos.x + size.x, clientSize.width, newWidth );
+                    wxS( "ensureWindowIsOnScreen: effective width %d above available %d, setting "
+                         "to %d" ), pos.x + size.x, clientSize.width, newWidth );
         size.x = newWidth;
     }
 
@@ -700,8 +772,8 @@ void EDA_BASE_FRAME::ensureWindowIsOnScreen()
     {
         int newHeight = clientSize.height - ( pos.y - clientSize.y );
         wxLogTrace( traceDisplayLocation,
-                    wxS( "ensureWindowIsOnScreen: effective height %d above available %d, setting to %d" ),
-                    pos.y + size.y, clientSize.height, newHeight );
+                    wxS( "ensureWindowIsOnScreen: effective height %d above available %d, setting "
+                         "to %d" ), pos.y + size.y, clientSize.height, newHeight );
         size.y = newHeight;
     }
 
@@ -719,7 +791,7 @@ void EDA_BASE_FRAME::LoadWindowSettings( const WINDOW_SETTINGS* aCfg )
     m_perspective = aCfg->perspective;
     m_mruPath = aCfg->mru_path;
 
-    TOOLS_HOLDER::CommonSettingsChanged( false, false );
+    TOOLS_HOLDER::CommonSettingsChanged();
 }
 
 
@@ -769,7 +841,7 @@ void EDA_BASE_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
     int fileHistorySize = Pgm().GetCommonSettings()->m_System.file_history_size;
 
     // Load the recently used files into the history menu
-    m_fileHistory = new FILE_HISTORY( (unsigned) std::max( 0, fileHistorySize ),
+    m_fileHistory = new FILE_HISTORY( (unsigned) std::max( 1, fileHistorySize ),
                                       ID_FILE1, ID_FILE_LIST_CLEAR );
     m_fileHistory->Load( *aCfg );
 }
@@ -1003,157 +1075,222 @@ void EDA_BASE_FRAME::OnPreferences( wxCommandEvent& event )
 
 void EDA_BASE_FRAME::ShowPreferences( wxString aStartPage, wxString aStartParentPage )
 {
-    wxBeginBusyCursor( wxHOURGLASS_CURSOR );
-
-    PAGED_DIALOG dlg( this, _( "Preferences" ), true );
+    PAGED_DIALOG dlg( this, _( "Preferences" ), true, true, wxEmptyString,
+                      wxWindow::FromDIP( wxSize( 980, 560 ), NULL ) );
 
     dlg.SetEvtHandlerEnabled( false );
-    wxTreebook*  book = dlg.GetTreebook();
 
-    PANEL_HOTKEYS_EDITOR*   hotkeysPanel = new PANEL_HOTKEYS_EDITOR( this, book, false );
-    KIFACE*                 kiface = nullptr;
-    std::vector<int>        expand;
+    {
+        WX_BUSY_INDICATOR busy_cursor;
 
-    Kiway().GetActions( hotkeysPanel->ActionsList() );
+        WX_TREEBOOK*            book = dlg.GetTreebook();
+        PANEL_HOTKEYS_EDITOR*   hotkeysPanel = new PANEL_HOTKEYS_EDITOR( this, book, false );
+        KIFACE*                 kiface = nullptr;
+        std::vector<int>        expand;
 
-    book->AddPage( new PANEL_COMMON_SETTINGS( &dlg, book ), _( "Common" ) );
-    book->AddPage( new PANEL_MOUSE_SETTINGS( &dlg, book ), _( "Mouse and Touchpad" ) );
-    book->AddPage( hotkeysPanel, _( "Hotkeys" ) );
+        Kiway().GetActions( hotkeysPanel->ActionsList() );
+
+        book->AddLazyPage(
+                []( wxWindow* aParent ) -> wxWindow*
+                {
+                    return new PANEL_COMMON_SETTINGS( aParent );
+                },
+                _( "Common" ) );
+
+        book->AddLazyPage(
+                []( wxWindow* aParent ) -> wxWindow*
+                {
+                    return new PANEL_MOUSE_SETTINGS( aParent );
+                }, _( "Mouse and Touchpad" ) );
+
+        book->AddPage( hotkeysPanel, _( "Hotkeys" ) );
+
+        // This currently allows pre-defined repositories that we
+        // don't use, so keep it disabled at the moment
+        if( ADVANCED_CFG::GetCfg().m_EnableGit && false )
+        {
+            book->AddLazyPage(
+                    []( wxWindow* aParent ) -> wxWindow*
+                    {
+                        return new PANEL_GIT_REPOS( aParent );
+                    }, _( "Version Control" ) );
+        }
 
 #ifdef KICAD_USE_SENTRY
-    book->AddPage( new PANEL_DATA_COLLECTION( &dlg, book ), _( "Data Collection" ) );
+        book->AddLazyPage(
+                []( wxWindow* aParent ) -> wxWindow*
+                {
+                    return new PANEL_DATA_COLLECTION( aParent );
+                }, _( "Data Collection" ) );
 #endif
 
-#define CREATE_PANEL( key ) kiface->CreateKiWindow( book, key, &Kiway() )
+#define LAZY_CTOR( key )                                                \
+        [this, kiface]( wxWindow* aParent )                             \
+        {                                                               \
+            return kiface->CreateKiWindow( aParent, key, &Kiway() );    \
+        }
 
-    // If a dll is not loaded, the loader will show an error message.
+        // If a dll is not loaded, the loader will show an error message.
 
-    try
-    {
-        kiface = Kiway().KiFACE( KIWAY::FACE_SCH );
+        try
+        {
+            kiface = Kiway().KiFACE( KIWAY::FACE_SCH );
 
-        kiface->GetActions( hotkeysPanel->ActionsList() );
+            if( !kiface )
+                return;
 
-        if( GetFrameType() == FRAME_SCH_SYMBOL_EDITOR )
-            expand.push_back( book->GetPageCount() );
+            kiface->GetActions( hotkeysPanel->ActionsList() );
 
-        book->AddPage( new wxPanel( book ), _( "Symbol Editor" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_SYM_DISP_OPTIONS ), _( "Display Options" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_SYM_EDIT_OPTIONS ), _( "Editing Options" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_SYM_COLORS ), _( "Colors" ) );
+            if( GetFrameType() == FRAME_SCH_SYMBOL_EDITOR )
+                expand.push_back( (int) book->GetPageCount() );
 
-        if( GetFrameType() == FRAME_SCH )
-            expand.push_back( book->GetPageCount() );
+            book->AddPage( new wxPanel( book ), _( "Symbol Editor" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_SYM_DISP_OPTIONS ), _( "Display Options" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_SYM_EDIT_GRIDS ), _( "Grids" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_SYM_EDIT_OPTIONS ), _( "Editing Options" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_SYM_COLORS ), _( "Colors" ) );
 
-        book->AddPage( new wxPanel( book ), _( "Schematic Editor" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_SCH_DISP_OPTIONS ), _( "Display Options" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_SCH_EDIT_OPTIONS ), _( "Editing Options" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_SCH_ANNO_OPTIONS ), _( "Annotation Options" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_SCH_COLORS ), _( "Colors" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_SCH_FIELD_NAME_TEMPLATES ),
-                          _( "Field Name Templates" ) );
+            if( GetFrameType() == FRAME_SCH )
+                expand.push_back( (int) book->GetPageCount() );
+
+            book->AddPage( new wxPanel( book ), _( "Schematic Editor" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_SCH_DISP_OPTIONS ), _( "Display Options" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_SCH_GRIDS ), _( "Grids" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_SCH_EDIT_OPTIONS ), _( "Editing Options" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_SCH_ANNO_OPTIONS ), _( "Annotation Options" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_SCH_COLORS ), _( "Colors" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_SCH_FIELD_NAME_TEMPLATES ),
+                                  _( "Field Name Templates" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_SCH_SIMULATOR ), _( "Simulator" ) );
+        }
+        catch( ... )
+        {
+        }
+
+        try
+        {
+            kiface = Kiway().KiFACE( KIWAY::FACE_PCB );
+
+            if( !kiface )
+                return;
+
+            kiface->GetActions( hotkeysPanel->ActionsList() );
+
+            if( GetFrameType() == FRAME_FOOTPRINT_EDITOR )
+                expand.push_back( (int) book->GetPageCount() );
+
+            book->AddPage( new wxPanel( book ), _( "Footprint Editor" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_FP_DISPLAY_OPTIONS ), _( "Display Options" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_FP_GRIDS ), _( "Grids" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_FP_ORIGINS_AXES ), _( "Origins & Axes" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_FP_EDIT_OPTIONS ), _( "Editing Options" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_FP_COLORS ), _( "Colors" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_FP_DEFAULT_FIELDS ), _( "Footprint Defaults" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_FP_DEFAULT_GRAPHICS_VALUES ),
+                                  _( "Graphics Defaults" ) );
+
+            if( GetFrameType() ==  FRAME_PCB_EDITOR )
+                expand.push_back( (int) book->GetPageCount() );
+
+            book->AddPage( new wxPanel( book ), _( "PCB Editor" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_PCB_DISPLAY_OPTS ), _( "Display Options" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_PCB_GRIDS ), _( "Grids" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_PCB_ORIGINS_AXES ), _( "Origins & Axes" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_PCB_EDIT_OPTIONS ), _( "Editing Options" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_PCB_COLORS ), _( "Colors" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_PCB_ACTION_PLUGINS ), _( "Plugins" ) );
+
+            if( GetFrameType() == FRAME_PCB_DISPLAY3D )
+                expand.push_back( (int) book->GetPageCount() );
+
+            book->AddPage( new wxPanel( book ), _( "3D Viewer" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_3DV_DISPLAY_OPTIONS ), _( "General" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_3DV_OPENGL ), _( "Realtime Renderer" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_3DV_RAYTRACING ), _( "Raytracing Renderer" ) );
+        }
+        catch( ... )
+        {
+        }
+
+        try
+        {
+            kiface = Kiway().KiFACE( KIWAY::FACE_GERBVIEW );
+
+            if( !kiface )
+                return;
+
+            kiface->GetActions( hotkeysPanel->ActionsList() );
+
+            if( GetFrameType() == FRAME_GERBER )
+                expand.push_back( (int) book->GetPageCount() );
+
+            book->AddPage( new wxPanel( book ), _( "Gerber Viewer" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_GBR_DISPLAY_OPTIONS ), _( "Display Options" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_GBR_COLORS ), _( "Colors" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_GBR_GRIDS ), _( "Grids" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_GBR_EXCELLON_OPTIONS ),
+                                  _( "Excellon Options" ) );
+        }
+        catch( ... )
+        {
+        }
+
+        try
+        {
+            kiface = Kiway().KiFACE( KIWAY::FACE_PL_EDITOR );
+
+            if( !kiface )
+                return;
+
+            kiface->GetActions( hotkeysPanel->ActionsList() );
+
+            if( GetFrameType() == FRAME_PL_EDITOR )
+                expand.push_back( (int) book->GetPageCount() );
+
+            book->AddPage( new wxPanel( book ), _( "Drawing Sheet Editor" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_DS_DISPLAY_OPTIONS ), _( "Display Options" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_DS_GRIDS ), _( "Grids" ) );
+            book->AddLazySubPage( LAZY_CTOR( PANEL_DS_COLORS ), _( "Colors" ) );
+
+            book->AddLazyPage(
+                    []( wxWindow* aParent ) -> wxWindow*
+                    {
+                        return new PANEL_PACKAGES_AND_UPDATES( aParent );
+                    }, _( "Packages and Updates" ) );
+        }
+        catch( ... )
+        {
+        }
+
+#ifdef KICAD_IPC_API
+        book->AddPage( new PANEL_PLUGIN_SETTINGS( book ), _( "Plugins" ) );
+#endif
+
+        // Update all of the action hotkeys. The process of loading the actions through
+        // the KiFACE will only get us the default hotkeys
+        ReadHotKeyConfigIntoActions( wxEmptyString, hotkeysPanel->ActionsList() );
+
+        for( size_t i = 0; i < book->GetPageCount(); ++i )
+            book->GetPage( i )->Layout();
+
+        for( int page : expand )
+            book->ExpandNode( page );
+
+        if( !aStartPage.IsEmpty() )
+            dlg.SetInitialPage( aStartPage, aStartParentPage );
+
+        dlg.SetEvtHandlerEnabled( true );
+#undef LAZY_CTOR
     }
-    catch( ... )
-    {
-    }
-
-    try
-    {
-        kiface = Kiway().KiFACE( KIWAY::FACE_PCB );
-
-        kiface->GetActions( hotkeysPanel->ActionsList() );
-
-        if( GetFrameType() == FRAME_FOOTPRINT_EDITOR )
-            expand.push_back( book->GetPageCount() );
-
-        book->AddPage( new wxPanel( book ), _( "Footprint Editor" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_FP_DISPLAY_OPTIONS ), _( "Display Options" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_FP_EDIT_OPTIONS ), _( "Editing Options" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_FP_COLORS ), _( "Colors" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_FP_DEFAULT_VALUES ), _( "Default Values" ) );
-
-        if( GetFrameType() ==  FRAME_PCB_EDITOR )
-            expand.push_back( book->GetPageCount() );
-
-        book->AddPage( new wxPanel( book ), _( "PCB Editor" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_PCB_DISPLAY_OPTIONS ), _( "Display Options" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_PCB_EDIT_OPTIONS ), _( "Editing Options" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_PCB_COLORS ), _( "Colors" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_PCB_ACTION_PLUGINS ), _( "Action Plugins" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_PCB_ORIGINS_AXES ), _( "Origins & Axes" ) );
-
-        if( GetFrameType() == FRAME_PCB_DISPLAY3D )
-            expand.push_back( book->GetPageCount() );
-
-        book->AddPage( new wxPanel( book ), _( "3D Viewer" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_3DV_DISPLAY_OPTIONS ), _( "General" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_3DV_OPENGL ), _( "Realtime Renderer" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_3DV_RAYTRACING ), _( "Raytracing Renderer" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_3DV_COLORS ), _( "Colors" ) );
-    }
-    catch( ... )
-    {
-    }
-
-    try
-    {
-        kiface = Kiway().KiFACE( KIWAY::FACE_GERBVIEW );
-
-        kiface->GetActions( hotkeysPanel->ActionsList() );
-
-        if( GetFrameType() == FRAME_GERBER )
-            expand.push_back( book->GetPageCount() );
-
-        book->AddPage( new wxPanel( book ), _( "Gerber Viewer" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_GBR_DISPLAY_OPTIONS ), _( "Display Options" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_GBR_COLORS ), _( "Colors" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_GBR_EXCELLON_OPTIONS ), _( "Excellon Options" ) );
-    }
-    catch( ... )
-    {
-    }
-
-    try
-    {
-        kiface = Kiway().KiFACE( KIWAY::FACE_PL_EDITOR );
-        kiface->GetActions( hotkeysPanel->ActionsList() );
-
-        if( GetFrameType() == FRAME_PL_EDITOR )
-            expand.push_back( book->GetPageCount() );
-
-        book->AddPage( new wxPanel( book ), _( "Drawing Sheet Editor" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_DS_DISPLAY_OPTIONS ), _( "Display Options" ) );
-        book->AddSubPage( CREATE_PANEL( PANEL_DS_COLORS ), _( "Colors" ) );
-
-        book->AddPage( new PANEL_PCM_SETTINGS( book ), _( "Plugin and Content Manager" ) );
-    }
-    catch( ... )
-    {
-    }
-
-    // Update all of the action hotkeys. The process of loading the actions through
-    // the KiFACE will only get us the default hotkeys
-    ReadHotKeyConfigIntoActions( wxEmptyString, hotkeysPanel->ActionsList() );
-
-    for( size_t i = 0; i < book->GetPageCount(); ++i )
-        book->GetPage( i )->Layout();
-
-    for( int page : expand )
-        book->ExpandNode( page );
-
-    if( !aStartPage.IsEmpty() )
-        dlg.SetInitialPage( aStartPage, aStartParentPage );
-
-    dlg.SetEvtHandlerEnabled( true );
-    wxEndBusyCursor();
 
     if( dlg.ShowModal() == wxID_OK )
     {
+        // Update our grids that are cached in the tool
+        m_toolManager->ResetTools( TOOL_BASE::REDRAW );
         Pgm().GetSettingsManager().Save();
-        dlg.Kiway().CommonSettingsChanged( false, false );
+        dlg.Kiway().CommonSettingsChanged( HOTKEYS_CHANGED );
     }
 
-#undef CREATE_PANEL
 }
 
 
@@ -1167,8 +1304,8 @@ void EDA_BASE_FRAME::OnDropFiles( wxDropFilesEvent& aEvent )
         wxString         ext = fn.GetExt();
 
         // Alias all gerber files as GerberFileExtension
-        if( IsGerberFileExtension( ext ) )
-            ext = GerberFileExtension;
+        if( FILEEXT::IsGerberFileExtension( ext ) )
+            ext = FILEEXT::GerberFileExtension;
 
         if( m_acceptedExts.find( ext.ToStdString() ) != m_acceptedExts.end() )
             m_AcceptedFiles.emplace_back( fn );
@@ -1184,7 +1321,7 @@ void EDA_BASE_FRAME::DoWithAcceptedFiles()
     for( const wxFileName& file : m_AcceptedFiles )
     {
         wxString fn = file.GetFullPath();
-        m_toolManager->RunAction( *m_acceptedExts.at( file.GetExt() ), true, &fn );
+        m_toolManager->RunAction<wxString*>( *m_acceptedExts.at( file.GetExt() ), &fn );
     }
 }
 
@@ -1242,7 +1379,7 @@ void EDA_BASE_FRAME::CheckForAutoSaveFile( const wxFileName& aFileName )
     wxFileName autoSaveFileName = aFileName;
 
     // Check for auto save file.
-    autoSaveFileName.SetName( GetAutoSaveFilePrefix() + aFileName.GetName() );
+    autoSaveFileName.SetName( FILEEXT::AutoSaveFilePrefix + aFileName.GetName() );
 
     wxLogTrace( traceAutoSave,
                 wxT( "Checking for auto save file " ) + autoSaveFileName.GetFullPath() );
@@ -1265,6 +1402,10 @@ void EDA_BASE_FRAME::CheckForAutoSaveFile( const wxFileName& aFileName )
     // the file name.
     if( response == wxYES )
     {
+        // Preserve the permissions of the current file
+        KIPLATFORM::IO::DuplicatePermissions( aFileName.GetFullPath(),
+                                              autoSaveFileName.GetFullPath() );
+
         if( !wxRenameFile( autoSaveFileName.GetFullPath(), aFileName.GetFullPath() ) )
         {
             wxMessageBox( _( "The auto save file could not be renamed to the board file name." ),
@@ -1273,11 +1414,25 @@ void EDA_BASE_FRAME::CheckForAutoSaveFile( const wxFileName& aFileName )
     }
     else
     {
-        wxLogTrace( traceAutoSave,
-                    wxT( "Removing auto save file " ) + autoSaveFileName.GetFullPath() );
+        DeleteAutoSaveFile( aFileName );
+    }
+}
 
-        // Remove the auto save file when using the previous file as is.
-        wxRemoveFile( autoSaveFileName.GetFullPath() );
+
+void EDA_BASE_FRAME::DeleteAutoSaveFile( const wxFileName& aFileName )
+{
+    if( !Pgm().IsGUI() )
+        return;
+
+    wxCHECK_RET( aFileName.IsOk(), wxT( "Invalid file name!" ) );
+
+    wxFileName autoSaveFn = aFileName;
+    autoSaveFn.SetName( FILEEXT::AutoSaveFilePrefix + aFileName.GetName() );
+
+    if( autoSaveFn.FileExists() )
+    {
+        wxLogTrace( traceAutoSave, wxT( "Removing auto save file " ) + autoSaveFn.GetFullPath() );
+        wxRemoveFile( autoSaveFn.GetFullPath() );
     }
 }
 
@@ -1347,6 +1502,24 @@ PICKED_ITEMS_LIST* EDA_BASE_FRAME::PopCommandFromRedoList( )
 }
 
 
+wxString EDA_BASE_FRAME::GetUndoActionDescription() const
+{
+    if( GetUndoCommandCount() > 0 )
+        return m_undoList.m_CommandsList.back()->GetDescription();
+
+    return wxEmptyString;
+}
+
+
+wxString EDA_BASE_FRAME::GetRedoActionDescription() const
+{
+    if( GetRedoCommandCount() > 0 )
+        return m_redoList.m_CommandsList.back()->GetDescription();
+
+    return wxEmptyString;
+}
+
+
 void EDA_BASE_FRAME::OnModify()
 {
     m_autoSaveRequired = true;
@@ -1358,7 +1531,8 @@ void EDA_BASE_FRAME::ChangeUserUnits( EDA_UNITS aUnits )
     SetUserUnits( aUnits );
     unitsChangeRefresh();
 
-    wxCommandEvent e( UNITS_CHANGED );
+    wxCommandEvent e( EDA_EVT_UNITS_CHANGED );
+    e.SetInt( static_cast<int>( aUnits ) );
     e.SetClientData( this );
     ProcessEventLocally( e );
 }
@@ -1448,7 +1622,7 @@ void EDA_BASE_FRAME::onIconize( wxIconizeEvent& aEvent )
 }
 
 
-#ifdef _WIN32
+#ifdef __WXMSW__
 WXLRESULT EDA_BASE_FRAME::MSWWindowProc( WXUINT message, WXWPARAM wParam, WXLPARAM lParam )
 {
     // This will help avoid the menu keeping focus when the alt key is released
@@ -1462,3 +1636,33 @@ WXLRESULT EDA_BASE_FRAME::MSWWindowProc( WXUINT message, WXWPARAM wParam, WXLPAR
     return wxFrame::MSWWindowProc( message, wParam, lParam );
 }
 #endif
+
+
+void EDA_BASE_FRAME::AddMenuLanguageList( ACTION_MENU* aMasterMenu, TOOL_INTERACTIVE* aControlTool )
+{
+    ACTION_MENU* langsMenu = new ACTION_MENU( false, aControlTool );
+    langsMenu->SetTitle( _( "Set Language" ) );
+    langsMenu->SetIcon( BITMAPS::language );
+
+    wxString tooltip;
+
+    for( unsigned ii = 0; LanguagesList[ii].m_KI_Lang_Identifier != 0; ii++ )
+    {
+        wxString label;
+
+        if( LanguagesList[ii].m_DoNotTranslate )
+            label = LanguagesList[ii].m_Lang_Label;
+        else
+            label = wxGetTranslation( LanguagesList[ii].m_Lang_Label );
+
+        wxMenuItem* item =
+                new wxMenuItem( langsMenu,
+                                LanguagesList[ii].m_KI_Lang_Identifier, // wxMenuItem wxID
+                                label, tooltip, wxITEM_CHECK );
+
+        langsMenu->Append( item );
+    }
+
+    // This must be done after the items are added
+    aMasterMenu->Add( langsMenu );
+}
