@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2012 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
- * Copyright (C) 2012-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,18 +28,22 @@
 #include <wx/tokenzr.h>
 #include <wx/clipbrd.h>
 #include <wx/log.h>
-#include <widgets/grid_readonly_text_helpers.h>
+#include <wx/stc/stc.h>
+#include <widgets/grid_text_helpers.h>
 
 
 // It works for table data on clipboard for an Excel spreadsheet,
 // why not us too for now.
 #define COL_SEP     wxT( '\t' )
 #define ROW_SEP     wxT( '\n' )
+#define ROW_SEP_R   wxT( '\r' )
 
 
 GRID_TRICKS::GRID_TRICKS( WX_GRID* aGrid ) :
     m_grid( aGrid ),
-    m_addHandler( []( wxCommandEvent& ) {} )
+    m_addHandler( []( wxCommandEvent& ) {} ),
+    m_enableSingleClickEdit( true ),
+    m_multiCellEditEnabled( true )
 {
     init();
 }
@@ -47,7 +51,9 @@ GRID_TRICKS::GRID_TRICKS( WX_GRID* aGrid ) :
 
 GRID_TRICKS::GRID_TRICKS( WX_GRID* aGrid, std::function<void( wxCommandEvent& )> aAddHandler ) :
     m_grid( aGrid ),
-    m_addHandler( aAddHandler )
+    m_addHandler( aAddHandler ),
+    m_enableSingleClickEdit( true ),
+    m_multiCellEditEnabled( true )
 {
     init();
 }
@@ -86,13 +92,36 @@ void GRID_TRICKS::init()
 }
 
 
-bool GRID_TRICKS::toggleCell( int aRow, int aCol, bool aPreserveSelection )
+bool GRID_TRICKS::isTextEntry( int aRow, int aCol )
+{
+    wxGridCellEditor* editor = m_grid->GetCellEditor( aRow, aCol );
+    bool              retval = ( dynamic_cast<wxTextEntry*>( editor )
+                              || dynamic_cast<GRID_CELL_STC_EDITOR*>( editor ) );
+
+    editor->DecRef();
+    return retval;
+}
+
+
+bool GRID_TRICKS::isCheckbox( int aRow, int aCol )
 {
     wxGridCellRenderer* renderer = m_grid->GetCellRenderer( aRow, aCol );
-    bool                isCheckbox = ( dynamic_cast<wxGridCellBoolRenderer*>( renderer ) );
-    renderer->DecRef();
+    bool                retval = ( dynamic_cast<wxGridCellBoolRenderer*>( renderer ) );
 
-    if( isCheckbox )
+    renderer->DecRef();
+    return retval;
+}
+
+
+bool GRID_TRICKS::isReadOnly( int aRow, int aCol )
+{
+    return !m_grid->IsEditable() || m_grid->IsReadOnly( aRow, aCol );
+}
+
+
+bool GRID_TRICKS::toggleCell( int aRow, int aCol, bool aPreserveSelection )
+{
+    if( isCheckbox( aRow, aCol ) )
     {
         if( !aPreserveSelection )
         {
@@ -135,10 +164,15 @@ bool GRID_TRICKS::showEditor( int aRow, int aCol )
     if( m_grid->GetGridCursorRow() != aRow || m_grid->GetGridCursorCol() != aCol )
         m_grid->SetGridCursor( aRow, aCol );
 
-    if( m_grid->IsEditable() && !m_grid->IsReadOnly( aRow, aCol ) )
+    if( !isReadOnly( aRow, aCol ) )
     {
         m_grid->ClearSelection();
 
+        m_sel_row_start = aRow;
+        m_sel_col_start = aCol;
+        m_sel_row_count = 1;
+        m_sel_col_count = 1;
+
         if( m_grid->GetSelectionMode() == wxGrid::wxGridSelectRows )
         {
             wxArrayInt rows = m_grid->GetSelectedRows();
@@ -177,7 +211,7 @@ void GRID_TRICKS::onGridCellLeftClick( wxGridEvent& aEvent )
 
         if( toggleCell( row, col, true ) )
             toggled = true;
-        else if( showEditor( row, col ) )
+        else if( m_enableSingleClickEdit && showEditor( row, col ) )
             return;
 
         // Apply checkbox changes to multi-selection.
@@ -289,11 +323,11 @@ void GRID_TRICKS::getSelectedArea()
 }
 
 
-void GRID_TRICKS::onGridCellRightClick( wxGridEvent&  )
+void GRID_TRICKS::onGridCellRightClick( wxGridEvent& aEvent  )
 {
     wxMenu menu;
 
-    showPopupMenu( menu );
+    showPopupMenu( menu, aEvent );
 }
 
 
@@ -320,42 +354,62 @@ void GRID_TRICKS::onGridLabelRightClick( wxGridEvent&  )
 }
 
 
-void GRID_TRICKS::showPopupMenu( wxMenu& menu )
+void GRID_TRICKS::showPopupMenu( wxMenu& menu, wxGridEvent& aEvent )
 {
     menu.Append( GRIDTRICKS_ID_CUT, _( "Cut" ) + "\tCtrl+X",
                  _( "Clear selected cells placing original contents on clipboard" ) );
     menu.Append( GRIDTRICKS_ID_COPY, _( "Copy" ) + "\tCtrl+C",
                  _( "Copy selected cells to clipboard" ) );
-    menu.Append( GRIDTRICKS_ID_PASTE, _( "Paste" ) + "\tCtrl+V",
-                 _( "Paste clipboard cells to matrix at current cell" ) );
-    menu.Append( GRIDTRICKS_ID_DELETE, _( "Delete" ) + "\tDel", _( "Delete selected cells" ) );
-    menu.Append( GRIDTRICKS_ID_SELECT, _( "Select All" ) + "\tCtrl+A",  _( "Select all cells" ) );
+
+    if( m_multiCellEditEnabled )
+    {
+        menu.Append( GRIDTRICKS_ID_PASTE, _( "Paste" ) + "\tCtrl+V",
+                     _( "Paste clipboard cells to matrix at current cell" ) );
+        menu.Append( GRIDTRICKS_ID_DELETE, _( "Delete" ) + "\tDel",
+                     _( "Clear contents of selected cells" ) );
+    }
+
+    menu.Append( GRIDTRICKS_ID_SELECT, _( "Select All" ) + "\tCtrl+A",
+                 _( "Select all cells" ) );
+
+    menu.Enable( GRIDTRICKS_ID_CUT,  false );
+    menu.Enable( GRIDTRICKS_ID_DELETE, false );
+    menu.Enable( GRIDTRICKS_ID_PASTE, false );
 
     getSelectedArea();
 
-    // if nothing is selected, disable cut, copy and delete.
-    if( !m_sel_row_count && !m_sel_col_count )
+    auto anyCellsWritable =
+            [&]()
+            {
+                for( int row = m_sel_row_start; row < m_sel_row_start + m_sel_row_count; ++row )
+                {
+                    for( int col = m_sel_col_start; col < m_sel_col_start + m_sel_col_count; ++col )
+                    {
+                        if( !isReadOnly( row, col ) && isTextEntry( row, col ) )
+                            return true;
+                    }
+                }
+
+                return false;
+            };
+
+    if( anyCellsWritable() )
     {
-        menu.Enable( GRIDTRICKS_ID_CUT,  false );
-        menu.Enable( GRIDTRICKS_ID_COPY, false );
-        menu.Enable( GRIDTRICKS_ID_DELETE, false );
-    }
-    else if( !m_grid->IsEditable() )
-    {
-        menu.Enable( GRIDTRICKS_ID_CUT,  false );
-        menu.Enable( GRIDTRICKS_ID_DELETE, false );
+        menu.Enable( GRIDTRICKS_ID_CUT,  true );
+        menu.Enable( GRIDTRICKS_ID_DELETE, true );
     }
 
-    menu.Enable( GRIDTRICKS_ID_PASTE, false );
+    // Paste can overflow the selection, so don't depend on the particular cell being writeable.
 
     wxLogNull doNotLog; // disable logging of failed clipboard actions
 
-    if( m_grid->IsEditable() && wxTheClipboard->Open() )
+    if( wxTheClipboard->Open() )
     {
         if( wxTheClipboard->IsSupported( wxDF_TEXT )
             || wxTheClipboard->IsSupported( wxDF_UNICODETEXT ) )
         {
-            menu.Enable( GRIDTRICKS_ID_PASTE, true );
+            if( m_grid->IsEditable() )
+                menu.Enable( GRIDTRICKS_ID_PASTE, true );
         }
 
         wxTheClipboard->Close();
@@ -401,7 +455,7 @@ void GRID_TRICKS::doPopupSelection( wxCommandEvent& event )
         break;
 
     default:
-        if( menu_id >= GRIDTRICKS_FIRST_SHOWHIDE )
+        if( menu_id >= GRIDTRICKS_FIRST_SHOWHIDE && m_grid->CommitPendingChanges( false ) )
         {
             int col = menu_id - GRIDTRICKS_FIRST_SHOWHIDE;
 
@@ -418,9 +472,16 @@ void GRID_TRICKS::onCharHook( wxKeyEvent& ev )
 {
     bool handled = false;
 
-    if( ev.GetKeyCode() == WXK_RETURN && m_grid->GetGridCursorRow() == m_grid->GetNumberRows() - 1 )
+    if( ( ev.GetKeyCode() == WXK_RETURN || ev.GetKeyCode() == WXK_NUMPAD_ENTER )
+        && ev.GetModifiers() == wxMOD_NONE
+        && m_grid->GetGridCursorRow() == m_grid->GetNumberRows() - 1 )
     {
-        if( m_grid->CommitPendingChanges() )
+        if( m_grid->IsCellEditControlShown() )
+        {
+            if( m_grid->CommitPendingChanges() )
+                handled = true;
+        }
+        else
         {
             wxCommandEvent dummy;
             m_addHandler( dummy );
@@ -439,8 +500,17 @@ void GRID_TRICKS::onCharHook( wxKeyEvent& ev )
 
                 if( data.GetText().Contains( COL_SEP ) || data.GetText().Contains( ROW_SEP ) )
                 {
-                    m_grid->CommitPendingChanges( true /* quiet mode */ );
-                    paste_text( data.GetText() );
+                    wxString stripped( data.GetText() );
+                    stripped.Replace( ROW_SEP, " " );
+                    stripped.Replace( ROW_SEP_R, " " );
+                    stripped.Replace( COL_SEP, " " );
+
+                    // Write to the CellEditControl if we can
+                    if( wxTextEntry* te = dynamic_cast<wxTextEntry*>( ev.GetEventObject() ) )
+                        te->WriteText( stripped );
+                    else
+                        paste_text( stripped );
+
                     handled = true;
                 }
             }
@@ -449,6 +519,14 @@ void GRID_TRICKS::onCharHook( wxKeyEvent& ev )
             m_grid->ForceRefresh();
         }
     }
+    else if( ev.GetKeyCode() == WXK_ESCAPE )
+    {
+        if( m_grid->IsCellEditControlShown() )
+        {
+            m_grid->CancelPendingChanges();
+            handled = true;
+        }
+    }
 
     if( !handled )
         ev.Skip( true );
@@ -498,9 +576,7 @@ void GRID_TRICKS::onKeyDown( wxKeyEvent& ev )
             wxArrayInt rowSel = m_grid->GetSelectedRows();
 
             for( unsigned int rowInd = 0; rowInd < rowSel.GetCount(); rowInd++ )
-            {
                 retVal |= toggleCell( rowSel[rowInd], 0, true );
-            }
         }
 
         // If only columns can be selected, only toggle the first cell in a column
@@ -509,9 +585,7 @@ void GRID_TRICKS::onKeyDown( wxKeyEvent& ev )
             wxArrayInt colSel = m_grid->GetSelectedCols();
 
             for( unsigned int colInd = 0; colInd < colSel.GetCount(); colInd++ )
-            {
                 retVal |= toggleCell( 0, colSel[colInd], true );
-            }
         }
 
         // If the user can select the individual cells, toggle each cell selected
@@ -533,18 +607,14 @@ void GRID_TRICKS::onKeyDown( wxKeyEvent& ev )
             for( unsigned int colInd = 0; colInd < colSel.GetCount(); colInd++ )
             {
                 for( int row = 0; row < m_grid->GetNumberRows(); row++ )
-                {
                     retVal |= toggleCell( row, colSel[colInd], true );
-                }
             }
 
             // Iterate over every row and try to toggle each cell in it
             for( unsigned int rowInd = 0; rowInd < rowSel.GetCount(); rowInd++ )
             {
                 for( int col = 0; col < m_grid->GetNumberCols(); col++ )
-                {
                     retVal |= toggleCell( rowSel[rowInd], col, true );
-                }
             }
 
             // Iterate over the selection blocks
@@ -556,9 +626,7 @@ void GRID_TRICKS::onKeyDown( wxKeyEvent& ev )
                 for( int row = start.GetRow(); row <= end.GetRow(); row++ )
                 {
                     for( int col = start.GetCol(); col <= end.GetCol(); col++ )
-                    {
                         retVal |= toggleCell( row, col, true );
-                    }
                 }
             }
         }
@@ -636,7 +704,14 @@ void GRID_TRICKS::paste_clipboard()
 
             wxTheClipboard->GetData( data );
 
-            paste_text( data.GetText() );
+            wxString text = data.GetText();
+
+#ifdef __WXMAC__
+            // Some editors use windows linefeeds (\r\n), which wx re-writes to \n\n
+            text.Replace( "\n\n", "\n" );
+#endif
+
+            paste_text( text );
         }
 
         wxTheClipboard->Close();
@@ -647,6 +722,9 @@ void GRID_TRICKS::paste_clipboard()
 
 void GRID_TRICKS::paste_text( const wxString& cb_text )
 {
+    if( !m_multiCellEditEnabled )
+        return;
+
     wxGridTableBase*   tbl = m_grid->GetTable();
 
     const int cur_row = m_grid->GetGridCursorRow();
@@ -668,10 +746,9 @@ void GRID_TRICKS::paste_text( const wxString& cb_text )
         if( m_sel_row_count > 1 )
             is_selection = true;
     }
-    else
+    else if( m_sel_col_count > 1 || m_sel_row_count > 1 )
     {
-        if( m_grid->IsSelection() )
-            is_selection = true;
+        is_selection = true;
     }
 
     wxStringTokenizer rows( cb_text, ROW_SEP, wxTOKEN_RET_EMPTY );
@@ -693,7 +770,18 @@ void GRID_TRICKS::paste_text( const wxString& cb_text )
         end_row = cur_row + rows.CountTokens();
 
         if( end_row > tbl->GetNumberRows() )
+        {
+            if( m_addHandler )
+            {
+                for( int ii = end_row - tbl->GetNumberRows(); ii > 0; --ii )
+                {
+                    wxCommandEvent dummy;
+                    m_addHandler( dummy );
+                }
+            }
+
             end_row = tbl->GetNumberRows();
+        }
 
         start_col = cur_col;
         end_col = start_col; // end_col actual value calculates later
@@ -711,18 +799,16 @@ void GRID_TRICKS::paste_text( const wxString& cb_text )
         wxStringTokenizer cols( rowTxt, COL_SEP, wxTOKEN_RET_EMPTY );
 
         if( !is_selection )
-        {
             end_col = cur_col + cols.CountTokens();
 
-            if( end_col > tbl->GetNumberCols() )
-                end_col = tbl->GetNumberCols();
-        }
-
-        for( int col = start_col;  col < end_col;  ++col )
+        for( int col = start_col;  col < end_col && col < tbl->GetNumberCols();  ++col )
         {
             // Skip hidden columns
             if( !m_grid->IsColShown( col ) )
+            {
+                end_col++;
                 continue;
+            }
 
             // If number of selected cols is larger than the count of cols on the clipboard,
             // paste again and again until the end of the selection is reached.
@@ -731,10 +817,20 @@ void GRID_TRICKS::paste_text( const wxString& cb_text )
 
             wxString cellTxt = cols.GetNextToken();
 
-            if( tbl->CanSetValueAs( row, col, wxGRID_VALUE_STRING ) )
+            // Allow paste to anything that can take a string, including things like color
+            // swatches and checkboxes
+            if( tbl->CanSetValueAs( row, col, wxGRID_VALUE_STRING ) && !isReadOnly( row, col ) )
             {
                 tbl->SetValue( row, col, cellTxt );
 
+                wxGridEvent evt( m_grid->GetId(), wxEVT_GRID_CELL_CHANGED, m_grid, row, col );
+                m_grid->GetEventHandler()->ProcessEvent( evt );
+            }
+            // Allow paste to any cell that can accept a boolean value
+            else if( tbl->CanSetValueAs( row, col, wxGRID_VALUE_BOOL ) )
+            {
+                tbl->SetValueAsBool( row, col, cellTxt == wxT( "1" ) );
+
                 wxGridEvent evt( m_grid->GetId(), wxEVT_GRID_CELL_CHANGED, m_grid, row, col );
                 m_grid->GetEventHandler()->ProcessEvent( evt );
             }
@@ -766,9 +862,11 @@ void GRID_TRICKS::cutcopy( bool doCopy, bool doDelete )
             if( col < m_sel_col_start + m_sel_col_count - 1 )   // that was not last column
                 txt += COL_SEP;
 
-            if( doDelete && m_grid->IsEditable() )
+            if( doDelete )
             {
-                if( tbl->CanSetValueAs( row, col, wxGRID_VALUE_STRING ) )
+                // Do NOT allow clear of things that can take strings but aren't textEntries
+                // (ie: color swatches, textboxes, etc.).
+                if( isTextEntry( row, col ) && !isReadOnly( row, col ) )
                     tbl->SetValue( row, col, wxEmptyString );
             }
         }
