@@ -4,7 +4,7 @@
  * Copyright (C) 2017 Chris Pavlina <pavlina.chris@gmail.com>
  * Copyright (C) 2014 Henner Zeller <h.zeller@acm.org>
  * Copyright (C) 2023 CERN
- * Copyright (C) 2014-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -87,15 +87,17 @@ bool LIB_TREE_NODE::Compare( LIB_TREE_NODE const& aNode1, LIB_TREE_NODE const& a
         return aNode1.m_Type < aNode2.m_Type;
 
     // Recently used sorts at top
-    if( aNode1.m_Name.StartsWith( wxT( "-- " ) ) )
+    if( aNode1.m_IsRecentlyUsedGroup )
     {
-        if( aNode2.m_Name.StartsWith( wxT( "-- " ) ) )
+        if( aNode2.m_IsRecentlyUsedGroup )
         {
-            // Make sure -- Recently Used is always at the top
-            if( aNode1.m_Name.StartsWith( wxT( "-- Recently Used" ) ) )
-                return true;
-            else if( aNode2.m_Name.StartsWith( wxT( "-- Recently Used" ) ) )
+            // Make sure "-- Recently Used" is always at the top
+            // Start by checking the name of aNode2, because we want to satisfy the irreflexive
+            // property of the strict weak ordering.
+            if( aNode2.m_IsRecentlyUsedGroup )
                 return false;
+            else if( aNode1.m_IsRecentlyUsedGroup )
+                return true;
 
             return aNode1.m_IntrinsicRank > aNode2.m_IntrinsicRank;
         }
@@ -127,13 +129,15 @@ bool LIB_TREE_NODE::Compare( LIB_TREE_NODE const& aNode1, LIB_TREE_NODE const& a
 
 LIB_TREE_NODE::LIB_TREE_NODE()
     : m_Parent( nullptr ),
-      m_Type( INVALID ),
+      m_Type( TYPE::INVALID ),
       m_IntrinsicRank( 0 ),
       m_Score( 0 ),
       m_Pinned( false ),
       m_PinCount( 0 ),
       m_Unit( 0 ),
-      m_IsRoot( false )
+      m_IsRoot( false ),
+      m_IsRecentlyUsedGroup( false ),
+      m_IsAlreadyPlacedGroup( false )
 {}
 
 
@@ -151,7 +155,7 @@ LIB_TREE_NODE_UNIT::LIB_TREE_NODE_UNIT( LIB_TREE_NODE* aParent, LIB_TREE_ITEM* a
     }
 
     m_Parent = aParent;
-    m_Type = UNIT;
+    m_Type = TYPE::UNIT;
 
     m_Unit = aUnit;
     m_LibId = aParent->m_LibId;
@@ -186,14 +190,14 @@ void LIB_TREE_NODE_UNIT::UpdateScore( EDA_COMBINED_MATCHER* aMatcher, const wxSt
 
 LIB_TREE_NODE_ITEM::LIB_TREE_NODE_ITEM( LIB_TREE_NODE* aParent, LIB_TREE_ITEM* aItem )
 {
-    m_Type = ITEM;
+    m_Type = TYPE::ITEM;
     m_Parent = aParent;
 
     m_LibId.SetLibNickname( aItem->GetLibNickname() );
     m_LibId.SetLibItemName( aItem->GetName() );
 
     m_Name = aItem->GetName();
-    m_Desc = aItem->GetDescription();
+    m_Desc = aItem->GetDesc();
     m_Footprint = aItem->GetFootprint();
     m_PinCount = aItem->GetPinCount();
 
@@ -203,9 +207,9 @@ LIB_TREE_NODE_ITEM::LIB_TREE_NODE_ITEM( LIB_TREE_NODE* aParent, LIB_TREE_ITEM* a
 
     m_IsRoot = aItem->IsRoot();
 
-    if( aItem->GetUnitCount() > 1 )
+    if( aItem->GetSubUnitCount() > 1 )
     {
-        for( int u = 1; u <= aItem->GetUnitCount(); ++u )
+        for( int u = 1; u <= aItem->GetSubUnitCount(); ++u )
             AddUnit( aItem, u );
     }
 }
@@ -221,11 +225,11 @@ LIB_TREE_NODE_UNIT& LIB_TREE_NODE_ITEM::AddUnit( LIB_TREE_ITEM* aItem, int aUnit
 
 void LIB_TREE_NODE_ITEM::Update( LIB_TREE_ITEM* aItem )
 {
-    m_LibId.SetLibNickname( aItem->GetLibId().GetLibNickname() );
+    m_LibId.SetLibNickname( aItem->GetLIB_ID().GetLibNickname() );
     m_LibId.SetLibItemName( aItem->GetName() );
 
     m_Name = aItem->GetName();
-    m_Desc = aItem->GetDescription();
+    m_Desc = aItem->GetDesc();
 
     aItem->GetChooserFields( m_Fields );
 
@@ -234,7 +238,7 @@ void LIB_TREE_NODE_ITEM::Update( LIB_TREE_ITEM* aItem )
     m_IsRoot = aItem->IsRoot();
     m_Children.clear();
 
-    for( int u = 1; u <= aItem->GetUnitCount(); ++u )
+    for( int u = 1; u <= aItem->GetSubUnitCount(); ++u )
         AddUnit( aItem, u );
 }
 
@@ -274,7 +278,7 @@ void LIB_TREE_NODE_ITEM::UpdateScore( EDA_COMBINED_MATCHER* aMatcher, const wxSt
 LIB_TREE_NODE_LIBRARY::LIB_TREE_NODE_LIBRARY( LIB_TREE_NODE* aParent, wxString const& aName,
                                               wxString const& aDesc )
 {
-    m_Type = LIBRARY;
+    m_Type = TYPE::LIBRARY;
     m_Name = aName;
     m_Desc = aDesc;
     m_Parent = aParent;
@@ -297,41 +301,45 @@ void LIB_TREE_NODE_LIBRARY::UpdateScore( EDA_COMBINED_MATCHER* aMatcher, const w
 {
     int maxChildScore = 0;
 
-    for( std::unique_ptr<LIB_TREE_NODE>& child: m_Children )
+    // children are additive
+    if( !m_Children.empty() )
     {
-        child->UpdateScore( aMatcher, aLib, aFilter );
-        maxChildScore = std::max( maxChildScore, child->m_Score );
-    }
+        for( std::unique_ptr<LIB_TREE_NODE>& child: m_Children )
+        {
+            child->UpdateScore( aMatcher, aLib, aFilter );
+            maxChildScore = std::max( maxChildScore, child->m_Score );
+        }
 
-    // Each time UpdateScore is called for a library, child (item) scores may go up or down.
-    // If the all go down to zero, we need to make sure to drop the library from the list.
-    if( maxChildScore > 0 )
         m_Score = std::max( m_Score, maxChildScore );
-    else
-        m_Score = 0;
-
-    // aLib test is additive, but only when we've already accumulated some score from children
-    if( !aLib.IsEmpty()
-        && m_Name.Lower().Matches( aLib )
-        && ( m_Score > 0 || m_Children.empty() ) )
-    {
-        m_Score += 1;
     }
 
+    // aLib test is additive
+    if( !aLib.IsEmpty() && m_Name.Lower().Matches( aLib ) )
+        m_Score += 1;
+
     // aMatcher test is additive
     if( aMatcher )
     {
         int ownScore = aMatcher->ScoreTerms( m_SearchTerms );
         m_Score += ownScore;
 
-        // If we have a hit on a library, show all children in that library
+        // If we have a hit on a library, show all children in that library that pass the filter
         if( maxChildScore <= 0 && ownScore > 0 )
         {
             for( std::unique_ptr<LIB_TREE_NODE>& child: m_Children )
-                child->ForceScore( 1 );
+            {
+                if( !aFilter || (*aFilter)( *this ) )
+                    child->ForceScore( 1 );
+
+                maxChildScore = std::max( maxChildScore, child->m_Score );
+            }
         }
     }
 
+    // If all children are excluded for one reason or another, drop the library from the list
+    if( !m_Children.empty() && maxChildScore <= 0 )
+        m_Score = 0;
+
     // show all nodes if no search/filter/etc. criteria are given
     if( m_Children.empty() && !aMatcher && aLib.IsEmpty() && ( !aFilter || (*aFilter)(*this) ) )
         m_Score = 1;
@@ -340,7 +348,7 @@ void LIB_TREE_NODE_LIBRARY::UpdateScore( EDA_COMBINED_MATCHER* aMatcher, const w
 
 LIB_TREE_NODE_ROOT::LIB_TREE_NODE_ROOT()
 {
-    m_Type = ROOT;
+    m_Type = TYPE::ROOT;
 }
 
 
@@ -352,6 +360,29 @@ LIB_TREE_NODE_LIBRARY& LIB_TREE_NODE_ROOT::AddLib( wxString const& aName, wxStri
 }
 
 
+void LIB_TREE_NODE_ROOT::RemoveGroup( bool aRecentlyUsedGroup, bool aAlreadyPlacedGroup )
+{
+    m_Children.erase( std::remove_if( m_Children.begin(), m_Children.end(),
+                                      [&]( std::unique_ptr<LIB_TREE_NODE>& aNode )
+                                      {
+                                          if( aRecentlyUsedGroup && aNode->m_IsRecentlyUsedGroup )
+                                              return true;
+
+                                          if( aAlreadyPlacedGroup && aNode->m_IsAlreadyPlacedGroup )
+                                              return true;
+
+                                          return false;
+                                      } ),
+                      m_Children.end() );
+}
+
+
+void LIB_TREE_NODE_ROOT::Clear()
+{
+    m_Children.clear();
+}
+
+
 void LIB_TREE_NODE_ROOT::UpdateScore( EDA_COMBINED_MATCHER* aMatcher, const wxString& aLib,
                                       std::function<bool( LIB_TREE_NODE& aNode )>* aFilter )
 {
