@@ -5,7 +5,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2017-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,6 +30,7 @@
 #include <string_utils.h>
 #include <convert_basic_shapes_to_polygon.h>
 #include <trigo.h>
+#include <fmt/core.h>
 
 /**
  * Oblique angle for DXF native text
@@ -37,19 +38,21 @@
  */
 static const double DXF_OBLIQUE_ANGLE = 15;
 
-/* The layer/colors palette. The acad/DXF palette is divided in 3 zones:
+/**
+ * The layer/colors palette.
+ *
+ * The acad/DXF palette is divided in 3 zones:
+ *
+ *  - The primary colors (1 - 9)
+ *  - An HSV zone (10-250, 5 values x 2 saturations x 10 hues
+ *  - Greys (251 - 255)
 
-   - The primary colors (1 - 9)
-   - An HSV zone (10-250, 5 values x 2 saturations x 10 hues
-   - Greys (251 - 255)
-
-   There is *no* black... the white does it on paper, usually, and
-   anyway it depends on the plotter configuration, since DXF colors
-   are meant to be logical only (they represent *both* line color and
-   width); later version with plot styles only complicate the matter!
-
-   As usual, brown and magenta/purple are difficult to place since
-   they are actually variations of other colors.
+ * There is *no* black... the white does it on paper, usually, and anyway it depends on the
+ * plotter configuration, since DXF colors are meant to be logical only (they represent *both*
+ * line color and width); later version with plot styles only complicate the matter!
+ *
+ * As usual, brown and magenta/purple are difficult to place since they are actually variations
+ * of other colors.
  */
 static const struct
 {
@@ -95,21 +98,21 @@ static const struct
 };
 
 
-static const char* getDXFLineType( PLOT_DASH_TYPE aType )
+static const char* getDXFLineType( LINE_STYLE aType )
 {
     switch( aType )
     {
-    case PLOT_DASH_TYPE::DEFAULT:
-    case PLOT_DASH_TYPE::SOLID:
+    case LINE_STYLE::DEFAULT:
+    case LINE_STYLE::SOLID:
         return "CONTINUOUS";
-    case PLOT_DASH_TYPE::DASH:
+    case LINE_STYLE::DASH:
         return "DASHED";
-    case PLOT_DASH_TYPE::DOT:
+    case LINE_STYLE::DOT:
         return "DOTTED";
-    case PLOT_DASH_TYPE::DASHDOT:
+    case LINE_STYLE::DASHDOT:
         return "DASHDOT";
     default:
-        wxFAIL_MSG( "Unhandled PLOT_DASH_TYPE" );
+        wxFAIL_MSG( "Unhandled LINE_STYLE" );
         return "CONTINUOUS";
     }
 }
@@ -146,6 +149,26 @@ void DXF_PLOTTER::SetUnits( DXF_UNITS aUnit )
 }
 
 
+// convert aValue to a string, and remove trailing zeros
+// In DXF files coordinates need a high precision: at least 9 digits when given
+// in inches and 7 digits when in mm.
+// So we use 16 digits and remove trailing 0 (if any)
+static std::string formatCoord( double aValue )
+{
+    std::string buf;
+
+    buf = fmt::format( "{:.16f}", aValue );
+
+    // remove trailing zeros
+    while( !buf.empty() && buf[buf.size() - 1] == '0' )
+    {
+        buf.pop_back();
+    }
+
+    return buf;
+}
+
+
 void DXF_PLOTTER::SetViewport( const VECTOR2I& aOffset, double aIusPerDecimil,
                                double aScale, bool aMirror )
 {
@@ -300,6 +323,7 @@ bool DXF_PLOTTER::StartPlot( const wxString& aPageNumber )
            "4\n", m_outputFile );
 
     static const char *style_name[4] = {"KICAD", "KICADB", "KICADI", "KICADBI"};
+
     for(int i = 0; i < 4; i++ )
     {
         fprintf( m_outputFile,
@@ -417,11 +441,26 @@ void DXF_PLOTTER::SetColor( const COLOR4D& color )
 void DXF_PLOTTER::Rect( const VECTOR2I& p1, const VECTOR2I& p2, FILL_T fill, int width )
 {
     wxASSERT( m_outputFile );
-    MoveTo( p1 );
-    LineTo( VECTOR2I( p1.x, p2.y ) );
-    LineTo( VECTOR2I( p2.x, p2.y ) );
-    LineTo( VECTOR2I( p2.x, p1.y ) );
-    FinishTo( VECTOR2I( p1.x, p1.y ) );
+
+    if( p1 != p2 )
+    {
+        MoveTo( p1 );
+        LineTo( VECTOR2I( p1.x, p2.y ) );
+        LineTo( VECTOR2I( p2.x, p2.y ) );
+        LineTo( VECTOR2I( p2.x, p1.y ) );
+        FinishTo( VECTOR2I( p1.x, p1.y ) );
+    }
+    else
+    {
+        // Draw as a point
+        wxString cname = getDXFColorName( m_currentColor );
+        VECTOR2D point_dev = userToDeviceCoordinates( p1 );
+
+        fprintf( m_outputFile, "0\nPOINT\n8\n%s\n10\n%s\n20\n%s\n",
+                 TO_UTF8( cname ),
+                 formatCoord( point_dev.x ).c_str(),
+                 formatCoord( point_dev.y ).c_str() );
+    }
 }
 
 
@@ -431,31 +470,45 @@ void DXF_PLOTTER::Circle( const VECTOR2I& centre, int diameter, FILL_T fill, int
     double   radius = userToDeviceSize( diameter / 2 );
     VECTOR2D centre_dev = userToDeviceCoordinates( centre );
 
+    wxString cname = getDXFColorName( m_currentColor );
+
     if( radius > 0 )
     {
-        wxString cname = getDXFColorName( m_currentColor );
-
         if( fill == FILL_T::NO_FILL )
         {
-            fprintf( m_outputFile, "0\nCIRCLE\n8\n%s\n10\n%g\n20\n%g\n40\n%g\n",
+            fprintf( m_outputFile, "0\nCIRCLE\n8\n%s\n10\n%s\n20\n%s\n40\n%s\n",
                      TO_UTF8( cname ),
-                     centre_dev.x, centre_dev.y, radius );
+                     formatCoord( centre_dev.x ).c_str(),
+                     formatCoord( centre_dev.y ).c_str(),
+                     formatCoord( radius ).c_str() );
         }
         else if( fill == FILL_T::FILLED_SHAPE )
         {
-            double r = radius*0.5;
+            double r = radius * 0.5;
             fprintf( m_outputFile, "0\nPOLYLINE\n" );
             fprintf( m_outputFile, "8\n%s\n66\n1\n70\n1\n", TO_UTF8( cname ) );
-            fprintf( m_outputFile, "40\n%g\n41\n%g\n", radius, radius);
+            fprintf( m_outputFile, "40\n%s\n41\n%s\n",
+                                    formatCoord( radius ).c_str(),
+                                    formatCoord( radius ).c_str() );
             fprintf( m_outputFile, "0\nVERTEX\n8\n%s\n", TO_UTF8( cname ) );
-            fprintf( m_outputFile, "10\n%g\n 20\n%g\n42\n1.0\n",
-                     centre_dev.x-r, centre_dev.y );
+            fprintf( m_outputFile, "10\n%s\n 20\n%s\n42\n1.0\n",
+                                    formatCoord( centre_dev.x-r ).c_str(),
+                                    formatCoord( centre_dev.y ).c_str() );
             fprintf( m_outputFile, "0\nVERTEX\n8\n%s\n", TO_UTF8( cname ) );
-            fprintf( m_outputFile, "10\n%g\n 20\n%g\n42\n1.0\n",
-                     centre_dev.x+r, centre_dev.y );
+            fprintf( m_outputFile, "10\n%s\n 20\n%s\n42\n1.0\n",
+                                    formatCoord( centre_dev.x+r ).c_str(),
+                                    formatCoord( centre_dev.y ).c_str() );
             fprintf( m_outputFile, "0\nSEQEND\n");
         }
     }
+    else
+    {
+        // Draw as a point
+        fprintf( m_outputFile, "0\nPOINT\n8\n%s\n10\n%s\n20\n%s\n",
+                 TO_UTF8( cname ),
+                 formatCoord( centre_dev.x ).c_str(),
+                 formatCoord( centre_dev.y ).c_str() );
+    }
 }
 
 
@@ -524,8 +577,8 @@ void DXF_PLOTTER::PlotPoly( const std::vector<VECTOR2I>& aCornerList, FILL_T aFi
     // polygon and its thick outline
 
     // create the outline which contains thick outline:
-    bufferPolybase.BooleanAdd( bufferOutline, SHAPE_POLY_SET::PM_FAST );
-    bufferPolybase.Fracture( SHAPE_POLY_SET::PM_FAST );
+    bufferPolybase.BooleanAdd( bufferOutline );
+    bufferPolybase.Fracture();
 
     if( bufferPolybase.OutlineCount() < 1 )      // should not happen
         return;
@@ -573,24 +626,28 @@ void DXF_PLOTTER::PenTo( const VECTOR2I& pos, char plume )
 
     if( m_penLastpos != pos && plume == 'D' )
     {
-        wxASSERT( m_currentLineType >= PLOT_DASH_TYPE::FIRST_TYPE
-                  && m_currentLineType <= PLOT_DASH_TYPE::LAST_TYPE );
+        wxASSERT( m_currentLineType >= LINE_STYLE::FIRST_TYPE
+                  && m_currentLineType <= LINE_STYLE::LAST_TYPE );
+
         // DXF LINE
         wxString    cname = getDXFColorName( m_currentColor );
-        const char* lname = getDXFLineType( static_cast<PLOT_DASH_TYPE>( m_currentLineType ) );
-        fprintf( m_outputFile, "0\nLINE\n8\n%s\n6\n%s\n10\n%g\n20\n%g\n11\n%g\n21\n%g\n",
+        const char* lname = getDXFLineType( static_cast<LINE_STYLE>( m_currentLineType ) );
+        fprintf( m_outputFile, "0\nLINE\n8\n%s\n6\n%s\n10\n%s\n20\n%s\n11\n%s\n21\n%s\n",
                  TO_UTF8( cname ), lname,
-                 pen_lastpos_dev.x, pen_lastpos_dev.y, pos_dev.x, pos_dev.y );
+                 formatCoord( pen_lastpos_dev.x ).c_str(),
+                 formatCoord( pen_lastpos_dev.y ).c_str(),
+                 formatCoord( pos_dev.x ).c_str(),
+                 formatCoord( pos_dev.y ).c_str() );
     }
 
     m_penLastpos = pos;
 }
 
 
-void DXF_PLOTTER::SetDash( int aLineWidth, PLOT_DASH_TYPE aLineStyle )
+void DXF_PLOTTER::SetDash( int aLineWidth, LINE_STYLE aLineStyle )
 {
-    wxASSERT( aLineStyle >= PLOT_DASH_TYPE::FIRST_TYPE
-                && aLineStyle <= PLOT_DASH_TYPE::LAST_TYPE );
+    wxASSERT( aLineStyle >= LINE_STYLE::FIRST_TYPE
+                && aLineStyle <= LINE_STYLE::LAST_TYPE );
 
     m_currentLineType = aLineStyle;
 }
@@ -626,20 +683,20 @@ void DXF_PLOTTER::ThickSegment( const VECTOR2I& aStart, const VECTOR2I& aEnd, in
 }
 
 
-void DXF_PLOTTER::Arc( const VECTOR2I& aCenter, const EDA_ANGLE& aStartAngle,
-                       const EDA_ANGLE& aEndAngle, int aRadius, FILL_T aFill, int aWidth )
+void DXF_PLOTTER::Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
+                       const EDA_ANGLE& aAngle, double aRadius, FILL_T aFill, int aWidth )
 {
     wxASSERT( m_outputFile );
 
     if( aRadius <= 0 )
         return;
 
-    EDA_ANGLE startAngle( aStartAngle );
-    EDA_ANGLE endAngle( aEndAngle );
+    EDA_ANGLE startAngle = -aStartAngle;
+    EDA_ANGLE endAngle = startAngle - aAngle;
 
     // In DXF, arcs are drawn CCW.
     // If startAngle > endAngle, it is CW. So transform it to CCW
-    if( startAngle > endAngle )
+    if( endAngle < startAngle )
         std::swap( startAngle, endAngle );
 
     VECTOR2D centre_device = userToDeviceCoordinates( aCenter );
@@ -648,9 +705,11 @@ void DXF_PLOTTER::Arc( const VECTOR2I& aCenter, const EDA_ANGLE& aStartAngle,
     // Emit a DXF ARC entity
     wxString cname = getDXFColorName( m_currentColor );
     fprintf( m_outputFile,
-             "0\nARC\n8\n%s\n10\n%g\n20\n%g\n40\n%g\n50\n%g\n51\n%g\n",
+             "0\nARC\n8\n%s\n10\n%s\n20\n%s\n40\n%s\n50\n%.8f\n51\n%.8f\n",
              TO_UTF8( cname ),
-             centre_device.x, centre_device.y, radius_device,
+             formatCoord( centre_device.x ).c_str(),
+             formatCoord( centre_device.y ).c_str(),
+             formatCoord( radius_device ).c_str(),
              startAngle.AsDegrees(), endAngle.AsDegrees() );
 }
 
@@ -761,6 +820,7 @@ void DXF_PLOTTER::FlashPadRoundRect( const VECTOR2I& aPadPos, const VECTOR2I& aS
     FinishTo( VECTOR2I( poly.CPoint( 0 ).x, poly.CPoint( 0 ).y ) );
 }
 
+
 void DXF_PLOTTER::FlashPadCustom( const VECTOR2I& aPadPos, const VECTOR2I& aSize,
                                   const EDA_ANGLE& aOrient, SHAPE_POLY_SET* aPolygons,
                                   OUTLINE_MODE aTraceMode, void* aData )
@@ -831,19 +891,20 @@ bool containsNonAsciiChars( const wxString& string )
 }
 
 
-void DXF_PLOTTER::Text( const VECTOR2I&             aPos,
-                        const COLOR4D&              aColor,
-                        const wxString&             aText,
-                        const EDA_ANGLE&            aOrient,
-                        const VECTOR2I&             aSize,
-                        enum GR_TEXT_H_ALIGN_T      aH_justify,
-                        enum GR_TEXT_V_ALIGN_T      aV_justify,
-                        int                         aWidth,
-                        bool                        aItalic,
-                        bool                        aBold,
-                        bool                        aMultilineAllowed,
-                        KIFONT::FONT*               aFont,
-                        void*                       aData )
+void DXF_PLOTTER::Text( const VECTOR2I&        aPos,
+                        const COLOR4D&         aColor,
+                        const wxString&        aText,
+                        const EDA_ANGLE&       aOrient,
+                        const VECTOR2I&        aSize,
+                        enum GR_TEXT_H_ALIGN_T aH_justify,
+                        enum GR_TEXT_V_ALIGN_T aV_justify,
+                        int                    aWidth,
+                        bool                   aItalic,
+                        bool                   aBold,
+                        bool                   aMultilineAllowed,
+                        KIFONT::FONT*          aFont,
+                        const KIFONT::METRICS& aFontMetrics,
+                        void*                  aData )
 {
     // Fix me: see how to use DXF text mode for multiline texts
     if( aMultilineAllowed && !aText.Contains( wxT( "\n" ) ) )
@@ -857,149 +918,202 @@ void DXF_PLOTTER::Text( const VECTOR2I&             aPos,
         // Perhaps multiline texts could be handled as DXF text entity
         // but I do not want spend time about this (JPC)
         PLOTTER::Text( aPos, aColor, aText, aOrient, aSize, aH_justify, aV_justify, aWidth, aItalic,
-                       aBold, aMultilineAllowed, aFont, aData );
+                       aBold, aMultilineAllowed, aFont, aFontMetrics, aData );
     }
     else
     {
-        /* Emit text as a text entity. This loses formatting and shape but it's
-           more useful as a CAD object */
-        VECTOR2D origin_dev = userToDeviceCoordinates( aPos );
-        SetColor( aColor );
-        wxString cname = getDXFColorName( m_currentColor );
-        VECTOR2D size_dev = userToDeviceSize( aSize );
-        int h_code = 0, v_code = 0;
-
-        switch( aH_justify )
-        {
-        case GR_TEXT_H_ALIGN_LEFT:   h_code = 0; break;
-        case GR_TEXT_H_ALIGN_CENTER: h_code = 1; break;
-        case GR_TEXT_H_ALIGN_RIGHT:  h_code = 2; break;
-        }
-
-        switch( aV_justify )
-        {
-        case GR_TEXT_V_ALIGN_TOP:    v_code = 3; break;
-        case GR_TEXT_V_ALIGN_CENTER: v_code = 2; break;
-        case GR_TEXT_V_ALIGN_BOTTOM: v_code = 1; break;
-        }
-
-        // Position, size, rotation and alignment
-        // The two alignment point usages is somewhat idiot (see the DXF ref)
-        // Anyway since we don't use the fit/aligned options, they're the same
-        fprintf( m_outputFile,
-                 "  0\n"
-                 "TEXT\n"
-                 "  7\n"
-                 "%s\n"          // Text style
-                 "  8\n"
-                 "%s\n"          // Layer name
-                 "  10\n"
-                 "%g\n"          // First point X
-                 "  11\n"
-                 "%g\n"          // Second point X
-                 "  20\n"
-                 "%g\n"          // First point Y
-                 "  21\n"
-                 "%g\n"          // Second point Y
-                 "  40\n"
-                 "%g\n"          // Text height
-                 "  41\n"
-                 "%g\n"          // Width factor
-                 "  50\n"
-                 "%g\n"          // Rotation
-                 "  51\n"
-                 "%g\n"          // Oblique angle
-                 "  71\n"
-                 "%d\n"          // Mirror flags
-                 "  72\n"
-                 "%d\n"          // H alignment
-                 "  73\n"
-                 "%d\n",         // V alignment
-                 aBold ? (aItalic ? "KICADBI" : "KICADB") : (aItalic ? "KICADI" : "KICAD"),
-                 TO_UTF8( cname ),
-                 origin_dev.x, origin_dev.x,
-                 origin_dev.y, origin_dev.y,
-                 size_dev.y, fabs( size_dev.x / size_dev.y ),
-                 aOrient.AsDegrees(),
-                 aItalic ? DXF_OBLIQUE_ANGLE : 0,
-                 size_dev.x < 0 ? 2 : 0, // X mirror flag
-                 h_code, v_code );
-
-        /* There are two issue in emitting the text:
-           - Our overline character (~) must be converted to the appropriate
-           control sequence %%O or %%o
-           - Text encoding in DXF is more or less unspecified since depends on
-           the DXF declared version, the acad version reading it *and* some
-           system variables to be put in the header handled only by newer acads
-           Also before R15 unicode simply is not supported (you need to use
-           bigfonts which are a massive PITA). Common denominator solution:
-           use Latin1 (and however someone could choke on it, anyway). Sorry
-           for the extended latin people. If somewant want to try fixing this
-           recent version seems to use UTF-8 (and not UCS2 like the rest of
-           Windows)
-
-           XXX Actually there is a *third* issue: older DXF formats are limited
-           to 255 bytes records (it was later raised to 2048); since I'm lazy
-           and text so long is not probable I just don't implement this rule.
-           If someone is interested in fixing this, you have to emit the first
-           partial lines with group code 3 (max 250 bytes each) and then finish
-           with a group code 1 (less than 250 bytes). The DXF refs explains it
-           in no more details...
-         */
-
-        int braceNesting = 0;
-        int overbarDepth = -1;
-
-        fputs( "  1\n", m_outputFile );
-
-        for( unsigned int i = 0; i < aText.length(); i++ )
-        {
-            /* Here I do a bad thing: writing the output one byte at a time!
-               but today I'm lazy and I have no idea on how to coerce a Unicode
-               wxString to spit out latin1 encoded text ...
-
-               At least stdio is *supposed* to do output buffering, so there is
-               hope is not too slow */
-            wchar_t ch = aText[i];
-
-            if( ch > 255 )
-            {
-                // I can't encode this...
-                putc( '?', m_outputFile );
-            }
-            else
-            {
-                if( aText[i] == '~' && i+1 < aText.length() && aText[i+1] == '{' )
-                {
-                    fputs( "%%o", m_outputFile );
-                    overbarDepth = braceNesting;
-
-                    // Skip the '{'
-                    i++;
-                    continue;
-                }
-                else if( aText[i] == '{' )
-                {
-                    braceNesting++;
-                }
-                else if( aText[i] == '}' )
-                {
-                    if( braceNesting > 0 )
-                        braceNesting--;
-
-                    if( braceNesting == overbarDepth )
-                    {
-                        fputs( "%%O", m_outputFile );
-                        overbarDepth = -1;
-                        continue;
-                    }
-                }
-
-                putc( ch, m_outputFile );
-            }
-        }
-
-        putc( '\n', m_outputFile );
+        TEXT_ATTRIBUTES attrs;
+        attrs.m_Halign = aH_justify;
+        attrs.m_Valign =aV_justify;
+        attrs.m_StrokeWidth = aWidth;
+        attrs.m_Angle = aOrient;
+        attrs.m_Italic = aItalic;
+        attrs.m_Bold = aBold;
+        attrs.m_Mirrored = aSize.x < 0;
+        attrs.m_Multiline = false;
+        plotOneLineOfText( aPos, aColor, aText, attrs );
     }
 }
 
+
+void DXF_PLOTTER::PlotText( const VECTOR2I&        aPos,
+                            const COLOR4D&         aColor,
+                            const wxString&        aText,
+                            const TEXT_ATTRIBUTES& aAttributes,
+                            KIFONT::FONT*          aFont,
+                            const KIFONT::METRICS& aFontMetrics,
+                            void*                  aData )
+{
+    TEXT_ATTRIBUTES attrs = aAttributes;
+
+    // Fix me: see how to use DXF text mode for multiline texts
+    if( attrs.m_Multiline && !aText.Contains( wxT( "\n" ) ) )
+        attrs.m_Multiline = false;  // the text has only one line.
+
+    bool processSuperSub = aText.Contains( wxT( "^{" ) ) || aText.Contains( wxT( "_{" ) );
+
+    if( m_textAsLines || containsNonAsciiChars( aText ) || attrs.m_Multiline || processSuperSub )
+    {
+        // output text as graphics.
+        // Perhaps multiline texts could be handled as DXF text entity
+        // but I do not want spend time about that (JPC)
+        PLOTTER::PlotText( aPos, aColor, aText, aAttributes, aFont, aFontMetrics, aData );
+    }
+    else
+    {
+        plotOneLineOfText( aPos, aColor, aText, attrs );
+    }
+}
+
+
+void DXF_PLOTTER::plotOneLineOfText( const VECTOR2I& aPos, const COLOR4D& aColor,
+                                     const wxString& aText, const TEXT_ATTRIBUTES& aAttributes )
+{
+    /* Emit text as a text entity. This loses formatting and shape but it's
+       more useful as a CAD object */
+    VECTOR2D origin_dev = userToDeviceCoordinates( aPos );
+    SetColor( aColor );
+    wxString cname = getDXFColorName( m_currentColor );
+    VECTOR2D size_dev = userToDeviceSize( aAttributes.m_Size );
+    int h_code = 0, v_code = 0;
+
+    switch( aAttributes.m_Halign )
+    {
+    case GR_TEXT_H_ALIGN_LEFT:   h_code = 0; break;
+    case GR_TEXT_H_ALIGN_CENTER: h_code = 1; break;
+    case GR_TEXT_H_ALIGN_RIGHT:  h_code = 2; break;
+    case GR_TEXT_H_ALIGN_INDETERMINATE:
+        wxFAIL_MSG( wxT( "Indeterminate state legal only in dialogs." ) );
+        break;
+    }
+
+    switch( aAttributes.m_Valign )
+    {
+    case GR_TEXT_V_ALIGN_TOP:    v_code = 3; break;
+    case GR_TEXT_V_ALIGN_CENTER: v_code = 2; break;
+    case GR_TEXT_V_ALIGN_BOTTOM: v_code = 1; break;
+    case GR_TEXT_V_ALIGN_INDETERMINATE:
+        wxFAIL_MSG( wxT( "Indeterminate state legal only in dialogs." ) );
+        break;
+    }
+
+    // Position, size, rotation and alignment
+    // The two alignment point usages is somewhat idiot (see the DXF ref)
+    // Anyway since we don't use the fit/aligned options, they're the same
+    fprintf( m_outputFile,
+             "  0\n"
+             "TEXT\n"
+             "  7\n"
+             "%s\n"          // Text style
+             "  8\n"
+             "%s\n"          // Layer name
+             "  10\n"
+             "%s\n"          // First point X
+             "  11\n"
+             "%s\n"          // Second point X
+             "  20\n"
+             "%s\n"          // First point Y
+             "  21\n"
+             "%s\n"          // Second point Y
+             "  40\n"
+             "%s\n"          // Text height
+             "  41\n"
+             "%s\n"          // Width factor
+             "  50\n"
+             "%.8f\n"        // Rotation
+             "  51\n"
+             "%.8f\n"        // Oblique angle
+             "  71\n"
+             "%d\n"          // Mirror flags
+             "  72\n"
+             "%d\n"          // H alignment
+             "  73\n"
+             "%d\n",         // V alignment
+             aAttributes.m_Bold ? ( aAttributes.m_Italic ? "KICADBI" : "KICADB" )
+                                : ( aAttributes.m_Italic ? "KICADI" : "KICAD" ),
+             TO_UTF8( cname ),
+             formatCoord( origin_dev.x ).c_str(), formatCoord( origin_dev.x ).c_str(),
+             formatCoord( origin_dev.y ).c_str(), formatCoord( origin_dev.y ).c_str(),
+             formatCoord( size_dev.y ).c_str(),
+             formatCoord( fabs( size_dev.x / size_dev.y ) ).c_str(),
+             aAttributes.m_Angle.AsDegrees(),
+             aAttributes.m_Italic ? DXF_OBLIQUE_ANGLE : 0,
+             aAttributes.m_Mirrored ? 2 : 0, // X mirror flag
+             h_code, v_code );
+
+    /* There are two issue in emitting the text:
+       - Our overline character (~) must be converted to the appropriate
+       control sequence %%O or %%o
+       - Text encoding in DXF is more or less unspecified since depends on
+       the DXF declared version, the acad version reading it *and* some
+       system variables to be put in the header handled only by newer acads
+       Also before R15 unicode simply is not supported (you need to use
+       bigfonts which are a massive PITA). Common denominator solution:
+       use Latin1 (and however someone could choke on it, anyway). Sorry
+       for the extended latin people. If somewant want to try fixing this
+       recent version seems to use UTF-8 (and not UCS2 like the rest of
+       Windows)
+
+       XXX Actually there is a *third* issue: older DXF formats are limited
+       to 255 bytes records (it was later raised to 2048); since I'm lazy
+       and text so long is not probable I just don't implement this rule.
+       If someone is interested in fixing this, you have to emit the first
+       partial lines with group code 3 (max 250 bytes each) and then finish
+       with a group code 1 (less than 250 bytes). The DXF refs explains it
+       in no more details...
+     */
+
+    int braceNesting = 0;
+    int overbarDepth = -1;
+
+    fputs( "  1\n", m_outputFile );
+
+    for( unsigned int i = 0; i < aText.length(); i++ )
+    {
+        /* Here I do a bad thing: writing the output one byte at a time!
+           but today I'm lazy and I have no idea on how to coerce a Unicode
+           wxString to spit out latin1 encoded text ...
+
+           At least stdio is *supposed* to do output buffering, so there is
+           hope is not too slow */
+        wchar_t ch = aText[i];
+
+        if( ch > 255 )
+        {
+            // I can't encode this...
+            putc( '?', m_outputFile );
+        }
+        else
+        {
+            if( aText[i] == '~' && i+1 < aText.length() && aText[i+1] == '{' )
+            {
+                fputs( "%%o", m_outputFile );
+                overbarDepth = braceNesting;
+
+                // Skip the '{'
+                i++;
+                continue;
+            }
+            else if( aText[i] == '{' )
+            {
+                braceNesting++;
+            }
+            else if( aText[i] == '}' )
+            {
+                if( braceNesting > 0 )
+                    braceNesting--;
+
+                if( braceNesting == overbarDepth )
+                {
+                    fputs( "%%O", m_outputFile );
+                    overbarDepth = -1;
+                    continue;
+                }
+            }
+
+            putc( ch, m_outputFile );
+        }
+    }
+
+    putc( '\n', m_outputFile );
+}
