@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2019 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -29,6 +29,7 @@
 #include <math/util.h>      // for KiROUND
 #include <trigo.h>
 #include <wx/log.h>
+#include <cstdio>
 
 #include <build_version.h>
 
@@ -157,6 +158,7 @@ void GERBER_PLOTTER::emitDcode( const VECTOR2D& pt, int dcode )
     fprintf( m_outputFile, "X%dY%dD%02d*\n", KiROUND( pt.x ), KiROUND( pt.y ), dcode );
 }
 
+
 void GERBER_PLOTTER::ClearAllAttributes()
 {
     // Remove all attributes from object attributes dictionary (TO. and TA commands)
@@ -250,7 +252,7 @@ bool GERBER_PLOTTER::StartPlot( const wxString& aPageNumber )
     // Create a temp file in system temp to avoid potential network share buffer issues for
     // the final read and save.
     m_workFilename = wxFileName::CreateTempFileName( "" );
-    workFile   = wxFopen( m_workFilename, wxT( "wt" ));
+    workFile = wxFopen( m_workFilename, wxT( "wt" ) );
     m_outputFile = workFile;
     wxASSERT( m_outputFile );
 
@@ -338,7 +340,7 @@ bool GERBER_PLOTTER::EndPlot()
         if( substr && strcmp( substr, "G04 APERTURE LIST*" ) == 0 )
         {
             // Add aperture list macro:
-            if( m_hasApertureRoundRect | m_hasApertureRotOval ||
+            if( m_hasApertureRoundRect || m_hasApertureRotOval ||
                 m_hasApertureOutline4P || m_hasApertureRotRect ||
                 m_hasApertureChamferedRect || m_am_freepoly_list.AmCount() )
             {
@@ -568,11 +570,10 @@ void GERBER_PLOTTER::selectAperture( int aDiameter, const EDA_ANGLE& aPolygonRot
     selectAperture( VECTOR2I( 0, 0 ), aDiameter / 2, aPolygonRotation, aType, aApertureAttribute );
 }
 
+
 void GERBER_PLOTTER::writeApertureList()
 {
     wxASSERT( m_outputFile );
-    char cbuf[1024];
-    std::string buffer;
 
     bool useX1StructuredComment = false;
 
@@ -598,8 +599,7 @@ void GERBER_PLOTTER::writeApertureList()
                             useX1StructuredComment ).c_str(), m_outputFile );
         }
 
-        sprintf( cbuf, "%%ADD%d", tool.m_DCode );
-        buffer = cbuf;
+        fprintf( m_outputFile, "%%ADD%d", tool.m_DCode );
 
         /* Please note: the Gerber specs for mass parameters say that
            exponential syntax is *not* allowed and the decimal point should
@@ -611,21 +611,21 @@ void GERBER_PLOTTER::writeApertureList()
         switch( tool.m_Type )
         {
         case APERTURE::AT_CIRCLE:
-            sprintf( cbuf, "C,%#f*%%\n", tool.GetDiameter() * fscale );
+            fprintf( m_outputFile, "C,%#f*%%\n", tool.GetDiameter() * fscale );
             break;
 
         case APERTURE::AT_RECT:
-            sprintf( cbuf, "R,%#fX%#f*%%\n", tool.m_Size.x * fscale,
-                                             tool.m_Size.y * fscale );
+            fprintf( m_outputFile, "R,%#fX%#f*%%\n", tool.m_Size.x * fscale,
+                     tool.m_Size.y * fscale );
             break;
 
         case APERTURE::AT_PLOTTING:
-            sprintf( cbuf, "C,%#f*%%\n", tool.m_Size.x * fscale );
+            fprintf( m_outputFile, "C,%#f*%%\n", tool.m_Size.x * fscale );
             break;
 
         case APERTURE::AT_OVAL:
-            sprintf( cbuf, "O,%#fX%#f*%%\n", tool.m_Size.x * fscale,
-                                             tool.m_Size.y * fscale );
+            fprintf( m_outputFile, "O,%#fX%#f*%%\n", tool.m_Size.x * fscale,
+                     tool.m_Size.y * fscale );
             break;
 
         case APERTURE::AT_REGULAR_POLY:
@@ -639,7 +639,7 @@ void GERBER_PLOTTER::writeApertureList()
         case APERTURE::AT_REGULAR_POLY10:
         case APERTURE::AT_REGULAR_POLY11:
         case APERTURE::AT_REGULAR_POLY12:
-            sprintf( cbuf, "P,%#fX%dX%#f*%%\n", tool.GetDiameter() * fscale,
+            fprintf( m_outputFile, "P,%#fX%dX%#f*%%\n", tool.GetDiameter() * fscale,
                      tool.GetRegPolyVerticeCount(), tool.GetRotation().AsDegrees() );
             break;
 
@@ -649,6 +649,16 @@ void GERBER_PLOTTER::writeApertureList()
             std::vector<VECTOR2I> corners;
             VECTOR2I half_size( tool.m_Size.x/2-tool.m_Radius, tool.m_Size.y/2-tool.m_Radius );
 
+            // Ensure half_size.x and half_size.y > minimal value to avoid shapes
+            // with null size (especially the rectangle with coordinates corners)
+            // Because the minimal value for a non nul Gerber coord in 10nm
+            // in format 4.5, use 10 nm as minimal value.
+            // (Even in 4.6 format, use 10 nm, because gerber viewers can have
+            // a internal unit bigger than 1 nm)
+            const int min_size_value = 10;
+            half_size.x = std::max( half_size.x, min_size_value );
+            half_size.y = std::max( half_size.y, min_size_value );
+
             corners.emplace_back( -half_size.x, -half_size.y );
             corners.emplace_back( half_size.x, -half_size.y );
             corners.emplace_back( half_size.x, half_size.y );
@@ -658,26 +668,21 @@ void GERBER_PLOTTER::writeApertureList()
             for( int ii = 0; ii < 4; ii++ )
                 RotatePoint( corners[ii], -tool.m_Rotation );
 
-            sprintf( cbuf, "%s,%#fX", APER_MACRO_ROUNDRECT_NAME,
-                     tool.m_Radius * fscale );
-            buffer += cbuf;
+            fprintf( m_outputFile, "%s,%#fX", APER_MACRO_ROUNDRECT_NAME, tool.m_Radius * fscale );
 
             // Add each corner
             for( int ii = 0; ii < 4; ii++ )
             {
-                sprintf( cbuf, "%#fX%#fX",
-                         corners[ii].x * fscale, corners[ii].y * fscale );
-                buffer += cbuf;
+                fprintf( m_outputFile, "%#fX%#fX", corners[ii].x * fscale, corners[ii].y * fscale );
             }
 
-            sprintf( cbuf, "0*%%\n" );
+            fprintf( m_outputFile, "0*%%\n" );
         }
             break;
 
         case APERTURE::AM_ROT_RECT:         // Aperture macro for rotated rect pads
-            sprintf( cbuf, "%s,%#fX%#fX%#f*%%\n", APER_MACRO_ROT_RECT_NAME,
-                     tool.m_Size.x * fscale, tool.m_Size.y * fscale,
-                     tool.m_Rotation.AsDegrees() );
+            fprintf( m_outputFile, "%s,%#fX%#fX%#f*%%\n", APER_MACRO_ROT_RECT_NAME,
+                     tool.m_Size.x * fscale, tool.m_Size.y * fscale, tool.m_Rotation.AsDegrees() );
             break;
 
         case APERTURE::APER_MACRO_OUTLINE4P:    // Aperture macro for trapezoid pads
@@ -688,33 +693,35 @@ void GERBER_PLOTTER::writeApertureList()
             switch( tool.m_Type )
             {
             case APERTURE::APER_MACRO_OUTLINE4P:
-                sprintf( cbuf, "%s,", APER_MACRO_OUTLINE4P_NAME ); break;
+                fprintf( m_outputFile, "%s,", APER_MACRO_OUTLINE4P_NAME );
+                break;
             case APERTURE::APER_MACRO_OUTLINE5P:
-                sprintf( cbuf, "%s,", APER_MACRO_OUTLINE5P_NAME ); break;
+                fprintf( m_outputFile, "%s,", APER_MACRO_OUTLINE5P_NAME );
+                break;
             case APERTURE::APER_MACRO_OUTLINE6P:
-                sprintf( cbuf, "%s,", APER_MACRO_OUTLINE6P_NAME ); break;
+                fprintf( m_outputFile, "%s,", APER_MACRO_OUTLINE6P_NAME );
+                break;
             case APERTURE::APER_MACRO_OUTLINE7P:
-                sprintf( cbuf, "%s,", APER_MACRO_OUTLINE7P_NAME ); break;
+                fprintf( m_outputFile, "%s,", APER_MACRO_OUTLINE7P_NAME );
+                break;
             case APERTURE::APER_MACRO_OUTLINE8P:
-                sprintf( cbuf, "%s,", APER_MACRO_OUTLINE8P_NAME ); break;
+                fprintf( m_outputFile, "%s,", APER_MACRO_OUTLINE8P_NAME );
+                break;
             default:
                 break;
             }
 
-            buffer += cbuf;
-
             // Output all corners (should be 4 to 8 corners)
             // Remember: the Y coordinate must be negated, due to the fact in Pcbnew
             // the Y axis is from top to bottom
             for( size_t ii = 0; ii < tool.m_Corners.size(); ii++ )
             {
-                sprintf( cbuf, "%#fX%#fX",
-                         tool.m_Corners[ii].x * fscale, -tool.m_Corners[ii].y * fscale );
-                buffer += cbuf;
+                fprintf( m_outputFile, "%#fX%#fX", tool.m_Corners[ii].x * fscale,
+                         -tool.m_Corners[ii].y * fscale );
             }
 
             // close outline and output rotation
-            sprintf( cbuf, "%#f*%%\n", tool.m_Rotation.AsDegrees() );
+            fprintf( m_outputFile, "%#f*%%\n", tool.m_Rotation.AsDegrees() );
             break;
 
         case APERTURE::AM_ROTATED_OVAL:         // Aperture macro for rotated oval pads
@@ -724,18 +731,20 @@ void GERBER_PLOTTER::writeApertureList()
         {
                 // the seg_len is the distance between the 2 circle centers
                 int seg_len = tool.m_Size.x - tool.m_Size.y;
+
                 // Center of the circle on the segment start point:
                 VECTOR2I start( seg_len/2, 0 );
+
                 // Center of the circle on the segment end point:
                 VECTOR2I end( - seg_len/2, 0 );
 
                 RotatePoint( start, tool.m_Rotation );
                 RotatePoint( end, tool.m_Rotation );
 
-                sprintf( cbuf, "%s,%#fX%#fX%#fX%#fX%#fX0*%%\n", APER_MACRO_SHAPE_OVAL_NAME,
-                         tool.m_Size.y * fscale,                // width
-                         start.x * fscale, -start.y * fscale,   // X,Y corner start pos
-                         end.x * fscale, -end.y * fscale );     // X,Y cornerend  pos
+                fprintf( m_outputFile, "%s,%#fX%#fX%#fX%#fX%#fX0*%%\n", APER_MACRO_SHAPE_OVAL_NAME,
+                         tool.m_Size.y * fscale,              // width
+                         start.x * fscale, -start.y * fscale, // X,Y corner start pos
+                         end.x * fscale, -end.y * fscale );   // X,Y cornerend  pos
         }
             break;
 
@@ -747,17 +756,12 @@ void GERBER_PLOTTER::writeApertureList()
 
             // Write DCODE id ( "%ADDxx" is already in buffer) and rotation
             // the full line is something like :%ADD12FreePoly1,45.000000*%
-            sprintf( cbuf, "%s%d,%#f*%%\n",
-                     AM_FREEPOLY_BASENAME,
-                     idx,
+            fprintf( m_outputFile, "%s%d,%#f*%%\n", AM_FREEPOLY_BASENAME, idx,
                      tool.m_Rotation.AsDegrees() );
             break;
         }
         }
 
-        buffer += cbuf;
-        fputs( buffer.c_str(), m_outputFile );
-
         m_apertureAttribute = attribute;
 
         // Currently reset the aperture attribute. Perhaps a better optimization
@@ -821,18 +825,32 @@ void GERBER_PLOTTER::Rect( const VECTOR2I& p1, const VECTOR2I& p2, FILL_T fill,
 
 void GERBER_PLOTTER::Circle( const VECTOR2I& aCenter, int aDiameter, FILL_T aFill, int aWidth )
 {
-    Arc( aCenter, ANGLE_0, ANGLE_360, aDiameter / 2, aFill, aWidth );
+    Arc( aCenter, ANGLE_0, ANGLE_180, aDiameter / 2, aFill, aWidth );
+    Arc( aCenter, ANGLE_180, ANGLE_180, aDiameter / 2, aFill, aWidth );
 }
 
 
 
 void GERBER_PLOTTER::Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
-                          const EDA_ANGLE& aEndAngle, double aRadius, FILL_T aFill, int aWidth )
+                          const EDA_ANGLE& aAngle, double aRadius, FILL_T aFill, int aWidth )
 {
     SetCurrentLineWidth( aWidth );
 
-    // aFill is not used here.
-    plotArc( aCenter, aStartAngle, aEndAngle, aRadius, false );
+    double arcLength = std::abs( aRadius * aAngle.AsRadians() );
+
+    if( arcLength < 100 || std::abs( aAngle.AsDegrees() ) < 0.1 )
+    {
+        // Prevent plotting very short arcs as full circles, especially with 4.5 mm precision.
+        // Also reduce the risk of integer overflow issues.
+        polyArc( aCenter, aStartAngle, aAngle, aRadius, aFill, aWidth );
+    }
+    else
+    {
+        EDA_ANGLE endAngle = aStartAngle + aAngle;
+
+        // aFill is not used here.
+        plotArc( aCenter, aStartAngle, endAngle, aRadius, false );
+    }
 }
 
 
@@ -875,29 +893,30 @@ void GERBER_PLOTTER::plotArc( const SHAPE_ARC& aArc, bool aPlotInRegion )
 
 
 void GERBER_PLOTTER::plotArc( const VECTOR2I& aCenter, const EDA_ANGLE& aStartAngle,
-                              const EDA_ANGLE& aEndAngle, int aRadius, bool aPlotInRegion )
+                              const EDA_ANGLE& aEndAngle, double aRadius, bool aPlotInRegion )
 {
     VECTOR2I start, end;
-    start.x = aCenter.x + KiROUND( aRadius * aStartAngle.Cos() );
-    start.y = aCenter.y - KiROUND( aRadius * aStartAngle.Sin() );
+    start.x = KiROUND( aCenter.x + aRadius * aStartAngle.Cos() );
+    start.y = KiROUND( aCenter.y + aRadius * aStartAngle.Sin() );
 
     if( !aPlotInRegion )
         MoveTo( start );
     else
         LineTo( start );
 
-    end.x = aCenter.x + KiROUND( aRadius * aEndAngle.Cos() );
-    end.y = aCenter.y - KiROUND( aRadius * aEndAngle.Sin() );
+    end.x = KiROUND( aCenter.x + aRadius * aEndAngle.Cos() );
+    end.y = KiROUND( aCenter.y + aRadius * aEndAngle.Sin() );
     VECTOR2D devEnd = userToDeviceCoordinates( end );
+
     // devRelCenter is the position on arc center relative to the arc start, in Gerber coord.
     VECTOR2D devRelCenter = userToDeviceCoordinates( aCenter ) - userToDeviceCoordinates( start );
 
-    fprintf( m_outputFile, "G75*\n" );        // Multiquadrant (360 degrees) mode
+    fprintf( m_outputFile, "G75*\n" ); // Multiquadrant (360 degrees) mode
 
-    if( aStartAngle < aEndAngle )
-        fprintf( m_outputFile, "G03*\n" );    // Active circular interpolation, CCW
+    if( aStartAngle > aEndAngle )
+        fprintf( m_outputFile, "G03*\n" ); // Active circular interpolation, CCW
     else
-        fprintf( m_outputFile, "G02*\n" );    // Active circular interpolation, CW
+        fprintf( m_outputFile, "G02*\n" ); // Active circular interpolation, CW
 
     fprintf( m_outputFile, "X%dY%dI%dJ%dD01*\n",
              KiROUND( devEnd.x ), KiROUND( devEnd.y ),
@@ -983,7 +1002,7 @@ void GERBER_PLOTTER::PlotPolyAsRegion( const SHAPE_LINE_CHAIN& aPoly, FILL_T aFi
 {
     // plot a filled polygon using Gerber region, therefore adding X2 attributes
     // to the solid polygon
-    if( aWidth )
+    if( aWidth || aFill == FILL_T::NO_FILL )
         PlotPoly( aPoly, FILL_T::NO_FILL, aWidth, aGbrMetadata );
 
     if( aFill != FILL_T::NO_FILL )
@@ -1041,7 +1060,7 @@ void GERBER_PLOTTER::PlotPoly( const SHAPE_LINE_CHAIN& aPoly, FILL_T aFill, int
         fputs( "G37*\n", m_outputFile );
     }
 
-    if( aWidth > 0 )    // Draw the polyline/polygon outline
+    if( aWidth > 0 || aFill == FILL_T::NO_FILL )    // Draw the polyline/polygon outline
     {
         SetCurrentLineWidth( aWidth, gbr_metadata );
 
@@ -1078,6 +1097,7 @@ void GERBER_PLOTTER::PlotPoly( const SHAPE_LINE_CHAIN& aPoly, FILL_T aFill, int
     }
 }
 
+
 void GERBER_PLOTTER::PlotPoly( const std::vector<VECTOR2I>& aCornerList, FILL_T aFill, int aWidth,
                                void * aData )
 {
@@ -1109,7 +1129,7 @@ void GERBER_PLOTTER::PlotPoly( const std::vector<VECTOR2I>& aCornerList, FILL_T
         fputs( "G37*\n", m_outputFile );
     }
 
-    if( aWidth > 0 )    // Draw the polyline/polygon outline
+    if( aWidth > 0 || aFill == FILL_T::NO_FILL )    // Draw the polyline/polygon outline
     {
         SetCurrentLineWidth( aWidth, gbr_metadata );
 
@@ -1149,11 +1169,12 @@ void GERBER_PLOTTER::ThickSegment( const VECTOR2I& start, const VECTOR2I& end, i
     }
 }
 
+
 void GERBER_PLOTTER::ThickArc( const VECTOR2D& aCentre, const EDA_ANGLE& aStartAngle,
-                               const EDA_ANGLE& aEndAngle, double aRadius, int aWidth,
+                               const EDA_ANGLE& aAngle, double aRadius, int aWidth,
                                OUTLINE_MODE aTraceMode, void* aData )
 {
-    GBR_METADATA *gbr_metadata = static_cast<GBR_METADATA*>( aData );
+    GBR_METADATA* gbr_metadata = static_cast<GBR_METADATA*>( aData );
     SetCurrentLineWidth( aWidth, gbr_metadata );
 
     if( gbr_metadata )
@@ -1161,57 +1182,19 @@ void GERBER_PLOTTER::ThickArc( const VECTOR2D& aCentre, const EDA_ANGLE& aStartA
 
     if( aTraceMode == FILLED )
     {
-        Arc( aCentre, aStartAngle, aEndAngle, aRadius, FILL_T::NO_FILL, DO_NOT_SET_LINE_WIDTH );
+        Arc( aCentre, aStartAngle, aAngle, aRadius, FILL_T::NO_FILL, DO_NOT_SET_LINE_WIDTH );
     }
     else
     {
         SetCurrentLineWidth( USE_DEFAULT_LINE_WIDTH );
-        Arc( aCentre, aStartAngle, aEndAngle, aRadius - ( aWidth - m_currentPenWidth ) / 2,
+        Arc( aCentre, aStartAngle, aAngle, aRadius - ( aWidth - m_currentPenWidth ) / 2,
              FILL_T::NO_FILL, DO_NOT_SET_LINE_WIDTH );
-        Arc( aCentre, aStartAngle, aEndAngle, aRadius + ( aWidth - m_currentPenWidth ) / 2,
+        Arc( aCentre, aStartAngle, aAngle, aRadius + ( aWidth - m_currentPenWidth ) / 2,
              FILL_T::NO_FILL, DO_NOT_SET_LINE_WIDTH );
     }
 }
 
 
-void GERBER_PLOTTER::ThickArc( const VECTOR2I& aCentre, const VECTOR2I& aStart,
-                               const VECTOR2I& aEnd, int aWidth,
-                               OUTLINE_MODE aTraceMode, void* aData )
-{
-    EDA_ANGLE start_angle( aStart - aCentre );
-    EDA_ANGLE end_angle( aEnd - aCentre );
-
-    if( start_angle > end_angle )
-    {
-        if( end_angle < ANGLE_0 )
-            end_angle.Normalize();
-        else
-            start_angle = start_angle.Normalize() - ANGLE_360;
-    }
-
-    int radius = (aStart - aCentre).EuclideanNorm();
-
-    if( !m_yaxisReversed )   // should be always the case
-    {
-        std::swap( end_angle, start_angle );
-        end_angle = -end_angle;
-        start_angle = -start_angle;
-    }
-
-    ThickArc( aCentre, start_angle, end_angle, radius, aWidth, aTraceMode, aData );
-}
-
-
-void GERBER_PLOTTER::ThickArc( const EDA_SHAPE& aArcShape,
-                           OUTLINE_MODE aTraceMode, void* aData )
-{
-    wxASSERT( aArcShape.GetShape() == SHAPE_T::ARC );
-
-    ThickArc( aArcShape.getCenter(), aArcShape.GetStart(), aArcShape.GetEnd(),
-              aArcShape.GetWidth(), aTraceMode, aData );
-}
-
-
 void GERBER_PLOTTER::ThickRect( const VECTOR2I& p1, const VECTOR2I& p2, int width,
                             OUTLINE_MODE tracemode, void* aData )
 {
@@ -1267,7 +1250,7 @@ void GERBER_PLOTTER::ThickCircle( const VECTOR2I& pos, int diametre, int width,
 
 
 void GERBER_PLOTTER::FilledCircle( const VECTOR2I& pos, int diametre,
-                              OUTLINE_MODE tracemode, void* aData )
+                                   OUTLINE_MODE tracemode, void* aData )
 {
     // A filled circle is a graphic item, not a pad.
     // So it is drawn, not flashed.
@@ -1378,6 +1361,7 @@ void GERBER_PLOTTER::FlashPadOval( const VECTOR2I& aPos, const VECTOR2I& aSize,
                 emitDcode( pos_device, 3 );
                 return;
             }
+
             // Draw the oval as round rect pad with a radius = 50% min size)
             // In gerber file, it will be drawn as a region with arcs, and can be
             // detected as pads (similar to a flashed pad)
@@ -1415,8 +1399,8 @@ void GERBER_PLOTTER::FlashPadRect( const VECTOR2I& pos, const VECTOR2I& aSize,
     // so use it for rotation n*90 deg
     if( aOrient.IsCardinal() )
     {
+        // Build the not rotated equivalent shape:
         if( aOrient.IsCardinal90() )
-            // Build the not rotated equivalent shape:
             std::swap( size.x, size.y );
 
         if( aTraceMode == SKETCH )
@@ -1462,11 +1446,11 @@ void GERBER_PLOTTER::FlashPadRect( const VECTOR2I& pos, const VECTOR2I& aSize,
         {
             // plot pad shape as Gerber region
             VECTOR2I coord[4];
+
             // coord[0] is assumed the lower left
             // coord[1] is assumed the upper left
             // coord[2] is assumed the upper right
             // coord[3] is assumed the lower right
-
             coord[0].x = -size.x/2;   // lower left
             coord[0].y = size.y/2;
             coord[1].x = -size.x/2;   // upper left
@@ -1481,9 +1465,10 @@ void GERBER_PLOTTER::FlashPadRect( const VECTOR2I& pos, const VECTOR2I& aSize,
     }
 }
 
+
 void GERBER_PLOTTER::FlashPadRoundRect( const VECTOR2I& aPadPos, const VECTOR2I& aSize,
-                                     int aCornerRadius, const EDA_ANGLE& aOrient,
-                                     OUTLINE_MODE aTraceMode, void* aData )
+                                        int aCornerRadius, const EDA_ANGLE& aOrient,
+                                        OUTLINE_MODE aTraceMode, void* aData )
 
 {
     GBR_METADATA* gbr_metadata = static_cast<GBR_METADATA*>( aData );
@@ -1497,6 +1482,7 @@ void GERBER_PLOTTER::FlashPadRoundRect( const VECTOR2I& aPadPos, const VECTOR2I&
         SetCurrentLineWidth( USE_DEFAULT_LINE_WIDTH, &gbr_metadata );
 
         std::vector<VECTOR2I> cornerList;
+
         // TransformRoundRectToPolygon creates only one convex polygon
         SHAPE_LINE_CHAIN& poly = outline.Outline( 0 );
         cornerList.reserve( poly.PointCount() + 1 );
@@ -1677,8 +1663,8 @@ void GERBER_PLOTTER::plotRoundRectAsRegion( const VECTOR2I& aRectCenter, const V
         if( aCornerRadius )     // Guard: ensure we do not create arcs with radius = 0
         {
             // LineTo( rr_edge.m_end ); // made in plotArc()
-            plotArc( rr_edge.m_center, rr_edge.m_arc_angle_start,
-                     rr_edge.m_arc_angle_start - ANGLE_90, aCornerRadius, true );
+            plotArc( rr_edge.m_center, -rr_edge.m_arc_angle_start,
+                     -rr_edge.m_arc_angle_start + ANGLE_90, aCornerRadius, true );
         }
         else
         {
@@ -1736,7 +1722,7 @@ void GERBER_PLOTTER::FlashPadCustom( const VECTOR2I& aPadPos, const VECTOR2I& aS
             }
             else
             {
-               // An AM will be created. the shape must be in position 0,0 and orientation 0
+                // An AM will be created. the shape must be in position 0,0 and orientation 0
                 // to be able to reuse the same AM for pads having the same shape
                 for( size_t ii = 0; ii < cornerList.size(); ii++ )
                 {
@@ -1795,7 +1781,9 @@ void GERBER_PLOTTER::FlashPadChamferRoundRect( const VECTOR2I& aShapePos, const
         cornerList.push_back( cornerList[0] );
 
         if( aPlotMode == SKETCH )
+        {
             PlotPoly( cornerList, FILL_T::NO_FILL, GetCurrentLineWidth(), &gbr_metadata );
+        }
         else
         {
 #ifdef GBR_USE_MACROS_FOR_CHAMFERED_ROUND_RECT
@@ -1805,7 +1793,7 @@ void GERBER_PLOTTER::FlashPadChamferRoundRect( const VECTOR2I& aShapePos, const
             }
             else
             {
-               // An AM will be created. the shape must be in position 0,0 and orientation 0
+                // An AM will be created. the shape must be in position 0,0 and orientation 0
                 // to be able to reuse the same AM for pads having the same shape
                 for( size_t ii = 0; ii < cornerList.size(); ii++ )
                 {
@@ -1920,6 +1908,7 @@ void GERBER_PLOTTER::FlashPadTrapez( const VECTOR2I& aPadPos, const VECTOR2I* aC
     {
         m_hasApertureOutline4P = true;
         VECTOR2D pos_dev = userToDeviceCoordinates( aPadPos );
+
         // polygon corners list
         std::vector<VECTOR2I> corners = { aCorners[0], aCorners[1], aCorners[2], aCorners[3] };
         int aperture_attrib = gbr_metadata ? gbr_metadata->GetApertureAttrib() : 0;
@@ -1985,19 +1974,20 @@ void GERBER_PLOTTER::FlashRegularPolygon( const VECTOR2I& aShapePos, int aDiamet
 }
 
 
-void GERBER_PLOTTER::Text( const VECTOR2I&             aPos,
-                           const COLOR4D&              aColor,
-                           const wxString&             aText,
-                           const EDA_ANGLE&            aOrient,
-                           const VECTOR2I&             aSize,
-                           enum GR_TEXT_H_ALIGN_T      aH_justify,
-                           enum GR_TEXT_V_ALIGN_T      aV_justify,
-                           int                         aWidth,
-                           bool                        aItalic,
-                           bool                        aBold,
-                           bool                        aMultilineAllowed,
-                           KIFONT::FONT*               aFont,
-                           void*                       aData )
+void GERBER_PLOTTER::Text( const VECTOR2I&        aPos,
+                           const COLOR4D&         aColor,
+                           const wxString&        aText,
+                           const EDA_ANGLE&       aOrient,
+                           const VECTOR2I&        aSize,
+                           enum GR_TEXT_H_ALIGN_T aH_justify,
+                           enum GR_TEXT_V_ALIGN_T aV_justify,
+                           int                    aWidth,
+                           bool                   aItalic,
+                           bool                   aBold,
+                           bool                   aMultilineAllowed,
+                           KIFONT::FONT*          aFont,
+                           const KIFONT::METRICS& aFontMetrics,
+                           void*                  aData )
 {
     GBR_METADATA* gbr_metadata = static_cast<GBR_METADATA*>( aData );
 
@@ -2005,24 +1995,27 @@ void GERBER_PLOTTER::Text( const VECTOR2I&             aPos,
         formatNetAttribute( &gbr_metadata->m_NetlistMetadata );
 
     PLOTTER::Text( aPos, aColor, aText, aOrient, aSize, aH_justify, aV_justify, aWidth,
-                   aItalic, aBold, aMultilineAllowed, aFont, aData );
+                   aItalic, aBold, aMultilineAllowed, aFont, aFontMetrics, aData );
 }
 
 
-void GERBER_PLOTTER::PlotText( const VECTOR2I& aPos, const COLOR4D& aColor,
-                    const wxString& aText,
-                    const TEXT_ATTRIBUTES& aAttributes,
-                    KIFONT::FONT* aFont,
-                    void* aData )
+void GERBER_PLOTTER::PlotText( const VECTOR2I&        aPos,
+                               const COLOR4D&         aColor,
+                               const wxString&        aText,
+                               const TEXT_ATTRIBUTES& aAttributes,
+                               KIFONT::FONT*          aFont,
+                               const KIFONT::METRICS& aFontMetrics,
+                               void*                  aData )
 {
     GBR_METADATA* gbr_metadata = static_cast<GBR_METADATA*>( aData );
 
     if( gbr_metadata )
         formatNetAttribute( &gbr_metadata->m_NetlistMetadata );
 
-    PLOTTER::PlotText( aPos, aColor, aText, aAttributes, aFont, aData );
+    PLOTTER::PlotText( aPos, aColor, aText, aAttributes, aFont, aFontMetrics, aData );
 }
 
+
 void GERBER_PLOTTER::SetLayerPolarity( bool aPositive )
 {
     if( aPositive )
