@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2020-2022 KiCad Developers, see change_log.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,6 +24,8 @@
 
 #include <string_utils.h>
 #include <scintilla_tricks.h>
+#include <widgets/wx_grid.h>
+#include <widgets/ui_common.h>
 #include <wx/stc/stc.h>
 #include <gal/color4d.h>
 #include <dialog_shim.h>
@@ -33,7 +35,9 @@
 #include <confirm.h>
 
 SCINTILLA_TRICKS::SCINTILLA_TRICKS( wxStyledTextCtrl* aScintilla, const wxString& aBraces,
-                                    bool aSingleLine, std::function<void()> aReturnCallback ) :
+                                    bool aSingleLine,
+                                    std::function<void( wxKeyEvent& )> onAcceptFn,
+                                    std::function<void( wxStyledTextEvent& )> onCharAddedFn ) :
         m_te( aScintilla ),
         m_braces( aBraces ),
         m_lastCaretPos( -1 ),
@@ -41,7 +45,8 @@ SCINTILLA_TRICKS::SCINTILLA_TRICKS( wxStyledTextCtrl* aScintilla, const wxString
         m_lastSelEnd( -1 ),
         m_suppressAutocomplete( false ),
         m_singleLine( aSingleLine ),
-        m_returnCallback( aReturnCallback )
+        m_onAcceptFn( std::move( onAcceptFn ) ),
+        m_onCharAddedFn( std::move( onCharAddedFn ) )
 {
     // Always use LF as eol char, regardless the platform
     m_te->SetEOLMode( wxSTC_EOL_LF );
@@ -51,15 +56,28 @@ SCINTILLA_TRICKS::SCINTILLA_TRICKS( wxStyledTextCtrl* aScintilla, const wxString
     m_te->SetScrollWidth( 1 );
     m_te->SetScrollWidthTracking( true );
 
+    if( m_singleLine )
+    {
+        m_te->SetUseVerticalScrollBar( false );
+        m_te->SetUseHorizontalScrollBar( false );
+    }
+
     setupStyles();
 
     // Set up autocomplete
     m_te->AutoCompSetIgnoreCase( true );
-    m_te->AutoCompSetFillUps( m_braces[1] );
     m_te->AutoCompSetMaxHeight( 20 );
 
+    if( aBraces.Length() >= 2 )
+        m_te->AutoCompSetFillUps( m_braces[1] );
+
     // Hook up events
     m_te->Bind( wxEVT_STC_UPDATEUI, &SCINTILLA_TRICKS::onScintillaUpdateUI, this );
+    m_te->Bind( wxEVT_STC_MODIFIED, &SCINTILLA_TRICKS::onModified, this );
+
+    // Handle autocomplete
+    m_te->Bind( wxEVT_STC_CHARADDED, &SCINTILLA_TRICKS::onChar, this );
+    m_te->Bind( wxEVT_STC_AUTOCOMP_CHAR_DELETED, &SCINTILLA_TRICKS::onChar, this );
 
     // Dispatch command-keys in Scintilla control.
     m_te->Bind( wxEVT_CHAR_HOOK, &SCINTILLA_TRICKS::onCharHook, this );
@@ -132,16 +150,16 @@ bool isCtrlSlash( wxKeyEvent& aEvent )
     // OK, now the wxWidgets hacks start.
     // (We should abandon these if https://trac.wxwidgets.org/ticket/18911 gets resolved.)
 
-    // Many Latin America and European keyboars have have the / over the 7.  We know that
+    // Many Latin America and European keyboards have have the / over the 7.  We know that
     // wxWidgets messes this up and returns Shift+7 through GetUnicodeKey().  However, other
     // keyboards (such as France and Belgium) have 7 in the shifted position, so a Shift+7
     // *could* be legitimate.
 
     // However, we *are* checking Ctrl, so to assume any Shift+7 is a Ctrl-/ really only
     // disallows Ctrl+Shift+7 from doing something else, which is probably OK.  (This routine
-    // is only used in the Scintilla editor, not in the rest of Kicad.)
+    // is only used in the Scintilla editor, not in the rest of KiCad.)
 
-    // The other main shifted loation of / is over : (France and Belgium), so we'll sacrifice
+    // The other main shifted location of / is over : (France and Belgium), so we'll sacrifice
     // Ctrl+Shift+: too.
 
     if( aEvent.ShiftDown() && ( aEvent.GetUnicodeKey() == '7' || aEvent.GetUnicodeKey() == ':' ) )
@@ -155,17 +173,98 @@ bool isCtrlSlash( wxKeyEvent& aEvent )
 }
 
 
+void SCINTILLA_TRICKS::onChar( wxStyledTextEvent& aEvent )
+{
+    m_onCharAddedFn( aEvent );
+}
+
+
+void SCINTILLA_TRICKS::onModified( wxStyledTextEvent& aEvent )
+{
+    if( m_singleLine )
+    {
+        wxString curr_text = m_te->GetText();
+
+        if( curr_text.Contains( wxS( "\n" ) ) || curr_text.Contains( wxS( "\r" ) ) )
+        {
+            // Scintilla won't allow us to call SetText() from within this event processor,
+            // so we have to delay the processing.
+            CallAfter( [this]()
+                       {
+                           wxString text = m_te->GetText();
+                           int currpos = m_te->GetCurrentPos();
+
+                           text.Replace( wxS( "\n" ), wxS( "" ) );
+                           text.Replace( wxS( "\r" ), wxS( "" ) );
+                           m_te->SetText( text );
+                           m_te->GotoPos( currpos-1 );
+                       } );
+        }
+    }
+}
+
+
 void SCINTILLA_TRICKS::onCharHook( wxKeyEvent& aEvent )
 {
     wxString c = aEvent.GetUnicodeKey();
 
+    if( m_te->AutoCompActive() )
+    {
+        if( aEvent.GetKeyCode() == WXK_ESCAPE )
+        {
+            m_te->AutoCompCancel();
+            m_suppressAutocomplete = true; // Don't run autocomplete again on the next char...
+        }
+        else if( aEvent.GetKeyCode() == WXK_RETURN || aEvent.GetKeyCode() == WXK_NUMPAD_ENTER )
+        {
+            int start = m_te->AutoCompPosStart();
+
+            m_te->AutoCompComplete();
+
+            int finish = m_te->GetCurrentPos();
+
+            if( finish > start )
+            {
+                // Select the last substitution token (if any) in the autocompleted text
+
+                int selStart = m_te->FindText( finish, start, "<" );
+                int selEnd = m_te->FindText( finish, start, ">" );
+
+                if( selStart > start && selEnd <= finish && selEnd > selStart )
+                    m_te->SetSelection( selStart, selEnd + 1 );
+            }
+        }
+        else
+        {
+            aEvent.Skip();
+        }
+
+        return;
+    }
+
+#ifdef __WXMAC__
+    if( aEvent.GetModifiers() == wxMOD_RAW_CONTROL && aEvent.GetKeyCode() == WXK_SPACE )
+#else
+    if( aEvent.GetModifiers() == wxMOD_CONTROL && aEvent.GetKeyCode() == WXK_SPACE )
+#endif
+    {
+        m_suppressAutocomplete = false;
+
+        wxStyledTextEvent event;
+        event.SetKey( ' ' );
+        event.SetModifiers( wxMOD_CONTROL );
+        m_onCharAddedFn( event );
+
+        return;
+    }
+
     if( !isalpha( aEvent.GetKeyCode() ) )
         m_suppressAutocomplete = false;
 
     if( ( aEvent.GetKeyCode() == WXK_RETURN || aEvent.GetKeyCode() == WXK_NUMPAD_ENTER )
         && ( m_singleLine || aEvent.ShiftDown() ) )
     {
-        m_returnCallback();
+        m_onAcceptFn( aEvent );
     }
     else if( ConvertSmartQuotesAndDashes( &c ) )
     {
@@ -173,6 +272,11 @@ void SCINTILLA_TRICKS::onCharHook( wxKeyEvent& aEvent )
     }
     else if( aEvent.GetKeyCode() == WXK_TAB )
     {
+        wxWindow* ancestor = m_te->GetParent();
+
+        while( ancestor && !dynamic_cast<WX_GRID*>( ancestor ) )
+            ancestor = ancestor->GetParent();
+
         if( aEvent.ControlDown() )
         {
             int flags = 0;
@@ -180,13 +284,49 @@ void SCINTILLA_TRICKS::onCharHook( wxKeyEvent& aEvent )
             if( !aEvent.ShiftDown() )
                 flags |= wxNavigationKeyEvent::IsForward;
 
-            wxWindow* parent = m_te->GetParent();
+            if( DIALOG_SHIM* dlg = dynamic_cast<DIALOG_SHIM*>( wxGetTopLevelParent( m_te ) ) )
+                dlg->NavigateIn( flags );
+        }
+        else if( dynamic_cast<WX_GRID*>( ancestor ) )
+        {
+            WX_GRID* grid = static_cast<WX_GRID*>( ancestor );
+            int      row = grid->GetGridCursorRow();
+            int      col = grid->GetGridCursorCol();
 
-            while( parent && dynamic_cast<DIALOG_SHIM*>( parent ) == nullptr )
-                parent = parent->GetParent();
+            if( aEvent.ShiftDown() )
+            {
+                if( col > 0 )
+                {
+                    col--;
+                }
+                else if( row > 0 )
+                {
+                    col = (int) grid->GetNumberCols() - 1;
 
-            if( parent )
-                parent->NavigateIn( flags );
+                    if( row > 0 )
+                        row--;
+                    else
+                        row = (int) grid->GetNumberRows() - 1;
+                }
+            }
+            else
+            {
+                if( col < (int) grid->GetNumberCols() - 1 )
+                {
+                    col++;
+                }
+                else if( row < grid->GetNumberRows() - 1 )
+                {
+                    col = 0;
+
+                    if( row < grid->GetNumberRows() - 1 )
+                        row++;
+                    else
+                        row = 0;
+                }
+            }
+
+            grid->SetGridCursor( row, col );
         }
         else
         {
@@ -209,10 +349,22 @@ void SCINTILLA_TRICKS::onCharHook( wxKeyEvent& aEvent )
     else if( aEvent.GetModifiers() == wxMOD_CONTROL && aEvent.GetKeyCode() == 'X' )
     {
         m_te->Cut();
+
+        if( wxTheClipboard->Open() )
+        {
+            wxTheClipboard->Flush(); // Allow data to be available after closing KiCad
+            wxTheClipboard->Close();
+        }
     }
     else if( aEvent.GetModifiers() == wxMOD_CONTROL && aEvent.GetKeyCode() == 'C' )
     {
         m_te->Copy();
+
+        if( wxTheClipboard->Open() )
+        {
+            wxTheClipboard->Flush(); // Allow data to be available after closing KiCad
+            wxTheClipboard->Close();
+        }
     }
     else if( aEvent.GetModifiers() == wxMOD_CONTROL && aEvent.GetKeyCode() == 'V' )
     {
@@ -233,6 +385,13 @@ void SCINTILLA_TRICKS::onCharHook( wxKeyEvent& aEvent )
                 str = data.GetText();
 
                 ConvertSmartQuotesAndDashes( &str );
+
+                if( m_singleLine )
+                {
+                    str.Replace( wxS( "\n" ), wxEmptyString );
+                    str.Replace( wxS( "\r" ), wxEmptyString );
+                }
+
                 m_te->BeginUndoAction();
                 m_te->AddText( str );
                 m_te->EndUndoAction();
@@ -260,18 +419,6 @@ void SCINTILLA_TRICKS::onCharHook( wxKeyEvent& aEvent )
         if( m_te->GetSelectionEnd() > m_te->GetSelectionStart() )
             m_te->DeleteBack();
     }
-    else if( aEvent.GetKeyCode() == WXK_ESCAPE )
-    {
-        if( m_te->AutoCompActive() )
-        {
-            m_te->AutoCompCancel();
-            m_suppressAutocomplete = true; // Don't run autocomplete again on the next char...
-        }
-        else
-        {
-            aEvent.Skip();
-        }
-    }
     else if( isCtrlSlash( aEvent ) )
     {
         int  startLine = m_te->LineFromPosition( m_te->GetSelectionStart() );
@@ -303,6 +450,28 @@ void SCINTILLA_TRICKS::onCharHook( wxKeyEvent& aEvent )
     {
         m_te->LineEndWrap();
     }
+    else if( ( aEvent.GetModifiers() & wxMOD_RAW_CONTROL ) && aEvent.GetKeyCode() == 'B' )
+    {
+        if( aEvent.GetModifiers() & wxMOD_ALT )
+            m_te->WordLeft();
+        else
+            m_te->CharLeft();
+    }
+    else if( ( aEvent.GetModifiers() & wxMOD_RAW_CONTROL ) && aEvent.GetKeyCode() == 'F' )
+    {
+        if( aEvent.GetModifiers() & wxMOD_ALT )
+            m_te->WordRight();
+        else
+            m_te->CharRight();
+    }
+    else if( aEvent.GetModifiers() == wxMOD_RAW_CONTROL && aEvent.GetKeyCode() == 'D' )
+    {
+        if( m_te->GetSelectionEnd() == m_te->GetSelectionStart() )
+            m_te->CharRightExtend();
+
+        if( m_te->GetSelectionEnd() > m_te->GetSelectionStart() )
+            m_te->DeleteBack();
+    }
 #endif
     else if( aEvent.GetKeyCode() == WXK_SPECIAL20 )
     {
@@ -396,6 +565,43 @@ void SCINTILLA_TRICKS::onScintillaUpdateUI( wxStyledTextEvent& aEvent )
 }
 
 
+void SCINTILLA_TRICKS::DoTextVarAutocomplete(
+        const std::function<void( const wxString& xRef, wxArrayString* tokens )>& getTokensFn )
+{
+    wxArrayString autocompleteTokens;
+    int           text_pos = m_te->GetCurrentPos();
+    int           start = m_te->WordStartPosition( text_pos, true );
+    wxString      partial;
+
+    auto textVarRef =
+            [&]( int pos )
+            {
+                return pos >= 2 && m_te->GetCharAt( pos-2 ) == '$'
+                                && m_te->GetCharAt( pos-1 ) == '{';
+            };
+
+    // Check for cross-reference
+    if( start > 1 && m_te->GetCharAt( start-1 ) == ':' )
+    {
+        int refStart = m_te->WordStartPosition( start-1, true );
+
+        if( textVarRef( refStart ) )
+        {
+            partial = m_te->GetRange( start, text_pos );
+            getTokensFn( m_te->GetRange( refStart, start-1 ), &autocompleteTokens );
+        }
+    }
+    else if( textVarRef( start ) )
+    {
+        partial = m_te->GetTextRange( start, text_pos );
+        getTokensFn( wxEmptyString, &autocompleteTokens );
+    }
+
+    DoAutocomplete( partial, autocompleteTokens );
+    m_te->SetFocus();
+}
+
+
 void SCINTILLA_TRICKS::DoAutocomplete( const wxString& aPartial, const wxArrayString& aTokens )
 {
     if( m_suppressAutocomplete )
@@ -420,7 +626,8 @@ void SCINTILLA_TRICKS::DoAutocomplete( const wxString& aPartial, const wxArraySt
                                 return first.CmpNoCase( second );
                             });
 
-        m_te->AutoCompShow( aPartial.size(), wxJoin( matchedTokens, m_te->AutoCompGetSeparator() ) );
+        m_te->AutoCompSetSeparator( '\t' );
+        m_te->AutoCompShow( aPartial.size(), wxJoin( matchedTokens, '\t' ) );
     }
 }
 
