@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2020 Jon Evans <jon@craftyjon.com>
- * Copyright (C) 2021-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,12 +19,12 @@
  */
 
 #include <layer_ids.h>
-#include <pgm_base.h>
 #include <settings/color_settings.h>
 #include <settings/json_settings_internals.h>
 #include <settings/parameters.h>
 #include <settings/settings_manager.h>
 #include <wx/log.h>
+#include <wx/translation.h>
 
 #include "builtin_color_themes.h"
 
@@ -36,21 +36,18 @@ const wxString COLOR_SETTINGS::COLOR_BUILTIN_CLASSIC = "_builtin_classic";
 
 
 COLOR_SETTINGS::COLOR_SETTINGS( const wxString& aFilename, bool aAbsolutePath ) :
-        JSON_SETTINGS( std::move( aFilename ), SETTINGS_LOC::COLORS, colorsSchemaVersion ),
-        m_overrideSchItemColors( false ),
-        m_useBoardStackupColors( true )
+        JSON_SETTINGS( aFilename, SETTINGS_LOC::COLORS, colorsSchemaVersion ),
+        m_overrideSchItemColors( false )
 {
     if( aAbsolutePath )
         SetLocation( SETTINGS_LOC::NONE );
 
-    m_params.emplace_back( new PARAM<wxString>( "meta.name", &m_displayName, wxS( "KiCad Default" ) ) );
+    m_params.emplace_back( new PARAM<wxString>( "meta.name", &m_displayName,
+                                                wxS( "KiCad Default" ) ) );
 
     m_params.emplace_back( new PARAM<bool>( "schematic.override_item_colors",
                                             &m_overrideSchItemColors, false ) );
 
-    m_params.emplace_back( new PARAM<bool>( "3d_viewer.use_board_stackup_colors",
-                                            &m_useBoardStackupColors, true ) );
-
 #define CLR( x, y ) \
     wxASSERT( s_defaultTheme.count( y ) ); \
     m_params.emplace_back( new COLOR_MAP_PARAM( x, y, s_defaultTheme.at( y ), &m_colors ) );
@@ -65,6 +62,8 @@ COLOR_SETTINGS::COLOR_SETTINGS( const wxString& aFilename, bool aAbsolutePath )
     CLR( "schematic.component_body",    LAYER_DEVICE_BACKGROUND      );
     CLR( "schematic.component_outline", LAYER_DEVICE                 );
     CLR( "schematic.cursor",            LAYER_SCHEMATIC_CURSOR       );
+    CLR( "schematic.dnp_marker",        LAYER_DNP_MARKER             );
+    CLR( "schematic.excluded_from_sim", LAYER_EXCLUDED_FROM_SIM      );
     CLR( "schematic.erc_error",         LAYER_ERC_ERR                );
     CLR( "schematic.erc_warning",       LAYER_ERC_WARN               );
     CLR( "schematic.erc_exclusion",     LAYER_ERC_EXCLUSION          );
@@ -77,6 +76,7 @@ COLOR_SETTINGS::COLOR_SETTINGS( const wxString& aFilename, bool aAbsolutePath )
     CLR( "schematic.label_hier",        LAYER_HIERLABEL              );
     CLR( "schematic.label_local",       LAYER_LOCLABEL               );
     CLR( "schematic.netclass_flag",     LAYER_NETCLASS_REFS          );
+    CLR( "schematic.rule_area",         LAYER_RULE_AREAS             );
     CLR( "schematic.no_connect",        LAYER_NOCONNECT              );
     CLR( "schematic.note",              LAYER_NOTES                  );
     CLR( "schematic.private_note",      LAYER_PRIVATE_NOTES          );
@@ -96,6 +96,8 @@ COLOR_SETTINGS::COLOR_SETTINGS( const wxString& aFilename, bool aAbsolutePath )
     CLR( "schematic.wire",              LAYER_WIRE                   );
     CLR( "schematic.worksheet",         LAYER_SCHEMATIC_DRAWINGSHEET );
     CLR( "schematic.page_limits",       LAYER_SCHEMATIC_PAGE_LIMITS  );
+    CLR( "schematic.op_voltages",       LAYER_OP_VOLTAGES            );
+    CLR( "schematic.op_currents",       LAYER_OP_CURRENTS            );
 
     CLR( "gerbview.axes",               LAYER_GERBVIEW_AXES          );
     CLR( "gerbview.background",         LAYER_GERBVIEW_BACKGROUND    );
@@ -108,6 +110,12 @@ COLOR_SETTINGS::COLOR_SETTINGS( const wxString& aFilename, bool aAbsolutePath )
     for( int i = 0, id = GERBVIEW_LAYER_ID_START;
          id < GERBER_DRAWLAYERS_COUNT + GERBVIEW_LAYER_ID_START; ++i, ++id )
     {
+        if( !s_defaultTheme.count( id ) )
+        {
+            wxLogTrace( "colors", "Missing default color for gerbview layer %d", id );
+            continue;
+        }
+
         m_params.emplace_back( new COLOR_MAP_PARAM( "gerbview.layers." + std::to_string( i ), id,
                                                     s_defaultTheme.at( id ), &m_colors ) );
     }
@@ -121,19 +129,18 @@ COLOR_SETTINGS::COLOR_SETTINGS( const wxString& aFilename, bool aAbsolutePath )
     CLR( "board.drc_error",                LAYER_DRC_ERROR          );
     CLR( "board.drc_warning",              LAYER_DRC_WARNING        );
     CLR( "board.drc_exclusion",            LAYER_DRC_EXCLUSION      );
-    CLR( "board.footprint_text_invisible", LAYER_MOD_TEXT_INVISIBLE );
     CLR( "board.grid",                     LAYER_GRID               );
     CLR( "board.grid_axes",                LAYER_GRID_AXES          );
     CLR( "board.pad_plated_hole",          LAYER_PAD_PLATEDHOLES    );
-    CLR( "board.pad_through_hole",         LAYER_PADS_TH            );
     CLR( "board.plated_hole",              LAYER_NON_PLATEDHOLES    );
     CLR( "board.ratsnest",                 LAYER_RATSNEST           );
-    CLR( "board.via_blind_buried",         LAYER_VIA_BBLIND         );
     CLR( "board.via_hole",                 LAYER_VIA_HOLES          );
-    CLR( "board.via_micro",                LAYER_VIA_MICROVIA       );
-    CLR( "board.via_through",              LAYER_VIA_THROUGH        );
+    CLR( "board.via_hole_walls",           LAYER_VIA_HOLEWALLS      );
     CLR( "board.worksheet",                LAYER_DRAWINGSHEET       );
     CLR( "board.page_limits",              LAYER_PAGE_LIMITS        );
+    CLR( "board.track_net_names",          NETNAMES_LAYER_ID_START  );
+    CLR( "board.pad_net_names",            LAYER_PAD_NETNAMES       );
+    CLR( "board.via_net_names",            LAYER_VIA_NETNAMES       );
 
     CLR( "board.copper.f",      F_Cu    );
     CLR( "board.copper.in1",    In1_Cu  );
@@ -195,12 +202,48 @@ COLOR_SETTINGS::COLOR_SETTINGS( const wxString& aFilename, bool aAbsolutePath )
     CLR( "board.user_7",        User_7    );
     CLR( "board.user_8",        User_8    );
     CLR( "board.user_9",        User_9    );
+    CLR( "board.user_10",       User_10   );
+    CLR( "board.user_11",       User_11   );
+    CLR( "board.user_12",       User_12   );
+    CLR( "board.user_13",       User_13   );
+    CLR( "board.user_14",       User_14   );
+    CLR( "board.user_15",       User_15   );
+    CLR( "board.user_16",       User_16   );
+    CLR( "board.user_17",       User_17   );
+    CLR( "board.user_18",       User_18   );
+    CLR( "board.user_19",       User_19   );
+    CLR( "board.user_20",       User_20   );
+    CLR( "board.user_21",       User_21   );
+    CLR( "board.user_22",       User_22   );
+    CLR( "board.user_23",       User_23   );
+    CLR( "board.user_24",       User_24   );
+    CLR( "board.user_25",       User_25   );
+    CLR( "board.user_26",       User_26   );
+    CLR( "board.user_27",       User_27   );
+    CLR( "board.user_28",       User_28   );
+    CLR( "board.user_29",       User_29   );
+    CLR( "board.user_30",       User_30   );
+    CLR( "board.user_31",       User_31   );
+    CLR( "board.user_32",       User_32   );
+    CLR( "board.user_33",       User_33   );
+    CLR( "board.user_34",       User_34   );
+    CLR( "board.user_35",       User_35   );
+    CLR( "board.user_36",       User_36   );
+    CLR( "board.user_37",       User_37   );
+    CLR( "board.user_38",       User_38   );
+    CLR( "board.user_39",       User_39   );
+    CLR( "board.user_40",       User_40   );
+    CLR( "board.user_41",       User_41   );
+    CLR( "board.user_42",       User_42   );
+    CLR( "board.user_43",       User_43   );
+    CLR( "board.user_44",       User_44   );
+    CLR( "board.user_45",       User_45   );
 
     // Colors for 3D viewer, which are used as defaults unless overridden by the board
     CLR( "3d_viewer.background_bottom", LAYER_3D_BACKGROUND_BOTTOM );
     CLR( "3d_viewer.background_top",    LAYER_3D_BACKGROUND_TOP    );
     CLR( "3d_viewer.board",             LAYER_3D_BOARD             );
-    CLR( "3d_viewer.copper",            LAYER_3D_COPPER            );
+    CLR( "3d_viewer.copper",            LAYER_3D_COPPER_TOP        );
     CLR( "3d_viewer.silkscreen_bottom", LAYER_3D_SILKSCREEN_BOTTOM );
     CLR( "3d_viewer.silkscreen_top",    LAYER_3D_SILKSCREEN_TOP    );
     CLR( "3d_viewer.soldermask_bottom", LAYER_3D_SOLDERMASK_BOTTOM );
@@ -251,6 +294,9 @@ COLOR_SETTINGS::COLOR_SETTINGS( const wxString& aFilename, bool aAbsolutePath )
 
                 return true;
             } );
+
+    // this bump shouldn't have happened; add a no-op migration to avoid future issues
+    registerMigration( 4, 5, []() { return true; } );
 }
 
 
@@ -275,7 +321,6 @@ void COLOR_SETTINGS::initFromOther( const COLOR_SETTINGS& aOther )
 {
     m_displayName           = aOther.m_displayName;
     m_overrideSchItemColors = aOther.m_overrideSchItemColors;
-    m_useBoardStackupColors = aOther.m_useBoardStackupColors;
     m_colors                = aOther.m_colors;
     m_defaultColors         = aOther.m_defaultColors;
     m_writeFile             = aOther.m_writeFile;
@@ -312,14 +357,16 @@ bool COLOR_SETTINGS::migrateSchema0to1()
 
     if( !Contains( "fpedit" ) )
     {
-        wxLogTrace( traceSettings, wxT( "migrateSchema0to1: %s doesn't have fpedit settings; skipping." ),
+        wxLogTrace( traceSettings,
+                    wxT( "migrateSchema0to1: %s doesn't have fpedit settings; skipping." ),
                     m_filename );
         return true;
     }
 
-    wxString filename = m_filename + wxT( "_footprints" );
+    wxString filename = GetFilename().BeforeLast( '.' ) + wxT( "_footprints" );
 
     COLOR_SETTINGS* fpsettings = m_manager->AddNewColorSettings( filename );
+    fpsettings->SetLocation( GetLocation() );
 
     // Start out with a clone
     fpsettings->m_internals->CloneFrom( *m_internals );
@@ -364,8 +411,10 @@ COLOR4D COLOR_SETTINGS::GetDefaultColor( int aLayer )
 
         if( p )
             m_defaultColors[aLayer] = p->GetDefault();
+        else if( IsCopperLayer( aLayer ) )
+            m_defaultColors[aLayer] = s_copperColors[aLayer % s_copperColors.size()];
         else
-            m_defaultColors[aLayer] = COLOR4D::UNSPECIFIED;
+            m_defaultColors[aLayer] = s_userColors[aLayer % s_userColors.size()];
     }
 
     return m_defaultColors.at( aLayer );
