@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,16 +22,23 @@
  */
 
 /**
- * @file string.cpp
+ * @file string_utils.cpp
  * @brief Some useful functions to handle strings.
  */
 
 #include <clocale>
 #include <cmath>
+#include <map>
+#include <core/map_helpers.h>
 #include <fmt/core.h>
 #include <macros.h>
 #include <richio.h>                        // StrPrintf
 #include <string_utils.h>
+#include <wx_filename.h>
+#include <fmt/chrono.h>
+#include <wx/log.h>
+#include <wx/regex.h>
+#include "locale_io.h"
 
 
 /**
@@ -39,7 +46,35 @@
  * platforms.  This is the list of illegal file name characters for Windows which includes
  * the illegal file name characters for Linux and OSX.
  */
-static const char illegalFileNameChars[] = "\\/:\"<>|";
+static const char illegalFileNameChars[] = "\\/:\"<>|*?";
+
+
+// Checks if a full filename is valid, i.e. does not contains illegal chars
+bool IsFullFileNameValid( const wxString& aFullFilename )
+{
+
+    // Test for forbidden chars in aFullFilename.
+    // '\'and '/' are allowed here because aFullFilename can be a full path, and
+    // ':' is allowed on Windows as second char in string.
+    // So remove allowed separators from string to test
+    wxString filtered_fullpath = aFullFilename;
+
+#ifdef __WINDOWS__
+    // On MSW, the list returned by wxFileName::GetForbiddenChars() contains separators
+    // '\'and '/'
+    filtered_fullpath.Replace( "/", "_" );
+    filtered_fullpath.Replace( "\\", "_" );
+
+    // A disk identifier is allowed, and therefore remove its separator
+    if( filtered_fullpath.Length() > 1 && filtered_fullpath[1] == ':' )
+        filtered_fullpath[1] = ' ';
+#endif
+
+    if( wxString::npos != filtered_fullpath.find_first_of( wxFileName::GetForbiddenChars() ) )
+        return false;
+
+    return true;
+}
 
 
 wxString ConvertToNewOverbarNotation( const wxString& aOldStr )
@@ -159,9 +194,12 @@ wxString EscapeString( const wxString& aSource, ESCAPE_CONTEXT aContext )
             else
                 converted += c;
         }
-        else if( aContext == CTX_LIBID )
+        else if( aContext == CTX_LIBID || aContext == CTX_LEGACY_LIBID )
         {
-            if( c == '\\' )
+            // We no longer escape '/' in LIB_IDs, but we used to
+            if( c == '/' && aContext == CTX_LEGACY_LIBID )
+                converted += wxT( "{slash}" );
+            else if( c == '\\' )
                 converted += wxT( "{backslash}" );
             else if( c == '<' )
                 converted += wxT( "{lt}" );
@@ -200,7 +238,7 @@ wxString EscapeString( const wxString& aSource, ESCAPE_CONTEXT aContext )
             {
                 unsigned int code = c;
                 char buffer[16];
-                snprintf( buffer, sizeof(buffer), "\\\\u%4.4X", code );
+                snprintf( buffer, sizeof(buffer), "\\u%4.4X", code );
                 converted += buffer;
             }
             else
@@ -277,64 +315,70 @@ wxString UnescapeString( const wxString& aSource )
     wxString newbuf;
     newbuf.reserve( sourceLen );
 
+    wxUniChar prev = 0;
+    wxUniChar ch = 0;
+
     for( size_t i = 0; i < sourceLen; ++i )
     {
-        wxUniChar ch = aSource[i];
+        prev = ch;
+        ch = aSource[i];
 
-        if( ( ch == '$' || ch == '~' || ch == '^' || ch == '_' )
-                && i + 1 < sourceLen && aSource[i+1] == '{' )
-        {
-            for( ; i < sourceLen; ++i )
-            {
-                ch = aSource[i];
-                newbuf += ch;
-
-                if( ch == '}' )
-                    break;
-            }
-        }
-        else if( ch == '{' )
+        if( ch == '{' )
         {
             wxString token;
             int      depth = 1;
+            bool     terminated = false;
 
             for( i = i + 1; i < sourceLen; ++i )
             {
                 ch = aSource[i];
+
                 if( ch == '{' )
                     depth++;
                 else if( ch == '}' )
                     depth--;
 
                 if( depth <= 0 )
+                {
+                    terminated = true;
                     break;
+                }
                 else
-                    token.append( ch );
+                {
+                    token << ch;
+                }
             }
 
-            if(      token == wxT( "dblquote" ) )  newbuf.append( wxT( "\"" ) );
-            else if( token == wxT( "quote" ) )     newbuf.append( wxT( "'" ) );
-            else if( token == wxT( "lt" ) )        newbuf.append( wxT( "<" ) );
-            else if( token == wxT( "gt" ) )        newbuf.append( wxT( ">" ) );
-            else if( token == wxT( "backslash" ) ) newbuf.append( wxT( "\\" ) );
-            else if( token == wxT( "slash" ) )     newbuf.append( wxT( "/" ) );
-            else if( token == wxT( "bar" ) )       newbuf.append( wxT( "|" ) );
-            else if( token == wxT( "comma" ) )     newbuf.append( wxT( "," ) );
-            else if( token == wxT( "colon" ) )     newbuf.append( wxT( ":" ) );
-            else if( token == wxT( "space" ) )     newbuf.append( wxT( " " ) );
-            else if( token == wxT( "dollar" ) )    newbuf.append( wxT( "$" ) );
-            else if( token == wxT( "tab" ) )       newbuf.append( wxT( "\t" ) );
-            else if( token == wxT( "return" ) )    newbuf.append( wxT( "\n" ) );
-            else if( token == wxT( "brace" ) )     newbuf.append( wxT( "{" ) );
-            else if( token.IsEmpty() )             newbuf.append( wxT( "{" ) );
+            if( !terminated )
+            {
+                newbuf << wxT( "{" ) << UnescapeString( token );
+            }
+            else if( prev == '$' || prev == '~' || prev == '^' || prev == '_' )
+            {
+                newbuf << wxT( "{" ) << UnescapeString( token ) << wxT( "}" );
+            }
+            else if( token == wxT( "dblquote" ) )  newbuf << wxT( "\"" );
+            else if( token == wxT( "quote" ) )     newbuf << wxT( "'" );
+            else if( token == wxT( "lt" ) )        newbuf << wxT( "<" );
+            else if( token == wxT( "gt" ) )        newbuf << wxT( ">" );
+            else if( token == wxT( "backslash" ) ) newbuf << wxT( "\\" );
+            else if( token == wxT( "slash" ) )     newbuf << wxT( "/" );
+            else if( token == wxT( "bar" ) )       newbuf << wxT( "|" );
+            else if( token == wxT( "comma" ) )     newbuf << wxT( "," );
+            else if( token == wxT( "colon" ) )     newbuf << wxT( ":" );
+            else if( token == wxT( "space" ) )     newbuf << wxT( " " );
+            else if( token == wxT( "dollar" ) )    newbuf << wxT( "$" );
+            else if( token == wxT( "tab" ) )       newbuf << wxT( "\t" );
+            else if( token == wxT( "return" ) )    newbuf << wxT( "\n" );
+            else if( token == wxT( "brace" ) )     newbuf << wxT( "{" );
             else
             {
-                newbuf.append( wxT( "{" ) + UnescapeString( token ) + wxT( "}" ) );
+                newbuf << wxT( "{" ) << UnescapeString( token ) << wxT( "}" );
             }
         }
         else
         {
-            newbuf.append( ch );
+            newbuf << ch;
         }
     }
 
@@ -402,7 +446,7 @@ int ReadDelimitedText( wxString* aDest, const char* aSource )
         }
     }
 
-    *aDest = FROM_UTF8( utf8.c_str() );
+    *aDest = From_UTF8( utf8.c_str() );
 
     return aSource - start;
 }
@@ -418,7 +462,7 @@ int ReadDelimitedText( char* aDest, const char* aSource, int aDestSize )
     char*       limit = aDest + aDestSize - 1;
     char        cc;
 
-    while( (cc = *aSource++) != 0 && aDest < limit )
+    while( ( cc = *aSource++ ) != 0 && aDest < limit )
     {
         if( cc == '"' )
         {
@@ -427,7 +471,6 @@ int ReadDelimitedText( char* aDest, const char* aSource, int aDestSize )
 
             inside = true;      // first delimiter found, make note, do not copy
         }
-
         else if( inside )
         {
             if( cc == '\\' )
@@ -524,6 +567,94 @@ wxString EscapeHTML( const wxString& aString )
 }
 
 
+wxString UnescapeHTML( const wxString& aString )
+{
+    wxString converted = aString;
+
+    // clang-format off
+    static const std::map<wxString, wxString> c_replacements = {
+        { wxS( "quot" ), wxS( "\"" ) },
+        { wxS( "apos" ), wxS( "'" ) },
+        { wxS( "amp" ), wxS( "&" ) },
+        { wxS( "lt" ), wxS( "<" ) },
+        { wxS( "gt" ), wxS( ">" ) }
+    };
+    // clang-format on
+
+    // Construct regex
+    wxString regexStr = "&(#(\\d*)|#x([a-zA-Z0-9]{4})";
+
+    for( auto& [key, value] : c_replacements )
+        regexStr << '|' << key;
+
+    regexStr << ");";
+
+    wxRegEx regex( regexStr );
+
+    // Process matches
+    size_t start = 0;
+    size_t len = 0;
+
+    wxString result;
+    wxString str = converted;
+
+    while( regex.Matches( str ) )
+    {
+        std::vector<wxString> matches;
+        regex.GetMatch( &start, &len );
+
+        result << str.Left( start );
+
+        wxString code = regex.GetMatch( str, 1 );
+        wxString codeDec = regex.GetMatch( str, 2 );
+        wxString codeHex = regex.GetMatch( str, 3 );
+
+        if( !codeDec.IsEmpty() || !codeHex.IsEmpty() )
+        {
+            unsigned long codeVal = 0;
+
+            if( !codeDec.IsEmpty() )
+                codeDec.ToCULong( &codeVal );
+            else if( !codeHex.IsEmpty() )
+                codeHex.ToCULong( &codeVal, 16 );
+
+            if( codeVal != 0 )
+                result << wxUniChar( codeVal );
+        }
+        else if( auto val = get_opt( c_replacements, code ) )
+        {
+            result << *val;
+        }
+
+        str = str.Mid( start + len );
+    }
+
+    result << str;
+
+    return result;
+}
+
+
+wxString RemoveHTMLTags( const wxString& aInput )
+{
+    wxString str = aInput;
+    wxRegEx( wxS( "<[^>]*>" ) ).ReplaceAll( &str, wxEmptyString );
+
+    return str;
+}
+
+
+wxString LinkifyHTML( wxString aStr )
+{
+    wxRegEx regex( wxS( "\\b(https?|ftp|file)://([-\\w+&@#/%?=~|!:,.;]*[^.,:;<>\\s\u00b6])" ),
+                   wxRE_ICASE );
+
+    regex.ReplaceAll( &aStr, "<a href=\"\\0\">\\0</a>" );
+
+    return aStr;
+}
+
+
 bool NoPrintableChars( const wxString& aString )
 {
     wxString tmp = aString;
@@ -532,10 +663,6 @@ bool NoPrintableChars( const wxString& aString )
 }
 
 
-/**
- * Return the number of printable (ie: non-formatting) chars.  Used to approximate rendered
- * text size when speed is more important than accuracy.
- */
 int PrintableCharCount( const wxString& aString )
 {
     int char_count = 0;
@@ -651,12 +778,17 @@ char* GetLine( FILE* File, char* Line, int* LineNum, int SizeLine )
 }
 
 
-wxString DateAndTime()
+wxString GetISO8601CurrentDateTime()
 {
-    wxDateTime datetime = wxDateTime::Now();
-
-    datetime.SetCountry( wxDateTime::Country_Default );
-    return datetime.Format( wxDefaultDateTimeFormat, wxDateTime::Local );
+    // on msys2 variant mingw64, in fmt::format the %z format
+    // (offset from UTC in the ISO 8601 format, e.g. -0430) does not work,
+    // and is in fact %Z (locale-dependent time zone name or abbreviation) and breaks our date.
+    // However, on msys2 variant ucrt64, it works (this is not the same code in fmt::format)
+#if defined(__MINGW32__) && !defined(_UCRT)
+    return fmt::format( "{:%FT%T}", fmt::localtime( std::time( nullptr ) ) );
+#else
+    return fmt::format( "{:%FT%T%z}", fmt::localtime( std::time( nullptr ) ) );
+#endif
 }
 
 
@@ -704,11 +836,14 @@ int StrNumCmp( const wxString& aString1, const wxString& aString2, bool aIgnoreC
         // Any numerical comparisons to here are identical.
         if( aIgnoreCase )
         {
-            if( wxToupper( c1 ) < wxToupper( c2 ) )
-                return -1;
+            if( c1 != c2 )
+            {
+                wxUniChar uc1 = wxToupper( c1 );
+                wxUniChar uc2 = wxToupper( c2 );
 
-            if( wxToupper( c1 ) > wxToupper( c2 ) )
-                return 1;
+                if( uc1 != uc2 )
+                    return uc1 < uc2 ? -1 : 1;
+            }
         }
         else
         {
@@ -742,8 +877,10 @@ int StrNumCmp( const wxString& aString1, const wxString& aString2, bool aIgnoreC
 bool WildCompareString( const wxString& pattern, const wxString& string_to_tst,
                         bool case_sensitive )
 {
-    const wxChar* cp = nullptr, * mp = nullptr;
-    const wxChar* wild, * str;
+    const wxChar* cp = nullptr;
+    const wxChar* mp = nullptr;
+    const wxChar* wild = nullptr;
+    const wxChar* str = nullptr;
     wxString      _pattern, _string_to_tst;
 
     if( case_sensitive )
@@ -757,7 +894,7 @@ bool WildCompareString( const wxString& pattern, const wxString& string_to_tst,
         _pattern.MakeUpper();
         _string_to_tst = string_to_tst;
         _string_to_tst.MakeUpper();
-        wild   = _pattern.GetData();
+        wild = _pattern.GetData();
         str = _string_to_tst.GetData();
     }
 
@@ -775,7 +912,8 @@ bool WildCompareString( const wxString& pattern, const wxString& string_to_tst,
         if( *wild == '*' )
         {
             if( !*++wild )
-                return 1;
+                return true;
+
             mp = wild;
             cp = str + 1;
         }
@@ -802,7 +940,8 @@ bool WildCompareString( const wxString& pattern, const wxString& string_to_tst,
 
 bool ApplyModifier( double& value, const wxString& aString )
 {
-    static const wxString modifiers( wxT( "pnumkKM" ) );
+    /// Although the two 'μ's look the same, they are U+03BC and U+00B5
+    static const wxString modifiers( wxT( "pnuµμmkKM" ) );
 
     if( !aString.length() )
         return false;
@@ -822,18 +961,21 @@ bool ApplyModifier( double& value, const wxString& aString )
     }
 
     if( units.length()
-            && !units.CmpNoCase( wxT( "F" ) )
-            && !units.CmpNoCase( wxT( "hz" ) )
-            && !units.CmpNoCase( wxT( "W" ) )
-            && !units.CmpNoCase( wxT( "V" ) )
-            && !units.CmpNoCase( wxT( "H" ) ) )
+            && !units.IsSameAs( wxT( "F" ), false )
+            && !units.IsSameAs( wxT( "hz" ), false )
+            && !units.IsSameAs( wxT( "W" ), false )
+            && !units.IsSameAs( wxT( "V" ), false )
+            && !units.IsSameAs( wxT( "A" ), false )
+            && !units.IsSameAs( wxT( "H" ), false ) )
+    {
         return false;
+    }
 
     if( modifier == 'p' )
         value *= 1.0e-12;
     if( modifier == 'n' )
         value *= 1.0e-9;
-    else if( modifier == 'u' )
+    else if( modifier == 'u' || modifier == wxS( "µ" )[0] || modifier == wxS( "μ" )[0] )
         value *= 1.0e-6;
     else if( modifier == 'm' )
         value *= 1.0e-3;
@@ -848,6 +990,125 @@ bool ApplyModifier( double& value, const wxString& aString )
 }
 
 
+bool convertSeparators( wxString* value )
+{
+    // Note: fetching the decimal separator from the current locale isn't a silver bullet because
+    // it assumes the current computer's locale is the same as the locale the schematic was
+    // authored in -- something that isn't true, for instance, when sharing designs through
+    // DIYAudio.com.
+    //
+    // Some values are self-describing: multiple instances of a single separator character must be
+    // thousands separators; a single instance of each character must be a thousands separator
+    // followed by a decimal separator; etc.
+    //
+    // Only when presented with an ambiguous value do we fall back on the current locale.
+
+    value->Replace( wxS( " " ), wxEmptyString );
+
+    wxChar ambiguousSeparator = '?';
+    wxChar thousandsSeparator = '?';
+    bool   thousandsSeparatorFound = false;
+    wxChar decimalSeparator = '?';
+    bool   decimalSeparatorFound = false;
+    int    digits = 0;
+
+    for( int ii = (int) value->length() - 1; ii >= 0; --ii )
+    {
+        wxChar c = value->GetChar( ii );
+
+        if( c >= '0' && c <= '9' )
+        {
+            digits += 1;
+        }
+        else if( c == '.' || c == ',' )
+        {
+            if( decimalSeparator != '?' || thousandsSeparator != '?' )
+            {
+                // We've previously found a non-ambiguous separator...
+
+                if( c == decimalSeparator )
+                {
+                    if( thousandsSeparatorFound )
+                        return false;       // decimal before thousands
+                    else if( decimalSeparatorFound )
+                        return false;       // more than one decimal
+                    else
+                        decimalSeparatorFound = true;
+                }
+                else if( c == thousandsSeparator )
+                {
+                    if( digits != 3 )
+                        return false;       // thousands not followed by 3 digits
+                    else
+                        thousandsSeparatorFound = true;
+                }
+            }
+            else if( ambiguousSeparator != '?' )
+            {
+                // We've previously found a separator, but we don't know for sure which...
+
+                if( c == ambiguousSeparator )
+                {
+                    // They both must be thousands separators
+                    thousandsSeparator = ambiguousSeparator;
+                    thousandsSeparatorFound = true;
+                    decimalSeparator = c == '.' ? ',' : '.';
+                }
+                else
+                {
+                    // The first must have been a decimal, and this must be a thousands.
+                    decimalSeparator = ambiguousSeparator;
+                    decimalSeparatorFound = true;
+                    thousandsSeparator = c;
+                    thousandsSeparatorFound = true;
+                }
+            }
+            else
+            {
+                // This is the first separator...
+
+                // If it's preceded by a '0' (only), or if it's followed by some number of
+                // digits not equal to 3, then it -must- be a decimal separator.
+                //
+                // In all other cases we don't really know what it is yet.
+
+                if( ( ii == 1 && value->GetChar( 0 ) == '0' ) || digits != 3 )
+                {
+                    decimalSeparator = c;
+                    decimalSeparatorFound = true;
+                    thousandsSeparator = c == '.' ? ',' : '.';
+                }
+                else
+                {
+                    ambiguousSeparator = c;
+                }
+            }
+
+            digits = 0;
+        }
+        else
+        {
+            digits = 0;
+        }
+    }
+
+    // If we found nothing definitive then we have to look at the current locale
+    if( decimalSeparator == '?' && thousandsSeparator == '?' )
+    {
+        const struct lconv* lc = localeconv();
+
+        decimalSeparator = lc->decimal_point[0];
+        thousandsSeparator = decimalSeparator == '.' ? ',' : '.';
+    }
+
+    // Convert to C-locale
+    value->Replace( thousandsSeparator, wxEmptyString );
+    value->Replace( decimalSeparator, '.' );
+
+    return true;
+}
+
+
 int ValueStringCompare( const wxString& strFWord, const wxString& strSWord )
 {
     // Compare unescaped text
@@ -880,6 +1141,11 @@ int ValueStringCompare( const wxString& strFWord, const wxString& strSWord )
         double lSecondNumber = 0;
         bool   endingIsModifier = false;
 
+        convertSeparators( &strFWordMid );
+        convertSeparators( &strSWordMid );
+
+        LOCALE_IO toggle;    // toggles on, then off, the C locale.
+
         strFWordMid.ToDouble( &lFirstNumber );
         strSWordMid.ToDouble( &lSecondNumber );
 
@@ -987,7 +1253,7 @@ int GetTrailingInt( const wxString& aStr )
 
 wxString GetIllegalFileNameWxChars()
 {
-    return FROM_UTF8( illegalFileNameChars );
+    return From_UTF8( illegalFileNameChars );
 }
 
 
@@ -1135,7 +1401,7 @@ std::string UIDouble2Str( double aValue )
     {
         // For these small values, %f works fine,
         // and %g gives an exponent
-        len = snprintf( buf, sizeof(buf), "%.16f", aValue );
+        len = snprintf( buf, sizeof( buf ), "%.16f", aValue );
 
         while( --len > 0 && buf[len] == '0' )
             buf[len] = '\0';
@@ -1149,8 +1415,63 @@ std::string UIDouble2Str( double aValue )
     {
         // For these values, %g works fine, and sometimes %f
         // gives a bad value (try aValue = 1.222222222222, with %.16f format!)
-        len = snprintf( buf, sizeof(buf), "%.10g", aValue );
+        len = snprintf( buf, sizeof( buf ), "%.10g", aValue );
     }
 
     return std::string( buf, len );
 }
+
+
+wxString From_UTF8( const char* cstring )
+{
+    // Convert an expected UTF8 encoded C string to a wxString
+    wxString line = wxString::FromUTF8( cstring );
+
+    if( line.IsEmpty() )  // happens when cstring is not a valid UTF8 sequence
+    {
+        line = wxConvCurrent->cMB2WC( cstring );    // try to use locale conversion
+
+        if( line.IsEmpty() )
+            line = wxString::From8BitData( cstring );    // try to use native string
+    }
+
+    return line;
+}
+
+
+wxString From_UTF8( const std::string& aString )
+{
+    // Convert an expected UTF8 encoded std::string to a wxString
+    wxString line = wxString::FromUTF8( aString );
+
+    if( line.IsEmpty() )  // happens when aString is not a valid UTF8 sequence
+    {
+        line = wxConvCurrent->cMB2WC( aString.c_str() );    // try to use locale conversion
+
+        if( line.IsEmpty() )
+            line = wxString::From8BitData( aString.c_str() );    // try to use native string
+    }
+
+   return line;
+}
+
+
+wxString NormalizeFileUri( const wxString& aFileUri )
+{
+    wxString uriPathAndFileName;
+
+    wxCHECK( aFileUri.StartsWith( wxS( "file://" ), &uriPathAndFileName ), aFileUri );
+
+    wxString tmp = uriPathAndFileName;
+    wxString retv = wxS( "file://" );
+
+    tmp.Replace( wxS( "\\" ), wxS( "/" ) );
+    tmp.Replace( wxS( ":" ), wxS( "" ) );
+
+    if( !tmp.IsEmpty() && tmp[0] != '/' )
+        tmp = wxS( "/" ) + tmp;
+
+    retv += tmp;
+
+    return retv;
+}
