@@ -1,8 +1,8 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2013-2018 CERN
- * Copyright (C) 2019-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2013-2023 CERN
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  *
@@ -25,16 +25,20 @@
  */
 
 #include <core/kicad_algo.h>
-#include <core/optional.h>
+#include <optional>
 #include <map>
 #include <stack>
 #include <trace_helpers.h>
+#include <kiplatform/ui.h>
 
 #include <wx/event.h>
 #include <wx/clipbrd.h>
 #include <wx/app.h>
 
+#include <math/vector2wx.h>
+
 #include <view/view.h>
+#include <view/view_controls.h>
 #include <eda_base_frame.h>
 #include <tool/tool_base.h>
 #include <tool/tool_interactive.h>
@@ -64,6 +68,7 @@ struct TOOL_MANAGER::TOOL_STATE
         contextMenu        = aState.contextMenu;
         contextMenuTrigger = aState.contextMenuTrigger;
         cofunc             = aState.cofunc;
+        initialEvent       = aState.initialEvent;
         wakeupEvent        = aState.wakeupEvent;
         waitEvents         = aState.waitEvents;
         transitions        = aState.transitions;
@@ -102,6 +107,9 @@ struct TOOL_MANAGER::TOOL_STATE
     /// Tool execution context
     COROUTINE<int, const TOOL_EVENT&>* cofunc;
 
+    /// The first event that triggered activation of the tool.
+    TOOL_EVENT initialEvent;
+
     /// The event that triggered the execution/wakeup of the tool after Wait() call
     TOOL_EVENT wakeupEvent;
 
@@ -125,10 +133,12 @@ struct TOOL_MANAGER::TOOL_STATE
         contextMenu        = aState.contextMenu;
         contextMenuTrigger = aState.contextMenuTrigger;
         cofunc             = aState.cofunc;
+        initialEvent       = aState.initialEvent;
         wakeupEvent        = aState.wakeupEvent;
         waitEvents         = aState.waitEvents;
         transitions        = aState.transitions;
         vcSettings         = aState.vcSettings;
+
         // do not copy stateStack
         return *this;
     }
@@ -178,10 +188,10 @@ struct TOOL_MANAGER::TOOL_STATE
     }
 
 private:
-    ///< Stack preserving previous states of a TOOL.
+    /// Stack preserving previous states of a TOOL.
     std::stack<std::unique_ptr<TOOL_STATE>> stateStack;
 
-    ///< Restores the initial state.
+    /// Restores the initial state.
     void clear()
     {
         idle               = true;
@@ -206,7 +216,8 @@ TOOL_MANAGER::TOOL_MANAGER() :
         m_warpMouseAfterContextMenu( true ),
         m_menuActive( false ),
         m_menuOwner( -1 ),
-        m_activeState( nullptr )
+        m_activeState( nullptr ),
+        m_shuttingDown( false )
 {
     m_actionMgr = new ACTION_MANAGER( this );
 }
@@ -236,6 +247,10 @@ void TOOL_MANAGER::RegisterTool( TOOL_BASE* aTool )
     wxASSERT_MSG( m_toolTypes.find( typeid( *aTool ).name() ) == m_toolTypes.end(),
                   wxT( "Adding two tools of the same type may result in unexpected behavior.") );
 
+    wxLogTrace( kicadTraceToolStack,
+                wxS( "TOOL_MANAGER::RegisterTool: Registering tool %s with ID %d" ),
+                aTool->GetName(), aTool->GetId() );
+
     m_toolOrder.push_back( aTool );
 
     TOOL_STATE* st = new TOOL_STATE( aTool );
@@ -256,7 +271,7 @@ bool TOOL_MANAGER::InvokeTool( TOOL_ID aToolId )
     if( tool && tool->GetType() == INTERACTIVE )
         return invokeTool( tool );
 
-    wxLogTrace( kicadTraceToolStack, "TOOL_MANAGER::InvokeTool - no tool with ID %d",
+    wxLogTrace( kicadTraceToolStack, wxS( "TOOL_MANAGER::InvokeTool - no tool with ID %d" ),
                                      aToolId );
 
     return false;       // there is no tool with the given id
@@ -270,14 +285,15 @@ bool TOOL_MANAGER::InvokeTool( const std::string& aToolName )
     if( tool && tool->GetType() == INTERACTIVE )
         return invokeTool( tool );
 
-    wxLogTrace( kicadTraceToolStack, "TOOL_MANAGER::InvokeTool - no tool with name %s",
-                                     aToolName );
+    wxLogTrace( kicadTraceToolStack, wxS( "TOOL_MANAGER::InvokeTool - no tool with name %s" ),
+                aToolName );
 
     return false;       // there is no tool with the given name
 }
 
 
-bool TOOL_MANAGER::RunAction( const std::string& aActionName, bool aNow, void* aParam )
+bool TOOL_MANAGER::doRunAction( const std::string& aActionName, bool aNow, const ki::any& aParam,
+                                COMMIT* aCommit )
 {
     TOOL_ACTION* action = m_actionMgr->FindAction( aActionName );
 
@@ -287,9 +303,9 @@ bool TOOL_MANAGER::RunAction( const std::string& aActionName, bool aNow, void* a
         return false;
     }
 
-    RunAction( *action, aNow, aParam );
+    doRunAction( *action, aNow, aParam, aCommit );
 
-    return false;
+    return true;
 }
 
 
@@ -298,7 +314,7 @@ VECTOR2D TOOL_MANAGER::GetMousePosition() const
     if( m_viewControls )
         return m_viewControls->GetMousePosition();
     else
-        return wxGetMousePosition();
+        return ToVECTOR2D( KIPLATFORM::UI::GetMousePosition() );
 }
 
 
@@ -307,26 +323,62 @@ VECTOR2D TOOL_MANAGER::GetCursorPosition() const
    if( m_viewControls )
        return m_viewControls->GetCursorPosition();
    else
-       return wxGetMousePosition();
+       return ToVECTOR2D( KIPLATFORM::UI::GetMousePosition() );
 }
 
 
-bool TOOL_MANAGER::RunAction( const TOOL_ACTION& aAction, bool aNow, void* aParam )
+bool TOOL_MANAGER::doRunAction( const TOOL_ACTION& aAction, bool aNow, const ki::any& aParam,
+                                COMMIT* aCommit )
 {
-    bool       handled = false;
+    if( m_shuttingDown )
+        return true;
+
+    bool       retVal = false;
     TOOL_EVENT event = aAction.MakeEvent();
 
+    // We initialize the SYNCHRONOUS state to finished so that tools that don't have an event
+    // loop won't hang if someone forgets to set the state.
+    std::atomic<SYNCRONOUS_TOOL_STATE> synchronousControl = STS_FINISHED;
+
     if( event.Category() == TC_COMMAND )
         event.SetMousePosition( GetCursorPosition() );
 
     // Allow to override the action parameter
-    if( aParam )
+    if( aParam.has_value() )
         event.SetParameter( aParam );
 
+    // Pass the commit (if any)
+    if( aCommit )
+    {
+        event.SetSynchronous( &synchronousControl );
+        event.SetCommit( aCommit );
+    }
+
     if( aNow )
     {
         TOOL_STATE* current = m_activeState;
-        handled = processEvent( event );
+
+        if( aCommit )
+        {
+            // An event with a commit must be run synchronously
+            processEvent( event );
+
+            while( synchronousControl == STS_RUNNING )
+            {
+                wxYield();          // Needed to honor mouse (and other) events during editing
+                wxMilliSleep( 1 );  // Needed to avoid 100% use of one cpu core.
+                                    // The sleeping time must be must be small to avoid
+                                    // noticeable lag in mouse and editing events
+                                    // (1 to 5 ms is a good value)
+            }
+
+            retVal = synchronousControl != STS_CANCELLED;
+        }
+        else
+        {
+            retVal = processEvent( event );
+        }
+
         setActiveState( current );
         UpdateUI( event );
     }
@@ -335,7 +387,7 @@ bool TOOL_MANAGER::RunAction( const TOOL_ACTION& aAction, bool aNow, void* aPara
         PostEvent( event );
     }
 
-    return handled;
+    return retVal;
 }
 
 
@@ -350,9 +402,15 @@ void TOOL_MANAGER::CancelTool()
 void TOOL_MANAGER::PrimeTool( const VECTOR2D& aPosition )
 {
     int modifiers = 0;
-    modifiers |= wxGetKeyState( WXK_SHIFT ) ? MD_SHIFT : 0;
-    modifiers |= wxGetKeyState( WXK_CONTROL ) ? MD_CTRL : 0;
-    modifiers |= wxGetKeyState( WXK_ALT ) ? MD_ALT : 0;
+
+    /*
+     * Don't include any modifiers.  They're part of the hotkey, not part of the resulting
+     * click.
+     *
+     * modifiers |= wxGetKeyState( WXK_SHIFT ) ? MD_SHIFT : 0;
+     * modifiers |= wxGetKeyState( WXK_CONTROL ) ? MD_CTRL : 0;
+     * modifiers |= wxGetKeyState( WXK_ALT ) ? MD_ALT : 0;
+     */
 
     TOOL_EVENT evt( TC_MOUSE, TA_PRIME, BUT_LEFT | modifiers );
     evt.SetMousePosition( aPosition );
@@ -372,12 +430,6 @@ void TOOL_MANAGER::PostEvent( const TOOL_EVENT& aEvent )
 }
 
 
-const std::map<std::string, TOOL_ACTION*>& TOOL_MANAGER::GetActions() const
-{
-    return m_actionMgr->GetActions();
-}
-
-
 int TOOL_MANAGER::GetHotKey( const TOOL_ACTION& aAction ) const
 {
     return m_actionMgr->GetHotKey( aAction );
@@ -411,7 +463,7 @@ bool TOOL_MANAGER::runTool( TOOL_BASE* aTool )
 
     TOOL_ID id = aTool->GetId();
 
-    wxLogTrace( kicadTraceToolStack, "TOOL_MANAGER::runTool - running tool %s",
+    wxLogTrace( kicadTraceToolStack, wxS( "TOOL_MANAGER::runTool - running tool %s" ),
                 aTool->GetName() );
 
     if( aTool->GetType() == INTERACTIVE )
@@ -446,10 +498,22 @@ bool TOOL_MANAGER::runTool( TOOL_BASE* aTool )
 
 void TOOL_MANAGER::ShutdownAllTools()
 {
+    m_shuttingDown = true;
+
     // Create a temporary list of tools to iterate over since when the tools shutdown
     // they remove themselves from the list automatically (invalidating the iterator)
     ID_LIST tmpList = m_activeTools;
 
+    // Make sure each tool knows that it is shutting down, so that loops get shut down
+    // at the dispatcher
+    for( auto id : tmpList )
+    {
+        if( m_toolIdIndex.count( id ) == 0 )
+            continue;
+
+        m_toolIdIndex[id]->shutdown = true;
+    }
+
     for( auto id : tmpList )
     {
         ShutdownTool( id );
@@ -464,7 +528,7 @@ void TOOL_MANAGER::ShutdownTool( TOOL_ID aToolId )
     if( tool && tool->GetType() == INTERACTIVE )
         ShutdownTool( tool );
 
-    wxLogTrace( kicadTraceToolStack, "TOOL_MANAGER::ShutdownTool - no tool with ID %d",
+    wxLogTrace( kicadTraceToolStack, wxS( "TOOL_MANAGER::ShutdownTool - no tool with ID %d" ),
                 aToolId );
 }
 
@@ -476,7 +540,7 @@ void TOOL_MANAGER::ShutdownTool( const std::string& aToolName )
     if( tool && tool->GetType() == INTERACTIVE )
         ShutdownTool( tool );
 
-    wxLogTrace( kicadTraceToolStack, "TOOL_MANAGER::ShutdownTool - no tool with name %s",
+    wxLogTrace( kicadTraceToolStack, wxS( "TOOL_MANAGER::ShutdownTool - no tool with name %s" ),
                 aToolName );
 }
 
@@ -505,7 +569,7 @@ void TOOL_MANAGER::ShutdownTool( TOOL_BASE* aTool )
             if( st->cofunc )
             {
                 wxLogTrace( kicadTraceToolStack,
-                            "TOOL_MANAGER::ShutdownTool - Shutting down tool %s",
+                            wxS( "TOOL_MANAGER::ShutdownTool - Shutting down tool %s" ),
                             st->theTool->GetName() );
 
                 setActiveState( st );
@@ -551,11 +615,16 @@ void TOOL_MANAGER::DeactivateTool()
 
 void TOOL_MANAGER::ResetTools( TOOL_BASE::RESET_REASON aReason )
 {
-    DeactivateTool();
+    if( aReason != TOOL_BASE::REDRAW )
+        DeactivateTool();
 
     for( auto& state : m_toolState )
     {
         TOOL_BASE* tool = state.first;
+
+        wxLogTrace( kicadTraceToolStack, wxS( "TOOL_MANAGER::ResetTools: Resetting tool '%s'" ),
+                                             tool->GetName() );
+
         setActiveState( state.second );
         tool->Reset( aReason );
 
@@ -567,7 +636,7 @@ void TOOL_MANAGER::ResetTools( TOOL_BASE::RESET_REASON aReason )
 
 void TOOL_MANAGER::InitTools()
 {
-    for( TOOL_VEC::iterator it = m_toolOrder.begin(); it != m_toolOrder.end(); /* iter inside */ )
+    for( auto it = m_toolOrder.begin(); it != m_toolOrder.end(); /* iter inside */ )
     {
         TOOL_BASE* tool = *it;
         wxASSERT( m_toolState.count( tool ) );
@@ -577,8 +646,9 @@ void TOOL_MANAGER::InitTools()
 
         if( !tool->Init() )
         {
-            wxLogTrace( kicadTraceToolStack, "TOOL_MANAGER initialization of tool '%s' failed",
-                                             tool->GetName() );
+            wxLogTrace( kicadTraceToolStack,
+                        wxS( "TOOL_MANAGER initialization of tool '%s' failed" ),
+                        tool->GetName() );
 
             // Unregister the tool
             setActiveState( nullptr );
@@ -633,6 +703,7 @@ void TOOL_MANAGER::RunMainStack( TOOL_BASE* aTool, std::function<void()> aFunc )
 {
     TOOL_STATE* st = m_toolState[aTool];
     setActiveState( st );
+    wxCHECK( st->cofunc, /* void */ );
     st->cofunc->RunMainStack( std::move( aFunc ) );
 }
 
@@ -641,13 +712,15 @@ TOOL_EVENT* TOOL_MANAGER::ScheduleWait( TOOL_BASE* aTool, const TOOL_EVENT_LIST&
 {
     TOOL_STATE* st = m_toolState[aTool];
 
-    wxASSERT( !st->pendingWait ); // everything collapses on two KiYield() in a row
+    wxCHECK( !st->pendingWait, nullptr ); // everything collapses on two KiYield() in a row
 
     // indicate to the manager that we are going to sleep and we shall be
     // woken up when an event matching aConditions arrive
     st->pendingWait = true;
     st->waitEvents = aConditions;
 
+    wxCHECK( st->cofunc, nullptr );
+
     // switch context back to event dispatcher loop
     st->cofunc->KiYield();
 
@@ -663,7 +736,7 @@ bool TOOL_MANAGER::dispatchInternal( TOOL_EVENT& aEvent )
 {
     bool handled = false;
 
-    wxLogTrace( kicadTraceToolStack, "TOOL_MANAGER::dispatchInternal - received event: %s",
+    wxLogTrace( kicadTraceToolStack, wxS( "TOOL_MANAGER::dispatchInternal - received event: %s" ),
                 aEvent.Format() );
 
     auto it = m_activeTools.begin();
@@ -699,7 +772,7 @@ bool TOOL_MANAGER::dispatchInternal( TOOL_EVENT& aEvent )
             st->waitEvents.clear();
 
             wxLogTrace( kicadTraceToolStack,
-                        "TOOL_MANAGER::dispatchInternal - Waking tool %s for event: %s",
+                        wxS( "TOOL_MANAGER::dispatchInternal - Waking tool %s for event: %s" ),
                         st->theTool->GetName(), aEvent.Format() );
 
             setActiveState( st );
@@ -715,7 +788,8 @@ bool TOOL_MANAGER::dispatchInternal( TOOL_EVENT& aEvent )
             if( !st->wakeupEvent.PassEvent() )
             {
                 wxLogTrace( kicadTraceToolStack,
-                            "TOOL_MANAGER::dispatchInternal - tool %s stopped passing event: %s",
+                            wxS( "TOOL_MANAGER::dispatchInternal - tool %s stopped passing "
+                                 "event: %s" ),
                             st->theTool->GetName(), aEvent.Format() );
 
                 return true;
@@ -748,24 +822,23 @@ bool TOOL_MANAGER::dispatchInternal( TOOL_EVENT& aEvent )
                     // and transfer the previous view control settings to the new context
                     if( st->cofunc )
                     {
-                        auto vc = st->vcSettings;
+                        KIGFX::VC_SETTINGS viewControlSettings = st->vcSettings;
                         st->Push();
-                        st->vcSettings = vc;
+                        st->vcSettings = std::move( viewControlSettings );
                     }
 
                     st->cofunc = new COROUTINE<int, const TOOL_EVENT&>( std::move( func_copy ) );
 
-                    // as the state changes, the transition table has to be set up again
-                    st->transitions.clear();
-
                     wxLogTrace( kicadTraceToolStack,
-                                "TOOL_MANAGER::dispatchInternal - Running tool %s for event: %s",
+                                wxS( "TOOL_MANAGER::dispatchInternal - Running tool %s for "
+                                     "event: %s" ),
                                 st->theTool->GetName(), aEvent.Format() );
 
                     // got match? Run the handler.
                     setActiveState( st );
                     st->idle = false;
-                    st->cofunc->Call( aEvent );
+                    st->initialEvent = aEvent;
+                    st->cofunc->Call( st->initialEvent );
                     handled = true;
 
                     if( !st->cofunc->Running() )
@@ -784,8 +857,8 @@ bool TOOL_MANAGER::dispatchInternal( TOOL_EVENT& aEvent )
             break;      // only the first tool gets the event
     }
 
-    wxLogTrace( kicadTraceToolStack, "TOOL_MANAGER::dispatchInternal - %s handle event: %s",
-                ( handled ? "Did" : "Did not" ), aEvent.Format() );
+    wxLogTrace( kicadTraceToolStack, wxS( "TOOL_MANAGER::dispatchInternal - %s handle event: %s" ),
+                ( handled ? wxS( "Did" ) : wxS( "Did not" ) ), aEvent.Format() );
 
     return handled;
 }
@@ -802,19 +875,17 @@ bool TOOL_MANAGER::DispatchHotKey( const TOOL_EVENT& aEvent )
 
 bool TOOL_MANAGER::dispatchActivation( const TOOL_EVENT& aEvent )
 {
-    wxLogTrace( kicadTraceToolStack, "TOOL_MANAGER::dispatchActivation - Received event: %s",
+    wxLogTrace( kicadTraceToolStack, wxS( "TOOL_MANAGER::dispatchActivation - Received event: %s" ),
                 aEvent.Format() );
 
     if( aEvent.IsActivate() )
     {
-        wxString cmdStr( *aEvent.GetCommandStr() );
-
-        auto tool = m_toolNameIndex.find( *aEvent.GetCommandStr() );
+        auto tool = m_toolNameIndex.find( aEvent.getCommandStr() );
 
         if( tool != m_toolNameIndex.end() )
         {
             wxLogTrace( kicadTraceToolStack,
-                        "TOOL_MANAGER::dispatchActivation - Running tool %s for event: %s",
+                        wxS( "TOOL_MANAGER::dispatchActivation - Running tool %s for event: %s" ),
                         tool->second->theTool->GetName(), aEvent.Format() );
 
             runTool( tool->second->theTool );
@@ -825,6 +896,7 @@ bool TOOL_MANAGER::dispatchActivation( const TOOL_EVENT& aEvent )
     return false;
 }
 
+
 void TOOL_MANAGER::DispatchContextMenu( const TOOL_EVENT& aEvent )
 {
     for( TOOL_ID toolId : m_activeTools )
@@ -857,7 +929,7 @@ void TOOL_MANAGER::DispatchContextMenu( const TOOL_EVENT& aEvent )
             m_menuCursor = m_viewControls->GetCursorPosition();
 
         // Save all tools cursor settings, as they will be overridden
-        for( auto idState : m_toolIdIndex )
+        for( const std::pair<const TOOL_ID, TOOL_STATE*>& idState : m_toolIdIndex )
         {
             TOOL_STATE* s = idState.second;
             const auto& vc = s->vcSettings;
@@ -865,7 +937,7 @@ void TOOL_MANAGER::DispatchContextMenu( const TOOL_EVENT& aEvent )
             if( vc.m_forceCursorPosition )
                 m_cursorSettings[idState.first] = vc.m_forcedPosition;
             else
-                m_cursorSettings[idState.first] = NULLOPT;
+                m_cursorSettings[idState.first] = std::nullopt;
         }
 
         if( m_viewControls )
@@ -880,8 +952,14 @@ void TOOL_MANAGER::DispatchContextMenu( const TOOL_EVENT& aEvent )
         if( wxWindow* frame = dynamic_cast<wxWindow*>( m_frame ) )
             frame->PopupMenu( menu.get() );
 
-        // If a menu is canceled then notify tool
-        if( menu->GetSelected() < 0 )
+        // Warp the cursor if a menu item was selected
+        if( menu->GetSelected() >= 0 )
+        {
+            if( m_viewControls && m_warpMouseAfterContextMenu )
+                m_viewControls->WarpMouseCursor( m_menuCursor, true, false );
+        }
+        // Otherwise notify the tool of a canceled menu
+        else
         {
             TOOL_EVENT evt( TC_COMMAND, TA_CHOICE_MENU_CHOICE, -1 );
             evt.SetHasPosition( false );
@@ -902,7 +980,8 @@ void TOOL_MANAGER::DispatchContextMenu( const TOOL_EVENT& aEvent )
         m_menuOwner = -1;
 
         // Restore cursor settings
-        for( auto cursorSetting : m_cursorSettings )
+        for( const std::pair<const TOOL_ID,
+             std::optional<VECTOR2D>>& cursorSetting : m_cursorSettings )
         {
             auto it = m_toolIdIndex.find( cursorSetting.first );
             wxASSERT( it != m_toolIdIndex.end() );
@@ -937,18 +1016,17 @@ TOOL_MANAGER::ID_LIST::iterator TOOL_MANAGER::finishTool( TOOL_STATE* aState )
     if( aState == m_activeState )
         setActiveState( nullptr );
 
-    // Set transitions to be ready for future TOOL_EVENTs
-    TOOL_BASE* tool = aState->theTool;
-
-    if( tool->GetType() == INTERACTIVE )
-        static_cast<TOOL_INTERACTIVE*>( tool )->resetTransitions();
-
     return it;
 }
 
 
 bool TOOL_MANAGER::ProcessEvent( const TOOL_EVENT& aEvent )
 {
+    // Once the tool manager is shutting down, don't start
+    // activating more tools
+    if( m_shuttingDown )
+        return true;
+
     bool handled = processEvent( aEvent );
 
     TOOL_STATE* activeTool = GetCurrentToolState();
@@ -958,9 +1036,6 @@ bool TOOL_MANAGER::ProcessEvent( const TOOL_EVENT& aEvent )
 
     if( m_view && m_view->IsDirty() )
     {
-        if( GetToolHolder() )
-            GetToolHolder()->RefreshCanvas();
-
 #if defined( __WXMAC__ )
         wxTheApp->ProcessPendingEvents(); // required for updating brightening behind a popup menu
 #endif
@@ -982,52 +1057,6 @@ void TOOL_MANAGER::ScheduleContextMenu( TOOL_BASE* aTool, ACTION_MENU* aMenu,
 }
 
 
-bool TOOL_MANAGER::SaveClipboard( const std::string& aTextUTF8 )
-{
-    wxLogNull doNotLog; // disable logging of failed clipboard actions
-
-    if( wxTheClipboard->Open() )
-    {
-        // Store the UTF8 string as Unicode string in clipboard:
-        wxTheClipboard->SetData( new wxTextDataObject( wxString( aTextUTF8.c_str(),
-                                                                 wxConvUTF8 ) ) );
-
-        wxTheClipboard->Flush(); // Allow data to be available after closing KiCad
-        wxTheClipboard->Close();
-
-        return true;
-    }
-
-    return false;
-}
-
-
-std::string TOOL_MANAGER::GetClipboardUTF8() const
-{
-    std::string result;
-
-    wxLogNull doNotLog; // disable logging of failed clipboard actions
-
-    if( wxTheClipboard->Open() )
-    {
-        if( wxTheClipboard->IsSupported( wxDF_TEXT )
-                || wxTheClipboard->IsSupported( wxDF_UNICODETEXT ) )
-        {
-            wxTextDataObject data;
-            wxTheClipboard->GetData( data );
-
-            // The clipboard is expected containing a Unicode string, so return it
-            // as UTF8 string
-            result = data.GetText().utf8_str();
-        }
-
-        wxTheClipboard->Close();
-    }
-
-    return result;
-}
-
-
 const KIGFX::VC_SETTINGS& TOOL_MANAGER::GetCurrentToolVC() const
 {
     if( TOOL_STATE* active = GetCurrentToolState() )
@@ -1084,13 +1113,13 @@ void TOOL_MANAGER::saveViewControls( TOOL_STATE* aState )
             if( !curr.m_forceCursorPosition || curr.m_forcedPosition != m_menuCursor )
             {
                 if( !curr.m_forceCursorPosition )
-                    it->second = NULLOPT;
+                    it->second = std::nullopt;
                 else
                     it->second = curr.m_forcedPosition;
             }
             else
             {
-                OPT<VECTOR2D> cursor = it->second;
+                std::optional<VECTOR2D> cursor = it->second;
 
                 if( cursor )
                 {
@@ -1115,7 +1144,7 @@ void TOOL_MANAGER::applyViewControls( const TOOL_STATE* aState )
 
 bool TOOL_MANAGER::processEvent( const TOOL_EVENT& aEvent )
 {
-    wxLogTrace( kicadTraceToolStack, "TOOL_MANAGER::processEvent - %s", aEvent.Format() );
+    wxLogTrace( kicadTraceToolStack, wxS( "TOOL_MANAGER::processEvent - %s" ), aEvent.Format() );
 
     // First try to dispatch the action associated with the event if it is a key press event
     bool handled = DispatchHotKey( aEvent );
@@ -1128,8 +1157,8 @@ bool TOOL_MANAGER::processEvent( const TOOL_EVENT& aEvent )
         if( GetToolHolder() && !GetToolHolder()->GetDoImmediateActions() )
         {
             // An tool-selection-event has no position
-            if( mod_event.GetCommandStr().is_initialized()
-                    && mod_event.GetCommandStr().get() != GetToolHolder()->CurrentToolName()
+            if( !mod_event.getCommandStr().empty()
+                    && mod_event.getCommandStr() != GetToolHolder()->CurrentToolName()
                     && !mod_event.ForceImmediate() )
             {
                 mod_event.SetHasPosition( false );
@@ -1153,7 +1182,7 @@ bool TOOL_MANAGER::processEvent( const TOOL_EVENT& aEvent )
         }
     }
 
-    wxLogTrace( kicadTraceToolStack, "TOOL_MANAGER::processEvent - %s handle event: %s",
+    wxLogTrace( kicadTraceToolStack, wxS( "TOOL_MANAGER::processEvent - %s handle event: %s" ),
                                      ( handled ? "Did" : "Did not" ), aEvent.Format() );
 
     return handled;
@@ -1175,6 +1204,10 @@ void TOOL_MANAGER::setActiveState( TOOL_STATE* aState )
 bool TOOL_MANAGER::IsToolActive( TOOL_ID aId ) const
 {
     auto it = m_toolIdIndex.find( aId );
+
+    if( it == m_toolIdIndex.end() )
+        return false;
+
     return !it->second->idle;
 }
 
