@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2014 Henner Zeller <h.zeller@acm.org>
- * Copyright (C) 2014-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,18 +23,25 @@
  */
 
 #include <widgets/lib_tree.h>
+#include <widgets/std_bitmap_button.h>
 #include <core/kicad_algo.h>
 #include <macros.h>
-#include <wxdataviewctrl_helpers.h>
-#include <wx/sizer.h>
+#include <bitmaps.h>
 #include <dialogs/eda_reorderable_list_dialog.h>
 #include <tool/tool_interactive.h>
 #include <tool/tool_manager.h>
 #include <tool/action_manager.h>
 #include <tool/actions.h>
-#include <wx/srchctrl.h>
+#include <tool/tool_dispatcher.h>
+#include <widgets/wx_dataviewctrl.h>
 #include <wx/settings.h>
-#include <wx/timer.h>
+#include <wx/sizer.h>
+#include <wx/srchctrl.h>
+#include <wx/popupwin.h>
+
+#include <eda_doc.h>                    // for GetAssociatedDocument()
+#include <pgm_base.h>                   // for PROJECT
+#include <settings/settings_manager.h>  // for PROJECT
 
 constexpr int RECENT_SEARCHES_MAX = 10;
 
@@ -46,14 +53,22 @@ LIB_TREE::LIB_TREE( wxWindow* aParent, const wxString& aRecentSearchesKey, LIB_T
                     HTML_WINDOW* aDetails ) :
         wxPanel( aParent, wxID_ANY, wxDefaultPosition, wxDefaultSize,
                  wxWANTS_CHARS | wxTAB_TRAVERSAL | wxNO_BORDER ),
-        m_lib_table( aLibTable ), m_adapter( aAdapter ), m_query_ctrl( nullptr ),
+        m_adapter( aAdapter ),
+        m_query_ctrl( nullptr ),
+        m_sort_ctrl( nullptr ),
         m_details_ctrl( nullptr ),
         m_inTimerEvent( false ),
         m_recentSearchesKey( aRecentSearchesKey ),
-        m_skipNextRightClick( false )
+        m_filtersSizer( nullptr ),
+        m_skipNextRightClick( false ),
+        m_previewWindow( nullptr ),
+        m_previewDisabled( false )
 {
     wxBoxSizer* sizer = new wxBoxSizer( wxVERTICAL );
 
+    m_hoverTimer.SetOwner( this );
+    Bind( wxEVT_TIMER, &LIB_TREE::onHoverTimer, this, m_hoverTimer.GetId() );
+
     // Search text control
     if( aFlags & SEARCH )
     {
@@ -71,24 +86,70 @@ LIB_TREE::LIB_TREE( wxWindow* aParent, const wxString& aRecentSearchesKey, LIB_T
 
         m_debounceTimer = new wxTimer( this );
 
-        search_sizer->Add( m_query_ctrl, 1, wxEXPAND, 5 );
+        search_sizer->Add( m_query_ctrl, 1, wxEXPAND | wxRIGHT, 5 );
 
-        sizer->Add( search_sizer, 0, wxEXPAND, 5 );
+        m_sort_ctrl = new STD_BITMAP_BUTTON( this, wxID_ANY, wxNullBitmap, wxDefaultPosition,
+                                             wxDefaultSize, wxBU_AUTODRAW|0 );
+        m_sort_ctrl->SetBitmap( KiBitmapBundle( BITMAPS::small_sort_desc ) );
+        m_sort_ctrl->Bind( wxEVT_LEFT_DOWN,
+                [&]( wxMouseEvent& aEvent )
+                {
+                    // Build a pop menu:
+                    wxMenu menu;
+
+                    menu.Append( 4201, _( "Sort by Best Match" ), wxEmptyString, wxITEM_CHECK );
+                    menu.Append( 4202, _( "Sort Alphabetically" ), wxEmptyString, wxITEM_CHECK );
+                    menu.AppendSeparator();
+                    menu.Append( 4203, ACTIONS::expandAll.GetMenuItem() );
+                    menu.Append( 4204, ACTIONS::collapseAll.GetMenuItem() );
+
+                    if( m_adapter->GetSortMode() == LIB_TREE_MODEL_ADAPTER::BEST_MATCH )
+                        menu.Check( 4201, true );
+                    else
+                        menu.Check( 4202, true );
+
+                    // menu_id is the selected submenu id from the popup menu or wxID_NONE
+                    int menu_id = m_sort_ctrl->GetPopupMenuSelectionFromUser( menu );
+
+                    if( menu_id == 0 || menu_id == 4201 )
+                    {
+                        m_adapter->SetSortMode( LIB_TREE_MODEL_ADAPTER::BEST_MATCH );
+                        Regenerate( true );
+                    }
+                    else if( menu_id == 1 || menu_id == 4202 )
+                    {
+                        m_adapter->SetSortMode( LIB_TREE_MODEL_ADAPTER::ALPHABETIC );
+                        Regenerate( true );
+                    }
+                    else if( menu_id == 3 || menu_id == 4203 )
+                    {
+                        ExpandAll();
+                    }
+                    else if( menu_id == 4 || menu_id == 4204 )
+                    {
+                        CollapseAll();
+                    }
+                } );
+
+        m_sort_ctrl->Bind( wxEVT_CHAR_HOOK, &LIB_TREE::onTreeCharHook, this );
+
+        search_sizer->Add( m_sort_ctrl, 0, wxEXPAND, 5 );
+
+        sizer->Add( search_sizer, 0, wxEXPAND | wxBOTTOM, 5 );
 
         m_query_ctrl->Bind( wxEVT_TEXT, &LIB_TREE::onQueryText, this );
 
-#if wxCHECK_VERSION( 3, 1, 1 )
         m_query_ctrl->Bind( wxEVT_SEARCH_CANCEL, &LIB_TREE::onQueryText, this );
-#else
-        m_query_ctrl->Bind( wxEVT_SEARCHCTRL_CANCEL_BTN, &LIB_TREE::onQueryText, this );
-#endif
         m_query_ctrl->Bind( wxEVT_CHAR_HOOK, &LIB_TREE::onQueryCharHook, this );
         m_query_ctrl->Bind( wxEVT_MOTION, &LIB_TREE::onQueryMouseMoved, this );
+
+#if defined( __WXOSX__ ) || wxCHECK_VERSION( 3, 3, 0 ) // Doesn't work properly on other ports
         m_query_ctrl->Bind( wxEVT_LEAVE_WINDOW,
-                            [this] ( wxMouseEvent& aEvt )
+                            [this]( wxMouseEvent& aEvt )
                             {
                                 SetCursor( wxCURSOR_ARROW );
                             } );
+#endif
 
         m_query_ctrl->Bind( wxEVT_MENU,
                 [this]( wxCommandEvent& aEvent )
@@ -104,15 +165,24 @@ LIB_TREE::LIB_TREE( wxWindow* aParent, const wxString& aRecentSearchesKey, LIB_T
         Bind( wxEVT_TIMER, &LIB_TREE::onDebounceTimer, this, m_debounceTimer->GetId() );
     }
 
+    if( aFlags & FILTERS )
+    {
+        m_filtersSizer = new wxBoxSizer( wxVERTICAL );
+        sizer->Add( m_filtersSizer, 0, wxEXPAND | wxLEFT, 4 );
+    }
+
     // Tree control
     int dvFlags = ( aFlags & MULTISELECT ) ? wxDV_MULTIPLE : wxDV_SINGLE;
-    m_tree_ctrl = new wxDataViewCtrl( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, dvFlags );
+    m_tree_ctrl = new WX_DATAVIEWCTRL( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, dvFlags );
     m_adapter->AttachTo( m_tree_ctrl );
 
-    if( aFlags & DETAILS )
-        sizer->AddSpacer( 5 );
+#ifdef __WXGTK__
+    // The GTK renderer seems to calculate row height incorrectly sometimes; but can be overridden
+    int rowHeight = FromDIP( 6 ) + GetTextExtent( wxS( "pdI" ) ).y;
+    m_tree_ctrl->SetRowHeight( rowHeight );
+#endif
 
-    sizer->Add( m_tree_ctrl, 5, wxRIGHT | wxBOTTOM | wxEXPAND, 1 );
+    sizer->Add( m_tree_ctrl, 5, wxEXPAND, 5 );
 
     // Description panel
     if( aFlags & DETAILS )
@@ -122,8 +192,7 @@ LIB_TREE::LIB_TREE( wxWindow* aParent, const wxString& aRecentSearchesKey, LIB_T
             wxPoint html_size = ConvertDialogToPixels( wxPoint( 80, 80 ) );
 
             m_details_ctrl = new HTML_WINDOW( this, wxID_ANY, wxDefaultPosition,
-                                              wxSize( html_size.x, html_size.y ),
-                                              wxHW_SCROLLBAR_AUTO );
+                                              wxSize( html_size.x, html_size.y ) );
 
             sizer->Add( m_details_ctrl, 2, wxTOP | wxEXPAND, 5 );
         }
@@ -137,17 +206,20 @@ LIB_TREE::LIB_TREE( wxWindow* aParent, const wxString& aRecentSearchesKey, LIB_T
 
     SetSizer( sizer );
 
-    m_tree_ctrl->Bind( wxEVT_SIZE, &LIB_TREE::onSize, this );
     m_tree_ctrl->Bind( wxEVT_DATAVIEW_ITEM_ACTIVATED, &LIB_TREE::onTreeActivate, this );
     m_tree_ctrl->Bind( wxEVT_DATAVIEW_SELECTION_CHANGED, &LIB_TREE::onTreeSelect, this );
     m_tree_ctrl->Bind( wxEVT_DATAVIEW_ITEM_CONTEXT_MENU, &LIB_TREE::onItemContextMenu, this );
     m_tree_ctrl->Bind( wxEVT_DATAVIEW_COLUMN_HEADER_RIGHT_CLICK, &LIB_TREE::onHeaderContextMenu,
                        this );
 
+    // wxDataViewCtrl eats its mouseMoved events, so we're forced to use idle events to track
+    // the hover state
+    Bind( wxEVT_IDLE, &LIB_TREE::onIdle, this );
+
     // Process hotkeys when the tree control has focus:
     m_tree_ctrl->Bind( wxEVT_CHAR_HOOK, &LIB_TREE::onTreeCharHook, this );
 
-    Bind( SYMBOL_PRESELECTED, &LIB_TREE::onPreselect, this );
+    Bind( EVT_LIBITEM_SELECTED, &LIB_TREE::onPreselect, this );
 
     if( m_query_ctrl )
     {
@@ -180,8 +252,39 @@ LIB_TREE::LIB_TREE( wxWindow* aParent, const wxString& aRecentSearchesKey, LIB_T
 
 LIB_TREE::~LIB_TREE()
 {
-    // Stop the timer during destruction early to avoid potential race conditions (that do happen)
-    m_debounceTimer->Stop();
+    Unbind( wxEVT_TIMER, &LIB_TREE::onHoverTimer, this, m_hoverTimer.GetId() );
+
+    m_tree_ctrl->Unbind( wxEVT_DATAVIEW_ITEM_ACTIVATED, &LIB_TREE::onTreeActivate, this );
+    m_tree_ctrl->Unbind( wxEVT_DATAVIEW_SELECTION_CHANGED, &LIB_TREE::onTreeSelect, this );
+    m_tree_ctrl->Unbind( wxEVT_DATAVIEW_ITEM_CONTEXT_MENU, &LIB_TREE::onItemContextMenu, this );
+    m_tree_ctrl->Unbind( wxEVT_DATAVIEW_COLUMN_HEADER_RIGHT_CLICK, &LIB_TREE::onHeaderContextMenu,
+                         this );
+
+    Unbind( wxEVT_IDLE, &LIB_TREE::onIdle, this );
+    m_tree_ctrl->Unbind( wxEVT_CHAR_HOOK, &LIB_TREE::onTreeCharHook, this );
+    Unbind( EVT_LIBITEM_SELECTED, &LIB_TREE::onPreselect, this );
+
+    if( m_query_ctrl )
+    {
+        m_query_ctrl->Unbind( wxEVT_TEXT, &LIB_TREE::onQueryText, this );
+
+        m_query_ctrl->Unbind( wxEVT_SEARCH_CANCEL, &LIB_TREE::onQueryText, this );
+        m_query_ctrl->Unbind( wxEVT_CHAR_HOOK, &LIB_TREE::onQueryCharHook, this );
+        m_query_ctrl->Unbind( wxEVT_MOTION, &LIB_TREE::onQueryMouseMoved, this );
+    }
+
+    // Stop the timer during destruction early to avoid potential race conditions (that do happen)]
+    if( m_debounceTimer )
+    {
+        m_debounceTimer->Stop();
+        Unbind( wxEVT_TIMER, &LIB_TREE::onDebounceTimer, this, m_debounceTimer->GetId() );
+    }
+
+    if( m_details_ctrl )
+        m_details_ctrl->Unbind( wxEVT_HTML_LINK_CLICKED, &LIB_TREE::onDetailsLink, this );
+
+    m_hoverTimer.Stop();
+    destroyPreview();
 }
 
 
@@ -251,7 +354,9 @@ void LIB_TREE::CenterLibId( const LIB_ID& aLibId )
 
 void LIB_TREE::Unselect()
 {
+    m_tree_ctrl->Freeze();
     m_tree_ctrl->UnselectAll();
+    m_tree_ctrl->Thaw();
 }
 
 
@@ -261,6 +366,18 @@ void LIB_TREE::ExpandLibId( const LIB_ID& aLibId )
 }
 
 
+void LIB_TREE::ExpandAll()
+{
+    m_tree_ctrl->ExpandAll();
+}
+
+
+void LIB_TREE::CollapseAll()
+{
+    m_tree_ctrl->CollapseAll();
+}
+
+
 void LIB_TREE::SetSearchString( const wxString& aSearchString )
 {
     m_query_ctrl->ChangeValue( aSearchString );
@@ -338,9 +455,7 @@ wxWindow* LIB_TREE::GetFocusTarget()
 void LIB_TREE::FocusSearchFieldIfExists()
 {
     if( m_query_ctrl )
-    {
         m_query_ctrl->SetFocus();
-    }
 }
 
 
@@ -425,14 +540,14 @@ void LIB_TREE::expandIfValid( const wxDataViewItem& aTreeId )
 
 void LIB_TREE::postPreselectEvent()
 {
-    wxCommandEvent event( SYMBOL_PRESELECTED );
+    wxCommandEvent event( EVT_LIBITEM_SELECTED );
     wxPostEvent( this, event );
 }
 
 
 void LIB_TREE::postSelectEvent()
 {
-    wxCommandEvent event( SYMBOL_SELECTED );
+    wxCommandEvent event( EVT_LIBITEM_CHOSEN );
     wxPostEvent( this, event );
 }
 
@@ -491,25 +606,54 @@ void LIB_TREE::onDebounceTimer( wxTimerEvent& aEvent )
 
 void LIB_TREE::onQueryCharHook( wxKeyEvent& aKeyStroke )
 {
-    const wxDataViewItem sel = m_tree_ctrl->GetSelection();
-    LIB_TREE_NODE::TYPE  type = sel.IsOk() ? m_adapter->GetTypeFor( sel ) : LIB_TREE_NODE::INVALID;
+    int hotkey = aKeyStroke.GetKeyCode();
+
+    if( aKeyStroke.GetModifiers() & wxMOD_CONTROL )
+        hotkey += MD_CTRL;
+
+    if( aKeyStroke.GetModifiers() & wxMOD_ALT )
+        hotkey += MD_ALT;
+
+    if( aKeyStroke.GetModifiers() & wxMOD_SHIFT )
+        hotkey += MD_SHIFT;
+
+    if( hotkey == ACTIONS::expandAll.GetHotKey()
+        || hotkey == ACTIONS::expandAll.GetHotKeyAlt() )
+    {
+        m_tree_ctrl->ExpandAll();
+        return;
+    }
+    else if( hotkey == ACTIONS::collapseAll.GetHotKey()
+             || hotkey == ACTIONS::collapseAll.GetHotKeyAlt() )
+    {
+        m_tree_ctrl->CollapseAll();
+        return;
+    }
+
+    wxDataViewItem sel = m_tree_ctrl->GetSelection();
+
+    if( !sel.IsOk() )
+        sel = m_adapter->GetCurrentDataViewItem();
+
+    LIB_TREE_NODE::TYPE type = sel.IsOk() ? m_adapter->GetTypeFor( sel )
+                                          : LIB_TREE_NODE::TYPE::INVALID;
 
     switch( aKeyStroke.GetKeyCode() )
     {
     case WXK_UP:
         updateRecentSearchMenu();
-        selectIfValid( GetPrevItem( *m_tree_ctrl, sel ) );
+        selectIfValid( m_tree_ctrl->GetPrevItem( sel ) );
         break;
 
     case WXK_DOWN:
         updateRecentSearchMenu();
-        selectIfValid( GetNextItem( *m_tree_ctrl, sel ) );
+        selectIfValid( m_tree_ctrl->GetNextItem( sel ) );
         break;
 
     case WXK_ADD:
         updateRecentSearchMenu();
 
-        if( type == LIB_TREE_NODE::LIB )
+        if( type == LIB_TREE_NODE::TYPE::LIBRARY )
             m_tree_ctrl->Expand( sel );
 
         break;
@@ -517,17 +661,18 @@ void LIB_TREE::onQueryCharHook( wxKeyEvent& aKeyStroke )
     case WXK_SUBTRACT:
         updateRecentSearchMenu();
 
-        if( type == LIB_TREE_NODE::LIB )
+        if( type == LIB_TREE_NODE::TYPE::LIBRARY )
             m_tree_ctrl->Collapse( sel );
 
         break;
 
     case WXK_RETURN:
+    case WXK_NUMPAD_ENTER:
         updateRecentSearchMenu();
 
         if( GetSelectedLibId().IsValid() )
             postSelectEvent();
-        else if( type == LIB_TREE_NODE::LIB )
+        else if( type == LIB_TREE_NODE::TYPE::LIBRARY )
             toggleExpand( sel );
 
         break;
@@ -541,6 +686,7 @@ void LIB_TREE::onQueryCharHook( wxKeyEvent& aKeyStroke )
 
 void LIB_TREE::onQueryMouseMoved( wxMouseEvent& aEvent )
 {
+#if defined( __WXOSX__ ) || wxCHECK_VERSION( 3, 3, 0 ) // Doesn't work properly on other ports
     wxPoint pos = aEvent.GetPosition();
     wxRect  ctrlRect = m_query_ctrl->GetScreenRect();
     int     buttonWidth = ctrlRect.GetHeight();         // Presume buttons are square
@@ -551,6 +697,136 @@ void LIB_TREE::onQueryMouseMoved( wxMouseEvent& aEvent )
         SetCursor( wxCURSOR_ARROW );
     else
         SetCursor( wxCURSOR_IBEAM );
+#endif
+}
+
+
+#define PREVIEW_SIZE wxSize( 240, 200 )
+#define HOVER_TIMER_MILLIS 400
+
+
+void LIB_TREE::showPreview( wxDataViewItem aItem )
+{
+    if( aItem.IsOk() && m_adapter->HasPreview( aItem ) )
+    {
+        m_previewItem = aItem;
+        m_previewItemRect = m_tree_ctrl->GetItemRect( m_previewItem );
+
+        wxWindow* topLevelParent = wxGetTopLevelParent( m_parent );
+
+        if( !m_previewWindow )
+            m_previewWindow = new wxPopupWindow( topLevelParent );
+
+        m_previewWindow->SetSize( PREVIEW_SIZE );
+
+        m_adapter->ShowPreview( m_previewWindow, aItem );
+
+        m_previewWindow->SetPosition( wxPoint( m_tree_ctrl->GetScreenRect().GetRight() - 10,
+                                               wxGetMousePosition().y - PREVIEW_SIZE.y / 2 ) );
+
+        m_previewWindow->Show();
+    }
+}
+
+
+void LIB_TREE::hidePreview()
+{
+    m_previewItem = wxDataViewItem();
+
+    if( m_previewWindow )
+        m_previewWindow->Hide();
+}
+
+
+void LIB_TREE::destroyPreview()
+{
+    hidePreview();
+
+    if( m_previewWindow )
+    {
+        m_previewWindow->Destroy();
+        m_previewWindow = nullptr;
+    }
+}
+
+
+void LIB_TREE::onIdle( wxIdleEvent& aEvent )
+{
+    // The wxDataViewCtrl won't give us its mouseMoved events so we're forced to use idle
+    // events to track the hover state
+
+    // The dang thing won't give us scroll events either, so we implement a poor-man's
+    // scroll-checker using the last-known positions of the preview or hover items.
+
+    wxWindow* topLevelParent = wxGetTopLevelParent( m_parent );
+    wxWindow* topLevelFocus = wxGetTopLevelParent( wxWindow::FindFocus() );
+
+    bool    mouseOverWindow = false;
+    wxPoint screenPos = wxGetMousePosition();
+
+    if( m_tree_ctrl->IsShownOnScreen() )
+        mouseOverWindow |= m_tree_ctrl->GetScreenRect().Contains( screenPos );
+
+    if( m_previewDisabled || topLevelFocus != topLevelParent || !mouseOverWindow )
+    {
+        m_hoverTimer.Stop();
+        hidePreview();
+        return;
+    }
+
+    wxPoint           clientPos = m_tree_ctrl->ScreenToClient( screenPos );
+    wxDataViewItem    item;
+    wxDataViewColumn* col = nullptr;
+
+    m_tree_ctrl->HitTest( clientPos, item, col );
+
+    if( m_previewItem.IsOk() )
+    {
+        if( item != m_previewItem )
+        {
+#ifdef __WXGTK__
+            // Hide the preview, because Wayland can't move windows.
+            if( wxGetDisplayInfo().type == wxDisplayType::wxDisplayWayland )
+                hidePreview();
+#endif
+            showPreview( item );
+        }
+
+        return;
+    }
+
+    if( m_hoverPos != clientPos )
+    {
+        m_hoverPos = clientPos;
+        m_hoverItem = item;
+        m_hoverItemRect = m_tree_ctrl->GetItemRect( m_hoverItem );
+        m_hoverTimer.StartOnce( HOVER_TIMER_MILLIS );
+    }
+}
+
+
+void LIB_TREE::onHoverTimer( wxTimerEvent& aEvent )
+{
+    hidePreview();
+
+    if( !m_tree_ctrl->IsShownOnScreen() || m_previewDisabled )
+        return;
+
+    wxDataViewItem    item;
+    wxDataViewColumn* col = nullptr;
+    m_tree_ctrl->HitTest( m_hoverPos, item, col );
+
+    if( item == m_hoverItem && m_tree_ctrl->GetItemRect( item ) == m_hoverItemRect )
+    {
+        if( item != m_tree_ctrl->GetSelection() )
+            showPreview( item );
+    }
+    else // view must have been scrolled
+    {
+        m_hoverItem = item;
+        m_hoverItemRect = m_tree_ctrl->GetItemRect( m_hoverItem );
+        m_hoverTimer.StartOnce( HOVER_TIMER_MILLIS );
+    }
 }
 
 
@@ -582,16 +858,23 @@ void LIB_TREE::onTreeCharHook( wxKeyEvent& aKeyStroke )
 
 void LIB_TREE::onTreeSelect( wxDataViewEvent& aEvent )
 {
+    if( m_tree_ctrl->IsFrozen() )
+        return;
+
     if( !m_inTimerEvent )
         updateRecentSearchMenu();
 
-    if( !m_tree_ctrl->IsFrozen() )
-        postPreselectEvent();
+    postPreselectEvent();
 }
 
 
 void LIB_TREE::onTreeActivate( wxDataViewEvent& aEvent )
 {
+    hidePreview();
+
+    if( !m_inTimerEvent )
+        updateRecentSearchMenu();
+
     if( !GetSelectedLibId().IsValid() )
         toggleExpand( m_tree_ctrl->GetSelection() );    // Expand library/part units subtree
     else
@@ -599,21 +882,20 @@ void LIB_TREE::onTreeActivate( wxDataViewEvent& aEvent )
 }
 
 
-void LIB_TREE::onSize( wxSizeEvent& aEvent )
-{
-    m_adapter->OnSize( aEvent );
-}
-
-
 void LIB_TREE::onDetailsLink( wxHtmlLinkEvent& aEvent )
 {
     const wxHtmlLinkInfo& info = aEvent.GetLinkInfo();
-    ::wxLaunchDefaultBrowser( info.GetHref() );
+    wxString docname = info.GetHref();
+    PROJECT& prj = Pgm().GetSettingsManager().Prj();
+
+    GetAssociatedDocument( this, docname, &prj );
 }
 
 
 void LIB_TREE::onPreselect( wxCommandEvent& aEvent )
 {
+    hidePreview();
+
     if( m_details_ctrl )
     {
         int unit = 0;
@@ -631,28 +913,22 @@ void LIB_TREE::onPreselect( wxCommandEvent& aEvent )
 
 void LIB_TREE::onItemContextMenu( wxDataViewEvent& aEvent )
 {
+    hidePreview();
+
     if( m_skipNextRightClick )
     {
         m_skipNextRightClick = false;
         return;
     }
 
+    m_previewDisabled = true;
+
     if( TOOL_INTERACTIVE* tool = m_adapter->GetContextMenuTool() )
     {
         if( !GetCurrentTreeNode() )
         {
             wxPoint pos = m_tree_ctrl->ScreenToClient( wxGetMousePosition() );
 
-            // What we actually want is the height of the column header, but wxWidgets gives us
-            // no way to get that, so we use the height of the search ctrl as a proxy.  And it's
-            // not even a very good proxy on Mac....
-            int headerHeight = m_tree_ctrl->GetPosition().y;
-#ifdef __WXMAC__
-            headerHeight += 5;
-#endif
-
-            pos.y -= headerHeight;
-
             wxDataViewItem    item;
             wxDataViewColumn* col;
             m_tree_ctrl->HitTest( pos, item, col );
@@ -676,7 +952,7 @@ void LIB_TREE::onItemContextMenu( wxDataViewEvent& aEvent )
     {
         LIB_TREE_NODE* current = GetCurrentTreeNode();
 
-        if( current && current->m_Type == LIB_TREE_NODE::LIB )
+        if( current && current->m_Type == LIB_TREE_NODE::TYPE::LIBRARY )
         {
             ACTION_MENU menu( true, nullptr );
 
@@ -696,14 +972,19 @@ void LIB_TREE::onItemContextMenu( wxDataViewEvent& aEvent )
             }
         }
     }
+
+    m_previewDisabled = false;
 }
 
 
 void LIB_TREE::onHeaderContextMenu( wxDataViewEvent& aEvent )
 {
+    hidePreview();
+    m_previewDisabled = true;
+
     ACTION_MENU menu( true, nullptr );
 
-    menu.Add( ACTIONS::selectColumns );
+    menu.Add( ACTIONS::selectLibTreeColumns );
 
     if( GetPopupMenuSelectionFromUser( menu ) != wxID_NONE )
     {
@@ -715,12 +996,9 @@ void LIB_TREE::onHeaderContextMenu( wxDataViewEvent& aEvent )
             m_adapter->SetShownColumns( dlg.EnabledList() );
     }
 
-#if !wxCHECK_VERSION( 3, 1, 0 )
-    // wxGTK 3.0 sends item right click events for header right clicks
-    m_skipNextRightClick = true;
-#endif
+    m_previewDisabled = false;
 }
 
 
-wxDEFINE_EVENT( SYMBOL_PRESELECTED, wxCommandEvent );
-wxDEFINE_EVENT( SYMBOL_SELECTED, wxCommandEvent );
+wxDEFINE_EVENT( EVT_LIBITEM_SELECTED, wxCommandEvent );
+wxDEFINE_EVENT( EVT_LIBITEM_CHOSEN, wxCommandEvent );
