@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2004-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,7 +27,8 @@
 #include <reporter.h>
 #include <sch_edit_frame.h>
 #include <schematic.h>
-#include <erc_settings.h>
+#include <sch_commit.h>
+#include <erc/erc_settings.h>
 #include <sch_reference_list.h>
 #include <symbol_library.h>
 #include <tools/ee_selection.h>
@@ -38,8 +39,7 @@
 void SCH_EDIT_FRAME::mapExistingAnnotation( std::map<wxString, wxString>& aMap )
 {
     SCH_REFERENCE_LIST references;
-
-    Schematic().GetSheets().GetSymbols( references );
+    Schematic().Hierarchy().GetSymbols( references );
 
     for( size_t i = 0; i < references.GetCount(); i++ )
     {
@@ -58,16 +58,40 @@ void SCH_EDIT_FRAME::mapExistingAnnotation( std::map<wxString, wxString>& aMap )
 
 
 void SCH_EDIT_FRAME::DeleteAnnotation( ANNOTATE_SCOPE_T aAnnotateScope, bool aRecursive,
-                                       bool* aAppendUndo )
+                                       REPORTER& aReporter )
 {
+
+    SCH_SHEET_LIST sheets = Schematic().Hierarchy();
+    SCH_SCREEN*    screen = GetScreen();
+    SCH_SHEET_PATH currentSheet = GetCurrentSheet();
+    SCH_COMMIT     commit( this );
+
     auto clearSymbolAnnotation =
         [&]( EDA_ITEM* aItem, SCH_SCREEN* aScreen, SCH_SHEET_PATH* aSheet, bool aResetPrefixes )
         {
             SCH_SYMBOL* symbol = static_cast<SCH_SYMBOL*>( aItem );
+            commit.Modify( aItem, aScreen );
 
-            SaveCopyInUndoList( aScreen, symbol, UNDO_REDO::CHANGED, *aAppendUndo );
-            *aAppendUndo = true;
-            symbol->ClearAnnotation( aSheet, aResetPrefixes );
+            // aSheet == nullptr means all sheets
+            if( !aSheet || symbol->IsAnnotated( aSheet ) )
+            {
+                wxString msg;
+
+                if( symbol->GetUnitCount() > 1 )
+                {
+                    msg.Printf( _( "Cleared annotation for %s (unit %s)." ),
+                                symbol->GetValue( true, aSheet, false ),
+                                symbol->SubReference( symbol->GetUnit(), false ) );
+                }
+                else
+                {
+                    msg.Printf( _( "Cleared annotation for %s." ),
+                                symbol->GetValue( true, aSheet, false ) );
+                }
+
+                symbol->ClearAnnotation( aSheet, aResetPrefixes );
+                aReporter.Report( msg, RPT_SEVERITY_ACTION );
+            }
         };
 
     auto clearSheetAnnotation =
@@ -77,10 +101,6 @@ void SCH_EDIT_FRAME::DeleteAnnotation( ANNOTATE_SCOPE_T aAnnotateScope, bool aRe
                     clearSymbolAnnotation( item, aScreen, aSheet, aResetPrefixes );
             };
 
-    SCH_SHEET_LIST sheets = Schematic().GetSheets();
-    SCH_SCREEN*    screen = GetScreen();
-    SCH_SHEET_PATH currentSheet = GetCurrentSheet();
-
     switch( aAnnotateScope )
     {
     case ANNOTATE_ALL:
@@ -151,9 +171,7 @@ void SCH_EDIT_FRAME::DeleteAnnotation( ANNOTATE_SCOPE_T aAnnotateScope, bool aRe
     if( erc_dlg )
         static_cast<DIALOG_ERC*>( erc_dlg )->UpdateAnnotationWarning();
 
-    SyncView();
-    GetCanvas()->Refresh();
-    OnModify();
+    commit.Push( _( "Delete Annotation" ) );
 
     // Must go after OnModify() so the connectivity graph has been updated
     UpdateNetHighlightStatus();
@@ -191,22 +209,17 @@ std::unordered_set<SCH_SYMBOL*> getInferredSymbols( const EE_SELECTION& aSelecti
 }
 
 
-void SCH_EDIT_FRAME::AnnotateSymbols( ANNOTATE_SCOPE_T  aAnnotateScope,
-                                      ANNOTATE_ORDER_T  aSortOption,
-                                      ANNOTATE_ALGO_T   aAlgoOption,
-                                      bool              aRecursive,
-                                      int               aStartNumber,
-                                      bool              aResetAnnotation,
-                                      bool              aRepairTimestamps,
-                                      REPORTER&         aReporter,
-                                      bool              appendUndo )
+void SCH_EDIT_FRAME::AnnotateSymbols( SCH_COMMIT* aCommit, ANNOTATE_SCOPE_T  aAnnotateScope,
+                                      ANNOTATE_ORDER_T aSortOption, ANNOTATE_ALGO_T aAlgoOption,
+                                      bool aRecursive, int aStartNumber, bool aResetAnnotation,
+                                      bool aRepairTimestamps, REPORTER& aReporter )
 {
     EE_SELECTION_TOOL* selTool = m_toolManager->GetTool<EE_SELECTION_TOOL>();
     EE_SELECTION&      selection = selTool->GetSelection();
 
     SCH_REFERENCE_LIST references;
     SCH_SCREENS        screens( Schematic().Root() );
-    SCH_SHEET_LIST     sheets = Schematic().GetSheets();
+    SCH_SHEET_LIST     sheets = Schematic().Hierarchy();
     SCH_SHEET_PATH     currentSheet = GetCurrentSheet();
 
 
@@ -276,12 +289,19 @@ void SCH_EDIT_FRAME::AnnotateSymbols( ANNOTATE_SCOPE_T  aAnnotateScope,
 
     case ANNOTATE_CURRENT_SHEET:
         currentSheet.GetMultiUnitSymbols( lockedSymbols );
+
+        if( aRecursive )
+            subSheets.GetMultiUnitSymbols( lockedSymbols );
+
         break;
 
     case ANNOTATE_SELECTION:
         for( SCH_SYMBOL* symbol : selectedSymbols )
             currentSheet.AppendMultiUnitSymbol( lockedSymbols, symbol );
 
+        if( aRecursive )
+            selectedSheets.GetMultiUnitSymbols( lockedSymbols );
+
         break;
     }
 
@@ -353,8 +373,7 @@ void SCH_EDIT_FRAME::AnnotateSymbols( ANNOTATE_SCOPE_T  aAnnotateScope,
         SCH_SYMBOL*     symbol = ref.GetSymbol();
         SCH_SHEET_PATH* sheet = &ref.GetSheetPath();
 
-        SaveCopyInUndoList( sheet->LastScreen(), symbol, UNDO_REDO::CHANGED, appendUndo );
-        appendUndo = true;
+        aCommit->Modify( symbol, sheet->LastScreen() );
         ref.Annotate();
 
         KIID_PATH full_uuid = sheet->Path();
@@ -364,7 +383,7 @@ void SCH_EDIT_FRAME::AnnotateSymbols( ANNOTATE_SCOPE_T  aAnnotateScope,
         wxString  newRef  = symbol->GetRef( sheet );
 
         if( symbol->GetUnitCount() > 1 )
-            newRef << LIB_SYMBOL::SubReference( symbol->GetUnitSelection( sheet ) );
+            newRef << symbol->SubReference( symbol->GetUnitSelection( sheet ) );
 
         wxString msg;
 
@@ -376,15 +395,15 @@ void SCH_EDIT_FRAME::AnnotateSymbols( ANNOTATE_SCOPE_T  aAnnotateScope,
             if( symbol->GetUnitCount() > 1 )
             {
                 msg.Printf( _( "Updated %s (unit %s) from %s to %s." ),
-                            symbol->GetValueFieldText( true, sheet, false ),
-                            LIB_SYMBOL::SubReference( symbol->GetUnit(), false ),
+                            symbol->GetValue( true, sheet, false ),
+                            symbol->SubReference( symbol->GetUnit(), false ),
                             prevRef,
                             newRef );
             }
             else
             {
                 msg.Printf( _( "Updated %s from %s to %s." ),
-                            symbol->GetValueFieldText( true, sheet, false ),
+                            symbol->GetValue( true, sheet, false ),
                             prevRef,
                             newRef );
             }
@@ -394,14 +413,14 @@ void SCH_EDIT_FRAME::AnnotateSymbols( ANNOTATE_SCOPE_T  aAnnotateScope,
             if( symbol->GetUnitCount() > 1 )
             {
                 msg.Printf( _( "Annotated %s (unit %s) as %s." ),
-                            symbol->GetValueFieldText( true, sheet, false ),
-                            LIB_SYMBOL::SubReference( symbol->GetUnit(), false ),
+                            symbol->GetValue( true, sheet, false ),
+                            symbol->SubReference( symbol->GetUnit(), false ),
                             newRef );
             }
             else
             {
                 msg.Printf( _( "Annotated %s as %s." ),
-                            symbol->GetValueFieldText( true, sheet, false ),
+                            symbol->GetValue( true, sheet, false ),
                             newRef );
             }
         }
@@ -444,14 +463,14 @@ int SCH_EDIT_FRAME::CheckAnnotate( ANNOTATION_ERROR_HANDLER aErrorHandler,
 {
     SCH_REFERENCE_LIST  referenceList;
     constexpr bool      includePowerSymbols = false;
-    SCH_SHEET_LIST      sheets = Schematic().GetSheets();
+    SCH_SHEET_LIST      sheets = Schematic().Hierarchy();
     SCH_SHEET_PATH      currentSheet = GetCurrentSheet();
 
     // Build the list of symbols
     switch( aAnnotateScope )
     {
     case ANNOTATE_ALL:
-        Schematic().GetSheets().GetSymbols( referenceList );
+        sheets.GetSymbols( referenceList );
         break;
 
     case ANNOTATE_CURRENT_SHEET:
