@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2019 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2011 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,16 +30,15 @@
 #include <sch_sheet.h>
 #include <sch_symbol.h>
 #include <sch_reference_list.h>
-#include <schematic.h>
-#include <reporter.h>
 #include <string_utils.h>
 #include <netlist_exporters/netlist_exporter_kicad.h>
 #include <project/project_file.h>
 #include <project/net_settings.h>
+#include <richio.h>
 #include <tools/ee_actions.h>
 #include <tools/sch_editor_control.h>
 #include <advanced_config.h>
-#include <netclass.h>
+#include <widgets/design_block_pane.h>
 #include <wx/log.h>
 
 SCH_ITEM* SCH_EDITOR_CONTROL::FindSymbolAndItem( const wxString* aPath, const wxString* aReference,
@@ -47,8 +46,7 @@ SCH_ITEM* SCH_EDITOR_CONTROL::FindSymbolAndItem( const wxString* aPath, const wx
                                                  const wxString& aSearchText )
 {
     SCH_SHEET_PATH* sheetWithSymbolFound = nullptr;
-    SCH_SYMBOL*     symbol                  = nullptr;
-    VECTOR2I        pos;
+    SCH_SYMBOL*     symbol = nullptr;
     SCH_PIN*        pin = nullptr;
     SCH_SHEET_LIST  sheetList;
     SCH_ITEM*       foundItem = nullptr;
@@ -56,7 +54,7 @@ SCH_ITEM* SCH_EDITOR_CONTROL::FindSymbolAndItem( const wxString* aPath, const wx
     if( !aSearchHierarchy )
         sheetList.push_back( m_frame->GetCurrentSheet() );
     else
-        sheetList = m_frame->Schematic().GetSheets();
+        sheetList = m_frame->Schematic().Hierarchy();
 
     for( SCH_SHEET_PATH& sheet : sheetList )
     {
@@ -86,8 +84,6 @@ SCH_ITEM* SCH_EDITOR_CONTROL::FindSymbolAndItem( const wxString* aPath, const wx
 
                 if( aSearchType == HIGHLIGHT_PIN )
                 {
-                    // temporary: will be changed if the pin is found.
-                    pos = symbol->GetPosition();
                     pin = symbol->GetPin( aSearchText );
 
                     // Ensure we have found the right unit in case of multi-units symbol
@@ -102,14 +98,12 @@ SCH_ITEM* SCH_EDITOR_CONTROL::FindSymbolAndItem( const wxString* aPath, const wx
                         }
 
                         // Get pin position in true schematic coordinate
-                        pos = pin->GetPosition();
                         foundItem = pin;
                         break;
                     }
                 }
                 else
                 {
-                    pos = symbol->GetPosition();
                     foundItem = symbol;
                     break;
                 }
@@ -122,7 +116,6 @@ SCH_ITEM* SCH_EDITOR_CONTROL::FindSymbolAndItem( const wxString* aPath, const wx
 
     CROSS_PROBING_SETTINGS& crossProbingSettings = m_frame->eeconfig()->m_CrossProbing;
 
-
     if( symbol )
     {
         if( *sheetWithSymbolFound != m_frame->GetCurrentSheet() )
@@ -176,13 +169,26 @@ SCH_ITEM* SCH_EDITOR_CONTROL::FindSymbolAndItem( const wxString* aPath, const wx
     }
 
     m_frame->SetStatusText( msg );
-
     m_frame->GetCanvas()->Refresh();
 
     return foundItem;
 }
 
 
+/* Execute a remote command sent via a socket on port KICAD_PCB_PORT_SERVICE_NUMBER
+ *
+ * Commands are:
+ *
+ * $PART: "reference"                  Put cursor on symbol.
+ * $PART: "reference" $REF: "ref"      Put cursor on symbol reference.
+ * $PART: "reference" $VAL: "value"    Put cursor on symbol value.
+ * $PART: "reference" $PAD: "pin name" Put cursor on the symbol pin.
+ * $NET: "netname"                     Highlight a specified net
+ * $CLEAR: "HIGHLIGHTED"               Clear symbols highlight
+ *
+ * $CONFIG     Show the Manage Symbol Libraries dialog
+ * $ERC        Show the ERC dialog
+ */
 void SCH_EDIT_FRAME::ExecuteRemoteCommand( const char* cmdline )
 {
     SCH_EDITOR_CONTROL* editor = m_toolManager->GetTool<SCH_EDITOR_CONTROL>();
@@ -199,25 +205,35 @@ void SCH_EDIT_FRAME::ExecuteRemoteCommand( const char* cmdline )
 
     CROSS_PROBING_SETTINGS& crossProbingSettings = eeconfig()->m_CrossProbing;
 
-    if( strcmp( idcmd, "$NET:" ) == 0 )
+    if( strcmp( idcmd, "$CONFIG" ) == 0 )
+    {
+        GetToolManager()->RunAction( ACTIONS::showSymbolLibTable );
+        return;
+    }
+    else if( strcmp( idcmd, "$ERC" ) == 0 )
+    {
+        GetToolManager()->RunAction( EE_ACTIONS::runERC );
+        return;
+    }
+    else if( strcmp( idcmd, "$NET:" ) == 0 )
     {
         if( !crossProbingSettings.auto_highlight )
             return;
 
-        wxString netName = FROM_UTF8( text );
+        wxString netName = From_UTF8( text );
 
         if( auto sg = Schematic().ConnectionGraph()->FindFirstSubgraphByName( netName ) )
-            m_highlightedConn = sg->m_driver_connection;
+            m_highlightedConn = sg->GetDriverConnection()->Name();
         else
-            m_highlightedConn = nullptr;
+            m_highlightedConn = wxEmptyString;
 
-        GetToolManager()->RunAction( EE_ACTIONS::updateNetHighlighting, true );
+        GetToolManager()->RunAction( EE_ACTIONS::updateNetHighlighting );
+        RefreshNetNavigator();
 
         SetStatusText( _( "Selected net:" ) + wxS( " " ) + UnescapeString( netName ) );
         return;
     }
-
-    if( strcmp( idcmd, "$CLEAR:" ) == 0 )
+    else if( strcmp( idcmd, "$CLEAR:" ) == 0 )
     {
         // Cross-probing is now done through selection so we no longer need a clear command
         return;
@@ -232,7 +248,7 @@ void SCH_EDIT_FRAME::ExecuteRemoteCommand( const char* cmdline )
     if( strcmp( idcmd, "$PART:" ) != 0 )
         return;
 
-    wxString part_ref = FROM_UTF8( text );
+    wxString part_ref = From_UTF8( text );
 
     /* look for a complement */
     idcmd = strtok( nullptr, " \n\r" );
@@ -249,7 +265,7 @@ void SCH_EDIT_FRAME::ExecuteRemoteCommand( const char* cmdline )
     if( text == nullptr )
         return;
 
-    wxString msg = FROM_UTF8( text );
+    wxString msg = From_UTF8( text );
 
     if( strcmp( idcmd, "$REF:" ) == 0 )
     {
@@ -285,11 +301,9 @@ void SCH_EDIT_FRAME::SendSelectItemsToPcb( const std::vector<EDA_ITEM*>& aItems,
         case SCH_SYMBOL_T:
         {
             SCH_SYMBOL* symbol = static_cast<SCH_SYMBOL*>( item );
-
-            wxString ref = symbol->GetField( REFERENCE_FIELD )->GetText();
+            wxString    ref = symbol->GetField( REFERENCE_FIELD )->GetText();
 
             parts.push_back( wxT( "F" ) + EscapeString( ref, CTX_IPC ) );
-
             break;
         }
 
@@ -297,35 +311,30 @@ void SCH_EDIT_FRAME::SendSelectItemsToPcb( const std::vector<EDA_ITEM*>& aItems,
         {
             // For cross probing, we need the full path of the sheet, because
             // we search by the footprint path prefix in the PCB editor
-
             wxString full_path = GetCurrentSheet().PathAsString() + item->m_Uuid.AsString();
 
             parts.push_back( wxT( "S" ) + full_path );
-
             break;
         }
 
         case SCH_PIN_T:
         {
-            SCH_PIN*    pin = static_cast<SCH_PIN*>( item );
-            SCH_SYMBOL* symbol = pin->GetParentSymbol();
-
-            wxString ref = symbol->GetField( REFERENCE_FIELD )->GetText();
+            SCH_PIN* pin = static_cast<SCH_PIN*>( item );
+            SYMBOL*  symbol = pin->GetParentSymbol();
+            wxString ref = symbol->GetRef( &GetCurrentSheet(), false );
 
             parts.push_back( wxT( "P" ) + EscapeString( ref, CTX_IPC ) + wxT( "/" )
                              + EscapeString( pin->GetShownNumber(), CTX_IPC ) );
-
             break;
         }
 
-        default: break;
+        default:
+            break;
         }
     }
 
     if( parts.empty() )
-    {
         return;
-    }
 
     std::string command = "$SELECT: 0,";
 
@@ -445,7 +454,7 @@ void SCH_EDIT_FRAME::SendCrossProbeClearHighlight()
 
 
 bool findSymbolsAndPins(
-        const SCHEMATIC& aSchematic, const SCH_SHEET_PATH& aSheetPath,
+        const SCH_SHEET_LIST& aSchematicSheetList, const SCH_SHEET_PATH& aSheetPath,
         std::unordered_map<wxString, std::vector<SCH_REFERENCE>>&             aSyncSymMap,
         std::unordered_map<wxString, std::unordered_map<wxString, SCH_PIN*>>& aSyncPinMap,
         bool                                                                  aRecursive = false )
@@ -453,12 +462,13 @@ bool findSymbolsAndPins(
     if( aRecursive )
     {
         // Iterate over children
-        for( const SCH_SHEET_PATH& candidate : aSchematic.GetSheets() )
+        for( const SCH_SHEET_PATH& candidate : aSchematicSheetList )
         {
             if( candidate == aSheetPath || !candidate.IsContainedWithin( aSheetPath ) )
                 continue;
 
-            findSymbolsAndPins( aSchematic, candidate, aSyncSymMap, aSyncPinMap, aRecursive );
+            findSymbolsAndPins( aSchematicSheetList, candidate, aSyncSymMap, aSyncPinMap,
+                                aRecursive );
         }
     }
 
@@ -524,7 +534,7 @@ bool findSymbolsAndPins(
 
 
 bool sheetContainsOnlyWantedItems(
-        const SCHEMATIC& aSchematic, const SCH_SHEET_PATH& aSheetPath,
+        const SCH_SHEET_LIST& aSchematicSheetList, const SCH_SHEET_PATH& aSheetPath,
         std::unordered_map<wxString, std::vector<SCH_REFERENCE>>&             aSyncSymMap,
         std::unordered_map<wxString, std::unordered_map<wxString, SCH_PIN*>>& aSyncPinMap,
         std::unordered_map<SCH_SHEET_PATH, bool>&                             aCache )
@@ -535,12 +545,12 @@ bool sheetContainsOnlyWantedItems(
         return cacheIt->second;
 
     // Iterate over children
-    for( const SCH_SHEET_PATH& candidate : aSchematic.GetSheets() )
+    for( const SCH_SHEET_PATH& candidate : aSchematicSheetList )
     {
         if( candidate == aSheetPath || !candidate.IsContainedWithin( aSheetPath ) )
             continue;
 
-        bool childRet = sheetContainsOnlyWantedItems( aSchematic, candidate, aSyncSymMap,
+        bool childRet = sheetContainsOnlyWantedItems( aSchematicSheetList, candidate, aSyncSymMap,
                                                       aSyncPinMap, aCache );
 
         if( !childRet )
@@ -610,10 +620,10 @@ findItemsFromSyncSelection( const SCHEMATIC& aSchematic, const std::string aSync
     std::optional<std::pair<wxString, wxString>>               focusPin;
     std::unordered_map<SCH_SHEET_PATH, std::vector<SCH_ITEM*>> focusItemResults;
 
-    const SCH_SHEET_LIST allSheetsList = aSchematic.GetSheets();
+    const SCH_SHEET_LIST allSheetsList = aSchematic.Hierarchy();
 
     // In orderedSheets, the current sheet comes first.
-    SCH_SHEET_PATHS orderedSheets;
+    std::vector<SCH_SHEET_PATH> orderedSheets;
     orderedSheets.reserve( allSheetsList.size() );
     orderedSheets.push_back( aSchematic.CurrentSheet() );
 
@@ -645,6 +655,7 @@ findItemsFromSyncSelection( const SCHEMATIC& aSchematic, const std::string aSync
             syncSymMap[symRef] = std::vector<SCH_REFERENCE>();
             break;
         }
+
         case 'P': // Select by pad: P<Footprint reference>/<Pad number>
         {
             wxString symRef = UnescapeString( syncData.BeforeFirst( '/' ) );
@@ -656,126 +667,125 @@ findItemsFromSyncSelection( const SCHEMATIC& aSchematic, const std::string aSync
             syncPinMap[symRef][padNum] = nullptr;
             break;
         }
-        default: break;
+
+        default:
+            break;
         }
     }
 
     // Lambda definitions
-    auto flattenSyncMaps = [&syncSymMap, &syncPinMap]() -> std::vector<SCH_ITEM*>
-    {
-        std::vector<SCH_ITEM*> allVec;
-
-        for( auto const& pairSym : syncSymMap )
-        {
-            for( const SCH_REFERENCE& ref : pairSym.second )
+    auto flattenSyncMaps =
+            [&syncSymMap, &syncPinMap]() -> std::vector<SCH_ITEM*>
             {
-                allVec.push_back( ref.GetSymbol() );
-            }
-        }
+                std::vector<SCH_ITEM*> allVec;
 
-        for( auto const& pairSym : syncPinMap )
-        {
-            for( auto const& pairPin : pairSym.second )
-            {
-                if( pairPin.second )
-                    allVec.push_back( pairPin.second );
-            }
-        }
-
-        return allVec;
-    };
-
-    auto clearSyncMaps = [&syncSymMap, &syncPinMap]()
-    {
-        for( auto& pairSym : syncSymMap )
-        {
-            pairSym.second.clear();
-        }
-
-        for( auto& pairSym : syncPinMap )
-        {
-            for( auto& pairPin : pairSym.second )
-            {
-                pairPin.second = nullptr;
-            }
-        }
-    };
-
-    auto syncMapsValuesEmpty = [&syncSymMap, &syncPinMap]() -> bool
-    {
-        for( auto const& pairSym : syncSymMap )
-        {
-            if( pairSym.second.size() > 0 )
-                return false;
-        }
-
-        for( auto const& pairSym : syncPinMap )
-        {
-            for( auto const& pairPin : pairSym.second )
-            {
-                if( pairPin.second )
-                    return false;
-            }
-        }
-
-        return true;
-    };
-
-    auto checkFocusItems = [&]( const SCH_SHEET_PATH& aSheetPath )
-    {
-        if( focusSymbol )
-        {
-            auto findIt = syncSymMap.find( *focusSymbol );
-            if( findIt != syncSymMap.end() )
-            {
-                if( findIt->second.size() > 0 )
+                for( const auto& [symRef, symbols] : syncSymMap )
                 {
-                    focusItemResults[aSheetPath].push_back( findIt->second.front().GetSymbol() );
+                    for( const SCH_REFERENCE& ref : symbols )
+                        allVec.push_back( ref.GetSymbol() );
                 }
-            }
-        }
-        else if( focusPin )
-        {
-            auto findIt = syncPinMap.find( focusPin->first );
-            if( findIt != syncPinMap.end() )
-            {
-                if( findIt->second[focusPin->second] )
+
+                for( const auto& [symRef, pinMap] : syncPinMap )
                 {
-                    focusItemResults[aSheetPath].push_back( findIt->second[focusPin->second] );
+                    for( const auto& [padNum, pin] : pinMap )
+                    {
+                        if( pin )
+                            allVec.push_back( pin );
+                    }
                 }
-            }
-        }
-    };
 
-    auto makeRetForSheet = [&]( const SCH_SHEET_PATH& aSheet, SCH_ITEM* aFocusItem )
-    {
-        clearSyncMaps();
+                return allVec;
+            };
 
-        // Fill sync maps
-        findSymbolsAndPins( aSchematic, aSheet, syncSymMap, syncPinMap );
-        std::vector<SCH_ITEM*> itemsVector = flattenSyncMaps();
-
-        // Add fully wanted sheets to vector
-        for( SCH_ITEM* item : aSheet.LastScreen()->Items().OfType( SCH_SHEET_T ) )
-        {
-            KIID_PATH kiidPath = aSheet.Path();
-            kiidPath.push_back( item->m_Uuid );
-
-            std::optional<SCH_SHEET_PATH> subsheetPath =
-                    allSheetsList.GetSheetPathByKIIDPath( kiidPath );
-
-            if( !subsheetPath )
-                continue;
-
-            if( sheetContainsOnlyWantedItems( aSchematic, *subsheetPath, syncSymMap, syncPinMap,
-                                              fullyWantedCache ) )
+    auto clearSyncMaps =
+            [&syncSymMap, &syncPinMap]()
             {
-                itemsVector.push_back( item );
-            }
-        }
+                for( auto& [symRef, symbols] : syncSymMap )
+                    symbols.clear();
 
-        return std::make_tuple( aSheet, aFocusItem, itemsVector );
-    };
+                for( auto& [reference, pins] : syncPinMap )
+                {
+                    for( auto& [number, pin] : pins )
+                        pin = nullptr;
+                }
+            };
+
+    auto syncMapsValuesEmpty =
+            [&syncSymMap, &syncPinMap]() -> bool
+            {
+                for( const auto& [symRef, symbols] : syncSymMap )
+                {
+                    if( symbols.size() > 0 )
+                        return false;
+                }
+
+                for( const auto& [symRef, pins] : syncPinMap )
+                {
+                    for( const auto& [padNum, pin] : pins )
+                    {
+                        if( pin )
+                            return false;
+                    }
+                }
+
+                return true;
+            };
+
+    auto checkFocusItems =
+            [&]( const SCH_SHEET_PATH& aSheet )
+            {
+                if( focusSymbol )
+                {
+                    auto findIt = syncSymMap.find( *focusSymbol );
+
+                    if( findIt != syncSymMap.end() )
+                    {
+                        if( findIt->second.size() > 0 )
+                            focusItemResults[aSheet].push_back( findIt->second.front().GetSymbol() );
+                    }
+                }
+                else if( focusPin )
+                {
+                    auto findIt = syncPinMap.find( focusPin->first );
+
+                    if( findIt != syncPinMap.end() )
+                    {
+                        if( findIt->second[focusPin->second] )
+                            focusItemResults[aSheet].push_back( findIt->second[focusPin->second] );
+                    }
+                }
+            };
+
+    auto makeRetForSheet =
+            [&]( const SCH_SHEET_PATH& aSheet, SCH_ITEM* aFocusItem )
+            {
+                clearSyncMaps();
+
+                // Fill sync maps
+                findSymbolsAndPins( allSheetsList, aSheet, syncSymMap, syncPinMap );
+                std::vector<SCH_ITEM*> itemsVector = flattenSyncMaps();
+
+                // Add fully wanted sheets to vector
+                for( SCH_ITEM* item : aSheet.LastScreen()->Items().OfType( SCH_SHEET_T ) )
+                {
+                    KIID_PATH kiidPath = aSheet.Path();
+                    kiidPath.push_back( item->m_Uuid );
+
+                    std::optional<SCH_SHEET_PATH> subsheetPath =
+                            allSheetsList.GetSheetPathByKIIDPath( kiidPath );
+
+                    if( !subsheetPath )
+                        continue;
+
+                    if( sheetContainsOnlyWantedItems( allSheetsList, *subsheetPath, syncSymMap,
+                                                      syncPinMap, fullyWantedCache ) )
+                    {
+                        itemsVector.push_back( item );
+                    }
+                }
+
+                return std::make_tuple( aSheet, aFocusItem, itemsVector );
+            };
 
     if( aFocusOnFirst )
     {
@@ -783,7 +793,7 @@ findItemsFromSyncSelection( const SCHEMATIC& aSchematic, const std::string aSync
         {
             clearSyncMaps();
 
-            findSymbolsAndPins( aSchematic, sheetPath, syncSymMap, syncPinMap );
+            findSymbolsAndPins( allSheetsList, sheetPath, syncSymMap, syncPinMap );
 
             checkFocusItems( sheetPath );
         }
@@ -792,10 +802,10 @@ findItemsFromSyncSelection( const SCHEMATIC& aSchematic, const std::string aSync
         {
             for( const SCH_SHEET_PATH& sheetPath : orderedSheets )
             {
-                auto vec = focusItemResults[sheetPath];
+                const std::vector<SCH_ITEM*>& items = focusItemResults[sheetPath];
 
-                if( !vec.empty() )
-                    return makeRetForSheet( sheetPath, vec.front() );
+                if( !items.empty() )
+                    return makeRetForSheet( sheetPath, items.front() );
             }
         }
     }
@@ -805,7 +815,7 @@ findItemsFromSyncSelection( const SCHEMATIC& aSchematic, const std::string aSync
         {
             clearSyncMaps();
 
-            findSymbolsAndPins( aSchematic, sheetPath, syncSymMap, syncPinMap );
+            findSymbolsAndPins( allSheetsList, sheetPath, syncSymMap, syncPinMap );
 
             if( !syncMapsValuesEmpty() )
             {
@@ -841,13 +851,15 @@ void SCH_EDIT_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
         // Try to select specified items.
 
         // $SELECT: 1,<spec1>,<spec2>,<spec3>
-        // Select and focus on <spec1> item, select other specified items that are on the same sheet.
+        // Select and focus on <spec1> item, select other specified items that are on the
+        // same sheet.
 
         std::string prefix = "$SELECT: ";
 
         std::string paramStr = payload.substr( prefix.size() );
 
-        if( paramStr.size() < 2 )   // Empty/broken command: we need at least 2 chars for sync string.
+        // Empty/broken command: we need at least 2 chars for sync string.
+        if( paramStr.size() < 2 )
             break;
 
         std::string syncStr = paramStr.substr( 2 );
@@ -883,14 +895,35 @@ void SCH_EDIT_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
                 return;
         }
 
+        if( ADVANCED_CFG::GetCfg().m_IncrementalConnectivity )
+            RecalculateConnections( nullptr, GLOBAL_CLEANUP );
+
         NETLIST_EXPORTER_KICAD exporter( &Schematic() );
         STRING_FORMATTER formatter;
 
         exporter.Format( &formatter, GNL_ALL | GNL_OPT_KICAD );
 
         payload = formatter.GetString();
-    }
         break;
+    }
+
+    case MAIL_SCH_GET_ITEM:
+    {
+        KIID           uuid( payload );
+        SCH_SHEET_PATH path;
+
+        if( SCH_ITEM* item = m_schematic->GetItem( uuid, &path ) )
+        {
+            if( item->Type() == SCH_SHEET_T )
+                payload = static_cast<SCH_SHEET*>( item )->GetShownName( false );
+            else if( item->Type() == SCH_SYMBOL_T )
+                payload = static_cast<SCH_SYMBOL*>( item )->GetRef( &path, true );
+            else
+                payload = item->GetFriendlyName();
+        }
+
+        break;
+    }
 
     case MAIL_ASSIGN_FOOTPRINTS:
         try
@@ -909,19 +942,28 @@ void SCH_EDIT_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
 
         GetCanvas()->GetView()->UpdateAllItems( KIGFX::ALL );
         GetCanvas()->Refresh();
-    }
         break;
+    }
 
     case MAIL_IMPORT_FILE:
     {
-        // Extract file format type and path (plugin type and path separated with \n)
-        size_t split = payload.find( '\n' );
-        wxCHECK( split != std::string::npos, /*void*/ );
+        // Extract file format type and path (plugin type, path and properties keys, values
+        // separated with \n)
+        std::stringstream ss( payload );
+        char              delim = '\n';
+
+        std::string formatStr;
+        wxCHECK( std::getline( ss, formatStr, delim ), /* void */ );
+
+        std::string fnameStr;
+        wxCHECK( std::getline( ss, fnameStr, delim ), /* void */ );
+        wxASSERT( !fnameStr.empty() );
+
         int importFormat;
 
         try
         {
-            importFormat = std::stoi( payload.substr( 0, split ) );
+            importFormat = std::stoi( formatStr );
         }
         catch( std::invalid_argument& )
         {
@@ -929,13 +971,26 @@ void SCH_EDIT_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
             importFormat = -1;
         }
 
-        std::string path = payload.substr( split + 1 );
-        wxASSERT( !path.empty() );
+        std::map<std::string, UTF8> props;
+
+        do
+        {
+            std::string key, value;
+
+            if( !std::getline( ss, key, delim ) )
+                break;
+
+            std::getline( ss, value, delim ); // We may want an empty string as value
+
+            props.emplace( key, value );
+
+        } while( true );
 
         if( importFormat >= 0 )
-            importFile( path, importFormat );
-    }
+            importFile( fnameStr, importFormat, props.empty() ? nullptr : &props );
+
         break;
+    }
 
     case MAIL_SCH_SAVE:
         if( SaveProject() )
@@ -944,10 +999,11 @@ void SCH_EDIT_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
         break;
 
     case MAIL_SCH_UPDATE:
-        m_toolManager->RunAction( ACTIONS::updateSchematicFromPcb, true );
+        m_toolManager->RunAction( ACTIONS::updateSchematicFromPcb );
         break;
 
     case MAIL_RELOAD_LIB:
+        m_designBlocksPane->RefreshLibs();
         SyncView();
         break;
 
