@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,6 +21,10 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include "dialog_lib_symbol_properties.h"
+
+#include <pgm_base.h>
+#include <eeschema_settings.h>
 #include <bitmaps.h>
 #include <confirm.h>
 #include <dialogs/dialog_text_entry.h>
@@ -34,19 +38,22 @@
 #include <widgets/wx_grid.h>
 #include <widgets/std_bitmap_button.h>
 #include <string_utils.h>
-
-#ifdef KICAD_SPICE
+#include <project_sch.h>
+#include <refdes_utils.h>
 #include <dialog_sim_model.h>
-#endif /* KICAD_SPICE */
 
-#include <dialog_lib_symbol_properties.h>
+#include <panel_embedded_files.h>
 #include <settings/settings_manager.h>
 #include <symbol_editor_settings.h>
+#include <widgets/listbox_tricks.h>
+
+#include <wx/clipbrd.h>
+#include <wx/msgdlg.h>
 
 
 int DIALOG_LIB_SYMBOL_PROPERTIES::m_lastOpenedPage = 0;
-DIALOG_LIB_SYMBOL_PROPERTIES::LAST_LAYOUT
-    DIALOG_LIB_SYMBOL_PROPERTIES::m_lastLayout = DIALOG_LIB_SYMBOL_PROPERTIES::NONE;
+DIALOG_LIB_SYMBOL_PROPERTIES::LAST_LAYOUT DIALOG_LIB_SYMBOL_PROPERTIES::m_lastLayout =
+        DIALOG_LIB_SYMBOL_PROPERTIES::LAST_LAYOUT::NONE;
 
 
 DIALOG_LIB_SYMBOL_PROPERTIES::DIALOG_LIB_SYMBOL_PROPERTIES( SYMBOL_EDIT_FRAME* aParent,
@@ -59,13 +66,17 @@ DIALOG_LIB_SYMBOL_PROPERTIES::DIALOG_LIB_SYMBOL_PROPERTIES( SYMBOL_EDIT_FRAME* a
     m_delayedFocusGrid( nullptr ),
     m_delayedFocusRow( -1 ),
     m_delayedFocusColumn( -1 ),
-    m_delayedFocusPage( -1 )
+    m_delayedFocusPage( -1 ),
+    m_fpFilterTricks( std::make_unique<LISTBOX_TRICKS>( *this, *m_FootprintFilterListBox ) )
 {
+    m_embeddedFiles = new PANEL_EMBEDDED_FILES( m_NoteBook, m_libEntry );
+    m_NoteBook->AddPage( m_embeddedFiles, _( "Embedded Files" ) );
+
     // Give a bit more room for combobox editors
     m_grid->SetDefaultRowSize( m_grid->GetDefaultRowSize() + 4 );
-    m_fields = new FIELDS_GRID_TABLE<LIB_FIELD>( this, aParent, m_grid, m_libEntry );
+    m_fields = new FIELDS_GRID_TABLE( this, aParent, m_grid, m_libEntry, m_embeddedFiles->GetLocalFiles() );
     m_grid->SetTable( m_fields );
-    m_grid->PushEventHandler( new FIELDS_GRID_TRICKS( m_grid, this,
+    m_grid->PushEventHandler( new FIELDS_GRID_TRICKS( m_grid, this, aLibEntry,
                                                       [&]( wxCommandEvent& aEvent )
                                                       {
                                                           OnAddField( aEvent );
@@ -77,19 +88,20 @@ DIALOG_LIB_SYMBOL_PROPERTIES::DIALOG_LIB_SYMBOL_PROPERTIES( SYMBOL_EDIT_FRAME* a
     m_grid->ShowHideColumns( cfg->m_EditSymbolVisibleColumns );
 
     wxGridCellAttr* attr = new wxGridCellAttr;
-    attr->SetEditor( new GRID_CELL_URL_EDITOR( this, Prj().SchSearchS() ) );
+    attr->SetEditor( new GRID_CELL_URL_EDITOR( this, PROJECT_SCH::SchSearchS( &Prj() ),
+                                               m_embeddedFiles->GetLocalFiles() ) );
     m_grid->SetAttr( DATASHEET_FIELD, FDC_VALUE, attr );
 
-    m_SymbolNameCtrl->SetValidator( SCH_FIELD_VALIDATOR( true, VALUE_FIELD ) );
+    m_SymbolNameCtrl->SetValidator( FIELD_VALIDATOR( VALUE_FIELD ) );
 
     // Configure button logos
-    m_bpAdd->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
-    m_bpDelete->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
-    m_bpMoveUp->SetBitmap( KiBitmap( BITMAPS::small_up ) );
-    m_bpMoveDown->SetBitmap( KiBitmap( BITMAPS::small_down ) );
-    m_addFilterButton->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
-    m_deleteFilterButton->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
-    m_editFilterButton->SetBitmap( KiBitmap( BITMAPS::small_edit ) );
+    m_bpAdd->SetBitmap( KiBitmapBundle( BITMAPS::small_plus ) );
+    m_bpDelete->SetBitmap( KiBitmapBundle( BITMAPS::small_trash ) );
+    m_bpMoveUp->SetBitmap( KiBitmapBundle( BITMAPS::small_up ) );
+    m_bpMoveDown->SetBitmap( KiBitmapBundle( BITMAPS::small_down ) );
+    m_addFilterButton->SetBitmap( KiBitmapBundle( BITMAPS::small_plus ) );
+    m_deleteFilterButton->SetBitmap( KiBitmapBundle( BITMAPS::small_trash ) );
+    m_editFilterButton->SetBitmap( KiBitmapBundle( BITMAPS::small_edit ) );
 
     SetupStandardButtons();
 
@@ -100,27 +112,39 @@ DIALOG_LIB_SYMBOL_PROPERTIES::DIALOG_LIB_SYMBOL_PROPERTIES( SYMBOL_EDIT_FRAME* a
         m_stdSizerButtonOK->Enable( false );
     }
 
-#ifndef KICAD_SPICE
-    m_excludeFromSim->Hide();
-    m_spiceFieldsButton->Hide();
-#endif
-
     // wxFormBuilder doesn't include this event...
     m_grid->Connect( wxEVT_GRID_CELL_CHANGING,
                      wxGridEventHandler( DIALOG_LIB_SYMBOL_PROPERTIES::OnGridCellChanging ),
                      nullptr, this );
 
-    if( m_lastLayout != DIALOG_LIB_SYMBOL_PROPERTIES::NONE )
+    // Forward the delete button to the tricks
+    m_deleteFilterButton->Bind( wxEVT_BUTTON,
+                                [&]( wxCommandEvent& aEvent )
+                                {
+                                    wxCommandEvent cmdEvent( EDA_EVT_LISTBOX_DELETE );
+                                    m_fpFilterTricks->ProcessEvent( cmdEvent );
+                                } );
+
+    // When the filter tricks modifies something, update ourselves
+    m_FootprintFilterListBox->Bind( EDA_EVT_LISTBOX_CHANGED,
+                                    [&]( wxCommandEvent& aEvent )
+                                    {
+                                        OnModify();
+                                    } );
+
+    if( m_lastLayout != DIALOG_LIB_SYMBOL_PROPERTIES::LAST_LAYOUT::NONE )
     {
-        if( ( m_lastLayout == DIALOG_LIB_SYMBOL_PROPERTIES::ALIAS && aLibEntry->IsRoot() )
-          || ( m_lastLayout == DIALOG_LIB_SYMBOL_PROPERTIES::PARENT && aLibEntry->IsAlias() ) )
+        if( ( m_lastLayout == DIALOG_LIB_SYMBOL_PROPERTIES::LAST_LAYOUT::ALIAS
+              && aLibEntry->IsRoot() )
+            || ( m_lastLayout == DIALOG_LIB_SYMBOL_PROPERTIES::LAST_LAYOUT::PARENT
+                 && aLibEntry->IsAlias() ) )
         {
             resetSize();
         }
     }
 
-    m_lastLayout = ( aLibEntry->IsAlias() ) ? DIALOG_LIB_SYMBOL_PROPERTIES::ALIAS
-                                            : DIALOG_LIB_SYMBOL_PROPERTIES::PARENT;
+    m_lastLayout = ( aLibEntry->IsAlias() ) ? DIALOG_LIB_SYMBOL_PROPERTIES::LAST_LAYOUT::ALIAS
+                                            : DIALOG_LIB_SYMBOL_PROPERTIES::LAST_LAYOUT::PARENT;
 
     m_grid->GetParent()->Layout();
     syncControlStates( m_libEntry->IsAlias() );
@@ -155,7 +179,36 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataToWindow()
         return false;
 
     // Push a copy of each field into m_updateFields
-    m_libEntry->GetFields( *m_fields );
+    m_libEntry->CopyFields( *m_fields );
+
+    std::set<wxString> defined;
+
+    for( SCH_FIELD& field : *m_fields )
+        defined.insert( field.GetName() );
+
+    // Add in any template fieldnames not yet defined:
+    // Read global fieldname templates
+    SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
+    EESCHEMA_SETTINGS* cfg = mgr.GetAppSettings<EESCHEMA_SETTINGS>( "eeschema" );
+
+    if( cfg )
+    {
+        TEMPLATES templateMgr;
+
+        if( !cfg->m_Drawing.field_names.IsEmpty() )
+            templateMgr.AddTemplateFieldNames( cfg->m_Drawing.field_names );
+
+        for( const TEMPLATE_FIELDNAME& templateFieldname : templateMgr.GetTemplateFieldNames() )
+        {
+            if( defined.count( templateFieldname.m_Name ) <= 0 )
+            {
+                SCH_FIELD field( VECTOR2I( 0, 0 ), -1, m_libEntry, templateFieldname.m_Name );
+                field.SetVisible( templateFieldname.m_Visible );
+                m_fields->push_back( field );
+                m_addedTemplateFields.insert( templateFieldname.m_Name );
+            }
+        }
+    }
 
     // The Y axis for components in lib is from bottom to top while the screen axis is top
     // to bottom: we must change the y coord sign for editing
@@ -173,30 +226,29 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataToWindow()
 
     m_SymbolNameCtrl->ChangeValue( UnescapeString( m_libEntry->GetName() ) );
 
-    m_DescCtrl->ChangeValue( m_libEntry->GetDescription() );
     m_KeywordCtrl->ChangeValue( m_libEntry->GetKeyWords() );
     m_SelNumberOfUnits->SetValue( m_libEntry->GetUnitCount() );
     m_OptionPartsInterchangeable->SetValue( !m_libEntry->UnitsLocked() ||
                                             m_libEntry->GetUnitCount() == 1 );
 
-    // If a symbol contains no conversion-specific pins or graphic items, symbol->HasConversion()
-    // will return false.  But when editing a symbol with DeMorgan option set, we don't want to
-    // keep turning it off just because there aren't any conversion-specific items yet, so we force
-    // it to on if the parent frame has it enabled.
-    m_AsConvertButt->SetValue( m_Parent->GetShowDeMorgan() );
+    // If a symbol contains no body-style-specific pins or graphic items,
+    // symbol->HasAlternateBodyStyle() will return false.
+    // But when editing a symbol with DeMorgan option set, we don't want to keep turning it off
+    // just because there aren't any body-style-specific items yet, so we force it to on if the
+    // parent frame has it enabled.
+    m_hasAlternateBodyStyles->SetValue( m_Parent->GetShowDeMorgan() );
 
     m_OptionPower->SetValue( m_libEntry->IsPower() );
 
-#ifdef KICAD_SPICE
-    LIB_FIELD* simEnableField = m_libEntry->FindField( SIM_ENABLE_FIELD );
-    m_excludeFromSim->SetValue( simEnableField && simEnableField->GetText() == wxT( "0" ) );
-#endif
+    if( m_libEntry->IsPower() )
+        m_spiceFieldsButton->Hide();
 
-    m_excludeFromBomCheckBox->SetValue( !m_libEntry->GetIncludeInBom() );
-    m_excludeFromBoardCheckBox->SetValue( !m_libEntry->GetIncludeOnBoard() );
+    m_excludeFromSimCheckBox->SetValue( m_libEntry->GetExcludedFromSim() );
+    m_excludeFromBomCheckBox->SetValue( m_libEntry->GetExcludedFromBOM() );
+    m_excludeFromBoardCheckBox->SetValue( m_libEntry->GetExcludedFromBoard() );
 
-    m_ShowPinNumButt->SetValue( m_libEntry->ShowPinNumbers() );
-    m_ShowPinNameButt->SetValue( m_libEntry->ShowPinNames() );
+    m_ShowPinNumButt->SetValue( m_libEntry->GetShowPinNumbers() );
+    m_ShowPinNameButt->SetValue( m_libEntry->GetShowPinNames() );
     m_PinsNameInsideButt->SetValue( m_libEntry->GetPinNameOffset() != 0 );
     m_pinNameOffset.ChangeValue( m_libEntry->GetPinNameOffset() );
 
@@ -206,83 +258,55 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataToWindow()
     // Populate the list of root parts for inherited objects.
     if( m_libEntry->IsAlias() )
     {
-        wxArrayString rootSymbolNames;
+        wxArrayString symbolNames;
         wxString libName = m_Parent->GetCurLib();
 
         // Someone forgot to set the current library in the editor frame window.
         wxCHECK( !libName.empty(), false );
 
-        m_Parent->GetLibManager().GetRootSymbolNames( libName, rootSymbolNames );
-        m_inheritanceSelectCombo->Append( rootSymbolNames );
+        m_Parent->GetLibManager().GetSymbolNames( libName, symbolNames );
 
-        LIB_SYMBOL_SPTR rootSymbol = m_libEntry->GetParent().lock();
+        // Sort the list of symbols for easier search
+        symbolNames.Sort(
+                []( const wxString& a, const wxString& b ) -> int
+                {
+                    return StrNumCmp( a, b, true );
+                } );
 
-        wxCHECK( rootSymbol, false );
+        // Do allow an inherited symbol to be derived from itself.
+        symbolNames.Remove( m_libEntry->GetName() );
+        m_inheritanceSelectCombo->Append( symbolNames );
 
-        wxString parentName = UnescapeString( rootSymbol->GetName() );
-        int selection = m_inheritanceSelectCombo->FindString( parentName );
+        if( LIB_SYMBOL_SPTR rootSymbol = m_libEntry->GetParent().lock() )
+        {
+            wxString parentName = UnescapeString( rootSymbol->GetName() );
+            int selection = m_inheritanceSelectCombo->FindString( parentName );
 
-        wxCHECK( selection != wxNOT_FOUND, false );
-        m_inheritanceSelectCombo->SetSelection( selection );
+            if( selection == wxNOT_FOUND )
+                return false;
+
+            m_inheritanceSelectCombo->SetSelection( selection );
+        }
 
         m_lastOpenedPage = 0;
     }
 
     m_NoteBook->SetSelection( (unsigned) m_lastOpenedPage );
 
+    m_embeddedFiles->TransferDataToWindow();
+
     return true;
 }
 
 
-void DIALOG_LIB_SYMBOL_PROPERTIES::OnExcludeFromSimulation( wxCommandEvent& event )
-{
-#ifdef KICAD_SPICE
-    int simEnableFieldRow = -1;
-
-    for( int ii = MANDATORY_FIELDS; ii < m_grid->GetNumberRows(); ++ii )
-    {
-        if( m_grid->GetCellValue( ii, FDC_NAME ) == SIM_ENABLE_FIELD )
-            simEnableFieldRow = ii;
-    }
-
-    if( event.IsChecked() )
-    {
-        if( simEnableFieldRow == -1 )
-        {
-            simEnableFieldRow = (int) m_fields->size();
-            m_fields->emplace_back( m_libEntry, simEnableFieldRow, SIM_ENABLE_FIELD );
-
-            // notify the grid
-            wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, 1 );
-            m_grid->ProcessTableMessage( msg );
-        }
-
-        m_grid->SetCellValue( simEnableFieldRow, FDC_VALUE, wxT( "0" ) );
-        m_grid->SetCellValue( simEnableFieldRow, FDC_SHOWN, wxT( "0" ) );
-        m_grid->SetCellValue( simEnableFieldRow, FDC_SHOW_NAME, wxT( "0" ) );
-    }
-    else if( simEnableFieldRow >= 0 )
-    {
-        m_fields->erase( m_fields->begin() + simEnableFieldRow );
-
-        // notify the grid
-        wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_DELETED, simEnableFieldRow, 1 );
-        m_grid->ProcessTableMessage( msg );
-    }
-
-    OnModify();
-#endif
-}
-
-
 bool DIALOG_LIB_SYMBOL_PROPERTIES::Validate()
 {
     if( !m_grid->CommitPendingChanges() )
         return false;
 
     // Alias symbol reference can be empty because it inherits from the parent symbol.
-    if( m_libEntry->IsRoot() &&
-        !SCH_SYMBOL::IsReferenceStringValid( m_fields->at( REFERENCE_FIELD ).GetText() ) )
+    if( m_libEntry->IsRoot()
+            && UTIL::GetRefDesPrefix( m_fields->at( REFERENCE_FIELD ).GetText() ).IsEmpty() )
     {
         if( m_NoteBook->GetSelection() != 0 )
             m_NoteBook->SetSelection( 0 );
@@ -297,10 +321,14 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::Validate()
     }
 
     // Check for missing field names.
-    for( int ii = MANDATORY_FIELDS; ii < (int) m_fields->size(); ++ii )
+    for( int ii = 0; ii < (int) m_fields->size(); ++ii )
     {
-        LIB_FIELD& field = m_fields->at( ii );
-        wxString   fieldName = field.GetName( false );
+        SCH_FIELD& field = m_fields->at( ii );
+
+        if( field.IsMandatory() )
+            continue;
+
+        wxString fieldName = field.GetName( false );
 
         if( fieldName.IsEmpty() && !field.GetText().IsEmpty() )
         {
@@ -340,7 +368,7 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::Validate()
             return false;
     }
 
-    if( !m_AsConvertButt->GetValue() && m_libEntry->HasConversion() )
+    if( !m_hasAlternateBodyStyles->GetValue() && m_libEntry->HasAlternateBodyStyle() )
     {
         if( !IsOK( this, _( "Delete alternate body style (De Morgan) from symbol?" ) ) )
             return false;
@@ -361,6 +389,12 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataFromWindow()
     wxString   newName = EscapeString( m_SymbolNameCtrl->GetValue(), CTX_LIBID );
     wxString   oldName = m_libEntry->GetName();
 
+    if( newName.IsEmpty() )
+    {
+        wxMessageBox( _( "Symbol must have a name." ) );
+        return false;
+    }
+
     if( oldName != newName )
     {
         wxString libName = m_Parent->GetCurLib();
@@ -376,11 +410,12 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataFromWindow()
             return false;
         }
 
-        m_Parent->SaveCopyInUndoList( m_libEntry, UNDO_REDO::LIB_RENAME );
+        m_Parent->SaveCopyInUndoList( _( "Edit Symbol Properties" ), m_libEntry,
+                                      UNDO_REDO::LIB_RENAME );
     }
     else
     {
-        m_Parent->SaveCopyInUndoList( m_libEntry );
+        m_Parent->SaveCopyInUndoList( _( "Edit Symbol Properties" ), m_libEntry );
     }
 
     // The Y axis for components in lib is from bottom to top while the screen axis is top
@@ -393,15 +428,24 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataFromWindow()
         m_fields->at( ii ).SetId( ii );
     }
 
-    for( int ii = m_fields->GetNumberRows() - 1; ii >= MANDATORY_FIELDS; ii-- )
+    for( int ii = m_fields->GetNumberRows() - 1; ii >= 0; ii-- )
     {
-        LIB_FIELD&      field = m_fields->at( ii );
+        SCH_FIELD& field = m_fields->at( ii );
+
+        if( field.IsMandatory() )
+            continue;
+
         const wxString& fieldName = field.GetCanonicalName();
 
-        if( fieldName.IsEmpty() && field.GetText().IsEmpty() )
-            m_fields->erase( m_fields->begin() + ii );
+        if( field.GetText().IsEmpty() )
+        {
+            if( fieldName.IsEmpty() || m_addedTemplateFields.contains( fieldName ) )
+                m_fields->erase( m_fields->begin() + ii );
+        }
         else if( fieldName.IsEmpty() )
+        {
             field.SetName( _( "untitled" ) );
+        }
     }
 
     m_libEntry->SetFields( *m_fields );
@@ -420,20 +464,16 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataFromWindow()
         // Verify that the requested parent exists
         wxCHECK( newParent, false );
 
-        // Verify that the new parent is not an alias.
-        wxCHECK( !newParent->IsAlias(), false );
-
         m_libEntry->SetParent( newParent );
     }
 
     m_libEntry->SetName( newName );
-    m_libEntry->SetDescription( m_DescCtrl->GetValue() );
     m_libEntry->SetKeyWords( m_KeywordCtrl->GetValue() );
     m_libEntry->SetUnitCount( m_SelNumberOfUnits->GetValue() );
     m_libEntry->LockUnits( m_libEntry->GetUnitCount() > 1 &&
                            !m_OptionPartsInterchangeable->GetValue() );
-    m_libEntry->SetConversion( m_AsConvertButt->GetValue() );
-    m_Parent->SetShowDeMorgan( m_AsConvertButt->GetValue() );
+    m_libEntry->SetHasAlternateBodyStyle( m_hasAlternateBodyStyles->GetValue() );
+    m_Parent->SetShowDeMorgan( m_hasAlternateBodyStyles->GetValue() );
 
     if( m_OptionPower->GetValue() )
     {
@@ -446,8 +486,9 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataFromWindow()
         m_libEntry->SetNormal();
     }
 
-    m_libEntry->SetIncludeInBom( !m_excludeFromBomCheckBox->GetValue() );
-    m_libEntry->SetIncludeOnBoard( !m_excludeFromBoardCheckBox->GetValue() );
+    m_libEntry->SetExcludedFromSim( m_excludeFromSimCheckBox->GetValue() );
+    m_libEntry->SetExcludedFromBOM( m_excludeFromBomCheckBox->GetValue() );
+    m_libEntry->SetExcludedFromBoard( m_excludeFromBoardCheckBox->GetValue() );
 
     m_libEntry->SetShowPinNumbers( m_ShowPinNumButt->GetValue() );
     m_libEntry->SetShowPinNames( m_ShowPinNameButt->GetValue() );
@@ -469,9 +510,10 @@ bool DIALOG_LIB_SYMBOL_PROPERTIES::TransferDataFromWindow()
     m_Parent->UpdateAfterSymbolProperties( &oldName );
 
     // It's possible that the symbol being edited has no pins, in which case there may be no
-    // alternate body style objects causing #LIB_SYMBOL::HasCoversion() to always return false.
-    // This allows the user to edit the alternate body style just in case this condition occurs.
-    m_Parent->SetShowDeMorgan( m_AsConvertButt->GetValue() );
+    // alternate body style objects causing #LIB_SYMBOL::HasAlternateBodyStyle() to always return
+    // false.  This allows the user to edit the alternate body style just in case this condition
+    // occurs.
+    m_Parent->SetShowDeMorgan( m_hasAlternateBodyStyles->GetValue() );
 
     return true;
 }
@@ -519,6 +561,8 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnSymbolNameText( wxCommandEvent& event )
 {
     if( m_OptionPower->IsChecked() )
         m_grid->SetCellValue( VALUE_FIELD, FDC_VALUE, m_SymbolNameCtrl->GetValue() );
+
+    OnModify();
 }
 
 
@@ -550,10 +594,11 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnAddField( wxCommandEvent& event )
 
     SYMBOL_EDITOR_SETTINGS* settings = m_Parent->GetSettings();
     int       fieldID = (int) m_fields->size();
-    LIB_FIELD newField( m_libEntry, fieldID );
+    SCH_FIELD newField( m_libEntry, fieldID );
 
-    newField.SetTextSize( wxSize( schIUScale.MilsToIU( settings->m_Defaults.text_size ),
-                                  schIUScale.MilsToIU( settings->m_Defaults.text_size ) ) );
+    newField.SetTextSize( VECTOR2I( schIUScale.MilsToIU( settings->m_Defaults.text_size ),
+                                    schIUScale.MilsToIU( settings->m_Defaults.text_size ) ) );
+    newField.SetVisible( false );
 
     m_fields->push_back( newField );
 
@@ -583,15 +628,16 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnDeleteField( wxCommandEvent& event )
 
     for( int row : selectedRows )
     {
-        if( row < MANDATORY_FIELDS )
+        if( row < m_fields->GetMandatoryRowCount() )
         {
             DisplayError( this, wxString::Format( _( "The first %d fields are mandatory." ),
-                                                  MANDATORY_FIELDS ) );
+                                                  m_fields->GetMandatoryRowCount() ) );
             return;
         }
     }
 
     m_grid->CommitPendingChanges( true /* quiet mode */ );
+    m_grid->ClearSelection();
 
     // Reverse sort so deleting a row doesn't change the indexes of the other rows.
     selectedRows.Sort( []( int* first, int* second ) { return *second - *first; } );
@@ -622,9 +668,9 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnMoveUp( wxCommandEvent& event )
 
     int i = m_grid->GetGridCursorRow();
 
-    if( i > MANDATORY_FIELDS )
+    if( i > m_fields->GetMandatoryRowCount() )
     {
-        LIB_FIELD tmp = m_fields->at( (unsigned) i );
+        SCH_FIELD tmp = m_fields->at( (unsigned) i );
         m_fields->erase( m_fields->begin() + i, m_fields->begin() + i + 1 );
         m_fields->insert( m_fields->begin() + i - 1, tmp );
         m_grid->ForceRefresh();
@@ -648,9 +694,9 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnMoveDown( wxCommandEvent& event )
 
     int i = m_grid->GetGridCursorRow();
 
-    if( i >= MANDATORY_FIELDS && i + 1 < m_fields->GetNumberRows() )
+    if( i >= m_fields->GetMandatoryRowCount() && i + 1 < m_fields->GetNumberRows() )
     {
-        LIB_FIELD tmp = m_fields->at( (unsigned) i );
+        SCH_FIELD tmp = m_fields->at( (unsigned) i );
         m_fields->erase( m_fields->begin() + i, m_fields->begin() + i + 1 );
         m_fields->insert( m_fields->begin() + i + 1, tmp );
         m_grid->ForceRefresh();
@@ -669,26 +715,25 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnMoveDown( wxCommandEvent& event )
 
 void DIALOG_LIB_SYMBOL_PROPERTIES::OnEditSpiceModel( wxCommandEvent& event )
 {
-#ifdef KICAD_SPICE
     if( !m_grid->CommitPendingChanges() )
         return;
 
-    std::vector<LIB_FIELD> fields;
+    std::vector<SCH_FIELD> fields;
 
-    for( const LIB_FIELD& field : *m_fields )
+    for( const SCH_FIELD& field : *m_fields )
         fields.emplace_back( field );
 
-    DIALOG_SIM_MODEL dialog( this, *m_libEntry, fields );
+    DIALOG_SIM_MODEL dialog( this, m_parentFrame, *m_libEntry, fields );
 
     if( dialog.ShowModal() != wxID_OK )
         return;
 
     // Add in any new fields
-    for( const LIB_FIELD& editedField : fields )
+    for( const SCH_FIELD& editedField : fields )
     {
         bool found = false;
 
-        for( LIB_FIELD& existingField : *m_fields )
+        for( SCH_FIELD& existingField : *m_fields )
         {
             if( existingField.GetName() == editedField.GetName() )
             {
@@ -707,12 +752,12 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnEditSpiceModel( wxCommandEvent& event )
     }
 
     // Remove any deleted fields
-    for( int ii = (int) m_fields->size() - 1; ii >= 0; /* advance in loop */ )
+    for( int ii = (int) m_fields->size() - 1; ii >= 0; --ii )
     {
-        LIB_FIELD& existingField = m_fields->at( ii );
+        SCH_FIELD& existingField = m_fields->at( ii );
         bool       found = false;
 
-        for( LIB_FIELD& editedField : fields )
+        for( SCH_FIELD& editedField : fields )
         {
             if( editedField.GetName() == existingField.GetName() )
             {
@@ -721,11 +766,7 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnEditSpiceModel( wxCommandEvent& event )
             }
         }
 
-        if( found )
-        {
-            ii--;
-        }
-        else
+        if( !found )
         {
             m_fields->erase( m_fields->begin() + ii );
             wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_DELETED, ii, 1 );
@@ -735,13 +776,12 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnEditSpiceModel( wxCommandEvent& event )
 
     OnModify();
     m_grid->ForceRefresh();
-#endif /* KICAD_SPICE */
 }
 
 
-void DIALOG_LIB_SYMBOL_PROPERTIES::OnFilterDClick( wxMouseEvent& event )
+void DIALOG_LIB_SYMBOL_PROPERTIES::OnFpFilterDClick( wxMouseEvent& event )
 {
-    int idx = m_FootprintFilterListBox->HitTest( event.GetPosition() );
+    int            idx = m_FootprintFilterListBox->HitTest( event.GetPosition() );
     wxCommandEvent dummy;
 
     if( idx >= 0 )
@@ -771,7 +811,6 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnAddFootprintFilter( wxCommandEvent& event )
     filterLine.Replace( wxT( " " ), wxT( "_" ) );
 
     // duplicate filters do no harm, so don't be a nanny.
-
     m_FootprintFilterListBox->Append( filterLine );
     m_FootprintFilterListBox->SetSelection( (int) m_FootprintFilterListBox->GetCount() - 1 );
 
@@ -779,31 +818,16 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnAddFootprintFilter( wxCommandEvent& event )
 }
 
 
-void DIALOG_LIB_SYMBOL_PROPERTIES::OnDeleteFootprintFilter( wxCommandEvent& event )
-{
-    int ii = m_FootprintFilterListBox->GetSelection();
-
-    if( ii >= 0 )
-    {
-        m_FootprintFilterListBox->Delete( (unsigned) ii );
-
-        if( m_FootprintFilterListBox->GetCount() == 0 )
-            m_FootprintFilterListBox->SetSelection( wxNOT_FOUND );
-        else
-            m_FootprintFilterListBox->SetSelection( std::max( 0, ii - 1 ) );
-    }
-
-    OnModify();
-}
-
-
 void DIALOG_LIB_SYMBOL_PROPERTIES::OnEditFootprintFilter( wxCommandEvent& event )
 {
-    int idx = m_FootprintFilterListBox->GetSelection();
+    wxArrayInt selections;
+    int n = m_FootprintFilterListBox->GetSelections( selections );
 
-    if( idx >= 0 )
+    if( n > 0 )
     {
-        wxString filter = m_FootprintFilterListBox->GetStringSelection();
+        // Just edit the first one
+        int idx = selections[0];
+        wxString filter = m_FootprintFilterListBox->GetString( idx );
 
         WX_TEXT_ENTRY_DIALOG dlg( this, _( "Filter:" ), _( "Edit Footprint Filter" ), filter );
 
@@ -906,21 +930,6 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnUpdateUI( wxUpdateUIEvent& event )
         m_delayedFocusRow = -1;
         m_delayedFocusColumn = -1;
     }
-
-#ifdef KICAD_SPICE
-    wxString simEnable;
-
-    for( int ii = MANDATORY_FIELDS; ii < m_fields->GetNumberRows(); ++ii )
-    {
-        if( m_fields->GetValue( ii, FDC_NAME ) == SIM_ENABLE_FIELD )
-        {
-            simEnable = m_fields->GetValue( ii, FDC_VALUE );
-            break;
-        }
-    }
-
-    m_excludeFromSim->SetValue( simEnable == wxS( "0" ) );
-#endif
 }
 
 
@@ -953,15 +962,20 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::onPowerCheckBox( wxCommandEvent& aEvent )
 {
     if( m_OptionPower->IsChecked() )
     {
+        m_excludeFromSimCheckBox->SetValue( true );
         m_excludeFromBomCheckBox->SetValue( true );
         m_excludeFromBoardCheckBox->SetValue( true );
         m_excludeFromBomCheckBox->Enable( false );
         m_excludeFromBoardCheckBox->Enable( false );
+        m_excludeFromSimCheckBox->Enable( false );
+        m_spiceFieldsButton->Show( false );
     }
     else
     {
         m_excludeFromBomCheckBox->Enable( true );
         m_excludeFromBoardCheckBox->Enable( true );
+        m_excludeFromSimCheckBox->Enable( true );
+        m_spiceFieldsButton->Show( true );
     }
 
     OnModify();
@@ -998,3 +1012,8 @@ void DIALOG_LIB_SYMBOL_PROPERTIES::OnSpinCtrlText( wxCommandEvent& event )
 }
 
 
+void DIALOG_LIB_SYMBOL_PROPERTIES::OnPageChanging( wxBookCtrlEvent& aEvent )
+{
+    if( !m_grid->CommitPendingChanges() )
+        aEvent.Veto();
+}
