@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2022 Mikolaj Wielgus
- * Copyright (C) 2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,7 +32,7 @@
 
 #include <sim/sim_model.h>
 #include <sim/sim_library.h>
-#include <sim/sim_library_kibis.h>
+#include <sim/sim_library_ibis.h>
 #include <sch_symbol.h>
 
 
@@ -41,7 +41,7 @@
 //    wxPG_NATIVE_DOUBLE_BUFFERING flag is not set.
 // 2. wxPropertyGridManager->ShowHeader() segfaults when called from this dialog's constructor.
 
-template <typename T_symbol, typename T_field>
+template <typename T>
 class DIALOG_SIM_MODEL : public DIALOG_SIM_MODEL_BASE
 {
 public:
@@ -61,7 +61,8 @@ public:
         MODEL
     };
 
-    DIALOG_SIM_MODEL( wxWindow* aParent, T_symbol& aSymbol, std::vector<T_field>& aFields );
+    DIALOG_SIM_MODEL( wxWindow* aParent, EDA_BASE_FRAME* aFrame, T& aSymbol,
+                      std::vector<SCH_FIELD>& aFields );
 
     ~DIALOG_SIM_MODEL();
 
@@ -71,14 +72,15 @@ private:
 
     void updateWidgets();
     void updateIbisWidgets( SIM_MODEL* aModel );
-    void updateInstanceWidgets( SIM_MODEL* aModel );
+    void updateBuiltinModelWidgets( SIM_MODEL* aModel );
     void updateModelParamsTab( SIM_MODEL* aModel );
     void updateModelCodeTab( SIM_MODEL* aModel );
-    void updatePinAssignments( SIM_MODEL* aModel );
+    void updatePinAssignments( SIM_MODEL* aModel, bool aForceUpdatePins );
 
     void removeOrphanedPinAssignments( SIM_MODEL* aModel );
 
-    bool loadLibrary( const wxString& aLibraryPath, bool aForceReload = false );
+    bool loadLibrary( const wxString& aLibraryPath, REPORTER& aReporter,
+                      bool aForceReload = false );
 
     void addParamPropertyIfRelevant( SIM_MODEL* aModel, int aParamIndex );
     wxPGProperty* newParamProperty( SIM_MODEL* aModel, int aParamIndex ) const;
@@ -93,15 +95,19 @@ private:
     int getModelPinIndex( const wxString& aModelPinString ) const;
 
     void onRadioButton( wxCommandEvent& aEvent ) override;
+    void onLibraryPathText( wxCommandEvent& aEvent ) override;
     void onLibraryPathTextEnter( wxCommandEvent& aEvent ) override;
     void onLibraryPathTextKillFocus( wxFocusEvent& aEvent ) override;
     void onBrowseButtonClick( wxCommandEvent& aEvent ) override;
+    void onFilterCharHook( wxKeyEvent& aKeyStroke ) override;
+    void onModelFilter( wxCommandEvent& aEvent ) override;
     void onModelNameChoice( wxCommandEvent& aEvent ) override;
-    void onIbisPinCombobox( wxCommandEvent& event ) override;
-    void onIbisPinComboboxTextEnter( wxCommandEvent& event ) override;
-    void onIbisModelCombobox( wxCommandEvent& event ) override;
-    void onIbisModelComboboxTextEnter( wxCommandEvent& event ) override;
+    void onPinCombobox( wxCommandEvent& event ) override;
+    void onPinComboboxTextEnter( wxCommandEvent& event ) override;
+    void onPinModelCombobox( wxCommandEvent& event ) override;
+    void onPinModelComboboxTextEnter( wxCommandEvent& event ) override;
     void onDeviceTypeChoice( wxCommandEvent& aEvent ) override;
+    void onWaveformChoice( wxCommandEvent& aEvent ) override;
     void onTypeChoice( wxCommandEvent& aEvent ) override;
     void onPageChanging( wxNotebookEvent& event ) override;
     void onPinAssignmentsGridCellChange( wxGridEvent& aEvent ) override;
@@ -115,28 +121,31 @@ private:
 
     void adjustParamGridColumns( int aWidth, bool aForce );
 
-    bool isIbisLoaded() { return dynamic_cast<const SIM_LIBRARY_KIBIS*>( library() ); }
+    bool isIbisLoaded() { return dynamic_cast<const SIM_LIBRARY_IBIS*>( library() ); }
 
 private:
-    T_symbol&              m_symbol;
-    std::vector<T_field>&  m_fields;
+    EDA_BASE_FRAME*         m_frame;
+    T&                      m_symbol;
+    std::vector<SCH_FIELD>& m_fields;
 
-    SIM_LIB_MGR            m_libraryModelsMgr;
-    SIM_LIB_MGR            m_builtinModelsMgr;
-    wxString               m_prevLibrary;
-    const SIM_MODEL*       m_prevModel;
+    SIM_LIB_MGR             m_libraryModelsMgr;
+    SIM_LIB_MGR             m_builtinModelsMgr;
+    wxString                m_prevLibrary;
+    const SIM_MODEL*        m_prevModel;
 
-    std::vector<LIB_PIN*>                          m_sortedPartPins; //< Pins of the current part.
+    std::map<wxString, int> m_modelListBoxEntryToLibraryIdx;
+
+    std::vector<SCH_PIN*>                          m_sortedPartPins; ///< Pins of the current part.
     std::map<SIM_MODEL::DEVICE_T, SIM_MODEL::TYPE> m_curModelTypeOfDeviceType;
     SIM_MODEL::TYPE                                m_curModelType;
 
-    SCINTILLA_TRICKS*      m_scintillaTricks;
+    SCINTILLA_TRICKS*       m_scintillaTricksCode;
+    SCINTILLA_TRICKS*       m_scintillaTricksSubckt;
 
-    wxPGProperty*          m_firstCategory;            // Used to add principal parameters to root.
-    wxPGProperty*          m_prevParamGridSelection;
+    wxPGProperty*           m_firstCategory;            // Used to add principal parameters to root.
+    wxPGProperty*           m_prevParamGridSelection;
 
-    int                    m_lastParamGridWidth;
-    bool                   m_inKillFocus;
+    int                     m_lastParamGridWidth;
 };
 
 #endif /* DIALOG_SIM_MODEL_H */
