@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2013 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2013 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,25 +24,23 @@
  */
 
 #include <sch_edit_frame.h>
+#include <symbol_edit_frame.h>
 #include <widgets/bitmap_button.h>
 #include <widgets/font_choice.h>
 #include <widgets/color_swatch.h>
 #include <widgets/wx_combobox.h>
-#include <base_units.h>
 #include <settings/color_settings.h>
-#include <tool/tool_manager.h>
-#include <general.h>
 #include <sch_textbox.h>
 #include <confirm.h>
 #include <schematic.h>
+#include <sch_commit.h>
 #include <dialogs/html_message_box.h>
-#include <string_utils.h>
 #include <scintilla_tricks.h>
 #include <dialog_text_properties.h>
-#include <gr_text.h>
+#include <string_utils.h>
 
 
-DIALOG_TEXT_PROPERTIES::DIALOG_TEXT_PROPERTIES( SCH_EDIT_FRAME* aParent, SCH_ITEM* aTextItem ) :
+DIALOG_TEXT_PROPERTIES::DIALOG_TEXT_PROPERTIES( SCH_BASE_FRAME* aParent, SCH_ITEM* aTextItem ) :
         DIALOG_TEXT_PROPERTIES_BASE( aParent ),
         m_frame( aParent ),
         m_currentItem( aTextItem ),
@@ -52,6 +50,8 @@ DIALOG_TEXT_PROPERTIES::DIALOG_TEXT_PROPERTIES( SCH_EDIT_FRAME* aParent, SCH_ITE
         m_scintillaTricks( nullptr ),
         m_helpWindow( nullptr )
 {
+    m_isSymbolEditor = dynamic_cast<SYMBOL_EDIT_FRAME*>( aParent ) != nullptr;
+
     COLOR_SETTINGS* colorSettings = m_frame->GetColorSettings();
     COLOR4D         schematicBackground = colorSettings->GetColor( LAYER_SCHEMATIC_BACKGROUND );
 
@@ -62,33 +62,18 @@ DIALOG_TEXT_PROPERTIES::DIALOG_TEXT_PROPERTIES( SCH_EDIT_FRAME* aParent, SCH_ITE
         m_borderColorSwatch->SetDefaultColor( COLOR4D::UNSPECIFIED );
         m_borderColorSwatch->SetSwatchBackground( schematicBackground );
 
-        for( const std::pair<const PLOT_DASH_TYPE, lineTypeStruct>& typeEntry : lineTypeNames )
-            m_borderStyleCombo->Append( typeEntry.second.name, KiBitmap( typeEntry.second.bitmap ) );
-
-        m_borderStyleCombo->Append( DEFAULT_STYLE );
+        for( const auto& [ lineStyle, lineStyleDesc ] : lineTypeNames )
+            m_borderStyleCombo->Append( lineStyleDesc.name, KiBitmapBundle( lineStyleDesc.bitmap ) );
 
         m_fillColorSwatch->SetDefaultColor( COLOR4D::UNSPECIFIED );
         m_fillColorSwatch->SetSwatchBackground( schematicBackground );
 
         if( m_frame->GetColorSettings()->GetOverrideSchItemColors() )
-            m_infoBar->ShowMessage( _( "Note: individual item colors overridden in Preferences." ) );
+            m_infoBar->ShowMessage( _( "Note: individual item colors overridden in "
+                                       "Preferences." ) );
     }
     else
     {
-        m_hAlignCenter->Show( false );
-        m_separator3->Show( false );
-        m_vAlignTop->Show( false );
-        m_vAlignCenter->Show( false );
-        m_vAlignBottom->Show( false );
-
-        wxSizer* parentSizer = m_vAlignTop->GetContainingSizer();
-        parentSizer->Detach( m_hAlignCenter );
-        parentSizer->Detach( m_separator3 );
-        parentSizer->Detach( m_vAlignTop );
-        parentSizer->Detach( m_vAlignCenter );
-        parentSizer->Detach( m_vAlignBottom );
-        parentSizer->Layout();
-
         m_borderCheckbox->Show( false );
         m_borderWidth.Show( false );
         m_borderColorLabel->Show( false );
@@ -100,12 +85,36 @@ DIALOG_TEXT_PROPERTIES::DIALOG_TEXT_PROPERTIES( SCH_EDIT_FRAME* aParent, SCH_ITE
         m_filledCtrl->Show( false );
     }
 
+    // DIALOG_SHIM needs a unique hash_key because classname is not sufficient because the
+    // different text item types (and even whether or not we're within the symbol editor) cause
+    // different dialog layouts).
+    m_hash_key = TO_UTF8( GetTitle() + aParent->GetName() );
+
     m_textCtrl->SetEOLMode( wxSTC_EOL_LF );
 
+#ifdef _WIN32
+    // Without this setting, on Windows, some esoteric unicode chars create display issue
+    // in a wxStyledTextCtrl.
+    // for SetTechnology() info, see https://www.scintilla.org/ScintillaDoc.html#SCI_SETTECHNOLOGY
+    m_textCtrl->SetTechnology( wxSTC_TECHNOLOGY_DIRECTWRITE );
+#endif
+
     m_scintillaTricks = new SCINTILLA_TRICKS( m_textCtrl, wxT( "{}" ), false,
-            [this]()
+            // onAcceptFn
+            [this]( wxKeyEvent& aEvent )
             {
                 wxPostEvent( this, wxCommandEvent( wxEVT_COMMAND_BUTTON_CLICKED, wxID_OK ) );
+            },
+
+            // onCharFn
+            [this]( wxStyledTextEvent& aEvent )
+            {
+                m_scintillaTricks->DoTextVarAutocomplete(
+                        // getTokensFn
+                        [this]( const wxString& xRef, wxArrayString* tokens )
+                        {
+                            getContextualTextVars( xRef, tokens );
+                        } );
             } );
 
     m_textEntrySizer->AddGrowableRow( 0 );
@@ -118,58 +127,72 @@ DIALOG_TEXT_PROPERTIES::DIALOG_TEXT_PROPERTIES( SCH_EDIT_FRAME* aParent, SCH_ITE
     m_separator1->SetIsSeparator();
 
     m_bold->SetIsCheckButton();
-    m_bold->SetBitmap( KiBitmap( BITMAPS::text_bold ) );
+    m_bold->SetBitmap( KiBitmapBundle( BITMAPS::text_bold ) );
     m_italic->SetIsCheckButton();
-    m_italic->SetBitmap( KiBitmap( BITMAPS::text_italic ) );
+    m_italic->SetBitmap( KiBitmapBundle( BITMAPS::text_italic ) );
 
     m_separator2->SetIsSeparator();
 
     m_hAlignLeft->SetIsRadioButton();
-    m_hAlignLeft->SetBitmap( KiBitmap( BITMAPS::text_align_left ) );
+    m_hAlignLeft->SetBitmap( KiBitmapBundle( BITMAPS::text_align_left ) );
     m_hAlignCenter->SetIsRadioButton();
-    m_hAlignCenter->SetBitmap( KiBitmap( BITMAPS::text_align_center ) );
+    m_hAlignCenter->SetBitmap( KiBitmapBundle( BITMAPS::text_align_center ) );
     m_hAlignRight->SetIsRadioButton();
-    m_hAlignRight->SetBitmap( KiBitmap( BITMAPS::text_align_right ) );
+    m_hAlignRight->SetBitmap( KiBitmapBundle( BITMAPS::text_align_right ) );
+
     m_separator3->SetIsSeparator();
 
     m_vAlignTop->SetIsRadioButton();
-    m_vAlignTop->SetBitmap( KiBitmap( BITMAPS::text_valign_top ) );
+    m_vAlignTop->SetBitmap( KiBitmapBundle( BITMAPS::text_valign_top ) );
     m_vAlignCenter->SetIsRadioButton();
-    m_vAlignCenter->SetBitmap( KiBitmap( BITMAPS::text_valign_center ) );
+    m_vAlignCenter->SetBitmap( KiBitmapBundle( BITMAPS::text_valign_center ) );
     m_vAlignBottom->SetIsRadioButton();
-    m_vAlignBottom->SetBitmap( KiBitmap( BITMAPS::text_valign_bottom ) );
+    m_vAlignBottom->SetBitmap( KiBitmapBundle( BITMAPS::text_valign_bottom ) );
 
     m_separator4->SetIsSeparator();
 
     m_horizontal->SetIsRadioButton();
-    m_horizontal->SetBitmap( KiBitmap( BITMAPS::text_horizontal ) );
+    m_horizontal->SetBitmap( KiBitmapBundle( BITMAPS::text_horizontal ) );
     m_vertical->SetIsRadioButton();
-    m_vertical->SetBitmap( KiBitmap( BITMAPS::text_vertical ) );
+    m_vertical->SetBitmap( KiBitmapBundle( BITMAPS::text_vertical ) );
 
     m_separator5->SetIsSeparator();
 
-    SCH_SHEET_LIST sheetList = m_frame->Schematic().GetSheets();
-    sheetList.SortByPageNumbers( false );
+    m_fgSymbolEditor->Show( m_isSymbolEditor );
 
-    for( const SCH_SHEET_PATH& sheet : sheetList )
+    if( SCH_EDIT_FRAME* schematicEditor = dynamic_cast<SCH_EDIT_FRAME*>( m_frame ) )
     {
-        wxString sheetPageNum = sheet.GetPageNumber();
-        wxString sheetName = sheet.size() == 1 ? _( "<root sheet>" ) : sheet.Last()->GetName();
+        const SCHEMATIC& schematic = schematicEditor->Schematic();
 
-        m_hyperlinkCombo->Append( wxT( "#" ) + sheetPageNum,
-                                  wxString::Format( _( "Page %s (%s)" ), sheetPageNum, sheetName ) );
-        m_pageNumbers.push_back( sheetPageNum );
+        for( const SCH_SHEET_PATH& sheet : schematic.Hierarchy() )
+        {
+            wxString sheetPageNum = sheet.GetPageNumber();
+            wxString sheetName = sheet.size() == 1 ? _( "<root sheet>" )
+                                                   : sheet.Last()->GetName();
+
+            m_hyperlinkCombo->Append( wxT( "#" ) + sheetPageNum,
+                                      wxString::Format( _( "Page %s (%s)" ),
+                                                        sheetPageNum,
+                                                        sheetName ) );
+            m_pageNumbers.push_back( sheetPageNum );
+        }
+
+        m_hyperlinkCombo->Append( wxT( "---" ) );
+        m_hyperlinkCombo->Append( wxT( "file://" ), wxT( "file://..." ) );
+        m_hyperlinkCombo->Append( wxT( "http://" ), wxT( "http://..." ) );
+        m_hyperlinkCombo->Append( wxT( "https://" ), wxT( "https://..." ) );
+    }
+    else
+    {
+        m_excludeFromSim->Hide();
+        m_syntaxHelp->Hide();
+        m_hyperlinkCb->Hide();
+        m_hyperlinkCombo->Hide();
     }
-
-    m_hyperlinkCombo->Append( wxT( "---" ) );
-    m_hyperlinkCombo->Append( wxT( "file://" ), wxT( "file://..." ) );
-    m_hyperlinkCombo->Append( wxT( "http://" ), wxT( "http://..." ) );
-    m_hyperlinkCombo->Append( wxT( "https://" ), wxT( "https://..." ) );
 
     SetupStandardButtons();
     Layout();
 
-    m_textCtrl->Bind( wxEVT_STC_CHARADDED, &DIALOG_TEXT_PROPERTIES::onScintillaCharAdded, this );
     m_hAlignLeft->Bind( wxEVT_BUTTON, &DIALOG_TEXT_PROPERTIES::onHAlignButton, this );
     m_hAlignCenter->Bind( wxEVT_BUTTON, &DIALOG_TEXT_PROPERTIES::onHAlignButton, this );
     m_hAlignRight->Bind( wxEVT_BUTTON, &DIALOG_TEXT_PROPERTIES::onHAlignButton, this );
@@ -193,20 +216,70 @@ DIALOG_TEXT_PROPERTIES::~DIALOG_TEXT_PROPERTIES()
 }
 
 
+void DIALOG_TEXT_PROPERTIES::getContextualTextVars( const wxString& aCrossRef,
+                                                    wxArrayString*  aTokens )
+{
+    SCHEMATIC* schematic = m_currentItem->Schematic();
+
+    if( !aCrossRef.IsEmpty() )
+    {
+        SCH_SYMBOL* refSymbol = nullptr;
+
+        if( schematic )
+        {
+            SCH_REFERENCE_LIST refs;
+            schematic->Hierarchy().GetSymbols( refs );
+
+            for( int jj = 0; jj < (int) refs.GetCount(); jj++ )
+            {
+                SCH_REFERENCE& ref = refs[jj];
+
+                if( ref.GetSymbol()->GetRef( &ref.GetSheetPath(), true ) == aCrossRef )
+                {
+                    refSymbol = ref.GetSymbol();
+                    break;
+                }
+            }
+        }
+
+        if( refSymbol )
+            refSymbol->GetContextualTextVars( aTokens );
+    }
+    else
+    {
+        if( schematic && schematic->CurrentSheet().Last() )
+        {
+            schematic->CurrentSheet().Last()->GetContextualTextVars( aTokens );
+        }
+        else
+        {
+            for( std::pair<wxString, wxString> entry : Prj().GetTextVars() )
+                aTokens->push_back( entry.first );
+        }
+    }
+}
+
+
 bool DIALOG_TEXT_PROPERTIES::TransferDataToWindow()
 {
     if( !wxDialog::TransferDataToWindow() )
         return false;
 
-    SCHEMATIC& schematic = m_frame->Schematic();
-
     m_hyperlinkCb->SetValue( m_currentText->HasHyperlink() );
     m_hyperlinkCombo->SetValue( m_currentText->GetHyperlink() );
 
+    wxString text = m_currentText->GetText();
+
     // show text variable cross-references in a human-readable format
-    m_textCtrl->SetValue( schematic.ConvertKIIDsToRefs( m_currentText->GetText() ) );
+    if( SCHEMATIC* schematic = m_currentItem->Schematic() )
+        text = schematic->ConvertKIIDsToRefs( text );
+
+    m_textCtrl->SetValue( text );
     m_textCtrl->EmptyUndoBuffer();
 
+    if( !m_isSymbolEditor )
+        m_excludeFromSim->SetValue( m_currentItem->GetExcludedFromSim() );
+
     m_fontCtrl->SetFontSelection( m_currentText->GetFont() );
     m_textSize.SetValue( m_currentText->GetTextWidth() );
     m_textColorSwatch->SetSwatchColor( m_currentText->GetTextColor(), false );
@@ -214,29 +287,31 @@ bool DIALOG_TEXT_PROPERTIES::TransferDataToWindow()
     m_bold->Check( m_currentText->IsBold() );
     m_italic->Check( m_currentText->IsItalic() );
 
+    switch( m_currentText->GetHorizJustify() )
+    {
+    case GR_TEXT_H_ALIGN_LEFT:          m_hAlignLeft->Check();   break;
+    case GR_TEXT_H_ALIGN_CENTER:        m_hAlignCenter->Check(); break;
+    case GR_TEXT_H_ALIGN_RIGHT:         m_hAlignRight->Check();  break;
+    case GR_TEXT_H_ALIGN_INDETERMINATE:                          break;
+    }
+
+    switch( m_currentText->GetVertJustify() )
+    {
+    case GR_TEXT_V_ALIGN_TOP:           m_vAlignTop->Check();    break;
+    case GR_TEXT_V_ALIGN_CENTER:        m_vAlignCenter->Check(); break;
+    case GR_TEXT_V_ALIGN_BOTTOM:        m_vAlignBottom->Check(); break;
+    case GR_TEXT_V_ALIGN_INDETERMINATE:                          break;
+    }
+
+    if( m_currentText->GetTextAngle() == ANGLE_VERTICAL )
+        m_vertical->Check();
+    else
+        m_horizontal->Check();
+
     if( m_currentItem->Type() == SCH_TEXTBOX_T )
     {
         SCH_TEXTBOX* textBox = static_cast<SCH_TEXTBOX*>( m_currentItem );
 
-        switch( m_currentText->GetHorizJustify() )
-        {
-        case GR_TEXT_H_ALIGN_LEFT:   m_hAlignLeft->Check();   break;
-        case GR_TEXT_H_ALIGN_CENTER: m_hAlignCenter->Check(); break;
-        case GR_TEXT_H_ALIGN_RIGHT:  m_hAlignRight->Check();  break;
-        }
-
-        switch( m_currentText->GetVertJustify() )
-        {
-        case GR_TEXT_V_ALIGN_TOP:    m_vAlignTop->Check();    break;
-        case GR_TEXT_V_ALIGN_CENTER: m_vAlignCenter->Check(); break;
-        case GR_TEXT_V_ALIGN_BOTTOM: m_vAlignBottom->Check(); break;
-        }
-
-        if( m_currentText->GetTextAngle() == ANGLE_VERTICAL )
-            m_vertical->Check();
-        else
-            m_horizontal->Check();
-
         m_borderCheckbox->SetValue( textBox->GetWidth() >= 0 );
 
         if( textBox->GetWidth() >= 0 )
@@ -244,14 +319,12 @@ bool DIALOG_TEXT_PROPERTIES::TransferDataToWindow()
 
         m_borderColorSwatch->SetSwatchColor( textBox->GetStroke().GetColor(), false );
 
-        int style = static_cast<int>( textBox->GetStroke().GetPlotStyle() );
+        int style = static_cast<int>( textBox->GetStroke().GetLineStyle() );
 
-        if( style == -1 )
-            m_borderStyleCombo->SetStringSelection( DEFAULT_STYLE );
-        else if( style < (int) lineTypeNames.size() )
+        if( style >= 0 && style < (int) lineTypeNames.size() )
             m_borderStyleCombo->SetSelection( style );
         else
-            wxFAIL_MSG( "Line type not found in the type lookup map" );
+            m_borderStyleCombo->SetSelection( 0 );
 
         m_borderWidth.Enable( textBox->GetWidth() >= 0 );
         m_borderColorLabel->Enable( textBox->GetWidth() >= 0 );
@@ -265,27 +338,17 @@ bool DIALOG_TEXT_PROPERTIES::TransferDataToWindow()
         m_fillColorLabel->Enable( textBox->IsFilled() );
         m_fillColorSwatch->Enable( textBox->IsFilled() );
     }
-    else
+
+    if( m_isSymbolEditor )
     {
-        switch( static_cast<SCH_TEXT*>( m_currentItem )->GetTextSpinStyle() )
-        {
-        case TEXT_SPIN_STYLE::RIGHT:
-            m_hAlignLeft->Check( true );    // Spin style to right means text aligned left
-            m_horizontal->Check( true );
-            break;
-        case TEXT_SPIN_STYLE::LEFT:
-            m_hAlignRight->Check( true );   // Spin style to left means text aligned right
-            m_horizontal->Check( true );
-            break;
-        case TEXT_SPIN_STYLE::UP:
-            m_hAlignLeft->Check( true );  // Spin style up means text aligned to bottom
-            m_vertical->Check( true );
-            break;
-        case TEXT_SPIN_STYLE::BOTTOM:       // Spin style down means text aligned to top
-            m_hAlignRight->Check( true );
-            m_vertical->Check( true );
-            break;
-        }
+        SYMBOL* symbol = m_currentItem->GetParentSymbol();
+
+        m_privateCheckbox->SetValue( m_currentItem->IsPrivate() );
+        m_commonToAllUnits->SetValue( symbol->IsMulti() && m_currentItem->GetUnit() == 0 );
+        m_commonToAllUnits->Enable( symbol->IsMulti() );
+        m_commonToAllBodyStyles->SetValue( symbol->HasAlternateBodyStyle()
+                                        && m_currentItem->GetBodyStyle() == 0 );
+        m_commonToAllBodyStyles->Enable( symbol->HasAlternateBodyStyle() );
     }
 
     return true;
@@ -349,67 +412,6 @@ void DIALOG_TEXT_PROPERTIES::onHyperlinkCombo( wxCommandEvent& aEvent )
 }
 
 
-void DIALOG_TEXT_PROPERTIES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
-{
-    wxStyledTextCtrl* te = m_textCtrl;
-    wxArrayString     autocompleteTokens;
-    int               text_pos = te->GetCurrentPos();
-    int               start = te->WordStartPosition( text_pos, true );
-    wxString          partial;
-
-    auto textVarRef =
-            [&]( int pos )
-            {
-                return pos >= 2 && te->GetCharAt( pos-2 ) == '$' && te->GetCharAt( pos-1 ) == '{';
-            };
-
-    // Check for cross-reference
-    if( start > 1 && te->GetCharAt( start-1 ) == ':' )
-    {
-        int refStart = te->WordStartPosition( start-1, true );
-
-        if( textVarRef( refStart ) )
-        {
-            partial = te->GetRange( start, text_pos );
-
-            wxString           ref = te->GetRange( refStart, start-1 );
-            SCH_SHEET_LIST     sheets = m_frame->Schematic().GetSheets();
-            SCH_REFERENCE_LIST refs;
-            SCH_SYMBOL*        refSymbol = nullptr;
-
-            sheets.GetSymbols( refs );
-
-            for( size_t jj = 0; jj < refs.GetCount(); jj++ )
-            {
-                if( refs[ jj ].GetSymbol()->GetRef( &refs[ jj ].GetSheetPath(), true ) == ref )
-                {
-                    refSymbol = refs[ jj ].GetSymbol();
-                    break;
-                }
-            }
-
-            if( refSymbol )
-                refSymbol->GetContextualTextVars( &autocompleteTokens );
-        }
-    }
-    else if( textVarRef( start ) )
-    {
-        partial = te->GetTextRange( start, text_pos );
-
-        SCHEMATIC* schematic = m_currentItem->Schematic();
-
-        if( schematic && schematic->CurrentSheet().Last() )
-            schematic->CurrentSheet().Last()->GetContextualTextVars( &autocompleteTokens );
-
-        for( std::pair<wxString, wxString> entry : Prj().GetTextVars() )
-            autocompleteTokens.push_back( entry.first );
-    }
-
-    m_scintillaTricks->DoAutocomplete( partial, autocompleteTokens );
-    m_textCtrl->SetFocus();
-}
-
-
 void DIALOG_TEXT_PROPERTIES::onHAlignButton( wxCommandEvent& aEvent )
 {
     for( BITMAP_BUTTON* btn : { m_hAlignLeft, m_hAlignCenter, m_hAlignRight } )
@@ -449,25 +451,25 @@ bool DIALOG_TEXT_PROPERTIES::TransferDataFromWindow()
     if( !m_textSize.Validate( 0.01, 1000.0, EDA_UNITS::MILLIMETRES ) )
         return false;
 
-    wxString text;
+    SCH_COMMIT commit( m_frame );
 
     /* save old text in undo list if not already in edit */
     if( m_currentItem->GetEditFlags() == 0 )
-    {
-        m_frame->SaveCopyInUndoList( m_frame->GetScreen(), m_currentItem, UNDO_REDO::CHANGED,
-                                     false, false );
-    }
+        commit.Modify( m_currentItem, m_frame->GetScreen() );
 
     m_frame->GetCanvas()->Refresh();
 
+    wxString text = m_textCtrl->GetValue();
+
     // convert any text variable cross-references to their UUIDs
-    text = m_frame->Schematic().ConvertRefsToKIIDs( m_textCtrl->GetValue() );
+    if( SCHEMATIC* schematic = m_currentItem->Schematic() )
+        text = schematic->ConvertRefsToKIIDs( text );
 
 #ifdef __WXMAC__
     // On macOS CTRL+Enter produces '\r' instead of '\n' regardless of EOL setting
     text.Replace( "\r", "\n" );
 #elif defined( __WINDOWS__ )
-    // On Windows, a new line is coded as \r\n.  We use only \n in kicad files and in
+    // On Windows, a new line is coded as \r\n.  We use only \n in KiCad files and in
     // drawing routines so strip the \r char.
     text.Replace( "\r", "" );
 #endif
@@ -488,7 +490,24 @@ bool DIALOG_TEXT_PROPERTIES::TransferDataFromWindow()
         return false;
     }
 
-    if( !m_currentText->ValidateHyperlink( m_hyperlinkCombo->GetValue() ) )
+    m_currentItem->SetExcludedFromSim( m_excludeFromSim->GetValue() );
+
+    if( SYMBOL_EDIT_FRAME* symbolEditor = dynamic_cast<SYMBOL_EDIT_FRAME*>( m_frame ) )
+    {
+        m_currentItem->SetPrivate( m_privateCheckbox->GetValue() );
+
+        if( !m_commonToAllUnits->GetValue() )
+            m_currentItem->SetUnit( symbolEditor->GetUnit() );
+        else
+            m_currentItem->SetUnit( 0 );
+
+        if( !m_commonToAllBodyStyles->GetValue() )
+            m_currentItem->SetBodyStyle( symbolEditor->GetBodyStyle() );
+        else
+            m_currentItem->SetBodyStyle( 0 );
+    }
+
+    if( !EDA_TEXT::ValidateHyperlink( m_hyperlinkCombo->GetValue() ) )
     {
         DisplayError( this, _( "Invalid hyperlink destination. Please enter either a valid URL "
                                "(e.g. file:// or http(s)://) or \"#<page number>\" to create "
@@ -501,7 +520,8 @@ bool DIALOG_TEXT_PROPERTIES::TransferDataFromWindow()
     }
 
     if( m_currentText->GetTextWidth() != m_textSize.GetValue() )
-        m_currentText->SetTextSize( wxSize( m_textSize.GetValue(), m_textSize.GetValue() ) );
+        m_currentText->SetTextSize( VECTOR2I( m_textSize.GetIntValue(),
+                                              m_textSize.GetIntValue() ) );
 
     if( m_fontCtrl->HaveFontSelection() )
     {
@@ -509,69 +529,39 @@ bool DIALOG_TEXT_PROPERTIES::TransferDataFromWindow()
                                                               m_italic->IsChecked() ) );
     }
 
-    if( m_bold->IsChecked() != m_currentText->IsBold() )
-    {
-        if( m_bold->IsChecked() )
-        {
-            m_currentText->SetBold( true );
-            m_currentText->SetTextThickness( GetPenSizeForBold( m_currentText->GetTextWidth() ) );
-        }
-        else
-        {
-            m_currentText->SetBold( false );
-            m_currentText->SetTextThickness( 0 ); // Use default pen width
-        }
-    }
-
+    // Must come after SetTextSize()
+    m_currentText->SetBold( m_bold->IsChecked() );
     m_currentText->SetItalic( m_italic->IsChecked() );
+
     m_currentText->SetTextColor( m_textColorSwatch->GetSwatchColor() );
 
-    if( m_currentItem->Type() == SCH_TEXT_T )
-    {
-        SCH_TEXT* textItem = static_cast<SCH_TEXT*>( m_currentItem );
-
-        if( m_hAlignRight->IsChecked() )
-        {
-            if( m_vertical->IsChecked() )
-                textItem->SetTextSpinStyle( TEXT_SPIN_STYLE::BOTTOM );
-            else
-                textItem->SetTextSpinStyle( TEXT_SPIN_STYLE::LEFT );
-        }
-        else
-        {
-            if( m_vertical->IsChecked() )
-                textItem->SetTextSpinStyle( TEXT_SPIN_STYLE::UP );
-            else
-                textItem->SetTextSpinStyle( TEXT_SPIN_STYLE::RIGHT );
-        }
-    }
+    if( m_hAlignRight->IsChecked() )
+        m_currentText->SetHorizJustify( GR_TEXT_H_ALIGN_RIGHT );
+    else if( m_hAlignCenter->IsChecked() )
+        m_currentText->SetHorizJustify( GR_TEXT_H_ALIGN_CENTER );
     else
+        m_currentText->SetHorizJustify( GR_TEXT_H_ALIGN_LEFT );
+
+    if( m_vAlignBottom->IsChecked() )
+        m_currentText->SetVertJustify( GR_TEXT_V_ALIGN_BOTTOM );
+    else if( m_vAlignCenter->IsChecked() )
+        m_currentText->SetVertJustify( GR_TEXT_V_ALIGN_CENTER );
+    else
+        m_currentText->SetVertJustify( GR_TEXT_V_ALIGN_TOP );
+
+    if( m_vertical->IsChecked() )
+        m_currentText->SetTextAngle( ANGLE_VERTICAL );
+    else
+        m_currentText->SetTextAngle( ANGLE_HORIZONTAL );
+
+    if( m_currentItem->Type() == SCH_TEXTBOX_T )
     {
         SCH_TEXTBOX* textBox = static_cast<SCH_TEXTBOX*>( m_currentItem );
 
-        if( m_hAlignRight->IsChecked() )
-            m_currentText->SetHorizJustify( GR_TEXT_H_ALIGN_RIGHT );
-        else if( m_hAlignCenter->IsChecked() )
-            m_currentText->SetHorizJustify( GR_TEXT_H_ALIGN_CENTER );
-        else
-            m_currentText->SetHorizJustify( GR_TEXT_H_ALIGN_LEFT );
-
-        if( m_vAlignBottom->IsChecked() )
-            m_currentText->SetVertJustify( GR_TEXT_V_ALIGN_BOTTOM );
-        else if( m_vAlignCenter->IsChecked() )
-            m_currentText->SetVertJustify( GR_TEXT_V_ALIGN_CENTER );
-        else
-            m_currentText->SetVertJustify( GR_TEXT_V_ALIGN_TOP );
-
-        if( m_vertical->IsChecked() )
-            m_currentText->SetTextAngle( ANGLE_VERTICAL );
-        else
-            m_currentText->SetTextAngle( ANGLE_HORIZONTAL );
-
         STROKE_PARAMS stroke = textBox->GetStroke();
 
         if( m_borderCheckbox->GetValue() )
-            stroke.SetWidth( std::max( (long long int) 0, m_borderWidth.GetValue() ) );
+            stroke.SetWidth( std::max( 0, m_borderWidth.GetIntValue() ) );
         else
             stroke.SetWidth( -1 );
 
@@ -579,9 +569,9 @@ bool DIALOG_TEXT_PROPERTIES::TransferDataFromWindow()
         std::advance( it, m_borderStyleCombo->GetSelection() );
 
         if( it == lineTypeNames.end() )
-            stroke.SetPlotStyle( PLOT_DASH_TYPE::DEFAULT );
+            stroke.SetLineStyle( LINE_STYLE::SOLID );
         else
-            stroke.SetPlotStyle( it->first );
+            stroke.SetLineStyle( it->first );
 
         stroke.SetColor( m_borderColorSwatch->GetSwatchColor() );
 
@@ -592,9 +582,8 @@ bool DIALOG_TEXT_PROPERTIES::TransferDataFromWindow()
         textBox->SetFillColor( m_fillColorSwatch->GetSwatchColor() );
     }
 
-    m_frame->UpdateItem( m_currentItem, false, true );
-    m_frame->GetCanvas()->Refresh();
-    m_frame->OnModify();
+    if( !commit.Empty() )
+        commit.Push( _( "Edit Text Properties" ) );
 
     return true;
 }
