@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2004 Jean-Pierre Charras, jaen-pierre.charras@gipsa-lab.inpg.com
  * Copyright (C) 2008 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2004-2020 KiCad Developers, see change_log.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,6 +25,7 @@
 
 #include <pgm_base.h>
 #include <kiface_base.h>
+#include <cli_progress_reporter.h>
 #include <confirm.h>
 #include <gestfich.h>
 #include <eda_dde.h>
@@ -32,11 +33,16 @@
 #include "eeschema_helpers.h"
 #include <eeschema_settings.h>
 #include <sch_edit_frame.h>
+#include <design_block_lib_table.h>
 #include <symbol_edit_frame.h>
 #include <symbol_viewer_frame.h>
-#include <transform.h>
+#include <symbol_chooser_frame.h>
 #include <symbol_lib_table.h>
+#include <dialogs/dialog_global_design_block_lib_table_config.h>
 #include <dialogs/dialog_global_sym_lib_table_config.h>
+#include <dialogs/panel_grid_settings.h>
+#include <dialogs/panel_simulator_preferences.h>
+#include <dialogs/panel_design_block_lib_table.h>
 #include <dialogs/panel_sym_lib_table.h>
 #include <kiway.h>
 #include <settings/settings_manager.h>
@@ -59,7 +65,7 @@
 #include <dialogs/panel_gal_display_options.h>
 #include <panel_eeschema_display_options.h>
 #include <panel_sym_display_options.h>
-#include <sim/sim_plot_frame.h>
+#include <sim/simulator_frame.h>
 
 #include <wx/crt.h>
 
@@ -70,17 +76,21 @@ SCH_SHEET*  g_RootSheet = nullptr;
 namespace SCH {
 
 
+// TODO: This should move out of this file
 static std::unique_ptr<SCHEMATIC> readSchematicFromFile( const std::string& aFilename )
 {
-    auto pi = SCH_IO_MGR::FindPlugin( SCH_IO_MGR::SCH_KICAD );
+    SCH_IO* pi = SCH_IO_MGR::FindPlugin( SCH_IO_MGR::SCH_KICAD );
     std::unique_ptr<SCHEMATIC> schematic = std::make_unique<SCHEMATIC>( nullptr );
 
-    auto &manager = Pgm().GetSettingsManager();
+    SETTINGS_MANAGER& manager = Pgm().GetSettingsManager();
 
+    // TODO: this must load the schematic's project, not a default project.  At the very minimum
+    // variable resolution won't work without the project, but there might also be issues with
+    // netclasses, etc.
     manager.LoadProject( "" );
     schematic->Reset();
     schematic->SetProject( &manager.Prj() );
-    schematic->SetRoot( pi->Load( aFilename, schematic.get() ) );
+    schematic->SetRoot( pi->LoadSchematicFile( aFilename, schematic.get() ) );
     schematic->CurrentSheet().push_back( &schematic->Root() );
 
     SCH_SCREENS screens( schematic->Root() );
@@ -88,11 +98,17 @@ static std::unique_ptr<SCHEMATIC> readSchematicFromFile( const std::string& aFil
     for( SCH_SCREEN* screen = screens.GetFirst(); screen; screen = screens.GetNext() )
         screen->UpdateLocalLibSymbolLinks();
 
-    SCH_SHEET_LIST sheets = schematic->GetSheets();
+    SCH_SHEET_LIST sheets = schematic->Hierarchy();
 
     // Restore all of the loaded symbol instances from the root sheet screen.
     sheets.UpdateSymbolInstanceData( schematic->RootScreen()->GetSymbolInstances() );
 
+    if( schematic->RootScreen()->GetFileFormatVersionAtLoad() < 20230221 )
+    {
+        for( SCH_SCREEN* screen = screens.GetFirst(); screen; screen = screens.GetNext() )
+            screen->FixLegacyPowerSymbolMismatches();
+    }
+
     for( SCH_SCREEN* screen = screens.GetFirst(); screen; screen = screens.GetNext() )
         screen->MigrateSimModels();
 
@@ -102,8 +118,11 @@ static std::unique_ptr<SCHEMATIC> readSchematicFromFile( const std::string& aFil
     for( SCH_SHEET_PATH& sheet : sheets )
         sheet.UpdateAllScreenReferences();
 
-    // NOTE: SchematicCleanUp is not called; QA schematics must already be clean or else
-    // SchematicCleanUp must be freed from its UI dependencies.
+    // TODO: this must handle SchematicCleanup somehow.  The original version didn't because
+    // it knew that QA test cases were saved in a clean state.
+
+    // TODO: does this need to handle PruneOrphanedSymbolInstances() and
+    // PruneOrphanedSheetInstances()?
 
     schematic->ConnectionGraph()->Recalculate( sheets, true );
 
@@ -111,7 +130,8 @@ static std::unique_ptr<SCHEMATIC> readSchematicFromFile( const std::string& aFil
 }
 
 
-bool generateSchematicNetlist( const wxString& aFilename, wxString& aNetlist )
+// TODO: This should move out of this file
+bool generateSchematicNetlist( const wxString& aFilename, std::string& aNetlist )
 {
     std::unique_ptr<SCHEMATIC> schematic = readSchematicFromFile( aFilename.ToStdString() );
     NETLIST_EXPORTER_KICAD exporter( schematic.get() );
@@ -133,7 +153,7 @@ static struct IFACE : public KIFACE_BASE, public UNITS_PROVIDER
             UNITS_PROVIDER( schIUScale, EDA_UNITS::MILLIMETRES )
     {}
 
-    bool OnKifaceStart( PGM_BASE* aProgram, int aCtlBits ) override;
+    bool OnKifaceStart( PGM_BASE* aProgram, int aCtlBits, KIWAY* aKiway ) override;
 
     void Reset() override;
 
@@ -165,18 +185,21 @@ static struct IFACE : public KIFACE_BASE, public UNITS_PROVIDER
             return frame;
         }
 
-#ifdef KICAD_SPICE
         case FRAME_SIMULATOR:
         {
-            SIM_PLOT_FRAME* frame = new SIM_PLOT_FRAME( aKiway, aParent );
+            SIMULATOR_FRAME* frame = new SIMULATOR_FRAME( aKiway, aParent );
             return frame;
         }
-#endif
+
         case FRAME_SCH_VIEWER:
-        case FRAME_SCH_VIEWER_MODAL:
         {
-            SYMBOL_VIEWER_FRAME* frame = new SYMBOL_VIEWER_FRAME( aKiway, aParent,
-                                                                  FRAME_T( aClassId ) );
+            SYMBOL_VIEWER_FRAME* frame = new SYMBOL_VIEWER_FRAME( aKiway, aParent );
+            return frame;
+        }
+
+        case FRAME_SYMBOL_CHOOSER:
+        {
+            SYMBOL_CHOOSER_FRAME* frame = new SYMBOL_CHOOSER_FRAME( aKiway, aParent );
             return frame;
         }
 
@@ -185,14 +208,37 @@ static struct IFACE : public KIFACE_BASE, public UNITS_PROVIDER
             // Dialog has completed; nothing to return.
             return nullptr;
 
+        case DIALOG_DESIGN_BLOCK_LIBRARY_TABLE:
+            InvokeEditDesignBlockLibTable( aKiway, aParent );
+            // Dialog has completed; nothing to return.
+            return nullptr;
+
         case PANEL_SYM_DISP_OPTIONS:
         {
             SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
-            APP_SETTINGS_BASE* cfg = mgr.GetAppSettings<SYMBOL_EDITOR_SETTINGS>();
+            APP_SETTINGS_BASE* cfg = mgr.GetAppSettings<SYMBOL_EDITOR_SETTINGS>( "symbol_editor" );
 
             return new PANEL_SYM_DISPLAY_OPTIONS( aParent, cfg );
         }
 
+        case PANEL_SYM_EDIT_GRIDS:
+        {
+            SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
+            APP_SETTINGS_BASE* cfg = mgr.GetAppSettings<SYMBOL_EDITOR_SETTINGS>( "symbol_editor" );
+            EDA_BASE_FRAME*    frame = aKiway->Player( FRAME_SCH_SYMBOL_EDITOR, false );
+
+            if( !frame )
+                frame = aKiway->Player( FRAME_SCH_VIEWER, false );
+
+            if( !frame )
+                frame = aKiway->Player( FRAME_SCH, false );
+
+            if( frame )
+                SetUserUnits( frame->GetUserUnits() );
+
+            return new PANEL_GRID_SETTINGS( aParent, this, frame, cfg, FRAME_SCH_SYMBOL_EDITOR );
+        }
+
         case PANEL_SYM_EDIT_OPTIONS:
         {
             EDA_BASE_FRAME* frame = aKiway->Player( FRAME_SCH_SYMBOL_EDITOR, false );
@@ -215,11 +261,29 @@ static struct IFACE : public KIFACE_BASE, public UNITS_PROVIDER
         case PANEL_SCH_DISP_OPTIONS:
         {
             SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
-            APP_SETTINGS_BASE* cfg = mgr.GetAppSettings<EESCHEMA_SETTINGS>();
+            EESCHEMA_SETTINGS* cfg = mgr.GetAppSettings<EESCHEMA_SETTINGS>( "eeschema" );
 
             return new PANEL_EESCHEMA_DISPLAY_OPTIONS( aParent, cfg );
         }
 
+        case PANEL_SCH_GRIDS:
+        {
+            SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
+            EESCHEMA_SETTINGS* cfg = mgr.GetAppSettings<EESCHEMA_SETTINGS>( "eeschema" );
+            EDA_BASE_FRAME*    frame = aKiway->Player( FRAME_SCH, false );
+
+            if( !frame )
+                frame = aKiway->Player( FRAME_SCH_SYMBOL_EDITOR, false );
+
+            if( !frame )
+                frame = aKiway->Player( FRAME_SCH_VIEWER, false );
+
+            if( frame )
+                SetUserUnits( frame->GetUserUnits() );
+
+            return new PANEL_GRID_SETTINGS( aParent, this, frame, cfg, FRAME_SCH );
+        }
+
         case PANEL_SCH_EDIT_OPTIONS:
         {
             EDA_BASE_FRAME* frame = aKiway->Player( FRAME_SCH, false );
@@ -249,6 +313,9 @@ static struct IFACE : public KIFACE_BASE, public UNITS_PROVIDER
         case PANEL_SCH_FIELD_NAME_TEMPLATES:
             return new PANEL_TEMPLATE_FIELDNAMES( aParent, nullptr );
 
+        case PANEL_SCH_SIMULATOR:
+            return new PANEL_SIMULATOR_PREFERENCES( aParent );
+
         default:
             return nullptr;
         }
@@ -285,10 +352,13 @@ static struct IFACE : public KIFACE_BASE, public UNITS_PROVIDER
                      const wxString& aSrcFilePath, wxString& aErrors ) override;
 
 
-    int HandleJob( JOB* aJob ) override;
+    int HandleJob( JOB* aJob, REPORTER* aReporter ) override;
+
+    bool HandleJobConfig( JOB* aJob, wxWindow* aParent ) override;
 
 private:
     bool loadGlobalLibTable();
+    bool loadGlobalDesignBlockLibTable();
 
     std::unique_ptr<EESCHEMA_JOBS_HANDLER> m_jobHandler;
 
@@ -298,48 +368,52 @@ private:
 
 using namespace SCH;
 
-static PGM_BASE* process;
-
 
 KIFACE_BASE& Kiface() { return kiface; }
 
 
 // KIFACE_GETTER's actual spelling is a substitution macro found in kiway.h.
 // KIFACE_GETTER will not have name mangling due to declaration in kiway.h.
-MY_API( KIFACE* ) KIFACE_GETTER(  int* aKIFACEversion, int aKiwayVersion, PGM_BASE* aProgram )
+KIFACE_API KIFACE* KIFACE_GETTER(  int* aKIFACEversion, int aKiwayVersion, PGM_BASE* aProgram )
 {
-    process = aProgram;
     return &kiface;
 }
 
 
-PGM_BASE& Pgm()
-{
-    wxASSERT( process );    // KIFACE_GETTER has already been called.
-    return *process;
-}
-
-// Similar to PGM_BASE& Pgm(), but return nullptr when a *.ki_face is run from
-// a python script or something else.
-PGM_BASE* PgmOrNull()
-{
-    return process;
-}
-
-
-bool IFACE::OnKifaceStart( PGM_BASE* aProgram, int aCtlBits )
+bool IFACE::OnKifaceStart( PGM_BASE* aProgram, int aCtlBits, KIWAY* aKiway )
 {
     // This is process-level-initialization, not project-level-initialization of the DSO.
     // Do nothing in here pertinent to a project!
     InitSettings( new EESCHEMA_SETTINGS );
+
+    // Register the symbol editor settings as well because they share a KiFACE and need to be
+    // loaded prior to use to avoid threading deadlocks
+    SYMBOL_EDITOR_SETTINGS* symSettings = new SYMBOL_EDITOR_SETTINGS();
+    aProgram->GetSettingsManager().RegisterSettings( symSettings ); // manager takes ownership
+
+    // We intentionally register KifaceSettings after SYMBOL_EDITOR_SETTINGS
+    // In legacy configs, many settings were in a single editor config nd the migration routine
+    // for the main editor file will try and call into the now separate settings stores
+    // to move the settings into them
     aProgram->GetSettingsManager().RegisterSettings( KifaceSettings() );
 
     start_common( aCtlBits );
 
-    if( !loadGlobalLibTable() )
+    if( !loadGlobalLibTable() || !loadGlobalDesignBlockLibTable() )
+    {
+        // we didnt get anywhere deregister the settings
+        aProgram->GetSettingsManager().FlushAndRelease( symSettings, false );
+        aProgram->GetSettingsManager().FlushAndRelease( KifaceSettings(), false );
         return false;
+    }
 
-    m_jobHandler = std::make_unique<EESCHEMA_JOBS_HANDLER>();
+    m_jobHandler = std::make_unique<EESCHEMA_JOBS_HANDLER>( aKiway );
+
+    if( m_start_flags & KFCTL_CLI )
+    {
+        m_jobHandler->SetReporter( &CLI_REPORTER::GetInstance() );
+        m_jobHandler->SetProgressReporter( &CLI_PROGRESS_REPORTER::GetInstance() );
+    }
 
     return true;
 }
@@ -359,9 +433,13 @@ bool IFACE::loadGlobalLibTable()
     {
         if( !( m_start_flags & KFCTL_CLI ) )
         {
-            DIALOG_GLOBAL_SYM_LIB_TABLE_CONFIG fpDialog( nullptr );
+            // Ensure the splash screen does not hide the dialog:
+            Pgm().HideSplash();
 
-            fpDialog.ShowModal();
+            DIALOG_GLOBAL_SYM_LIB_TABLE_CONFIG symDialog( nullptr );
+
+            if( symDialog.ShowModal() != wxID_OK )
+                return false;
         }
     }
     else
@@ -391,6 +469,41 @@ bool IFACE::loadGlobalLibTable()
 }
 
 
+bool IFACE::loadGlobalDesignBlockLibTable()
+{
+    try
+    {
+        wxFileName fn = DESIGN_BLOCK_LIB_TABLE::GetGlobalTableFileName();
+
+        if( !fn.FileExists() )
+        {
+            DESIGN_BLOCK_LIB_TABLE emptyTable;
+            emptyTable.Save( fn.GetFullPath() );
+        }
+
+        // The global table is not related to a specific project.  All projects
+        // will use the same global table.  So the KIFACE::OnKifaceStart() contract
+        // of avoiding anything project specific is not violated here.
+        if( !DESIGN_BLOCK_LIB_TABLE::LoadGlobalTable(
+                    DESIGN_BLOCK_LIB_TABLE::GetGlobalLibTable() ) )
+            return false;
+    }
+    catch( const IO_ERROR& ioe )
+    {
+        // if we are here, a incorrect global design block library table was found.
+        // Incorrect global design block library table is not a fatal error:
+        // the user just has to edit the (partially) loaded table.
+        wxString msg =
+                _( "An error occurred attempting to load the global design block library table.\n"
+                   "Please edit this global design block library table in Preferences menu." );
+
+        DisplayErrorMessage( nullptr, msg, ioe.What() );
+    }
+
+    return true;
+}
+
+
 void IFACE::OnKifaceEnd()
 {
     end_common();
@@ -424,10 +537,10 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aProje
 
     destFile.SetPath( destPath );
 
-    if( ext == LegacySchematicFileExtension ||
-        ext == LegacySchematicFileExtension + BackupFileSuffix ||
-        ext == KiCadSchematicFileExtension ||
-        ext == KiCadSchematicFileExtension + BackupFileSuffix )
+    if( ext == FILEEXT::LegacySchematicFileExtension
+        || ext == FILEEXT::LegacySchematicFileExtension + FILEEXT::BackupFileSuffix
+        || ext == FILEEXT::KiCadSchematicFileExtension
+        || ext == FILEEXT::KiCadSchematicFileExtension + FILEEXT::BackupFileSuffix )
     {
         if( destFile.GetName() == aProjectName )
         {
@@ -457,13 +570,14 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aProje
 
         KiCopyFile( aSrcFilePath, destFile.GetFullPath(), aErrors );
     }
-    else if( ext == SchematicSymbolFileExtension )
+    else if( ext == FILEEXT::SchematicSymbolFileExtension )
     {
         // Symbols are not project-specific.  Keep their source names.
         KiCopyFile( aSrcFilePath, destFile.GetFullPath(), aErrors );
     }
-    else if( ext == LegacySymbolLibFileExtension || ext == LegacySymbolDocumentFileExtension ||
-             ext == KiCadSymbolLibFileExtension )
+    else if( ext == FILEEXT::LegacySymbolLibFileExtension
+             || ext == FILEEXT::LegacySymbolDocumentFileExtension
+             || ext == FILEEXT::KiCadSymbolLibFileExtension )
     {
         if( destFile.GetName() == aProjectName + wxS( "-cache" ) )
             destFile.SetName( aNewProjectName + wxS( "-cache" ) );
@@ -473,7 +587,7 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aProje
 
         KiCopyFile( aSrcFilePath, destFile.GetFullPath(), aErrors );
     }
-    else if( ext == NetlistFileExtension )
+    else if( ext == FILEEXT::NetlistFileExtension )
     {
         bool success = false;
 
@@ -537,7 +651,7 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aProje
             aErrors += msg;
         }
     }
-    else if( destFile.GetName() == wxS( "sym-lib-table" ) )
+    else if( destFile.GetName() == FILEEXT::SymbolLibraryTableFileName )
     {
         SYMBOL_LIB_TABLE symbolLibTable;
         symbolLibTable.Load( aSrcFilePath );
@@ -578,7 +692,14 @@ void IFACE::SaveFileAs( const wxString& aProjectBasePath, const wxString& aProje
     }
 }
 
-int IFACE::HandleJob( JOB* aJob )
+
+int IFACE::HandleJob( JOB* aJob, REPORTER* aReporter )
 {
-    return m_jobHandler->RunJob( aJob );
-}
\ No newline at end of file
+    return m_jobHandler->RunJob( aJob, aReporter );
+}
+
+
+bool IFACE::HandleJobConfig( JOB* aJob, wxWindow* aParent )
+{
+    return m_jobHandler->HandleJobConfig( aJob, aParent );
+}
