@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,17 +27,20 @@
 #include <string_utils.h>
 #include <core/mirror.h>
 #include <sch_painter.h>
-#include <plotters/plotter.h>
+#include <sch_plotter.h>
 #include <geometry/shape_segment.h>
 #include <sch_line.h>
 #include <sch_edit_frame.h>
 #include <settings/color_settings.h>
-#include <schematic.h>
 #include <connection_graph.h>
 #include <project/project_file.h>
 #include <project/net_settings.h>
 #include <trigo.h>
 #include <board_item.h>
+#include <api/api_enums.h>
+#include <api/api_utils.h>
+#include <api/schematic/schematic_types.pb.h>
+#include <properties/property.h>
 
 
 SCH_LINE::SCH_LINE( const VECTOR2I& pos, int layer ) :
@@ -46,7 +49,7 @@ SCH_LINE::SCH_LINE( const VECTOR2I& pos, int layer ) :
     m_start           = pos;
     m_end             = pos;
     m_stroke.SetWidth( 0 );
-    m_stroke.SetPlotStyle( PLOT_DASH_TYPE::DEFAULT );
+    m_stroke.SetLineStyle( LINE_STYLE::DEFAULT );
     m_stroke.SetColor( COLOR4D::UNSPECIFIED );
 
     switch( layer )
@@ -68,7 +71,7 @@ SCH_LINE::SCH_LINE( const VECTOR2I& pos, int layer ) :
     else
         m_lastResolvedWidth = schIUScale.MilsToIU( DEFAULT_LINE_WIDTH_MILS );
 
-    m_lastResolvedLineStyle = PLOT_DASH_TYPE::SOLID;
+    m_lastResolvedLineStyle = LINE_STYLE::SOLID;
     m_lastResolvedColor = COLOR4D::UNSPECIFIED;
 }
 
@@ -85,46 +88,65 @@ SCH_LINE::SCH_LINE( const SCH_LINE& aLine ) :
     m_lastResolvedLineStyle = aLine.m_lastResolvedLineStyle;
     m_lastResolvedWidth = aLine.m_lastResolvedWidth;
     m_lastResolvedColor = aLine.m_lastResolvedColor;
+
+    m_operatingPoint = aLine.m_operatingPoint;
 }
 
-wxString SCH_LINE::GetNetname( const SCH_SHEET_PATH& aSheet )
+
+void SCH_LINE::Serialize( google::protobuf::Any &aContainer ) const
 {
-    std::list<const SCH_LINE *> checkedLines;
-    checkedLines.push_back(this);
-    return FindWireSegmentNetNameRecursive( this, checkedLines, aSheet );
+    kiapi::schematic::types::Line line;
+
+    line.mutable_id()->set_value( m_Uuid.AsStdString() );
+    kiapi::common::PackVector2( *line.mutable_start(), GetStartPoint() );
+    kiapi::common::PackVector2( *line.mutable_end(), GetEndPoint() );
+    line.set_layer(
+            ToProtoEnum<SCH_LAYER_ID, kiapi::schematic::types::SchematicLayer>( GetLayer() ) );
+
+    aContainer.PackFrom( line );
 }
 
-wxString SCH_LINE::FindWireSegmentNetNameRecursive( SCH_LINE *line,
-                                                    std::list<const SCH_LINE *> &checkedLines,
-                                                    const SCH_SHEET_PATH& aSheet ) const
+
+bool SCH_LINE::Deserialize( const google::protobuf::Any &aContainer )
 {
-    for ( auto connected : line->ConnectedItems( aSheet ) )
+    kiapi::schematic::types::Line line;
+
+    if( !aContainer.UnpackTo( &line ) )
+        return false;
+
+    const_cast<KIID&>( m_Uuid ) = KIID( line.id().value() );
+    SetStartPoint( kiapi::common::UnpackVector2( line.start() ) );
+    SetEndPoint( kiapi::common::UnpackVector2( line.end() ) );
+    SCH_LAYER_ID layer =
+            FromProtoEnum<SCH_LAYER_ID, kiapi::schematic::types::SchematicLayer>( line.layer() );
+
+    switch( layer )
     {
-        if( connected->Type() == SCH_LINE_T )
-        {
-            if( std::find(checkedLines.begin(), checkedLines.end(), connected ) == checkedLines.end() )
-            {
-                SCH_LINE* connectedLine = static_cast<SCH_LINE*>( connected );
-                checkedLines.push_back( connectedLine );
-
-                wxString netName = FindWireSegmentNetNameRecursive( connectedLine, checkedLines,
-                                                                    aSheet );
-
-                if( !netName.IsEmpty() )
-                    return netName;
-            }
-        }
-        else if( connected->Type() == SCH_LABEL_T
-                 || connected->Type() == SCH_GLOBAL_LABEL_T
-                 || connected->Type() == SCH_DIRECTIVE_LABEL_T)
-        {
-            return static_cast<SCH_TEXT*>( connected )->GetText();
-        }
+    case LAYER_WIRE:
+    case LAYER_BUS:
+    case LAYER_NOTES:
+        SetLayer( layer );
+        break;
 
+    default:
+        break;
     }
-    return "";
+
+    return true;
 }
 
+
+wxString SCH_LINE::GetFriendlyName() const
+{
+    switch( GetLayer() )
+    {
+    case LAYER_WIRE: return _( "Wire" );
+    case LAYER_BUS:  return _( "Bus" );
+    default:         return _( "Graphic Line" );
+    }
+}
+
+
 EDA_ITEM* SCH_LINE::Clone() const
 {
     return new SCH_LINE( *this );
@@ -133,32 +155,20 @@ EDA_ITEM* SCH_LINE::Clone() const
 
 void SCH_LINE::Move( const VECTOR2I& aOffset )
 {
-    if( aOffset != VECTOR2I( 0, 0 ) )
-    {
-        m_start += aOffset;
-        m_end += aOffset;
-        SetModified();
-    }
+    m_start += aOffset;
+    m_end += aOffset;
 }
 
 
 void SCH_LINE::MoveStart( const VECTOR2I& aOffset )
 {
-    if( aOffset != VECTOR2I( 0, 0 ) )
-    {
-        m_start += aOffset;
-        SetModified();
-    }
+    m_start += aOffset;
 }
 
 
 void SCH_LINE::MoveEnd( const VECTOR2I& aOffset )
 {
-    if( aOffset != VECTOR2I( 0, 0 ) )
-    {
-        m_end += aOffset;
-        SetModified();
-    }
+    m_end += aOffset;
 }
 
 
@@ -179,12 +189,32 @@ void SCH_LINE::Show( int nestLevel, std::ostream& os ) const
 #endif
 
 
-void SCH_LINE::ViewGetLayers( int aLayers[], int& aCount ) const
+std::vector<int> SCH_LINE::ViewGetLayers() const
 {
-    aCount     = 3;
-    aLayers[0] = LAYER_DANGLING;
-    aLayers[1] = m_layer;
-    aLayers[2] = LAYER_SELECTION_SHADOWS;
+    return { LAYER_DANGLING, m_layer, LAYER_SELECTION_SHADOWS, LAYER_NET_COLOR_HIGHLIGHT,
+             LAYER_OP_VOLTAGES };
+}
+
+
+double SCH_LINE::ViewGetLOD( int aLayer, const KIGFX::VIEW* aView ) const
+{
+    if( aLayer == LAYER_OP_VOLTAGES )
+    {
+        if( m_start == m_end )
+            return LOD_HIDE;
+
+        const int height = std::abs( m_end.y - m_start.y );
+
+        // Operating points will be shown only if zoom is appropriate
+        if( height > 0 )
+            return lodScaleForThreshold( height, schIUScale.mmToIU( 5 ) );
+
+        const int width = std::abs( m_end.x - m_start.x );
+        return lodScaleForThreshold( width, schIUScale.mmToIU( 15 ) );
+    }
+
+    // Other layers are always drawn.
+    return LOD_SHOW;
 }
 
 
@@ -206,7 +236,7 @@ const BOX2I SCH_LINE::GetBoundingBox() const
 
 double SCH_LINE::GetLength() const
 {
-    return GetLineLength( m_start, m_end );
+    return m_start.Distance( m_end );
 }
 
 
@@ -247,36 +277,30 @@ COLOR4D SCH_LINE::GetLineColor() const
 }
 
 
-void SCH_LINE::SetLineStyle( const int aStyleId )
+void SCH_LINE::SetLineStyle( const LINE_STYLE aStyle )
 {
-    SetLineStyle( static_cast<PLOT_DASH_TYPE>( aStyleId ) );
+    m_stroke.SetLineStyle( aStyle );
+    m_lastResolvedLineStyle = GetEffectiveLineStyle();
 }
 
 
-void SCH_LINE::SetLineStyle( const PLOT_DASH_TYPE aStyle )
+LINE_STYLE SCH_LINE::GetLineStyle() const
 {
-    m_stroke.SetPlotStyle( aStyle );
-    m_lastResolvedLineStyle = GetLineStyle();
+    if( IsGraphicLine() && m_stroke.GetLineStyle() == LINE_STYLE::DEFAULT )
+        return LINE_STYLE::SOLID;
+    else
+        return m_stroke.GetLineStyle();
 }
 
 
-PLOT_DASH_TYPE SCH_LINE::GetLineStyle() const
+LINE_STYLE SCH_LINE::GetEffectiveLineStyle() const
 {
-    if( m_stroke.GetPlotStyle() != PLOT_DASH_TYPE::DEFAULT )
-        return m_stroke.GetPlotStyle();
-
-    return PLOT_DASH_TYPE::SOLID;
-}
-
-
-PLOT_DASH_TYPE SCH_LINE::GetEffectiveLineStyle() const
-{
-    if( m_stroke.GetPlotStyle() != PLOT_DASH_TYPE::DEFAULT )
-        m_lastResolvedLineStyle = m_stroke.GetPlotStyle();
+    if( m_stroke.GetLineStyle() != LINE_STYLE::DEFAULT )
+        m_lastResolvedLineStyle = m_stroke.GetLineStyle();
     else if( !IsConnectable() )
-        m_lastResolvedLineStyle = PLOT_DASH_TYPE::SOLID;
+        m_lastResolvedLineStyle = LINE_STYLE::SOLID;
     else if( !IsConnectivityDirty() )
-        m_lastResolvedLineStyle = (PLOT_DASH_TYPE) GetEffectiveNetClass()->GetLineStyle();
+        m_lastResolvedLineStyle = (LINE_STYLE) GetEffectiveNetClass()->GetLineStyle();
 
     return m_lastResolvedLineStyle;
 }
@@ -323,7 +347,8 @@ int SCH_LINE::GetPenWidth() const
 }
 
 
-void SCH_LINE::Print( const RENDER_SETTINGS* aSettings, const VECTOR2I& offset )
+void SCH_LINE::Print( const SCH_RENDER_SETTINGS* aSettings, int aUnit, int aBodyStyle,
+                      const VECTOR2I& offset, bool aForceNoFill, bool aDimmed )
 {
     wxDC*   DC = aSettings->GetPrintDC();
     COLOR4D color = GetLineColor();
@@ -331,12 +356,12 @@ void SCH_LINE::Print( const RENDER_SETTINGS* aSettings, const VECTOR2I& offset )
     if( color == COLOR4D::UNSPECIFIED )
         color = aSettings->GetLayerColor( GetLayer() );
 
-    VECTOR2I       start = m_start;
-    VECTOR2I       end = m_end;
-    PLOT_DASH_TYPE lineStyle = GetEffectiveLineStyle();
-    int            penWidth = std::max( GetPenWidth(), aSettings->GetDefaultPenWidth() );
+    VECTOR2I   start = m_start;
+    VECTOR2I   end = m_end;
+    LINE_STYLE lineStyle = GetEffectiveLineStyle();
+    int        penWidth = GetEffectivePenWidth( aSettings );
 
-    if( lineStyle <= PLOT_DASH_TYPE::FIRST_TYPE )
+    if( lineStyle <= LINE_STYLE::FIRST_TYPE )
     {
         GRLine( DC, start.x, start.y, end.x, end.y, penWidth, color );
     }
@@ -373,28 +398,13 @@ void SCH_LINE::MirrorHorizontally( int aCenter )
 }
 
 
-void SCH_LINE::Rotate( const VECTOR2I& aCenter )
+void SCH_LINE::Rotate( const VECTOR2I& aCenter, bool aRotateCCW )
 {
-    // When we allow off grid items, the
-    // else if should become a plain if to allow
-    // rotation around the center of the line
     if( m_flags & STARTPOINT )
-        RotatePoint( m_start, aCenter, ANGLE_90 );
+        RotatePoint( m_start, aCenter, aRotateCCW ? ANGLE_90 : ANGLE_270 );
 
-    else if( m_flags & ENDPOINT )
-        RotatePoint( m_end, aCenter, ANGLE_90 );
-}
-
-
-void SCH_LINE::RotateStart( const VECTOR2I& aCenter )
-{
-    RotatePoint( m_start, aCenter, ANGLE_90 );
-}
-
-
-void SCH_LINE::RotateEnd( const VECTOR2I& aCenter )
-{
-    RotatePoint( m_end, aCenter, ANGLE_90 );
+    if( m_flags & ENDPOINT )
+        RotatePoint( m_end, aCenter, aRotateCCW ? ANGLE_90 : ANGLE_270 );
 }
 
 
@@ -580,44 +590,56 @@ void SCH_LINE::GetEndPoints( std::vector <DANGLING_END_ITEM>& aItemList )
 }
 
 
-bool SCH_LINE::UpdateDanglingState( std::vector<DANGLING_END_ITEM>& aItemList,
-                                    const SCH_SHEET_PATH* aPath )
+bool SCH_LINE::UpdateDanglingState( std::vector<DANGLING_END_ITEM>& aItemListByType,
+                                    std::vector<DANGLING_END_ITEM>& aItemListByPos,
+                                    const SCH_SHEET_PATH*           aPath )
 {
-    if( IsConnectable() )
+    if( !IsConnectable() )
+        return false;
+
+    bool previousStartState = m_startIsDangling;
+    bool previousEndState = m_endIsDangling;
+
+    m_startIsDangling = m_endIsDangling = true;
+
+    for( auto it = DANGLING_END_ITEM_HELPER::get_lower_pos( aItemListByPos, m_start );
+         it < aItemListByPos.end() && it->GetPosition() == m_start; it++ )
     {
-        bool previousStartState = m_startIsDangling;
-        bool previousEndState = m_endIsDangling;
+        DANGLING_END_ITEM& item = *it;
 
-        m_startIsDangling = m_endIsDangling = true;
+        if( item.GetItem() == this )
+            continue;
 
-        for( DANGLING_END_ITEM item : aItemList )
+        if( ( IsWire() && item.GetType() != BUS_END && item.GetType() != BUS_ENTRY_END )
+            || ( IsBus() && item.GetType() != WIRE_END && item.GetType() != PIN_END ) )
         {
-            if( item.GetItem() == this )
-                continue;
-
-            if( ( IsWire() && item.GetType() != BUS_END && item.GetType() != BUS_ENTRY_END )
-                || ( IsBus() && item.GetType() != WIRE_END && item.GetType() != PIN_END ) )
-            {
-                if( m_start == item.GetPosition() )
-                    m_startIsDangling = false;
-
-                if( m_end == item.GetPosition() )
-                    m_endIsDangling = false;
-
-                if( !m_startIsDangling && !m_endIsDangling )
-                    break;
-            }
+            m_startIsDangling = false;
+            break;
         }
-
-        // We only use the bus dangling state for automatic line starting, so we don't care if it
-        // has changed or not (and returning true will result in extra work)
-        if( IsBus() )
-            return false;
-
-        return previousStartState != m_startIsDangling || previousEndState != m_endIsDangling;
     }
 
-    return false;
+    for( auto it = DANGLING_END_ITEM_HELPER::get_lower_pos( aItemListByPos, m_end );
+         it < aItemListByPos.end() && it->GetPosition() == m_end; it++ )
+    {
+        DANGLING_END_ITEM& item = *it;
+
+        if( item.GetItem() == this )
+            continue;
+
+        if( ( IsWire() && item.GetType() != BUS_END && item.GetType() != BUS_ENTRY_END )
+            || ( IsBus() && item.GetType() != WIRE_END && item.GetType() != PIN_END ) )
+        {
+            m_endIsDangling = false;
+            break;
+        }
+    }
+
+    // We only use the bus dangling state for automatic line starting, so we don't care if it
+    // has changed or not (and returning true will result in extra work)
+    if( IsBus() )
+        return false;
+
+    return previousStartState != m_startIsDangling || previousEndState != m_endIsDangling;
 }
 
 
@@ -632,44 +654,44 @@ bool SCH_LINE::IsConnectable() const
 
 bool SCH_LINE::CanConnect( const SCH_ITEM* aItem ) const
 {
-    if( m_layer == LAYER_WIRE )
+    switch( aItem->Type() )
     {
-        switch( aItem->Type() )
-        {
-        case SCH_JUNCTION_T:
-        case SCH_NO_CONNECT_T:
-        case SCH_LABEL_T:
-        case SCH_GLOBAL_LABEL_T:
-        case SCH_HIER_LABEL_T:
-        case SCH_DIRECTIVE_LABEL_T:
-        case SCH_BUS_WIRE_ENTRY_T:
-        case SCH_SYMBOL_T:
-        case SCH_SHEET_T:
-        case SCH_SHEET_PIN_T:
-            return true;
-        default:
-            break;
-        }
-    }
-    else if( m_layer == LAYER_BUS )
-    {
-        switch( aItem->Type() )
-        {
-        case SCH_JUNCTION_T:
-        case SCH_LABEL_T:
-        case SCH_GLOBAL_LABEL_T:
-        case SCH_HIER_LABEL_T:
-        case SCH_DIRECTIVE_LABEL_T:
-        case SCH_BUS_WIRE_ENTRY_T:
-        case SCH_SHEET_T:
-        case SCH_SHEET_PIN_T:
-            return true;
-        default:
-            break;
-        }
-    }
+    case SCH_NO_CONNECT_T:
+    case SCH_SYMBOL_T:
+        return IsWire();
 
-    return aItem->GetLayer() == m_layer;
+    case SCH_JUNCTION_T:
+    case SCH_LABEL_T:
+    case SCH_GLOBAL_LABEL_T:
+    case SCH_HIER_LABEL_T:
+    case SCH_DIRECTIVE_LABEL_T:
+    case SCH_BUS_WIRE_ENTRY_T:
+    case SCH_SHEET_T:
+    case SCH_SHEET_PIN_T:
+        return IsWire() || IsBus();
+
+    default:
+        return m_layer == aItem->GetLayer();
+    }
+}
+
+
+bool SCH_LINE::HasConnectivityChanges( const SCH_ITEM* aItem,
+                                       const SCH_SHEET_PATH* aInstance ) const
+{
+    // Do not compare to ourself.
+    if( aItem == this || !IsConnectable() )
+        return false;
+
+    const SCH_LINE* line = dynamic_cast<const SCH_LINE*>( aItem );
+
+    // Don't compare against a different SCH_ITEM.
+    wxCHECK( line, false );
+
+    if( GetStartPoint() != line->GetStartPoint() )
+        return true;
+
+    return GetEndPoint() != line->GetEndPoint();
 }
 
 
@@ -702,7 +724,7 @@ void SCH_LINE::GetSelectedPoints( std::vector<VECTOR2I>& aPoints ) const
 }
 
 
-wxString SCH_LINE::GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const
+wxString SCH_LINE::GetItemDescription( UNITS_PROVIDER* aUnitsProvider, bool aFull ) const
 {
     wxString txtfmt;
 
@@ -735,7 +757,7 @@ wxString SCH_LINE::GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const
     }
 
     return wxString::Format( txtfmt,
-                             aUnitsProvider->MessageTextFromValue( EuclideanNorm( m_start - m_end ) ) );
+                             aUnitsProvider->MessageTextFromValue( m_start.Distance( m_end ) ) );
 }
 
 
@@ -807,6 +829,8 @@ bool SCH_LINE::HitTest( const BOX2I& aRect, bool aContained, int aAccuracy ) con
 
 void SCH_LINE::SwapData( SCH_ITEM* aItem )
 {
+    SCH_ITEM::SwapFlags( aItem );
+
     SCH_LINE* item = (SCH_LINE*) aItem;
 
     std::swap( m_layer, item->m_layer );
@@ -828,17 +852,18 @@ bool SCH_LINE::doIsConnected( const VECTOR2I& aPosition ) const
 }
 
 
-void SCH_LINE::Plot( PLOTTER* aPlotter, bool aBackground ) const
+void SCH_LINE::Plot( PLOTTER* aPlotter, bool aBackground, const SCH_PLOT_OPTS& aPlotOpts,
+                     int aUnit, int aBodyStyle, const VECTOR2I& aOffset, bool aDimmed )
 {
     if( aBackground )
         return;
 
-    auto*   settings = static_cast<KIGFX::SCH_RENDER_SETTINGS*>( aPlotter->RenderSettings() );
-    int     penWidth = std::max( GetPenWidth(), settings->GetMinPenWidth() );
-    COLOR4D color = GetLineColor();
+    SCH_RENDER_SETTINGS* renderSettings = getRenderSettings( aPlotter );
+    int                  penWidth = GetEffectivePenWidth( renderSettings );
+    COLOR4D              color = GetLineColor();
 
     if( color == COLOR4D::UNSPECIFIED )
-        color = settings->GetLayerColor( GetLayer() );
+        color = renderSettings->GetLayerColor( GetLayer() );
 
     aPlotter->SetColor( color );
 
@@ -848,38 +873,40 @@ void SCH_LINE::Plot( PLOTTER* aPlotter, bool aBackground ) const
     aPlotter->MoveTo( m_start );
     aPlotter->FinishTo( m_end );
 
-    aPlotter->SetDash( penWidth, PLOT_DASH_TYPE::SOLID );
+    aPlotter->SetDash( penWidth, LINE_STYLE::SOLID );
 
     // Plot attributes to a hypertext menu
     std::vector<wxString> properties;
     BOX2I                 bbox = GetBoundingBox();
-    bbox.Inflate( GetPenWidth() * 3 );
+    bbox.Inflate( penWidth * 3 );
 
-    if( GetLayer() == LAYER_WIRE )
+    if( aPlotOpts.m_PDFPropertyPopups )
     {
-        if( SCH_CONNECTION* connection = Connection() )
+        if( GetLayer() == LAYER_WIRE )
         {
-            properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
-                                                       _( "Net" ),
-                                                       connection->Name() ) );
+            if( SCH_CONNECTION* connection = Connection() )
+            {
+                properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
+                                                           _( "Net" ),
+                                                           connection->Name() ) );
 
-            properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
-                                                       _( "Resolved netclass" ),
-                                                       GetEffectiveNetClass()->GetName() ) );
+                properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
+                                                           _( "Resolved netclass" ),
+                                                           GetEffectiveNetClass()->GetHumanReadableName() ) );
+            }
         }
-    }
-    else if( GetLayer() == LAYER_BUS )
-    {
-        if( SCH_CONNECTION* connection = Connection() )
+        else if( GetLayer() == LAYER_BUS )
         {
-            for( const std::shared_ptr<SCH_CONNECTION>& member : connection->Members() )
-                properties.emplace_back( wxT( "!" ) + member->Name() );
+            if( SCH_CONNECTION* connection = Connection() )
+            {
+                for( const std::shared_ptr<SCH_CONNECTION>& member : connection->Members() )
+                    properties.emplace_back( wxT( "!" ) + member->Name() );
+            }
         }
 
+        if( !properties.empty() )
+            aPlotter->HyperlinkMenu( bbox, properties );
     }
-
-    if( !properties.empty() )
-        aPlotter->HyperlinkMenu( bbox, properties );
 }
 
 
@@ -903,7 +930,7 @@ void SCH_LINE::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_IT
 
     aList.emplace_back( _( "Line Type" ), msg );
 
-    PLOT_DASH_TYPE lineStyle = GetLineStyle();
+    LINE_STYLE lineStyle = GetStroke().GetLineStyle();
 
     if( GetEffectiveLineStyle() != lineStyle )
         aList.emplace_back( _( "Line Style" ), _( "from netclass" ) );
@@ -922,7 +949,7 @@ void SCH_LINE::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_IT
         if( !conn->IsBus() )
         {
             aList.emplace_back( _( "Resolved Netclass" ),
-                                UnescapeString( GetEffectiveNetClass()->GetName() ) );
+                                UnescapeString( GetEffectiveNetClass()->GetHumanReadableName() ) );
         }
     }
 }
@@ -945,3 +972,133 @@ bool SCH_LINE::IsBus() const
     return ( GetLayer() == LAYER_BUS );
 }
 
+
+bool SCH_LINE::operator==( const SCH_ITEM& aOther ) const
+{
+    if( Type() != aOther.Type() )
+        return false;
+
+    const SCH_LINE& other = static_cast<const SCH_LINE&>( aOther );
+
+    if( GetLayer() != other.GetLayer() )
+        return false;
+
+    if( m_start != other.m_start )
+        return false;
+
+    if( m_end != other.m_end )
+        return false;
+
+    if( m_stroke.GetWidth() != other.m_stroke.GetWidth() )
+        return false;
+
+    if( m_stroke.GetColor() != other.m_stroke.GetColor() )
+        return false;
+
+    if( m_stroke.GetLineStyle() != other.m_stroke.GetLineStyle() )
+        return false;
+
+    return true;
+}
+
+
+double SCH_LINE::Similarity( const SCH_ITEM& aOther ) const
+{
+    if( m_Uuid == aOther.m_Uuid )
+        return 1.0;
+
+    if( Type() != aOther.Type() )
+        return 0.0;
+
+    const SCH_LINE& other = static_cast<const SCH_LINE&>( aOther );
+
+    if( GetLayer() != other.GetLayer() )
+        return 0.0;
+
+    double similarity = 1.0;
+
+    if( m_start != other.m_start )
+        similarity *= 0.9;
+
+    if( m_end != other.m_end )
+        similarity *= 0.9;
+
+    if( m_stroke.GetWidth() != other.m_stroke.GetWidth() )
+        similarity *= 0.9;
+
+    if( m_stroke.GetColor() != other.m_stroke.GetColor() )
+        similarity *= 0.9;
+
+    if( m_stroke.GetLineStyle() != other.m_stroke.GetLineStyle() )
+        similarity *= 0.9;
+
+    return similarity;
+}
+
+
+static struct SCH_LINE_DESC
+{
+    SCH_LINE_DESC()
+    {
+        ENUM_MAP<LINE_STYLE>& lineStyleEnum = ENUM_MAP<LINE_STYLE>::Instance();
+
+        if( lineStyleEnum.Choices().GetCount() == 0 )
+        {
+            lineStyleEnum.Map( LINE_STYLE::SOLID, _HKI( "Solid" ) )
+                         .Map( LINE_STYLE::DASH, _HKI( "Dashed" ) )
+                         .Map( LINE_STYLE::DOT, _HKI( "Dotted" ) )
+                         .Map( LINE_STYLE::DASHDOT, _HKI( "Dash-Dot" ) )
+                         .Map( LINE_STYLE::DASHDOTDOT, _HKI( "Dash-Dot-Dot" ) );
+        }
+
+        ENUM_MAP<WIRE_STYLE>& wireLineStyleEnum = ENUM_MAP<WIRE_STYLE>::Instance();
+
+        if( wireLineStyleEnum.Choices().GetCount() == 0 )
+        {
+            wireLineStyleEnum.Map( WIRE_STYLE::DEFAULT, _HKI( "Default" ) )
+                             .Map( WIRE_STYLE::SOLID, _HKI( "Solid" ) )
+                             .Map( WIRE_STYLE::DASH, _HKI( "Dashed" ) )
+                             .Map( WIRE_STYLE::DOT, _HKI( "Dotted" ) )
+                             .Map( WIRE_STYLE::DASHDOT, _HKI( "Dash-Dot" ) )
+                             .Map( WIRE_STYLE::DASHDOTDOT, _HKI( "Dash-Dot-Dot" ) );
+        }
+
+        PROPERTY_MANAGER& propMgr = PROPERTY_MANAGER::Instance();
+        REGISTER_TYPE( SCH_LINE );
+        propMgr.InheritsAfter( TYPE_HASH( SCH_LINE ), TYPE_HASH( SCH_ITEM ) );
+
+        auto isGraphicLine =
+                []( INSPECTABLE* aItem ) -> bool
+                {
+                    if( SCH_LINE* line = dynamic_cast<SCH_LINE*>( aItem ) )
+                        return line->IsGraphicLine();
+
+                    return false;
+                };
+
+        auto isWireOrBus =
+                []( INSPECTABLE* aItem ) -> bool
+                {
+                    if( SCH_LINE* line = dynamic_cast<SCH_LINE*>( aItem ) )
+                        return line->IsWire() || line->IsBus();
+
+                    return false;
+                };
+
+        propMgr.AddProperty( new PROPERTY_ENUM<SCH_LINE, LINE_STYLE>( _HKI( "Line Style" ),
+                    &SCH_LINE::SetLineStyle, &SCH_LINE::GetLineStyle ) )
+                .SetAvailableFunc( isGraphicLine );
+
+        propMgr.AddProperty( new PROPERTY_ENUM<SCH_LINE, WIRE_STYLE>( _HKI( "Line Style" ),
+                    &SCH_LINE::SetWireStyle, &SCH_LINE::GetWireStyle ) )
+                .SetAvailableFunc( isWireOrBus );
+
+        propMgr.AddProperty( new PROPERTY<SCH_LINE, int>( _HKI( "Line Width" ),
+                    &SCH_LINE::SetLineWidth, &SCH_LINE::GetLineWidth, PROPERTY_DISPLAY::PT_SIZE ) );
+
+        propMgr.AddProperty( new PROPERTY<SCH_LINE, COLOR4D>( _HKI( "Color" ),
+                    &SCH_LINE::SetLineColor, &SCH_LINE::GetLineColor ) );
+    }
+} _SCH_LINE_DESC;
+
+IMPLEMENT_ENUM_TO_WXANY( WIRE_STYLE )
