@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2017 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2011 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,6 +23,8 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <refdes_utils.h>
+#include <hash.h>
 #include <sch_screen.h>
 #include <sch_item.h>
 #include <sch_marker.h>
@@ -36,13 +38,13 @@
 #include <template_fieldnames.h>
 #include <trace_helpers.h>
 
-#include <boost/functional/hash.hpp>
 #include <wx/filename.h>
 #include <wx/log.h>
 
 
 /**
  * A singleton item of this class is returned for a weak reference that no longer exists.
+ *
  * Its sole purpose is to flag the item as having been deleted.
  */
 class DELETED_SHEET_ITEM : public SCH_ITEM
@@ -52,7 +54,7 @@ public:
         SCH_ITEM( nullptr, NOT_USED )
     {}
 
-    wxString GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const override
+    wxString GetItemDescription( UNITS_PROVIDER* aUnitsProvider, bool aFull ) const override
     {
         return _( "(Deleted Item)" );
     }
@@ -74,11 +76,20 @@ public:
 
     // pure virtuals:
     void SetPosition( const VECTOR2I& ) override {}
-    void Print( const RENDER_SETTINGS* aSettings, const VECTOR2I& aOffset ) override {}
     void Move( const VECTOR2I& aMoveVector ) override {}
     void MirrorHorizontally( int aCenter ) override {}
     void MirrorVertically( int aCenter ) override {}
-    void Rotate( const VECTOR2I& aCenter ) override {}
+    void Rotate( const VECTOR2I& aCenter, bool aRotateCCW ) override {}
+
+    double Similarity( const SCH_ITEM& aOther ) const override
+    {
+        return 0.0;
+    }
+
+    bool operator==( const SCH_ITEM& aOther ) const override
+    {
+        return false;
+    }
 
 #if defined(DEBUG)
     void Show( int , std::ostream&  ) const override {}
@@ -86,15 +97,6 @@ public:
 };
 
 
-bool SortSymbolInstancesByProjectUuid( const SCH_SYMBOL_INSTANCE& aLhs,
-                                       const SCH_SYMBOL_INSTANCE& aRhs )
-{
-    wxCHECK( !aLhs.m_Path.empty() && !aRhs.m_Path.empty(), false );
-
-    return aLhs.m_Path[0] < aRhs.m_Path[0];
-}
-
-
 namespace std
 {
     size_t hash<SCH_SHEET_PATH>::operator()( const SCH_SHEET_PATH& path ) const
@@ -139,12 +141,14 @@ SCH_SHEET_PATH SCH_SHEET_PATH::operator+( const SCH_SHEET_PATH& aOther )
 
 void SCH_SHEET_PATH::initFromOther( const SCH_SHEET_PATH& aOther )
 {
-    m_sheets            = aOther.m_sheets;
-    m_virtualPageNumber = aOther.m_virtualPageNumber;
-    m_current_hash      = aOther.m_current_hash;
+    m_sheets             = aOther.m_sheets;
+    m_virtualPageNumber  = aOther.m_virtualPageNumber;
+    m_current_hash       = aOther.m_current_hash;
+    m_cached_page_number = aOther.m_cached_page_number;
 
-    // Note: don't copy m_recursion_test_cache as it is slow and we want SCH_SHEET_PATHS to be
-    // very fast to construct for use in the connectivity algorithm.
+    // Note: don't copy m_recursion_test_cache as it is slow and we want
+    // std::vector<SCH_SHEET_PATH> to be very fast to construct for use in
+    // the connectivity algorithm.
 }
 
 
@@ -160,7 +164,7 @@ void SCH_SHEET_PATH::Rehash()
     m_current_hash = 0;
 
     for( SCH_SHEET* sheet : m_sheets )
-        boost::hash_combine( m_current_hash, sheet->m_Uuid.Hash() );
+        hash_combine( m_current_hash, sheet->m_Uuid.Hash() );
 }
 
 
@@ -172,7 +176,7 @@ int SCH_SHEET_PATH::Cmp( const SCH_SHEET_PATH& aSheetPathToTest ) const
     if( size() < aSheetPathToTest.size() )
         return -1;
 
-    //otherwise, same number of sheets.
+    // otherwise, same number of sheets.
     for( unsigned i = 0; i < size(); i++ )
     {
         if( at( i )->m_Uuid < aSheetPathToTest.at( i )->m_Uuid )
@@ -262,6 +266,54 @@ SCH_SCREEN* SCH_SHEET_PATH::LastScreen() const
 }
 
 
+bool SCH_SHEET_PATH::GetExcludedFromSim() const
+{
+    for( SCH_SHEET* sheet : m_sheets )
+    {
+        if( sheet->GetExcludedFromSim() )
+            return true;
+    }
+
+    return false;
+}
+
+
+bool SCH_SHEET_PATH::GetExcludedFromBOM() const
+{
+    for( SCH_SHEET* sheet : m_sheets )
+    {
+        if( sheet->GetExcludedFromBOM() )
+            return true;
+    }
+
+    return false;
+}
+
+
+bool SCH_SHEET_PATH::GetExcludedFromBoard() const
+{
+    for( SCH_SHEET* sheet : m_sheets )
+    {
+        if( sheet->GetExcludedFromBoard() )
+            return true;
+    }
+
+    return false;
+}
+
+
+bool SCH_SHEET_PATH::GetDNP() const
+{
+    for( SCH_SHEET* sheet : m_sheets )
+    {
+        if( sheet->GetDNP() )
+            return true;
+    }
+
+    return false;
+}
+
+
 wxString SCH_SHEET_PATH::PathAsString() const
 {
     wxString s;
@@ -280,6 +332,7 @@ wxString SCH_SHEET_PATH::PathAsString() const
 KIID_PATH SCH_SHEET_PATH::Path() const
 {
     KIID_PATH path;
+    path.reserve( m_sheets.size() );
 
     for( const SCH_SHEET* sheet : m_sheets )
         path.push_back( sheet->m_Uuid );
@@ -311,7 +364,7 @@ wxString SCH_SHEET_PATH::PathHumanReadable( bool aUseShortRootName,
 
     // Start at 1 since we've already processed the root sheet.
     for( unsigned i = 1; i < size(); i++ )
-        s << at( i )->GetFields()[SHEETNAME].GetShownText() << wxS( "/" );
+        s << at( i )->GetFields()[SHEETNAME].GetShownText( false ) << wxS( "/" );
 
     if( aStripTrailingSeparator && s.EndsWith( "/" ) )
         s = s.Left( s.length() - 1 );
@@ -338,7 +391,7 @@ void SCH_SHEET_PATH::UpdateAllScreenReferences() const
             SCH_SYMBOL* symbol = static_cast<SCH_SYMBOL*>( item );
 
             symbol->GetField( REFERENCE_FIELD )->SetText( symbol->GetRef( this ) );
-            symbol->UpdateUnit( symbol->GetUnitSelection( this ) );
+            symbol->SetUnit( symbol->GetUnitSelection( this ) );
             LastScreen()->Update( item, false );
         }
         else if( item->Type() == SCH_GLOBAL_LABEL_T )
@@ -356,7 +409,7 @@ void SCH_SHEET_PATH::UpdateAllScreenReferences() const
                         && intersheetRefs.GetPosition() == VECTOR2I( 0, 0 )
                         && !intersheetRefs.IsVisible() )
                 {
-                    label->AutoplaceFields( LastScreen(), false );
+                    label->AutoplaceFields( LastScreen(), AUTOPLACE_AUTO );
                 }
 
                 intersheetRefs.SetVisible( label->Schematic()->Settings().m_IntersheetRefsShow );
@@ -367,7 +420,6 @@ void SCH_SHEET_PATH::UpdateAllScreenReferences() const
 }
 
 
-
 void SCH_SHEET_PATH::GetSymbols( SCH_REFERENCE_LIST& aReferences, bool aIncludePowerSymbols,
                                  bool aForceIncludeOrphanSymbols ) const
 {
@@ -387,11 +439,9 @@ void SCH_SHEET_PATH::AppendSymbol( SCH_REFERENCE_LIST& aReferences, SCH_SYMBOL*
     // affects power symbols.
     if( aIncludePowerSymbols || aSymbol->GetRef( this )[0] != wxT( '#' ) )
     {
-        LIB_SYMBOL* symbol = aSymbol->GetLibSymbolRef().get();
-
-        if( symbol || aForceIncludeOrphanSymbols )
+        if( aSymbol->GetLibSymbolRef() || aForceIncludeOrphanSymbols )
         {
-            SCH_REFERENCE schReference( aSymbol, symbol, *this );
+            SCH_REFERENCE schReference( aSymbol, *this );
 
             schReference.SetSheetNumber( m_virtualPageNumber );
             aReferences.AddItem( schReference );
@@ -424,7 +474,7 @@ void SCH_SHEET_PATH::AppendMultiUnitSymbol( SCH_MULTI_UNIT_REFERENCE_MAP& aRefLi
 
     if( symbol && symbol->GetUnitCount() > 1 )
     {
-        SCH_REFERENCE schReference = SCH_REFERENCE( aSymbol, symbol, *this );
+        SCH_REFERENCE schReference = SCH_REFERENCE( aSymbol, *this );
         schReference.SetSheetNumber( m_virtualPageNumber );
         wxString reference_str = schReference.GetRef();
 
@@ -464,7 +514,6 @@ bool SCH_SHEET_PATH::TestForRecursion( const wxString& aSrcFileName, const wxStr
     if( destFn.IsRelative() )
         destFn.MakeAbsolute( rootFn.GetPath() );
 
-
     // The source and destination sheet file names cannot be the same.
     if( srcFn == destFn )
     {
@@ -530,8 +579,7 @@ wxString SCH_SHEET_PATH::GetPageNumber() const
 
     wxCHECK( sheet, wxEmptyString );
 
-    SCH_SHEET_PATH tmpPath = *this;
-
+    KIID_PATH tmpPath = Path();
     tmpPath.pop_back();
 
     return sheet->getPageNumber( tmpPath );
@@ -544,7 +592,7 @@ void SCH_SHEET_PATH::SetPageNumber( const wxString& aPageNumber )
 
     wxCHECK( sheet, /* void */ );
 
-    SCH_SHEET_PATH tmpPath = *this;
+    KIID_PATH tmpPath = Path();
 
     tmpPath.pop_back();
 
@@ -553,8 +601,11 @@ void SCH_SHEET_PATH::SetPageNumber( const wxString& aPageNumber )
 }
 
 
-void SCH_SHEET_PATH::AddNewSymbolInstances( const SCH_SHEET_PATH& aPrefixSheetPath )
+void SCH_SHEET_PATH::AddNewSymbolInstances( const SCH_SHEET_PATH& aPrefixSheetPath,
+                                            const wxString& aProjectName )
 {
+    wxCHECK( !aProjectName.IsEmpty(), /* void */ );
+
     SCH_SHEET_PATH newSheetPath( aPrefixSheetPath );
     SCH_SHEET_PATH currentSheetPath( *this );
 
@@ -571,19 +622,25 @@ void SCH_SHEET_PATH::AddNewSymbolInstances( const SCH_SHEET_PATH& aPrefixSheetPa
 
         if( symbol->GetInstance( newSymbolInstance, Path(), true ) )
         {
+            newSymbolInstance.m_ProjectName = aProjectName;
+
             // Use an existing symbol instance for this path if it exists.
             newSymbolInstance.m_Path = newSheetPath.Path();
             symbol->AddHierarchicalReference( newSymbolInstance );
         }
-        else if( !symbol->GetInstanceReferences().empty() )
+        else if( !symbol->GetInstances().empty() )
         {
+            newSymbolInstance.m_ProjectName = aProjectName;
+
             // Use the first symbol instance if any symbol instance data exists.
-            newSymbolInstance = symbol->GetInstanceReferences()[0];
+            newSymbolInstance = symbol->GetInstances()[0];
             newSymbolInstance.m_Path = newSheetPath.Path();
             symbol->AddHierarchicalReference( newSymbolInstance );
         }
         else
         {
+            newSymbolInstance.m_ProjectName = aProjectName;
+
             // Fall back to the last saved symbol field and unit settings if there is no
             // instance data.
             newSymbolInstance.m_Path = newSheetPath.Path();
@@ -613,6 +670,49 @@ void SCH_SHEET_PATH::RemoveSymbolInstances( const SCH_SHEET_PATH& aPrefixSheetPa
 }
 
 
+void SCH_SHEET_PATH::CheckForMissingSymbolInstances( const wxString& aProjectName )
+{
+    wxCHECK( !aProjectName.IsEmpty() && LastScreen(), /* void */ );
+
+    for( SCH_ITEM* item : LastScreen()->Items().OfType( SCH_SYMBOL_T ) )
+    {
+        SCH_SYMBOL* symbol = static_cast<SCH_SYMBOL*>( item );
+
+        wxCHECK2( symbol, continue );
+
+        SCH_SYMBOL_INSTANCE symbolInstance;
+
+        if( !symbol->GetInstance( symbolInstance, Path() ) )
+        {
+            wxLogTrace( traceSchSheetPaths, "Adding missing symbol \"%s\" instance data for "
+                        "sheet path '%s'.",
+                        symbol->m_Uuid.AsString(), PathHumanReadable( false ) );
+
+            // Legacy schematics that are not shared do not contain separate instance data.
+            // The symbol reference and unit are saved in the reference field and unit entries.
+            if( ( LastScreen()->GetRefCount() <= 1 ) &&
+                ( LastScreen()->GetFileFormatVersionAtLoad() <= 20200310 ) )
+            {
+                symbolInstance.m_Reference =
+                        symbol->GetField( REFERENCE_FIELD )->GetShownText( this, true );
+                symbolInstance.m_Unit = symbol->GetUnit();
+            }
+            else
+            {
+                // When schematics are shared, we cannot know which instance the current symbol
+                // reference field and unit belong to.  In this case, we clear the reference
+                // annotation and set the unit to 1.
+                symbolInstance.m_Reference = UTIL::GetRefDesUnannotated( symbol->GetPrefix() );
+            }
+
+            symbolInstance.m_ProjectName = aProjectName;
+            symbolInstance.m_Path = Path();
+            symbol->AddHierarchicalReference( symbolInstance );
+        }
+    }
+}
+
+
 void SCH_SHEET_PATH::MakeFilePathRelativeToParentSheet()
 {
     wxCHECK( m_sheets.size() > 1, /* void */  );
@@ -657,15 +757,10 @@ void SCH_SHEET_PATH::MakeFilePathRelativeToParentSheet()
 }
 
 
-SCH_SHEET_LIST::SCH_SHEET_LIST( SCH_SHEET* aSheet, bool aCheckIntegrity )
+SCH_SHEET_LIST::SCH_SHEET_LIST( SCH_SHEET* aSheet )
 {
     if( aSheet != nullptr )
-    {
-        BuildSheetList( aSheet, aCheckIntegrity );
-
-        if( aSheet->IsRootSheet() )
-            SortByPageNumbers();
-    }
+        BuildSheetList( aSheet, false );
 }
 
 
@@ -698,6 +793,11 @@ void SCH_SHEET_LIST::BuildSheetList( SCH_SHEET* aSheet, bool aCheckIntegrity )
             }
             else
             {
+                // If we are not performing a full recursion test, at least check if we are in
+                // a simple recursion scenario to prevent stack overflow crashes
+                wxCHECK2_MSG( sheet->GetFileName() != aSheet->GetFileName(), continue,
+                              wxT( "Recursion prevented in SCH_SHEET_LIST::BuildSheetList" ) );
+
                 BuildSheetList( sheet, false );
             }
         }
@@ -718,17 +818,27 @@ void SCH_SHEET_LIST::BuildSheetList( SCH_SHEET* aSheet, bool aCheckIntegrity )
 
 void SCH_SHEET_LIST::SortByPageNumbers( bool aUpdateVirtualPageNums )
 {
+    for( const SCH_SHEET_PATH& path : *this )
+        path.CachePageNumber();
+
     std::sort( begin(), end(),
-        []( SCH_SHEET_PATH a, SCH_SHEET_PATH b ) -> bool
+        []( const SCH_SHEET_PATH& a, const SCH_SHEET_PATH& b ) -> bool
         {
-            int retval = a.ComparePageNum( b );
+            int retval = SCH_SHEET::ComparePageNum( a.GetCachedPageNumber(),
+                                                    b.GetCachedPageNumber() );
 
             if( retval < 0 )
                 return true;
             else if( retval > 0 )
                 return false;
-            else /// Enforce strict ordering.  If the page numbers are the same, use UUIDs
-                return a.GetCurrentHash() < b.GetCurrentHash();
+
+            if( a.GetVirtualPageNumber() < b.GetVirtualPageNumber() )
+                return true;
+            else if( a.GetVirtualPageNumber() > b.GetVirtualPageNumber() )
+                return false;
+
+            // Enforce strict ordering.  If the page numbers are the same, use UUIDs
+            return a.GetCurrentHash() < b.GetCurrentHash();
         } );
 
     if( aUpdateVirtualPageNums )
@@ -736,9 +846,7 @@ void SCH_SHEET_LIST::SortByPageNumbers( bool aUpdateVirtualPageNums )
         int virtualPageNum = 1;
 
         for( SCH_SHEET_PATH& sheet : *this )
-        {
             sheet.SetVirtualPageNumber( virtualPageNum++ );
-        }
     }
 }
 
@@ -767,6 +875,20 @@ bool SCH_SHEET_LIST::PageNumberExists( const wxString& aPageNumber ) const
 }
 
 
+void SCH_SHEET_LIST::TrimToPageNumbers( const std::vector<wxString>& aPageInclusions )
+{
+    auto it = std::remove_if( begin(), end(),
+                              [&]( SCH_SHEET_PATH sheet )
+                              {
+                                  return std::find( aPageInclusions.begin(), aPageInclusions.end(),
+                                                    sheet.GetPageNumber() )
+                                         == aPageInclusions.end();
+                              } );
+
+    erase( it, end() );
+}
+
+
 bool SCH_SHEET_LIST::IsModified() const
 {
     for( const SCH_SHEET_PATH& sheet : *this )
@@ -872,7 +994,7 @@ void SCH_SHEET_LIST::AnnotatePowerSymbols()
 
             if( libSymbol && libSymbol->IsPower() )
             {
-                SCH_REFERENCE schReference( symbol, libSymbol, sheet );
+                SCH_REFERENCE schReference( symbol, sheet );
                 references.AddItem( schReference );
             }
         }
@@ -944,7 +1066,7 @@ void SCH_SHEET_LIST::GetSymbolsWithinPath( SCH_REFERENCE_LIST&   aReferences,
 }
 
 
-void SCH_SHEET_LIST::GetSheetsWithinPath( SCH_SHEET_PATHS&      aSheets,
+void SCH_SHEET_LIST::GetSheetsWithinPath( std::vector<SCH_SHEET_PATH>& aSheets,
                                           const SCH_SHEET_PATH& aSheetPath ) const
 {
     for( const SCH_SHEET_PATH& sheet : *this )
@@ -976,7 +1098,7 @@ std::optional<SCH_SHEET_PATH> SCH_SHEET_LIST::GetSheetPathByKIIDPath( const KIID
 void SCH_SHEET_LIST::GetMultiUnitSymbols( SCH_MULTI_UNIT_REFERENCE_MAP &aRefList,
                                           bool aIncludePowerSymbols ) const
 {
-    for( SCH_SHEET_PATHS::const_iterator it = begin(); it != end(); ++it )
+    for( auto it = begin(); it != end(); ++it )
     {
         SCH_MULTI_UNIT_REFERENCE_MAP tempMap;
         ( *it ).GetMultiUnitSymbols( tempMap, aIncludePowerSymbols );
@@ -1045,15 +1167,15 @@ SCH_SHEET_PATH* SCH_SHEET_LIST::FindSheetForPath( const SCH_SHEET_PATH* aPath )
 }
 
 
-SCH_SHEET_PATH* SCH_SHEET_LIST::FindSheetForScreen( const SCH_SCREEN* aScreen )
+SCH_SHEET_PATH SCH_SHEET_LIST::FindSheetForScreen( const SCH_SCREEN* aScreen )
 {
     for( SCH_SHEET_PATH& sheetpath : *this )
     {
         if( sheetpath.LastScreen() == aScreen )
-            return &sheetpath;
+            return sheetpath;
     }
 
-    return nullptr;
+    return SCH_SHEET_PATH();
 }
 
 
@@ -1138,7 +1260,7 @@ void SCH_SHEET_LIST::UpdateSheetInstanceData( const std::vector<SCH_SHEET_INSTAN
         }
 
         wxLogTrace( traceSchSheetPaths, "Setting sheet '%s' instance '%s' page number '%s'",
-                    ( sheet->GetName().IsEmpty() ) ? wxT( "root" ) : sheet->GetName(),
+                    ( sheet->GetName().IsEmpty() ) ? wxString( wxT( "root" ) ) : sheet->GetName(),
                     path.Path().AsString(), it->m_PageNumber );
         path.SetPageNumber( it->m_PageNumber );
     }
@@ -1209,10 +1331,11 @@ void SCH_SHEET_LIST::SetInitialPageNumbers()
 }
 
 
-void SCH_SHEET_LIST::AddNewSymbolInstances( const SCH_SHEET_PATH& aPrefixSheetPath )
+void SCH_SHEET_LIST::AddNewSymbolInstances( const SCH_SHEET_PATH& aPrefixSheetPath,
+                                            const wxString& aProjectName )
 {
     for( SCH_SHEET_PATH& sheetPath : *this )
-        sheetPath.AddNewSymbolInstances( aPrefixSheetPath );
+        sheetPath.AddNewSymbolInstances( aPrefixSheetPath, aProjectName );
 }
 
 
@@ -1248,7 +1371,7 @@ void SCH_SHEET_LIST::AddNewSheetInstances( const SCH_SHEET_PATH& aPrefixSheetPat
 
     for( SCH_SHEET_PATH& sheetPath : *this )
     {
-        SCH_SHEET_PATH tmp( sheetPath );
+        KIID_PATH tmp = sheetPath.Path();
         SCH_SHEET_PATH newSheetPath( aPrefixSheetPath );
 
         // Prefix the new hierarchical path.
@@ -1265,13 +1388,15 @@ void SCH_SHEET_LIST::AddNewSheetInstances( const SCH_SHEET_PATH& aPrefixSheetPat
 
         SCH_SHEET_INSTANCE instance;
 
-        if( sheet->getInstance( instance, tmp.Path(), true )
-          && !instance.m_PageNumber.IsEmpty() )
+        // Add the instance if it doesn't already exist
+        if( !sheet->getInstance( instance, tmp, true ) )
         {
-            newSheetPath.SetPageNumber( instance.m_PageNumber );
-            usedPageNumbers.push_back( instance.m_PageNumber );
+            sheet->addInstance( tmp );
+            sheet->getInstance( instance, tmp, true );
         }
-        else
+
+        // Get a new page number if we don't have one
+        if( instance.m_PageNumber.IsEmpty() )
         {
             // Generate the next available page number.
             do
@@ -1281,14 +1406,23 @@ void SCH_SHEET_LIST::AddNewSheetInstances( const SCH_SHEET_PATH& aPrefixSheetPat
             } while( std::find( usedPageNumbers.begin(), usedPageNumbers.end(), pageNumber ) !=
                      usedPageNumbers.end() );
 
+            instance.m_PageNumber = pageNumber;
             newSheetPath.SetVirtualPageNumber( nextVirtualPageNumber );
-            newSheetPath.SetPageNumber( pageNumber );
-            usedPageNumbers.push_back( pageNumber );
         }
+
+        newSheetPath.SetPageNumber( instance.m_PageNumber );
+        usedPageNumbers.push_back( instance.m_PageNumber );
     }
 }
 
 
+void SCH_SHEET_LIST::CheckForMissingSymbolInstances( const wxString& aProjectName )
+{
+    for( SCH_SHEET_PATH& sheetPath : *this )
+        sheetPath.CheckForMissingSymbolInstances( aProjectName );
+}
+
+
 int SCH_SHEET_LIST::GetLastVirtualPageNumber() const
 {
     int lastVirtualPageNumber = 1;
@@ -1302,3 +1436,45 @@ int SCH_SHEET_LIST::GetLastVirtualPageNumber() const
     return lastVirtualPageNumber;
 }
 
+
+bool SCH_SHEET_LIST::HasPath( const KIID_PATH& aPath ) const
+{
+    for( const SCH_SHEET_PATH& path : *this )
+    {
+        if( path.Path() == aPath )
+            return true;
+    }
+
+    return false;
+}
+
+
+bool SCH_SHEET_LIST::ContainsSheet( const SCH_SHEET* aSheet ) const
+{
+    for( const SCH_SHEET_PATH& path : *this )
+    {
+        for( size_t i = 0; i < path.size(); i++ )
+        {
+            if( path.at( i ) == aSheet )
+                return true;
+        }
+    }
+
+    return false;
+}
+
+
+std::optional<SCH_SHEET_PATH> SCH_SHEET_LIST::GetOrdinalPath( const SCH_SCREEN* aScreen ) const
+{
+    // Sheet paths with sheets that do not have a screen object are not valid.
+    if( !aScreen )
+        return std::nullopt;
+
+    for( const SCH_SHEET_PATH& path: *this )
+    {
+        if( path.LastScreen() == aScreen )
+            return std::optional<SCH_SHEET_PATH>( path );
+    }
+
+    return std::nullopt;
+}
