@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2013-2018 CERN
- * Copyright (C) 2019-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  * @author Maciej Suminski <maciej.suminski@cern.ch>
@@ -31,11 +31,13 @@
 #include <view/view_rtree.h>
 #include <view/wx_view_controls.h>
 #include <drawing_sheet/ds_proxy_view_item.h>
+#include <tool/tool_manager.h>
 #include <layer_ids.h>
 #include <sch_screen.h>
 #include <schematic.h>
 #include <sch_base_frame.h>
 #include <sch_edit_frame.h>
+#include <string_utils.h>
 
 #include "sch_view.h"
 
@@ -43,8 +45,8 @@
 namespace KIGFX {
 
 
-SCH_VIEW::SCH_VIEW( bool aIsDynamic, SCH_BASE_FRAME* aFrame ) :
-    VIEW( aIsDynamic )
+SCH_VIEW::SCH_VIEW( SCH_BASE_FRAME* aFrame ) :
+    VIEW()
 {
     m_frame = aFrame;
 
@@ -95,8 +97,7 @@ void SCH_VIEW::DisplaySheet( const SCH_SCREEN *aScreen )
     for( SCH_ITEM* item : aScreen->Items() )
         Add( item );
 
-    m_drawingSheet.reset( new DS_PROXY_VIEW_ITEM( static_cast<int>( schIUScale.IU_PER_MILS ),
-                                                  &aScreen->GetPageSettings(),
+    m_drawingSheet.reset( new DS_PROXY_VIEW_ITEM( schIUScale, &aScreen->GetPageSettings(),
                                                   &aScreen->Schematic()->Prj(),
                                                   &aScreen->GetTitleBlock(),
                                                   aScreen->Schematic()->GetProperties() ) );
@@ -127,6 +128,10 @@ void SCH_VIEW::DisplaySheet( const SCH_SCREEN *aScreen )
     Add( m_drawingSheet.get() );
 
     InitPreview();
+
+    // Allow tools to add anything they require to the view (such as the selection VIEW_GROUP)
+    if( m_frame && m_frame->GetToolManager() )
+        m_frame->GetToolManager()->ResetTools( TOOL_BASE::REDRAW );
 }
 
 
@@ -137,40 +142,31 @@ void SCH_VIEW::DisplaySymbol( LIB_SYMBOL* aSymbol )
     if( !aSymbol )
         return;
 
-    std::shared_ptr< LIB_SYMBOL > parent;
-    LIB_SYMBOL* drawnSymbol = aSymbol;
-
-    // Draw the mandatory fields for aliases and parent symbols.
-    for( LIB_ITEM& item : aSymbol->GetDrawItems() )
+    // Draw the fields.
+    for( SCH_ITEM& item : aSymbol->GetDrawItems() )
     {
-        if( item.Type() != LIB_FIELD_T )
-            continue;
-
-        LIB_FIELD* field = static_cast< LIB_FIELD* >( &item );
-
-        wxCHECK2( field, continue );
-
-        if( field->GetText().IsEmpty() )
-            continue;
-
-        Add( &item );
+        if( item.Type() == SCH_FIELD_T )
+            Add( &item );
     }
 
+    LIB_SYMBOL* drawnSymbol = aSymbol;
+
     // Draw the parent items if the symbol is inherited from another symbol.
     if( aSymbol->IsAlias() )
     {
-        parent = aSymbol->GetParent().lock();
-
-        wxCHECK( parent, /* void */ );
-
-        drawnSymbol = parent.get();
+        if( std::shared_ptr< LIB_SYMBOL > parent = aSymbol->GetRootSymbol() )
+            drawnSymbol = parent.get();
+        else
+        {
+            wxCHECK( false, /* void */ );
+        }
     }
 
-    for( LIB_ITEM& item : drawnSymbol->GetDrawItems() )
+    for( SCH_ITEM& item : drawnSymbol->GetDrawItems() )
     {
-        // Don't show parent symbol fields.  Users may be confused by shown fields that can not
-        // be edited.
-        if( aSymbol->IsAlias() && item.Type() == LIB_FIELD_T )
+        // Fields already drawn above.  (Besides, we don't want to show parent symbol fields as
+        // users may be confused by shown fields that can not be edited.)
+        if( item.Type() == SCH_FIELD_T )
             continue;
 
         Add( &item );
@@ -183,7 +179,12 @@ void SCH_VIEW::DisplaySymbol( LIB_SYMBOL* aSymbol )
 void SCH_VIEW::ClearHiddenFlags()
 {
     for( VIEW_ITEM* item : *m_allItems )
+    {
+        if( !item )
+            continue;
+
         Hide( item, false );
+    }
 }
 
 
