@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2020-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -18,19 +18,29 @@
  */
 
 #include <bus_alias.h>
+#include <commit.h>
 #include <connection_graph.h>
+#include <core/ignore.h>
 #include <core/kicad_algo.h>
-#include <erc_settings.h>
-#include <sch_marker.h>
+#include <ee_collectors.h>
+#include <erc/erc_settings.h>
+#include <font/outline_font.h>
+#include <netlist_exporter_spice.h>
 #include <project.h>
-#include <project/project_file.h>
 #include <project/net_settings.h>
+#include <project/project_file.h>
 #include <schematic.h>
 #include <sch_junction.h>
+#include <sch_label.h>
 #include <sch_line.h>
+#include <sch_marker.h>
 #include <sch_screen.h>
 #include <sim/spice_settings.h>
-#include <sch_label.h>
+#include <sim/spice_value.h>
+
+#include <wx/log.h>
+
+bool SCHEMATIC::m_IsSchematicExists = false;
 
 SCHEMATIC::SCHEMATIC( PROJECT* aPrj ) :
           EDA_ITEM( nullptr, SCHEMATIC_T ),
@@ -39,29 +49,104 @@ SCHEMATIC::SCHEMATIC( PROJECT* aPrj ) :
 {
     m_currentSheet    = new SCH_SHEET_PATH();
     m_connectionGraph = new CONNECTION_GRAPH( this );
+    m_IsSchematicExists = true;
 
     SetProject( aPrj );
+
+    PROPERTY_MANAGER::Instance().RegisterListener( TYPE_HASH( SCH_FIELD ),
+            [&]( INSPECTABLE* aItem, PROPERTY_BASE* aProperty, COMMIT* aCommit )
+            {
+                // Special case: propagate value, footprint, and datasheet fields to other units
+                // of a given symbol if they aren't in the selection
+
+                SCH_FIELD* field = dynamic_cast<SCH_FIELD*>( aItem );
+
+                if( !field || !IsValid() )
+                    return;
+
+                SCH_SYMBOL* symbol = dynamic_cast<SCH_SYMBOL*>( field->GetParent() );
+
+                if( !symbol || aProperty->Name() != _HKI( "Text" ) )
+                    return;
+
+                // TODO(JE) This will need to get smarter to enable API access
+                SCH_SHEET_PATH sheetPath = CurrentSheet();
+
+                wxString newValue = aItem->Get<wxString>( aProperty );
+
+                wxString ref = symbol->GetRef( &sheetPath );
+                int      unit = symbol->GetUnit();
+                LIB_ID   libId = symbol->GetLibId();
+
+                for( SCH_SHEET_PATH& sheet : Hierarchy() )
+                {
+                    std::vector<SCH_SYMBOL*> otherUnits;
+
+                    CollectOtherUnits( ref, unit, libId, sheet, &otherUnits );
+
+                    for( SCH_SYMBOL* otherUnit : otherUnits )
+                    {
+                        switch( field->GetId() )
+                        {
+                        case VALUE_FIELD:
+                        {
+                            if( aCommit )
+                                aCommit->Modify( otherUnit, sheet.LastScreen() );
+
+                            otherUnit->SetValueFieldText( newValue );
+                            break;
+                        }
+
+                        case FOOTPRINT_FIELD:
+                        {
+                            if( aCommit )
+                                aCommit->Modify( otherUnit, sheet.LastScreen() );
+
+                            otherUnit->SetFootprintFieldText( newValue );
+                            break;
+                        }
+
+                        case DATASHEET_FIELD:
+                        {
+                            if( aCommit )
+                                aCommit->Modify( otherUnit, sheet.LastScreen() );
+
+                            otherUnit->GetField( DATASHEET_FIELD )->SetText( newValue );
+                            break;
+                        }
+
+                        default:
+                            break;
+                        }
+                    }
+                }
+            } );
 }
 
 
 SCHEMATIC::~SCHEMATIC()
 {
+    PROPERTY_MANAGER::Instance().UnregisterListeners( TYPE_HASH( SCH_FIELD ) );
+
     delete m_currentSheet;
     delete m_connectionGraph;
+
+    m_IsSchematicExists = false;
 }
 
 
 void SCHEMATIC::Reset()
 {
-    // Assume project already saved
     if( m_project )
     {
         PROJECT_FILE& project = m_project->GetProjectFile();
 
+        // d'tor will save settings to file
         delete project.m_ErcSettings;
-        delete project.m_SchematicSettings;
+        project.m_ErcSettings = nullptr;
 
-        project.m_ErcSettings       = nullptr;
+        // d'tor will save settings to file
+        delete project.m_SchematicSettings;
         project.m_SchematicSettings = nullptr;
 
         m_project = nullptr; // clear the project, so we don't do this again when setting a new one
@@ -82,10 +167,12 @@ void SCHEMATIC::SetProject( PROJECT* aPrj )
     {
         PROJECT_FILE& project = m_project->GetProjectFile();
 
+        // d'tor will save settings to file
         delete project.m_ErcSettings;
-        delete project.m_SchematicSettings;
+        project.m_ErcSettings = nullptr;
 
-        project.m_ErcSettings       = nullptr;
+        // d'tor will save settings to file
+        delete project.m_SchematicSettings;
         project.m_SchematicSettings = nullptr;
     }
 
@@ -98,7 +185,7 @@ void SCHEMATIC::SetProject( PROJECT* aPrj )
         project.m_SchematicSettings = new SCHEMATIC_SETTINGS( &project, "schematic" );
 
         project.m_SchematicSettings->LoadFromFile();
-        project.m_SchematicSettings->m_NgspiceSimulatorSettings->LoadFromFile();
+        project.m_SchematicSettings->m_NgspiceSettings->LoadFromFile();
         project.m_ErcSettings->LoadFromFile();
     }
 }
@@ -113,6 +200,7 @@ void SCHEMATIC::SetRoot( SCH_SHEET* aRootSheet )
     m_currentSheet->clear();
     m_currentSheet->push_back( m_rootSheet );
 
+    m_hierarchy = BuildSheetListSortedByPageNumbers();
     m_connectionGraph->Reset();
 }
 
@@ -123,6 +211,20 @@ SCH_SCREEN* SCHEMATIC::RootScreen() const
 }
 
 
+SCH_SHEET_LIST SCHEMATIC::Hierarchy() const
+{
+    wxCHECK( !m_hierarchy.empty(), m_hierarchy );
+
+    return m_hierarchy;
+}
+
+
+void SCHEMATIC::RefreshHierarchy()
+{
+    m_hierarchy = BuildSheetListSortedByPageNumbers();
+}
+
+
 void SCHEMATIC::GetContextualTextVars( wxArrayString* aVars ) const
 {
     auto add =
@@ -137,6 +239,7 @@ void SCHEMATIC::GetContextualTextVars( wxArrayString* aVars ) const
     add( wxT( "SHEETPATH" ) );
     add( wxT( "SHEETNAME" ) );
     add( wxT( "FILENAME" ) );
+    add( wxT( "FILEPATH" ) );
     add( wxT( "PROJECTNAME" ) );
 
     if( !CurrentSheet().empty() )
@@ -147,45 +250,51 @@ void SCHEMATIC::GetContextualTextVars( wxArrayString* aVars ) const
 }
 
 
-bool SCHEMATIC::ResolveTextVar( wxString* token, int aDepth ) const
+bool SCHEMATIC::ResolveTextVar( const SCH_SHEET_PATH* aSheetPath, wxString* token,
+                                int aDepth ) const
 {
-    if( !CurrentSheet().empty() )
-    {
-        if( token->IsSameAs( wxT( "#" ) ) )
-        {
-            *token = CurrentSheet().GetPageNumber();
-            return true;
-        }
-        else if( token->IsSameAs( wxT( "##" ) ) )
-        {
-            *token = wxString::Format( "%i", Root().CountSheets() );
-            return true;
-        }
-        else if( token->IsSameAs( wxT( "SHEETPATH" ) ) )
-        {
-            *token = CurrentSheet().PathHumanReadable();
-            return true;
-        }
-        else if( token->IsSameAs( wxT( "SHEETNAME" ) ) )
-        {
-            *token = CurrentSheet().Last()->GetName();
-            return true;
-        }
-        else if( token->IsSameAs( wxT( "FILENAME" ) ) )
-        {
-            wxFileName fn( GetFileName() );
-            *token = fn.GetFullName();
-            return true;
-        }
-        else if( token->IsSameAs( wxT( "PROJECTNAME" ) ) )
-        {
-            *token = Prj().GetProjectName();
-            return true;
-        }
+    wxCHECK( aSheetPath, false );
 
-        if( CurrentSheet().LastScreen()->GetTitleBlock().TextVarResolver( token, m_project ) )
-            return true;
+    if( token->IsSameAs( wxT( "#" ) ) )
+    {
+        *token = aSheetPath->GetPageNumber();
+        return true;
     }
+    else if( token->IsSameAs( wxT( "##" ) ) )
+    {
+        *token = wxString::Format( "%i", Root().CountSheets() );
+        return true;
+    }
+    else if( token->IsSameAs( wxT( "SHEETPATH" ) ) )
+    {
+        *token = aSheetPath->PathHumanReadable();
+        return true;
+    }
+    else if( token->IsSameAs( wxT( "SHEETNAME" ) ) )
+    {
+        *token = aSheetPath->Last()->GetName();
+        return true;
+    }
+    else if( token->IsSameAs( wxT( "FILENAME" ) ) )
+    {
+        wxFileName fn( GetFileName() );
+        *token = fn.GetFullName();
+        return true;
+    }
+    else if( token->IsSameAs( wxT( "FILEPATH" ) ) )
+    {
+        wxFileName fn( GetFileName() );
+        *token = fn.GetFullPath();
+        return true;
+    }
+    else if( token->IsSameAs( wxT( "PROJECTNAME" ) ) )
+    {
+        *token = Prj().GetProjectName();
+        return true;
+    }
+
+    if( aSheetPath->LastScreen()->GetTitleBlock().TextVarResolver( token, m_project ) )
+        return true;
 
     if( Prj().TextVarResolver( token ) )
         return true;
@@ -216,7 +325,7 @@ ERC_SETTINGS& SCHEMATIC::ErcSettings() const
 
 std::vector<SCH_MARKER*> SCHEMATIC::ResolveERCExclusions()
 {
-    SCH_SHEET_LIST sheetList = GetSheets();
+    SCH_SHEET_LIST sheetList = Hierarchy();
     ERC_SETTINGS&  settings  = ErcSettings();
 
     // Migrate legacy marker exclusions to new format to ensure exclusion matching functions across
@@ -228,15 +337,23 @@ std::vector<SCH_MARKER*> SCHEMATIC::ResolveERCExclusions()
 
     for( auto it = settings.m_ErcExclusions.begin(); it != settings.m_ErcExclusions.end(); )
     {
-        SCH_MARKER* testMarker = SCH_MARKER::Deserialize( this, *it );
+        SCH_MARKER* testMarker = SCH_MARKER::DeserializeFromString( sheetList, *it );
+
+        if( !testMarker )
+        {
+            it = settings.m_ErcExclusions.erase( it );
+            continue;
+        }
+
         if( testMarker->IsLegacyMarker() )
         {
             const wxString settingsKey = testMarker->GetRCItem()->GetSettingsKey();
 
-            if( settingsKey != wxT( "pin_to_pin" ) && settingsKey != wxT( "hier_label_mismatch" )
+            if(    settingsKey != wxT( "pin_to_pin" )
+                && settingsKey != wxT( "hier_label_mismatch" )
                 && settingsKey != wxT( "different_unit_net" ) )
             {
-                migratedExclusions.insert( testMarker->Serialize() );
+                migratedExclusions.insert( testMarker->SerializeToString() );
             }
 
             it = settings.m_ErcExclusions.erase( it );
@@ -245,6 +362,7 @@ std::vector<SCH_MARKER*> SCHEMATIC::ResolveERCExclusions()
         {
             ++it;
         }
+
         delete testMarker;
     }
 
@@ -256,12 +374,13 @@ std::vector<SCH_MARKER*> SCHEMATIC::ResolveERCExclusions()
     {
         for( SCH_ITEM* item : sheet.LastScreen()->Items().OfType( SCH_MARKER_T ) )
         {
-            SCH_MARKER* marker = static_cast<SCH_MARKER*>( item );
-            auto        it = settings.m_ErcExclusions.find( marker->Serialize() );
+            SCH_MARKER*                  marker = static_cast<SCH_MARKER*>( item );
+            wxString                     serialized = marker->SerializeToString();
+            std::set<wxString>::iterator it = settings.m_ErcExclusions.find( serialized );
 
             if( it != settings.m_ErcExclusions.end() )
             {
-                marker->SetExcluded( true );
+                marker->SetExcluded( true, settings.m_ErcExclusionComments[serialized] );
                 settings.m_ErcExclusions.erase( it );
             }
         }
@@ -269,13 +388,13 @@ std::vector<SCH_MARKER*> SCHEMATIC::ResolveERCExclusions()
 
     std::vector<SCH_MARKER*> newMarkers;
 
-    for( const wxString& exclusionData : settings.m_ErcExclusions )
+    for( const wxString& serialized : settings.m_ErcExclusions )
     {
-        SCH_MARKER* marker = SCH_MARKER::Deserialize( this, exclusionData );
+        SCH_MARKER* marker = SCH_MARKER::DeserializeFromString( sheetList, serialized );
 
         if( marker )
         {
-            marker->SetExcluded( true );
+            marker->SetExcluded( true, settings.m_ErcExclusionComments[serialized] );
             newMarkers.push_back( marker );
         }
     }
@@ -288,7 +407,7 @@ std::vector<SCH_MARKER*> SCHEMATIC::ResolveERCExclusions()
 
 std::shared_ptr<BUS_ALIAS> SCHEMATIC::GetBusAlias( const wxString& aLabel ) const
 {
-    for( const SCH_SHEET_PATH& sheet : GetSheets() )
+    for( const SCH_SHEET_PATH& sheet : Hierarchy() )
     {
         for( const std::shared_ptr<BUS_ALIAS>& alias : sheet.LastScreen()->GetBusAliases() )
         {
@@ -309,7 +428,7 @@ std::set<wxString> SCHEMATIC::GetNetClassAssignmentCandidates()
     {
         CONNECTION_SUBGRAPH* firstSubgraph = subgraphList[0];
 
-        if( !firstSubgraph->m_driver_connection->IsBus()
+        if( !firstSubgraph->GetDriverConnection()->IsBus()
                 && firstSubgraph->GetDriverPriority() >= CONNECTION_SUBGRAPH::PRIORITY::PIN )
         {
             names.insert( key.Name );
@@ -322,17 +441,24 @@ std::set<wxString> SCHEMATIC::GetNetClassAssignmentCandidates()
 
 bool SCHEMATIC::ResolveCrossReference( wxString* token, int aDepth ) const
 {
-    SCH_SHEET_LIST sheetList = GetSheets();
     wxString       remainder;
     wxString       ref = token->BeforeFirst( ':', &remainder );
+    KIID_PATH      path( ref );
+    KIID           uuid = path.back();
     SCH_SHEET_PATH sheetPath;
-    SCH_ITEM*      refItem = sheetList.GetItem( KIID( ref ), &sheetPath );
+    SCH_ITEM*      refItem = GetItem( KIID( uuid ), &sheetPath );
+
+    if( path.size() > 1 )
+    {
+        path.pop_back();
+        sheetPath = Hierarchy().GetSheetPathByKIIDPath( path ).value_or( sheetPath );
+    }
 
     if( refItem && refItem->Type() == SCH_SYMBOL_T )
     {
         SCH_SYMBOL* refSymbol = static_cast<SCH_SYMBOL*>( refItem );
 
-        if( refSymbol->ResolveTextVar( &remainder, aDepth + 1 ) )
+        if( refSymbol->ResolveTextVar( &sheetPath, &remainder, aDepth + 1 ) )
             *token = remainder;
         else
             *token = refSymbol->GetRef( &sheetPath, true ) + wxS( ":" ) + remainder;
@@ -343,7 +469,9 @@ bool SCHEMATIC::ResolveCrossReference( wxString* token, int aDepth ) const
     {
         SCH_SHEET* refSheet = static_cast<SCH_SHEET*>( refItem );
 
-        if( refSheet->ResolveTextVar( &remainder, aDepth + 1 ) )
+        sheetPath.push_back( refSheet );
+
+        if( refSheet->ResolveTextVar( &sheetPath, &remainder, aDepth + 1 ) )
             *token = remainder;
 
         return true;    // Cross-reference is resolved whether or not the actual textvar was
@@ -357,7 +485,7 @@ std::map<int, wxString> SCHEMATIC::GetVirtualPageToSheetNamesMap() const
 {
     std::map<int, wxString> namesMap;
 
-    for( const SCH_SHEET_PATH& sheet : GetSheets() )
+    for( const SCH_SHEET_PATH& sheet : Hierarchy() )
     {
         if( sheet.size() == 1 )
             namesMap[sheet.GetVirtualPageNumber()] = _( "<root sheet>" );
@@ -373,7 +501,7 @@ std::map<int, wxString> SCHEMATIC::GetVirtualPageToSheetPagesMap() const
 {
     std::map<int, wxString> pagesMap;
 
-    for( const SCH_SHEET_PATH& sheet : GetSheets() )
+    for( const SCH_SHEET_PATH& sheet : Hierarchy() )
         pagesMap[sheet.GetVirtualPageNumber()] = sheet.GetPageNumber();
 
     return pagesMap;
@@ -417,12 +545,11 @@ wxString SCHEMATIC::ConvertRefsToKIIDs( const wxString& aSource ) const
 
             if( isCrossRef )
             {
-                SCH_SHEET_LIST     sheetList = GetSheets();
                 wxString           remainder;
                 wxString           ref = token.BeforeFirst( ':', &remainder );
                 SCH_REFERENCE_LIST references;
 
-                sheetList.GetSymbols( references );
+                Hierarchy().GetSymbols( references );
 
                 for( size_t jj = 0; jj < references.GetCount(); jj++ )
                 {
@@ -430,7 +557,10 @@ wxString SCHEMATIC::ConvertRefsToKIIDs( const wxString& aSource ) const
 
                     if( ref == refSymbol->GetRef( &references[ jj ].GetSheetPath(), true ) )
                     {
-                        token = refSymbol->m_Uuid.AsString() + wxS( ":" ) + remainder;
+                        KIID_PATH path = references[ jj ].GetSheetPath().Path();
+                        path.push_back( refSymbol->m_Uuid );
+
+                        token = path.AsString() + wxS( ":" ) + remainder;
                         break;
                     }
                 }
@@ -473,17 +603,23 @@ wxString SCHEMATIC::ConvertKIIDsToRefs( const wxString& aSource ) const
 
             if( isCrossRef )
             {
-                SCH_SHEET_LIST sheetList = GetSheets();
                 wxString       remainder;
                 wxString       ref = token.BeforeFirst( ':', &remainder );
+                KIID_PATH      path( ref );
+                KIID           uuid = path.back();
+                SCH_SHEET_PATH sheetPath;
+                SCH_ITEM*      refItem = GetItem( uuid, &sheetPath );
 
-                SCH_SHEET_PATH refSheetPath;
-                SCH_ITEM*      refItem = sheetList.GetItem( KIID( ref ), &refSheetPath );
+                if( path.size() > 1 )
+                {
+                    path.pop_back();
+                    sheetPath = Hierarchy().GetSheetPathByKIIDPath( path ).value_or( sheetPath );
+                }
 
                 if( refItem && refItem->Type() == SCH_SYMBOL_T )
                 {
                     SCH_SYMBOL* refSymbol = static_cast<SCH_SYMBOL*>( refItem );
-                    token = refSymbol->GetRef( &refSheetPath, true ) + wxS( ":" ) + remainder;
+                    token = refSymbol->GetRef( &sheetPath, true ) + wxS( ":" ) + remainder;
                 }
             }
 
@@ -499,17 +635,6 @@ wxString SCHEMATIC::ConvertKIIDsToRefs( const wxString& aSource ) const
 }
 
 
-SCH_SHEET_LIST& SCHEMATIC::GetFullHierarchy() const
-{
-    static SCH_SHEET_LIST hierarchy;
-
-    hierarchy.clear();
-    hierarchy.BuildSheetList( m_rootSheet, false );
-
-    return hierarchy;
-}
-
-
 void SCHEMATIC::SetLegacySymbolInstanceData()
 {
     SCH_SCREENS screens( m_rootSheet );
@@ -546,7 +671,7 @@ void SCHEMATIC::SetSheetNumberAndCount()
 
     // @todo Remove all pseudo page number system is left over from prior to real page number
     //       implementation.
-    for( const SCH_SHEET_PATH& sheet : GetSheets() )
+    for( const SCH_SHEET_PATH& sheet : Hierarchy() )
     {
         if( sheet.Path() == current_sheetpath ) // Current sheet path found
             break;
@@ -563,43 +688,27 @@ void SCHEMATIC::SetSheetNumberAndCount()
 }
 
 
-void SCHEMATIC::RecomputeIntersheetRefs( const std::function<void( SCH_GLOBALLABEL* )>& aItemCallback )
+void SCHEMATIC::RecomputeIntersheetRefs(
+        const std::function<void( SCH_GLOBALLABEL* )>& aItemCallback )
 {
     std::map<wxString, std::set<int>>& pageRefsMap = GetPageRefsMap();
 
     pageRefsMap.clear();
 
-    SCH_SCREENS      screens( Root() );
-    std::vector<int> virtualPageNumbers;
-
-    /* Iterate over screens */
-    for( SCH_SCREEN* screen = screens.GetFirst(); screen != nullptr; screen = screens.GetNext() )
+    for( const SCH_SHEET_PATH& sheet : Hierarchy() )
     {
-        virtualPageNumbers.clear();
-
-        /* Find in which sheets this screen is used */
-        for( const SCH_SHEET_PATH& sheet : GetSheets() )
+        for( SCH_ITEM* item : sheet.LastScreen()->Items().OfType( SCH_GLOBAL_LABEL_T ) )
         {
-            if( sheet.LastScreen() == screen )
-                virtualPageNumbers.push_back( sheet.GetVirtualPageNumber() );
-        }
+            SCH_GLOBALLABEL* global = static_cast<SCH_GLOBALLABEL*>( item );
+            wxString         resolvedLabel = global->GetShownText( &sheet, false );
 
-        for( SCH_ITEM* item : screen->Items() )
-        {
-            if( item->Type() == SCH_GLOBAL_LABEL_T )
-            {
-                SCH_GLOBALLABEL* globalLabel = static_cast<SCH_GLOBALLABEL*>( item );
-                std::set<int>&   virtualpageList = pageRefsMap[globalLabel->GetText()];
-
-                for( const int& pageNo : virtualPageNumbers )
-                    virtualpageList.insert( pageNo );
-            }
+            pageRefsMap[ resolvedLabel ].insert( sheet.GetVirtualPageNumber() );
         }
     }
 
     bool show = Settings().m_IntersheetRefsShow;
 
-    // Refresh all global labels.  Note that we have to collect them first as the
+    // Refresh all visible global labels.  Note that we have to collect them first as the
     // SCH_SCREEN::Update() call is going to invalidate the RTree iterator.
 
     std::vector<SCH_GLOBALLABEL*> currentSheetGlobalLabels;
@@ -616,7 +725,7 @@ void SCHEMATIC::RecomputeIntersheetRefs( const std::function<void( SCH_GLOBALLAB
         if( show )
         {
             if( fields.size() == 1 && fields[0].GetTextPos() == globalLabel->GetPosition() )
-                globalLabel->AutoplaceFields( CurrentSheet().LastScreen(), false );
+                globalLabel->AutoplaceFields( CurrentSheet().LastScreen(), AUTOPLACE_AUTO );
 
             CurrentSheet().LastScreen()->Update( globalLabel );
             aItemCallback( globalLabel );
@@ -625,12 +734,32 @@ void SCHEMATIC::RecomputeIntersheetRefs( const std::function<void( SCH_GLOBALLAB
 }
 
 
+wxString SCHEMATIC::GetOperatingPoint( const wxString& aNetName, int aPrecision,
+                                       const wxString& aRange )
+{
+    wxString spiceNetName( aNetName.Lower() );
+    NETLIST_EXPORTER_SPICE::ConvertToSpiceMarkup( &spiceNetName );
+
+    if( spiceNetName == wxS( "gnd" ) || spiceNetName == wxS( "0" ) )
+        return wxEmptyString;
+
+    auto it = m_operatingPoints.find( spiceNetName );
+
+    if( it != m_operatingPoints.end() )
+        return SPICE_VALUE( it->second ).ToString( { aPrecision, aRange } );
+    else if( m_operatingPoints.empty() )
+        return wxS( "--" );
+    else
+        return wxS( "?" );
+}
+
+
 void SCHEMATIC::FixupJunctions()
 {
-    for( const SCH_SHEET_PATH& sheet : GetSheets() )
-    {
-        SCH_SCREEN* screen = sheet.LastScreen();
+    SCH_SCREENS screens( Root() );
 
+    for( SCH_SCREEN* screen = screens.GetFirst(); screen; screen = screens.GetNext() )
+    {
         std::deque<EDA_ITEM*> allItems;
 
         for( auto item : screen->Items() )
@@ -651,3 +780,212 @@ void SCHEMATIC::FixupJunctions()
         }
     }
 }
+
+
+void SCHEMATIC::OnItemsAdded( std::vector<SCH_ITEM*>& aNewItems )
+{
+    InvokeListeners( &SCHEMATIC_LISTENER::OnSchItemsAdded, *this, aNewItems );
+}
+
+
+void SCHEMATIC::OnItemsRemoved( std::vector<SCH_ITEM*>& aRemovedItems )
+{
+    InvokeListeners( &SCHEMATIC_LISTENER::OnSchItemsRemoved, *this, aRemovedItems );
+}
+
+
+void SCHEMATIC::OnItemsChanged( std::vector<SCH_ITEM*>& aItems )
+{
+    InvokeListeners( &SCHEMATIC_LISTENER::OnSchItemsChanged, *this, aItems );
+}
+
+
+void SCHEMATIC::OnSchSheetChanged()
+{
+    InvokeListeners( &SCHEMATIC_LISTENER::OnSchSheetChanged, *this );
+}
+
+
+void SCHEMATIC::AddListener( SCHEMATIC_LISTENER* aListener )
+{
+    if( !alg::contains( m_listeners, aListener ) )
+        m_listeners.push_back( aListener );
+}
+
+
+void SCHEMATIC::RemoveListener( SCHEMATIC_LISTENER* aListener )
+{
+    auto i = std::find( m_listeners.begin(), m_listeners.end(), aListener );
+
+    if( i != m_listeners.end() )
+    {
+        std::iter_swap( i, m_listeners.end() - 1 );
+        m_listeners.pop_back();
+    }
+}
+
+
+void SCHEMATIC::RemoveAllListeners()
+{
+    m_listeners.clear();
+}
+
+
+void SCHEMATIC::RecordERCExclusions()
+{
+    // Use a sorted sheetList to reduce file churn
+    SCH_SHEET_LIST sheetList = Hierarchy();
+    ERC_SETTINGS&  ercSettings = ErcSettings();
+
+    ercSettings.m_ErcExclusions.clear();
+    ercSettings.m_ErcExclusionComments.clear();
+
+    for( unsigned i = 0; i < sheetList.size(); i++ )
+    {
+        for( SCH_ITEM* item : sheetList[i].LastScreen()->Items().OfType( SCH_MARKER_T ) )
+        {
+            SCH_MARKER* marker = static_cast<SCH_MARKER*>( item );
+
+            if( marker->IsExcluded() )
+            {
+                wxString serialized = marker->SerializeToString();
+                ercSettings.m_ErcExclusions.insert( serialized );
+                ercSettings.m_ErcExclusionComments[ serialized ] = marker->GetComment();
+            }
+        }
+    }
+}
+
+
+void SCHEMATIC::ResolveERCExclusionsPostUpdate()
+{
+    SCH_SHEET_LIST sheetList = Hierarchy();
+
+    for( SCH_MARKER* marker : ResolveERCExclusions() )
+    {
+        SCH_SHEET_PATH errorPath;
+        ignore_unused( sheetList.GetItem( marker->GetRCItem()->GetMainItemID(), &errorPath ) );
+
+        if( errorPath.LastScreen() )
+            errorPath.LastScreen()->Append( marker );
+        else
+            RootScreen()->Append( marker );
+    }
+}
+
+
+EMBEDDED_FILES* SCHEMATIC::GetEmbeddedFiles()
+{
+    return static_cast<EMBEDDED_FILES*>( this );
+}
+
+
+const EMBEDDED_FILES* SCHEMATIC::GetEmbeddedFiles() const
+{
+    return static_cast<const EMBEDDED_FILES*>( this );
+}
+
+
+std::set<KIFONT::OUTLINE_FONT*> SCHEMATIC::GetFonts() const
+{
+    std::set<KIFONT::OUTLINE_FONT*> fonts;
+
+    SCH_SHEET_LIST sheetList = Hierarchy();
+
+    for( const SCH_SHEET_PATH& sheet : sheetList )
+    {
+        for( SCH_ITEM* item : sheet.LastScreen()->Items() )
+        {
+            if( EDA_TEXT* text = dynamic_cast<EDA_TEXT*>( item ) )
+            {
+                KIFONT::FONT* font = text->GetFont();
+
+                if( !font || font->IsStroke() )
+                    continue;
+
+                using EMBEDDING_PERMISSION = KIFONT::OUTLINE_FONT::EMBEDDING_PERMISSION;
+                auto* outline = static_cast<KIFONT::OUTLINE_FONT*>( font );
+
+                if( outline->GetEmbeddingPermission() == EMBEDDING_PERMISSION::EDITABLE
+                    || outline->GetEmbeddingPermission() == EMBEDDING_PERMISSION::INSTALLABLE )
+                {
+                    fonts.insert( outline );
+                }
+            }
+        }
+    }
+
+    return fonts;
+}
+
+
+void SCHEMATIC::EmbedFonts()
+{
+    std::set<KIFONT::OUTLINE_FONT*> fonts = GetFonts();
+
+    for( KIFONT::OUTLINE_FONT* font : fonts )
+    {
+        auto file = GetEmbeddedFiles()->AddFile( font->GetFileName(), false );
+
+        if( !file )
+        {
+            wxLogTrace( "EMBED", "Failed to add font file: %s", font->GetFileName() );
+            continue;
+        }
+
+        file->type = EMBEDDED_FILES::EMBEDDED_FILE::FILE_TYPE::FONT;
+    }
+}
+
+
+std::set<const SCH_SCREEN*> SCHEMATIC::GetSchematicsSharedByMultipleProjects() const
+{
+    std::set<const SCH_SCREEN*> retv;
+
+    wxCHECK( m_rootSheet, retv );
+
+    SCH_SHEET_LIST hierarchy( m_rootSheet );
+    SCH_SCREENS screens( m_rootSheet );
+
+    for( const SCH_SCREEN* screen = screens.GetFirst(); screen; screen = screens.GetNext() )
+    {
+        for( const SCH_ITEM* item : screen->Items().OfType( SCH_SYMBOL_T ) )
+        {
+            const SCH_SYMBOL* symbol = static_cast<const SCH_SYMBOL*>( item );
+
+            const std::vector<SCH_SYMBOL_INSTANCE> symbolInstances = symbol->GetInstances();
+
+            for( const SCH_SYMBOL_INSTANCE& instance : symbolInstances )
+            {
+                if( !hierarchy.HasPath( instance.m_Path ) )
+                {
+                    retv.insert( screen );
+                    break;
+                }
+            }
+
+            if( retv.count( screen ) )
+                break;
+        }
+    }
+
+    return retv;
+}
+
+
+bool SCHEMATIC::IsComplexHierarchy() const
+{
+    wxCHECK( m_rootSheet, false );
+
+    SCH_SCREENS screens( m_rootSheet );
+
+    for( const SCH_SCREEN* screen = screens.GetFirst(); screen; screen = screens.GetNext() )
+    {
+        wxCHECK2( screen, continue );
+
+        if( screen->GetRefCount() > 1 )
+            return true;
+    }
+
+    return false;
+}
