@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2004 Jean-Pierre Charras, jaen-pierre.charras@gipsa-lab.inpg.com
- * Copyright (C) 2004-2022 KiCad Developers, see change_log.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,12 +27,16 @@
 #include <tool/tool_manager.h>
 #include <schematic.h>
 #include <sch_bus_entry.h>
+#include <sch_commit.h>
 #include <sch_junction.h>
 #include <sch_line.h>
 #include <sch_bitmap.h>
+#include <sch_sheet_pin.h>
+#include <sch_table.h>
 #include <tools/ee_selection_tool.h>
 #include <drawing_sheet/ds_proxy_undo_item.h>
 #include <tool/actions.h>
+#include <wx/log.h>
 
 
 /* Functions to undo and redo edit commands.
@@ -96,13 +100,6 @@
  * swapped data is data modified by editing, so not all values are swapped
  */
 
-void SCH_EDIT_FRAME::StartNewUndo()
-{
-    PICKED_ITEMS_LIST* blank = new PICKED_ITEMS_LIST();
-    PushCommandToUndoList( blank );
-}
-
-
 void SCH_EDIT_FRAME::SaveCopyInUndoList( SCH_SCREEN* aScreen, SCH_ITEM* aItem,
                                          UNDO_REDO aCommandType, bool aAppend,
                                          bool aDirtyConnectivity )
@@ -112,7 +109,16 @@ void SCH_EDIT_FRAME::SaveCopyInUndoList( SCH_SCREEN* aScreen, SCH_ITEM* aItem,
     wxCHECK( aItem, /* void */ );
 
     if( aDirtyConnectivity )
+    {
+        if( !aItem->IsConnectivityDirty() && aItem->Connection()
+            && ( aItem->Connection()->Name() == m_highlightedConn
+                 || aItem->Connection()->HasDriverChanged() ) )
+        {
+            m_highlightedConnChanged = true;
+        }
+
         aItem->SetConnectivityDirty();
+    }
 
     PICKED_ITEMS_LIST* lastUndo = PopCommandFromUndoList();
 
@@ -187,11 +193,25 @@ void SCH_EDIT_FRAME::SaveCopyInUndoList( const PICKED_ITEMS_LIST& aItemsList,
     }
 
     if( !commandToUndo )
+    {
         commandToUndo = new PICKED_ITEMS_LIST();
+        commandToUndo->SetDescription( aItemsList.GetDescription() );
+    }
 
     // Copy picker list:
     if( !commandToUndo->GetCount() )
+    {
         commandToUndo->CopyList( aItemsList );
+
+        for( const std::unique_ptr<SCH_ITEM>& item : GetRepeatItems() )
+        {
+            EDA_ITEM* repeatItemClone = item->Clone();
+            repeatItemClone->SetFlags( UR_TRANSIENT );
+
+            ITEM_PICKER repeatItemPicker( nullptr, repeatItemClone, UNDO_REDO::REPEAT_ITEM );
+            commandToUndo->PushItem( repeatItemPicker );
+        }
+    }
     else
     {
         // Unless we are appending, in which case, get the picker items
@@ -208,9 +228,6 @@ void SCH_EDIT_FRAME::SaveCopyInUndoList( const PICKED_ITEMS_LIST& aItemsList,
         if( !sch_item )
             continue;
 
-        if( aDirtyConnectivity )
-            sch_item->SetConnectivityDirty();
-
         UNDO_REDO command = commandToUndo->GetPickedItemStatus( ii );
 
         if( command == UNDO_REDO::UNSPECIFIED )
@@ -235,8 +252,8 @@ void SCH_EDIT_FRAME::SaveCopyInUndoList( const PICKED_ITEMS_LIST& aItemsList,
 
         case UNDO_REDO::NEWITEM:
         case UNDO_REDO::DELETED:
-        case UNDO_REDO::EXCHANGE_T:
         case UNDO_REDO::PAGESETTINGS:
+        case UNDO_REDO::REPEAT_ITEM:
             break;
 
         default:
@@ -262,6 +279,17 @@ void SCH_EDIT_FRAME::SaveCopyInUndoList( const PICKED_ITEMS_LIST& aItemsList,
 
 void SCH_EDIT_FRAME::PutDataInPreviousState( PICKED_ITEMS_LIST* aList )
 {
+    std::vector<SCH_ITEM*> bulkAddedItems;
+    std::vector<SCH_ITEM*> bulkRemovedItems;
+    std::vector<SCH_ITEM*> bulkChangedItems;
+    std::set<SCH_TABLE*>   changedTables;
+    bool                   dirtyConnectivity = false;
+    bool                   rebuildHierarchyNavigator = false;
+    bool                   refreshHierarchy = false;
+    SCH_CLEANUP_FLAGS      connectivityCleanUp = NO_CLEANUP;
+    SCH_SHEET_LIST         sheets;
+    bool                   clearedRepeatItems = false;
+
     // Undo in the reverse order of list creation: (this can allow stacked changes like the
     // same item can be changed and deleted in the same complex command).
     // After hitting 0, subtracting 1 will roll the value over to its max representation
@@ -271,80 +299,195 @@ void SCH_EDIT_FRAME::PutDataInPreviousState( PICKED_ITEMS_LIST* aList )
         EDA_ITEM*   eda_item = aList->GetPickedItem( ii );
         SCH_SCREEN* screen = dynamic_cast<SCH_SCREEN*>( aList->GetScreenForItem( ii ) );
 
-        wxCHECK( screen, /* void */ );
-
         eda_item->SetFlags( aList->GetPickerFlags( ii ) );
         eda_item->ClearEditFlags();
         eda_item->ClearTempFlags();
 
-        if( status == UNDO_REDO::NOP )
-        {
-            continue;
-        }
+        SCH_ITEM*   schItem = dynamic_cast<SCH_ITEM*>( eda_item );
+
+        // Set connectable object connectivity status.
+        auto updateConnectivityFlag =
+                [&]()
+                {
+                    if( schItem->IsConnectable() )
+                    {
+                        schItem->SetConnectivityDirty();
+
+                        if( schItem->Type() == SCH_SYMBOL_T )
+                        {
+                            SCH_SYMBOL* symbol = static_cast<SCH_SYMBOL*>( schItem );
+
+                            wxCHECK( symbol, /* void */ );
+
+                            for( SCH_PIN* pin : symbol->GetPins() )
+                                pin->SetConnectivityDirty();
+                        }
+                        else if( schItem->Type() == SCH_SHEET_T )
+                        {
+                            SCH_SHEET* sheet = static_cast<SCH_SHEET*>( schItem );
+
+                            wxCHECK( sheet, /* void */ );
+
+                            for( SCH_SHEET_PIN* pin : sheet->GetPins() )
+                                pin->SetConnectivityDirty();
+                        }
+
+                        m_highlightedConnChanged = true;
+                        dirtyConnectivity = true;
+
+                        // Do a local clean up if there are any connectable objects in the commit
+                        if( connectivityCleanUp == NO_CLEANUP )
+                            connectivityCleanUp = LOCAL_CLEANUP;
+
+                        // Do a full rebauild of the connectivity if there is a sheet in the commit
+                        if( schItem->Type() == SCH_SHEET_T )
+                            connectivityCleanUp = GLOBAL_CLEANUP;
+                    }
+                    else if( schItem->Type() == SCH_RULE_AREA_T )
+                    {
+                        dirtyConnectivity = true;
+                    }
+                };
+
         if( status == UNDO_REDO::NEWITEM )
         {
+            if( schItem )
+                updateConnectivityFlag();
+
             // If we are removing the current sheet, get out first
-            if( SCH_SHEET* sheet = dyn_cast<SCH_SHEET*>( eda_item ) )
+            if( eda_item->Type() == SCH_SHEET_T )
             {
-                if( sheet->GetScreen() == GetScreen() )
-                    GetToolManager()->RunAction( EE_ACTIONS::leaveSheet );
+                rebuildHierarchyNavigator = true;
+                refreshHierarchy = true;
+
+                if( static_cast<SCH_SHEET*>( eda_item )->GetScreen() == GetScreen() )
+                    GetToolManager()->PostAction( EE_ACTIONS::leaveSheet );
             }
 
             RemoveFromScreen( eda_item, screen );
             aList->SetPickedItemStatus( UNDO_REDO::DELETED, ii );
+
+            bulkRemovedItems.emplace_back( schItem );
         }
         else if( status == UNDO_REDO::DELETED )
         {
+            if( eda_item->Type() == SCH_SHEET_T )
+            {
+                rebuildHierarchyNavigator = true;
+                refreshHierarchy = true;
+            }
+
+            if( schItem )
+                updateConnectivityFlag();
+
             // deleted items are re-inserted on undo
             AddToScreen( eda_item, screen );
             aList->SetPickedItemStatus( UNDO_REDO::NEWITEM, ii );
+
+            bulkAddedItems.emplace_back( schItem );
         }
         else if( status == UNDO_REDO::PAGESETTINGS )
         {
-            SetCurrentSheet( *m_schematic->GetSheets().FindSheetForScreen( screen ) );
-            DisplayCurrentSheet();
+            // Lazy eval of sheet list; this is expensive even when unsorted
+            if( sheets.empty() )
+                sheets = m_schematic->Hierarchy();
+
+            SCH_SHEET_PATH undoSheet = sheets.FindSheetForScreen( screen );
+
+            if( GetCurrentSheet() != undoSheet )
+            {
+                SetCurrentSheet( undoSheet );
+                DisplayCurrentSheet();
+            }
 
             // swap current settings with stored settings
             DS_PROXY_UNDO_ITEM  alt_item( this );
             DS_PROXY_UNDO_ITEM* item = static_cast<DS_PROXY_UNDO_ITEM*>( eda_item );
             item->Restore( this );
-            *item = alt_item;
+            *item = std::move( alt_item );
         }
-        else if( SCH_ITEM* item = dynamic_cast<SCH_ITEM*>( eda_item ) )
+        else if( status == UNDO_REDO::REPEAT_ITEM )
         {
-            // everything else is modified in place
-            SCH_ITEM* alt_item = static_cast<SCH_ITEM*>( aList->GetPickedItemLink( ii ) );
+            if( !clearedRepeatItems )
+            {
+                ClearRepeatItemsList();
+                clearedRepeatItems = true;
+            }
+
+            if( schItem )
+            {
+                updateConnectivityFlag();
+                AddCopyForRepeatItem( schItem );
+
+                if( schItem->Type() == SCH_SHEET_T )
+                {
+                    rebuildHierarchyNavigator = true;
+                    refreshHierarchy = true;
+                }
+            }
+        }
+        else if( schItem )
+        {
+            SCH_ITEM* itemCopy = dynamic_cast<SCH_ITEM*>( aList->GetPickedItemLink( ii ) );
+
+            wxCHECK2( itemCopy, continue );
+
+            if( schItem->HasConnectivityChanges( itemCopy, &GetCurrentSheet() ) )
+                updateConnectivityFlag();
 
             // The root sheet is a pseudo object that owns the root screen object but is not on
             // the root screen so do not attempt to remove it from the screen it owns.
-            if( item != &Schematic().Root() )
-                RemoveFromScreen( item, screen );
+            if( schItem != &Schematic().Root() )
+                RemoveFromScreen( schItem, screen );
 
             switch( status )
             {
             case UNDO_REDO::CHANGED:
-                item->SwapData( alt_item );
-
-                // Special cases for items which have instance data
-                if( item->GetParent() && item->GetParent()->Type() == SCH_SYMBOL_T
-                        && item->Type() == SCH_FIELD_T )
+                if( schItem->Type() == SCH_SHEET_T )
                 {
-                    SCH_FIELD*  field = static_cast<SCH_FIELD*>( item );
-                    SCH_SYMBOL* symbol = static_cast<SCH_SYMBOL*>( item->GetParent() );
+                    const SCH_SHEET* origSheet = static_cast<const SCH_SHEET*>( schItem );
+                    const SCH_SHEET* copySheet = static_cast<const SCH_SHEET*>( itemCopy );
 
-                    if( field->GetId() == REFERENCE_FIELD )
+                    wxCHECK2( origSheet && copySheet, continue );
+
+                    if( ( origSheet->GetName() != copySheet->GetName() )
+                      || ( origSheet->GetFileName() != copySheet->GetFileName() )
+                      || origSheet->HasPageNumberChanges( *copySheet ) )
                     {
-                        symbol->SetRef( m_schematic->GetSheets().FindSheetForScreen( screen ),
-                                        field->GetText() );
+                        rebuildHierarchyNavigator = true;
+                    }
+
+                    // Sheet name changes do not require rebuilding the hiearchy.
+                    if( ( origSheet->GetFileName() != copySheet->GetFileName() )
+                      || origSheet->HasPageNumberChanges( *copySheet ) )
+                    {
+                        refreshHierarchy = true;
                     }
                 }
 
-                break;
+                schItem->SwapData( itemCopy );
+                bulkChangedItems.emplace_back( schItem );
+
+                // Special cases for items which have instance data
+                if( schItem->GetParent() && schItem->GetParent()->Type() == SCH_SYMBOL_T
+                  && schItem->Type() == SCH_FIELD_T )
+                {
+                    SCH_FIELD*  field = static_cast<SCH_FIELD*>( schItem );
+                    SCH_SYMBOL* symbol = static_cast<SCH_SYMBOL*>( schItem->GetParent() );
+
+                    if( field->GetId() == REFERENCE_FIELD )
+                    {
+                        // Lazy eval of sheet list; this is expensive even when unsorted
+                        if( sheets.empty() )
+                            sheets = m_schematic->Hierarchy();
+
+                        SCH_SHEET_PATH sheet = sheets.FindSheetForScreen( screen );
+                        symbol->SetRef( &sheet, field->GetText() );
+                    }
+
+                    bulkChangedItems.emplace_back( symbol );
+                }
 
-            case UNDO_REDO::EXCHANGE_T:
-                aList->SetPickedItem( alt_item, ii );
-                aList->SetPickedItemLink( item, ii );
-                item = alt_item;
                 break;
 
             default:
@@ -353,18 +496,49 @@ void SCH_EDIT_FRAME::PutDataInPreviousState( PICKED_ITEMS_LIST* aList )
                 break;
             }
 
-            if( item->Type() == SCH_SYMBOL_T )
+            if( schItem->Type() == SCH_SYMBOL_T )
             {
-                SCH_SYMBOL* sym = static_cast<SCH_SYMBOL*>( item );
+                SCH_SYMBOL* sym = static_cast<SCH_SYMBOL*>( schItem );
                 sym->UpdatePins();
             }
 
-            if( item != &Schematic().Root() )
-                AddToScreen( item, screen );
+            if( schItem != &Schematic().Root() )
+                AddToScreen( schItem, screen );
         }
     }
 
     GetCanvas()->GetView()->ClearHiddenFlags();
+
+    // Notify our listeners
+    if( bulkAddedItems.size() > 0 )
+        Schematic().OnItemsAdded( bulkAddedItems );
+
+    if( bulkRemovedItems.size() > 0 )
+        Schematic().OnItemsRemoved( bulkRemovedItems );
+
+    if( bulkChangedItems.size() > 0 )
+        Schematic().OnItemsChanged( bulkChangedItems );
+
+    if( refreshHierarchy )
+        Schematic().RefreshHierarchy();
+
+    if( dirtyConnectivity )
+    {
+        wxLogTrace( wxS( "CONN_PROFILE" ),
+                    wxS( "Undo/redo %s clean up connectivity rebuild." ),
+                    ( connectivityCleanUp == LOCAL_CLEANUP ) ? wxS( "local" ) : wxS( "global" ) );
+
+        SCH_COMMIT localCommit( m_toolManager );
+
+        RecalculateConnections( &localCommit, connectivityCleanUp );
+
+        if( connectivityCleanUp == GLOBAL_CLEANUP )
+            SetSheetNumberAndCount();
+    }
+
+    // Update the hierarchy navigator when there are sheet changes.
+    if( rebuildHierarchyNavigator )
+        UpdateHierarchyNavigator();
 }
 
 
@@ -373,8 +547,7 @@ void SCH_EDIT_FRAME::RollbackSchematicFromUndo()
     PICKED_ITEMS_LIST* undo = PopCommandFromUndoList();
 
     // Skip empty frames
-    while( undo && ( !undo->GetCount()
-            || ( undo->GetCount() == 1 && undo->GetPickedItemStatus( 0 ) == UNDO_REDO::NOP ) ) )
+    while( undo && !undo->GetCount() )
     {
         delete undo;
         undo = PopCommandFromUndoList();
@@ -387,13 +560,9 @@ void SCH_EDIT_FRAME::RollbackSchematicFromUndo()
                                        {
                                            delete aItem;
                                        } );
+
         delete undo;
 
-        SetSheetNumberAndCount();
-        UpdateHierarchyNavigator();
-
-        TestDanglingEnds();
-
         m_toolManager->GetTool<EE_SELECTION_TOOL>()->RebuildSelection();
     }
 
@@ -406,18 +575,29 @@ void SCH_EDIT_FRAME::ClearUndoORRedoList( UNDO_REDO_LIST whichList, int aItemCou
     if( aItemCount == 0 )
         return;
 
-    UNDO_REDO_CONTAINER& list = whichList == UNDO_LIST ? m_undoList : m_redoList;
+    UNDO_REDO_CONTAINER& list = ( whichList == UNDO_LIST ) ? m_undoList : m_redoList;
 
-    for( PICKED_ITEMS_LIST* command : list.m_CommandsList )
+    if( aItemCount < 0 )
     {
-        command->ClearListAndDeleteItems( []( EDA_ITEM* aItem )
-                                          {
-                                              delete aItem;
-                                          } );
-        delete command;
+        list.ClearCommandList();
     }
+    else
+    {
+        for( int ii = 0; ii < aItemCount; ii++ )
+        {
+            if( list.m_CommandsList.size() == 0 )
+                break;
 
-    list.m_CommandsList.clear();
+            PICKED_ITEMS_LIST* curr_cmd = list.m_CommandsList[0];
+            list.m_CommandsList.erase( list.m_CommandsList.begin() );
+
+            curr_cmd->ClearListAndDeleteItems( []( EDA_ITEM* aItem )
+                                               {
+                                                   delete aItem;
+                                               } );
+            delete curr_cmd;    // Delete command
+        }
+    }
 }
 
 
