@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2014 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2008 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2004-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * Copyright (C) 2017 CERN
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  *
@@ -30,16 +30,13 @@
 
 #include <sch_base_frame.h>
 #include <sch_screen.h>
-#include <lib_item.h>
 #include <ee_collectors.h>
+#include <symbol_tree_pane.h>
 #include <optional>
 
 class SCH_EDIT_FRAME;
 class SYMBOL_LIB_TABLE;
 class LIB_SYMBOL;
-class LIB_FIELD;
-class DIALOG_LIB_TEXT_PROPERTIES;
-class SYMBOL_TREE_PANE;
 class LIB_TREE_NODE;
 class LIB_ID;
 class LIB_SYMBOL_LIBRARY_MANAGER;
@@ -57,6 +54,8 @@ public:
 
     ~SYMBOL_EDIT_FRAME() override;
 
+    std::unique_ptr<GRID_HELPER> MakeGridHelper() override;
+
     /**
      * Switch currently used canvas ( Cairo / OpenGL).
      */
@@ -91,7 +90,7 @@ public:
      */
     wxString SetCurLib( const wxString& aLibNickname );
 
-    LIB_TREE_NODE* GetCurrentTreeNode() const;
+    LIB_TREE* GetLibTree() const override { return m_treePane->GetLibTree(); }
 
     /**
      * Return the LIB_ID of the library or symbol selected in the symbol tree.
@@ -133,8 +132,11 @@ public:
 
     /**
      * Create a new symbol in the selected library.
+     *
+     * @param newName is the name of the symbol to derive the new symbol from or empty
+     *                     to create a new root symbol.
      */
-    void CreateNewSymbol();
+    void CreateNewSymbol( const wxString& newName = wxEmptyString );
 
     void ImportSymbol();
     void ExportSymbol();
@@ -147,7 +149,7 @@ public:
     /**
      * Save the currently selected symbol to a new name and/or location.
      */
-    void SaveSymbolAs();
+    void SaveSymbolCopyAs( bool aOpenCopy );
 
     /**
      * Save the currently selected library to a new file.
@@ -180,9 +182,11 @@ public:
 
     void OnSelectUnit( wxCommandEvent& event );
 
-    void OnToggleSymbolTree( wxCommandEvent& event );
+    void ToggleProperties() override;
 
-    bool IsSymbolTreeShown() const;
+    void ToggleLibraryTree() override;
+    bool IsLibraryTreeShown() const override;
+    void FocusLibraryTreeInput() override;
     void FreezeLibraryTree();
     void ThawLibraryTree();
 
@@ -229,14 +233,17 @@ public:
     void OnModify() override;
 
     int GetUnit() const { return m_unit; }
-    void SetUnit( int aUnit ) { m_unit = aUnit; }
+    void SetUnit( int aUnit );
 
-    int GetConvert() const { return m_convert; }
-    void SetConvert( int aConvert ) { m_convert = aConvert; }
+    int  GetBodyStyle() const { return m_bodyStyle; }
+    void SetBodyStyle( int aBodyStyle ) { m_bodyStyle = aBodyStyle; }
 
     bool GetShowDeMorgan() const { return m_showDeMorgan; }
     void SetShowDeMorgan( bool show ) { m_showDeMorgan = show; }
 
+    bool GetShowInvisibleFields();
+    bool GetShowInvisiblePins();
+
     void ClearMsgPanel() override
     {
         UpdateSymbolMsgPanelInfo();
@@ -247,7 +254,20 @@ public:
         UpdateSymbolMsgPanelInfo();
     }
 
-    bool IsSymbolFromSchematic() const { return m_isSymbolFromSchematic; }
+    /**
+     * Update the main window title bar with the current library name and read only status
+     * of the library.
+     */
+    void UpdateTitle();
+
+    bool IsSymbolFromSchematic() const
+    {
+        // If we've already vetted closing this window, then we have no symbol anymore
+        if( m_isClosing )
+            return false;
+
+        return m_isSymbolFromSchematic;
+    }
 
     bool IsSymbolFromLegacyLibrary() const;
 
@@ -263,14 +283,15 @@ public:
      * Because a symbol in library editor does not have a lot of primitives, the full data is
      * duplicated. It is not worth to try to optimize this save function.
      */
-    void SaveCopyInUndoList( EDA_ITEM* aItem, UNDO_REDO aUndoType = UNDO_REDO::LIBEDIT,
-                             bool aAppend = false );
+    void SaveCopyInUndoList( const wxString& aDescription, LIB_SYMBOL* aSymbol,
+                             UNDO_REDO aUndoType = UNDO_REDO::LIBEDIT );
+
+    void PushSymbolToUndoList( const wxString& aDescription, LIB_SYMBOL* aSymbolCopy,
+                               UNDO_REDO aUndoType = UNDO_REDO::LIBEDIT );
 
     void GetSymbolFromUndoList();
     void GetSymbolFromRedoList();
 
-    void RollbackSymbolFromUndo();
-
     /**
      * Free the undo or redo list from \a aList element.
      *
@@ -290,10 +311,10 @@ public:
      *
      * @param aLibId is the #LIB_ID of the symbol to select.
      * @param aUnit the unit to show
-     * @param aConvert the DeMorgan variant to show
+     * @param aBodyStyle the DeMorgan variant to show
      * @return true if the symbol defined by \a aLibId was loaded.
      */
-    bool LoadSymbol( const LIB_ID& aLibId, int aUnit, int aConvert );
+    bool LoadSymbol( const LIB_ID& aLibId, int aUnit, int aBodyStyle );
 
     /**
      * Print a page.
@@ -305,7 +326,7 @@ public:
      * @param aFullFileName is the full filename
      * @param aOffset is a plot offset, in iu
      */
-    void SVGPlotSymbol( const wxString& aFullFileName, VECTOR2I aOffset );
+    void SVGPlotSymbol( const wxString& aFullFileName, const VECTOR2I& aOffset );
 
     /**
      * Synchronize the library manager to the symbol library table, and then the symbol tree
@@ -314,13 +335,6 @@ public:
     void SyncLibraries( bool aShowProgress, bool aPreloadCancelled = false,
                         const wxString& aForceRefresh = wxEmptyString );
 
-    /**
-     * Filter, sort, and redisplay the library tree.
-     *
-     * Does NOT synchronize it with libraries in disk.
-     */
-    void RegenerateLibraryTree();
-
     /**
      * Redisplay the library tree.  Used after changing modified states, descriptions, etc.
      */
@@ -335,7 +349,7 @@ public:
      * Return either the symbol selected in the symbol tree (if context menu is active) or the
      * symbol on the editor canvas.
      */
-    LIB_ID GetTargetLibId() const;
+    LIB_ID GetTargetLibId() const override;
 
     /**
      * @return a list of selected items in the symbol tree
@@ -347,7 +361,7 @@ public:
     /**
      * Called after the preferences dialog is run.
      */
-    void CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVarsChanged ) override;
+    void CommonSettingsChanged( int aFlags ) override;
 
     void ShowChangedLanguage() override;
 
@@ -367,6 +381,8 @@ public:
 
     void KiwayMailIn( KIWAY_EXPRESS& mail ) override;
 
+    void FocusOnItem( SCH_ITEM* aItem );
+
     /**
      * Load a symbol from the schematic to edit in place.
      *
@@ -399,14 +415,13 @@ protected:
 
     void doReCreateMenuBar() override;
 
+    void updateSelectionFilterVisbility() override;
+
 private:
     // Set up the tool framework
     void setupTools();
 
-    EDA_LIST_DIALOG* buildSaveAsDialog( const wxString& aSymbolName,
-                                        const wxString& aLibraryPreselect );
-
-    void saveSymbolAs();
+    void saveSymbolCopyAs( bool aOpenCopy );
 
     /**
      * Save the changes to the current library.
@@ -420,12 +435,6 @@ private:
      */
     bool saveLibrary( const wxString& aLibrary, bool aNewFile );
 
-    /**
-     * Update the main window title bar with the current library name and read only status
-     * of the library.
-     */
-    void updateTitle();
-
     /**
      * Set the current active library to \a aLibrary.
      *
@@ -440,10 +449,10 @@ private:
      *
      * @param aAliasName The symbol alias name to load from the current library.
      * @param aUnit Unit to be selected
-     * @param aConvert Convert to be selected
+     * @param aBodyStyle Convert to be selected
      * @return true if the symbol loaded correctly.
      */
-    bool LoadSymbolFromCurrentLib( const wxString& aAliasName, int aUnit = 0, int aConvert = 0 );
+    bool LoadSymbolFromCurrentLib( const wxString& aAliasName, int aUnit = 0, int aBodyStyle = 0 );
 
     /**
      * Create a copy of \a aLibEntry into memory.
@@ -452,11 +461,11 @@ private:
      * @param aLibrary the path to the library file that \a aLibEntry was loaded from.  This is
      *                 for error messaging purposes only.
      * @param aUnit the initial unit to show.
-     * @param aConvert the initial DeMorgan variant to show.
+     * @param aBodyStyle the initial DeMorgan variant to show.
      * @return True if a copy of \a aLibEntry was successfully copied.
      */
     bool LoadOneLibrarySymbolAux( LIB_SYMBOL* aLibEntry, const wxString& aLibrary, int aUnit,
-                                  int aConvert );
+                                  int aBodyStyle );
 
     ///< Create a backup copy of a file with requested extension.
     bool backupFile( const wxFileName& aOriginalFile, const wxString& aBackupExt );
@@ -559,8 +568,8 @@ private:
     // The unit number to edit and show
     int         m_unit;
 
-    // Show the normal shape ( m_convert <= 1 ) or the converted shape ( m_convert > 1 )
-    int         m_convert;
+    // Show the normal shape (m_bodyStyle <= 1) or the DeMorgan converted shape (m_bodyStyle > 1)
+    int         m_bodyStyle;
 
     ///< Flag if the symbol being edited was loaded directly from a schematic.
     bool        m_isSymbolFromSchematic;
