@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2018 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2008 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2004-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,10 +26,10 @@
 #include <bitmaps.h>
 #include <symbol_library_common.h>
 #include <confirm.h>
-#include <dialog_choose_symbol.h>
 #include <dialogs/html_message_box.h>
 #include <eeschema_id.h>
 #include <eeschema_settings.h>
+#include <gal/graphics_abstraction_layer.h>
 #include <kiface_base.h>
 #include <kiway.h>
 #include <kiway_express.h>
@@ -37,12 +37,15 @@
 #include <symbol_viewer_frame.h>
 #include <widgets/msgpanel.h>
 #include <widgets/wx_listbox.h>
+#include <widgets/wx_aui_utils.h>
+#include <widgets/wx_progress_reporters.h>
 #include <sch_view.h>
 #include <sch_painter.h>
 #include <symbol_lib_table.h>
 #include <symbol_tree_model_adapter.h>
 #include <pgm_base.h>
 #include <project/project_file.h>
+#include <project_sch.h>
 #include <settings/settings_manager.h>
 #include <symbol_async_loader.h>
 #include <tool/action_toolbar.h>
@@ -57,8 +60,8 @@
 #include <tools/symbol_editor_control.h>
 #include <tools/ee_inspection_tool.h>
 #include <view/view_controls.h>
-#include <widgets/wx_progress_reporters.h>
 #include <wx/srchctrl.h>
+#include <wx/log.h>
 
 #include <default_values.h>
 #include <string_utils.h>
@@ -68,7 +71,7 @@
 LIB_ID SYMBOL_VIEWER_FRAME::m_currentSymbol;
 
 int SYMBOL_VIEWER_FRAME::m_unit = 1;
-int SYMBOL_VIEWER_FRAME::m_convert = 1;
+int SYMBOL_VIEWER_FRAME::m_bodyStyle = 1;
 bool SYMBOL_VIEWER_FRAME::m_show_progress = true;
 
 
@@ -78,7 +81,6 @@ BEGIN_EVENT_TABLE( SYMBOL_VIEWER_FRAME, SCH_BASE_FRAME )
     EVT_ACTIVATE( SYMBOL_VIEWER_FRAME::OnActivate )
 
     // Toolbar events
-    EVT_TOOL( ID_LIBVIEW_SELECT_PART, SYMBOL_VIEWER_FRAME::OnSelectSymbol )
     EVT_TOOL( ID_LIBVIEW_NEXT, SYMBOL_VIEWER_FRAME::onSelectNextSymbol )
     EVT_TOOL( ID_LIBVIEW_PREVIOUS, SYMBOL_VIEWER_FRAME::onSelectPreviousSymbol )
     EVT_CHOICE( ID_LIBVIEW_SELECT_UNIT_NUMBER, SYMBOL_VIEWER_FRAME::onSelectSymbolUnit )
@@ -92,42 +94,26 @@ BEGIN_EVENT_TABLE( SYMBOL_VIEWER_FRAME, SCH_BASE_FRAME )
 
     // Menu (and/or hotkey) events
     EVT_MENU( wxID_CLOSE, SYMBOL_VIEWER_FRAME::CloseLibraryViewer )
-    EVT_MENU( ID_GRID_SETTINGS, SCH_BASE_FRAME::OnGridSettings )
 
     EVT_UPDATE_UI( ID_LIBVIEW_SELECT_UNIT_NUMBER, SYMBOL_VIEWER_FRAME::onUpdateUnitChoice )
 
 END_EVENT_TABLE()
 
 
-#define LIB_VIEW_NAME "ViewlibFrame"
-#define LIB_VIEW_NAME_MODAL "ViewlibFrameModal"
-
-#define LIB_VIEW_STYLE ( KICAD_DEFAULT_DRAWFRAME_STYLE )
-#define LIB_VIEW_STYLE_MODAL ( KICAD_DEFAULT_DRAWFRAME_STYLE | wxFRAME_FLOAT_ON_PARENT )
-
-
-SYMBOL_VIEWER_FRAME::SYMBOL_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAME_T aFrameType,
-                                          const wxString& aLibraryName ) :
-    SCH_BASE_FRAME( aKiway, aParent, aFrameType, _( "Symbol Library Browser" ),
-                    wxDefaultPosition, wxDefaultSize,
-                    aFrameType == FRAME_SCH_VIEWER_MODAL ? LIB_VIEW_STYLE_MODAL : LIB_VIEW_STYLE,
-                    aFrameType == FRAME_SCH_VIEWER_MODAL ? LIB_VIEW_NAME_MODAL : LIB_VIEW_NAME ),
+SYMBOL_VIEWER_FRAME::SYMBOL_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
+        SCH_BASE_FRAME( aKiway, aParent, FRAME_SCH_VIEWER, _( "Symbol Library Browser" ),
+                        wxDefaultPosition, wxDefaultSize, KICAD_DEFAULT_DRAWFRAME_STYLE,
+                        LIB_VIEW_FRAME_NAME ),
         m_unitChoice( nullptr ),
         m_libList( nullptr ),
-        m_symbolList( nullptr ),
-        m_previewItem( nullptr )
+        m_symbolList( nullptr )
 {
-    wxASSERT( aFrameType == FRAME_SCH_VIEWER || aFrameType == FRAME_SCH_VIEWER_MODAL );
-
-    if( aFrameType == FRAME_SCH_VIEWER_MODAL )
-        SetModal( true );
-
-    m_aboutTitle = _HKI( "KiCad Symbol Library Viewer" );
+    m_aboutTitle = _HKI( "KiCad Symbol Library Browser" );
 
     // Force the frame name used in config. the lib viewer frame has a name
     // depending on aFrameType (needed to identify the frame by wxWidgets),
     // but only one configuration is preferable.
-    m_configName = LIB_VIEW_NAME;
+    m_configName = LIB_VIEW_FRAME_NAME;
 
     // Give an icon
     wxIcon  icon;
@@ -194,8 +180,8 @@ SYMBOL_VIEWER_FRAME::SYMBOL_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAM
     m_symbolFilter->SetMinSize( wxSize( -1, GetTextExtent( wxT( "qb" ) ).y + 10 ) );
 #endif
 
-    m_symbolList = new WX_LISTBOX( symbolPanel, ID_LIBVIEW_SYM_LIST, wxDefaultPosition, wxDefaultSize,
-                                   0, nullptr, wxLB_HSCROLL | wxNO_BORDER );
+    m_symbolList = new WX_LISTBOX( symbolPanel, ID_LIBVIEW_SYM_LIST, wxDefaultPosition,
+                                   wxDefaultSize, 0, nullptr, wxLB_HSCROLL | wxNO_BORDER );
     symbolSizer->Add( m_symbolList, 1, wxEXPAND, 5 );
 
     symbolPanel->SetSizer( symbolSizer );
@@ -203,18 +189,7 @@ SYMBOL_VIEWER_FRAME::SYMBOL_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAM
 
     // Preload libraries
     loadAllLibraries();
-
-    if( aLibraryName.empty() )
-    {
-        ReCreateLibList();
-    }
-    else
-    {
-        m_currentSymbol.SetLibNickname( aLibraryName );
-        m_currentSymbol.SetLibItemName( "" );
-        m_unit = 1;
-        m_convert = 1;
-    }
+    ReCreateLibList();
 
     m_selection_changed = false;
 
@@ -235,55 +210,18 @@ SYMBOL_VIEWER_FRAME::SYMBOL_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAM
 
     m_auimgr.AddPane( GetCanvas(), EDA_PANE().Canvas().Name( "DrawFrame" ).Center() );
 
-    m_auimgr.GetPane( libPanel ).Show( aLibraryName.empty() );
-
     m_auimgr.Update();
 
     if( m_libListWidth > 0 )
-    {
-        wxAuiPaneInfo& treePane = m_auimgr.GetPane( "Libraries" );
-
-        // wxAUI hack: force width by setting MinSize() and then Fixed()
-        // thanks to ZenJu http://trac.wxwidgets.org/ticket/13180
-        treePane.MinSize( m_libListWidth, -1 );
-        treePane.Fixed();
-        m_auimgr.Update();
-
-        // now make it resizable again
-        treePane.Resizable();
-        m_auimgr.Update();
-
-        // Note: DO NOT call m_auimgr.Update() anywhere after this; it will nuke the size
-        // back to minimum.
-        treePane.MinSize( 100, -1 );
-    }
+        SetAuiPaneSize( m_auimgr, m_auimgr.GetPane( "Libraries" ), m_libListWidth, -1 );
 
     if( m_symbolListWidth > 0 )
-    {
-        wxAuiPaneInfo& treePane = m_auimgr.GetPane( "Symbols" );
-
-        // wxAUI hack: force width by setting MinSize() and then Fixed()
-        // thanks to ZenJu http://trac.wxwidgets.org/ticket/13180
-        treePane.MinSize( m_symbolListWidth, -1 );
-        treePane.Fixed();
-        m_auimgr.Update();
-
-        // now make it resizable again
-        treePane.Resizable();
-        m_auimgr.Update();
-
-        // Note: DO NOT call m_auimgr.Update() anywhere after this; it will nuke the size
-        // back to minimum.
-        treePane.MinSize( 100, -1 );
-    }
+        SetAuiPaneSize( m_auimgr, m_auimgr.GetPane( "Symbols" ), m_symbolListWidth, -1 );
 
     FinishAUIInitialization();
 
-    if( !IsModal() )        // For modal mode, calling ShowModal() will show this frame
-    {
-        Raise();
-        Show( true );
-    }
+    Raise();
+    Show( true );
 
     SyncView();
     GetCanvas()->SetCanFocus( false );
@@ -298,7 +236,7 @@ SYMBOL_VIEWER_FRAME::SYMBOL_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAM
     bbox.SetOrigin( -max_size_x / 2, -max_size_y / 2 );
     bbox.SetSize( max_size_x, max_size_y );
     GetCanvas()->GetView()->SetBoundary( bbox );
-    GetToolManager()->RunAction( ACTIONS::zoomFitScreen, true );
+    GetToolManager()->RunAction( ACTIONS::zoomFitScreen );
 
     // If a symbol was previously selected in m_symbolList from a previous run, show it
     wxString symbName = m_symbolList->GetStringSelection();
@@ -318,14 +256,17 @@ SYMBOL_VIEWER_FRAME::~SYMBOL_VIEWER_FRAME()
         m_toolManager->ShutdownAllTools();
 
     if( m_previewItem )
-        GetCanvas()->GetView()->Remove( m_previewItem );
+    {
+        GetCanvas()->GetView()->Remove( m_previewItem.get() );
+        m_previewItem = nullptr;
+    }
 }
 
 
 void SYMBOL_VIEWER_FRAME::loadAllLibraries()
 {
     // TODO: deduplicate with SYMBOL_TREE_MODEL_ADAPTER::AddLibraries
-    std::vector<wxString> libraryNames = Prj().SchSymbolLibTable()->GetLogicalLibs();
+    std::vector<wxString> libraryNames = PROJECT_SCH::SchSymbolLibTable( &Prj() )->GetLogicalLibs();
     std::unique_ptr<WX_PROGRESS_REPORTER> progressReporter = nullptr;
 
     if( m_show_progress )
@@ -340,8 +281,8 @@ void SYMBOL_VIEWER_FRAME::loadAllLibraries()
 
     std::unordered_map<wxString, std::vector<LIB_SYMBOL*>> loadedSymbols;
 
-    SYMBOL_ASYNC_LOADER loader( libraryNames, Prj().SchSymbolLibTable(), false, nullptr,
-                                progressReporter.get() );
+    SYMBOL_ASYNC_LOADER loader( libraryNames, PROJECT_SCH::SchSymbolLibTable( &Prj() ), false,
+                                nullptr, progressReporter.get() );
 
     LOCALE_IO toggle;
 
@@ -418,44 +359,42 @@ void SYMBOL_VIEWER_FRAME::setupUIConditions()
     auto electricalTypesShownCondition =
             [this]( const SELECTION& aSel )
             {
-                return GetRenderSettings()->m_ShowPinsElectricalType;
+                return GetRenderSettings() && GetRenderSettings()->m_ShowPinsElectricalType;
             };
 
     auto pinNumbersShownCondition =
             [this]( const SELECTION& )
             {
-                return GetRenderSettings()->m_ShowPinNumbers;
+                return GetRenderSettings() && GetRenderSettings()->m_ShowPinNumbers;
             };
 
     auto demorganCond =
             [this]( const SELECTION& )
             {
                 LIB_SYMBOL* symbol = GetSelectedSymbol();
-
-                return symbol && symbol->HasConversion();
+                return symbol && symbol->HasAlternateBodyStyle();
             };
 
     auto demorganStandardCond =
             []( const SELECTION& )
             {
-                return m_convert == LIB_ITEM::LIB_CONVERT::BASE;
+                return m_bodyStyle == BODY_STYLE::BASE;
             };
 
     auto demorganAlternateCond =
             []( const SELECTION& )
             {
-                return m_convert == LIB_ITEM::LIB_CONVERT::DEMORGAN;
+                return m_bodyStyle == BODY_STYLE::DEMORGAN;
             };
 
     auto haveDatasheetCond =
             [this]( const SELECTION& )
             {
                 LIB_SYMBOL* symbol = GetSelectedSymbol();
-
                 return symbol && !symbol->GetDatasheetField().GetText().IsEmpty();
             };
 
-    mgr->SetConditions( EE_ACTIONS::showDatasheet,       ENABLE( haveDatasheetCond ) );
+    mgr->SetConditions( ACTIONS::showDatasheet,       ENABLE( haveDatasheetCond ) );
     mgr->SetConditions( EE_ACTIONS::showElectricalTypes, CHECK( electricalTypesShownCondition ) );
     mgr->SetConditions( EE_ACTIONS::showPinNumbers,      CHECK( pinNumbersShownCondition ) );
 
@@ -469,10 +408,10 @@ void SYMBOL_VIEWER_FRAME::setupUIConditions()
 }
 
 
-void SYMBOL_VIEWER_FRAME::SetUnitAndConvert( int aUnit, int aConvert )
+void SYMBOL_VIEWER_FRAME::SetUnitAndBodyStyle( int aUnit, int aBodyStyle )
 {
     m_unit = aUnit > 0 ? aUnit : 1;
-    m_convert = aConvert > 0 ? aConvert : LIB_ITEM::LIB_CONVERT::BASE;
+    m_bodyStyle = aBodyStyle > 0 ? aBodyStyle : BODY_STYLE::BASE;
     m_selection_changed = false;
 
     updatePreviewSymbol();
@@ -484,7 +423,7 @@ LIB_SYMBOL* SYMBOL_VIEWER_FRAME::GetSelectedSymbol() const
     LIB_SYMBOL* symbol = nullptr;
 
     if( m_currentSymbol.IsValid() )
-        symbol = Prj().SchSymbolLibTable()->LoadSymbol( m_currentSymbol );
+        symbol = PROJECT_SCH::SchSymbolLibTable( &Prj() )->LoadSymbol( m_currentSymbol );
 
     return symbol;
 }
@@ -497,7 +436,7 @@ void SYMBOL_VIEWER_FRAME::updatePreviewSymbol()
 
     if( m_previewItem )
     {
-        view->Remove( m_previewItem );
+        view->Remove( m_previewItem.get() );
         m_previewItem = nullptr;
     }
 
@@ -506,13 +445,13 @@ void SYMBOL_VIEWER_FRAME::updatePreviewSymbol()
     if( symbol )
     {
         GetRenderSettings()->m_ShowUnit = m_unit;
-        GetRenderSettings()->m_ShowConvert = m_convert;
+        GetRenderSettings()->m_ShowBodyStyle = m_bodyStyle;
 
-        m_previewItem = symbol;
-        view->Add( m_previewItem );
+        m_previewItem = symbol->Flatten();
+        view->Add( m_previewItem.get() );
 
         wxString parentName;
-        std::shared_ptr<LIB_SYMBOL> parent  = m_previewItem->GetParent().lock();
+        std::shared_ptr<LIB_SYMBOL> parent  = symbol->GetParent().lock();
 
         if( parent )
             parentName = parent->GetName();
@@ -523,67 +462,15 @@ void SYMBOL_VIEWER_FRAME::updatePreviewSymbol()
         AppendMsgPanel( _( "Keywords" ), m_previewItem->GetKeyWords() );
     }
 
-    m_toolManager->RunAction( ACTIONS::zoomFitScreen, true );
+    m_toolManager->RunAction( ACTIONS::zoomFitScreen );
     GetCanvas()->Refresh();
 }
 
 
-bool SYMBOL_VIEWER_FRAME::ShowModal( wxString* aSymbol, wxWindow* aParent )
-{
-    if( aSymbol && !aSymbol->IsEmpty() )
-    {
-        wxString msg;
-        LIB_TABLE* libTable = Prj().SchSymbolLibTable();
-        LIB_ID libid;
-
-        libid.Parse( *aSymbol, true );
-
-        if( libid.IsValid() )
-        {
-            wxString libName = libid.GetLibNickname();
-
-            if( !libTable->HasLibrary( libid.GetLibNickname(), false ) )
-            {
-                msg.Printf( _( "The current configuration does not include the library '%s'.\n"
-                               "Use Manage Symbol Libraries to edit the configuration." ),
-                             UnescapeString( libName ) );
-                DisplayErrorMessage( this, _( "Library not found in symbol library table." ), msg );
-            }
-            else if ( !libTable->HasLibrary( libid.GetLibNickname(), true ) )
-            {
-                msg.Printf( _( "The library '%s' is not enabled in the current configuration.\n"
-                               "Use Manage Symbol Libraries to edit the configuration." ),
-                             UnescapeString( libName ) );
-                DisplayErrorMessage( aParent, _( "Symbol library not enabled." ), msg );
-            }
-            else
-            {
-                SetSelectedLibrary( libid.GetLibNickname(), libid.GetSubLibraryName() );
-                SetSelectedSymbol( libid.GetLibItemName() );
-            }
-        }
-    }
-
-    m_libFilter->SetFocus();
-    return KIWAY_PLAYER::ShowModal( aSymbol, aParent );
-}
-
-
 void SYMBOL_VIEWER_FRAME::doCloseWindow()
 {
     GetCanvas()->StopDrawing();
-
-    if( !IsModal() )
-    {
-        Destroy();
-    }
-    else if( !IsDismissed() )
-    {
-        // only dismiss modal frame if not already dismissed.
-        DismissModal( false );
-
-        // Modal frame will be destroyed by the calling function.
-    }
+    Destroy();
 }
 
 
@@ -641,7 +528,7 @@ bool SYMBOL_VIEWER_FRAME::ReCreateLibList()
 
     COMMON_SETTINGS*      cfg = Pgm().GetCommonSettings();
     PROJECT_FILE&         project = Kiway().Prj().GetProjectFile();
-    SYMBOL_LIB_TABLE*     libTable = Prj().SchSymbolLibTable();
+    SYMBOL_LIB_TABLE*     libTable = PROJECT_SCH::SchSymbolLibTable( &Prj() );
     std::vector<wxString> libs = libTable->GetLogicalLibs();
     std::vector<wxString> pinnedMatches;
     std::vector<wxString> otherMatches;
@@ -675,7 +562,9 @@ bool SYMBOL_VIEWER_FRAME::ReCreateLibList()
                 {
                     wxArrayString aliasNames;
 
-                    Prj().SchSymbolLibTable()->EnumerateSymbolLib( aLib, aliasNames, true );
+                    PROJECT_SCH::SchSymbolLibTable( &Prj() )->EnumerateSymbolLib( aLib,
+                                                                                  aliasNames,
+                                                                                  true );
 
                     if( aliasNames.IsEmpty() )
                         return;
@@ -683,6 +572,8 @@ bool SYMBOL_VIEWER_FRAME::ReCreateLibList()
 
                 SYMBOL_LIB_TABLE_ROW* row = libTable->FindRow( aLib );
 
+                wxCHECK( row, /* void */ );
+
                 if( !row->GetIsVisible() )
                     return;
 
@@ -693,7 +584,7 @@ bool SYMBOL_VIEWER_FRAME::ReCreateLibList()
 
                     for( const wxString& lib : subLibraries )
                     {
-                        wxString suffix = lib.IsEmpty() ? wxT( "" )
+                        wxString suffix = lib.IsEmpty() ? wxString( wxT( "" ) )
                                                         : wxString::Format( wxT( " - %s" ), lib );
                         wxString name = wxString::Format( wxT( "%s%s" ), aLib, suffix );
 
@@ -719,11 +610,10 @@ bool SYMBOL_VIEWER_FRAME::ReCreateLibList()
         {
             const wxString       term = tokenizer.GetNextToken().Lower();
             EDA_COMBINED_MATCHER matcher( term, CTX_LIBITEM );
-            int                  matches, position;
 
             for( const wxString& lib : libs )
             {
-                if( matcher.Find( lib.Lower(), matches, position ) )
+                if( matcher.Find( lib.Lower() ) )
                     process( lib );
             }
         }
@@ -739,7 +629,8 @@ bool SYMBOL_VIEWER_FRAME::ReCreateLibList()
         m_libList->Append( UnescapeString( name ) );
 
     // Search for a previous selection:
-    int index = m_libList->FindString( UnescapeString( m_currentSymbol.GetUniStringLibNickname() ) );
+    int index =
+            m_libList->FindString( UnescapeString( m_currentSymbol.GetUniStringLibNickname() ) );
 
     if( index != wxNOT_FOUND )
     {
@@ -750,10 +641,10 @@ bool SYMBOL_VIEWER_FRAME::ReCreateLibList()
         // If not found, clear current library selection because it can be deleted after a
         // config change.
         m_currentSymbol.SetLibNickname( m_libList->GetCount() > 0
-                                        ? m_libList->GetBaseString( 0 ) : wxT( "" ) );
+                                        ? m_libList->GetBaseString( 0 ) : wxString( wxT( "" ) ) );
         m_currentSymbol.SetLibItemName( wxEmptyString );
         m_unit = 1;
-        m_convert = LIB_ITEM::LIB_CONVERT::BASE;
+        m_bodyStyle = BODY_STYLE::BASE;
     }
 
     bool cmp_changed = ReCreateSymbolList();
@@ -777,12 +668,13 @@ bool SYMBOL_VIEWER_FRAME::ReCreateSymbolList()
         return false;
 
     std::vector<LIB_SYMBOL*> symbols;
-    SYMBOL_LIB_TABLE_ROW* row = Prj().SchSymbolLibTable()->FindRow( libName );
+    SYMBOL_LIB_TABLE_ROW* row = PROJECT_SCH::SchSymbolLibTable( &Prj() )->FindRow( libName );
 
     try
     {
         if( row )
-            Prj().SchSymbolLibTable()->LoadSymbolLib( symbols, libName, m_listPowerOnly );
+            PROJECT_SCH::SchSymbolLibTable( &Prj() )->LoadSymbolLib( symbols, libName,
+                                                                     m_listPowerOnly );
     }
     catch( const IO_ERROR& ) {}   // ignore, it is handled below
 
@@ -794,17 +686,16 @@ bool SYMBOL_VIEWER_FRAME::ReCreateSymbolList()
 
         while( tokenizer.HasMoreTokens() )
         {
-            const wxString       term = tokenizer.GetNextToken().Lower();
-            EDA_COMBINED_MATCHER matcher( term, CTX_LIBITEM );
-            int                  matches, position;
+            const wxString       filterTerm = tokenizer.GetNextToken().Lower();
+            EDA_COMBINED_MATCHER matcher( filterTerm, CTX_LIBITEM );
 
             for( LIB_SYMBOL* symbol : symbols )
             {
-                wxString search = symbol->GetName() + wxS( " " ) + symbol->GetSearchText();
-                bool     matched = matcher.Find( search.Lower(), matches, position );
+                std::vector<SEARCH_TERM> searchTerms = symbol->GetSearchTerms();
+                int                      matched = matcher.ScoreTerms( searchTerms );
 
-                if( !matched && term.IsNumber() )
-                    matched = ( wxAtoi( term ) == (int)symbol->GetPinCount() );
+                if( filterTerm.IsNumber() && wxAtoi( filterTerm ) == (int)symbol->GetPinCount() )
+                    matched++;
 
                 if( !matched )
                     excludes.insert( symbol->GetName() );
@@ -829,19 +720,20 @@ bool SYMBOL_VIEWER_FRAME::ReCreateSymbolList()
     if( m_symbolList->IsEmpty() )
     {
         SetSelectedSymbol( wxEmptyString );
-        m_convert = LIB_ITEM::LIB_CONVERT::BASE;
+        m_bodyStyle = BODY_STYLE::BASE;
         m_unit    = 1;
         return true;
     }
 
-    int  index = m_symbolList->FindString( UnescapeString( m_currentSymbol.GetUniStringLibItemName() ) );
+    int index =
+            m_symbolList->FindString( UnescapeString( m_currentSymbol.GetUniStringLibItemName() ) );
     bool changed = false;
 
     if( index == wxNOT_FOUND )
     {
         // Select the first library entry when the previous entry name does not exist in
         // the current library.
-        m_convert   = LIB_ITEM::LIB_CONVERT::BASE;
+        m_bodyStyle = BODY_STYLE::BASE;
         m_unit      = 1;
         index       = -1;
         changed     = true;
@@ -865,7 +757,7 @@ void SYMBOL_VIEWER_FRAME::ClickOnLibList( wxCommandEvent& event )
 
     wxString selection = EscapeString( m_libList->GetBaseString( ii ), CTX_LIBID );
 
-    if( !Prj().SchSymbolLibTable()->FindRow( selection )
+    if( !PROJECT_SCH::SchSymbolLibTable( &Prj() )->FindRow( selection )
         && selection.Find( '-' ) != wxNOT_FOUND )
     {
         // Probably a sub-library
@@ -884,8 +776,8 @@ void SYMBOL_VIEWER_FRAME::ClickOnLibList( wxCommandEvent& event )
 void SYMBOL_VIEWER_FRAME::SetSelectedLibrary( const wxString& aLibraryName,
                                               const wxString& aSubLibName )
 {
-    if( m_currentSymbol.GetUniStringLibNickname() == aLibraryName &&
-        wxString( m_currentSymbol.GetSubLibraryName() ) == aSubLibName )
+    if( m_currentSymbol.GetUniStringLibNickname() == aLibraryName
+        && wxString( m_currentSymbol.GetSubLibraryName().wx_str() ) == aSubLibName )
         return;
 
     m_currentSymbol.SetLibNickname( aLibraryName );
@@ -941,7 +833,7 @@ void SYMBOL_VIEWER_FRAME::SetSelectedSymbol( const wxString& aSymbolName )
         if( m_selection_changed )
         {
             m_unit = 1;
-            m_convert = LIB_ITEM::LIB_CONVERT::BASE;
+            m_bodyStyle = BODY_STYLE::BASE;
             m_selection_changed = false;
         }
 
@@ -952,13 +844,14 @@ void SYMBOL_VIEWER_FRAME::SetSelectedSymbol( const wxString& aSymbolName )
 
 void SYMBOL_VIEWER_FRAME::DClickOnSymbolList( wxCommandEvent& event )
 {
-    m_toolManager->RunAction( EE_ACTIONS::addSymbolToSchematic, true );
+    m_toolManager->RunAction( EE_ACTIONS::addSymbolToSchematic );
 }
 
 
 void SYMBOL_VIEWER_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
 {
-    auto cfg = Pgm().GetSettingsManager().GetAppSettings<EESCHEMA_SETTINGS>();
+    SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
+    EESCHEMA_SETTINGS* cfg = mgr.GetAppSettings<EESCHEMA_SETTINGS>( "eeschema" );
 
     SCH_BASE_FRAME::LoadSettings( cfg );
 
@@ -984,7 +877,8 @@ void SYMBOL_VIEWER_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
 
 void SYMBOL_VIEWER_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg)
 {
-    EESCHEMA_SETTINGS* cfg = Pgm().GetSettingsManager().GetAppSettings<EESCHEMA_SETTINGS>();
+    SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
+    EESCHEMA_SETTINGS* cfg = mgr.GetAppSettings<EESCHEMA_SETTINGS>( "eeschema" );
 
     SCH_BASE_FRAME::SaveSettings( cfg );
 
@@ -996,7 +890,7 @@ void SYMBOL_VIEWER_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg)
     cfg->m_LibViewPanel.lib_list_width = m_libListWidth;
     cfg->m_LibViewPanel.cmp_list_width = m_symbolListWidth;
 
-    if( KIGFX::SCH_RENDER_SETTINGS* renderSettings = GetRenderSettings() )
+    if( SCH_RENDER_SETTINGS* renderSettings = GetRenderSettings() )
     {
         cfg->m_LibViewPanel.show_pin_electrical_type = renderSettings->m_ShowPinsElectricalType;
         cfg->m_LibViewPanel.show_pin_numbers = renderSettings->m_ShowPinNumbers;
@@ -1006,24 +900,25 @@ void SYMBOL_VIEWER_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg)
 
 WINDOW_SETTINGS* SYMBOL_VIEWER_FRAME::GetWindowSettings( APP_SETTINGS_BASE* aCfg )
 {
-    auto cfg = dynamic_cast<EESCHEMA_SETTINGS*>( aCfg );
+    EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( aCfg );
     wxASSERT( cfg );
     return &cfg->m_LibViewPanel.window;
 }
 
 
-void SYMBOL_VIEWER_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVarsChanged )
+void SYMBOL_VIEWER_FRAME::CommonSettingsChanged( int aFlags )
 {
-    SCH_BASE_FRAME::CommonSettingsChanged( aEnvVarsChanged, aTextVarsChanged );
+    SCH_BASE_FRAME::CommonSettingsChanged( aFlags );
 
-    auto cfg = Pgm().GetSettingsManager().GetAppSettings<EESCHEMA_SETTINGS>();
+    SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
+    EESCHEMA_SETTINGS* cfg = mgr.GetAppSettings<EESCHEMA_SETTINGS>( "eeschema" );
     GetGalDisplayOptions().ReadWindowSettings( cfg->m_LibViewPanel.window );
 
     GetCanvas()->GetGAL()->SetAxesColor( m_colorSettings->GetColor( LAYER_SCHEMATIC_GRID_AXES ) );
     GetCanvas()->GetGAL()->DrawGrid();
     GetCanvas()->ForceRefresh();
 
-    if( aEnvVarsChanged )
+    if( aFlags && ENVVARS_CHANGED )
         ReCreateLibList();
 }
 
@@ -1052,21 +947,6 @@ void SYMBOL_VIEWER_FRAME::CloseLibraryViewer( wxCommandEvent& event )
 }
 
 
-void SYMBOL_VIEWER_FRAME::SetFilter( const SYMBOL_LIBRARY_FILTER* aFilter )
-{
-    m_listPowerOnly = false;
-    m_allowedLibs.Clear();
-
-    if( aFilter )
-    {
-        m_allowedLibs = aFilter->GetAllowedLibList();
-        m_listPowerOnly = aFilter->GetFilterPowerSymbols();
-    }
-
-    ReCreateLibList();
-}
-
-
 const BOX2I SYMBOL_VIEWER_FRAME::GetDocumentExtents( bool aIncludeAllVisible ) const
 {
     LIB_SYMBOL* symbol = GetSelectedSymbol();
@@ -1082,79 +962,11 @@ const BOX2I SYMBOL_VIEWER_FRAME::GetDocumentExtents( bool aIncludeAllVisible ) c
 
         wxCHECK( tmp, BOX2I( VECTOR2I( -200, -200 ), VECTOR2I( 400, 400 ) ) );
 
-        return tmp->GetUnitBoundingBox( m_unit, m_convert );
+        return tmp->GetUnitBoundingBox( m_unit, m_bodyStyle );
     }
 }
 
 
-void SYMBOL_VIEWER_FRAME::FinishModal()
-{
-    if( m_symbolList->GetSelection() >= 0 )
-    {
-        DismissModal( true, m_currentSymbol.Format() );
-    }
-    else
-    {
-        DismissModal( false );
-    }
-
-    Close( true );
-}
-
-
-void SYMBOL_VIEWER_FRAME::OnSelectSymbol( wxCommandEvent& aEvent )
-{
-    std::unique_lock<std::mutex> dialogLock( DIALOG_CHOOSE_SYMBOL::g_Mutex, std::defer_lock );
-
-    // One CHOOSE_SYMBOL dialog at a time.  User probably can't handle more anyway.
-    if( !dialogLock.try_lock() )
-        return;
-
-    // Container doing search-as-you-type.
-    SYMBOL_LIB_TABLE* libs = Prj().SchSymbolLibTable();
-    wxObjectDataPtr<LIB_TREE_MODEL_ADAPTER> dataPtr
-                                    = SYMBOL_TREE_MODEL_ADAPTER::Create( this, libs );
-    SYMBOL_TREE_MODEL_ADAPTER* modelAdapter
-                                    = static_cast<SYMBOL_TREE_MODEL_ADAPTER*>( dataPtr.get() );
-
-    if( !modelAdapter->AddLibraries( libs->GetLogicalLibs(), this ) )
-    {
-        // loading cancelled by user
-        return;
-    }
-
-    LIB_SYMBOL* current = GetSelectedSymbol();
-    LIB_ID id;
-    int unit = 0;
-
-    if( current )
-    {
-        id = current->GetLibId();
-        modelAdapter->SetPreselectNode( id, unit );
-    }
-
-    wxString dialogTitle;
-    dialogTitle.Printf( _( "Choose Symbol (%d items loaded)" ), modelAdapter->GetItemCount() );
-
-    DIALOG_CHOOSE_SYMBOL dlg( this, dialogTitle, dataPtr, m_convert, false, false, false );
-
-    if( dlg.ShowQuasiModal() == wxID_CANCEL )
-        return;
-
-    // Save any changes to column widths, etc.
-    modelAdapter->SaveSettings();
-
-    id = dlg.GetSelectedLibId( &unit );
-
-    if( !id.IsValid() )
-        return;
-
-    SetSelectedLibrary( id.GetLibNickname(), id.GetSubLibraryName() );
-    SetSelectedSymbol( id.GetLibItemName() );
-    SetUnitAndConvert( unit, 1 );
-}
-
-
 void SYMBOL_VIEWER_FRAME::OnLibFilter( wxCommandEvent& aEvent )
 {
     ReCreateLibList();
@@ -1233,7 +1045,8 @@ void SYMBOL_VIEWER_FRAME::OnCharHook( wxKeyEvent& aEvent )
         else
             aEvent.Skip();
     }
-    else if( aEvent.GetKeyCode() == WXK_RETURN && m_symbolList->GetSelection() >= 0 )
+    else if( ( aEvent.GetKeyCode() == WXK_RETURN || aEvent.GetKeyCode() == WXK_NUMPAD_ENTER )
+             && m_symbolList->GetSelection() >= 0 )
     {
         wxCommandEvent dummy;
         DClickOnSymbolList( dummy );
@@ -1251,7 +1064,7 @@ void SYMBOL_VIEWER_FRAME::onSelectNextSymbol( wxCommandEvent& aEvent )
     int            ii = m_symbolList->GetSelection();
 
     // Select the next symbol or stop at the end of the list.
-    if( ii != wxNOT_FOUND || ii != (int) m_symbolList->GetCount() - 1 )
+    if( ii != wxNOT_FOUND && ii < (int) ( m_symbolList->GetCount() - 1 ) )
         ii += 1;
 
     m_symbolList->SetSelection( ii );
@@ -1293,7 +1106,7 @@ void SYMBOL_VIEWER_FRAME::DisplayLibInfos()
     if( m_libList && !m_libList->IsEmpty() && !libName.IsEmpty() )
     {
         const SYMBOL_LIB_TABLE_ROW* row =
-                Prj().SchSymbolLibTable()->FindRow( libName, true );
+                PROJECT_SCH::SchSymbolLibTable( &Prj() )->FindRow( libName, true );
 
         wxString title = row ? row->GetFullURI( true ) : _( "[no library selected]" );
 
@@ -1311,6 +1124,7 @@ SELECTION& SYMBOL_VIEWER_FRAME::GetCurrentSelection()
 
 void SYMBOL_VIEWER_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
 {
+
     switch( mail.Command() )
     {
     case MAIL_RELOAD_LIB:
@@ -1318,6 +1132,33 @@ void SYMBOL_VIEWER_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
         ReCreateLibList();
         break;
     }
+    case MAIL_REFRESH_SYMBOL:
+    {
+        SYMBOL_LIB_TABLE* tbl = PROJECT_SCH::SchSymbolLibTable( &Prj() );
+        LIB_SYMBOL* symbol = GetSelectedSymbol();
+
+        wxCHECK2( tbl && symbol, break );
+
+        const SYMBOL_LIB_TABLE_ROW* row = tbl->FindRow( symbol->GetLibId().GetLibNickname() );
+
+        if( !row )
+            return;
+
+        wxString libfullname = row->GetFullURI( true );
+
+        wxString lib( mail.GetPayload() );
+        wxLogTrace( "KICAD_LIB_WATCH", "Received refresh symbol request for %s, current symbols "
+                    "is %s", lib, libfullname );
+
+        if( lib == libfullname )
+        {
+            wxLogTrace( "KICAD_LIB_WATCH", "Refreshing symbol %s", symbol->GetName() );
+            updatePreviewSymbol();
+            GetCanvas()->GetView()->UpdateAllItems( KIGFX::ALL );
+        }
+
+        break;
+    }
     default:;
     }
 }
