@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2019 CERN
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <sch_commit.h>
 #include <sch_sheet_pin.h>
 #include <schematic.h>
 #include <tools/ee_actions.h>
@@ -39,6 +40,8 @@ int SCH_FIND_REPLACE_TOOL::FindAndReplace( const TOOL_EVENT& aEvent )
 int SCH_FIND_REPLACE_TOOL::UpdateFind( const TOOL_EVENT& aEvent )
 {
     EDA_SEARCH_DATA& data = m_frame->GetFindReplaceData();
+    SCH_SEARCH_DATA* schSearchData = dynamic_cast<SCH_SEARCH_DATA*>( &data );
+    bool             selectedOnly = schSearchData ? schSearchData->searchSelectedOnly : false;
 
     auto visit =
             [&]( EDA_ITEM* aItem, SCH_SHEET_PATH* aSheet )
@@ -48,7 +51,8 @@ int SCH_FIND_REPLACE_TOOL::UpdateFind( const TOOL_EVENT& aEvent )
                 // closed....so we need to double check the dialog is open.
                 if( m_frame->m_findReplaceDialog != nullptr
                     && !data.findString.IsEmpty()
-                    && aItem->Matches( data, aSheet ) )
+                    && aItem->Matches( data, aSheet )
+                    && ( !selectedOnly || aItem->IsSelected() ) )
                 {
                     aItem->SetForceVisible( true );
                     m_selectionTool->BrightenItem( aItem );
@@ -61,42 +65,57 @@ int SCH_FIND_REPLACE_TOOL::UpdateFind( const TOOL_EVENT& aEvent )
                 }
             };
 
+    auto visitAll =
+            [&]()
+            {
+                for( SCH_ITEM* item : m_frame->GetScreen()->Items() )
+                {
+                    visit( item, &m_frame->GetCurrentSheet() );
+
+                    item->RunOnChildren(
+                            [&]( SCH_ITEM* aChild )
+                            {
+                                visit( aChild, &m_frame->GetCurrentSheet() );
+                            } );
+                }
+            };
+
     if( aEvent.IsAction( &ACTIONS::find ) || aEvent.IsAction( &ACTIONS::findAndReplace )
         || aEvent.IsAction( &ACTIONS::updateFind ) )
     {
         m_foundItemHighlighted = false;
-        m_selectionTool->ClearSelection();
-
-        for( SCH_ITEM* item : m_frame->GetScreen()->Items() )
-        {
-            visit( item, &m_frame->GetCurrentSheet() );
-
-            item->RunOnChildren(
-                    [&]( SCH_ITEM* aChild )
-                    {
-                        visit( aChild, &m_frame->GetCurrentSheet() );
-                    } );
-        }
+        visitAll();
     }
     else if( aEvent.Matches( EVENTS::SelectedItemsModified ) )
     {
         for( EDA_ITEM* item : m_selectionTool->GetSelection() )
             visit( item, &m_frame->GetCurrentSheet() );
     }
+    else if( aEvent.Matches( EVENTS::PointSelectedEvent )
+             || aEvent.Matches( EVENTS::SelectedEvent )
+             || aEvent.Matches( EVENTS::UnselectedEvent )
+             || aEvent.Matches( EVENTS::ClearedEvent ) )
+    {
+        if( !m_frame->m_findReplaceDialog )
+        {
+            if( m_foundItemHighlighted )
+            {
+                m_foundItemHighlighted = false;
+                visitAll();
+            }
+        }
+        else if( selectedOnly )
+        {
+            // Normal find modifies the selection, but selection-based find does not, so we want
+            // to start over in the items we are searching through when the selection changes
+            m_afterItem = nullptr;
+            visitAll();
+        }
+    }
     else if( m_foundItemHighlighted )
     {
         m_foundItemHighlighted = false;
-
-        for( SCH_ITEM* item : m_frame->GetScreen()->Items() )
-        {
-            visit( item, &m_frame->GetCurrentSheet() );
-
-            item->RunOnChildren(
-                    [&]( SCH_ITEM* aChild )
-                    {
-                        visit( aChild, &m_frame->GetCurrentSheet() );
-                    } );
-        }
+        visitAll();
     }
 
     getView()->UpdateItems();
@@ -108,42 +127,57 @@ int SCH_FIND_REPLACE_TOOL::UpdateFind( const TOOL_EVENT& aEvent )
 
 
 SCH_ITEM* SCH_FIND_REPLACE_TOOL::nextMatch( SCH_SCREEN* aScreen, SCH_SHEET_PATH* aSheet,
-                                            SCH_ITEM* aAfter, EDA_SEARCH_DATA& aData )
+                                            SCH_ITEM* aAfter, EDA_SEARCH_DATA& aData,
+                                            bool reversed )
 {
+    SCH_SEARCH_DATA*       schSearchData = dynamic_cast<SCH_SEARCH_DATA*>( &aData );
+    bool                   selectedOnly = schSearchData ? schSearchData->searchSelectedOnly : false;
     bool                   past_item = !aAfter;
     std::vector<SCH_ITEM*> sorted_items;
 
-    for( SCH_ITEM* item : aScreen->Items() )
+    auto addItem =
+            [&](SCH_ITEM* item)
+            {
+                sorted_items.push_back( item );
+
+                if( item->Type() == SCH_SYMBOL_T )
+                {
+                    SCH_SYMBOL* cmp = static_cast<SCH_SYMBOL*>( item );
+
+                    for( SCH_FIELD& field : cmp->GetFields() )
+                        sorted_items.push_back( &field );
+
+                    for( SCH_PIN* pin : cmp->GetPins() )
+                        sorted_items.push_back( pin );
+                }
+                else if( item->Type() == SCH_SHEET_T )
+                {
+                    SCH_SHEET* sheet = static_cast<SCH_SHEET*>( item );
+
+                    for( SCH_FIELD& field : sheet->GetFields() )
+                        sorted_items.push_back( &field );
+
+                    for( SCH_SHEET_PIN* pin : sheet->GetPins() )
+                        sorted_items.push_back( pin );
+                }
+                else if( item->IsType( { SCH_LABEL_LOCATE_ANY_T } ) )
+                {
+                    SCH_LABEL_BASE* label = static_cast<SCH_LABEL_BASE*>( item );
+
+                    for( SCH_FIELD& field : label->GetFields() )
+                        sorted_items.push_back( &field );
+                }
+            };
+
+    if( selectedOnly )
     {
-        sorted_items.push_back( item );
-
-        if( item->Type() == SCH_SYMBOL_T )
-        {
-            SCH_SYMBOL* cmp = static_cast<SCH_SYMBOL*>( item );
-
-            for( SCH_FIELD& field : cmp->GetFields() )
-                sorted_items.push_back( &field );
-
-            for( SCH_PIN* pin : cmp->GetPins() )
-                sorted_items.push_back( pin );
-        }
-        else if( item->Type() == SCH_SHEET_T )
-        {
-            SCH_SHEET* sheet = static_cast<SCH_SHEET*>( item );
-
-            for( SCH_FIELD& field : sheet->GetFields() )
-                sorted_items.push_back( &field );
-
-            for( SCH_SHEET_PIN* pin : sheet->GetPins() )
-                sorted_items.push_back( pin );
-        }
-        else if( item->IsType( { SCH_LABEL_LOCATE_ANY_T } ) )
-        {
-            SCH_LABEL_BASE* label = static_cast<SCH_LABEL_BASE*>( item );
-
-            for( SCH_FIELD& field : label->GetFields() )
-                sorted_items.push_back( &field );
-        }
+        for( EDA_ITEM* item : m_selectionTool->GetSelection() )
+            addItem( static_cast<SCH_ITEM*>( item ) );
+    }
+    else
+    {
+        for( SCH_ITEM* item : aScreen->Items() )
+            addItem( item );
     }
 
     std::sort( sorted_items.begin(), sorted_items.end(),
@@ -161,6 +195,9 @@ SCH_ITEM* SCH_FIND_REPLACE_TOOL::nextMatch( SCH_SCREEN* aScreen, SCH_SHEET_PATH*
                     return a->GetPosition().x < b->GetPosition().x;
             } );
 
+    if( reversed )
+        std::reverse( sorted_items.begin(), sorted_items.end() );
+
     for( SCH_ITEM* item : sorted_items )
     {
         if( item == aAfter )
@@ -183,13 +220,18 @@ SCH_ITEM* SCH_FIND_REPLACE_TOOL::nextMatch( SCH_SCREEN* aScreen, SCH_SHEET_PATH*
 
 int SCH_FIND_REPLACE_TOOL::FindNext( const TOOL_EVENT& aEvent )
 {
-    EDA_SEARCH_DATA& data = m_frame->GetFindReplaceData();
+    EDA_SEARCH_DATA& data            = m_frame->GetFindReplaceData();
     bool             searchAllSheets = false;
+    bool             selectedOnly    = false;
+    bool             isReversed      = aEvent.IsAction( &ACTIONS::findPrevious );
+    SCH_ITEM*        item            = nullptr;
+    SCH_SHEET_PATH*  afterSheet      = &m_frame->GetCurrentSheet();
 
     try
     {
         const SCH_SEARCH_DATA& schSearchData = dynamic_cast<const SCH_SEARCH_DATA&>( data );
         searchAllSheets = !( schSearchData.searchCurrentSheetOnly );
+        selectedOnly = schSearchData.searchSelectedOnly;
     }
     catch( const std::bad_cast& )
     {
@@ -200,67 +242,60 @@ int SCH_FIND_REPLACE_TOOL::FindNext( const TOOL_EVENT& aEvent )
     else if( data.findString.IsEmpty() )
         return FindAndReplace( ACTIONS::find.MakeEvent() );
 
-    EE_SELECTION& selection       = m_selectionTool->GetSelection();
-    SCH_ITEM*     afterItem       = dynamic_cast<SCH_ITEM*>( selection.Front() );
-    SCH_ITEM*     item            = nullptr;
-
-    SCH_SHEET_PATH* afterSheet    = &m_frame->GetCurrentSheet();
-
     if( m_wrapAroundTimer.IsRunning() )
     {
         afterSheet = nullptr;
-        afterItem = nullptr;
+        m_afterItem = nullptr;
         m_wrapAroundTimer.Stop();
         m_frame->ClearFindReplaceStatus();
     }
 
-    m_selectionTool->ClearSelection();
-
     if( afterSheet || !searchAllSheets )
-        item = nextMatch( m_frame->GetScreen(), &m_frame->GetCurrentSheet(), afterItem, data );
+    {
+        item = nextMatch( m_frame->GetScreen(), &m_frame->GetCurrentSheet(), m_afterItem, data,
+                          isReversed );
+    }
 
     if( !item && searchAllSheets )
     {
-        SCH_SCREENS                  screens( m_frame->Schematic().Root() );
-        std::vector<SCH_SHEET_PATH*> paths;
+        SCH_SCREENS    screens( m_frame->Schematic().Root() );
+        SCH_SHEET_LIST paths;
 
         screens.BuildClientSheetPathList();
 
         for( SCH_SCREEN* screen = screens.GetFirst(); screen; screen = screens.GetNext() )
         {
             for( SCH_SHEET_PATH& sheet : screen->GetClientSheetPaths() )
-                paths.push_back( &sheet );
+                paths.push_back( sheet );
         }
 
-        std::sort( paths.begin(), paths.end(), [] ( const SCH_SHEET_PATH* lhs,
-                                                    const SCH_SHEET_PATH* rhs ) -> bool
-                {
-                    int retval = lhs->ComparePageNum( *rhs );
+        paths.SortByPageNumbers( false );
 
-                    if( retval < 0 )
-                        return true;
-                    else if( retval > 0 )
-                        return false;
-                    else /// Enforce strict ordering.  If the page numbers are the same, use UUIDs
-                        return lhs->GetCurrentHash() < rhs->GetCurrentHash();
-                } );
+        if( isReversed )
+            std::reverse( paths.begin(), paths.end() );
 
-        for( SCH_SHEET_PATH* sheet : paths )
+        for( SCH_SHEET_PATH& sheet : paths )
         {
             if( afterSheet )
             {
-                if( afterSheet->GetCurrentHash() == sheet->GetCurrentHash() )
+                if( afterSheet->GetCurrentHash() == sheet.GetCurrentHash() )
                     afterSheet = nullptr;
 
                 continue;
             }
 
-            item = nextMatch( sheet->LastScreen(), sheet, nullptr, data );
+            item = nextMatch( sheet.LastScreen(), &sheet, nullptr, data, isReversed );
 
             if( item )
             {
-                m_frame->Schematic().SetCurrentSheet( *sheet );
-                m_frame->DisplayCurrentSheet();
+                if( m_frame->Schematic().CurrentSheet() != sheet )
+                {
+                    // Store the current zoom level into the current screen before switching
+                    m_frame->GetScreen()->m_LastZoomLevel = m_frame->GetCanvas()->GetView()->GetScale();
+
+                    m_frame->Schematic().SetCurrentSheet( sheet );
+                    m_frame->DisplayCurrentSheet();
+                }
 
                 break;
             }
@@ -269,6 +304,14 @@ int SCH_FIND_REPLACE_TOOL::FindNext( const TOOL_EVENT& aEvent )
 
     if( item )
     {
+        m_afterItem = item;
+
+        if( !selectedOnly )
+        {
+            m_selectionTool->ClearSelection();
+            m_selectionTool->AddItemToSel( item );
+        }
+
         if( !item->IsBrightened() )
         {
             // Clear any previous brightening
@@ -280,7 +323,6 @@ int SCH_FIND_REPLACE_TOOL::FindNext( const TOOL_EVENT& aEvent )
             m_foundItemHighlighted = true;
         }
 
-        m_selectionTool->AddItemToSel( item );
         m_frame->FocusOnLocation( item->GetBoundingBox().GetCenter() );
         m_frame->GetCanvas()->Refresh();
     }
@@ -298,36 +340,44 @@ int SCH_FIND_REPLACE_TOOL::FindNext( const TOOL_EVENT& aEvent )
     return 0;
 }
 
+EDA_ITEM* SCH_FIND_REPLACE_TOOL::getCurrentMatch()
+{
+    EDA_SEARCH_DATA& data = m_frame->GetFindReplaceData();
+    SCH_SEARCH_DATA* schSearchData = dynamic_cast<SCH_SEARCH_DATA*>( &data );
+    bool             selectedOnly = schSearchData ? schSearchData->searchSelectedOnly : false;
+
+    return selectedOnly ? m_afterItem : m_selectionTool->GetSelection().Front();
+}
 
 bool SCH_FIND_REPLACE_TOOL::HasMatch()
 {
     EDA_SEARCH_DATA& data = m_frame->GetFindReplaceData();
-    EDA_ITEM*        item = m_selectionTool->GetSelection().Front();
+    EDA_ITEM*        match = getCurrentMatch();
 
-    return item && item->Matches( data, &m_frame->GetCurrentSheet() );
+    return match && match->Matches( data, &m_frame->GetCurrentSheet() );
 }
 
 
 int SCH_FIND_REPLACE_TOOL::ReplaceAndFindNext( const TOOL_EVENT& aEvent )
 {
     EDA_SEARCH_DATA& data = m_frame->GetFindReplaceData();
-    EDA_ITEM*        item = m_selectionTool->GetSelection().Front();
+    EDA_ITEM*        item = getCurrentMatch();
     SCH_SHEET_PATH*  sheet = &m_frame->GetCurrentSheet();
 
     if( data.findString.IsEmpty() )
         return FindAndReplace( ACTIONS::find.MakeEvent() );
 
-    if( item && item->Matches( data, sheet ) )
+    if( item && HasMatch() )
     {
+        SCH_COMMIT commit( m_frame );
         SCH_ITEM* sch_item = static_cast<SCH_ITEM*>( item );
 
-        m_frame->SaveCopyInUndoList( sheet->LastScreen(), sch_item, UNDO_REDO::CHANGED, false );
+        commit.Modify( sch_item, sheet->LastScreen() );
 
         if( item->Replace( data, sheet ) )
         {
-            m_frame->UpdateItem( item, false, true );
             m_frame->GetCurrentSheet().UpdateAllScreenReferences();
-            m_frame->OnModify();
+            commit.Push( wxS( "Find and Replace" ) );
         }
 
         FindNext( ACTIONS::findNext.MakeEvent() );
@@ -341,17 +391,20 @@ int SCH_FIND_REPLACE_TOOL::ReplaceAll( const TOOL_EVENT& aEvent )
 {
     EDA_SEARCH_DATA& data = m_frame->GetFindReplaceData();
     bool             currentSheetOnly = false;
+    bool             selectedOnly = false;
 
     try
     {
         const SCH_SEARCH_DATA& schSearchData = dynamic_cast<const SCH_SEARCH_DATA&>( data );
         currentSheetOnly = schSearchData.searchCurrentSheetOnly;
+        selectedOnly     = schSearchData.searchSelectedOnly;
     }
     catch( const std::bad_cast& )
     {
     }
 
-    bool modified = false;
+    SCH_COMMIT commit( m_frame );
+    bool modified = false;      // TODO: move to SCH_COMMIT....
 
     if( data.findString.IsEmpty() )
         return FindAndReplace( ACTIONS::find.MakeEvent() );
@@ -359,8 +412,7 @@ int SCH_FIND_REPLACE_TOOL::ReplaceAll( const TOOL_EVENT& aEvent )
     auto doReplace =
             [&]( SCH_ITEM* aItem, SCH_SHEET_PATH* aSheet, EDA_SEARCH_DATA& aData )
             {
-                m_frame->SaveCopyInUndoList( aSheet->LastScreen(), aItem, UNDO_REDO::CHANGED,
-                                             modified );
+                commit.Modify( aItem, aSheet->LastScreen() );
 
                 if( aItem->Replace( aData, aSheet ) )
                 {
@@ -369,21 +421,23 @@ int SCH_FIND_REPLACE_TOOL::ReplaceAll( const TOOL_EVENT& aEvent )
                 }
             };
 
-    if( currentSheetOnly )
+    if( currentSheetOnly || selectedOnly )
     {
         SCH_SHEET_PATH* currentSheet = &m_frame->GetCurrentSheet();
 
-        SCH_ITEM* item = nextMatch( m_frame->GetScreen(), currentSheet, nullptr, data );
+        SCH_ITEM* item = nextMatch( m_frame->GetScreen(), currentSheet, nullptr, data, false );
 
         while( item )
         {
-            doReplace( item, currentSheet, data );
-            item = nextMatch( m_frame->GetScreen(), currentSheet, item, data );
+            if( !selectedOnly || item->IsSelected() )
+                doReplace( item, currentSheet, data );
+
+            item = nextMatch( m_frame->GetScreen(), currentSheet, item, data, false );
         }
     }
     else
     {
-        SCH_SHEET_LIST allSheets = m_frame->Schematic().GetSheets();
+        SCH_SHEET_LIST allSheets = m_frame->Schematic().Hierarchy();
         SCH_SCREENS    screens( m_frame->Schematic().Root() );
 
         for( SCH_SCREEN* screen = screens.GetFirst(); screen; screen = screens.GetNext() )
@@ -392,7 +446,7 @@ int SCH_FIND_REPLACE_TOOL::ReplaceAll( const TOOL_EVENT& aEvent )
 
             for( unsigned ii = 0; ii < sheets.size(); ++ii )
             {
-                SCH_ITEM* item = nextMatch( screen, &sheets[ii], nullptr, data );
+                SCH_ITEM* item = nextMatch( screen, &sheets[ii], nullptr, data, false );
 
                 while( item )
                 {
@@ -422,7 +476,7 @@ int SCH_FIND_REPLACE_TOOL::ReplaceAll( const TOOL_EVENT& aEvent )
                         }
                     }
 
-                    item = nextMatch( screen, &sheets[ii], item, data );
+                    item = nextMatch( screen, &sheets[ii], item, data, false );
                 }
             }
         }
@@ -430,8 +484,8 @@ int SCH_FIND_REPLACE_TOOL::ReplaceAll( const TOOL_EVENT& aEvent )
 
     if( modified )
     {
+        commit.Push( wxS( "Find and Replace All" ) );
         m_frame->GetCurrentSheet().UpdateAllScreenReferences();
-        m_frame->OnModify();
     }
 
     return 0;
@@ -443,6 +497,7 @@ void SCH_FIND_REPLACE_TOOL::setTransitions()
     Go( &SCH_FIND_REPLACE_TOOL::FindAndReplace,        ACTIONS::find.MakeEvent() );
     Go( &SCH_FIND_REPLACE_TOOL::FindAndReplace,        ACTIONS::findAndReplace.MakeEvent() );
     Go( &SCH_FIND_REPLACE_TOOL::FindNext,              ACTIONS::findNext.MakeEvent() );
+    Go( &SCH_FIND_REPLACE_TOOL::FindNext,              ACTIONS::findPrevious.MakeEvent() );
     Go( &SCH_FIND_REPLACE_TOOL::FindNext,              ACTIONS::findNextMarker.MakeEvent() );
     Go( &SCH_FIND_REPLACE_TOOL::ReplaceAndFindNext,    ACTIONS::replaceAndFindNext.MakeEvent() );
     Go( &SCH_FIND_REPLACE_TOOL::ReplaceAll,            ACTIONS::replaceAll.MakeEvent() );
@@ -450,4 +505,6 @@ void SCH_FIND_REPLACE_TOOL::setTransitions()
     Go( &SCH_FIND_REPLACE_TOOL::UpdateFind,            EVENTS::SelectedItemsModified );
     Go( &SCH_FIND_REPLACE_TOOL::UpdateFind,            EVENTS::PointSelectedEvent );
     Go( &SCH_FIND_REPLACE_TOOL::UpdateFind,            EVENTS::SelectedEvent );
+    Go( &SCH_FIND_REPLACE_TOOL::UpdateFind,            EVENTS::UnselectedEvent );
+    Go( &SCH_FIND_REPLACE_TOOL::UpdateFind,            EVENTS::ClearedEvent );
 }
