@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2019 CERN
- * Copyright (C) 2019-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,7 +27,6 @@
 #include <wx/debug.h>
 #include <wx/gdicmn.h>
 #include <wx/string.h>
-#include <wx/stringimpl.h>
 #include <wx/translation.h>
 #include <algorithm>
 #include <cstdlib>
@@ -36,41 +35,46 @@
 #include <utility>
 #include <vector>
 
-#include <eda_item.h>
-#include <bitmaps.h>
-#include <core/typeinfo.h>
 #include <layer_ids.h>
 #include <math/vector2d.h>
 #include <advanced_config.h>
+#include <gal/graphics_abstraction_layer.h>
+#include <view/view_controls.h>
 #include <tool/actions.h>
 #include <tool/conditional_menu.h>
 #include <tool/selection.h>
 #include <tool/selection_conditions.h>
 #include <tool/tool_event.h>
 #include <trigo.h>
-#include <undo_redo_container.h>
-#include <connection_graph.h>
 #include <eeschema_id.h>
 #include <sch_bus_entry.h>
 #include <sch_connection.h>
 #include <sch_edit_frame.h>
-#include <sch_item.h>
 #include <sch_line.h>
 #include <sch_screen.h>
 #include <sch_sheet.h>
 #include <sch_sheet_pin.h>
 #include <schematic.h>
+#include <sch_commit.h>
 #include <ee_actions.h>
 #include <ee_grid_helper.h>
 #include <ee_selection.h>
 #include <ee_selection_tool.h>
 
+
+using BUS_GETTER = std::function<SCH_LINE*()>;
+
 class BUS_UNFOLD_MENU : public ACTION_MENU
 {
 public:
-    BUS_UNFOLD_MENU() :
+    /**
+     * @param aBusGetter Function to get the bus to unfold, which will probably
+     *                   be looking for a likely bus in a selection.
+    */
+    BUS_UNFOLD_MENU( BUS_GETTER aBusGetter ) :
         ACTION_MENU( true ),
-        m_showTitle( false )
+        m_showTitle( false ),
+        m_busGetter( aBusGetter )
     {
         SetIcon( BITMAPS::add_line2bus );
         SetTitle( _( "Unfold from Bus" ) );
@@ -86,21 +90,16 @@ public:
 protected:
     ACTION_MENU* create() const override
     {
-        return new BUS_UNFOLD_MENU();
+        return new BUS_UNFOLD_MENU( m_busGetter );
     }
 
 private:
     void update() override
     {
-        EE_SELECTION_TOOL* selTool = getToolManager()->GetTool<EE_SELECTION_TOOL>();
-        EE_SELECTION&      selection = selTool->RequestSelection( { SCH_ITEM_LOCATE_BUS_T } );
-        SCH_LINE*          bus = (SCH_LINE*) selection.Front();
-
+        SCH_LINE* bus = m_busGetter();
         Clear();
-
         // Pick up the pointer again because it may have been changed by SchematicCleanUp
-        selection = selTool->RequestSelection( { SCH_ITEM_LOCATE_BUS_T } );
-        bus = (SCH_LINE*) selection.Front();
+        bus = m_busGetter();
 
         if( !bus )
         {
@@ -151,6 +150,21 @@ private:
     }
 
     bool m_showTitle;
+    BUS_GETTER m_busGetter;
+};
+
+
+/**
+ * Settings for bus unfolding that are persistent across invocations of the tool
+ */
+struct BUS_UNFOLD_PERSISTENT_SETTINGS
+{
+    SPIN_STYLE label_spin_style;
+};
+
+
+static BUS_UNFOLD_PERSISTENT_SETTINGS busUnfoldPersistentSettings = {
+    SPIN_STYLE::RIGHT,
 };
 
 
@@ -172,11 +186,17 @@ bool SCH_LINE_WIRE_BUS_TOOL::Init()
 {
     EE_TOOL_BASE::Init();
 
-    std::shared_ptr<BUS_UNFOLD_MENU> busUnfoldMenu = std::make_shared<BUS_UNFOLD_MENU>();
-    busUnfoldMenu->SetTool( this );
-    m_menu.RegisterSubMenu( busUnfoldMenu );
+    const auto busGetter = [this]()
+    {
+        return getBusForUnfolding();
+    };
 
-    std::shared_ptr<BUS_UNFOLD_MENU> selBusUnfoldMenu = std::make_shared<BUS_UNFOLD_MENU>();
+    std::shared_ptr<BUS_UNFOLD_MENU>
+            busUnfoldMenu = std::make_shared<BUS_UNFOLD_MENU>( busGetter );
+    busUnfoldMenu->SetTool( this );
+    m_menu->RegisterSubMenu( busUnfoldMenu );
+
+    std::shared_ptr<BUS_UNFOLD_MENU> selBusUnfoldMenu = std::make_shared<BUS_UNFOLD_MENU>( busGetter );
     selBusUnfoldMenu->SetTool( m_selectionTool );
     m_selectionTool->GetToolMenu().RegisterSubMenu( selBusUnfoldMenu );
 
@@ -210,15 +230,13 @@ bool SCH_LINE_WIRE_BUS_TOOL::Init()
                 return editFrame && !editFrame->GetHighlightedConnection().IsEmpty();
             };
 
-    auto& ctxMenu = m_menu.GetMenu();
+    auto& ctxMenu = m_menu->GetMenu();
 
     // Build the tool menu
     //
     ctxMenu.AddItem( EE_ACTIONS::clearHighlight,       haveHighlight && EE_CONDITIONS::Idle, 1 );
     ctxMenu.AddSeparator(                              haveHighlight && EE_CONDITIONS::Idle, 1 );
 
-    ctxMenu.AddItem( EE_ACTIONS::leaveSheet,           belowRootSheetCondition, 2 );
-
     ctxMenu.AddSeparator( 10 );
     ctxMenu.AddItem( EE_ACTIONS::drawWire,             wireOrBusTool && EE_CONDITIONS::Idle, 10 );
     ctxMenu.AddItem( EE_ACTIONS::drawBus,              wireOrBusTool && EE_CONDITIONS::Idle, 10 );
@@ -226,10 +244,7 @@ bool SCH_LINE_WIRE_BUS_TOOL::Init()
 
     ctxMenu.AddItem( EE_ACTIONS::undoLastSegment,      EE_CONDITIONS::ShowAlways, 10 );
     ctxMenu.AddItem( EE_ACTIONS::switchSegmentPosture, EE_CONDITIONS::ShowAlways, 10 );
-
-    ctxMenu.AddItem( EE_ACTIONS::finishWire,           IsDrawingWire, 10 );
-    ctxMenu.AddItem( EE_ACTIONS::finishBus,            IsDrawingBus, 10 );
-    ctxMenu.AddItem( EE_ACTIONS::finishLine,           IsDrawingLine, 10 );
+    ctxMenu.AddItem( ACTIONS::finishInteractive,       IsDrawingLineWireOrBus, 10 );
 
     ctxMenu.AddMenu( busUnfoldMenu.get(),              EE_CONDITIONS::Idle, 10 );
 
@@ -242,6 +257,8 @@ bool SCH_LINE_WIRE_BUS_TOOL::Init()
     ctxMenu.AddItem( EE_ACTIONS::breakWire,            wireOrBusTool && EE_CONDITIONS::Idle, 100 );
     ctxMenu.AddItem( EE_ACTIONS::slice,                ( wireOrBusTool || lineTool )
                                                                      && EE_CONDITIONS::Idle, 100 );
+    ctxMenu.AddItem( EE_ACTIONS::leaveSheet,           belowRootSheetCondition, 150 );
+
     ctxMenu.AddSeparator( 200 );
     ctxMenu.AddItem( EE_ACTIONS::selectNode,           wireOrBusTool && EE_CONDITIONS::Idle, 200 );
     ctxMenu.AddItem( EE_ACTIONS::selectConnection,     wireOrBusTool && EE_CONDITIONS::Idle, 200 );
@@ -256,27 +273,6 @@ bool SCH_LINE_WIRE_BUS_TOOL::Init()
 }
 
 
-bool SCH_LINE_WIRE_BUS_TOOL::IsDrawingLine( const SELECTION& aSelection )
-{
-    return IsDrawingLineWireOrBus( aSelection )
-                && aSelection.Front()->IsType( { SCH_ITEM_LOCATE_GRAPHIC_LINE_T } );
-}
-
-
-bool SCH_LINE_WIRE_BUS_TOOL::IsDrawingWire( const SELECTION& aSelection )
-{
-    return IsDrawingLineWireOrBus( aSelection )
-                && aSelection.Front()->IsType( { SCH_ITEM_LOCATE_WIRE_T } );
-}
-
-
-bool SCH_LINE_WIRE_BUS_TOOL::IsDrawingBus( const SELECTION& aSelection )
-{
-    return IsDrawingLineWireOrBus( aSelection )
-                && aSelection.Front()->IsType( { SCH_ITEM_LOCATE_BUS_T } );
-}
-
-
 bool SCH_LINE_WIRE_BUS_TOOL::IsDrawingLineWireOrBus( const SELECTION& aSelection )
 {
     // NOTE: for immediate hotkeys, it is NOT required that the line, wire or bus tool
@@ -293,18 +289,20 @@ int SCH_LINE_WIRE_BUS_TOOL::DrawSegments( const TOOL_EVENT& aEvent )
 
     REENTRANCY_GUARD guard( &m_inDrawingTool );
 
-    DRAW_SEGMENT_EVENT_PARAMS* params = aEvent.Parameter<DRAW_SEGMENT_EVENT_PARAMS*>();
+    const DRAW_SEGMENT_EVENT_PARAMS* params = aEvent.Parameter<const DRAW_SEGMENT_EVENT_PARAMS*>();
 
     m_frame->PushTool( aEvent );
-    m_toolMgr->RunAction( EE_ACTIONS::clearSelection, true );
+    m_toolMgr->RunAction( EE_ACTIONS::clearSelection );
 
     if( aEvent.HasPosition() )
     {
-        EE_GRID_HELPER   grid( m_toolMgr );
+        EE_GRID_HELPER    grid( m_toolMgr );
+        GRID_HELPER_GRIDS gridType = ( params->layer == LAYER_NOTES ) ? GRID_GRAPHICS : GRID_WIRES;
+
         grid.SetSnap( !aEvent.Modifier( MD_SHIFT ) );
         grid.SetUseGrid( getView()->GetGAL()->GetGridSnapping() && !aEvent.DisableGridSnapping() );
 
-        VECTOR2D cursorPos = grid.BestSnapAnchor( aEvent.Position(), LAYER_CONNECTABLE, nullptr );
+        VECTOR2D cursorPos = grid.BestSnapAnchor( aEvent.Position(), gridType, nullptr );
         startSegments( params->layer, cursorPos, params->sourceSegment );
     }
 
@@ -333,7 +331,11 @@ int SCH_LINE_WIRE_BUS_TOOL::UnfoldBus( const TOOL_EVENT& aEvent )
     }
     else
     {
-        BUS_UNFOLD_MENU unfoldMenu;
+        const auto busGetter = [this]()
+                {
+                    return getBusForUnfolding();
+                };
+        BUS_UNFOLD_MENU unfoldMenu( busGetter );
         unfoldMenu.SetTool( this );
         unfoldMenu.SetShowTitle();
 
@@ -378,24 +380,46 @@ int SCH_LINE_WIRE_BUS_TOOL::UnfoldBus( const TOOL_EVENT& aEvent )
 }
 
 
-SCH_LINE* SCH_LINE_WIRE_BUS_TOOL::doUnfoldBus( const wxString& aNet, const VECTOR2I& aPos )
+SCH_LINE* SCH_LINE_WIRE_BUS_TOOL::getBusForUnfolding()
+{
+    EE_SELECTION& selection = m_selectionTool->RequestSelection( { SCH_ITEM_LOCATE_BUS_T } );
+    return static_cast<SCH_LINE*>( selection.Front() );
+}
+
+
+SCH_LINE* SCH_LINE_WIRE_BUS_TOOL::doUnfoldBus( const wxString&                aNet,
+                                               const std::optional<VECTOR2I>& aPos )
 {
     SCHEMATIC_SETTINGS& cfg = getModel<SCHEMATIC>()->Settings();
+    SCH_SCREEN*         screen = m_frame->GetScreen();
+    // use the same function as the menu selector, so we choose the same bus segment
+    SCH_LINE* const     bus = getBusForUnfolding();
 
-    VECTOR2I pos = aPos;
+    if ( bus == nullptr )
+    {
+        wxASSERT_MSG( false,
+                      wxString::Format( "Couldn't find the originating bus line (but had a net: %s )",
+                                        aNet ) );
+        return nullptr;
+    }
 
-    if( aPos == VECTOR2I( 0, 0 ) )
-        pos = static_cast<VECTOR2I>( getViewControls()->GetCursorPosition() );
+    VECTOR2I pos = aPos.value_or( static_cast<VECTOR2I>( getViewControls()->GetCursorPosition() ) );
 
-    m_toolMgr->RunAction( EE_ACTIONS::clearSelection, true );
+    // It is possible for the position to be near the bus, but not exactly on it, but
+    // we need the bus entry to be on the bus exactly to connect.
+    // If the bus segment is H or V, this will be on the selection grid, if it's not,
+    // it might not be, but it won't be a broken connection (and the user asked for it!)
+    pos = bus->GetSeg().NearestPoint( pos );
+
+    m_toolMgr->RunAction( EE_ACTIONS::clearSelection );
 
     m_busUnfold.entry = new SCH_BUS_WIRE_ENTRY( pos );
-    m_busUnfold.entry->SetParent( m_frame->GetScreen() );
+    m_busUnfold.entry->SetParent( screen );
     m_frame->AddToScreen( m_busUnfold.entry, m_frame->GetScreen() );
 
     m_busUnfold.label = new SCH_LABEL( m_busUnfold.entry->GetEnd(), aNet );
-    m_busUnfold.label->SetTextSize( wxSize( cfg.m_DefaultTextSize, cfg.m_DefaultTextSize ) );
-    m_busUnfold.label->SetTextSpinStyle( TEXT_SPIN_STYLE::RIGHT );
+    m_busUnfold.label->SetTextSize( VECTOR2I( cfg.m_DefaultTextSize, cfg.m_DefaultTextSize ) );
+    m_busUnfold.label->SetSpinStyle( busUnfoldPersistentSettings.label_spin_style );
     m_busUnfold.label->SetParent( m_frame->GetScreen() );
     m_busUnfold.label->SetFlags( IS_NEW | IS_MOVING );
 
@@ -405,6 +429,17 @@ SCH_LINE* SCH_LINE_WIRE_BUS_TOOL::doUnfoldBus( const wxString& aNet, const VECTO
 
     getViewControls()->SetCrossHairCursorPosition( m_busUnfold.entry->GetEnd(), false );
 
+    std::vector<DANGLING_END_ITEM> endPointsByType;
+
+    for( SCH_ITEM* item : screen->Items().Overlapping( m_busUnfold.entry->GetBoundingBox() ) )
+        item->GetEndPoints( endPointsByType );
+
+    std::vector<DANGLING_END_ITEM> endPointsByPos = endPointsByType;
+    DANGLING_END_ITEM_HELPER::sort_dangling_end_items( endPointsByType, endPointsByPos );
+    m_busUnfold.entry->UpdateDanglingState( endPointsByType, endPointsByPos );
+    m_busUnfold.entry->SetEndDangling( false );
+    m_busUnfold.label->SetIsDangling( false );
+
     return startSegments( LAYER_WIRE, m_busUnfold.entry->GetEnd() );
 }
 
@@ -570,6 +605,7 @@ int SCH_LINE_WIRE_BUS_TOOL::doDrawSegments( const TOOL_EVENT& aTool, int aType,
     SCH_SCREEN*           screen = m_frame->GetScreen();
     SCH_LINE*             segment = nullptr;
     EE_GRID_HELPER        grid( m_toolMgr );
+    GRID_HELPER_GRIDS     gridType = ( aType == LAYER_NOTES ) ? GRID_GRAPHICS : GRID_WIRES;
     KIGFX::VIEW_CONTROLS* controls = getViewControls();
     int                   lastMode = m_frame->eeconfig()->m_Drawing.line_mode;
     static bool           posture = false;
@@ -590,7 +626,7 @@ int SCH_LINE_WIRE_BUS_TOOL::doDrawSegments( const TOOL_EVENT& aTool, int aType,
     auto cleanup =
             [&] ()
             {
-                m_toolMgr->RunAction( EE_ACTIONS::clearSelection, true );
+                m_toolMgr->RunAction( EE_ACTIONS::clearSelection );
 
                 for( SCH_LINE* wire : m_wires )
                     delete wire;
@@ -629,7 +665,7 @@ int SCH_LINE_WIRE_BUS_TOOL::doDrawSegments( const TOOL_EVENT& aTool, int aType,
     if( !m_wires.empty() )
         segment = m_wires.back();
 
-    wxPoint contextMenuPos;
+    VECTOR2I contextMenuPos;
 
     // Main loop: keep receiving events
     while( TOOL_EVENT* evt = Wait() )
@@ -658,7 +694,7 @@ int SCH_LINE_WIRE_BUS_TOOL::doDrawSegments( const TOOL_EVENT& aTool, int aType,
         VECTOR2D eventPosition = evt->HasPosition() ? evt->Position()
                                                     : controls->GetMousePosition();
 
-        VECTOR2I cursorPos = grid.BestSnapAnchor( eventPosition, LAYER_CONNECTABLE, segment );
+        VECTOR2I cursorPos = grid.BestSnapAnchor( eventPosition, gridType, segment );
         controls->ForceCursorPosition( true, cursorPos );
 
         // Need to handle change in H/V mode while drawing
@@ -737,10 +773,7 @@ int SCH_LINE_WIRE_BUS_TOOL::doDrawSegments( const TOOL_EVENT& aTool, int aType,
         //------------------------------------------------------------------------
         // Handle finish:
         //
-        else if( evt->IsAction( &EE_ACTIONS::finishLineWireOrBus )
-                     || evt->IsAction( &EE_ACTIONS::finishWire )
-                     || evt->IsAction( &EE_ACTIONS::finishBus )
-                     || evt->IsAction( &EE_ACTIONS::finishLine ) )
+        else if( evt->IsAction( &ACTIONS::finishInteractive ) )
         {
             if( segment || m_busUnfold.in_progress )
             {
@@ -820,8 +853,10 @@ int SCH_LINE_WIRE_BUS_TOOL::doDrawSegments( const TOOL_EVENT& aTool, int aType,
             if( evt->IsDblClick( BUT_LEFT ) && segment )
             {
                 if( twoSegments && m_wires.size() >= 2 )
+                {
                     computeBreakPoint( { m_wires[m_wires.size() - 2], segment }, cursorPos,
                                        currentMode, posture );
+                }
 
                 finishSegments();
                 segment = nullptr;
@@ -852,7 +887,7 @@ int SCH_LINE_WIRE_BUS_TOOL::doDrawSegments( const TOOL_EVENT& aTool, int aType,
                 // Erase and redraw if necessary
                 if( flipX != m_busUnfold.flipX || flipY != m_busUnfold.flipY )
                 {
-                    wxSize size  = entry->GetSize();
+                    VECTOR2I size  = entry->GetSize();
                     int    ySign = flipY ? -1 : 1;
                     int    xSign = flipX ? -1 : 1;
 
@@ -896,7 +931,9 @@ int SCH_LINE_WIRE_BUS_TOOL::doDrawSegments( const TOOL_EVENT& aTool, int aType,
                     m_view->AddToPreview( wire->Clone() );
             }
         }
-        else if( evt->IsAction( &EE_ACTIONS::undoLastSegment ) )
+        else if( evt->IsAction( &EE_ACTIONS::undoLastSegment )
+                 || evt->IsAction( &ACTIONS::doDelete )
+                 || evt->IsAction( &ACTIONS::undo ) )
         {
             if( ( currentMode == LINE_MODE::LINE_MODE_FREE && m_wires.size() > 1 )
                 || ( LINE_MODE::LINE_MODE_90 && m_wires.size() > 2 ) )
@@ -908,7 +945,8 @@ int SCH_LINE_WIRE_BUS_TOOL::doDrawSegments( const TOOL_EVENT& aTool, int aType,
                 delete segment;
 
                 segment = m_wires.back();
-                segment->SetEndPoint( cursorPos );
+                cursorPos = segment->GetEndPoint();
+                getViewControls()->WarpMouseCursor( cursorPos, true );
 
                 // Find new bend point for current mode
                 if( twoSegments && m_wires.size() >= 2 )
@@ -927,6 +965,11 @@ int SCH_LINE_WIRE_BUS_TOOL::doDrawSegments( const TOOL_EVENT& aTool, int aType,
                         m_view->AddToPreview( wire->Clone() );
                 }
             }
+            else if( evt->IsAction( &ACTIONS::undo ) )
+            {
+                // Dispatch as normal undo event
+                evt->SetPassEvent();
+            }
             else
             {
                 wxBell();
@@ -963,7 +1006,7 @@ int SCH_LINE_WIRE_BUS_TOOL::doDrawSegments( const TOOL_EVENT& aTool, int aType,
                 computeBreakPoint( { m_wires[m_wires.size() - 2], segment }, cursorPos, currentMode,
                                    posture );
 
-                m_toolMgr->RunAction( ACTIONS::refreshPreview );
+                m_toolMgr->PostAction( ACTIONS::refreshPreview );
             }
         }
         //------------------------------------------------------------------------
@@ -975,8 +1018,8 @@ int SCH_LINE_WIRE_BUS_TOOL::doDrawSegments( const TOOL_EVENT& aTool, int aType,
             if( !segment )
                 m_toolMgr->VetoContextMenuMouseWarp();
 
-            contextMenuPos = (wxPoint) cursorPos;
-            m_menu.ShowContextMenu( m_selectionTool->GetSelection() );
+            contextMenuPos = cursorPos;
+            m_menu->ShowContextMenu( m_selectionTool->GetSelection() );
         }
         else if( evt->Category() == TC_COMMAND && evt->Action() == TA_CHOICE_MENU_CHOICE )
         {
@@ -998,16 +1041,18 @@ int SCH_LINE_WIRE_BUS_TOOL::doDrawSegments( const TOOL_EVENT& aTool, int aType,
             if( m_busUnfold.in_progress )
             {
                 m_busUnfold.label->Rotate90( evt->IsAction( &EE_ACTIONS::rotateCW ) );
-                m_toolMgr->RunAction( ACTIONS::refreshPreview );
+                busUnfoldPersistentSettings.label_spin_style = m_busUnfold.label->GetSpinStyle();
+
+                m_toolMgr->PostAction( ACTIONS::refreshPreview );
             }
             else
             {
                 wxBell();
             }
         }
-        else if( evt->IsAction( &ACTIONS::doDelete ) && ( segment || m_busUnfold.in_progress ) )
+        else if( evt->IsAction( &ACTIONS::redo ) )
         {
-            cleanup();
+            wxBell();
         }
         else
         {
@@ -1030,8 +1075,10 @@ int SCH_LINE_WIRE_BUS_TOOL::doDrawSegments( const TOOL_EVENT& aTool, int aType,
 SCH_LINE* SCH_LINE_WIRE_BUS_TOOL::startSegments( int aType, const VECTOR2D& aPos,
                                                  SCH_LINE* aSegment )
 {
-    // If a segment isn't provided to copy properties from, we need to create one
-    if( aSegment == nullptr )
+    if( !aSegment )
+        aSegment = m_frame->GetScreen()->GetLine( aPos, 0, aType );
+
+    if( !aSegment )
     {
         switch( aType )
         {
@@ -1122,10 +1169,10 @@ void SCH_LINE_WIRE_BUS_TOOL::finishSegments()
     // Clear selection when done so that a new wire can be started.
     // NOTE: this must be done before simplifyWireList is called or we might end up with
     // freed selected items.
-    m_toolMgr->RunAction( EE_ACTIONS::clearSelection, true );
+    m_toolMgr->RunAction( EE_ACTIONS::clearSelection );
 
-    SCH_SCREEN*       screen = m_frame->GetScreen();
-    PICKED_ITEMS_LIST itemList;
+    SCH_SCREEN* screen = m_frame->GetScreen();
+    SCH_COMMIT  commit( m_toolMgr );
 
     // Remove segments backtracking over others
     simplifyWireList();
@@ -1150,17 +1197,17 @@ void SCH_LINE_WIRE_BUS_TOOL::finishSegments()
                 new_ends.push_back( pt );
         }
 
-        itemList.PushItem( ITEM_PICKER( screen, wire, UNDO_REDO::NEWITEM ) );
+        commit.Added( wire, screen );
     }
 
     if( m_busUnfold.in_progress && m_busUnfold.label_placed )
     {
         wxASSERT( m_busUnfold.entry && m_busUnfold.label );
 
-        itemList.PushItem( ITEM_PICKER( screen, m_busUnfold.entry, UNDO_REDO::NEWITEM ) );
+        commit.Added( m_busUnfold.entry, screen );
         m_frame->SaveCopyForRepeatItem( m_busUnfold.entry );
 
-        itemList.PushItem( ITEM_PICKER( screen, m_busUnfold.label, UNDO_REDO::NEWITEM ) );
+        commit.Added( m_busUnfold.label, screen );
         m_frame->AddCopyForRepeatItem( m_busUnfold.label );
         m_busUnfold.label->ClearEditFlags();
     }
@@ -1190,10 +1237,8 @@ void SCH_LINE_WIRE_BUS_TOOL::finishSegments()
     getViewControls()->CaptureCursor( false );
     getViewControls()->SetAutoPan( false );
 
-    m_frame->SaveCopyInUndoList( itemList, UNDO_REDO::NEWITEM, false );
-
     // Correct and remove segments that need to be merged.
-    m_frame->SchematicCleanUp();
+    m_frame->SchematicCleanUp( &commit );
 
     std::vector<SCH_ITEM*> symbols;
 
@@ -1210,14 +1255,14 @@ void SCH_LINE_WIRE_BUS_TOOL::finishSegments()
         for( auto pt = pts.begin(); pt != pts.end(); pt++ )
         {
             for( auto secondPt = pt + 1; secondPt != pts.end(); secondPt++ )
-                m_frame->TrimWire( *pt, *secondPt );
+                m_frame->TrimWire( &commit, *pt, *secondPt );
         }
     }
 
     for( const VECTOR2I& pt : new_ends )
     {
         if( m_frame->GetScreen()->IsExplicitJunctionNeeded( pt ) )
-            m_frame->AddJunction( m_frame->GetScreen(), pt, true, false );
+            m_frame->AddJunction( &commit, m_frame->GetScreen(), pt );
     }
 
     if( m_busUnfold.in_progress )
@@ -1226,14 +1271,11 @@ void SCH_LINE_WIRE_BUS_TOOL::finishSegments()
     for( SCH_ITEM* item : m_frame->GetScreen()->Items() )
         item->ClearEditFlags();
 
-    m_frame->TestDanglingEnds();
-    m_toolMgr->PostEvent( EVENTS::SelectedItemsModified );
-
-    m_frame->OnModify();
+    commit.Push( _( "Draw Wires" ) );
 }
 
 
-int SCH_LINE_WIRE_BUS_TOOL::TrimOverLappingWires(  EE_SELECTION* aSelection  )
+int SCH_LINE_WIRE_BUS_TOOL::TrimOverLappingWires( SCH_COMMIT* aCommit, EE_SELECTION* aSelection  )
 {
     SCHEMATIC* sch = getModel<SCHEMATIC>();
     SCH_SCREEN* screen = sch->CurrentSheet().LastScreen();
@@ -1259,7 +1301,7 @@ int SCH_LINE_WIRE_BUS_TOOL::TrimOverLappingWires(  EE_SELECTION* aSelection  )
         {
             std::vector<VECTOR2I> conn_pts;
 
-            for( VECTOR2I pt : pts )
+            for( const VECTOR2I& pt : pts )
             {
                 if( IsPointOnSegment( line->GetStartPoint(), line->GetEndPoint(), pt ) )
                     conn_pts.push_back( pt );
@@ -1269,7 +1311,7 @@ int SCH_LINE_WIRE_BUS_TOOL::TrimOverLappingWires(  EE_SELECTION* aSelection  )
             }
 
             if( conn_pts.size() == 2 )
-                m_frame->TrimWire( conn_pts[0], conn_pts[1] );
+                m_frame->TrimWire( aCommit, conn_pts[0], conn_pts[1] );
         }
     }
 
@@ -1277,12 +1319,12 @@ int SCH_LINE_WIRE_BUS_TOOL::TrimOverLappingWires(  EE_SELECTION* aSelection  )
 }
 
 
-int SCH_LINE_WIRE_BUS_TOOL::AddJunctionsIfNeeded( EE_SELECTION* aSelection )
+int SCH_LINE_WIRE_BUS_TOOL::AddJunctionsIfNeeded( SCH_COMMIT* aCommit, EE_SELECTION* aSelection )
 {
-    SCH_SCREEN*   screen = m_frame->GetScreen();
+    SCH_SCREEN* screen = m_frame->GetScreen();
 
     for( const VECTOR2I& point : screen->GetNeededJunctions( aSelection->Items() ) )
-        m_frame->AddJunction( m_frame->GetScreen(), point, true, false );
+        m_frame->AddJunction( aCommit, m_frame->GetScreen(), point );
 
     return 0;
 }
