@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2004 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2008 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2004-2024 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,6 +27,7 @@
 #include <sch_edit_frame.h>
 #include <sch_sheet.h>
 #include <sch_sheet_path.h>
+#include <sch_commit.h>
 #include <schematic.h>
 #include <tool/tool_manager.h>
 #include <tools/ee_actions.h>
@@ -47,11 +48,7 @@ class TREE_ITEM_DATA : public wxTreeItemData
 public:
     SCH_SHEET_PATH m_SheetPath;
 
-    TREE_ITEM_DATA( SCH_SHEET_PATH& sheet ) :
-            wxTreeItemData()
-    {
-        m_SheetPath = sheet;
-    }
+    TREE_ITEM_DATA( SCH_SHEET_PATH& sheet ) : wxTreeItemData(), m_SheetPath( sheet ) {}
 };
 
 
@@ -99,7 +96,7 @@ HIERARCHY_PANE::HIERARCHY_PANE( SCH_EDIT_FRAME* aParent ) :
     m_tree->AssignImageList( imageList );
 #endif
 
-    sizer->Add( m_tree, 1, wxEXPAND, wxBORDER_NONE, 0 );
+    sizer->Add( m_tree, 1, wxEXPAND, wxBORDER_NONE );
 
     m_events_bound = false;
 
@@ -108,8 +105,9 @@ HIERARCHY_PANE::HIERARCHY_PANE( SCH_EDIT_FRAME* aParent ) :
     // Enable selection events
     Bind( wxEVT_TREE_ITEM_ACTIVATED, &HIERARCHY_PANE::onSelectSheetPath, this );
     Bind( wxEVT_TREE_SEL_CHANGED, &HIERARCHY_PANE::onSelectSheetPath, this );
-    Bind( wxEVT_TREE_ITEM_RIGHT_CLICK, &HIERARCHY_PANE::onRightClick, this );
-
+    Bind( wxEVT_TREE_ITEM_RIGHT_CLICK, &HIERARCHY_PANE::onTreeItemRightClick, this );
+    Bind( wxEVT_CHAR_HOOK, &HIERARCHY_PANE::onCharHook, this );
+    m_tree->Bind( wxEVT_TREE_END_LABEL_EDIT, &HIERARCHY_PANE::onTreeEditFinished, this );
     m_events_bound = true;
 }
 
@@ -118,7 +116,9 @@ HIERARCHY_PANE::~HIERARCHY_PANE()
 {
     Unbind( wxEVT_TREE_ITEM_ACTIVATED, &HIERARCHY_PANE::onSelectSheetPath, this );
     Unbind( wxEVT_TREE_SEL_CHANGED, &HIERARCHY_PANE::onSelectSheetPath, this );
-    Unbind( wxEVT_TREE_ITEM_RIGHT_CLICK, &HIERARCHY_PANE::onRightClick, this );
+    Unbind( wxEVT_TREE_ITEM_RIGHT_CLICK, &HIERARCHY_PANE::onTreeItemRightClick, this );
+    Unbind( wxEVT_CHAR_HOOK, &HIERARCHY_PANE::onCharHook, this );
+    m_tree->Unbind( wxEVT_TREE_END_LABEL_EDIT, &HIERARCHY_PANE::onTreeEditFinished, this );
 }
 
 
@@ -132,8 +132,9 @@ void HIERARCHY_PANE::buildHierarchyTree( SCH_SHEET_PATH* aList, const wxTreeItem
         SCH_SHEET* sheet = static_cast<SCH_SHEET*>( aItem );
         aList->push_back( sheet );
 
-        wxString sheetName = formatPageString( sheet->GetFields()[SHEETNAME].GetShownText( false ),
-                                               aList->GetPageNumber() );
+        wxString     sheetNameBase = sheet->GetFields()[SHEETNAME].GetShownText( false );
+        wxString     sheetName = formatPageString( sheetNameBase, aList->GetPageNumber() );
+        wxString     sheetNumber = aList->GetPageNumber();
         wxTreeItemId child = m_tree->AppendItem( aParent, sheetName, 0, 1 );
         m_tree->SetItemData( child, new TREE_ITEM_DATA( *aList ) );
 
@@ -154,7 +155,7 @@ void HIERARCHY_PANE::UpdateHierarchySelection()
         // Disable selection events
         Unbind( wxEVT_TREE_ITEM_ACTIVATED, &HIERARCHY_PANE::onSelectSheetPath, this );
         Unbind( wxEVT_TREE_SEL_CHANGED, &HIERARCHY_PANE::onSelectSheetPath, this );
-        Unbind( wxEVT_TREE_ITEM_RIGHT_CLICK, &HIERARCHY_PANE::onRightClick, this );
+        Unbind( wxEVT_TREE_ITEM_RIGHT_CLICK, &HIERARCHY_PANE::onTreeItemRightClick, this );
 
         m_events_bound = false;
     }
@@ -201,7 +202,7 @@ void HIERARCHY_PANE::UpdateHierarchySelection()
         // Enable selection events
         Bind( wxEVT_TREE_ITEM_ACTIVATED, &HIERARCHY_PANE::onSelectSheetPath, this );
         Bind( wxEVT_TREE_SEL_CHANGED, &HIERARCHY_PANE::onSelectSheetPath, this );
-        Bind( wxEVT_TREE_ITEM_RIGHT_CLICK, &HIERARCHY_PANE::onRightClick, this );
+        Bind( wxEVT_TREE_ITEM_RIGHT_CLICK, &HIERARCHY_PANE::onTreeItemRightClick, this );
 
         m_events_bound = true;
     }
@@ -219,12 +220,12 @@ void HIERARCHY_PANE::UpdateHierarchyTree()
         // Disable selection events
         Unbind( wxEVT_TREE_ITEM_ACTIVATED, &HIERARCHY_PANE::onSelectSheetPath, this );
         Unbind( wxEVT_TREE_SEL_CHANGED, &HIERARCHY_PANE::onSelectSheetPath, this );
-        Unbind( wxEVT_TREE_ITEM_RIGHT_CLICK, &HIERARCHY_PANE::onRightClick, this );
+        Unbind( wxEVT_TREE_ITEM_RIGHT_CLICK, &HIERARCHY_PANE::onTreeItemRightClick, this );
 
         m_events_bound = false;
     }
 
-    SCH_SHEET_LIST hierarchy = m_frame->Schematic().GetUnorderedSheets();
+    SCH_SHEET_LIST hierarchy = m_frame->Schematic().Hierarchy();
     std::set<SCH_SHEET_PATH> expandedNodes;
 
     std::function<void( const wxTreeItemId& )> getExpandedNodes =
@@ -296,7 +297,7 @@ void HIERARCHY_PANE::UpdateHierarchyTree()
         // Enable selection events
         Bind( wxEVT_TREE_ITEM_ACTIVATED, &HIERARCHY_PANE::onSelectSheetPath, this );
         Bind( wxEVT_TREE_SEL_CHANGED, &HIERARCHY_PANE::onSelectSheetPath, this );
-        Bind( wxEVT_TREE_ITEM_RIGHT_CLICK, &HIERARCHY_PANE::onRightClick, this );
+        Bind( wxEVT_TREE_ITEM_RIGHT_CLICK, &HIERARCHY_PANE::onTreeItemRightClick, this );
 
         m_events_bound = true;
     }
@@ -334,8 +335,8 @@ void HIERARCHY_PANE::UpdateLabelsHierarchyTree()
             [&]( const wxTreeItemId& id )
             {
                 TREE_ITEM_DATA* itemData = static_cast<TREE_ITEM_DATA*>( m_tree->GetItemData( id ) );
-                SCH_SHEET* sheet = itemData->m_SheetPath.Last();
-                wxString sheetNameLabel =
+                SCH_SHEET*      sheet = itemData->m_SheetPath.Last();
+                wxString        sheetNameLabel =
                         formatPageString( sheet->GetFields()[SHEETNAME].GetShownText( false ),
                                           itemData->m_SheetPath.GetPageNumber() );
 
@@ -365,23 +366,43 @@ void HIERARCHY_PANE::UpdateLabelsHierarchyTree()
 }
 
 
-void HIERARCHY_PANE::onRightClick( wxTreeEvent& aEvent )
+void HIERARCHY_PANE::onTreeItemRightClick( wxTreeEvent& aEvent )
 {
-    wxTreeItemId  itemSel = aEvent.GetItem();
+    onRightClick( aEvent.GetItem() );
+}
 
-    if( !itemSel.IsOk() )
-        return;
 
-    TREE_ITEM_DATA* itemData = static_cast<TREE_ITEM_DATA*>( m_tree->GetItemData( itemSel ) );
+void HIERARCHY_PANE::onRightClick( wxTreeItemId aItem )
+{
+    wxMenu          ctxMenu;
+    TREE_ITEM_DATA* itemData = nullptr;
 
-    if( !itemData )
-        return;
+    if( !aItem.IsOk() )
+        aItem = m_tree->GetSelection();
 
-    wxMenu ctxMenu;
+    if( aItem.IsOk() )
+        itemData = static_cast<TREE_ITEM_DATA*>( m_tree->GetItemData( aItem ) );
 
-    ctxMenu.Append( 1, _( "Edit Page Number" ) );
+    if( itemData )
+    {
+        ctxMenu.Append( EDIT_PAGE_NUMBER, _( "Edit Page Number" ) );
+        // The root item cannot be renamed
+        if( m_tree->GetRootItem() != aItem.GetID() )
+        {
+            ctxMenu.Append( RENAME, _( "Rename" ), _( "Change name of this sheet" ) );
+        }
 
-    if( GetPopupMenuSelectionFromUser( ctxMenu ) == 1 )
+        ctxMenu.AppendSeparator();
+    }
+    ctxMenu.Append( EXPAND_ALL, ACTIONS::expandAll.GetMenuItem() );
+    ctxMenu.Append( COLLAPSE_ALL, ACTIONS::collapseAll.GetMenuItem() );
+
+
+    int selected = GetPopupMenuSelectionFromUser( ctxMenu );
+
+    switch( selected )
+    {
+    case EDIT_PAGE_NUMBER:
     {
         wxString msg;
         wxString sheetPath = itemData->m_SheetPath.PathHumanReadable( false, true );
@@ -397,11 +418,11 @@ void HIERARCHY_PANE::onRightClick( wxTreeEvent& aEvent )
 
         if( dlg.ShowModal() == wxID_OK && dlg.GetValue() != itemData->m_SheetPath.GetPageNumber() )
         {
+            SCH_COMMIT commit( m_frame );
             SCH_SHEET_PATH parentPath = itemData->m_SheetPath;
             parentPath.pop_back();
 
-            m_frame->SaveCopyInUndoList( parentPath.LastScreen(), itemData->m_SheetPath.Last(),
-                                         UNDO_REDO::CHANGED, false );
+            commit.Modify( itemData->m_SheetPath.Last(), parentPath.LastScreen() );
 
             itemData->m_SheetPath.SetPageNumber( dlg.GetValue() );
 
@@ -411,10 +432,100 @@ void HIERARCHY_PANE::onRightClick( wxTreeEvent& aEvent )
                 m_frame->OnPageSettingsChange();
             }
 
-            m_frame->OnModify();
+            commit.Push( wxS( "Change sheet page number." ) );
 
             UpdateLabelsHierarchyTree();
         }
+
+        break;
+    }
+    case EXPAND_ALL:
+        m_tree->ExpandAll();
+        break;
+    case COLLAPSE_ALL:
+        m_tree->CollapseAll();
+        break;
+    case RENAME:
+        m_tree->SetItemText( aItem, itemData->m_SheetPath.Last()->GetName() );
+        m_tree->EditLabel( aItem );
+        setIdenticalSheetsHighlighted( itemData->m_SheetPath );
+        break;
+    }
+}
+
+
+void HIERARCHY_PANE::onTreeEditFinished( wxTreeEvent& event )
+{
+    TREE_ITEM_DATA* data = static_cast<TREE_ITEM_DATA*>( m_tree->GetItemData( event.GetItem() ) );
+    wxString        newName = event.GetLabel();
+
+    if( !newName.IsEmpty() )
+    {
+        // The editor holds only the page name as a text, while normally
+        // the tree items displaying it suffixed with the page number
+        if( data->m_SheetPath.Last()->GetName() != newName )
+        {
+            const SCH_SHEET* parentSheet =
+                    data->m_SheetPath.GetSheet( data->m_SheetPath.size() - 2 );
+
+            if( parentSheet )
+            {
+                SCH_COMMIT commit( m_frame );
+                commit.Modify( &data->m_SheetPath.Last()->GetFields()[SHEETNAME],
+                               parentSheet->GetScreen() );
+
+                data->m_SheetPath.Last()->SetName( newName );
+
+                renameIdenticalSheets( data->m_SheetPath, newName, &commit );
+
+                if( !commit.Empty() )
+                    commit.Push( _( "Renaming sheet" ) );
+
+                if( data->m_SheetPath == m_frame->GetCurrentSheet() )
+                {
+                    m_frame->OnPageSettingsChange();
+                }
+            }
+        }
+    }
+
+    m_tree->SetItemText( event.GetItem(), formatPageString( data->m_SheetPath.Last()->GetName(),
+                                                            data->m_SheetPath.GetPageNumber() ) );
+    setIdenticalSheetsHighlighted( data->m_SheetPath, false );
+    // The event needs to be rejected otherwise the SetItemText call above
+    // will be ineffective (the treeview item will hold the editor's content)
+    event.Veto();
+}
+
+
+void HIERARCHY_PANE::onCharHook( wxKeyEvent& aKeyStroke )
+{
+    int hotkey = aKeyStroke.GetKeyCode();
+
+    if( aKeyStroke.GetModifiers() & wxMOD_CONTROL )
+        hotkey += MD_CTRL;
+
+    if( aKeyStroke.GetModifiers() & wxMOD_ALT )
+        hotkey += MD_ALT;
+
+    if( aKeyStroke.GetModifiers() & wxMOD_SHIFT )
+        hotkey += MD_SHIFT;
+
+    if( hotkey == ACTIONS::expandAll.GetHotKey()
+        || hotkey == ACTIONS::expandAll.GetHotKeyAlt() )
+    {
+        m_tree->ExpandAll();
+        return;
+    }
+    else if( hotkey == ACTIONS::collapseAll.GetHotKey()
+             || hotkey == ACTIONS::collapseAll.GetHotKeyAlt() )
+    {
+        m_tree->CollapseAll();
+        return;
+    }
+    else
+    {
+        aKeyStroke.Skip();
     }
 }
 
@@ -433,3 +544,72 @@ wxString HIERARCHY_PANE::formatPageString( const wxString& aName, const wxString
 {
     return aName + wxT( " " ) + wxString::Format( _( "(page %s)" ), aPage );
 }
+
+void HIERARCHY_PANE::setIdenticalSheetsHighlighted( const SCH_SHEET_PATH& path, bool highLighted )
+{
+    std::function<void( const wxTreeItemId& )> recursiveDescent = [&]( const wxTreeItemId& id )
+    {
+        wxCHECK_RET( id.IsOk(), wxT( "Invalid tree item" ) );
+
+        TREE_ITEM_DATA* itemData = static_cast<TREE_ITEM_DATA*>( m_tree->GetItemData( id ) );
+
+        if( itemData->m_SheetPath.Cmp( path ) != 0 && itemData->m_SheetPath.Last() == path.Last() )
+        {
+            wxFont font = m_tree->GetItemFont( id );
+            font.SetUnderlined( highLighted );
+            m_tree->SetItemFont( id, font );
+        }
+
+        wxTreeItemIdValue cookie;
+        wxTreeItemId      child = m_tree->GetFirstChild( id, cookie );
+
+        while( child.IsOk() )
+        {
+            recursiveDescent( child );
+            child = m_tree->GetNextChild( id, cookie );
+        }
+    };
+
+    recursiveDescent( m_tree->GetRootItem() );
+}
+
+void HIERARCHY_PANE::renameIdenticalSheets( const SCH_SHEET_PATH& renamedSheet,
+                                            const wxString newName, SCH_COMMIT* commit )
+{
+    std::function<void( const wxTreeItemId& )> recursiveDescent = [&]( const wxTreeItemId& id )
+    {
+        wxCHECK_RET( id.IsOk(), wxT( "Invalid tree item" ) );
+
+        TREE_ITEM_DATA* data = static_cast<TREE_ITEM_DATA*>( m_tree->GetItemData( id ) );
+
+        const SCH_SHEET* parentSheet = data->m_SheetPath.GetSheet( data->m_SheetPath.size() - 2 );
+
+        if( parentSheet && data->m_SheetPath.Cmp( renamedSheet ) != 0
+            && data->m_SheetPath.Last() == renamedSheet.Last() )
+        {
+            commit->Modify( &data->m_SheetPath.Last()->GetFields()[SHEETNAME],
+                            parentSheet->GetScreen() );
+
+            data->m_SheetPath.Last()->SetName( newName );
+
+            if( data->m_SheetPath == m_frame->GetCurrentSheet() )
+            {
+                m_frame->OnPageSettingsChange();
+            }
+
+            m_tree->SetItemText( id, formatPageString( data->m_SheetPath.Last()->GetName(),
+                                                       data->m_SheetPath.GetPageNumber() ) );
+        }
+
+        wxTreeItemIdValue cookie;
+        wxTreeItemId      child = m_tree->GetFirstChild( id, cookie );
+
+        while( child.IsOk() )
+        {
+            recursiveDescent( child );
+            child = m_tree->GetNextChild( id, cookie );
+        }
+    };
+
+    recursiveDescent( m_tree->GetRootItem() );
+}
