@@ -1,8 +1,8 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2007-2014 Jean-Pierre Charras  jp.charras at wanadoo.fr
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2007-2023 Jean-Pierre Charras  jp.charras at wanadoo.fr
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,6 +29,7 @@
 #include <confirm.h>
 #include <string_utils.h>
 #include <locale_io.h>
+#include <lset.h>
 #include <macros.h>
 #include <trigo.h>
 #include <gerbview_frame.h>
@@ -37,9 +38,7 @@
 #include <build_version.h>
 #include <wildcards_and_files_ext.h>
 #include "excellon_image.h"
-
-// Imported function
-extern const wxString GetPCBDefaultLayerName( int aLayerNumber );
+#include <wx/log.h>
 
 
 GBR_TO_PCB_EXPORTER::GBR_TO_PCB_EXPORTER( GERBVIEW_FRAME* aFrame, const wxString& aFileName )
@@ -75,7 +74,6 @@ bool GBR_TO_PCB_EXPORTER::ExportPcb( const int* aLayerLookUpTable, int aCopperLa
     writePcbHeader( aLayerLookUpTable );
 
     // create an image of gerber data
-    const int pcbCopperLayerMax = 31;
     GERBER_FILE_IMAGE_LIST* images = m_gerbview_frame->GetGerberLayout()->GetImagesList();
 
     // First collect all the holes.  We'll use these to generate pads, vias, etc.
@@ -84,6 +82,7 @@ bool GBR_TO_PCB_EXPORTER::ExportPcb( const int* aLayerLookUpTable, int aCopperLa
         int pcb_layer_number = aLayerLookUpTable[layer];
         EXCELLON_IMAGE* excellon = dynamic_cast<EXCELLON_IMAGE*>( images->GetGbrImage( layer ) );
         GERBER_FILE_IMAGE* gerb  = dynamic_cast<GERBER_FILE_IMAGE*>( images->GetGbrImage( layer ) );
+
         if( excellon )
         {
             for( GERBER_DRAW_ITEM* gerb_item : excellon->GetItems() )
@@ -111,10 +110,7 @@ bool GBR_TO_PCB_EXPORTER::ExportPcb( const int* aLayerLookUpTable, int aCopperLa
 
         int pcb_layer_number = aLayerLookUpTable[layer];
 
-        if( !IsPcbLayer( pcb_layer_number ) )
-            continue;
-
-        if( pcb_layer_number <= pcbCopperLayerMax ) // copper layer
+        if( !IsPcbLayer( pcb_layer_number ) || IsCopperLayer( pcb_layer_number ) )
             continue;
 
         for( GERBER_DRAW_ITEM* gerb_item : gerber->GetItems() )
@@ -131,7 +127,7 @@ bool GBR_TO_PCB_EXPORTER::ExportPcb( const int* aLayerLookUpTable, int aCopperLa
 
         int pcb_layer_number = aLayerLookUpTable[layer];
 
-        if( pcb_layer_number < 0 || pcb_layer_number > pcbCopperLayerMax )
+        if( !IsCopperLayer( pcb_layer_number ) )
             continue;
 
         for( GERBER_DRAW_ITEM* gerb_item : gerber->GetItems() )
@@ -182,49 +178,17 @@ void GBR_TO_PCB_EXPORTER::export_non_copper_item( const GERBER_DRAW_ITEM* aGbrIt
     case GBR_SPOT_POLY:
     case GBR_SPOT_MACRO:
         d_codeDescr->ConvertShapeToPolygon( aGbrItem );
-        writePcbPolygon( d_codeDescr->m_Polygon, aLayer, aGbrItem->GetABPosition( seg_start ) );
+        {
+        SHAPE_POLY_SET polyshape = d_codeDescr->m_Polygon;
+
+        // Compensate the Y axis orientation ( writePcbPolygon invert the Y coordinate )
+        polyshape.Outline( 0 ).Mirror( { 0, 0 }, FLIP_DIRECTION::TOP_BOTTOM );
+        writePcbPolygon( polyshape, aLayer, aGbrItem->GetABPosition( seg_start ) );
+        }
         break;
 
     case GBR_ARC:
-    {
-        double a = atan2( (double) ( aGbrItem->m_Start.y - aGbrItem->m_ArcCentre.y ),
-                          (double) ( aGbrItem->m_Start.x - aGbrItem->m_ArcCentre.x ) );
-        double b = atan2( (double) ( aGbrItem->m_End.y - aGbrItem->m_ArcCentre.y ),
-                          (double) ( aGbrItem->m_End.x - aGbrItem->m_ArcCentre.x ) );
-
-        double angle = RAD2DEG(b - a);
-        seg_start = aGbrItem->m_ArcCentre;
-
-        // Ensure arc orientation is CCW
-        if( angle < 0 )
-            angle += 360.0;
-
-        // Reverse Y axis:
-        seg_start.y = -seg_start.y;
-        seg_end.y = -seg_end.y;
-
-        if( angle == 360.0 ||  angle == 0 )
-        {
-            fprintf( m_fp, "(gr_circle (center %s %s) (end %s %s) (layer %s) (width %s))\n",
-                     FormatDouble2Str( MapToPcbUnits( seg_start.x ) ).c_str(),
-                     FormatDouble2Str( MapToPcbUnits( seg_start.y ) ).c_str(),
-                     FormatDouble2Str( MapToPcbUnits( seg_end.x ) ).c_str(),
-                     FormatDouble2Str( MapToPcbUnits( seg_end.y ) ).c_str(),
-                     TO_UTF8( GetPCBDefaultLayerName( aLayer ) ),
-                     FormatDouble2Str( MapToPcbUnits( aGbrItem->m_Size.x ) ).c_str() );
-        }
-        else
-        {
-            fprintf( m_fp, "(gr_arc (start %s %s) (end %s %s) (angle %s) (layer %s) (width %s))\n",
-                     FormatDouble2Str( MapToPcbUnits( seg_start.x ) ).c_str(),
-                     FormatDouble2Str( MapToPcbUnits( seg_start.y ) ).c_str(),
-                     FormatDouble2Str( MapToPcbUnits( seg_end.x ) ).c_str(),
-                     FormatDouble2Str( MapToPcbUnits( seg_end.y ) ).c_str(),
-                     FormatDouble2Str( angle ).c_str(),
-                     TO_UTF8( GetPCBDefaultLayerName( aLayer ) ),
-                     FormatDouble2Str( MapToPcbUnits( aGbrItem->m_Size.x ) ).c_str() );
-        }
-    }
+        export_non_copper_arc( aGbrItem, aLayer );
         break;
 
     case GBR_CIRCLE:
@@ -232,28 +196,99 @@ void GBR_TO_PCB_EXPORTER::export_non_copper_item( const GERBER_DRAW_ITEM* aGbrIt
         seg_start.y = -seg_start.y;
         seg_end.y = -seg_end.y;
 
-        fprintf( m_fp, "(gr_circle (start %s %s) (end %s %s) (layer %s) (width %s))\n",
+        fprintf( m_fp, "\t(gr_circle (start %s %s) (end %s %s) (layer %s)\n",
                  FormatDouble2Str( MapToPcbUnits( seg_start.x ) ).c_str(),
                  FormatDouble2Str( MapToPcbUnits( seg_start.y ) ).c_str(),
                  FormatDouble2Str( MapToPcbUnits( seg_end.x ) ).c_str(),
                  FormatDouble2Str( MapToPcbUnits( seg_end.y ) ).c_str(),
-                 TO_UTF8( GetPCBDefaultLayerName( aLayer ) ),
-                 FormatDouble2Str( MapToPcbUnits( aGbrItem->m_Size.x ) ).c_str() );
+                 LSET::Name( PCB_LAYER_ID( aLayer ) ).ToStdString().c_str() );
+        export_stroke_info( aGbrItem->m_Size.x );
+        fprintf( m_fp, "\t)\n" );
         break;
 
     case GBR_SEGMENT:
+        if( d_codeDescr->m_ApertType == APT_RECT )
+        {
+            // Using a rectangular aperture to draw a line is deprecated since 2020
+            // However old gerber file can use it (rare case) and can generate
+            // strange shapes, because the rect aperture is not rotated to match the
+            // line orientation.
+            // So draw this line as polygon
+            SHAPE_POLY_SET polyshape;
+            aGbrItem->ConvertSegmentToPolygon( &polyshape );
+            writePcbPolygon( polyshape, aLayer );
+        }
+        else
+        {
+            // Reverse Y axis:
+            seg_start.y = -seg_start.y;
+            seg_end.y = -seg_end.y;
+
+            fprintf( m_fp, "\t(gr_line\n\t\t(start %s %s) (end %s %s) (layer %s)\n",
+                     FormatDouble2Str( MapToPcbUnits( seg_start.x ) ).c_str(),
+                     FormatDouble2Str( MapToPcbUnits( seg_start.y ) ).c_str(),
+                     FormatDouble2Str( MapToPcbUnits( seg_end.x ) ).c_str(),
+                     FormatDouble2Str( MapToPcbUnits( seg_end.y ) ).c_str(),
+                     LSET::Name( PCB_LAYER_ID( aLayer ) ).ToStdString().c_str() );
+
+            export_stroke_info( aGbrItem->m_Size.x );
+            fprintf( m_fp, "\t)\n" );
+        }
+        break;
+    }
+}
+
+
+void GBR_TO_PCB_EXPORTER::export_non_copper_arc( const GERBER_DRAW_ITEM* aGbrItem, int aLayer )
+{
+    double a = atan2( (double) ( aGbrItem->m_Start.y - aGbrItem->m_ArcCentre.y ),
+                      (double) ( aGbrItem->m_Start.x - aGbrItem->m_ArcCentre.x ) );
+    double b = atan2( (double) ( aGbrItem->m_End.y - aGbrItem->m_ArcCentre.y ),
+                      (double) ( aGbrItem->m_End.x - aGbrItem->m_ArcCentre.x ) );
+
+    VECTOR2I arc_center = aGbrItem->m_ArcCentre;
+    VECTOR2I seg_start = aGbrItem->m_Start;
+    VECTOR2I seg_end = aGbrItem->m_End;
+
+    if( a > b )
+        b += 2 * M_PI;
+
+    if( seg_start == seg_end )
+    {
         // Reverse Y axis:
+        arc_center.y = -arc_center.y;
+        seg_end.y = -seg_end.y;
+
+        fprintf( m_fp, "\t(gr_circle\n\t\t(center %s %s) (end %s %s) (layer %s)\n",
+                 FormatDouble2Str( MapToPcbUnits( arc_center.x ) ).c_str(),
+                 FormatDouble2Str( MapToPcbUnits( arc_center.y ) ).c_str(),
+                 FormatDouble2Str( MapToPcbUnits( seg_end.x ) ).c_str(),
+                 FormatDouble2Str( MapToPcbUnits( seg_end.y ) ).c_str(),
+                 LSET::Name( PCB_LAYER_ID( aLayer ) ).ToStdString().c_str() );
+        export_stroke_info( aGbrItem->m_Size.x );
+        fprintf( m_fp, "\t)\n" );
+    }
+    else
+    {
+        VECTOR2I seg_middle = GetRotated( seg_start, arc_center,
+                                          -EDA_ANGLE( (b-a)/2, RADIANS_T ));
+
+        // Reverse Y axis:
+        seg_middle.y = -seg_middle.y;
         seg_start.y = -seg_start.y;
         seg_end.y = -seg_end.y;
 
-        fprintf( m_fp, "(gr_line (start %s %s) (end %s %s) (layer %s) (width %s))\n",
+        fprintf( m_fp, "\t(gr_arc\n\t\t(start %s %s) (mid %s %s) (end %s %s) (layer %s)\n",
                  FormatDouble2Str( MapToPcbUnits( seg_start.x ) ).c_str(),
                  FormatDouble2Str( MapToPcbUnits( seg_start.y ) ).c_str(),
+                 FormatDouble2Str( MapToPcbUnits( seg_middle.x ) ).c_str(),
+                 FormatDouble2Str( MapToPcbUnits( seg_middle.y ) ).c_str(),
                  FormatDouble2Str( MapToPcbUnits( seg_end.x ) ).c_str(),
                  FormatDouble2Str( MapToPcbUnits( seg_end.y ) ).c_str(),
-                 TO_UTF8( GetPCBDefaultLayerName( aLayer ) ),
-                 FormatDouble2Str( MapToPcbUnits( aGbrItem->m_Size.x ) ).c_str() );
-        break;
+                 LSET::Name( PCB_LAYER_ID( aLayer ) ).ToStdString().c_str() );
+
+        export_stroke_info( aGbrItem->m_Size.x );
+        fprintf( m_fp, "\t)\n" );
     }
 }
 
@@ -280,8 +315,8 @@ void GBR_TO_PCB_EXPORTER::export_via( const EXPORT_VIA& aVia )
              FormatDouble2Str( MapToPcbUnits( aVia.m_Drill ) ).c_str() );
 
     fprintf( m_fp, " (layers %s %s))\n",
-             TO_UTF8( GetPCBDefaultLayerName( F_Cu ) ),
-             TO_UTF8( GetPCBDefaultLayerName( B_Cu ) ) );
+             LSET::Name( F_Cu ).ToStdString().c_str(),
+             LSET::Name( B_Cu ).ToStdString().c_str() );
 }
 
 
@@ -297,6 +332,7 @@ void GBR_TO_PCB_EXPORTER::export_copper_item( const GERBER_DRAW_ITEM* aGbrItem,
         export_flashed_copper_item( aGbrItem, aLayer );
         break;
 
+    case GBR_CIRCLE:
     case GBR_ARC:
         export_segarc_copper_item( aGbrItem, aLayer );
         break;
@@ -314,8 +350,26 @@ void GBR_TO_PCB_EXPORTER::export_copper_item( const GERBER_DRAW_ITEM* aGbrItem,
 #endif
         break;
 
+    case GBR_SEGMENT:
+    {
+        D_CODE* code = aGbrItem->GetDcodeDescr();
+
+        if( code && code->m_ApertType == APT_RECT )
+        {
+            if( aGbrItem->m_ShapeAsPolygon.OutlineCount() == 0 )
+                const_cast<GERBER_DRAW_ITEM*>( aGbrItem )->ConvertSegmentToPolygon();
+
+            writePcbPolygon( aGbrItem->m_ShapeAsPolygon, aLayer );
+        }
+        else
+        {
+            export_segline_copper_item( aGbrItem, aLayer );
+        }
+
+        break;
+    }
+
     default:
-        export_segline_copper_item( aGbrItem, aLayer );
         break;
     }
 }
@@ -339,13 +393,20 @@ void GBR_TO_PCB_EXPORTER::export_segline_copper_item( const GERBER_DRAW_ITEM* aG
 void GBR_TO_PCB_EXPORTER::writeCopperLineItem( const VECTOR2I& aStart, const VECTOR2I& aEnd,
                                                int aWidth, int aLayer )
 {
-  fprintf( m_fp, "(segment (start %s %s) (end %s %s) (width %s) (layer %s) (net 0))\n",
+  fprintf( m_fp, "\t(segment (start %s %s) (end %s %s) (width %s) (layer %s) (net 0))\n",
            FormatDouble2Str( MapToPcbUnits(aStart.x) ).c_str(),
            FormatDouble2Str( MapToPcbUnits(aStart.y) ).c_str(),
            FormatDouble2Str( MapToPcbUnits(aEnd.x) ).c_str(),
            FormatDouble2Str( MapToPcbUnits(aEnd.y) ).c_str(),
            FormatDouble2Str( MapToPcbUnits( aWidth ) ).c_str(),
-           TO_UTF8( GetPCBDefaultLayerName( aLayer ) ) );
+           LSET::Name( PCB_LAYER_ID( aLayer ) ).ToStdString().c_str() );
+}
+
+
+void GBR_TO_PCB_EXPORTER::export_stroke_info( double aWidth )
+{
+    fprintf( m_fp, "\t\t(stroke (width %s) (type solid))\n",
+                FormatDouble2Str( MapToPcbUnits( aWidth ) ).c_str() );
 }
 
 
@@ -356,53 +417,36 @@ void GBR_TO_PCB_EXPORTER::export_segarc_copper_item( const GERBER_DRAW_ITEM* aGb
     double  b = atan2( (double) ( aGbrItem->m_End.y - aGbrItem->m_ArcCentre.y ),
                        (double) ( aGbrItem->m_End.x - aGbrItem->m_ArcCentre.x ) );
 
-    VECTOR2I start = aGbrItem->m_Start;
-    VECTOR2I end = aGbrItem->m_End;
-
-    /* Because Pcbnew does not know arcs in tracks,
-     * approximate arc by segments (SEG_COUNT__CIRCLE segment per 360 deg)
-     * The arc is drawn anticlockwise from the start point to the end point.
-     */
-    #define SEG_COUNT_CIRCLE    16
-    #define DELTA_ANGLE         2 * M_PI / SEG_COUNT_CIRCLE
-
-    // calculate the number of segments from a to b.
-    // we want CNT_PER_360 segments fo a circle
     if( a > b )
         b += 2 * M_PI;
 
-    VECTOR2I curr_start = start;
-    VECTOR2I seg_start, seg_end;
+    VECTOR2I arc_center = aGbrItem->m_ArcCentre;
+    VECTOR2I seg_end = aGbrItem->m_End;
+    VECTOR2I seg_start = aGbrItem->m_Start;
 
-    int     ii = 1;
+    VECTOR2I seg_middle = GetRotated( seg_start, arc_center,
+                                      -EDA_ANGLE( (b-a)/2, RADIANS_T ));
 
-    for( double rot = a; rot < (b - DELTA_ANGLE); rot += DELTA_ANGLE, ii++ )
-    {
-        seg_start = curr_start;
-        VECTOR2I curr_end = start;
-        RotatePoint( curr_end, aGbrItem->m_ArcCentre, -EDA_ANGLE( DELTA_ANGLE, RADIANS_T ) * ii );
-        seg_end = curr_end;
+    // Reverse Y axis:
+    seg_end.y = -seg_end.y;
+    seg_start.y = -seg_start.y;
+    seg_middle.y = -seg_middle.y;
 
-        // Reverse Y axis:
-        seg_start.y = -seg_start.y;
-        seg_end.y = -seg_end.y;
-        writeCopperLineItem( seg_start, seg_end, aGbrItem->m_Size.x, aLayer );
-        curr_start = curr_end;
-    }
+    fprintf( m_fp, "\t(arc\n\t\t(start %s %s) (mid %s %s) (end %s %s) (layer %s)\n",
+             FormatDouble2Str( MapToPcbUnits( seg_start.x ) ).c_str(),
+             FormatDouble2Str( MapToPcbUnits( seg_start.y ) ).c_str(),
+             FormatDouble2Str( MapToPcbUnits( seg_middle.x ) ).c_str(),
+             FormatDouble2Str( MapToPcbUnits( seg_middle.y ) ).c_str(),
+             FormatDouble2Str( MapToPcbUnits( seg_end.x ) ).c_str(),
+             FormatDouble2Str( MapToPcbUnits( seg_end.y ) ).c_str(),
+             LSET::Name( PCB_LAYER_ID( aLayer ) ).ToStdString().c_str() );
 
-    if( end != curr_start )
-    {
-        seg_start   = curr_start;
-        seg_end     = end;
-
-        // Reverse Y axis:
-        seg_start.y = -seg_start.y;
-        seg_end.y = -seg_end.y;
-        writeCopperLineItem( seg_start, seg_end, aGbrItem->m_Size.x, aLayer );
-    }
+    fprintf( m_fp, "\t\t(width %s) (net 0 )\n",
+             FormatDouble2Str( MapToPcbUnits( aGbrItem->m_Size.x ) ).c_str() );
+    fprintf( m_fp, "\t)\n" );
 }
 
-#include <wx/log.h>
+
 void GBR_TO_PCB_EXPORTER::export_flashed_copper_item( const GERBER_DRAW_ITEM* aGbrItem, int aLayer )
 {
     static D_CODE  flashed_item_D_CODE( 0 );
@@ -445,7 +489,7 @@ void GBR_TO_PCB_EXPORTER::export_flashed_copper_item( const GERBER_DRAW_ITEM* aG
         macroShape = *macro->GetApertureMacroShape( aGbrItem, VECTOR2I( 0, 0 ) );
 
         // Compensate the Y axis orientation ( writePcbPolygon invert the Y coordinate )
-        macroShape.Outline( 0 ).Mirror( false, true );
+        macroShape.Outline( 0 ).Mirror( { 0, 0 }, FLIP_DIRECTION::TOP_BOTTOM );
 
         writePcbPolygon( macroShape, aLayer, offset );
     }
@@ -461,45 +505,44 @@ void GBR_TO_PCB_EXPORTER::export_flashed_copper_item( const GERBER_DRAW_ITEM* aG
 void GBR_TO_PCB_EXPORTER::writePcbFilledCircle( const VECTOR2I& aCenterPosition, int aRadius,
                                                 int aLayer )
 {
-
-    fprintf( m_fp, "(gr_circle (center %s %s) (end %s %s)",
+    fprintf( m_fp, "\t(gr_circle\n\t\t(center %s %s) (end %s %s)\n",
              FormatDouble2Str( MapToPcbUnits( aCenterPosition.x ) ).c_str(),
              FormatDouble2Str( MapToPcbUnits( aCenterPosition.y ) ).c_str(),
              FormatDouble2Str( MapToPcbUnits( aCenterPosition.x + aRadius ) ).c_str(),
              FormatDouble2Str( MapToPcbUnits( aCenterPosition.y ) ).c_str() );
 
-
-    fprintf( m_fp, "(layer %s) (width 0) (fill solid) )\n",
-             TO_UTF8( GetPCBDefaultLayerName( aLayer ) ) );
+    export_stroke_info( 0 );
+    fprintf( m_fp, "\t\t(fill yes) (layer %s)",
+             LSET::Name( PCB_LAYER_ID( aLayer ) ).ToStdString().c_str() );
+    fprintf( m_fp, "\n\t)\n" );
 }
 
 
 void GBR_TO_PCB_EXPORTER::writePcbHeader( const int* aLayerLookUpTable )
 {
-    fprintf( m_fp, "(kicad_pcb (version 4) (generator gerbview)\n\n" );
+    // Note: the .kicad_pcb version used here is after layers_id changes
+    fprintf( m_fp, "(kicad_pcb (version 20240928)\n" );
+    fprintf( m_fp, "\t(generator \"gerbview\")\n\t(generator_version \"%s\")\n\n",
+             GetMajorMinorVersion().c_str().AsChar() );
 
     // Write layers section
-    fprintf( m_fp, "  (layers \n" );
+    fprintf( m_fp, "\t(layers \n" );
 
-    for( int ii = 0; ii < m_pcbCopperLayersCount; ii++ )
+    LSET layer_set = LSET::AllCuMask( m_pcbCopperLayersCount ) | LSET::AllTechMask() | LSET::UserMask();
+
+    for( auto cu_it = layer_set.copper_layers_begin(); cu_it != layer_set.copper_layers_end(); ++cu_it )
     {
-        int id = ii;
-
-        if( ii == m_pcbCopperLayersCount-1)
-            id = B_Cu;
-
-        fprintf( m_fp, "    (%d %s signal)\n", id, TO_UTF8( GetPCBDefaultLayerName( id ) ) );
+        fprintf( m_fp, "\t\t(%d %s signal)\n",
+                 *cu_it, LSET::Name( *cu_it ).ToStdString().c_str() );
     }
 
-    for( int ii = B_Adhes; ii < PCB_LAYER_ID_COUNT; ii++ )
+    for( auto non_cu_it = layer_set.non_copper_layers_begin(); non_cu_it != layer_set.non_copper_layers_end(); ++non_cu_it )
     {
-        if( GetPCBDefaultLayerName( ii ).IsEmpty() )    // Layer not available for export
-            continue;
-
-        fprintf( m_fp, "    (%d %s user)\n", ii, TO_UTF8( GetPCBDefaultLayerName( ii ) ) );
+        fprintf( m_fp, "\t\t(%d %s user)\n",
+                 *non_cu_it, LSET::Name( *non_cu_it ).ToStdString().c_str() );
     }
 
-    fprintf( m_fp, "  )\n\n" );
+    fprintf( m_fp, "\t)\n\n" );
 }
 
 
@@ -514,7 +557,7 @@ void GBR_TO_PCB_EXPORTER::writePcbPolygon( const SHAPE_POLY_SET& aPolys, int aLa
     // (because it comes from a gerber file or is built from a aperture )
     const SHAPE_LINE_CHAIN& poly = aPolys.COutline( 0 );
 
-    fprintf( m_fp, "(gr_poly (pts " );
+    fprintf( m_fp, "\t(gr_poly\n\t\t(pts\n\t\t\t" );
 
     #define MAX_COORD_CNT 4
     int jj = MAX_COORD_CNT;
@@ -529,7 +572,7 @@ void GBR_TO_PCB_EXPORTER::writePcbPolygon( const SHAPE_POLY_SET& aPolys, int aLa
         if( --jj == 0 )
         {
             jj = MAX_COORD_CNT;
-            fprintf( m_fp, "\n" );
+            fprintf( m_fp, "\n\t\t\t" );
         }
 
         fprintf( m_fp, " (xy %s %s)",
@@ -539,23 +582,24 @@ void GBR_TO_PCB_EXPORTER::writePcbPolygon( const SHAPE_POLY_SET& aPolys, int aLa
 
     fprintf( m_fp, ")" );
 
-    if( jj != MAX_COORD_CNT )
-        fprintf( m_fp, "\n" );
-
-    fprintf( m_fp, "(layer %s) (width 0) )\n", TO_UTF8( GetPCBDefaultLayerName( aLayer ) ) );
+    fprintf( m_fp, "\n" );
+    export_stroke_info( 0 );
+    fprintf( m_fp, "\t\t(fill yes) (layer %s)",
+             LSET::Name( PCB_LAYER_ID( aLayer ) ).ToStdString().c_str() );
+    fprintf( m_fp, "\n\t)\n" );
 }
 
 
 void GBR_TO_PCB_EXPORTER::writePcbZoneItem( const GERBER_DRAW_ITEM* aGbrItem, int aLayer )
 {
     SHAPE_POLY_SET polys = aGbrItem->m_ShapeAsPolygon.CloneDropTriangulation();
-    polys.Simplify( SHAPE_POLY_SET::PM_FAST );
+    polys.Simplify();
 
     if( polys.OutlineCount() == 0 )
         return;
 
-    fprintf( m_fp, "(zone (net 0) (net_name \"\") (layer %s) (tstamp 0000000) (hatch edge 0.508)\n",
-             TO_UTF8( GetPCBDefaultLayerName( aLayer ) ) );
+    fprintf( m_fp, "\t(zone (net 0) (net_name \"\") (layer %s) (tstamp 0000000) (hatch edge 0.508)\n",
+             LSET::Name( PCB_LAYER_ID( aLayer ) ).ToStdString().c_str() );
 
     fprintf( m_fp, "  (connect_pads (clearance 0.0))\n" );
 
