@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2017 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 2004-2019 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <wx/debug.h>
 #include <wx/filedlg.h>
 #include <wx/wfstream.h>
 #include <wx/zipstrm.h>
@@ -32,15 +33,17 @@
 #include <gerber_file_image.h>
 #include <gerber_file_image_list.h>
 #include <excellon_image.h>
+#include <lset.h>
 #include <wildcards_and_files_ext.h>
 #include <view/view.h>
 #include <widgets/wx_progress_reporters.h>
 #include "widgets/gerbview_layer_widget.h"
+#include <tool/tool_manager.h>
 
 // HTML Messages used more than one time:
 #define MSG_NO_MORE_LAYER _( "<b>No more available layers</b> in GerbView to load files" )
-#define MSG_NOT_LOADED _( "\n<b>Not loaded:</b> <i>%s</i>" )
-#define MSG_OOM _( "\n<b>Memory was exhausted reading:</b> <i>%s</i>" )
+#define MSG_NOT_LOADED _( "<b>Not loaded:</b> <i>%s</i>" )
+#define MSG_OOM _( "<b>Memory was exhausted reading:</b> <i>%s</i>" )
 
 
 void GERBVIEW_FRAME::OnGbrFileHistory( wxCommandEvent& event )
@@ -51,7 +54,6 @@ void GERBVIEW_FRAME::OnGbrFileHistory( wxCommandEvent& event )
 
     if( !fn.IsEmpty() )
     {
-        Erase_Current_DrawLayer( false );
         LoadGerberFiles( fn );
     }
 }
@@ -70,7 +72,6 @@ void GERBVIEW_FRAME::OnDrlFileHistory( wxCommandEvent& event )
 
     if( !fn.IsEmpty() )
     {
-        Erase_Current_DrawLayer( false );
         LoadExcellonFiles( fn );
     }
 }
@@ -89,7 +90,6 @@ void GERBVIEW_FRAME::OnZipFileHistory( wxCommandEvent& event )
 
     if( !filename.IsEmpty() )
     {
-        Erase_Current_DrawLayer( false );
         LoadZipArchiveFile( filename );
     }
 }
@@ -116,48 +116,17 @@ void GERBVIEW_FRAME::OnClearJobFileHistory( wxCommandEvent& aEvent )
 }
 
 
-bool GERBVIEW_FRAME::LoadGerberFiles( const wxString& aFullFileName )
+bool GERBVIEW_FRAME::LoadFileOrShowDialog( const wxString& aFileName,
+                                           const wxString& dialogFiletypes,
+                                           const wxString& dialogTitle, const int filetype )
 {
     static int lastGerberFileWildcard = 0;
-    wxString   filetypes;
     wxArrayString filenamesList;
-    wxFileName filename = aFullFileName;
+    wxFileName    filename = aFileName;
     wxString currentPath;
 
     if( !filename.IsOk() )
     {
-        /* Standard gerber filetypes
-         * (See http://en.wikipedia.org/wiki/Gerber_File)
-         * The .gbr (.pho in legacy files) extension is the default used in Pcbnew; however
-         * there are a lot of other extensions used for gerber files.  Because the first letter
-         * is usually g, we accept g* as extension.
-         * (Mainly internal copper layers do not have specific extension, and filenames are like
-         * *.g1, *.g2 *.gb1 ...)
-         * Now (2014) Ucamco (the company which manages the Gerber format) encourages use of .gbr
-         * only and the Gerber X2 file format.
-         */
-        filetypes = _( "Gerber files (.g* .lgr .pho)" );
-        filetypes << wxT("|");
-        filetypes += wxT("*.g*;*.G*;*.pho;*.PHO" );
-        filetypes << wxT("|");
-
-        /* Special gerber filetypes */
-        filetypes += _( "Top layer" ) + AddFileExtListToFilter( { "GTL" } ) + wxT( "|" );
-        filetypes += _( "Bottom layer" ) + AddFileExtListToFilter( { "GBL" } ) + wxT( "|" );
-        filetypes += _( "Bottom solder resist" ) + AddFileExtListToFilter( { "GBS" } ) + wxT( "|" );
-        filetypes += _( "Top solder resist" ) + AddFileExtListToFilter( { "GTS" } ) + wxT( "|" );
-        filetypes += _( "Bottom overlay" ) + AddFileExtListToFilter( { "GBO" } ) + wxT( "|" );
-        filetypes += _( "Top overlay" ) + AddFileExtListToFilter( { "GTO" } ) + wxT( "|" );
-        filetypes += _( "Bottom paste" ) + AddFileExtListToFilter( { "GBP" } ) + wxT( "|" );
-        filetypes += _( "Top paste" ) + AddFileExtListToFilter( { "GTP" } ) + wxT( "|" );
-        filetypes += _( "Keep-out layer" ) + AddFileExtListToFilter( { "GKO" } ) + wxT( "|" );
-        filetypes += _( "Mechanical layers" ) + AddFileExtListToFilter( { "GM1", "GM2", "GM3", "GM4", "GM5", "GM6", "GM7", "GM8", "GM9" } ) + wxT( "|" );
-        filetypes += _( "Top Pad Master" ) + AddFileExtListToFilter( { "GPT" } ) + wxT( "|" );
-        filetypes += _( "Bottom Pad Master" ) + AddFileExtListToFilter( { "GPB" } ) + wxT( "|" );
-
-        // All filetypes
-        filetypes += AllFilesWildcard();
-
         // Use the current working directory if the file name path does not exist.
         if( filename.DirExists() )
             currentPath = filename.GetPath();
@@ -171,8 +140,7 @@ bool GERBVIEW_FRAME::LoadGerberFiles( const wxString& aFullFileName )
                 currentPath.RemoveLast();
         }
 
-        wxFileDialog dlg( this, _( "Open Gerber File(s)" ), currentPath, filename.GetFullName(),
-                          filetypes,
+        wxFileDialog dlg( this, dialogTitle, currentPath, filename.GetFullName(), dialogFiletypes,
                           wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_MULTIPLE | wxFD_CHANGE_DIR );
         dlg.SetFilterIndex( lastGerberFileWildcard );
 
@@ -185,43 +153,113 @@ bool GERBVIEW_FRAME::LoadGerberFiles( const wxString& aFullFileName )
     }
     else
     {
-        filenamesList.Add( aFullFileName );
-        m_mruPath = currentPath = filename.GetPath();
+        filenamesList.Add( aFileName );
+        currentPath = filename.GetPath();
+        m_mruPath = currentPath;
     }
 
-    Erase_Current_DrawLayer( false );
-
     // Set the busy cursor
     wxBusyCursor wait;
 
     bool isFirstFile = GetImagesList()->GetLoadedImageCount() == 0;
 
-    bool success = LoadListOfGerberAndDrillFiles( currentPath, filenamesList );
+    std::vector<int> fileTypesVec( filenamesList.Count(), filetype );
+    bool success = LoadListOfGerberAndDrillFiles( currentPath, filenamesList, &fileTypesVec );
 
-    // Auto zoom is only applied if there is only one file loaded
+    // Auto zoom / sort is only applied when no other files have been loaded
     if( isFirstFile )
     {
+        int ly = GetActiveLayer();
+
+        SortLayersByFileExtension();
         Zoom_Automatique( false );
+
+        // Ensure the initial active graphic layer is updated after sorting.
+        SetActiveLayer( ly, true );
     }
 
     return success;
 }
 
 
-bool GERBVIEW_FRAME::LoadListOfGerberAndDrillFiles( const wxString& aPath,
-                                            const wxArrayString& aFilenameList,
-                                            const std::vector<int>* aFileType )
+bool GERBVIEW_FRAME::LoadAutodetectedFiles( const wxString& aFileName )
 {
+    // 2 = autodetect files
+    return LoadFileOrShowDialog( aFileName, FILEEXT::AllFilesWildcard(), _( "Open Autodetected File(s)" ),
+                                 2 );
+}
+
+
+bool GERBVIEW_FRAME::LoadGerberFiles( const wxString& aFileName )
+{
+    wxString   filetypes;
+    wxFileName filename = aFileName;
+
+    /* Standard gerber filetypes
+     * (See http://en.wikipedia.org/wiki/Gerber_File)
+     * The .gbr (.pho in legacy files) extension is the default used in Pcbnew; however
+     * there are a lot of other extensions used for gerber files.  Because the first letter
+     * is usually g, we accept g* as extension.
+     * (Mainly internal copper layers do not have specific extension, and filenames are like
+     * *.g1, *.g2 *.gb1 ...)
+     * Now (2014) Ucamco (the company which manages the Gerber format) encourages use of .gbr
+     * only and the Gerber X2 file format.
+     */
+    filetypes = _( "Gerber files" ) + AddFileExtListToFilter( { "g*", "pho" } ) + wxT( "|" );
+
+    /* Special gerber filetypes */
+    filetypes += _( "Top layer" ) + AddFileExtListToFilter( { "gtl" } ) + wxT( "|" );
+    filetypes += _( "Bottom layer" ) + AddFileExtListToFilter( { "gbl" } ) + wxT( "|" );
+    filetypes += _( "Bottom solder resist" ) + AddFileExtListToFilter( { "gbs" } ) + wxT( "|" );
+    filetypes += _( "Top solder resist" ) + AddFileExtListToFilter( { "gts" } ) + wxT( "|" );
+    filetypes += _( "Bottom overlay" ) + AddFileExtListToFilter( { "gbo" } ) + wxT( "|" );
+    filetypes += _( "Top overlay" ) + AddFileExtListToFilter( { "gto" } ) + wxT( "|" );
+    filetypes += _( "Bottom paste" ) + AddFileExtListToFilter( { "gbp" } ) + wxT( "|" );
+    filetypes += _( "Top paste" ) + AddFileExtListToFilter( { "gtp" } ) + wxT( "|" );
+    filetypes += _( "Keep-out layer" ) + AddFileExtListToFilter( { "gko" } ) + wxT( "|" );
+    filetypes += _( "Mechanical layers" )
+                 + AddFileExtListToFilter(
+                         { "gm1", "gm2", "gm3", "gm4", "gm5", "gm6", "gm7", "gm8", "gm9" } )
+                 + wxT( "|" );
+    filetypes += _( "Top Pad Master" ) + AddFileExtListToFilter( { "gpt" } ) + wxT( "|" );
+    filetypes += _( "Bottom Pad Master" ) + AddFileExtListToFilter( { "gpb" } ) + wxT( "|" );
+
+    // All filetypes
+    filetypes += FILEEXT::AllFilesWildcard();
+
+    // 0 = gerber files
+    return LoadFileOrShowDialog( aFileName, filetypes, _( "Open Gerber File(s)" ), 0 );
+}
+
+
+bool GERBVIEW_FRAME::LoadExcellonFiles( const wxString& aFileName )
+{
+    wxString filetypes = FILEEXT::DrillFileWildcard();
+    filetypes << wxT( "|" );
+    filetypes += FILEEXT::AllFilesWildcard();
+
+    // 1 = drill files
+    return LoadFileOrShowDialog( aFileName, filetypes, _( "Open NC (Excellon) Drill File(s)" ), 1 );
+}
+
+
+bool GERBVIEW_FRAME::LoadListOfGerberAndDrillFiles( const wxString&      aPath,
+                                                    const wxArrayString& aFilenameList,
+                                                    std::vector<int>*    aFileType )
+{
+    wxCHECK_MSG( aFilenameList.Count() == aFileType->size(), false,
+                 "Mismatch in file names and file types count" );
+
     wxFileName filename;
 
     // Read gerber files: each file is loaded on a new GerbView layer
     bool success = true;
     int layer = GetActiveLayer();
+    int  firstLoadedLayer = NO_AVAILABLE_LAYERS;
     LSET visibility = GetVisibleLayers();
 
     // Manage errors when loading files
-    wxString msg;
-    WX_STRING_REPORTER reporter( &msg );
+    WX_STRING_REPORTER reporter;
 
     // Create progress dialog (only used if more than 1 file to load
     std::unique_ptr<WX_PROGRESS_REPORTER> progress = nullptr;
@@ -238,19 +276,31 @@ bool GERBVIEW_FRAME::LoadListOfGerberAndDrillFiles( const wxString& aPath,
         if( !filename.FileExists() )
         {
             wxString warning;
-            warning << "<b>" << _( "File not found:" ) << "</b><br>"
-                    << filename.GetFullPath() << "<br>";
+            warning << wxT( "<b>" ) << _( "File not found:" ) << wxT( "</b><br>" )
+                    << filename.GetFullPath() << wxT( "<br>" );
             reporter.Report( warning, RPT_SEVERITY_WARNING );
             success = false;
             continue;
         }
 
+        if( filename.GetExt() == FILEEXT::GerberJobFileExtension.c_str() )
+        {
+            //We cannot read a gerber job file as a gerber plot file: skip it
+            wxString txt;
+            txt.Printf( _( "<b>A gerber job file cannot be loaded as a plot file</b> "
+                           "<i>%s</i>" ),
+                        filename.GetFullName() );
+            success = false;
+            reporter.Report( txt, RPT_SEVERITY_ERROR );
+            continue;
+        }
+
+
         m_lastFileName = filename.GetFullPath();
 
         if( !progress && ( aFilenameList.GetCount() > 1 ) )
         {
-            progress = std::make_unique<WX_PROGRESS_REPORTER>( this,
-                                                               _( "Loading Gerber files..." ), 1,
+            progress = std::make_unique<WX_PROGRESS_REPORTER>( this, _( "Loading files..." ), 1,
                                                                false );
             progress->SetMaxProgress( aFilenameList.GetCount() - 1 );
             progress->Report( wxString::Format( _("Loading %u/%zu %s..." ),
@@ -267,57 +317,72 @@ bool GERBVIEW_FRAME::LoadListOfGerberAndDrillFiles( const wxString& aPath,
             progress->KeepRefreshing();
         }
 
-        SetActiveLayer( layer, false );
 
+        // Make sure we have a layer available to load into
+        layer = getNextAvailableLayer();
+
+        if( layer == NO_AVAILABLE_LAYERS )
+        {
+            success = false;
+            reporter.Report( MSG_NO_MORE_LAYER, RPT_SEVERITY_ERROR );
+
+            // Report the name of not loaded files:
+            while( ii < aFilenameList.GetCount() )
+            {
+                filename = aFilenameList[ii++];
+                wxString txt = wxString::Format( MSG_NOT_LOADED, filename.GetFullName() );
+                reporter.Report( txt, RPT_SEVERITY_ERROR );
+            }
+            break;
+        }
+
+        SetActiveLayer( layer, false );
         visibility[ layer ] = true;
 
         try
         {
-            if( aFileType && ( *aFileType )[ii] == 1 )
+            // 2 = Autodetect
+            if( ( *aFileType )[ii] == 2 )
             {
-                LoadExcellonFiles( filename.GetFullPath() );
-                layer = GetActiveLayer(); // Loading NC drill file changes the active layer
+                if( EXCELLON_IMAGE::TestFileIsExcellon( filename.GetFullPath() ) )
+                    ( *aFileType )[ii] = 1;
+                else if( GERBER_FILE_IMAGE::TestFileIsRS274( filename.GetFullPath() ) )
+                    ( *aFileType )[ii] = 0;
             }
-            else
+
+            switch( ( *aFileType )[ii] )
             {
-                if( filename.GetExt() == GerberJobFileExtension.c_str() )
+            case 0:
+
+                if( Read_GERBER_File( filename.GetFullPath() ) )
                 {
-                    //We cannot read a gerber job file as a gerber plot file: skip it
-                    wxString txt;
-                    txt.Printf( _( "<b>A gerber job file cannot be loaded as a plot file</b> "
-                                   "<i>%s</i>" ),
-                                filename.GetFullName() );
-                    success = false;
-                    reporter.Report( txt, RPT_SEVERITY_ERROR );
-                }
-                else if( Read_GERBER_File( filename.GetFullPath() ) )
-                {
-                    UpdateFileHistory( m_lastFileName );
+                    UpdateFileHistory( filename.GetFullPath() );
 
-                    GetCanvas()->GetView()->SetLayerHasNegatives(
-                            GERBER_DRAW_LAYER( layer ), GetGbrImage( layer )->HasNegativeItems() );
-
-                    layer = getNextAvailableLayer( layer );
-
-                    if( layer == NO_AVAILABLE_LAYERS && ii < aFilenameList.GetCount() - 1 )
+                    if( firstLoadedLayer == NO_AVAILABLE_LAYERS )
                     {
-                        success = false;
-                        reporter.Report( MSG_NO_MORE_LAYER, RPT_SEVERITY_ERROR );
-
-                        // Report the name of not loaded files:
-                        ii += 1;
-                        while( ii < aFilenameList.GetCount() )
-                        {
-                            filename = aFilenameList[ii++];
-                            wxString txt =
-                                    wxString::Format( MSG_NOT_LOADED, filename.GetFullName() );
-                            reporter.Report( txt, RPT_SEVERITY_ERROR );
-                        }
-                        break;
+                        firstLoadedLayer = layer;
                     }
-
-                    SetActiveLayer( layer, false );
                 }
+
+                break;
+
+            case 1:
+
+                if( Read_EXCELLON_File( filename.GetFullPath() ) )
+                {
+                    UpdateFileHistory( filename.GetFullPath(), &m_drillFileHistory );
+
+                    // Select the first added layer by default when done loading
+                    if( firstLoadedLayer == NO_AVAILABLE_LAYERS )
+                    {
+                        firstLoadedLayer = layer;
+                    }
+                }
+
+                break;
+            default:
+                wxString txt = wxString::Format( MSG_NOT_LOADED, filename.GetFullName() );
+                reporter.Report( txt, RPT_SEVERITY_ERROR );
             }
         }
         catch( const std::bad_alloc& )
@@ -337,22 +402,18 @@ bool GERBVIEW_FRAME::LoadListOfGerberAndDrillFiles( const wxString& aPath,
         wxSafeYield();  // Allows slice of time to redraw the screen
                         // to refresh widgets, before displaying messages
         HTML_MESSAGE_BOX mbox( this, _( "Errors" ) );
-        mbox.ListSet( msg );
+        mbox.ListSet( reporter.GetMessages() );
         mbox.ShowModal();
     }
 
     SetVisibleLayers( visibility );
 
+    if( firstLoadedLayer != NO_AVAILABLE_LAYERS )
+        SetActiveLayer( firstLoadedLayer, true );
+
     // Synchronize layers tools with actual active layer:
     ReFillLayerWidget();
 
-    // TODO: it would be nice if we could set the active layer to one of the
-    // ones that was just loaded, but to maintain the previous user experience
-    // we need to set it to a blank layer in case they load another file.
-    // We can't start with the next available layer when loading files because
-    // some users expect the behavior of overwriting the active layer on load.
-    SetActiveLayer( getNextAvailableLayer( layer ), true );
-
     m_LayersManager->UpdateLayerIcons();
     syncLayerBox( true );
 
@@ -362,117 +423,17 @@ bool GERBVIEW_FRAME::LoadListOfGerberAndDrillFiles( const wxString& aPath,
 }
 
 
-bool GERBVIEW_FRAME::LoadExcellonFiles( const wxString& aFullFileName )
-{
-    wxString   filetypes;
-    wxArrayString filenamesList;
-    wxFileName filename = aFullFileName;
-    wxString currentPath;
-
-    if( !filename.IsOk() )
-    {
-        filetypes = DrillFileWildcard();
-        filetypes << wxT( "|" );
-
-        /* All filetypes */
-        filetypes += AllFilesWildcard();
-
-        /* Use the current working directory if the file name path does not exist. */
-        if( filename.DirExists() )
-            currentPath = filename.GetPath();
-        else
-            currentPath = m_mruPath;
-
-        wxFileDialog dlg( this, _( "Open NC (Excellon) Drill File(s)" ),
-                          currentPath, filename.GetFullName(), filetypes,
-                          wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_MULTIPLE | wxFD_CHANGE_DIR );
-
-        if( dlg.ShowModal() == wxID_CANCEL )
-            return false;
-
-        dlg.GetPaths( filenamesList );
-        currentPath = wxGetCwd();
-        m_mruPath = currentPath;
-    }
-    else
-    {
-        filenamesList.Add( aFullFileName );
-        currentPath = filename.GetPath();
-        m_mruPath = currentPath;
-    }
-
-    // Read Excellon drill files: each file is loaded on a new GerbView layer
-    bool success = true;
-    int layer = GetActiveLayer();
-
-    // Manage errors when loading files
-    wxString msg;
-    WX_STRING_REPORTER reporter( &msg );
-
-    for( unsigned ii = 0; ii < filenamesList.GetCount(); ii++ )
-    {
-        filename = filenamesList[ii];
-
-        if( !filename.IsAbsolute() )
-            filename.SetPath( currentPath );
-
-        m_lastFileName = filename.GetFullPath();
-
-        SetActiveLayer( layer, false );
-
-        if( Read_EXCELLON_File( filename.GetFullPath() ) )
-        {
-            // Update the list of recent drill files.
-            UpdateFileHistory( filename.GetFullPath(), &m_drillFileHistory );
-
-            layer = getNextAvailableLayer( layer );
-
-            if( layer == NO_AVAILABLE_LAYERS && ii < filenamesList.GetCount()-1 )
-            {
-                success = false;
-                reporter.Report( MSG_NO_MORE_LAYER, RPT_SEVERITY_ERROR );
-
-                // Report the name of not loaded files:
-                ii += 1;
-                while( ii < filenamesList.GetCount() )
-                {
-                    filename = filenamesList[ii++];
-                    wxString txt = wxString::Format( MSG_NOT_LOADED, filename.GetFullName() );
-                    reporter.Report( txt, RPT_SEVERITY_ERROR );
-                }
-                break;
-            }
-
-            SetActiveLayer( layer, false );
-        }
-    }
-
-    if( !success )
-    {
-        HTML_MESSAGE_BOX mbox( this, _( "Errors" ) );
-        mbox.ListSet( msg );
-        mbox.ShowModal();
-    }
-
-    Zoom_Automatique( false );
-
-    // Synchronize layers tools with actual active layer:
-    ReFillLayerWidget();
-    SetActiveLayer( GetActiveLayer() );
-    m_LayersManager->UpdateLayerIcons();
-    syncLayerBox();
-
-    return success;
-}
 
 
 bool GERBVIEW_FRAME::unarchiveFiles( const wxString& aFullFileName, REPORTER* aReporter )
 {
+    bool     foundX2Gerbers = false;
     wxString msg;
+    int      firstLoadedLayer = NO_AVAILABLE_LAYERS;
 
     // Extract the path of aFullFileName. We use it to store temporary files
     wxFileName fn( aFullFileName );
-    wxString unzipDir = fn.GetPath();
+    wxString   unzipDir = fn.GetPath();
 
     wxFFileInputStream zipFile( aFullFileName );
 
@@ -499,48 +460,40 @@ bool GERBVIEW_FRAME::unarchiveFiles( const wxString& aFullFileName, REPORTER* aR
     wxString unzipped_tempfile = temp_fn.GetFullPath();
 
 
-    bool success = true;
+    bool             success = true;
     wxZipInputStream zipArchive( zipFile );
-    wxZipEntry* entry;
-    bool reported_no_more_layer = false;
+    wxZipEntry*      entry;
+    bool             reported_no_more_layer = false;
+    KIGFX::VIEW*     view = GetCanvas()->GetView();
 
-    while( ( entry = zipArchive.GetNextEntry() ) )
+    while( ( entry = zipArchive.GetNextEntry() ) != nullptr )
     {
-        wxString fname = entry->GetName();
+        if( entry->IsDir() )
+            continue;
+
+        wxString   fname = entry->GetName();
         wxFileName uzfn = fname;
-        wxString curr_ext = uzfn.GetExt().Lower();
+        wxString   curr_ext = uzfn.GetExt().Lower();
 
         // The archive contains Gerber and/or Excellon drill files. Use the right loader.
         // However it can contain a few other files (reports, pdf files...),
         // which will be skipped.
-        // Gerber files ext is usually "gbr", but can be also another value, starting by "g"
-        // old gerber files ext from kicad is .pho
-        // drill files do not have a well defined ext
-        // It is .drl in kicad, but .txt in Altium for instance
-        // Allows only .drl for drill files.
-        if( curr_ext[0] != 'g' && curr_ext != "pho" && curr_ext != "drl" )
-        {
-            if( aReporter )
-            {
-                msg.Printf( _( "Skipped file '%s' (unknown type).\n" ), entry->GetName() );
-                aReporter->Report( msg, RPT_SEVERITY_WARNING );
-            }
-
-            continue;
-        }
-
-        if( curr_ext == GerberJobFileExtension.c_str() )
+        if( curr_ext == FILEEXT::GerberJobFileExtension.c_str() )
         {
             //We cannot read a gerber job file as a gerber plot file: skip it
             if( aReporter )
             {
-                msg.Printf( _( "Skipped file '%s' (gerber job file).\n" ), entry->GetName() );
+                msg.Printf( _( "Skipped file '%s' (gerber job file)." ), entry->GetName() );
                 aReporter->Report( msg, RPT_SEVERITY_WARNING );
             }
 
             continue;
         }
 
+        wxString               matchedExt;
+        enum GERBER_ORDER_ENUM order;
+        GERBER_FILE_IMAGE_LIST::GetGerberLayerFromFilename( fname, order, matchedExt );
+
         int layer = GetActiveLayer();
 
         if( layer == NO_AVAILABLE_LAYERS )
@@ -575,7 +528,7 @@ bool GERBVIEW_FRAME::unarchiveFiles( const wxString& aFullFileName, REPORTER* aR
 
                 if( aReporter )
                 {
-                    msg.Printf( _( "<b>Unable to create temporary file '%s'.</b>\n"),
+                    msg.Printf( _( "<b>Unable to create temporary file '%s'.</b>" ),
                                 unzipped_tempfile );
                     aReporter->Report( msg, RPT_SEVERITY_ERROR );
                 }
@@ -584,18 +537,48 @@ bool GERBVIEW_FRAME::unarchiveFiles( const wxString& aFullFileName, REPORTER* aR
 
         bool read_ok = true;
 
-        if( curr_ext[0] == 'g' || curr_ext == "pho" )
+        // Try to parse files if we can't tell from file extension
+        if( order == GERBER_ORDER_ENUM::GERBER_LAYER_UNKNOWN )
+        {
+            if( EXCELLON_IMAGE::TestFileIsExcellon( unzipped_tempfile ) )
+            {
+                order = GERBER_ORDER_ENUM::GERBER_DRILL;
+            }
+            else if( GERBER_FILE_IMAGE::TestFileIsRS274( unzipped_tempfile ) )
+            {
+                // If we have no way to know what layer it is, just guess
+                order = GERBER_ORDER_ENUM::GERBER_TOP_COPPER;
+            }
+            else
+            {
+                if( aReporter )
+                {
+                    msg.Printf( _( "Skipped file '%s' (unknown type)." ), entry->GetName() );
+                    aReporter->Report( msg, RPT_SEVERITY_WARNING );
+                }
+            }
+        }
+
+        if( order == GERBER_ORDER_ENUM::GERBER_DRILL )
+        {
+            read_ok = Read_EXCELLON_File( unzipped_tempfile );
+        }
+        else if( order != GERBER_ORDER_ENUM::GERBER_LAYER_UNKNOWN )
         {
             // Read gerber files: each file is loaded on a new GerbView layer
             read_ok = Read_GERBER_File( unzipped_tempfile );
 
             if( read_ok )
-                GetCanvas()->GetView()->SetLayerHasNegatives(
-                        GERBER_DRAW_LAYER( layer ), GetGbrImage( layer )->HasNegativeItems() );
+            {
+                view->SetLayerHasNegatives( GERBER_DRAW_LAYER( layer ),
+                                            GetGbrImage( layer )->HasNegativeItems() );
+            }
         }
-        else // if( curr_ext == "drl" )
+
+        // Select the first added layer by default when done loading
+        if( read_ok && firstLoadedLayer == NO_AVAILABLE_LAYERS )
         {
-            read_ok = Read_EXCELLON_File( unzipped_tempfile );
+            firstLoadedLayer = layer;
         }
 
         delete entry;
@@ -609,7 +592,7 @@ bool GERBVIEW_FRAME::unarchiveFiles( const wxString& aFullFileName, REPORTER* aR
 
             if( aReporter )
             {
-                msg.Printf( _("<b>unzipped file %s read error</b>\n"), unzipped_tempfile );
+                msg.Printf( _( "<b>unzipped file %s read error</b>" ), unzipped_tempfile );
                 aReporter->Report( msg, RPT_SEVERITY_ERROR );
             }
         }
@@ -618,13 +601,26 @@ bool GERBVIEW_FRAME::unarchiveFiles( const wxString& aFullFileName, REPORTER* aR
             GERBER_FILE_IMAGE* gerber_image = GetGbrImage( layer );
 
             if( gerber_image )
+            {
                 gerber_image->m_FileName = fname;
+                if( gerber_image->m_IsX2_file )
+                    foundX2Gerbers = true;
+            }
 
-            layer = getNextAvailableLayer( layer );
+            layer = getNextAvailableLayer();
             SetActiveLayer( layer, false );
         }
     }
 
+    if( foundX2Gerbers )
+        SortLayersByX2Attributes();
+    else
+        SortLayersByFileExtension();
+
+    // Select the first layer loaded so we don't show another layer on top after
+    if( firstLoadedLayer != NO_AVAILABLE_LAYERS )
+        SetActiveLayer( firstLoadedLayer, true );
+
     return success;
 }
 
@@ -645,7 +641,8 @@ bool GERBVIEW_FRAME::LoadZipArchiveFile( const wxString& aFullFileName )
             currentPath = m_mruPath;
 
         wxFileDialog dlg( this, _( "Open Zip File" ), currentPath, filename.GetFullName(),
-                          ZipFileWildcard(), wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_CHANGE_DIR );
+                          FILEEXT::ZipFileWildcard(),
+                          wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_CHANGE_DIR );
 
         if( dlg.ShowModal() == wxID_CANCEL )
             return false;
@@ -660,8 +657,7 @@ bool GERBVIEW_FRAME::LoadZipArchiveFile( const wxString& aFullFileName )
         m_mruPath = currentPath;
     }
 
-    wxString msg;
-    WX_STRING_REPORTER reporter( &msg );
+    WX_STRING_REPORTER reporter;
 
     if( filename.IsOk() )
         unarchiveFiles( filename.GetFullPath(), &reporter );
@@ -674,14 +670,38 @@ bool GERBVIEW_FRAME::LoadZipArchiveFile( const wxString& aFullFileName )
     m_LayersManager->UpdateLayerIcons();
     syncLayerBox();
 
-    if( !msg.IsEmpty() )
+    if( reporter.HasMessage() )
     {
         wxSafeYield();  // Allows slice of time to redraw the screen
                         // to refresh widgets, before displaying messages
         HTML_MESSAGE_BOX mbox( this, _( "Messages" ) );
-        mbox.ListSet( msg );
+        mbox.ListSet( reporter.GetMessages() );
         mbox.ShowModal();
     }
 
     return true;
 }
+
+void GERBVIEW_FRAME::DoWithAcceptedFiles()
+{
+    wxString gerbFn; // param to be sent with action event.
+
+    for( const wxFileName& file : m_AcceptedFiles )
+    {
+        if( file.GetExt() == FILEEXT::ArchiveFileExtension )
+        {
+            wxString fn = file.GetFullPath();
+            // Open zip archive in editor
+            m_toolManager->RunAction<wxString*>( *m_acceptedExts.at( FILEEXT::ArchiveFileExtension ), &fn );
+        }
+        else
+        {
+            // Store FileName in variable to open later
+            gerbFn += '"' + file.GetFullPath() + '"';
+        }
+    }
+
+    // Open files in editor
+    if( !gerbFn.IsEmpty() )
+        m_toolManager->RunAction<wxString*>( *m_acceptedExts.at( FILEEXT::GerberFileExtension ), &gerbFn );
+}
