@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2010-2019 Jean-Pierre Charras  jp.charras at wanadoo.fr
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,12 +31,9 @@
 #include <dcode.h>
 #include <gerber_draw_item.h>
 #include <am_primitive.h>
+#include <aperture_macro.h>
 #include <gbr_netlist_metadata.h>
 
-// An useful macro used when reading gerber files;
-#define IsNumber( x ) ( ( ( (x) >= '0' ) && ( (x) <='9' ) )   \
-                       || ( (x) == '-' ) || ( (x) == '+' )  || ( (x) == '.' ) )
-
 typedef std::vector<GERBER_DRAW_ITEM*> GERBER_DRAW_ITEMS;
 
 class GERBVIEW_FRAME;
@@ -102,6 +99,14 @@ public:
                                         // and the actual coordinates calculation must handle this
 };
 
+// Currently, the Gerber file is parsed line by line.
+// This is most of time OK
+// GERBER_BUFZ is the max size of a single line of text from a gerber file.
+// But warning: in rare cases some files can have *very long* lines,
+// so the buffer must be very large.
+// I saw a file using only one line of 1,400,000 chars
+#define GERBER_BUFZ 5000000
+
 /**
  * Hold the image data and parameters for one gerber file and layer parameters.
  *
@@ -118,6 +123,16 @@ public:
         return wxT( "GERBER_FILE_IMAGE" );
     }
 
+    /**
+     * @brief Performs a heuristics-based check of whether the file is an RS274 gerber file.
+     *
+     * Does not invoke the full parser.
+     *
+     * @param aFullFileName aFullFileName is the full filename of the gerber file.
+     * @return True if RS274 file, false otherwise
+     */
+    static bool TestFileIsRS274( const wxString& aFullFileName );
+
     /**
      * Read and load a gerber file.
      *
@@ -202,8 +217,6 @@ public:
         m_messagesList.Add( aMessage );
     }
 
-    void InitToolTable();
-
     /**
      * Return the current coordinate type pointed to by XnnYnn Text (XnnnnYmmmm).
      *
@@ -212,7 +225,7 @@ public:
      * it forces truncation of a digit string to a maximum length because the exact coordinate
      * format is not always known.
      */
-    wxPoint ReadXYCoord( char*& aText, bool aExcellonMode = false );
+    VECTOR2I ReadXYCoord( char*& aText, bool aExcellonMode = false );
 
     /**
      * Return the current coordinate type pointed to by InnJnn Text (InnnnJmmmm)
@@ -220,11 +233,14 @@ public:
      * These coordinates are relative, so if coordinate is absent, its value
      * defaults to 0
      */
-    wxPoint ReadIJCoord( char*& Text );
+    VECTOR2I ReadIJCoord( char*& Text );
 
-    // functions to read G commands or D commands:
-    int GCodeNumber( char*& Text );
-    int DCodeNumber( char*& Text );
+    /**
+     * Reads the next number and returns the value
+     * @param aText Pointer to the input string vector
+     * @return
+     */
+    int CodeNumber( char*& aText );
 
     /**
      * Return a pointer to the D_CODE within this GERBER for the given \a aDCODE.
@@ -272,6 +288,15 @@ public:
      */
     void DisplayImageInfo( GERBVIEW_FRAME* aMainFrame );
 
+    /**
+     * Set the offset and rotation to draw a file image
+     * Does not change any coordinate od draw items
+     * @param aOffsetMM is the draw offset in millimeters
+     * @param aRotation is the draw roation
+     * draw transform order is rotation and after offset
+     */
+    void SetDrawOffetAndRotation( VECTOR2D aOffsetMM, EDA_ANGLE aRotation );
+
     /**
      * Called when a %TD command is found the Gerber file
      *
@@ -286,7 +311,8 @@ public:
     void RemoveAttribute( X2_ATTRIBUTE& aAttribute );
 
     ///< @copydoc EDA_ITEM::Visit()
-    SEARCH_RESULT Visit( INSPECTOR inspector, void* testData, const KICAD_T scanTypes[] ) override;
+    INSPECT_RESULT Visit( INSPECTOR inspector, void* testData,
+                          const std::vector<KICAD_T>& aScanTypes ) override;
 
 #if defined(DEBUG)
 
@@ -343,78 +369,58 @@ private:
      * @param gerber_file Which file to read from for continuation.
      * @return true if a macro was read in successfully, else false.
      */
-    bool ReadApertureMacro( char *aBuff, unsigned int aBuffSize,
-                            char* & text, FILE * gerber_file );
+    bool ReadApertureMacro( char *aBuff, unsigned int aBuffSize, char*& text, FILE* gerber_file );
 
     // functions to execute G commands or D basic commands:
     bool Execute_G_Command( char*& text, int G_command );
     bool Execute_DCODE_Command( char*& text, int D_command );
 
 public:
-    bool               m_InUse;                          // true if this image is currently in use
-                                                         // (a file is loaded in it)
+    bool               m_InUse;                ///< true if this image is currently in use (a file
+                                               ///<   is loaded in it)
+                                               ///< false if it must be not drawn
+    COLOR4D            m_PositiveDrawColor;    ///< The color used to draw positive items
+    wxString           m_FileName;             ///< Full File Name for this layer
+    wxString           m_ImageName;            ///< Image name, from IN <name>* command
 
-    ///< True if the draw layer is visible and must be drawn.
-    bool               m_IsVisible;
-                                                         // false if it must be not drawn
-    COLOR4D            m_PositiveDrawColor;              // The color used to draw positive items
-    wxString           m_FileName;                       // Full File Name for this layer
-    wxString           m_ImageName;                      // Image name, from IN <name>* command
+    bool               m_IsX2_file;            ///< True if a X2 gerber attribute was found in file
+    X2_ATTRIBUTE_FILEFUNCTION* m_FileFunction; ///< file function parameters, found in a %TF
+                                               ///<   command or a G04
+    wxString           m_MD5_value;            ///< MD5 value found in a %TF.MD5 command
+    wxString           m_PartString;           ///< string found in a %TF.Part command
+    int                m_GraphicLayer;         ///< Graphic layer Number
+    bool               m_ImageNegative;        ///< true = Negative image
 
-    ///< True if a X2 gerber attribute was found in file.
-    bool               m_IsX2_file;
-    X2_ATTRIBUTE_FILEFUNCTION* m_FileFunction;           // file function parameters, found in a
-                                                         //  %TF command or a G04
-    wxString           m_MD5_value;                      // MD5 value found in a %TF.MD5 command
-    wxString           m_PartString;                     // string found in a %TF.Part command
-    int                m_GraphicLayer;                   // Graphic layer Number
-    bool               m_ImageNegative;                  // true = Negative image
+    bool               m_ImageJustifyXCenter;  ///< Image Justify Center on X axis (default = false)
+    bool               m_ImageJustifyYCenter;  ///< Image Justify Center on Y axis (default = false)
+    VECTOR2I           m_ImageJustifyOffset;   ///< Image Justify Offset on XY axis (default = 0,0)
 
-    ///< Image Justify Center on X axis (default = false).
-    bool               m_ImageJustifyXCenter;
+    bool               m_GerbMetric;           ///< false = Inches, true = metric
+    bool               m_Relative;             ///< false = absolute Coord, true = relative Coord.
+    bool               m_NoTrailingZeros;      ///< true: remove tailing zeros.
+    VECTOR2I           m_ImageOffset;          ///< Coord Offset, from IO command
+    wxSize             m_FmtScale;             ///< Fmt 2.3: m_FmtScale = 3, fmt 3.4: m_FmtScale = 4
+    wxSize             m_FmtLen;               ///< Nb chars per coord. ex fmt 2.3, m_FmtLen = 5
 
-    ///< Image Justify Center on Y axis (default = false).
-    bool               m_ImageJustifyYCenter;
+    int                m_ImageRotation;        ///< Image rotation (0, 90, 180, 270 only) in degrees
+    double             m_LocalRotation;        ///< Local rotation added to m_ImageRotation
+                                               ///< @note This value is stored in 0.1 degrees
 
-    ///< Image Justify Offset on XY axis (default = 0,0).
-    wxPoint            m_ImageJustifyOffset;
-    bool               m_GerbMetric;                     // false = Inches, true = metric
-
-    ///< false = absolute Coord, true = relative Coord.
-    bool               m_Relative;
-    bool               m_NoTrailingZeros;                // true: remove tailing zeros.
-    wxPoint            m_ImageOffset;                    // Coord Offset, from IO command
-
-    ///< Fmt 2.3: m_FmtScale = 3, fmt 3.4: m_FmtScale = 4.
-    wxSize             m_FmtScale;
-
-    ///< Nb chars per coord. ex fmt 2.3, m_FmtLen = 5.
-    wxSize             m_FmtLen;
-
-    ///< Image rotation (0, 90, 180, 270 only) in degrees.
-    int                m_ImageRotation;
-
-    ///< Local rotation in degrees added to m_ImageRotation.
-    ///< @note This value is stored in 0.1 degrees.
-    double             m_LocalRotation;
-    wxPoint            m_Offset;                         // Coord Offset, from OF command
-    wxRealPoint        m_Scale;                          // scale (X and Y) of layer.
-    bool               m_SwapAxis;                       // false (default) if A = X and B = Y
-                                                         // true if A = Y, B = X
-    bool               m_MirrorA;                        // true: mirror / axis A (X)
-    bool               m_MirrorB;                        // true: mirror / axis B (Y)
-    int                m_Iterpolation;                   // Linear, 90 arc, Circ.
-    int                m_Current_Tool;                   // Current Tool (Dcode) number selected
-
-    ///< Current or last pen state (0..9, set by Dn option with n < 10.
-    int                m_Last_Pen_Command;
-    int                m_CommandState;                   // state of gerber analysis command.
-
-    ///< Line number of the gerber file while reading.
-    int                m_LineNum;
-    wxPoint            m_CurrentPos;                     // current specified coord for plot
-    wxPoint            m_PreviousPos;                    // old current specified coord for plot
-    wxPoint            m_IJPos;                          // IJ coord (for arcs & circles )
+    VECTOR2I           m_Offset;               ///< Coord Offset, from OF command
+    VECTOR2I           m_Scale;                ///< scale (X and Y) of layer.
+    bool               m_SwapAxis;             ///< false if A = X and B = Y (default); true if
+                                               ///<   A = Y, B = X
+    bool               m_MirrorA;              ///< true: mirror / axis A (X)
+    bool               m_MirrorB;              ///< true: mirror / axis B (Y)
+    int                m_Iterpolation;         ///< Linear, 90 arc, Circ.
+    int                m_Current_Tool;         ///< Current Tool (Dcode) number selected
+    int                m_Last_Pen_Command;     ///< Current or last pen state (0..9, set by Dn
+                                               ///<   option with n < 10
+    int                m_CommandState;         ///< state of gerber analysis command
+    int                m_LineNum;              ///< Line number of the gerber file while reading.
+    VECTOR2I           m_CurrentPos;           ///< current specified coord for plot
+    VECTOR2I           m_PreviousPos;          ///< old current specified coord for plot
+    VECTOR2I           m_IJPos;                ///< IJ coord (for arcs & circles )
 
     ///< True if a IJ coord was read (for arcs & circles ).
     bool               m_LastCoordIsIJPos;
@@ -466,6 +472,14 @@ public:
     GERBER_LAYER       m_GBRLayerParams;                 // hold params for the current gerber layer
     GERBER_DRAW_ITEMS  m_drawings;                       // linked list of Gerber Items to draw
 
+    ///< Parameters used only to draw (display) items on this layer.
+    ///< Do not change actual coordinates/orientation
+    VECTOR2I           m_DisplayOffset;
+    EDA_ANGLE          m_DisplayRotation;
+
+    // A large buffer to store one line
+    static char m_LineBuffer[GERBER_BUFZ+1];
+
 private:
     wxArrayString      m_messagesList;         // A list of messages created when reading a file
 
