@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2009-2019 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,13 +29,14 @@
 
 #include <netclass.h>
 #include <project/net_settings.h>
-#include <config_params.h>
 #include <board_stackup_manager/board_stackup.h>
 #include <drc/drc_engine.h>
+#include <lset.h>
 #include <settings/nested_settings.h>
 #include <widgets/ui_common.h>
 #include <zone_settings.h>
 #include <teardrop/teardrop_parameters.h>
+#include <router/pns_meander.h>
 
 
 #define DEFAULT_SILK_LINE_WIDTH       0.1
@@ -76,6 +77,9 @@
 #define DEFAULT_CUSTOMDPAIRGAP        0.18
 #define DEFAULT_CUSTOMDPAIRVIAGAP     0.18
 
+#define DEFAULT_MEANDER_SPACING       0.6
+#define DEFAULT_DP_MEANDER_SPACING    1.0
+
 #define DEFAULT_MINCLEARANCE          0.0     // overall min clearance
 #define DEFAULT_MINCONNECTION         0.0     // overall min connection width
 #define DEFAULT_TRACKMINWIDTH         0.0     // track width min value
@@ -90,12 +94,22 @@
 #define LEGACY_COPPEREDGECLEARANCE   -0.01    // A flag to indicate the legacy method (based
                                               // on edge cut line thicknesses) should be used.
 #define DEFAULT_SILKCLEARANCE         0.0
+#define DEFAULT_MINGROOVEWIDTH        0.0
 
 #define DEFAULT_MINRESOLVEDSPOKES     2       // Fewer resolved spokes indicates a starved thermal
 
-#define MINIMUM_ERROR_SIZE_MM         0.001
-#define MAXIMUM_ERROR_SIZE_MM         0.1
+#define MINIMUM_ERROR_SIZE_MM         0.001   // For arc approximation
+#define MAXIMUM_ERROR_SIZE_MM         0.1     // For arc approximation
 
+// Min/max values used in dialogs to validate settings
+#define MINIMUM_LINE_WIDTH_MM         0.005   // minimal line width entered in a dialog
+#define MAXIMUM_LINE_WIDTH_MM         100.0   // max line width entered in a dialog
+
+// Default pad properties
+#define DEFAULT_PAD_WIDTH_MM 2.54         // master pad width
+#define DEFAULT_PAD_HEIGTH_MM 1.27        // master pad height
+#define DEFAULT_PAD_DRILL_DIAMETER_MM 0.8 // master pad drill diameter for PTH
+#define DEFAULT_PAD_RR_RADIUS_RATIO 0.15  // master pad corner radius ratio
 
 /**
  * Container to handle a stock of specific vias each with unique diameter and drill sizes
@@ -123,6 +137,8 @@ struct VIA_DIMENSION
         return ( m_Diameter == aOther.m_Diameter ) && ( m_Drill == aOther.m_Drill );
     }
 
+    bool operator!=( const VIA_DIMENSION& aOther ) const { return !operator==( aOther ); }
+
     bool operator<( const VIA_DIMENSION& aOther ) const
     {
         if( m_Diameter != aOther.m_Diameter )
@@ -164,6 +180,8 @@ struct DIFF_PAIR_DIMENSION
                 && ( m_ViaGap == aOther.m_ViaGap );
     }
 
+    bool operator!=( const DIFF_PAIR_DIMENSION& aOther ) const { return !operator==( aOther ); }
+
     bool operator<( const DIFF_PAIR_DIMENSION& aOther ) const
     {
         if( m_Width != aOther.m_Width )
@@ -192,16 +210,23 @@ enum
 
 struct TEXT_ITEM_INFO
 {
-    wxString m_Text;
-    bool     m_Visible;
-    int      m_Layer;
+    wxString     m_Text;
+    bool         m_Visible;
+    PCB_LAYER_ID m_Layer;
 
-    TEXT_ITEM_INFO( const wxString& aText, bool aVisible, int aLayer )
+    TEXT_ITEM_INFO( const wxString& aText, bool aVisible, PCB_LAYER_ID aLayer )
     {
         m_Text = aText;
         m_Visible = aVisible;
         m_Layer = aLayer;
     }
+
+    bool operator==( const TEXT_ITEM_INFO& aOther ) const
+    {
+        return m_Text.IsSameAs( aOther.m_Text )
+                && ( m_Visible == aOther.m_Visible )
+                && ( m_Layer == aOther.m_Layer );
+    }
 };
 
 
@@ -212,6 +237,7 @@ enum class VIATYPE : int;
 enum class DIM_UNITS_FORMAT : int;
 enum class DIM_TEXT_POSITION : int;
 enum class DIM_UNITS_MODE : int;
+enum class DIM_PRECISION : int;
 
 class PAD;
 
@@ -225,6 +251,12 @@ public:
 
     virtual ~BOARD_DESIGN_SETTINGS();
 
+    bool operator==( const BOARD_DESIGN_SETTINGS& aOther ) const;
+    bool operator!=( const BOARD_DESIGN_SETTINGS& aOther ) const
+    {
+        return !operator==( aOther );
+    }
+
     BOARD_DESIGN_SETTINGS( const BOARD_DESIGN_SETTINGS& aOther);
 
     BOARD_DESIGN_SETTINGS& operator=( const BOARD_DESIGN_SETTINGS& aOther );
@@ -444,7 +476,7 @@ public:
      * Sets custom track width for differential pairs (i.e. not available in netclasses or
      * preset list).
      *
-     * @param aDrill is the new track wdith.
+     * @param aDrill is the new track width.
      */
     inline void SetCustomDiffPairWidth( int aWidth )
     {
@@ -580,6 +612,21 @@ public:
      */
     void SetCopperLayerCount( int aNewLayerCount );
 
+    /**
+     * @return the number of enabled user defined layers.
+     */
+    inline int GetUserDefinedLayerCount() const
+    {
+        return m_userDefinedLayerCount;
+    }
+
+    /**
+     * Set the number of user defined layers to \a aNewLayerCount.
+     *
+     * @param aNewLayerCount The new number of enabled user defined layers.
+     */
+    void SetUserDefinedLayerCount( int aNewLayerCount );
+
     /**
      * The full thickness of the board including copper and masks.
      * @return
@@ -587,7 +634,7 @@ public:
     inline int GetBoardThickness() const { return m_boardThickness; }
     inline void SetBoardThickness( int aThickness ) { m_boardThickness = aThickness; }
 
-    /*
+    /**
      * Return an epsilon which accounts for rounding errors, etc.
      *
      * While currently an advanced cfg, going through this API allows us to easily change
@@ -610,7 +657,7 @@ public:
     /**
      * Return the default text size from the layer class for the given layer.
      */
-    wxSize GetTextSize( PCB_LAYER_ID aLayer ) const;
+    VECTOR2I GetTextSize( PCB_LAYER_ID aLayer ) const;
 
     /**
      * Return the default text thickness from the layer class for the given layer.
@@ -628,6 +675,8 @@ public:
     void            SetGridOrigin( const VECTOR2I& aOrigin ) { m_gridOrigin = aOrigin; }
     const VECTOR2I& GetGridOrigin() { return m_gridOrigin; }
 
+    void SetDefaultMasterPad();
+
 private:
     void initFromOther( const BOARD_DESIGN_SETTINGS& aOther );
 
@@ -639,18 +688,25 @@ public:
     std::vector<VIA_DIMENSION>       m_ViasDimensionsList;
     std::vector<DIFF_PAIR_DIMENSION> m_DiffPairDimensionsList;
 
-    /** The parameters of teardrops for the different teardrop targets (via/pad, track end)
-     *  3 set of parameters always exist: for round shapes, for rect shapes, for track ends
+    /**
+     * The parameters of teardrops for the different teardrop targets (via/pad, track end).
+     *
+     * 3 set of parameters always exist: for round shapes, for rect shapes, for track ends.
      */
     TEARDROP_PARAMETERS_LIST         m_TeardropParamsList;
 
+    PNS::MEANDER_SETTINGS            m_SingleTrackMeanderSettings;
+    PNS::MEANDER_SETTINGS            m_DiffPairMeanderSettings;
+    PNS::MEANDER_SETTINGS            m_SkewMeanderSettings;
+
     VIATYPE    m_CurrentViaType;            ///< (VIA_BLIND_BURIED, VIA_THROUGH, VIA_MICROVIA)
 
     bool       m_UseConnectedTrackWidth;    // use width of existing track when creating a new,
                                             // connected track
     bool       m_TempOverrideTrackWidth;    // use selected track width temporarily even when
                                             // using connected track width
-    int        m_MinClearance;              // overall min clearance
+    int        m_MinClearance;              // overall min
+    int        m_MinGrooveWidth;            // Minimum groove width for creepage checks
     int        m_MinConn;                   // overall min connection width
     int        m_TrackMinWidth;             // overall min track width
     int        m_ViasMinAnnularWidth;       // overall minimum width of the via copper ring
@@ -666,9 +722,10 @@ public:
     int        m_MinSilkTextHeight;         // Min text height for silkscreen layers
     int        m_MinSilkTextThickness;      // Min text thickness for silkscreen layers
 
-    std::shared_ptr<DRC_ENGINE> m_DRCEngine;
-    std::map<int, SEVERITY>     m_DRCSeverities;   // Map from DRCErrorCode to SEVERITY
-    std::set<wxString>          m_DrcExclusions;
+    std::shared_ptr<DRC_ENGINE>  m_DRCEngine;
+    std::map<int, SEVERITY>      m_DRCSeverities;           // Map from DRCErrorCode to SEVERITY
+    std::set<wxString>           m_DrcExclusions;           // Serialized excluded DRC markers
+    std::map<wxString, wxString> m_DrcExclusionComments;    // Map from serialization to comment
 
     // When smoothing the zone's outline there's the question of external fillets (that is, those
     // applied to concave corners).  While it seems safer to never have copper extend outside the
@@ -689,22 +746,27 @@ public:
     double     m_SolderPasteMarginRatio;      // Solder mask margin ratio value of pad size
                                               // The final margin is the sum of these 2 values
     bool       m_AllowSoldermaskBridgesInFPs;
+    bool       m_TentViasFront;               // The default tenting option if not overridden on an
+    bool       m_TentViasBack;                // individual via
 
     std::shared_ptr<NET_SETTINGS> m_NetSettings;
 
     // Variables used in footprint editing (default value in item/footprint creation)
     std::vector<TEXT_ITEM_INFO>   m_DefaultFPTextItems;
 
+    // Map between user layer default names and custom names
+    std::map<std::string, wxString>  m_UserLayerNames;
+
     // Arrays of default values for the various layer classes.
     int        m_LineThickness[ LAYER_CLASS_COUNT ];
-    wxSize     m_TextSize[ LAYER_CLASS_COUNT ];
+    VECTOR2I   m_TextSize[LAYER_CLASS_COUNT];
     int        m_TextThickness[ LAYER_CLASS_COUNT ];
     bool       m_TextItalic[ LAYER_CLASS_COUNT ];
     bool       m_TextUpright[ LAYER_CLASS_COUNT ];
 
     // Default values for dimension objects
     DIM_UNITS_MODE    m_DimensionUnitsMode;
-    int               m_DimensionPrecision; ///< Number of digits after the decimal
+    DIM_PRECISION     m_DimensionPrecision; ///< Number of digits after the decimal
     DIM_UNITS_FORMAT  m_DimensionUnitsFormat;
     bool              m_DimensionSuppressZeroes;
     DIM_TEXT_POSITION m_DimensionTextPosition;
@@ -712,6 +774,10 @@ public:
     int               m_DimensionArrowLength;
     int               m_DimensionExtensionOffset;
 
+    bool              m_StyleFPFields;
+    bool              m_StyleFPText;
+    bool              m_StyleFPShapes;
+
     // Miscellaneous
     std::unique_ptr<PAD> m_Pad_Master; // A dummy pad to store all default parameters
                                        // when importing values or creating a new pad
@@ -745,6 +811,8 @@ private:
 
     int        m_copperLayerCount; ///< Number of copper layers for this design
 
+    int        m_userDefinedLayerCount; ///< Number of user defined layers for this design
+
     LSET       m_enabledLayers;    ///< Bit-mask for layer enabling
 
     int        m_boardThickness;   ///< Board thickness for 3D viewer
@@ -753,13 +821,15 @@ private:
     /// This is also the last used netclass after starting a track.
     wxString   m_currentNetClassName;
 
-    /** the description of layers stackup, for board fabrication
-     * only physical layers are in layers stackup.
-     * It includes not only layers enabled for the board edition, but also dielectric layers
+    /**
+     * The description of layers stackup, for board fabrication only physical layers are in
+     * layers stackup.
+     *
+     * It includes not only layers enabled for the board edition, but also dielectric layers.
      */
     BOARD_STACKUP m_stackup;
 
-    /// The default settings that will be used for new zones
+    /// The default settings that will be used for new zones.
     ZONE_SETTINGS m_defaultZoneSettings;
 };
 
