@@ -3,7 +3,8 @@
  *
  * Copyright (C) 2009-2015 Jean-Pierre Charras, jp.charras wanadoo.fr
  * Copyright (C) 2011 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2023 CERN (www.cern.ch)
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -46,6 +47,7 @@
 #include <undo_redo_container.h>
 #include <units_provider.h>
 #include <origin_transforms.h>
+#include <ui_events.h>
 
 // Option for main frames
 #define KICAD_DEFAULT_DRAWFRAME_STYLE wxDEFAULT_FRAME_STYLE | wxWANTS_CHARS
@@ -65,7 +67,6 @@ class EDA_ITEM;
 class EDA_DRAW_PANEL_GAL;
 class EDA_MSG_PANEL;
 class BASE_SCREEN;
-class PARAM_CFG;
 class PAGE_INFO;
 class PLOTTER;
 class TITLE_BLOCK;
@@ -80,8 +81,11 @@ class FILE_HISTORY;
 class SETTINGS_MANAGER;
 class SEARCH_STACK;
 class APP_SETTINGS_BASE;
+class APPEARANCE_CONTROLS_3D;
 struct WINDOW_SETTINGS;
 struct WINDOW_STATE;
+class ACTION_MENU;
+class TOOL_INTERACTIVE;
 
 #define DEFAULT_MAX_UNDO_ITEMS 0
 #define ABS_MAX_UNDO_ITEMS (INT_MAX / 2)
@@ -89,8 +93,6 @@ struct WINDOW_STATE;
 /// This is the handler functor for the update UI events
 typedef std::function< void( wxUpdateUIEvent& ) > UIUpdateHandler;
 
-wxDECLARE_EVENT( UNITS_CHANGED, wxCommandEvent );
-
 
 /**
  * The base frame for deriving all KiCad main window classes.
@@ -106,7 +108,7 @@ class EDA_BASE_FRAME : public wxFrame, public TOOLS_HOLDER, public KIWAY_HOLDER,
 {
 public:
     /**
-     * Specifies whether we are interacting with the undo or redo stacks
+     * Specify whether we are interacting with the undo or redo stacks.
      */
     enum UNDO_REDO_LIST
     {
@@ -209,15 +211,12 @@ public:
 
     virtual wxString help_name();
 
+    // Event handlers for menu events generated by the macOS application menu
     void OnKicadAbout( wxCommandEvent& event );
-
-    /**
-     * Displays the preferences and settings of all opened editors paged dialog
-     */
     void OnPreferences( wxCommandEvent& event );
 
     /**
-     * Displays the preferences and settings of all opened editors paged dialog, starting with
+     * Display the preferences and settings of all opened editors paged dialog, starting with
      * a particular page
      */
     void ShowPreferences( wxString aStartPage, wxString aStartParentPage );
@@ -285,12 +284,13 @@ public:
     void ShowInfoBarMsg( const wxString& aMsg, bool aShowCloseButton = false );
 
     /**
-     * Returns the settings object used in SaveSettings(), and is overloaded in
+     * Return the settings object used in SaveSettings(), and is overloaded in
      * #KICAD_MANAGER_FRAME.
      */
     virtual APP_SETTINGS_BASE* config() const;
 
     void LoadWindowState( const wxString& aFileName );
+
     /**
      * Load window settings from the given settings object.
      *
@@ -333,7 +333,7 @@ public:
     virtual WINDOW_SETTINGS* GetWindowSettings( APP_SETTINGS_BASE* aCfg );
 
     /**
-     * Load frame state info from a configuration file
+     * Load frame state info from a configuration file.
      */
     virtual void LoadWindowState( const WINDOW_STATE& aState );
 
@@ -354,14 +354,17 @@ public:
     }
 
     /**
-     * Save changes to the project settings to the project (.pro) file.
+     * Save changes to the project local settings.
+     *
+     * These settings are used to save/restore the view state for a specific project, and
+     * should never contain design data.  This method is normally called automatically at
+     * various points in the workflow so that the user's most recent display settings are
+     * automatically persisted.
      *
      * The method is virtual so you can override it to call the suitable save method.
      * The base method does nothing.
-     *
-     * @param aAskForSave true to open a dialog before saving the settings.
      */
-    virtual void SaveProjectSettings() {};
+    virtual void SaveProjectLocalSettings() {};
 
     /**
      * Prompt the user for a hotkey file to read, and read it.
@@ -374,7 +377,7 @@ public:
                                      const wxString& aDefaultShortname );
 
     /**
-     * Fetches the file name from the file history list.
+     * Fetch the file name from the file history list.
      *
      * This removes the selected file, if this file does not exist.  The menu is also updated,
      * if #FILE_HISTORY::UseMenu was called at initialization time.
@@ -389,7 +392,7 @@ public:
                                  FILE_HISTORY* aFileHistory = nullptr );
 
     /**
-     * Removes all files from the file history.
+     * Remove all files from the file history.
      *
      * @param aFileHistory The FILE_HISTORY in use. If null, the main application file
      *                     history is used
@@ -431,19 +434,21 @@ public:
     virtual wxString GetCurrentFileName() const { return wxEmptyString; }
 
     /**
-     * Recreates the menu bar.
+     * Recreate the menu bar.
      *
      * Needed when the language or icons are changed
      */
     void ReCreateMenuBar();
 
+    void SetMenuBar( wxMenuBar *menu_bar ) override;
+
     /**
-     * Adds the standard KiCad help menu to the menubar.
+     * Add the standard KiCad help menu to the menubar.
      */
     void AddStandardHelpMenu( wxMenuBar* aMenuBar );
 
     /**
-     * Checks if \a aFileName can be written.
+     * Check if \a aFileName can be written.
      *
      * The function performs a number of tests on \a aFileName to verify that it can be saved.
      * If \a aFileName defines a path with no file name, them the path is tested for user write
@@ -473,6 +478,8 @@ public:
      */
     virtual void CheckForAutoSaveFile( const wxFileName& aFileName );
 
+    virtual void DeleteAutoSaveFile( const wxFileName& aFileName );
+
     /**
      * Update the status bar information.
      *
@@ -491,7 +498,7 @@ public:
      * Notification event that some of the common (suite-wide) settings have changed.
      * Update menus, toolbars, local variables, etc.
      */
-    void CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVarsChanged ) override;
+    void CommonSettingsChanged( int aFlags ) override;
 
     /**
      * Process light/dark theme change.
@@ -573,6 +580,9 @@ public:
     virtual int GetUndoCommandCount() const { return m_undoList.m_CommandsList.size(); }
     virtual int GetRedoCommandCount() const { return m_redoList.m_CommandsList.size(); }
 
+    virtual wxString GetUndoActionDescription() const;
+    virtual wxString GetRedoActionDescription() const;
+
     int GetMaxUndoItems() const { return m_undoRedoCountMax; }
 
     /**
@@ -592,17 +602,17 @@ public:
      */
     virtual void HandleSystemColorChange();
 
-protected:
-    ///< Default style flags used for wxAUI toolbars.
-    static constexpr int KICAD_AUI_TB_STYLE = wxAUI_TB_DEFAULT_STYLE | wxAUI_TB_PLAIN_BACKGROUND;
-
     /**
-     * @return the string to prepend to a file name for automatic save.
+     * Check if this frame is ready to accept API commands.
+     *
+     * A frame might not accept commands if a long-running process is underway, a dialog is open,
+     * the user is interacting with a tool, etc.
      */
-    static wxString GetAutoSaveFilePrefix()
-    {
-        return wxT( "_autosave-" );
-    }
+    virtual bool CanAcceptApiCommands() { return IsEnabled(); }
+
+protected:
+    /// Default style flags used for wxAUI toolbars.
+    static constexpr int KICAD_AUI_TB_STYLE = wxAUI_TB_DEFAULT_STYLE | wxAUI_TB_PLAIN_BACKGROUND;
 
     virtual void doReCreateMenuBar() {}
 
@@ -654,7 +664,7 @@ protected:
     virtual void setupUIConditions();
 
     /**
-     * Sets the common key-pair for exiting the application (Ctrl-Q) and ties it
+     * Set the common key-pair for exiting the application (Ctrl-Q) and ties it
      * to the wxID_EXIT event id.
      *
      * This is useful in sub-applications to pass the event up to a non-owning window.
@@ -664,15 +674,33 @@ protected:
     void ensureWindowIsOnScreen();
 
     /**
-     * Handles event fired when a file is dropped to the window.
+     * Save any design-related project settings associated with this frame.
+     *
+     * This method should only be called as the result of direct user action, for example from an
+     * explicit "Save Project" command or as a consequence of saving a design document.
+     */
+    virtual void saveProjectSettings() {}
+
+    /**
+     * Handle event fired when a file is dropped to the window.
+     *
      * In this base class, stores the path of files accepted.
      * Calls DoWithAcceptedFiles() to execute actions on files.
      */
     virtual void OnDropFiles( wxDropFilesEvent& aEvent );
 
+    /**
+     * Create a menu list for language choice, and add it as submenu to \a MasterMenu.
+     *
+     * @param aMasterMenu is the main menu.
+     * @param aControlTool is the tool to associate with the menu.
+     */
+    void AddMenuLanguageList( ACTION_MENU* aMasterMenu, TOOL_INTERACTIVE* aControlTool );
+
     /**
      * Execute action on accepted dropped file.
-     * Called in OnDropFiles and should be populated with
+     *
+     * Called in OnDropFiles() and should be populated with
      * the action to execute in inherited classes.
      */
     virtual void            DoWithAcceptedFiles();
@@ -709,9 +737,9 @@ private:
      */
     virtual bool IsModal() const { return false; }
 
-#ifdef _WIN32
+#ifdef __WXMSW__
     /**
-     * Windows specific override of the wxWidgets message handler for a window
+     * Windows specific override of the wxWidgets message handler for a window.
      */
     WXLRESULT MSWWindowProc( WXUINT message, WXWPARAM wParam, WXLPARAM lParam ) override;
 #endif
@@ -727,46 +755,44 @@ private:
     wxPoint         m_normalFramePos;
     wxSize          m_normalFrameSize;
 
-    wxString        m_aboutTitle;           // Name of program displayed in About.
+    wxString                m_aboutTitle;        // Name of program displayed in About.
 
-    wxAuiManager    m_auimgr;
-    wxString        m_perspective;          // wxAuiManager perspective.
+    wxAuiManager            m_auimgr;
+    wxString                m_perspective;       // wxAuiManager perspective.
+    WX_INFOBAR*             m_infoBar;           // Infobar for the frame
+    APPEARANCE_CONTROLS_3D* m_appearancePanel;
+    wxString                m_configName;        // Prefix used to identify some params (frame
+                                                 // size) and to name some config files (legacy
+                                                 // hotkey files)
+    SETTINGS_MANAGER*       m_settingsManager;
 
-    WX_INFOBAR*     m_infoBar;              // Infobar for the frame
+    FILE_HISTORY*           m_fileHistory;       // The frame's recently opened file list
+    bool                    m_supportsAutoSave;
+    bool                    m_autoSavePending;
+    bool                    m_autoSaveRequired;
+    wxTimer*                m_autoSaveTimer;
 
-    wxString        m_configName;           // Prefix used to identify some params (frame size...)
-                                            // and to name some config files (legacy hotkey files)
+    int                     m_undoRedoCountMax;  // undo/Redo command Max depth
 
-    SETTINGS_MANAGER* m_settingsManager;
+    UNDO_REDO_CONTAINER     m_undoList;          // Objects list for the undo command (old data)
+    UNDO_REDO_CONTAINER     m_redoList;          // Objects list for the redo command (old data)
 
-    FILE_HISTORY*   m_fileHistory;          // The frame's recently opened file list
+    wxString                m_mruPath;           // Most recently used path.
 
-    bool            m_supportsAutoSave;
-    bool            m_autoSavePending;
-    bool            m_autoSaveRequired;
-    wxTimer*        m_autoSaveTimer;
+    ORIGIN_TRANSFORMS       m_originTransforms;  // Default display origin transforms object.
 
-    int                 m_undoRedoCountMax; // undo/Redo command Max depth
-
-    UNDO_REDO_CONTAINER m_undoList;         // Objects list for the undo command (old data)
-    UNDO_REDO_CONTAINER m_redoList;         // Objects list for the redo command (old data)
-
-    wxString            m_mruPath;          // Most recently used path.
-
-    ORIGIN_TRANSFORMS   m_originTransforms; // Default display origin transforms object.
-
-    ///< Map containing the UI update handlers registered with wx for each action.
+    /// Map containing the UI update handlers registered with wx for each action.
     std::map<int, UIUpdateHandler> m_uiUpdateMap;
 
-    ///< Set by the close window event handler after frames are asked if they can close.
-    ///< Allows other functions when called to know our state is cleanup.
+    /// Set by the close window event handler after frames are asked if they can close.
+    /// Allows other functions when called to know our state is cleanup.
     bool            m_isClosing;
 
-    ///< Set by #NonUserClose() to indicate that the user did not request the current close.
+    /// Set by #NonUserClose() to indicate that the user did not request the current close.
     bool            m_isNonUserClose;
 
     /**
-     * Associates files extensions with action to execute.
+     * Associate file extensions with action to execute.
      */
     std::map<const wxString, TOOL_ACTION*> m_acceptedExts;
 };
