@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2013 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,34 +32,42 @@
 #include <eda_search_data.h>
 #include <font/glyph.h>
 #include <font/text_attributes.h>
+#include <api/serializable.h>
+
 
 class OUTPUTFORMATTER;
 class SHAPE_COMPOUND;
 class SHAPE_POLY_SET;
 
 
+// These are only here for algorithmic safety, not to tell the user what to do.
+// PL_EDITOR has the least resolution (its internal units are microns), so the min size is chosen
+// to yield 1 in PL_EDITOR.
+// The max size chosen is somewhat arbitrary, but no one has complained yet.
+#define TEXT_MIN_SIZE_MM  0.001    ///< Minimum text size (1 micron).
+#define TEXT_MAX_SIZE_MM  250.0    ///< Maximum text size in mm (~10 inches)
+
+
 namespace KIGFX
 {
     class RENDER_SETTINGS;
     class COLOR4D;
 }
 
+namespace KIFONT
+{
+    class METRICS;
+    }
+
 using KIGFX::RENDER_SETTINGS;
 using KIGFX::COLOR4D;
 
-// part of the kicad_plugin.h family of defines.
-// See kicad_plugin.h for the choice of the value
-// When set when calling  EDA_TEXT::Format, disable writing the "hide" keyword in save file
-#define CTL_OMIT_HIDE               (1 << 6)
-
 
 /**
- * This is the "default-of-the-default" hardcoded text size; individual
- * application define their own default policy starting with this
- * (usually with a user option or project).
+ * This is the "default-of-the-default" hardcoded text size; individual application define their
+ * own default policy starting with this (usually with a user option or project).
  */
 #define DEFAULT_SIZE_TEXT   50     // default text height (in mils, i.e. 1/1000")
-#define DIM_ANCRE_TEXTE     2      // Anchor size for text
 
 
 /**
@@ -68,7 +76,7 @@ using KIGFX::COLOR4D;
  * function names (accessors) that to not collide with function names likely to be seen
  * in the combined derived classes.
  */
-class EDA_TEXT
+class EDA_TEXT : public SERIALIZABLE
 {
 public:
     EDA_TEXT( const EDA_IU_SCALE& aIuScale, const wxString& aText = wxEmptyString );
@@ -79,6 +87,9 @@ public:
 
     EDA_TEXT& operator=( const EDA_TEXT& aItem );
 
+    void Serialize( google::protobuf::Any &aContainer ) const override;
+    bool Deserialize( const google::protobuf::Any &aContainer ) override;
+
     /**
      * Return the string associated with the text object.
      *
@@ -129,10 +140,34 @@ public:
     }
     double GetTextAngleDegrees() const          { return m_attributes.m_Angle.AsDegrees(); }
 
+    /**
+     * Set the text to be italic - this will also update the font if needed.
+     *
+     * This is the properties system interface.
+     */
     void SetItalic( bool aItalic );
+
+    /**
+     * Set only the italic flag, without changing the font.
+     *
+     * Used when bulk-changing text attributes (e.g. from a dialog or import).
+     */
+    void SetItalicFlag( bool aItalic );
     bool IsItalic() const                       { return m_attributes.m_Italic; }
 
+    /**
+     * Set the text to be bold - this will also update the font if needed.
+     *
+     * This is the properties system interface.
+     */
     void SetBold( bool aBold );
+
+    /**
+     * Set only the italic flag, without changing the font.
+     *
+     * Used when bulk-changing text attributes (e.g. from a dialog or import).
+     */
+    void SetBoldFlag( bool aBold );
     bool IsBold() const                         { return m_attributes.m_Bold; }
 
     virtual void SetVisible( bool aVisible );
@@ -157,10 +192,18 @@ public:
     void SetKeepUpright( bool aKeepUpright );
     bool IsKeepUpright() const                  { return m_attributes.m_KeepUpright; }
 
+    void FlipHJustify()
+    {
+        if( GetHorizJustify() == GR_TEXT_H_ALIGN_RIGHT )
+            SetHorizJustify( GR_TEXT_H_ALIGN_LEFT );
+        else if( GetHorizJustify() == GR_TEXT_H_ALIGN_LEFT )
+            SetHorizJustify( GR_TEXT_H_ALIGN_RIGHT );
+    }
+
     /**
      * Set the text attributes from another instance.
      */
-    void SetAttributes( const EDA_TEXT& aSrc );
+    void SetAttributes( const EDA_TEXT& aSrc, bool aSetPosition = true );
 
     /**
      * Swap the text attributes of the two involved instances.
@@ -190,12 +233,18 @@ public:
     void SetFont( KIFONT::FONT* aFont );
     KIFONT::FONT* GetFont() const               { return m_attributes.m_Font; }
 
+    void SetUnresolvedFontName( const wxString& aFontName ) { m_unresolvedFontName = aFontName; }
+    bool ResolveFont( const std::vector<wxString>* aEmbeddedFonts );
+
     wxString GetFontName() const;
 
+    void SetFontIndex( int aIdx );
+    int GetFontIndex() const;
+
     void SetLineSpacing( double aLineSpacing );
     double GetLineSpacing() const               { return m_attributes.m_LineSpacing; }
 
-    void SetTextSize( const VECTOR2I& aNewSize );
+    void SetTextSize( VECTOR2I aNewSize, bool aEnforceMinTextSize = true );
     VECTOR2I GetTextSize() const                { return m_attributes.m_Size; }
 
     void SetTextWidth( int aWidth );
@@ -228,8 +277,8 @@ public:
      * @param aColor text color.
      * @param aDisplay_mode #FILLED or #SKETCH.
      */
-    void Print( const RENDER_SETTINGS* aSettings, const VECTOR2I& aOffset,
-                const COLOR4D& aColor, OUTLINE_MODE aDisplay_mode = FILLED );
+    void Print( const RENDER_SETTINGS* aSettings, const VECTOR2I& aOffset, const COLOR4D& aColor,
+                OUTLINE_MODE aDisplay_mode = FILLED );
 
     /**
      * build a list of segments (SHAPE_SEGMENT) to describe a text shape.
@@ -238,7 +287,8 @@ public:
      * false to build a list of shape for a not rotated text ("native" shapes).
      */
     std::shared_ptr<SHAPE_COMPOUND> GetEffectiveTextShape( bool aTriangulate = true,
-                                                           bool aUseTextRotation = true ) const;
+                                                           const BOX2I& aBBox = BOX2I(),
+                                                           const EDA_ANGLE& aAngle = ANGLE_0 ) const;
 
     /**
      * Test if \a aPoint is within the bounds of this object.
@@ -264,12 +314,11 @@ public:
      * locate functions....)
      *
      * @param aLine The line of text to consider.  Pass -1 for all lines.
-     * @param aInvertY Invert the Y axis when calculating bounding box.
      * @return the rect containing the line of text (i.e. the position and the size of one line)
      *         this rectangle is calculated for 0 orient text.
      *         If orientation is not 0 the rect must be rotated to match the physical area
      */
-    BOX2I GetTextBox( int aLine = -1, bool aInvertY = false ) const;
+    BOX2I GetTextBox( int aLine = -1 ) const;
 
     /**
      * Return the distance between two lines of text.
@@ -294,15 +343,24 @@ public:
      */
     void GetLinePositions( std::vector<VECTOR2I>& aPositions, int aLineCount ) const;
 
+    /**
+     * Return the levenstein distance between two texts.
+     *
+     * Return a value of 0.0 - 1.0 where 1.0 is a perfect match.
+    */
+    double Levenshtein( const EDA_TEXT& aOther ) const;
+
+
+    double Similarity( const EDA_TEXT& aOther ) const;
+
     /**
      * Output the object to \a aFormatter in s-expression form.
      *
      * @param aFormatter The #OUTPUTFORMATTER object to write to.
-     * @param aNestLevel The indentation next level.
      * @param aControlBits The control bit definition for object specific formatting.
      * @throw IO_ERROR on write error.
      */
-    virtual void Format( OUTPUTFORMATTER* aFormatter, int aNestLevel, int aControlBits ) const;
+    virtual void Format( OUTPUTFORMATTER* aFormatter, int aControlBits ) const;
 
     virtual EDA_ANGLE GetDrawRotation() const               { return GetTextAngle(); }
     virtual VECTOR2I GetDrawPos() const                     { return GetTextPos(); }
@@ -315,11 +373,16 @@ public:
                     const VECTOR2I& aOffset = { 0, 0 } ) const;
 
     // Support for reading the cache from disk.
-    void SetupRenderCache( const wxString& aResolvedText, const EDA_ANGLE& aAngle );
+    void SetupRenderCache( const wxString& aResolvedText, const KIFONT::FONT* aFont,
+                           const EDA_ANGLE& aAngle, const VECTOR2I& aOffset );
     void AddRenderCacheGlyph( const SHAPE_POLY_SET& aPoly );
 
     int Compare( const EDA_TEXT* aOther ) const;
 
+    bool operator==( const EDA_TEXT& aRhs ) const { return Compare( &aRhs ) == 0; }
+    bool operator<( const EDA_TEXT& aRhs ) const { return Compare( &aRhs ) < 0; }
+    bool operator>( const EDA_TEXT& aRhs ) const { return Compare( &aRhs ) > 0; }
+
     virtual bool HasHyperlink() const           { return !m_hyperlink.IsEmpty(); }
     wxString     GetHyperlink() const           { return m_hyperlink; }
     void         SetHyperlink( wxString aLink ) { m_hyperlink = aLink; }
@@ -354,7 +417,9 @@ public:
 protected:
     virtual KIFONT::FONT* getDrawFont() const;
 
-    void cacheShownText();
+    virtual const KIFONT::METRICS& getFontMetrics() const;
+
+    virtual void cacheShownText();
 
     /**
      * Print each line of this EDA_TEXT.
@@ -380,23 +445,39 @@ private:
     wxString         m_shown_text;           // Cache of unescaped text for efficient access
     bool             m_shown_text_has_text_var_refs;
 
-    std::reference_wrapper<const EDA_IU_SCALE> m_IuScale;
+    std::reference_wrapper<const EDA_IU_SCALE>          m_IuScale;
 
-    mutable const KIFONT::FONT*                         m_render_cache_font;
     mutable wxString                                    m_render_cache_text;
+    mutable const KIFONT::FONT*                         m_render_cache_font;
     mutable EDA_ANGLE                                   m_render_cache_angle;
     mutable VECTOR2I                                    m_render_cache_offset;
     mutable std::vector<std::unique_ptr<KIFONT::GLYPH>> m_render_cache;
 
-    mutable bool     m_bounding_box_cache_valid;
-    mutable VECTOR2I m_bounding_box_cache_pos;
-    mutable int      m_bounding_box_cache_line;
-    mutable bool     m_bounding_box_cache_inverted;
-    mutable BOX2I    m_bounding_box_cache;
+    struct BBOX_CACHE_ENTRY
+    {
+        VECTOR2I m_pos;
+        BOX2I    m_bbox;
+    };
+
+    mutable std::map<int, BBOX_CACHE_ENTRY> m_bbox_cache;
 
     TEXT_ATTRIBUTES  m_attributes;
+    wxString         m_unresolvedFontName;
     VECTOR2I         m_pos;
 };
 
 
+extern std::ostream& operator<<( std::ostream& aStream, const EDA_TEXT& aAttributes );
+
+
+template<>
+struct std::hash<EDA_TEXT>
+{
+    std::size_t operator()( const EDA_TEXT& aText ) const
+    {
+        return hash_val( aText.GetText(), aText.GetAttributes(), aText.GetTextPos().x,
+                         aText.GetTextPos().y );
+    }
+};
+
 #endif   //  EDA_TEXT_H_
