@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2014 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
- * Copyright (C) 2016-2020 KiCad Developers, see AUTHORS.TXT for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.TXT for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -104,7 +104,6 @@
 #include <frame_type.h>
 #include <mail_type.h>
 #include <ki_exception.h>
-#include <jobs/job.h>
 
 
 #define KIFACE_VERSION      1
@@ -133,6 +132,8 @@ class KIWAY;
 class KIWAY_PLAYER;
 class wxTopLevelWindow;
 class TOOL_ACTION;
+class JOB;
+class REPORTER;
 
 
 /**
@@ -174,7 +175,7 @@ struct KIFACE
      *         any UI because that is the duty of this function to say why it is returning
      *         false.  Never return false without having reported to the UI why.
      */
-    virtual bool OnKifaceStart( PGM_BASE* aProgram, int aCtlBits ) = 0;
+    virtual bool OnKifaceStart( PGM_BASE* aProgram, int aCtlBits, KIWAY* aKiway ) = 0;
 
     /**
      * Called just once just before the DSO is to be unloaded.
@@ -241,7 +242,12 @@ struct KIFACE
      */
     virtual void GetActions( std::vector<TOOL_ACTION*>& aActions ) const = 0;
 
-    virtual int HandleJob( JOB* aJob )
+    virtual int HandleJob( JOB* aJob, REPORTER* aReporter )
+    {
+        return 0;
+    }
+
+    virtual bool HandleJobConfig( JOB* aJob, wxWindow* aParent )
     {
         return 0;
     }
@@ -275,7 +281,7 @@ struct KIFACE
  * and a #NETLIST, (anything relating to production of a single #BOARD and added to class
  * #PROJECT.)
  */
-class KIWAY : public wxEvtHandler
+class KICOMMON_API KIWAY : public wxEvtHandler
 {
     friend struct PGM_SINGLE_TOP;        // can use set_kiface()
 
@@ -358,6 +364,11 @@ public:
      */
     virtual bool PlayersClose( bool doForce );
 
+    /**
+     * Notifies a Kiway that a player has been closed.
+     */
+    void PlayerDidClose( FRAME_T aFrameType );
+
     /**
      * Send @a aPayload to @a aDestination from @a aSource.
      *
@@ -390,7 +401,7 @@ public:
      *
      * Use after changing suite-wide options such as panning, autosave interval, etc.
      */
-    virtual void CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVarsChanged );
+    virtual void CommonSettingsChanged( int aFlags );
 
     /**
      * Calls ProjectChanged() on all KIWAY_PLAYERs.
@@ -398,7 +409,7 @@ public:
      */
     virtual void ProjectChanged();
 
-    KIWAY( PGM_BASE* aProgram, int aCtlBits, wxFrame* aTop = nullptr );
+    KIWAY( int aCtlBits, wxFrame* aTop = nullptr );
 
     /**
      * Overwrites previously set ctl bits, only for use in kicad.cpp to flip between
@@ -421,7 +432,8 @@ public:
 
     bool ProcessEvent( wxEvent& aEvent ) override;
 
-    int ProcessJob( KIWAY::FACE_T aFace, JOB* job );
+    int  ProcessJob( KIWAY::FACE_T aFace, JOB* aJob, REPORTER* aReporter = nullptr );
+    bool ProcessJobConfigDialog( KIWAY::FACE_T aFace, JOB* aJob, wxWindow* aWindow );
 
     /**
      * Gets the window pointer to the blocking dialog (to send it signals)
@@ -434,11 +446,6 @@ private:
     /// Get the [path &] name of the DSO holding the requested FACE_T.
     const wxString dso_search_path( FACE_T aFaceId );
 
-#if 0
-    /// hooked into m_top in SetTop(), marks child frame as closed.
-    void player_destroy_handler( wxWindowDestroyEvent& event );
-#endif
-
     bool set_kiface( FACE_T aFaceType, KIFACE* aKiface )
     {
         if( (unsigned) aFaceType < (unsigned) KIWAY_FACE_COUNT )
@@ -459,7 +466,6 @@ private:
     static KIFACE*  m_kiface[KIWAY_FACE_COUNT];
     static int      m_kiface_version[KIWAY_FACE_COUNT];
 
-    PGM_BASE*       m_program;
     int             m_ctl;
 
     wxFrame*        m_top;      // Usually m_top is the Project manager
@@ -503,9 +509,9 @@ typedef KIFACE* KIFACE_GETTER_FUNC( int* aKIFACEversion, int aKIWAYversion, PGM_
 /// No name mangling.  Each KIFACE (DSO/DLL) will implement this once.
 extern "C" {
 #if defined(BUILD_KIWAY_DLL)
- MY_API( KIFACE* ) KIFACE_GETTER(  int* aKIFACEversion, int aKIWAYversion, PGM_BASE* aProgram );
+    KIFACE_API KIFACE* KIFACE_GETTER(  int* aKIFACEversion, int aKIWAYversion, PGM_BASE* aProgram );
 #else
- KIFACE* KIFACE_GETTER(  int* aKIFACEversion, int aKIWAYversion, PGM_BASE* aProgram );
+    KIFACE* KIFACE_GETTER(  int* aKIFACEversion, int aKIWAYversion, PGM_BASE* aProgram );
 #endif
 }
 
