@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2016-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -35,7 +35,8 @@
 class PSLIKE_PLOTTER : public PLOTTER
 {
 public:
-    PSLIKE_PLOTTER() :
+    PSLIKE_PLOTTER( const PROJECT* aProject = nullptr) :
+            PLOTTER( aProject ),
             plotScaleAdjX( 1 ),
             plotScaleAdjY( 1 ),
             m_textMode( PLOT_TEXT_MODE::PHANTOM )
@@ -83,9 +84,8 @@ public:
                                       void* aData ) override;
 
     /**
-     * The SetColor implementation is split with the subclasses:
-     * The PSLIKE computes the rgb values, the subclass emits the
-     * operator to actually do it
+     * The SetColor implementation is split with the subclasses: the PSLIKE computes the rgb
+     * values, the subclass emits the operator to actually do it
      */
     virtual void SetColor( const COLOR4D& color ) override;
 
@@ -93,9 +93,8 @@ protected:
     /**
      * This is the core for postscript/PDF text alignment.
      *
-     * It computes the transformation matrix to generate a user space
-     * system aligned with the text. Even the PS uses the concat
-     * operator to simplify PDF generation (concat is everything PDF
+     * It computes the transformation matrix to generate a user space system aligned with the text.
+     * Even the PS uses the concat operator to simplify PDF generation (concat is everything PDF
      * has to modify the CTM. Lots of parameters, both in and out.
      */
     void computeTextParameters( const VECTOR2I&          aPos,
@@ -117,15 +116,6 @@ protected:
                                 double                   *ctm_f,
                                 double                   *heightFactor );
 
-    /**
-     * Computes the x coordinates for the overlining in a string of text.
-     * Fills the passed vector with couples of (start, stop) values to be
-     * used in the text coordinate system (use computeTextParameters to
-     * obtain the parameters to establish such a system)
-     */
-    void postscriptOverlinePositions( const wxString& aText, int aXSize, bool aItalic, bool aBold,
-                                      std::vector<int> *pos_pairs );
-
     /// convert a wxString unicode string to a char string compatible with the accepted
     /// string plotter format (convert special chars and non ascii7 chars)
     virtual std::string encodeStringForPlotter( const wxString& aUnicode );
@@ -137,10 +127,9 @@ protected:
     static const double postscriptTextAscent; // = 0.718;
 
     /**
-     * Sister function for the GraphicTextWidth in drawtxt.cpp
-     * Does the same processing (i.e. calculates a text string width) but
-     * using postscript metrics for the Helvetica font (optionally used for
-     * PS and PDF plotting
+     * Sister function for the GRTextWidth in gr_text.cpp
+     * Does the same processing (i.e. calculates a text string width) but using postscript metrics
+     * for the Helvetica font (optionally used for PS and PDF plotting
      */
     int returnPostscriptTextWidth( const wxString& aText, int aXSize, bool aItalic, bool aBold );
 
@@ -155,7 +144,8 @@ protected:
 class PS_PLOTTER : public PSLIKE_PLOTTER
 {
 public:
-    PS_PLOTTER()
+    PS_PLOTTER( const PROJECT* aProject = nullptr ) :
+        PSLIKE_PLOTTER( aProject )
     {
         // The phantom plot in postscript is an hack and reportedly
         // crashes Adobe's own postscript interpreter!
@@ -196,7 +186,7 @@ public:
     /**
      * PostScript supports dashed lines.
      */
-    virtual void SetDash( int aLineWidth, PLOT_DASH_TYPE aLineStyle ) override;
+    virtual void SetDash( int aLineWidth, LINE_STYLE aLineStyle ) override;
 
     virtual void SetViewport( const VECTOR2I& aOffset, double aIusPerDecimil,
                               double aScale, bool aMirror ) override;
@@ -204,8 +194,9 @@ public:
                        int width = USE_DEFAULT_LINE_WIDTH ) override;
     virtual void Circle( const VECTOR2I& pos, int diametre, FILL_T fill,
                          int width = USE_DEFAULT_LINE_WIDTH ) override;
-    virtual void Arc( const VECTOR2I& aCenter, const VECTOR2I& aStart, const VECTOR2I& aEnd,
-                      FILL_T aFill, int aWidth, int aMaxError ) override;
+    virtual void Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
+                      const EDA_ANGLE& aAngle, double aRadius, FILL_T aFill,
+                      int aWidth = USE_DEFAULT_LINE_WIDTH ) override;
 
     virtual void PlotPoly( const std::vector<VECTOR2I>& aCornerList, FILL_T aFill,
                            int aWidth = USE_DEFAULT_LINE_WIDTH, void* aData = nullptr ) override;
@@ -217,19 +208,29 @@ public:
                             double aScaleFactor ) override;
 
     virtual void PenTo( const VECTOR2I& pos, char plume ) override;
-    virtual void Text( const VECTOR2I&             aPos,
-                       const COLOR4D&              aColor,
-                       const wxString&             aText,
-                       const EDA_ANGLE&            aOrient,
-                       const VECTOR2I&             aSize,
-                       enum GR_TEXT_H_ALIGN_T      aH_justify,
-                       enum GR_TEXT_V_ALIGN_T      aV_justify,
-                       int                         aWidth,
-                       bool                        aItalic,
-                       bool                        aBold,
-                       bool                        aMultilineAllowed = false,
-                       KIFONT::FONT*               aFont = nullptr,
-                       void*                       aData = nullptr ) override;
+    virtual void Text( const VECTOR2I&        aPos,
+                       const COLOR4D&         aColor,
+                       const wxString&        aText,
+                       const EDA_ANGLE&       aOrient,
+                       const VECTOR2I&        aSize,
+                       enum GR_TEXT_H_ALIGN_T aH_justify,
+                       enum GR_TEXT_V_ALIGN_T aV_justify,
+                       int                    aWidth,
+                       bool                   aItalic,
+                       bool                   aBold,
+                       bool                   aMultilineAllowed,
+                       KIFONT::FONT*          aFont,
+                       const KIFONT::METRICS& aFontMetrics,
+                       void*                  aData = nullptr ) override;
+
+    virtual void PlotText( const VECTOR2I&        aPos,
+                           const COLOR4D&         aColor,
+                           const wxString&        aText,
+                           const TEXT_ATTRIBUTES& aAttributes,
+                           KIFONT::FONT*          aFont,
+                           const KIFONT::METRICS& aFontMetrics,
+                           void*                  aData = nullptr ) override;
+
 
 protected:
     virtual void emitSetRGBColor( double r, double g, double b, double a ) override;
@@ -239,12 +240,16 @@ protected:
 class PDF_PLOTTER : public PSLIKE_PLOTTER
 {
 public:
-    PDF_PLOTTER() :
+    PDF_PLOTTER( const PROJECT* aProject = nullptr ) :
+            PSLIKE_PLOTTER( aProject ),
             m_pageTreeHandle( 0 ),
             m_fontResDictHandle( 0 ),
+            m_imgResDictHandle( 0 ),
+            m_jsNamesHandle( 0 ),
             m_pageStreamHandle( 0 ),
             m_streamLengthHandle( 0 ),
-            m_workFile( nullptr )
+            m_workFile( nullptr ),
+            m_totalOutlineNodes( 0 )
     {
     }
 
@@ -283,7 +288,8 @@ public:
     /**
      * Start a new page in the PDF document.
      */
-    virtual void StartPage( const wxString& aPageNumber, const wxString& aPageName = wxEmptyString );
+    virtual void StartPage( const wxString& aPageNumber,
+                            const wxString& aPageName = wxEmptyString );
 
     /**
      * Close the current page in the PDF document (and emit its compressed stream).
@@ -303,7 +309,7 @@ public:
     /**
      * PDF supports dashed lines
      */
-    virtual void SetDash( int aLineWidth, PLOT_DASH_TYPE aLineStyle ) override;
+    virtual void SetDash( int aLineWidth, LINE_STYLE aLineStyle ) override;
 
     /**
      * PDF can have multiple pages, so SetPageSettings can be called
@@ -327,8 +333,9 @@ public:
     /**
      * The PDF engine can't directly plot arcs so we use polygonization.
      */
-    virtual void Arc( const VECTOR2I& aCenter, const VECTOR2I& aStart, const VECTOR2I& aEnd,
-                      FILL_T aFill, int aWidth, int aMaxError ) override;
+    virtual void Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
+                      const EDA_ANGLE& aAngle, double aRadius, FILL_T aFill,
+                      int aWidth = USE_DEFAULT_LINE_WIDTH ) override;
 
     /**
      * Polygon plotting for PDF. Everything is supported
@@ -338,25 +345,35 @@ public:
 
     virtual void PenTo( const VECTOR2I& pos, char plume ) override;
 
-    virtual void Text( const VECTOR2I&             aPos,
-                       const COLOR4D&              aColor,
-                       const wxString&             aText,
-                       const EDA_ANGLE&            aOrient,
-                       const VECTOR2I&             aSize,
-                       enum GR_TEXT_H_ALIGN_T      aH_justify,
-                       enum GR_TEXT_V_ALIGN_T      aV_justify,
-                       int                         aWidth,
-                       bool                        aItalic,
-                       bool                        aBold,
-                       bool                        aMultilineAllowed = false,
-                       KIFONT::FONT*               aFont = nullptr,
-                       void*                       aData = nullptr ) override;
+    virtual void Text( const VECTOR2I&        aPos,
+                       const COLOR4D&         aColor,
+                       const wxString&        aText,
+                       const EDA_ANGLE&       aOrient,
+                       const VECTOR2I&        aSize,
+                       enum GR_TEXT_H_ALIGN_T aH_justify,
+                       enum GR_TEXT_V_ALIGN_T aV_justify,
+                       int                    aWidth,
+                       bool                   aItalic,
+                       bool                   aBold,
+                       bool                   aMultilineAllowed,
+                       KIFONT::FONT*          aFont,
+                       const KIFONT::METRICS& aFontMetrics,
+                       void*                  aData = nullptr ) override;
+
+    virtual void PlotText( const VECTOR2I&        aPos,
+                           const COLOR4D&         aColor,
+                           const wxString&        aText,
+                           const TEXT_ATTRIBUTES& aAttributes,
+                           KIFONT::FONT*          aFont,
+                           const KIFONT::METRICS& aFontMetrics,
+                           void*                  aData = nullptr ) override;
 
     void HyperlinkBox( const BOX2I& aBox, const wxString& aDestinationURL ) override;
 
     void HyperlinkMenu( const BOX2I& aBox, const std::vector<wxString>& aDestURLs ) override;
 
-    void Bookmark( const BOX2I& aBox, const wxString& aName, const wxString& aGroupName = wxEmptyString ) override;
+    void Bookmark( const BOX2I& aBox, const wxString& aName,
+                   const wxString& aGroupName = wxEmptyString ) override;
 
     /**
      * PDF images are handles as inline, not XObject streams...
@@ -396,21 +413,18 @@ protected:
     };
 
     /**
-     * Adds a new outline node entry
+     * Add a new outline node entry.
      *
-     * The PDF object handle is automacially allocated
+     * The PDF object handle is automatically allocated.
      *
-     * @param aParent Parent node to append the new node to
-     * @param aActionHandle The handle of an action that may be performed on click, set to -1 for no action
+     * @param aParent Parent node to append the new node to.
+     * @param aActionHandle The handle of an action that may be performed on click, set to -1
+     *                      for no action.
      * @param aTitle Title of node to display
      */
     OUTLINE_NODE* addOutlineNode( OUTLINE_NODE* aParent, int aActionHandle,
                                   const wxString& aTitle );
 
-    virtual void Arc( const VECTOR2I& aCenter, const EDA_ANGLE& aStartAngle,
-                      const EDA_ANGLE& aEndAngle, int aRadius,
-                      FILL_T aFill, int aWidth = USE_DEFAULT_LINE_WIDTH ) override;
-
     /// convert a wxString unicode string to a char string compatible with the accepted
     /// string PDF format (convert special chars and non ascii7 chars)
     std::string encodeStringForPlotter( const wxString& aUnicode ) override;
@@ -427,8 +441,9 @@ protected:
     virtual void emitSetRGBColor( double r, double g, double b, double a ) override;
 
     /**
-     * Allocate a new handle in the table of the PDF object. The
-     * handle must be completed using startPdfObject. It's an in-RAM operation
+     * Allocate a new handle in the table of the PDF object.
+     *
+     * The handle must be completed using startPdfObject. It's an in-RAM operation
      * only, no output is done.
      */
     int allocPdfObject();
@@ -437,7 +452,7 @@ protected:
      * Open a new PDF object and returns the handle if the parameter is -1.
      * Otherwise fill in the xref entry for the passed object
      */
-    int startPdfObject(int handle = -1);
+    int startPdfObject( int handle = -1 );
 
     /**
      * Close the current PDF object
@@ -445,68 +460,73 @@ protected:
     void closePdfObject();
 
     /**
-     * Starts a PDF stream (for the page). Returns the object handle opened
-     * Pass -1 (default) for a fresh object. Especially from PDF 1.5 streams
-     * can contain a lot of things, but for the moment we only handle page
-     * content.
+     * Start a PDF stream (for the page).
+     *
+     * @param handle -1 (default) for a new object. Especially from PDF 1.5 streams can contain
+     *               a lot of things, but for the moment we only handle page content.
+     * @eturn The object handle opened
      */
-    int startPdfStream(int handle = -1);
+    int startPdfStream( int handle = -1 );
 
     /**
-     * Finish the current PDF stream (writes the deferred length, too)
+     * Finish the current PDF stream (writes the deferred length, too).
      */
     void closePdfStream();
 
     /**
-     * Starts emitting the outline object
+     * Starts emitting the outline object.
      */
     int emitOutline();
 
     /**
-     * Emits a outline item object and recurses into any children
+     * Emits a outline item object and recurses into any children.
      */
     void emitOutlineNode( OUTLINE_NODE* aNode, int aParentHandle, int aNextNode, int aPrevNode );
 
     /**
-     * Emits an action object that instructs a goto coordinates on a page
+     * Emit an action object that instructs a goto coordinates on a page.
      *
-     * @return Generated action handle
+     * @return Generated action handle.
      */
     int emitGoToAction( int aPageHandle, const VECTOR2I& aBottomLeft, const VECTOR2I& aTopRight );
     int emitGoToAction( int aPageHandle );
 
-    int m_pageTreeHandle;           ///< Handle to the root of the page tree object
-    int m_fontResDictHandle;        ///< Font resource dictionary
-    std::vector<int> m_pageHandles; ///< Handles to the page objects
-    int m_pageStreamHandle;         ///< Handle of the page content object
-    int m_streamLengthHandle;       ///< Handle to the deferred stream length
+    int m_pageTreeHandle;           ///< Handle to the root of the page tree object.
+    int m_fontResDictHandle;        ///< Font resource dictionary.
+    int m_imgResDictHandle;         ///< Image resource dictionary.
+    int m_jsNamesHandle;            ///< Handle for Names dictionary with JS.
+    std::vector<int> m_pageHandles; ///< Handles to the page objects.
+    int m_pageStreamHandle;         ///< Handle of the page content object.
+    int m_streamLengthHandle;       ///< Handle to the deferred stream length.
     wxString m_workFilename;
     wxString m_pageName;
-    FILE* m_workFile;               ///< Temporary file to construct the stream before zipping
-    std::vector<long> m_xrefTable;  ///< The PDF xref offset table
+    FILE* m_workFile;               ///< Temporary file to construct the stream before zipping.
+    std::vector<long> m_xrefTable;  ///< The PDF xref offset table.
 
-    ///< List of user-space page numbers for resolving internal hyperlinks
+    /// List of user-space page numbers for resolving internal hyperlinks.
     std::vector<wxString>                                  m_pageNumbers;
 
-    ///< List of loaded hyperlinks in current page
+    /// List of loaded hyperlinks in current page.
     std::vector<std::pair<BOX2I, wxString>>                m_hyperlinksInPage;
     std::vector<std::pair<BOX2I, std::vector<wxString>>>   m_hyperlinkMenusInPage;
 
-    ///< Handles for all the hyperlink objects that will be deferred
+    /// Handles for all the hyperlink objects that will be deferred.
     std::map<int, std::pair<BOX2D, wxString>>              m_hyperlinkHandles;
     std::map<int, std::pair<BOX2D, std::vector<wxString>>> m_hyperlinkMenuHandles;
 
     std::map<wxString, std::vector<std::pair<BOX2I, wxString>>>      m_bookmarksInPage;
 
-    std::unique_ptr<OUTLINE_NODE> m_outlineRoot;    ///< Root outline node
-    int                           m_totalOutlineNodes;  ///< Total number of outline nodes
+    std::map<int, wxImage> m_imageHandles;
+
+    std::unique_ptr<OUTLINE_NODE> m_outlineRoot;        ///< Root outline node.
+    int                           m_totalOutlineNodes;  ///< Total number of outline nodes.
 };
 
 
 class SVG_PLOTTER : public PSLIKE_PLOTTER
 {
 public:
-    SVG_PLOTTER();
+    SVG_PLOTTER( const PROJECT* aProject = nullptr );
 
     static wxString GetDefaultFileExtension()
     {
@@ -518,8 +538,6 @@ public:
         return PLOT_FORMAT::SVG;
     }
 
-    virtual void SetColor( const COLOR4D& color ) override;
-
     /**
      * Create SVG file header.
      */
@@ -534,7 +552,7 @@ public:
     /**
      * SVG supports dashed lines.
      */
-    virtual void SetDash( int aLineWidth, PLOT_DASH_TYPE aLineStyle ) override;
+    virtual void SetDash( int aLineWidth, LINE_STYLE aLineStyle ) override;
 
     virtual void SetViewport( const VECTOR2I& aOffset, double aIusPerDecimil,
                               double aScale, bool aMirror ) override;
@@ -542,6 +560,9 @@ public:
                        int width = USE_DEFAULT_LINE_WIDTH ) override;
     virtual void Circle( const VECTOR2I& pos, int diametre, FILL_T fill,
                          int width = USE_DEFAULT_LINE_WIDTH ) override;
+    virtual void Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
+                      const EDA_ANGLE& aAngle, double aRadius, FILL_T aFill,
+                      int aWidth = USE_DEFAULT_LINE_WIDTH ) override;
 
     virtual void BezierCurve( const VECTOR2I& aStart, const VECTOR2I& aControl1,
                               const VECTOR2I& aControl2, const VECTOR2I& aEnd,
@@ -585,25 +606,31 @@ public:
      */
     virtual void EndBlock( void* aData ) override;
 
-    virtual void Text( const VECTOR2I&             aPos,
-                       const COLOR4D&              aColor,
-                       const wxString&             aText,
-                       const EDA_ANGLE&            aOrient,
-                       const VECTOR2I&             aSize,
-                       enum GR_TEXT_H_ALIGN_T      aH_justify,
-                       enum GR_TEXT_V_ALIGN_T      aV_justify,
-                       int                         aWidth,
-                       bool                        aItalic,
-                       bool                        aBold,
-                       bool                        aMultilineAllowed = false,
-                       KIFONT::FONT*               aFont = nullptr,
-                       void*                       aData = nullptr ) override;
+    virtual void Text( const VECTOR2I&        aPos,
+                       const COLOR4D&         aColor,
+                       const wxString&        aText,
+                       const EDA_ANGLE&       aOrient,
+                       const VECTOR2I&        aSize,
+                       enum GR_TEXT_H_ALIGN_T aH_justify,
+                       enum GR_TEXT_V_ALIGN_T aV_justify,
+                       int                    aWidth,
+                       bool                   aItalic,
+                       bool                   aBold,
+                       bool                   aMultilineAllowed,
+                       KIFONT::FONT*          aFont,
+                       const KIFONT::METRICS& aFontMetrics,
+                       void*                  aData = nullptr ) override;
+
+
+    virtual void PlotText( const VECTOR2I&        aPos,
+                           const COLOR4D&         aColor,
+                           const wxString&        aText,
+                           const TEXT_ATTRIBUTES& aAttributes,
+                           KIFONT::FONT*          aFont,
+                           const KIFONT::METRICS& aFontMetrics,
+                           void*                  aData = nullptr ) override;
 
 protected:
-    virtual void Arc( const VECTOR2I& aCenter, const EDA_ANGLE& aStartAngle,
-                      const EDA_ANGLE& aEndAngle, int aRadius,
-                      FILL_T aFill, int aWidth = USE_DEFAULT_LINE_WIDTH ) override;
-
     /**
      * Initialize m_pen_rgb_color from reduced values r, g ,b
      * ( reduced values are 0.0 to 1.0 )
@@ -624,24 +651,18 @@ protected:
      */
     void setFillMode( FILL_T fill );
 
-    FILL_T         m_fillMode;          // true if the current contour
-                                        // rect, arc, circle, polygon must be filled
-    long           m_pen_rgb_color;     // current rgb color value: each color has
-                                        // a value 0 ... 255, and the 3 colors are
-                                        // grouped in a 3x8 bits value
-                                        // (written in hex to svg files)
-    long           m_brush_rgb_color;   // same as m_pen_rgb_color, used to fill
-                                        // some contours.
-    double         m_brush_alpha;
-    bool           m_graphics_changed;  // true if a pen/brush parameter is modified
-                                        // color, pen size, fill mode ...
-                                        // the new SVG stype must be output on file
-    PLOT_DASH_TYPE m_dashed;            // plot line style
-    bool           m_useInch;           // is 0 if the step size is 10**-n*mm
-                                        // is 1 if the step size is 10**-n*inch
-                                        // Where n is given from m_precision
-    unsigned       m_precision;         // How fine the step size is
-                                        // Use 3-6 (3 means um precision, 6 nm precision) in PcbNew
-                                        // 3-4 in other modules (avoid values >4 to avoid overflow)
-                                        // see also comment for m_useInch.
+    FILL_T     m_fillMode;          // true if the current contour rect, arc, circle, polygon must
+                                    // be filled
+    long       m_pen_rgb_color;     // current rgb color value: each color has a value 0 ... 255,
+                                    //   and the 3 colors are grouped in a 3x8 bits value (written
+                                    //   in hex to svg files)
+    long       m_brush_rgb_color;   // same as m_pen_rgb_color, used to fill some contours.
+    double     m_brush_alpha;
+    bool       m_graphics_changed;  // true if a pen/brush parameter is modified color, pen size,
+                                    //   fill mode ... the new SVG stype must be output on file
+    LINE_STYLE m_dashed;            // plot line style
+    unsigned   m_precision;         // How fine the step size is
+                                    // Use 3-6 (3 means um precision, 6 nm precision) in PcbNew
+                                    // 3-4 in other modules (avoid values >4 to avoid overflow)
+                                    // see also comment for m_useInch.
 };
