@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2014 Henner Zeller <h.zeller@acm.org>
- * Copyright (C) 2014-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,15 +26,18 @@
 #define LIB_TREE_H
 
 #include <wx/panel.h>
+#include <wx/timer.h>
 #include <lib_tree_model_adapter.h>
-#include <html_window.h>
+#include <widgets/html_window.h>
+#include <widgets/wx_dataviewctrl.h>
 
-class wxDataViewCtrl;
 class wxTextCtrl;
 class wxHtmlLinkEvent;
 class wxSearchCtrl;
 class wxTimer;
 class wxTimerEvent;
+class wxPopupWindow;
+class STD_BITMAP_BUTTON;
 class ACTION_MENU;
 class LIB_ID;
 class LIB_TABLE;
@@ -48,9 +51,10 @@ public:
     ///< Flags to select extra widgets and options
     enum FLAGS
     {
-        NONE        = 0x00,
+        FLAGS_NONE  = 0x00,
         SEARCH      = 0x01,
-        DETAILS     = 0x02,
+        FILTERS     = 0x02,
+        DETAILS     = 0x04,
         ALL_WIDGETS = 0x0F,
         MULTISELECT = 0x10
     };
@@ -58,12 +62,12 @@ public:
     /**
      * Construct a symbol tree.
      *
-     * @param aParent parent window containing this tree widget
+     * @param aParent parent window containing this tree widget.
      * @param aRecentSearchesKey a key into a global map storing recent searches (usually "power",
-     *                           "symbols", or "footprints", but could be further differentiated)
-     * @param aLibTable table containing libraries and items to display
-     * @param aAdapter a LIB_TREE_MODEL_ADAPTER instance to use
-     * @param aFlags selection of sub-widgets to include and other options
+     *                           "symbols", or "footprints", but could be further differentiated).
+     * @param aLibTable table containing libraries and items to display.
+     * @param aAdapter a LIB_TREE_MODEL_ADAPTER instance to use.
+     * @param aFlags selection of sub-widgets to include and other options.
      * @param aDetails if not null, a custom HTML_WINDOW to hold symbol details. If null this
      *                 will be created inside the LIB_TREE.
      */
@@ -90,11 +94,12 @@ public:
     }
 
     /**
-     * Retrieves a list of selections for trees that allow multi-selection
+     * Retrieve a list of selections for trees that allow multi-selection.
+     *
      * @see GetSelectedLibId for details on how aUnit will be filled.
-     * @param aSelection will be filled with a list of selected LIB_IDs
-     * @param aUnit is an optional pointer to a list to fill with unit numbers
-     * @return the number of selected items
+     * @param aSelection will be filled with a list of selected LIB_IDs.
+     * @param aUnit is an optional pointer to a list to fill with unit numbers.
+     * @return the number of selected items.
      */
     int GetSelectedLibIds( std::vector<LIB_ID>& aSelection,
                            std::vector<int>* aUnit = nullptr ) const;
@@ -112,7 +117,7 @@ public:
     void CenterLibId( const LIB_ID& aLibId );
 
     /**
-     * Unselect currently selected item in wxDataViewCtrl
+     * Unselect currently selected item in wxDataViewCtrl.
      */
     void Unselect();
 
@@ -121,31 +126,47 @@ public:
      */
     void ExpandLibId( const LIB_ID& aLibId );
 
+    void ExpandAll();
+    void CollapseAll();
+
     /**
      * Save/restore search string.
      */
     void SetSearchString( const wxString& aSearchString );
     wxString GetSearchString() const;
 
+    /**
+     * Save/restore the sorting mode.
+     */
+    void SetSortMode( LIB_TREE_MODEL_ADAPTER::SORT_MODE aMode ) { m_adapter->SetSortMode( aMode ); }
+    LIB_TREE_MODEL_ADAPTER::SORT_MODE GetSortMode() const { return m_adapter->GetSortMode(); }
+
     /**
      * Regenerate the tree.
      */
     void Regenerate( bool aKeepState );
 
     /**
-     * Refreshes the tree (mainly to update highlighting and asterisking)
+     * Refresh the tree (mainly to update highlighting and asterisking)
      */
     void RefreshLibTree();
 
     wxWindow* GetFocusTarget();
 
+    wxSizer* GetFiltersSizer() { return m_filtersSizer; }
+
     /**
-     * Focus the search widget if it exists
+     * Focus the search widget if it exists.
      */
     void FocusSearchFieldIfExists();
 
     void ShowChangedLanguage();
 
+    void BlockPreview( bool aBlock )
+    {
+        m_previewDisabled = aBlock;
+    }
+
 protected:
     /**
      * Expand or collapse a node, switching it to the opposite state.
@@ -168,7 +189,7 @@ protected:
     void postPreselectEvent();
 
     /**
-     * Post SYMBOL_SELECTED event to notify the selection handler that a part has been selected.
+     * Post #SYMBOL_SELECTED event to notify the selection handler that a part has been selected.
      */
     void postSelectEvent();
 
@@ -177,11 +198,10 @@ protected:
      */
     struct STATE
     {
-        ///< List of expanded nodes
+        /// List of expanded nodes.
         std::vector<wxDataViewItem> expanded;
-        std::vector<wxString>       pinned;
 
-        ///< Current selection, might be not valid if nothing was selected
+        /// Current selection, might be not valid if nothing was selected.
         LIB_ID selection;
     };
 
@@ -197,6 +217,10 @@ protected:
 
     void updateRecentSearchMenu();
 
+    void showPreview( wxDataViewItem aItem );
+    void hidePreview();
+    void destroyPreview();
+
     void onQueryText( wxCommandEvent& aEvent );
     void onQueryCharHook( wxKeyEvent& aEvent );
     void onQueryMouseMoved( wxMouseEvent& aEvent );
@@ -204,7 +228,9 @@ protected:
     void onTreeSelect( wxDataViewEvent& aEvent );
     void onTreeActivate( wxDataViewEvent& aEvent );
     void onTreeCharHook( wxKeyEvent& aEvent );
-    void onSize( wxSizeEvent& aEvent );
+
+    void onIdle( wxIdleEvent& aEvent );
+    void onHoverTimer( wxTimerEvent& aEvent );
 
     void onDetailsLink( wxHtmlLinkEvent& aEvent );
     void onPreselect( wxCommandEvent& aEvent );
@@ -214,26 +240,35 @@ protected:
     void onDebounceTimer( wxTimerEvent& aEvent );
 
 protected:
-    LIB_TABLE*       m_lib_table;
-
     wxObjectDataPtr<LIB_TREE_MODEL_ADAPTER> m_adapter;
 
-    wxSearchCtrl*    m_query_ctrl;
-    wxDataViewCtrl*  m_tree_ctrl;
-    HTML_WINDOW*     m_details_ctrl;
-    wxTimer*         m_debounceTimer;
-    bool             m_inTimerEvent;
+    wxSearchCtrl*      m_query_ctrl;
+    STD_BITMAP_BUTTON* m_sort_ctrl;
+    WX_DATAVIEWCTRL*   m_tree_ctrl;
+    HTML_WINDOW*       m_details_ctrl;
+    wxTimer*           m_debounceTimer;
+    bool               m_inTimerEvent;
 
-    LIB_ID           m_last_libid;
-    wxString         m_recentSearchesKey;
+    wxString           m_recentSearchesKey;
 
-    bool             m_skipNextRightClick;
+    wxBoxSizer*        m_filtersSizer;
+
+    bool               m_skipNextRightClick;
+
+    wxPoint            m_hoverPos;
+    wxDataViewItem     m_hoverItem;
+    wxRect             m_hoverItemRect;
+    wxTimer            m_hoverTimer;
+    wxDataViewItem     m_previewItem;
+    wxRect             m_previewItemRect;
+    wxPopupWindow*     m_previewWindow;
+    bool               m_previewDisabled;
 };
 
-///< Custom event sent when a new symbol is preselected
-wxDECLARE_EVENT( SYMBOL_PRESELECTED, wxCommandEvent );
+/// Custom event sent when an item is selected in the list.
+wxDECLARE_EVENT( EVT_LIBITEM_SELECTED, wxCommandEvent );
 
-///< Custom event sent when a symbol is selected
-wxDECLARE_EVENT( SYMBOL_SELECTED, wxCommandEvent );
+/// Custom event sent when an item is chosen (double-clicked).
+wxDECLARE_EVENT( EVT_LIBITEM_CHOSEN, wxCommandEvent );
 
 #endif /* LIB_TREE_H */
