@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2017 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2009 Wayne Stambaugh <stambaughw@verizon.net>
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * Copyright (C) 2019 CERN
  *
  * This program is free software; you can redistribute it and/or
@@ -24,10 +24,10 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <advanced_config.h>
 #include <bitmaps.h>
 #include <file_history.h>
 #include <kiplatform/policy.h>
-#include <menus_helpers.h>
 #include <paths.h>
 #include <policy_keys.h>
 #include <tool/action_manager.h>
@@ -76,6 +76,11 @@ void KICAD_MANAGER_FRAME::doReCreateMenuBar()
     fileMenu->Add( KICAD_MANAGER_ACTIONS::newProject );
     fileMenu->Add( KICAD_MANAGER_ACTIONS::newFromTemplate );
 
+    if( ADVANCED_CFG::GetCfg().m_EnableGit )
+    {
+        fileMenu->Add( KICAD_MANAGER_ACTIONS::newFromRepository );
+    }
+
     if( wxDir::Exists( PATHS::GetStockDemosPath() ) )
     {
         fileMenu->Add( KICAD_MANAGER_ACTIONS::openDemoProject );
@@ -85,6 +90,10 @@ void KICAD_MANAGER_FRAME::doReCreateMenuBar()
 
     wxMenuItem* item = fileMenu->Add( openRecentMenu->Clone() );
 
+    fileMenu->AppendSeparator();
+    fileMenu->Add( KICAD_MANAGER_ACTIONS::newJobsetFile );
+    fileMenu->Add( KICAD_MANAGER_ACTIONS::openJobsetFile );
+
     // Add the file menu condition here since it needs the item ID for the submenu
     ACTION_CONDITIONS cond;
     cond.Enable( FILE_HISTORY::FileHistoryNotEmpty( fileHistory ) );
@@ -113,6 +122,15 @@ void KICAD_MANAGER_FRAME::doReCreateMenuBar()
                      ID_IMPORT_EAGLE_PROJECT,
                      BITMAPS::import_project );
 
+    importMenu->Add( _( "EasyEDA (JLCEDA) Std Backup..." ),
+                     _( "Import EasyEDA (JLCEDA) Standard schematic and board" ),
+                     ID_IMPORT_EASYEDA_PROJECT,
+                     BITMAPS::import_project );
+
+    importMenu->Add( _( "EasyEDA (JLCEDA) Pro Project..." ),
+                     _( "Import EasyEDA (JLCEDA) Professional schematic and board" ),
+                     ID_IMPORT_EASYEDAPRO_PROJECT, BITMAPS::import_project );
+
     fileMenu->Add( importMenu );
 
     fileMenu->AppendSeparator();
@@ -177,8 +195,8 @@ void KICAD_MANAGER_FRAME::doReCreateMenuBar()
 
     wxMenuItem* pcmMenuItem = toolsMenu->Add( KICAD_MANAGER_ACTIONS::showPluginManager );
 
-    if( KIPLATFORM::POLICY::GetPolicyState( POLICY_KEY_PCM )
-        == KIPLATFORM::POLICY::STATE::DISABLED )
+    if( KIPLATFORM::POLICY::GetPolicyBool( POLICY_KEY_PCM )
+        == KIPLATFORM::POLICY::PBOOL::DISABLED )
     {
         pcmMenuItem->Enable( false );
     }
@@ -196,13 +214,9 @@ void KICAD_MANAGER_FRAME::doReCreateMenuBar()
     prefsMenu->Add( ACTIONS::configurePaths );
     prefsMenu->Add( ACTIONS::showSymbolLibTable );
     prefsMenu->Add( ACTIONS::showFootprintLibTable );
-
-    // We can't use ACTIONS::showPreferences yet because wxWidgets moves this on
-    // Mac, and it needs the wxID_PREFERENCES id to find it.
-    prefsMenu->Add( _( "Preferences..." ) + wxS( "\tCtrl+," ),
-                    _( "Show preferences for all open tools" ),
-                    wxID_PREFERENCES,
-                    BITMAPS::preference );
+    if( ADVANCED_CFG::GetCfg().m_EnableDesignBlocks )
+        prefsMenu->Add( ACTIONS::showDesignBlockLibTable );
+    prefsMenu->Add( ACTIONS::openPreferences );
 
     prefsMenu->AppendSeparator();
     AddMenuLanguageList( prefsMenu, controlTool );
@@ -223,9 +237,9 @@ void KICAD_MANAGER_FRAME::doReCreateMenuBar()
 
 
 /**
- * @brief (Re)Create the horizontal toolbar
+ * @brief (Re)Create the left vertical toolbar
  */
-void KICAD_MANAGER_FRAME::RecreateBaseHToolbar()
+void KICAD_MANAGER_FRAME::RecreateBaseLeftToolbar()
 {
     if( m_mainToolBar )
     {
@@ -234,7 +248,7 @@ void KICAD_MANAGER_FRAME::RecreateBaseHToolbar()
     else
     {
         m_mainToolBar = new ACTION_TOOLBAR( this, ID_H_TOOLBAR, wxDefaultPosition, wxDefaultSize,
-                                            KICAD_AUI_TB_STYLE | wxAUI_TB_HORZ_LAYOUT );
+                                            KICAD_AUI_TB_STYLE | wxAUI_TB_VERTICAL );
         m_mainToolBar->SetAuiManager( &m_auimgr );
     }
 
@@ -244,19 +258,25 @@ void KICAD_MANAGER_FRAME::RecreateBaseHToolbar()
 
     m_mainToolBar->AddScaledSeparator( this );
     m_mainToolBar->AddTool( ID_SAVE_AND_ZIP_FILES, wxEmptyString,
-                            KiScaledBitmap( BITMAPS::zip, this ),
-                            _( "Archive all project files" ) );
+                            KiBitmapBundle( BITMAPS::zip ),
+                            KiDisabledBitmapBundle( BITMAPS::zip ),
+                            wxITEM_NORMAL,
+                            _( "Archive all project files" ),
+                            wxEmptyString, nullptr );
 
     m_mainToolBar->AddTool( ID_READ_ZIP_ARCHIVE, wxEmptyString,
-                            KiScaledBitmap( BITMAPS::unzip, this ),
-                            _( "Unarchive project files from zip archive" ) );
+                            KiBitmapBundle( BITMAPS::unzip ),
+                            KiDisabledBitmapBundle( BITMAPS::unzip ),
+                            wxITEM_NORMAL,
+                            _( "Unarchive project files from zip archive" ),
+                            wxEmptyString, nullptr );
 
     m_mainToolBar->AddScaledSeparator( this );
     m_mainToolBar->Add( ACTIONS::zoomRedraw );
 
     m_mainToolBar->AddScaledSeparator( this );
     m_mainToolBar->AddTool( ID_BROWSE_IN_FILE_EXPLORER, wxEmptyString,
-                            KiScaledBitmap( BITMAPS::directory_browser, this ),
+                            KiBitmapBundle( BITMAPS::directory_browser ),
 #ifdef __APPLE__
                             _( "Reveal project folder in Finder" ) );
 #else
@@ -264,5 +284,5 @@ void KICAD_MANAGER_FRAME::RecreateBaseHToolbar()
 #endif
 
     // Create m_mainToolBar
-    m_mainToolBar->Realize();
+    m_mainToolBar->KiRealize();
 }
