@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2007 Jean-Pierre Charras, jaen-pierre.charras@gipsa-lab.inpg.com
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,16 +26,20 @@
 #define CLASS_BOARD_H_
 
 #include <board_item_container.h>
+#include <board_stackup_manager/board_stackup.h>
+#include <component_class_manager.h>
+#include <embedded_files.h>
 #include <common.h> // Needed for stl hash extensions
 #include <convert_shape_list_to_polygon.h> // for OUTLINE_ERROR_HANDLER
 #include <hash.h>
 #include <layer_ids.h>
+#include <lset.h>
 #include <netinfo.h>
 #include <pcb_item_containers.h>
 #include <pcb_plot_params.h>
 #include <title_block.h>
 #include <tools/pcb_selection.h>
-#include <mutex>
+#include <shared_mutex>
 #include <list>
 
 class BOARD_DESIGN_SETTINGS;
@@ -51,6 +55,7 @@ class ZONE;
 class PCB_TRACK;
 class PAD;
 class PCB_GROUP;
+class PCB_GENERATOR;
 class PCB_MARKER;
 class MSG_PANEL_ITEM;
 class NETLIST;
@@ -60,6 +65,11 @@ class CONNECTIVITY_DATA;
 class COMPONENT;
 class PROJECT;
 class PROGRESS_REPORTER;
+namespace KIFONT
+{
+    class OUTLINE_FONT;
+}
+
 struct ISOLATED_ISLANDS;
 
 // The default value for m_outlinesChainingEpsilon to convert a board outlines to polygons
@@ -150,7 +160,10 @@ enum LAYER_T
     LT_SIGNAL,
     LT_POWER,
     LT_MIXED,
-    LT_JUMPER
+    LT_JUMPER,
+    LT_AUX,
+    LT_FRONT,
+    LT_BACK
 };
 
 
@@ -171,6 +184,7 @@ struct LAYER
         m_number  = 0;
         m_name.clear();
         m_userName.clear();
+        m_opposite = m_number;
     }
 
     /*
@@ -189,6 +203,7 @@ struct LAYER
     LAYER_T     m_type;      ///< The type of the layer. @see #LAYER_T
     bool        m_visible;
     int         m_number;    ///< The layer ID. @see PCB_LAYER_ID
+    int         m_opposite;  ///< Similar layer on opposite side of the board, if any.
 
     /**
      * Convert a #LAYER_T enum to a string representation of the layer type.
@@ -250,8 +265,18 @@ public:
     virtual void OnBoardItemChanged( BOARD& aBoard, BOARD_ITEM* aBoardItem ) { }
     virtual void OnBoardItemsChanged( BOARD& aBoard, std::vector<BOARD_ITEM*>& aBoardItem ) { }
     virtual void OnBoardHighlightNetChanged( BOARD& aBoard ) { }
+    virtual void OnBoardRatsnestChanged( BOARD& aBoard ) { }
+    virtual void OnBoardCompositeUpdate( BOARD& aBoard, std::vector<BOARD_ITEM*>& aAddedItems,
+                                         std::vector<BOARD_ITEM*>& aRemovedItems,
+                                         std::vector<BOARD_ITEM*>& aChangedItems )
+    {
+    }
 };
 
+/**
+ * Set of BOARD_ITEMs ordered by UUID.
+ */
+typedef std::set<BOARD_ITEM*, CompareByUuid> BOARD_ITEM_SET;
 
 /**
  * Flags to specify how the board is being used.
@@ -266,7 +291,7 @@ enum class BOARD_USE
 /**
  * Information pertinent to a Pcbnew printed circuit board.
  */
-class BOARD : public BOARD_ITEM_CONTAINER
+class BOARD : public BOARD_ITEM_CONTAINER, public EMBEDDED_FILES
 {
 public:
     static inline bool ClassOf( const EDA_ITEM* aItem )
@@ -306,22 +331,26 @@ public:
 
     const wxString &GetFileName() const { return m_fileName; }
 
-    TRACKS& Tracks() { return m_tracks; }
     const TRACKS& Tracks() const { return m_tracks; }
 
-    FOOTPRINTS& Footprints() { return m_footprints; }
     const FOOTPRINTS& Footprints() const { return m_footprints; }
 
-    DRAWINGS& Drawings() { return m_drawings; }
     const DRAWINGS& Drawings() const { return m_drawings; }
 
-    ZONES& Zones() { return m_zones; }
     const ZONES& Zones() const { return m_zones; }
 
-    MARKERS& Markers() { return m_markers; }
+    const GENERATORS& Generators() const { return m_generators; }
+
     const MARKERS& Markers() const { return m_markers; }
 
-    const std::set<BOARD_ITEM*> GetItemSet();
+    // SWIG requires non-const accessors for some reason to make the custom iterators in board.i
+    // work.  It would be good to remove this if we can figure out how to fix that.
+#ifdef SWIG
+    DRAWINGS& Drawings() { return m_drawings; }
+    TRACKS& Tracks() { return m_tracks; }
+#endif
+
+    const BOARD_ITEM_SET GetItemSet();
 
     /**
      * The groups must maintain the following invariants. These are checked by
@@ -331,7 +360,6 @@ public:
      *   - If a group specifies a name, it must be unique
      *   - The graph of groups containing subgroups must be cyclic.
      */
-    GROUPS& Groups() { return m_groups; }
     const GROUPS& Groups() const { return m_groups; }
 
     const std::vector<BOARD_CONNECTED_ITEM*> AllConnectedItems();
@@ -367,6 +395,9 @@ public:
 
     void Move( const VECTOR2I& aMoveVector ) override;
 
+    void RunOnDescendants( const std::function<void ( BOARD_ITEM* )>& aFunction,
+                           int aDepth = 0 ) const override;
+
     void SetFileFormatVersionAtLoad( int aVersion ) { m_fileFormatVersionAtLoad = aVersion; }
     int GetFileFormatVersionAtLoad() const { return m_fileFormatVersionAtLoad; }
 
@@ -380,6 +411,19 @@ public:
     ///< @copydoc BOARD_ITEM_CONTAINER::Remove()
     void Remove( BOARD_ITEM* aBoardItem, REMOVE_MODE aMode = REMOVE_MODE::NORMAL ) override;
 
+    /**
+     * An efficient way to remove all items of a certain type from the board.
+     * Because of how items are stored, this method has some limitations in order to preserve
+     * performance: tracks, vias, and arcs are all removed together by PCB_TRACE_T, and all graphics
+     * and text object types are removed together by PCB_SHAPE_T.  If you need something more
+     * granular than that, use BOARD::Remove.
+     * @param aTypes is a list of one or more types to remove, or leave default to remove all
+     */
+    void RemoveAll( std::initializer_list<KICAD_T> aTypes = { PCB_NETINFO_T, PCB_MARKER_T,
+                                                              PCB_GROUP_T, PCB_ZONE_T,
+                                                              PCB_GENERATOR_T, PCB_FOOTPRINT_T,
+                                                              PCB_TRACE_T, PCB_SHAPE_T } );
+
     /**
      * Must be used if Add() is used using a BULK_x ADD_MODE to generate a change event for
      * listeners.
@@ -392,6 +436,13 @@ public:
      */
     void FinalizeBulkRemove( std::vector<BOARD_ITEM*>& aRemovedItems );
 
+    /**
+     * After loading a file from disk, the footprints do not yet contain the full
+     * data for their embedded files, only a reference.  This iterates over all footprints
+     * in the board and updates them with the full embedded data.
+    */
+    void FixupEmbeddedData();
+
     void CacheTriangulation( PROGRESS_REPORTER* aReporter = nullptr,
                              const std::vector<ZONE*>& aZones = {} );
 
@@ -468,6 +519,11 @@ public:
      */
     std::vector<PCB_MARKER*> ResolveDRCExclusions( bool aCreateMarkers );
 
+    /**
+     * Scan existing markers and record data from any that are Excluded.
+     */
+    void RecordDRCExclusions();
+
     /**
      * Update the visibility flags on the current unconnected ratsnest lines.
      */
@@ -522,6 +578,17 @@ public:
     int  GetCopperLayerCount() const;
     void SetCopperLayerCount( int aCount );
 
+    int GetUserDefinedLayerCount() const;
+    void SetUserDefinedLayerCount( int aCount );
+
+    /**
+     * @return The copper layer max PCB_LAYER_ID in the BOARD.
+     * similar to GetCopperLayerCount(), but returns the max PCB_LAYER_ID
+     */
+    PCB_LAYER_ID  GetCopperLayerStackMaxId() const;
+
+    PCB_LAYER_ID FlipLayer( PCB_LAYER_ID aLayer ) const;
+
     int LayerDepth( PCB_LAYER_ID aStartLayer, PCB_LAYER_ID aEndLayer ) const;
 
     /**
@@ -628,12 +695,7 @@ public:
      */
     BOARD_DESIGN_SETTINGS& GetDesignSettings() const;
 
-    const ZONE_SETTINGS& GetZoneSettings() const override;
-
-    void SetZoneSettings( const ZONE_SETTINGS& aSettings ) override;
-
-    bool GetTentVias() const            { return !m_plotOptions.GetPlotViaOnMaskLayer(); }
-    void SetTentVias( bool aFlag )      { m_plotOptions.SetPlotViaOnMaskLayer( !aFlag ); }
+    BOARD_STACKUP GetStackupOrDefault() const;
 
     const PAGE_INFO& GetPageSettings() const                { return m_paper; }
     void SetPageSettings( const PAGE_INFO& aPageSettings )  { m_paper = aPageSettings; }
@@ -645,7 +707,7 @@ public:
     const TITLE_BLOCK& GetTitleBlock() const                { return m_titles; }
     void SetTitleBlock( const TITLE_BLOCK& aTitleBlock )    { m_titles = aTitleBlock; }
 
-    wxString GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const override;
+    wxString GetItemDescription( UNITS_PROVIDER* aUnitsProvider, bool aFull ) const override;
 
     EDA_UNITS GetUserUnits()                                { return m_userUnits; }
     void SetUserUnits( EDA_UNITS aUnits )                   { m_userUnits = aUnits; }
@@ -668,11 +730,15 @@ public:
      * @param aAllowUseArcsInPolygons = an optional option to allow adding arcs in
      *  SHAPE_LINE_CHAIN polylines/polygons when building outlines from aShapeList
      *  This is mainly for export to STEP files
+     * @param aIncludeNPTHAsOutlines = an optional option to include NPTH pad holes
+     * in board outlines. These holes can be seen like holes created by closed shapes
+     * drawn on edge cut layer inside the board main outline.
      * @return true if success, false if a contour is not valid
      */
     bool GetBoardPolygonOutlines( SHAPE_POLY_SET& aOutlines,
                                   OUTLINE_ERROR_HANDLER* aErrorHandler = nullptr,
-                                  bool aAllowUseArcsInPolygons = false );
+                                  bool aAllowUseArcsInPolygons = false,
+                                  bool aIncludeNPTHAsOutlines = false );
 
     /**
      * @return a epsilon value that is the max distance between 2 points to see them
@@ -699,7 +765,7 @@ public:
     /**
      * Return the ID of a layer.
      */
-    const PCB_LAYER_ID GetLayerID( const wxString& aLayerName ) const;
+    PCB_LAYER_ID GetLayerID( const wxString& aLayerName ) const;
 
     /**
      * Return the name of a \a aLayer.
@@ -797,14 +863,30 @@ public:
      */
     NETINFO_ITEM* FindNet( const wxString& aNetname ) const;
 
+    /**
+     * Fetch the coupled netname for a given net.
+     *
+     * This accepts netnames suffixed by 'P', 'N', '+', '-', as well as additional numbers and
+     * underscores following the suffix.  So NET_P_123 is a valid positive net name matched to
+     * NET_N_123.
+     *
+     * @return the polarity of the given net (or 0 if it is not a diffpair net).
+     */
+    int MatchDpSuffix( const wxString& aNetName, wxString& aComplementNet );
+
+    /**
+     * @return the coupled net for a given net.  If not a diffpair, nullptr is returned.
+     */
+    NETINFO_ITEM* DpCoupledNet( const NETINFO_ITEM* aNet );
+
     const NETINFO_LIST& GetNetInfo() const
     {
         return m_NetInfo;
     }
 
-    NETINFO_LIST& GetNetInfo()
+    void RemoveUnusedNets( BOARD_COMMIT* aCommit )
     {
-        return m_NetInfo;
+        m_NetInfo.RemoveUnusedNets( aCommit );
     }
 
 #ifndef SWIG
@@ -913,6 +995,16 @@ public:
      */
     void SynchronizeProperties();
 
+    /**
+     * Return the Similarity.  Because we compare board to board, we just return 1.0 here
+    */
+    double Similarity( const BOARD_ITEM& aOther ) const override
+    {
+        return 1.0;
+    }
+
+    bool operator==( const BOARD_ITEM& aOther ) const override;
+
     wxString GetClass() const override
     {
         return wxT( "BOARD" );
@@ -1086,12 +1178,19 @@ public:
     FOOTPRINT* GetFootprint( const VECTOR2I& aPosition, PCB_LAYER_ID aActiveLayer,
                              bool aVisibleOnly, bool aIgnoreLocked = false ) const;
 
+    /**
+     * Returns the maximum clearance value for any object on the board.  This considers
+     * the clearances from board design settings as well as embedded clearances in footprints,
+     * pads and zones.  Includes electrical, physical, hole and edge clearances.
+    */
+    int GetMaxClearanceValue() const;
+
     /**
      * Map all nets in the given board to nets with the same name (if any) in the destination
-     * board.  This allows us to share layouts which came from the same hierarchical sheet in
-     * the schematic.
+     * board.  If there are missing nets in the destination board, they will be created.
+     *
      */
-    void MapNets( const BOARD* aDestBoard );
+    void MapNets( BOARD* aDestBoard );
 
     void SanitizeNetcodes();
 
@@ -1128,6 +1227,19 @@ public:
       */
     void OnItemsChanged( std::vector<BOARD_ITEM*>& aItems );
 
+    /**
+      * Notify the board and its listeners that items on the board have
+      * been modified in a composite operations
+      */
+    void OnItemsCompositeUpdate( std::vector<BOARD_ITEM*>& aAddedItems,
+                                 std::vector<BOARD_ITEM*>& aRemovedItems,
+                                 std::vector<BOARD_ITEM*>& aChangedItems );
+
+    /**
+     * Notify the board and its listeners that the ratsnest has been recomputed.
+     */
+    void OnRatsnestChanged();
+
     /**
      * Consistency check of internal m_groups structure.
      *
@@ -1157,6 +1269,27 @@ public:
      */
     GroupLegalOpsField GroupLegalOps( const PCB_SELECTION& selection ) const;
 
+    bool LegacyTeardrops() const { return m_legacyTeardrops; }
+    void SetLegacyTeardrops( bool aFlag ) { m_legacyTeardrops = aFlag; }
+
+    EMBEDDED_FILES* GetEmbeddedFiles() override;
+    const EMBEDDED_FILES* GetEmbeddedFiles() const;
+
+    /**
+     * Get the list of all outline fonts used in the board
+     */
+    std::set<KIFONT::OUTLINE_FONT*> GetFonts() const override;
+
+    /**
+     * Finds all fonts used in the board and embeds them in the file if permissions allow
+    */
+    void EmbedFonts() override;
+
+    /**
+     * Gets the component class manager
+     */
+    COMPONENT_CLASS_MANAGER& GetComponentClassManager() { return m_componentClassManager; }
+
     // --------- Item order comparators ---------
 
     struct cmp_items
@@ -1169,8 +1302,9 @@ public:
         bool operator()( const BOARD_ITEM* aFirst, const BOARD_ITEM* aSecond ) const;
     };
 
+public:
     // ------------ Run-time caches -------------
-    std::mutex                                            m_CachesMutex;
+    mutable std::shared_mutex                             m_CachesMutex;
     std::unordered_map<PTR_PTR_CACHE_KEY, bool>           m_IntersectsCourtyardCache;
     std::unordered_map<PTR_PTR_CACHE_KEY, bool>           m_IntersectsFCourtyardCache;
     std::unordered_map<PTR_PTR_CACHE_KEY, bool>           m_IntersectsBCourtyardCache;
@@ -1178,15 +1312,16 @@ public:
     std::unordered_map<PTR_PTR_LAYER_CACHE_KEY, bool>     m_EnclosedByAreaCache;
     std::unordered_map< wxString, LSET >                  m_LayerExpressionCache;
     std::unordered_map<ZONE*, std::unique_ptr<DRC_RTREE>> m_CopperZoneRTreeCache;
-    std::unique_ptr<DRC_RTREE>                            m_CopperItemRTreeCache;
+    std::shared_ptr<DRC_RTREE>                            m_CopperItemRTreeCache;
     mutable std::unordered_map<const ZONE*, BOX2I>        m_ZoneBBoxCache;
+    mutable std::optional<int>                            m_maxClearanceValue;
 
     // ------------ DRC caches -------------
     std::vector<ZONE*>    m_DRCZones;
     std::vector<ZONE*>    m_DRCCopperZones;
     int                   m_DRCMaxClearance;
     int                   m_DRCMaxPhysicalClearance;
-    ZONE*                 m_SolderMask;
+    ZONE*                 m_SolderMaskBridges;  // A container to build bridges on solder mask layers
     std::map<ZONE*, std::map<PCB_LAYER_ID, ISOLATED_ISLANDS>> m_ZoneIsolatedIslandsMap;
 
 private:
@@ -1203,25 +1338,34 @@ private:
             ( l->*aFunc )( std::forward<Args>( args )... );
     }
 
+    // Refresh user layer opposites.
+    void recalcOpposites();
+
     friend class PCB_EDIT_FRAME;
 
-
+private:
     /// the max distance between 2 end point to see them connected when building the board outlines
-    int m_outlinesChainingEpsilon;
+    int                 m_outlinesChainingEpsilon;
 
     /// What is this board being used for
     BOARD_USE           m_boardUse;
     int                 m_timeStamp;                // actually a modification counter
 
     wxString            m_fileName;
+
+    // These containers only have const accessors and must only be modified by Add()/Remove()
     MARKERS             m_markers;
     DRAWINGS            m_drawings;
     FOOTPRINTS          m_footprints;
     TRACKS              m_tracks;
     GROUPS              m_groups;
     ZONES               m_zones;
+    GENERATORS          m_generators;
 
-    LAYER               m_layers[PCB_LAYER_ID_COUNT];
+    // Cache for fast access to items in the containers above by KIID, including children
+    std::unordered_map<KIID, BOARD_ITEM*> m_itemByIdCache;
+
+    std::map<int, LAYER> m_layers;                  // layer data
 
     HIGH_LIGHT_INFO     m_highLight;                // current high light data
     HIGH_LIGHT_INFO     m_highLightPrevious;        // a previously stored high light data
@@ -1250,9 +1394,19 @@ private:
      */
     std::unique_ptr<BOARD_DESIGN_SETTINGS> m_designSettings;
 
+    /**
+     * Teardrops in 7.0 were applied as a post-processing step (rather than from pad and via
+     * properties).  If this flag is set, then auto-teardrop-generation will be disabled.
+     */
+    bool                         m_legacyTeardrops = false;
+
     NETINFO_LIST                 m_NetInfo;         // net info list (name, design constraints...
 
     std::vector<BOARD_LISTENER*> m_listeners;
+
+    bool                         m_embedFonts;
+
+    COMPONENT_CLASS_MANAGER m_componentClassManager;
 };
 
 
