@@ -2,7 +2,7 @@
  * This program source code file is part of KICAD, a free EDA CAD application.
  *
  * Copyright (C) 2017 CERN
- * Copyright (C) 2018-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
  * This program is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@
  */
 
 #ifdef PROFILE
-#include <profile.h>
+#include <core/profile.h>
 #endif
 
 #include <algorithm>
@@ -44,18 +44,21 @@
 #include <trigo.h>
 #include <drc/drc_rtree.h>
 
-CONNECTIVITY_DATA::CONNECTIVITY_DATA()
+CONNECTIVITY_DATA::CONNECTIVITY_DATA() :
+        m_skipRatsnestUpdate( false )
 {
-    m_connAlgo.reset( new CN_CONNECTIVITY_ALGO );
+    m_connAlgo.reset( new CN_CONNECTIVITY_ALGO( this ) );
     m_progressReporter = nullptr;
     m_fromToCache.reset( new FROM_TO_CACHE );
 }
 
 
-CONNECTIVITY_DATA::CONNECTIVITY_DATA( const std::vector<BOARD_ITEM*>& aItems, bool aSkipRatsnest )
-    : m_skipRatsnest( aSkipRatsnest )
+CONNECTIVITY_DATA::CONNECTIVITY_DATA( std::shared_ptr<CONNECTIVITY_DATA> aGlobalConnectivity,
+                                      const std::vector<BOARD_ITEM*>& aLocalItems,
+                                      bool aSkipRatsnestUpdate ) :
+        m_skipRatsnestUpdate( aSkipRatsnestUpdate )
 {
-    Build( aItems );
+    Build( aGlobalConnectivity, aLocalItems );
     m_progressReporter = nullptr;
     m_fromToCache.reset( new FROM_TO_CACHE );
 }
@@ -107,20 +110,18 @@ bool CONNECTIVITY_DATA::Build( BOARD* aBoard, PROGRESS_REPORTER* aReporter )
         aReporter->KeepRefreshing( false );
     }
 
-    std::shared_ptr<NET_SETTINGS>& netSettings = aBoard->GetDesignSettings().m_NetSettings;
+    for( auto net : m_nets )
+        if ( net )
+            delete net;
 
-    m_connAlgo.reset( new CN_CONNECTIVITY_ALGO );
+    m_nets.clear();
+
+    m_connAlgo.reset( new CN_CONNECTIVITY_ALGO( this ) );
     m_connAlgo->Build( aBoard, aReporter );
 
-    m_netclassMap.clear();
+    m_netSettings = aBoard->GetDesignSettings().m_NetSettings;
 
-    for( NETINFO_ITEM* net : aBoard->GetNetInfo() )
-    {
-        net->SetNetClass( netSettings->GetEffectiveNetClass( net->GetNetname() ) );
-
-        if( net->GetNetClass()->GetName() != NETCLASS::Default )
-            m_netclassMap[ net->GetNetCode() ] = net->GetNetClass()->GetName();
-    }
+    RefreshNetcodeMap( aBoard );
 
     if( aReporter )
     {
@@ -140,15 +141,25 @@ bool CONNECTIVITY_DATA::Build( BOARD* aBoard, PROGRESS_REPORTER* aReporter )
 }
 
 
-void CONNECTIVITY_DATA::Build( const std::vector<BOARD_ITEM*>& aItems )
+void CONNECTIVITY_DATA::RefreshNetcodeMap( BOARD* aBoard )
+{
+    m_netcodeMap.clear();
+
+    for( NETINFO_ITEM* net : aBoard->GetNetInfo() )
+        m_netcodeMap[net->GetNetCode()] = net->GetNetname();
+}
+
+
+void CONNECTIVITY_DATA::Build( std::shared_ptr<CONNECTIVITY_DATA>& aGlobalConnectivity,
+                               const std::vector<BOARD_ITEM*>& aLocalItems )
 {
     std::unique_lock<KISPINLOCK> lock( m_lock, std::try_to_lock );
 
     if( !lock )
         return;
 
-    m_connAlgo.reset( new CN_CONNECTIVITY_ALGO );
-    m_connAlgo->LocalBuild( aItems );
+    m_connAlgo.reset( new CN_CONNECTIVITY_ALGO( this ) );
+    m_connAlgo->LocalBuild( aGlobalConnectivity, aLocalItems );
 
     internalRecalculateRatsnest();
 }
@@ -244,14 +255,11 @@ void CONNECTIVITY_DATA::internalRecalculateRatsnest( BOARD_COMMIT* aCommit  )
 
     const std::vector<std::shared_ptr<CN_CLUSTER>>& clusters = m_connAlgo->GetClusters();
 
-    int dirtyNets = 0;
-
     for( int net = 0; net < lastNet; net++ )
     {
         if( m_connAlgo->IsNetDirty( net ) )
         {
             m_nets[net]->Clear();
-            dirtyNets++;
         }
     }
 
@@ -272,7 +280,7 @@ void CONNECTIVITY_DATA::internalRecalculateRatsnest( BOARD_COMMIT* aCommit  )
 
     m_connAlgo->ClearDirtyFlags();
 
-    if( !m_skipRatsnest )
+    if( !m_skipRatsnestUpdate )
         updateRatsnest();
 }
 
@@ -381,6 +389,10 @@ void CONNECTIVITY_DATA::ComputeLocalRatsnest( const std::vector<BOARD_ITEM*>& aI
     {
         const std::shared_ptr<const CN_ANCHOR>& nodeA = edge.GetSourceNode();
         const std::shared_ptr<const CN_ANCHOR>& nodeB = edge.GetTargetNode();
+
+        if( !nodeA || nodeA->Dirty() || !nodeB || nodeB->Dirty() )
+            continue;
+
         RN_DYNAMIC_LINE l;
 
         // Use the parents' positions
@@ -434,8 +446,15 @@ bool CONNECTIVITY_DATA::IsConnectedOnLayer( const BOARD_CONNECTED_ITEM *aItem, i
         {
             CN_ZONE_LAYER* zoneLayer = dynamic_cast<CN_ZONE_LAYER*>( connected );
 
+            // lyIdx is compatible with StartLayer() and EndLayer() notation in CN_ITEM
+            // items, where B_Cu is set to INT_MAX (std::numeric_limits<int>::max())
+            int lyIdx = aLayer;
+
+            if( aLayer == B_Cu )
+                lyIdx = std::numeric_limits<int>::max();
+
             if( connected->Valid()
-                    && connected->Layers().Overlaps( aLayer )
+                    && connected->StartLayer() <= lyIdx && connected->EndLayer() >= lyIdx
                     && matchType( connected->Parent()->Type() )
                     && connected->Net() == aItem->GetNetCode() )
             {
@@ -447,8 +466,10 @@ bool CONNECTIVITY_DATA::IsConnectedOnLayer( const BOARD_CONNECTED_ITEM *aItem, i
 
                     if( zone->IsFilled() )
                     {
-                        const SHAPE_POLY_SET*   zoneFill = zone->GetFill( ToLAYER_ID( aLayer ) );
-                        const SHAPE_LINE_CHAIN& padHull = pad->GetEffectivePolygon( ERROR_INSIDE )->Outline( 0 );
+                        PCB_LAYER_ID pcbLayer = ToLAYER_ID( aLayer );
+                        const SHAPE_POLY_SET*   zoneFill = zone->GetFill( pcbLayer );
+                        const SHAPE_LINE_CHAIN& padHull =
+                            pad->GetEffectivePolygon( pcbLayer, ERROR_INSIDE )->Outline( 0 );
 
                         for( const VECTOR2I& pt : zoneFill->COutline( islandIdx ).CPoints() )
                         {
@@ -471,8 +492,9 @@ bool CONNECTIVITY_DATA::IsConnectedOnLayer( const BOARD_CONNECTED_ITEM *aItem, i
 
                     if( zone->IsFilled() )
                     {
-                        const SHAPE_POLY_SET* zoneFill = zone->GetFill( ToLAYER_ID( aLayer ) );
-                        SHAPE_CIRCLE          viaHull( via->GetCenter(), via->GetWidth() / 2 );
+                        PCB_LAYER_ID lyr = ToLAYER_ID( aLayer );
+                        const SHAPE_POLY_SET* zoneFill = zone->GetFill( lyr );
+                        SHAPE_CIRCLE          viaHull( via->GetCenter(), via->GetWidth( lyr ) / 2 );
 
                         for( const VECTOR2I& pt : zoneFill->COutline( islandIdx ).CPoints() )
                         {
@@ -526,7 +548,7 @@ void CONNECTIVITY_DATA::ClearRatsnest()
 
 const std::vector<BOARD_CONNECTED_ITEM*>
 CONNECTIVITY_DATA::GetConnectedItems( const BOARD_CONNECTED_ITEM *aItem,
-                                      const std::initializer_list<KICAD_T>& aTypes,
+                                      const std::vector<KICAD_T>& aTypes,
                                       bool aIgnoreNetcodes ) const
 {
     std::vector<BOARD_CONNECTED_ITEM*> rv;
@@ -556,7 +578,6 @@ CONNECTIVITY_DATA::GetConnectedItems( const BOARD_CONNECTED_ITEM *aItem,
 }
 
 
-// We don't use std::initializer_list<KICAD_T>& to allow the use from Python
 const std::vector<BOARD_CONNECTED_ITEM*>
 CONNECTIVITY_DATA::GetNetItems( int aNetCode, const std::vector<KICAD_T>& aTypes ) const
 {
@@ -638,6 +659,28 @@ const
 }
 
 
+void CONNECTIVITY_DATA::GetConnectedPadsAndVias( const BOARD_CONNECTED_ITEM* aItem,
+                                                 std::vector<PAD*>* pads,
+                                                 std::vector<PCB_VIA*>* vias )
+{
+    for( CN_ITEM* citem : m_connAlgo->ItemEntry( aItem ).GetItems() )
+    {
+        for( CN_ITEM* connected : citem->ConnectedItems() )
+        {
+            if( connected->Valid() )
+            {
+                BOARD_CONNECTED_ITEM* parent = connected->Parent();
+
+                if( parent->Type() == PCB_PAD_T )
+                    pads->push_back( static_cast<PAD*>( parent ) );
+                else if( parent->Type() == PCB_VIA_T )
+                    vias->push_back( static_cast<PCB_VIA*>( parent ) );
+            }
+        }
+    }
+}
+
+
 unsigned int CONNECTIVITY_DATA::GetNodeCount( int aNet ) const
 {
     int sum = 0;
@@ -700,12 +743,12 @@ static int getMinDist( BOARD_CONNECTED_ITEM* aItem, const VECTOR2I& aPoint )
     {
         PCB_TRACK* track = static_cast<PCB_TRACK*>( aItem );
 
-        return std::min( GetLineLength( track->GetStart(), aPoint ),
-                         GetLineLength( track->GetEnd(), aPoint ) );
+        return std::min( track->GetStart().Distance(aPoint ),
+                         track->GetEnd().Distance( aPoint ) );
     }
 
     default:
-        return GetLineLength( aItem->GetPosition(), aPoint );
+        return aItem->GetPosition().Distance( aPoint );
     }
 }
 
@@ -851,7 +894,7 @@ bool CONNECTIVITY_DATA::TestTrackEndpointDangling( PCB_TRACK* aTrack, bool aIgno
 const std::vector<BOARD_CONNECTED_ITEM*>
 CONNECTIVITY_DATA::GetConnectedItemsAtAnchor( const BOARD_CONNECTED_ITEM* aItem,
                                               const VECTOR2I& aAnchor,
-                                              const std::initializer_list<KICAD_T>& aTypes,
+                                              const std::vector<KICAD_T>& aTypes,
                                               const int& aMaxError ) const
 {
     CN_CONNECTIVITY_ALGO::ITEM_MAP_ENTRY& entry = m_connAlgo->ItemEntry( aItem );
@@ -888,9 +931,7 @@ CONNECTIVITY_DATA::GetConnectedItemsAtAnchor( const BOARD_CONNECTED_ITEM* aItem,
 RN_NET* CONNECTIVITY_DATA::GetRatsnestForNet( int aNet )
 {
     if ( aNet < 0 || aNet >= (int) m_nets.size() )
-    {
         return nullptr;
-    }
 
     return m_nets[ aNet ];
 }
@@ -903,10 +944,18 @@ void CONNECTIVITY_DATA::MarkItemNetAsDirty( BOARD_ITEM *aItem )
         for( PAD* pad : static_cast<FOOTPRINT*>( aItem )->Pads() )
             m_connAlgo->MarkNetAsDirty( pad->GetNetCode() );
     }
+
     if (aItem->IsConnected() )
-    {
         m_connAlgo->MarkNetAsDirty( static_cast<BOARD_CONNECTED_ITEM*>( aItem )->GetNetCode() );
-    }
+}
+
+
+void CONNECTIVITY_DATA::RemoveInvalidRefs()
+{
+    m_connAlgo->RemoveInvalidRefs();
+
+    for( RN_NET* rnNet : m_nets )
+        rnNet->RemoveInvalidRefs();
 }
 
 
@@ -917,7 +966,8 @@ void CONNECTIVITY_DATA::SetProgressReporter( PROGRESS_REPORTER* aReporter )
 }
 
 
-const std::vector<CN_EDGE> CONNECTIVITY_DATA::GetRatsnestForItems( std::vector<BOARD_ITEM*> aItems )
+const std::vector<CN_EDGE>
+CONNECTIVITY_DATA::GetRatsnestForItems( const std::vector<BOARD_ITEM*>& aItems )
 {
     std::set<int> nets;
     std::vector<CN_EDGE> edges;
@@ -935,8 +985,10 @@ const std::vector<CN_EDGE> CONNECTIVITY_DATA::GetRatsnestForItems( std::vector<B
                 item_set.insert( pad );
             }
         }
-        else if( auto conn_item = dyn_cast<BOARD_CONNECTED_ITEM*>( item ) )
+        else if( item->IsConnected() )
         {
+            BOARD_CONNECTED_ITEM* conn_item = static_cast<BOARD_CONNECTED_ITEM*>( item );
+
             item_set.insert( conn_item );
             nets.insert( conn_item->GetNetCode() );
         }
@@ -946,11 +998,17 @@ const std::vector<CN_EDGE> CONNECTIVITY_DATA::GetRatsnestForItems( std::vector<B
     {
         RN_NET* net = GetRatsnestForNet( netcode );
 
+        if( !net )
+            continue;
+
         for( const CN_EDGE& edge : net->GetEdges() )
         {
             std::shared_ptr<const CN_ANCHOR> srcNode = edge.GetSourceNode();
             std::shared_ptr<const CN_ANCHOR> dstNode = edge.GetTargetNode();
 
+            if( !srcNode || srcNode->Dirty() || !dstNode || dstNode->Dirty() )
+                continue;
+
             BOARD_CONNECTED_ITEM* srcParent = srcNode->Parent();
             BOARD_CONNECTED_ITEM* dstParent = dstNode->Parent();
 
@@ -971,8 +1029,17 @@ const std::vector<CN_EDGE> CONNECTIVITY_DATA::GetRatsnestForPad( const PAD* aPad
     std::vector<CN_EDGE> edges;
     RN_NET* net = GetRatsnestForNet( aPad->GetNetCode() );
 
+    if( !net )
+        return edges;
+
     for( const CN_EDGE& edge : net->GetEdges() )
     {
+        if( !edge.GetSourceNode() || edge.GetSourceNode()->Dirty() )
+            continue;
+
+        if( !edge.GetTargetNode() || edge.GetTargetNode()->Dirty() )
+            continue;
+
         if( edge.GetSourceNode()->Parent() == aPad || edge.GetTargetNode()->Parent() == aPad )
             edges.push_back( edge );
     }
@@ -981,7 +1048,8 @@ const std::vector<CN_EDGE> CONNECTIVITY_DATA::GetRatsnestForPad( const PAD* aPad
 }
 
 
-const std::vector<CN_EDGE> CONNECTIVITY_DATA::GetRatsnestForComponent( FOOTPRINT* aComponent, bool aSkipInternalConnections )
+const std::vector<CN_EDGE> CONNECTIVITY_DATA::GetRatsnestForComponent( FOOTPRINT* aComponent,
+                                                                       bool aSkipInternalConnections )
 {
     std::set<int> nets;
     std::set<const PAD*> pads;
@@ -993,14 +1061,20 @@ const std::vector<CN_EDGE> CONNECTIVITY_DATA::GetRatsnestForComponent( FOOTPRINT
         pads.insert( pad );
     }
 
-    for( const auto& netcode : nets )
+    for( int netcode : nets )
     {
         RN_NET* net = GetRatsnestForNet( netcode );
 
+        if( !net )
+            continue;
+
         for( const CN_EDGE& edge : net->GetEdges() )
         {
-            auto srcNode = edge.GetSourceNode();
-            auto dstNode = edge.GetTargetNode();
+            const std::shared_ptr<const CN_ANCHOR>& srcNode = edge.GetSourceNode();
+            const std::shared_ptr<const CN_ANCHOR>& dstNode = edge.GetTargetNode();
+
+            if( !srcNode || srcNode->Dirty() || !dstNode || dstNode->Dirty() )
+                continue;
 
             const PAD* srcParent = static_cast<const PAD*>( srcNode->Parent() );
             const PAD* dstParent = static_cast<const PAD*>( dstNode->Parent() );
@@ -1009,13 +1083,9 @@ const std::vector<CN_EDGE> CONNECTIVITY_DATA::GetRatsnestForComponent( FOOTPRINT
             bool dstFound = ( pads.find(dstParent) != pads.end() );
 
             if ( srcFound && dstFound && !aSkipInternalConnections )
-            {
                 edges.push_back( edge );
-            }
             else if ( srcFound || dstFound )
-            {
                 edges.push_back( edge );
-            }
         }
     }
 
@@ -1023,3 +1093,10 @@ const std::vector<CN_EDGE> CONNECTIVITY_DATA::GetRatsnestForComponent( FOOTPRINT
 }
 
 
+const NET_SETTINGS* CONNECTIVITY_DATA::GetNetSettings() const
+{
+    if( std::shared_ptr<NET_SETTINGS> netSettings = m_netSettings.lock() )
+        return netSettings.get();
+    else
+        return nullptr;
+}
