@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2020 Jon Evans <jon@craftyjon.com>
- * Copyright (C) 2020-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -46,7 +46,8 @@ DIALOG_DIMENSION_PROPERTIES::DIALOG_DIMENSION_PROPERTIES( PCB_BASE_EDIT_FRAME* a
         m_orientation( aParent, m_lblTextOrientation, m_cbTextOrientation, nullptr ),
         m_lineThickness( aParent, m_lblLineThickness, m_txtLineThickness, m_lblLineThicknessUnits ),
         m_arrowLength( aParent, m_lblArrowLength, m_txtArrowLength, m_lblArrowLengthUnits ),
-        m_extensionOffset( aParent, m_lblExtensionOffset, m_txtExtensionOffset, m_lblExtensionOffsetUnits )
+        m_extensionOffset( aParent, m_lblExtensionOffset, m_txtExtensionOffset, m_lblExtensionOffsetUnits ),
+        m_extensionOvershoot( aParent, m_lblExtensionOvershoot, m_txtExtensionOvershoot, m_lblExtensionOvershootUnits )
 {
     wxASSERT( BaseType( aItem->Type() ) == PCB_DIMENSION_T );
     m_dimension = static_cast<PCB_DIMENSION_BASE*>( aItem );
@@ -56,7 +57,6 @@ DIALOG_DIMENSION_PROPERTIES::DIALOG_DIMENSION_PROPERTIES( PCB_BASE_EDIT_FRAME* a
     switch( m_dimension->Type() )
     {
     case PCB_DIM_LEADER_T:
-    case PCB_FP_DIM_LEADER_T:
         // Hide the main format controls and keep the leader controls shown
         m_sizerFormat->GetStaticBox()->Hide();
         m_sizerCenter->GetStaticBox()->Hide();
@@ -70,7 +70,6 @@ DIALOG_DIMENSION_PROPERTIES::DIALOG_DIMENSION_PROPERTIES( PCB_BASE_EDIT_FRAME* a
         break;
 
     case PCB_DIM_CENTER_T:
-    case PCB_FP_DIM_CENTER_T:
         m_sizerLeader->GetStaticBox()->Hide();
         m_sizerFormat->GetStaticBox()->Hide();
         m_sizerText->GetStaticBox()->Hide();
@@ -95,23 +94,23 @@ DIALOG_DIMENSION_PROPERTIES::DIALOG_DIMENSION_PROPERTIES( PCB_BASE_EDIT_FRAME* a
     m_separator0->SetIsSeparator();
 
     m_bold->SetIsCheckButton();
-    m_bold->SetBitmap( KiBitmap( BITMAPS::text_bold ) );
+    m_bold->SetBitmap( KiBitmapBundle( BITMAPS::text_bold ) );
     m_italic->SetIsCheckButton();
-    m_italic->SetBitmap( KiBitmap( BITMAPS::text_italic ) );
+    m_italic->SetBitmap( KiBitmapBundle( BITMAPS::text_italic ) );
 
     m_separator1->SetIsSeparator();
 
     m_alignLeft->SetIsRadioButton();
-    m_alignLeft->SetBitmap( KiBitmap( BITMAPS::text_align_left ) );
+    m_alignLeft->SetBitmap( KiBitmapBundle( BITMAPS::text_align_left ) );
     m_alignCenter->SetIsRadioButton();
-    m_alignCenter->SetBitmap( KiBitmap( BITMAPS::text_align_center ) );
+    m_alignCenter->SetBitmap( KiBitmapBundle( BITMAPS::text_align_center ) );
     m_alignRight->SetIsRadioButton();
-    m_alignRight->SetBitmap( KiBitmap( BITMAPS::text_align_right ) );
+    m_alignRight->SetBitmap( KiBitmapBundle( BITMAPS::text_align_right ) );
 
     m_separator2->SetIsSeparator();
 
     m_mirrored->SetIsCheckButton();
-    m_mirrored->SetBitmap( KiBitmap( BITMAPS::text_mirrored ) );
+    m_mirrored->SetBitmap( KiBitmapBundle( BITMAPS::text_mirrored ) );
 
     m_separator3->SetIsSeparator();
 
@@ -212,6 +211,12 @@ bool DIALOG_DIMENSION_PROPERTIES::TransferDataToWindow()
         case DIM_UNITS_MODE::AUTOMATIC:     m_cbUnits->SetSelection( 3 ); break;
     }
 
+    switch ( m_dimension->GetArrowDirection() )
+    {
+        case DIM_ARROW_DIRECTION::INWARD:       m_cbArrowDirection->SetSelection( 0 ); break;
+        case DIM_ARROW_DIRECTION::OUTWARD:      m_cbArrowDirection->SetSelection( 1 ); break;
+    }
+
     m_cbUnitsFormat->SetSelection( static_cast<int>( m_dimension->GetUnitsFormat() ) );
     m_cbPrecision->SetSelection( static_cast<int>( m_dimension->GetPrecision() ) );
 
@@ -253,9 +258,10 @@ bool DIALOG_DIMENSION_PROPERTIES::TransferDataToWindow()
 
     switch ( m_dimension->GetHorizJustify() )
     {
-    case GR_TEXT_H_ALIGN_LEFT:   m_alignLeft->Check( true );   break;
-    case GR_TEXT_H_ALIGN_CENTER: m_alignCenter->Check( true ); break;
-    case GR_TEXT_H_ALIGN_RIGHT:  m_alignRight->Check( true );  break;
+    case GR_TEXT_H_ALIGN_LEFT:          m_alignLeft->Check( true );   break;
+    case GR_TEXT_H_ALIGN_CENTER:        m_alignCenter->Check( true ); break;
+    case GR_TEXT_H_ALIGN_RIGHT:         m_alignRight->Check( true );  break;
+    case GR_TEXT_H_ALIGN_INDETERMINATE:                               break;
     }
 
     m_mirrored->Check( m_dimension->IsMirrored() );
@@ -264,6 +270,11 @@ bool DIALOG_DIMENSION_PROPERTIES::TransferDataToWindow()
     m_arrowLength.SetValue( m_dimension->GetArrowLength() );
     m_extensionOffset.SetValue( m_dimension->GetExtensionOffset() );
 
+    if( PCB_DIM_ALIGNED* aligned = dynamic_cast<PCB_DIM_ALIGNED*>( m_dimension ) )
+        m_extensionOvershoot.SetValue( aligned->GetExtensionHeight() );
+    else
+        m_extensionOvershoot.Show( false );
+
     // Do this last; it depends on the other settings
     if( m_dimension->GetOverrideTextEnabled() )
     {
@@ -275,11 +286,8 @@ bool DIALOG_DIMENSION_PROPERTIES::TransferDataToWindow()
         m_txtValueActual->SetValue( m_dimension->GetValueText() );
     }
 
-    if( m_dimension->Type() == PCB_DIM_LEADER_T || m_dimension->Type() == PCB_FP_DIM_LEADER_T )
-    {
-        PCB_DIM_LEADER* leader = static_cast<PCB_DIM_LEADER*>( m_dimension );
+    if( PCB_DIM_LEADER* leader = static_cast<PCB_DIM_LEADER*>( m_dimension ) )
         m_cbTextFrame->SetSelection( static_cast<int>( leader->GetTextBorder() ) );
-    }
 
     return DIALOG_DIMENSION_PROPERTIES_BASE::TransferDataToWindow();
 }
@@ -307,7 +315,7 @@ bool DIALOG_DIMENSION_PROPERTIES::TransferDataFromWindow()
     updateDimensionFromDialog( m_dimension );
 
     if( pushCommit )
-        commit.Push( _( "Change dimension properties" ) );
+        commit.Push( _( "Edit Dimension Properties" ) );
 
     return true;
 }
@@ -381,6 +389,11 @@ void DIALOG_DIMENSION_PROPERTIES::updateDimensionFromDialog( PCB_DIMENSION_BASE*
     aTarget->SetSuffix( board->ConvertCrossReferencesToKIIDs( m_txtSuffix->GetValue() ) );
     aTarget->SetLayer( static_cast<PCB_LAYER_ID>( m_cbLayerActual->GetLayerSelection() ) );
 
+    switch ( m_cbArrowDirection->GetSelection() ) {
+        case 0: aTarget->SetArrowDirection( DIM_ARROW_DIRECTION::INWARD );     break;
+        case 1: aTarget->SetArrowDirection( DIM_ARROW_DIRECTION::OUTWARD );    break;
+    }
+
     switch( m_cbUnits->GetSelection() )
     {
     case 0: aTarget->SetUnitsMode( DIM_UNITS_MODE::INCHES );      break;
@@ -412,6 +425,7 @@ void DIALOG_DIMENSION_PROPERTIES::updateDimensionFromDialog( PCB_DIMENSION_BASE*
     if( m_fontCtrl->HaveFontSelection() )
         aTarget->SetFont( m_fontCtrl->GetFontSelection( m_bold->IsChecked(), m_italic->IsChecked() ) );
 
+    // Must come after SetTextWidth/Height()
     aTarget->SetBold( m_bold->IsChecked() );
     aTarget->SetItalic( m_italic->IsChecked() );
 
@@ -428,11 +442,11 @@ void DIALOG_DIMENSION_PROPERTIES::updateDimensionFromDialog( PCB_DIMENSION_BASE*
     aTarget->SetArrowLength( m_arrowLength.GetValue() );
     aTarget->SetExtensionOffset( m_extensionOffset.GetValue() );
 
-    if( aTarget->Type() == PCB_DIM_LEADER_T || m_dimension->Type() == PCB_FP_DIM_LEADER_T )
-    {
-        PCB_DIM_LEADER* leader = static_cast<PCB_DIM_LEADER*>( aTarget );
+    if( PCB_DIM_ALIGNED* aligned = dynamic_cast<PCB_DIM_ALIGNED*>( aTarget ) )
+        aligned->SetExtensionHeight( m_extensionOvershoot.GetValue() );
+
+    if( PCB_DIM_LEADER* leader = dynamic_cast<PCB_DIM_LEADER*>( aTarget ) )
         leader->SetTextBorder( static_cast<DIM_TEXT_BORDER>( m_cbTextFrame->GetSelection()));
-    }
 
     aTarget->Update();
 }
@@ -441,5 +455,5 @@ void DIALOG_DIMENSION_PROPERTIES::updateDimensionFromDialog( PCB_DIMENSION_BASE*
 void DIALOG_DIMENSION_PROPERTIES::updatePreviewText()
 {
     updateDimensionFromDialog( m_previewDimension );
-    m_staticTextPreview->SetLabel( m_previewDimension->GetShownText() );
+    m_staticTextPreview->SetLabel( m_previewDimension->GetShownText( true ) );
 }
