@@ -4,7 +4,7 @@
  * Copyright (C) 2018 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2015 Dick Hollenbeck, dick@softplc.com
  * Copyright (C) 2008 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,33 +24,33 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-#include <confirm.h>
-#include <dialogs/dialog_text_entry.h>
+#include <3d_rendering/opengl/3d_model.h>
 #include <3d_viewer/eda_3d_viewer_frame.h>
-#include <validators.h>
-#include <board_design_settings.h>
-#include <board_commit.h>
 #include <bitmaps.h>
-#include <kiplatform/ui.h>
-#include <widgets/grid_text_button_helpers.h>
-#include <widgets/wx_grid.h>
-#include <widgets/std_bitmap_button.h>
-#include <widgets/text_ctrl_eval.h>
+#include <board_commit.h>
+#include <board_design_settings.h>
+#include <confirm.h>
+#include <dialog_footprint_properties_fp_editor.h>
+#include <dialogs/dialog_text_entry.h>
+#include <dialogs/panel_preview_3d_model.h>
+#include <embedded_files.h>
+#include <filename_resolver.h>
 #include <footprint.h>
 #include <footprint_edit_frame.h>
 #include <footprint_editor_settings.h>
-#include <dialog_footprint_properties_fp_editor.h>
+#include <grid_layer_box_helpers.h>
+#include <kiplatform/ui.h>
+#include <panel_embedded_files.h>
 #include <panel_fp_properties_3d_model.h>
-#include "3d_rendering/opengl/3d_model.h"
-#include "filename_resolver.h"
 #include <pgm_base.h>
-#include <pcbnew.h>
-#include "dialogs/panel_preview_3d_model.h"
-#include "dialogs/3d_cache_dialogs.h"
 #include <settings/settings_manager.h>
 #include <tool/tool_manager.h>
 #include <tools/pcb_selection_tool.h>
-#include <grid_layer_box_helpers.h>
+#include <validators.h>
+#include <widgets/grid_text_button_helpers.h>
+#include <widgets/std_bitmap_button.h>
+#include <widgets/text_ctrl_eval.h>
+#include <widgets/wx_grid.h>
 
 #include <fp_lib_table.h>
 
@@ -86,10 +86,10 @@ bool PRIVATE_LAYERS_GRID_TABLE::CanSetValueAs( int aRow, int aCol, const wxStrin
 
 
 wxGridCellAttr* PRIVATE_LAYERS_GRID_TABLE::GetAttr( int aRow, int aCol,
-                                                    wxGridCellAttr::wxAttrKind  )
+                                                    wxGridCellAttr::wxAttrKind aKind  )
 {
     m_layerColAttr->IncRef();
-    return m_layerColAttr;
+    return enhanceAttr( m_layerColAttr, aRow, aCol, aKind );
 }
 
 
@@ -123,27 +123,32 @@ NOTEBOOK_PAGES DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::m_page = NOTEBOOK_PAGES::P
 
 
 DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR(
-        FOOTPRINT_EDIT_FRAME* aParent,
-        FOOTPRINT* aFootprint ) :
-    DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR_BASE( aParent ),
-    m_frame( aParent ),
-    m_footprint( aFootprint ),
-    m_netClearance( aParent, m_NetClearanceLabel, m_NetClearanceCtrl, m_NetClearanceUnits ),
-    m_solderMask( aParent, m_SolderMaskMarginLabel, m_SolderMaskMarginCtrl,
-                  m_SolderMaskMarginUnits ),
-    m_solderPaste( aParent, m_SolderPasteMarginLabel, m_SolderPasteMarginCtrl,
-                   m_SolderPasteMarginUnits ),
-    m_solderPasteRatio( aParent, m_PasteMarginRatioLabel, m_PasteMarginRatioCtrl,
-                        m_PasteMarginRatioUnits ),
-    m_gridSize( 0, 0 ),
-    m_lastRequestedSize( 0, 0 )
+                                                                    FOOTPRINT_EDIT_FRAME* aParent,
+                                                                    FOOTPRINT* aFootprint ) :
+        DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR_BASE( aParent ),
+        m_frame( aParent ),
+        m_footprint( aFootprint ),
+        m_initialized( false ),
+        m_netClearance( aParent, m_NetClearanceLabel, m_NetClearanceCtrl, m_NetClearanceUnits ),
+        m_solderMask( aParent, m_SolderMaskMarginLabel, m_SolderMaskMarginCtrl,
+                      m_SolderMaskMarginUnits ),
+        m_solderPaste( aParent, m_SolderPasteMarginLabel, m_SolderPasteMarginCtrl,
+                       m_SolderPasteMarginUnits ),
+        m_solderPasteRatio( aParent, m_PasteMarginRatioLabel, m_PasteMarginRatioCtrl,
+                            m_PasteMarginRatioUnits ),
+        m_gridSize( 0, 0 ),
+        m_lastRequestedSize( 0, 0 )
 {
     SetEvtHandlerEnabled( false );
-    // Create the 3D models page
-    m_3dPanel = new PANEL_FP_PROPERTIES_3D_MODEL( m_frame, m_footprint, this, m_NoteBook );
-    m_NoteBook->AddPage( m_3dPanel, _("3D Models"), false );
 
-    m_texts = new FP_TEXT_GRID_TABLE( m_frame );
+    // Create the extra panels.
+    m_embeddedFiles = new PANEL_EMBEDDED_FILES( m_NoteBook, m_footprint );
+    m_3dPanel = new PANEL_FP_PROPERTIES_3D_MODEL( m_frame, m_footprint, this, m_embeddedFiles, m_NoteBook );
+
+    m_NoteBook->AddPage( m_3dPanel, _("3D Models"), false );
+    m_NoteBook->AddPage( m_embeddedFiles, _( "Embedded Files" ) );
+
+    m_fields = new PCB_FIELDS_GRID_TABLE( m_frame, this, m_embeddedFiles->GetLocalFiles() );
     m_privateLayers = new PRIVATE_LAYERS_GRID_TABLE( m_frame );
 
     m_delayedErrorMessage = wxEmptyString;
@@ -162,7 +167,7 @@ DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR(
     m_itemsGrid->SetDefaultRowSize( m_itemsGrid->GetDefaultRowSize() + 4 );
     m_privateLayersGrid->SetDefaultRowSize( m_privateLayersGrid->GetDefaultRowSize() + 4 );
 
-    m_itemsGrid->SetTable( m_texts );
+    m_itemsGrid->SetTable( m_fields );
     m_privateLayersGrid->SetTable( m_privateLayers );
 
     m_itemsGrid->PushEventHandler( new GRID_TRICKS( m_itemsGrid ) );
@@ -213,12 +218,12 @@ DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR(
     m_solderPasteRatio.SetNegativeZero();
 
     // Configure button logos
-    m_bpAdd->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
-    m_bpDelete->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
-    m_bpAddLayer->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
-    m_bpDeleteLayer->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
-    m_bpAddPadGroup->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
-    m_bpRemovePadGroup->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
+    m_bpAdd->SetBitmap( KiBitmapBundle( BITMAPS::small_plus ) );
+    m_bpDelete->SetBitmap( KiBitmapBundle( BITMAPS::small_trash ) );
+    m_bpAddLayer->SetBitmap( KiBitmapBundle( BITMAPS::small_plus ) );
+    m_bpDeleteLayer->SetBitmap( KiBitmapBundle( BITMAPS::small_trash ) );
+    m_bpAddPadGroup->SetBitmap( KiBitmapBundle( BITMAPS::small_plus ) );
+    m_bpRemovePadGroup->SetBitmap( KiBitmapBundle( BITMAPS::small_trash ) );
 
     SetupStandardButtons();
 
@@ -229,11 +234,10 @@ DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR(
 
 DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::~DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR()
 {
-    m_frame->GetSettings()->m_FootprintTextShownColumns =
-            m_itemsGrid->GetShownColumns().ToStdString();
+    m_frame->GetSettings()->m_FootprintTextShownColumns = m_itemsGrid->GetShownColumnsAsString();
 
     // Prevents crash bug in wxGrid's d'tor
-    m_itemsGrid->DestroyTable( m_texts );
+    m_itemsGrid->DestroyTable( m_fields );
     m_privateLayersGrid->DestroyTable( m_privateLayers );
 
     // Delete the GRID_TRICKS.
@@ -255,7 +259,7 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataToWindow()
 
     m_FootprintNameCtrl->ChangeValue( footprintName );
 
-    m_DocCtrl->SetValue( m_footprint->GetDescription() );
+    m_DocCtrl->SetValue( EscapeString( m_footprint->GetLibDescription(), CTX_LINE ) );
     m_KeywordCtrl->SetValue( m_footprint->GetKeywords() );
 
     if( !wxDialog::TransferDataToWindow() )
@@ -268,20 +272,16 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataToWindow()
     if( !m_3dPanel->TransferDataToWindow() )
         return false;
 
-    // Footprint Texts
-    m_texts->push_back( m_footprint->Reference() );
-    m_texts->push_back( m_footprint->Value() );
+    if( !m_embeddedFiles->TransferDataToWindow() )
+        return false;
 
-    for( BOARD_ITEM* item : m_footprint->GraphicalItems() )
-    {
-        FP_TEXT* textItem = dyn_cast<FP_TEXT*>( item );
-
-        if( textItem )
-            m_texts->push_back( *textItem );
-    }
+    // Footprint Fields
+    for( PCB_FIELD* field : m_footprint->GetFields() )
+        m_fields->push_back( *field );
 
     // Notify the grid
-    wxGridTableMessage tmsg( m_texts, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, m_texts->GetNumberRows() );
+    wxGridTableMessage tmsg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED,
+                             m_fields->GetNumberRows() );
     m_itemsGrid->ProcessTableMessage( tmsg );
 
     if( m_footprint->GetAttributes() & FP_THROUGH_HOLE )
@@ -304,16 +304,33 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataToWindow()
     m_excludeFromPosFiles->SetValue( m_footprint->GetAttributes() & FP_EXCLUDE_FROM_POS_FILES );
     m_excludeFromBOM->SetValue( m_footprint->GetAttributes() & FP_EXCLUDE_FROM_BOM );
     m_noCourtyards->SetValue( m_footprint->GetAttributes() & FP_ALLOW_MISSING_COURTYARD );
+    m_cbDNP->SetValue( m_footprint->GetAttributes() & FP_DNP );
 
     // Local Clearances
 
-    m_netClearance.SetValue( m_footprint->GetLocalClearance() );
-    m_solderMask.SetValue( m_footprint->GetLocalSolderMaskMargin() );
-    m_solderPaste.SetValue( m_footprint->GetLocalSolderPasteMargin() );
-    m_solderPasteRatio.SetDoubleValue( m_footprint->GetLocalSolderPasteMarginRatio() * 100.0 );
+    if( m_footprint->GetLocalClearance().has_value() )
+        m_netClearance.SetValue( m_footprint->GetLocalClearance().value() );
+    else
+        m_netClearance.SetValue( wxEmptyString );
+
+    if( m_footprint->GetLocalSolderMaskMargin().has_value() )
+        m_solderMask.SetValue( m_footprint->GetLocalSolderMaskMargin().value() );
+    else
+        m_solderMask.SetValue( wxEmptyString );
+
+    if( m_footprint->GetLocalSolderPasteMargin().has_value() )
+        m_solderPaste.SetValue( m_footprint->GetLocalSolderPasteMargin().value() );
+    else
+        m_solderPaste.SetValue( wxEmptyString );
+
+    if( m_footprint->GetLocalSolderPasteMarginRatio().has_value() )
+        m_solderPasteRatio.SetDoubleValue( m_footprint->GetLocalSolderPasteMarginRatio().value() * 100.0 );
+    else
+        m_solderPasteRatio.SetValue( wxEmptyString );
+
     m_allowBridges->SetValue( m_footprint->GetAttributes() & FP_ALLOW_SOLDERMASK_BRIDGES );
 
-    switch( m_footprint->GetZoneConnection() )
+    switch( m_footprint->GetLocalZoneConnection() )
     {
     default:
     case ZONE_CONNECTION::INHERITED: m_ZoneConnectionChoice->SetSelection( 0 ); break;
@@ -339,25 +356,26 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataToWindow()
         // Adjust the column size.
         int col_size = m_itemsGrid->GetVisibleWidth( col );
 
-        if( col == FPT_LAYER )  // This one's a drop-down.  Check all possible values.
+        if( col == PFC_LAYER )  // This one's a drop-down.  Check all possible values.
         {
             BOARD* board = m_footprint->GetBoard();
 
             for( PCB_LAYER_ID layer : board->GetEnabledLayers().Seq() )
                 col_size = std::max( col_size, GetTextExtent( board->GetLayerName( layer ) ).x );
 
-            // And the swatch:
-            col_size += 20;
+            // Swatch and gaps:
+            col_size += KiROUND( 14 * GetDPIScaleFactor() ) + 12;
         }
 
         if( m_itemsGrid->IsColShown( col ) )
             m_itemsGrid->SetColSize( col, col_size );
     }
 
-    m_itemsGrid->SetRowLabelSize( m_itemsGrid->GetVisibleWidth( -1, true, true, true ) );
+    m_itemsGrid->SetRowLabelSize( 0 );
 
     Layout();
     adjustGridColumns();
+    m_initialized = true;
 
     return true;
 }
@@ -403,60 +421,61 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::Validate()
         return false;
     }
 
-    // Check for empty texts.
-    for( size_t i = 2; i < m_texts->size(); ++i )
+    // Check for valid field text properties
+    for( size_t i = 0; i < m_fields->size(); ++i )
     {
-        FP_TEXT& text = m_texts->at( i );
+        PCB_FIELD& field = m_fields->at( i );
 
-        if( text.GetText().IsEmpty() )
+        // Check for missing field names.
+        if( field.GetName( false ).IsEmpty() )
         {
-            if( m_NoteBook->GetSelection() != 0 )
-                m_NoteBook->SetSelection( 0 );
-
-            m_delayedErrorMessage = _( "Text items must have some content." );
             m_delayedFocusGrid = m_itemsGrid;
-            m_delayedFocusColumn = FPT_TEXT;
+            m_delayedErrorMessage = wxString::Format( _( "Fields must have a name." ) );
+            m_delayedFocusColumn = PFC_NAME;
             m_delayedFocusRow = i;
 
             return false;
         }
 
-        if( text.GetTextWidth() < TEXTS_MIN_SIZE || text.GetTextWidth() > TEXTS_MAX_SIZE )
+        int minSize = pcbIUScale.mmToIU( TEXT_MIN_SIZE_MM );
+        int maxSize = pcbIUScale.mmToIU( TEXT_MAX_SIZE_MM );
+
+        if( field.GetTextWidth() < minSize || field.GetTextWidth() > maxSize )
         {
             m_delayedFocusGrid = m_itemsGrid;
             m_delayedErrorMessage = wxString::Format( _( "The text width must be between %s and %s." ),
-                                                      m_frame->StringFromValue( TEXTS_MIN_SIZE, true ),
-                                                      m_frame->StringFromValue( TEXTS_MAX_SIZE, true ) );
-            m_delayedFocusColumn = FPT_WIDTH;
+                                                      m_frame->StringFromValue( minSize, true ),
+                                                      m_frame->StringFromValue( maxSize, true ) );
+            m_delayedFocusColumn = PFC_WIDTH;
             m_delayedFocusRow = i;
 
             return false;
         }
 
-        if( text.GetTextHeight() < TEXTS_MIN_SIZE || text.GetTextHeight() > TEXTS_MAX_SIZE )
+        if( field.GetTextHeight() < minSize || field.GetTextHeight() > maxSize )
         {
             m_delayedFocusGrid = m_itemsGrid;
             m_delayedErrorMessage = wxString::Format( _( "The text height must be between %s and %s." ),
-                                                      m_frame->StringFromValue( TEXTS_MIN_SIZE, true ),
-                                                      m_frame->StringFromValue( TEXTS_MAX_SIZE, true ) );
-            m_delayedFocusColumn = FPT_HEIGHT;
+                                                      m_frame->StringFromValue( minSize, true ),
+                                                      m_frame->StringFromValue( maxSize, true ) );
+            m_delayedFocusColumn = PFC_HEIGHT;
             m_delayedFocusRow = i;
 
             return false;
         }
 
         // Test for acceptable values for thickness and size and clamp if fails
-        int maxPenWidth = Clamp_Text_PenSize( text.GetTextThickness(), text.GetTextSize() );
+        int maxPenWidth = ClampTextPenSize( field.GetTextThickness(), field.GetTextSize() );
 
-        if( text.GetTextThickness() > maxPenWidth )
+        if( field.GetTextThickness() > maxPenWidth )
         {
-            m_itemsGrid->SetCellValue( i, FPT_THICKNESS,
+            m_itemsGrid->SetCellValue( i, PFC_THICKNESS,
                                        m_frame->StringFromValue( maxPenWidth, true ) );
 
             m_delayedFocusGrid = m_itemsGrid;
             m_delayedErrorMessage = _( "The text thickness is too large for the text size.\n"
                                        "It will be clamped." );
-            m_delayedFocusColumn = FPT_THICKNESS;
+            m_delayedFocusColumn = PFC_THICKNESS;
             m_delayedFocusRow = i;
 
             return false;
@@ -475,9 +494,6 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataFromWindow()
     if( !Validate() )
         return false;
 
-    if( !DIALOG_SHIM::TransferDataFromWindow() )
-        return false;
-
     if( !m_itemsGrid->CommitPendingChanges()
             || !m_privateLayersGrid->CommitPendingChanges()
             || !m_padGroupsGrid->CommitPendingChanges() )
@@ -485,61 +501,79 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataFromWindow()
         return false;
     }
 
-    // This only commits the editor, model updating is done below so it is inside
-    // the commit
-    if( !m_3dPanel->TransferDataFromWindow() )
-        return false;
-
     KIGFX::PCB_VIEW* view = m_frame->GetCanvas()->GetView();
     BOARD_COMMIT     commit( m_frame );
     commit.Modify( m_footprint );
 
+    // Must be done inside the commit to capture the undo state
+    // This will call TransferDataToWindow() on the 3D panel and
+    // the embedded files panel.
+    if( !DIALOG_SHIM::TransferDataFromWindow() )
+        return false;
+
+    // Clear out embedded files that are no longer in use
+    std::set<wxString> files;
+    std::set<wxString> files_to_delete;
+
+    // Get the new files from the footprint fields
+    for( size_t ii = 0; ii < m_fields->size(); ++ii )
+    {
+        const wxString& name = m_fields->at( ii ).GetText();
+
+        if( name.StartsWith( FILEEXT::KiCadUriPrefix ) )
+            files.insert( name );
+    }
+
+    // Find any files referenced in the old fields that are not in the new fields
+    for( PCB_FIELD* field : m_footprint->GetFields() )
+    {
+        if( field->GetText().StartsWith( FILEEXT::KiCadUriPrefix ) )
+        {
+            if( files.find( field->GetText() ) == files.end() )
+                files_to_delete.insert( field->GetText() );
+        }
+    }
+
+    for( const wxString& file : files_to_delete )
+    {
+        wxString name = file.Mid( FILEEXT::KiCadUriPrefix.size() + 3 ); // Skip "kicad-embed://"
+        m_footprint->RemoveFile( name );
+    }
+
     LIB_ID fpID = m_footprint->GetFPID();
     fpID.SetLibItemName( m_FootprintNameCtrl->GetValue() );
     m_footprint->SetFPID( fpID );
 
-    m_footprint->SetDescription( m_DocCtrl->GetValue() );
+    m_footprint->SetLibDescription( UnescapeString( m_DocCtrl->GetValue() ) );
     m_footprint->SetKeywords( m_KeywordCtrl->GetValue() );
 
-    // copy reference and value
-    m_footprint->Reference() = m_texts->at( 0 );
-    m_footprint->Value() = m_texts->at( 1 );
+    // Update fields
 
-    size_t i = 2;
-    std::vector<FP_TEXT*> items_to_remove;
+    std::vector<PCB_FIELD*> items_to_remove;
+    size_t                  i = 0;
 
-    for( BOARD_ITEM* item : m_footprint->GraphicalItems() )
+    for( PCB_FIELD* field : m_footprint->GetFields() )
     {
-        FP_TEXT* textItem = dynamic_cast<FP_TEXT*>( item );
-
-        if( textItem )
-        {
-            // copy grid table entries till we run out, then delete any remaining texts
-            if( i < m_texts->size() )
-                *textItem = m_texts->at( i++ );
-            else    // store this item to remove and delete it later,
-                    // after the graphic list is explored:
-                items_to_remove.push_back( textItem );
-        }
+        // copy grid table entries till we run out, then delete any remaining texts
+        if( i < m_fields->size() )
+            *field = m_fields->at( i++ );
+        else
+            items_to_remove.push_back( field );
     }
 
     // Remove text items:
     PCB_SELECTION_TOOL* selTool = m_frame->GetToolManager()->GetTool<PCB_SELECTION_TOOL>();
 
-    for( FP_TEXT* item: items_to_remove )
+    for( PCB_TEXT* item : items_to_remove )
     {
         selTool->RemoveItemFromSel( item );
         view->Remove( item );
         item->DeleteStructure();
     }
 
-    // if there are still grid table entries, create new texts for them
-    while( i < m_texts->size() )
-    {
-        FP_TEXT* newText = new FP_TEXT( m_texts->at( i++ ) );
-        m_footprint->Add( newText, ADD_MODE::APPEND );
-        view->Add( newText );
-    }
+    // if there are still grid table entries, create new fields for them
+    while( i < m_fields->size() )
+        view->Add( m_footprint->AddField( m_fields->at( i++ ) ) );
 
     LSET privateLayers;
 
@@ -569,24 +603,42 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataFromWindow()
     if( m_noCourtyards->GetValue() )
         attributes |= FP_ALLOW_MISSING_COURTYARD;
 
+    if( m_cbDNP->GetValue() )
+        attributes |= FP_DNP;
+
     if( m_allowBridges->GetValue() )
         attributes |= FP_ALLOW_SOLDERMASK_BRIDGES;
 
     m_footprint->SetAttributes( attributes );
 
-    // Initialize masks clearances
-    m_footprint->SetLocalClearance( m_netClearance.GetValue() );
-    m_footprint->SetLocalSolderMaskMargin( m_solderMask.GetValue() );
-    m_footprint->SetLocalSolderPasteMargin( m_solderPaste.GetValue() );
-    m_footprint->SetLocalSolderPasteMarginRatio( m_solderPasteRatio.GetDoubleValue() / 100.0 );
+    // Initialize mask clearances
+    if( m_netClearance.IsNull() )
+        m_footprint->SetLocalClearance( {} );
+    else
+        m_footprint->SetLocalClearance( m_netClearance.GetValue() );
+
+    if( m_solderMask.IsNull() )
+        m_footprint->SetLocalSolderMaskMargin( {} );
+    else
+        m_footprint->SetLocalSolderMaskMargin( m_solderMask.GetValue() );
+
+    if( m_solderPaste.IsNull() )
+        m_footprint->SetLocalSolderPasteMargin( {} );
+    else
+        m_footprint->SetLocalSolderPasteMargin( m_solderPaste.GetValue() );
+
+    if( m_solderPasteRatio.IsNull() )
+        m_footprint->SetLocalSolderPasteMarginRatio( {} );
+    else
+        m_footprint->SetLocalSolderPasteMarginRatio( m_solderPasteRatio.GetDoubleValue() / 100.0 );
 
     switch( m_ZoneConnectionChoice->GetSelection() )
     {
     default:
-    case 0:  m_footprint->SetZoneConnection( ZONE_CONNECTION::INHERITED ); break;
-    case 1:  m_footprint->SetZoneConnection( ZONE_CONNECTION::FULL );      break;
-    case 2:  m_footprint->SetZoneConnection( ZONE_CONNECTION::THERMAL );   break;
-    case 3:  m_footprint->SetZoneConnection( ZONE_CONNECTION::NONE );      break;
+    case 0:  m_footprint->SetLocalZoneConnection( ZONE_CONNECTION::INHERITED ); break;
+    case 1:  m_footprint->SetLocalZoneConnection( ZONE_CONNECTION::FULL );      break;
+    case 2:  m_footprint->SetLocalZoneConnection( ZONE_CONNECTION::THERMAL );   break;
+    case 3:  m_footprint->SetLocalZoneConnection( ZONE_CONNECTION::NONE );      break;
     }
 
     m_footprint->ClearNetTiePadGroups();
@@ -605,57 +657,46 @@ bool DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::TransferDataFromWindow()
     fpList->clear();
     fpList->insert( fpList->end(), panelList.begin(), panelList.end() );
 
-    commit.Push( _( "Modify footprint properties" ) );
+    commit.Push( _( "Edit Footprint Properties" ) );
 
     return true;
 }
 
 
-static bool footprintIsFromBoard( FOOTPRINT* aFootprint )
-{
-    return aFootprint->GetLink() != niluuid;
-}
-
-
-void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnFootprintNameText( wxCommandEvent& event )
-{
-    if( !footprintIsFromBoard( m_footprint ) )
-    {
-        // Currently: nothing to do
-    }
-}
-
-
 void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnAddField( wxCommandEvent& event )
 {
     if( !m_itemsGrid->CommitPendingChanges() )
         return;
 
     const BOARD_DESIGN_SETTINGS& dsnSettings = m_frame->GetDesignSettings();
-    FP_TEXT textItem( m_footprint );
+
+    PCB_FIELD newField( m_footprint, m_footprint->GetNextFieldId(),
+                        GetUserFieldName( m_fields->GetNumberRows(), DO_TRANSLATE ) );
 
     // Set active layer if legal; otherwise copy layer from previous text item
     if( LSET::AllTechMask().test( m_frame->GetActiveLayer() ) )
-        textItem.SetLayer( m_frame->GetActiveLayer() );
+        newField.SetLayer( m_frame->GetActiveLayer() );
     else
-        textItem.SetLayer( m_texts->at( m_texts->size() - 1 ).GetLayer() );
+        newField.SetLayer( m_fields->at( m_fields->size() - 1 ).GetLayer() );
 
-    textItem.SetTextSize( dsnSettings.GetTextSize( textItem.GetLayer() ) );
-    textItem.SetTextThickness( dsnSettings.GetTextThickness( textItem.GetLayer() ) );
-    textItem.SetItalic( dsnSettings.GetTextItalic( textItem.GetLayer() ) );
+    newField.SetTextSize( dsnSettings.GetTextSize( newField.GetLayer() ) );
+    newField.SetTextThickness( dsnSettings.GetTextThickness( newField.GetLayer() ) );
+    newField.SetItalic( dsnSettings.GetTextItalic( newField.GetLayer() ) );
 
-    m_texts->push_back( textItem );
+    m_fields->push_back( newField );
 
     // notify the grid
-    wxGridTableMessage msg( m_texts, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, 1 );
+    wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, 1 );
     m_itemsGrid->ProcessTableMessage( msg );
 
     m_itemsGrid->SetFocus();
-    m_itemsGrid->MakeCellVisible( m_texts->size() - 1, 0 );
-    m_itemsGrid->SetGridCursor( m_texts->size() - 1, 0 );
+    m_itemsGrid->MakeCellVisible( m_fields->size() - 1, 0 );
+    m_itemsGrid->SetGridCursor( m_fields->size() - 1, 0 );
 
     m_itemsGrid->EnableCellEditControl( true );
     m_itemsGrid->ShowCellEditControl();
+
+    OnModify();
 }
 
 
@@ -674,22 +715,29 @@ void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnDeleteField( wxCommandEvent& event
 
     for( int row : selectedRows )
     {
-        if( row < 2 )
+        if( row < m_fields->GetMandatoryRowCount() )
         {
-            DisplayError( nullptr, _( "Reference and value are mandatory." ) );
+            DisplayError( this, wxString::Format( _( "The first %d fields are mandatory." ),
+                                                  m_fields->GetMandatoryRowCount() ) );
             return;
         }
     }
 
+    m_itemsGrid->CommitPendingChanges( true /* quiet mode */ );
+    m_itemsGrid->ClearSelection();
+
     // Reverse sort so deleting a row doesn't change the indexes of the other rows.
-    selectedRows.Sort( []( int* first, int* second ) { return *second - *first; } );
+    selectedRows.Sort( []( int* first, int* second )
+                       {
+                           return *second - *first;
+                       } );
 
     for( int row : selectedRows )
     {
-        m_texts->erase( m_texts->begin() + row );
+        m_fields->erase( m_fields->begin() + row );
 
         // notify the grid
-        wxGridTableMessage msg( m_texts, wxGRIDTABLE_NOTIFY_ROWS_DELETED, row, 1 );
+        wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_DELETED, row, 1 );
         m_itemsGrid->ProcessTableMessage( msg );
 
         if( m_itemsGrid->GetNumberRows() > 0 )
@@ -698,6 +746,8 @@ void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnDeleteField( wxCommandEvent& event
             m_itemsGrid->SetGridCursor( std::max( 0, row-1 ), m_itemsGrid->GetGridCursorCol() );
         }
     }
+
+    OnModify();
 }
 
 
@@ -708,7 +758,7 @@ void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnAddLayer( wxCommandEvent& event )
 
     PCB_LAYER_ID nextLayer = User_1;
 
-    while( alg::contains( *m_privateLayers, nextLayer ) && nextLayer < User_9 )
+    while( alg::contains( *m_privateLayers, nextLayer ) && nextLayer < User_45 )
         nextLayer = ToLAYER_ID( nextLayer + 1 );
 
     m_privateLayers->push_back( nextLayer );
@@ -720,6 +770,8 @@ void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnAddLayer( wxCommandEvent& event )
     m_privateLayersGrid->SetFocus();
     m_privateLayersGrid->MakeCellVisible( m_privateLayers->size() - 1, 0 );
     m_privateLayersGrid->SetGridCursor( m_privateLayers->size() - 1, 0 );
+
+    OnModify();
 }
 
 
@@ -742,10 +794,12 @@ void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnDeleteLayer( wxCommandEvent& event
     if( m_privateLayersGrid->GetNumberRows() > 0 )
     {
         m_privateLayersGrid->MakeCellVisible( std::max( 0, curRow-1 ),
-                                               m_privateLayersGrid->GetGridCursorCol() );
+                                              m_privateLayersGrid->GetGridCursorCol() );
         m_privateLayersGrid->SetGridCursor( std::max( 0, curRow-1 ),
-                                             m_privateLayersGrid->GetGridCursorCol() );
+                                            m_privateLayersGrid->GetGridCursorCol() );
     }
+
+    OnModify();
 }
 
 
@@ -762,6 +816,8 @@ void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnAddPadGroup( wxCommandEvent& event
 
     m_padGroupsGrid->EnableCellEditControl( true );
     m_padGroupsGrid->ShowCellEditControl();
+
+    OnModify();
 }
 
 
@@ -786,6 +842,8 @@ void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnRemovePadGroup( wxCommandEvent& ev
     curRow = std::max( 0, curRow - 1 );
     m_padGroupsGrid->MakeCellVisible( curRow, m_padGroupsGrid->GetGridCursorCol() );
     m_padGroupsGrid->SetGridCursor( curRow, m_padGroupsGrid->GetGridCursorCol() );
+
+    OnModify();
 }
 
 
@@ -796,11 +854,16 @@ void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::adjustGridColumns()
 
     itemsWidth -= m_itemsGrid->GetRowLabelSize();
 
-    for( int i = 1; i < m_itemsGrid->GetNumberCols(); i++ )
-        itemsWidth -= m_itemsGrid->GetColSize( i );
+    for( int i = 0; i < m_itemsGrid->GetNumberCols(); i++ )
+    {
+        if( i == 1 )
+            continue;
 
-    m_itemsGrid->SetColSize( 0, std::max( itemsWidth,
-                                          m_itemsGrid->GetVisibleWidth( 0, true, false ) ) );
+        itemsWidth -= m_itemsGrid->GetColSize( i );
+    }
+
+    m_itemsGrid->SetColSize(
+            1, std::max( itemsWidth, m_itemsGrid->GetVisibleWidth( 0, true, false ) ) );
 
     // Update the width private layers grid
     m_privateLayersGrid->SetColSize( 0, std::max( m_privateLayersGrid->GetClientSize().x,
@@ -856,7 +919,9 @@ void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnUpdateUI( wxUpdateUIEvent& event )
         m_delayedFocusGrid->MakeCellVisible( m_delayedFocusRow, m_delayedFocusColumn );
         m_delayedFocusGrid->SetGridCursor( m_delayedFocusRow, m_delayedFocusColumn );
 
-        m_delayedFocusGrid->EnableCellEditControl( true );
+        if( !( m_delayedFocusColumn == 0 && m_delayedFocusRow < m_fields->GetMandatoryRowCount() ) )
+            m_delayedFocusGrid->EnableCellEditControl( true );
+
         m_delayedFocusGrid->ShowCellEditControl();
 
         m_delayedFocusGrid = nullptr;
@@ -897,3 +962,34 @@ void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnGridSize( wxSizeEvent& aEvent )
     // Always propagate for a grid repaint (needed if the height changes, as well as width)
     aEvent.Skip();
 }
+
+
+void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnPageChanging( wxNotebookEvent& aEvent )
+{
+    if( !m_itemsGrid->CommitPendingChanges() )
+        aEvent.Veto();
+
+    if( !m_privateLayersGrid->CommitPendingChanges() )
+        aEvent.Veto();
+}
+
+
+void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnCheckBox( wxCommandEvent& event )
+{
+    if( m_initialized )
+        OnModify();
+}
+
+
+void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnText( wxCommandEvent& event )
+{
+    if( m_initialized )
+        OnModify();
+}
+
+
+void DIALOG_FOOTPRINT_PROPERTIES_FP_EDITOR::OnChoice( wxCommandEvent& event )
+{
+    if( m_initialized )
+        OnModify();
+}
