@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 1992-2019 Jean_Pierre Charras <jp.charras at wanadoo.fr>
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,19 +32,17 @@
 #include <gendrill_gerber_writer.h>
 #include <bitmaps.h>
 #include <tools/board_editor_control.h>
-#include <board.h>
-#include <board_design_settings.h>
 #include <footprint.h>
 #include <pad.h>
 #include <pcb_track.h>
 #include <paths.h>
-#include <project.h>
 #include <dialog_gendrill.h>
 #include <wildcards_and_files_ext.h>
 #include <reporter.h>
 #include <wx/msgdlg.h>
 #include <wx/dirdlg.h>
 #include <wx/filedlg.h>
+#include <jobs/job_export_pcb_drill.h>
 
 // list of allowed precision for EXCELLON files, for integer format:
 // Due to difference between inches and mm,
@@ -54,6 +52,22 @@ static DRILL_PRECISION precisionListForInches( 2, 4 );
 static DRILL_PRECISION precisionListForMetric( 3, 3 );
 
 
+// Static members of DIALOG_GENDRILL
+int  DIALOG_GENDRILL::g_unitDrillIsInch  = false;     // Only for Excellon format
+int  DIALOG_GENDRILL::g_zerosFormat      = EXCELLON_WRITER::DECIMAL_FORMAT;
+bool DIALOG_GENDRILL::g_minimalHeader    = false;    // Only for Excellon format
+bool DIALOG_GENDRILL::g_mirror           = false;    // Only for Excellon format
+bool DIALOG_GENDRILL::g_merge_PTH_NPTH   = false;    // Only for Excellon format
+bool DIALOG_GENDRILL::g_generateMap      = false;
+int  DIALOG_GENDRILL::g_mapFileType      = 4;        // The last choice in m_Choice_Drill_Map
+int  DIALOG_GENDRILL::g_drillFileType    = 0;
+
+bool DIALOG_GENDRILL::g_useRouteModeForOvalHoles = true;    // Use G00 route mode to "drill" oval
+                                                            // holes
+DRILL_PRECISION  DIALOG_GENDRILL::g_precision;
+VECTOR2I         DIALOG_GENDRILL::g_drillFileOffset;
+
+
 /* This function displays the dialog frame for drill tools
  */
 int BOARD_EDITOR_CONTROL::GenerateDrillFiles( const TOOL_EVENT& aEvent )
@@ -66,142 +80,158 @@ int BOARD_EDITOR_CONTROL::GenerateDrillFiles( const TOOL_EVENT& aEvent )
 }
 
 
-DIALOG_GENDRILL::DIALOG_GENDRILL(  PCB_EDIT_FRAME* aPcbEditFrame, wxWindow* aParent  ) :
-    DIALOG_GENDRILL_BASE( aParent )
+DIALOG_GENDRILL::DIALOG_GENDRILL( PCB_EDIT_FRAME* aPcbEditFrame, wxWindow* aParent  ) :
+        DIALOG_GENDRILL_BASE( aParent )
 {
     m_pcbEditFrame = aPcbEditFrame;
     m_board  = m_pcbEditFrame->GetBoard();
+    m_job = nullptr;
     m_plotOpts = m_pcbEditFrame->GetPlotSettings();
 
-    SetupStandardButtons( { { wxID_OK,     _( "Generate Drill File" ) },
-                            { wxID_APPLY,  _( "Generate Map File" )   },
-                            { wxID_CANCEL, _( "Close" )               } } );
+    m_browseButton->SetBitmap( KiBitmapBundle( BITMAPS::small_folder ) );
 
-    m_buttonsSizer->Layout();
+    SetupStandardButtons( { { wxID_OK,     _( "Generate" ) },
+                            { wxID_CANCEL, _( "Close" ) } } );
 
-    SetReturnCode( 1 );
     initDialog();
-    GetSizer()->SetSizeHints( this );
+    finishDialogSettings();
 }
 
 
-// Static members of DIALOG_GENDRILL
-int DIALOG_GENDRILL::m_UnitDrillIsInch  = true;     // Only for Excellon format
-int DIALOG_GENDRILL::m_ZerosFormat      = EXCELLON_WRITER::DECIMAL_FORMAT;
-bool DIALOG_GENDRILL::m_MinimalHeader   = false;    // Only for Excellon format
-bool DIALOG_GENDRILL::m_Mirror          = false;    // Only for Excellon format
-bool DIALOG_GENDRILL::m_Merge_PTH_NPTH  = false;    // Only for Excellon format
-int DIALOG_GENDRILL::m_mapFileType      = 4;        // The last choice in m_Choice_Drill_Map
-int DIALOG_GENDRILL::m_drillFileType    = 0;
-bool DIALOG_GENDRILL::m_UseRouteModeForOvalHoles = true;    // Use G00 route mode to "drill" oval holes
+DIALOG_GENDRILL::DIALOG_GENDRILL( PCB_EDIT_FRAME* aPcbEditFrame, JOB_EXPORT_PCB_DRILL* aJob,
+                                  wxWindow* aParent ) :
+        DIALOG_GENDRILL_BASE( aParent )
+{
+    m_pcbEditFrame = aPcbEditFrame;
+    m_board = m_pcbEditFrame->GetBoard();
+    m_job = aJob;
+
+    m_browseButton->SetBitmap( KiBitmapBundle( BITMAPS::small_folder ) );
+
+    // hide ui elements that dont belong for job config
+    m_buttonReport->Hide();
+    bMainSizer->Remove( bMsgSizer );
+    m_messagesBox->Hide();
+
+    SetupStandardButtons();
+
+    initDialog();
+    finishDialogSettings();
+}
+
 
 DIALOG_GENDRILL::~DIALOG_GENDRILL()
 {
 }
 
 
-void DIALOG_GENDRILL::initDialog()
+bool DIALOG_GENDRILL::TransferDataFromWindow()
 {
-    auto cfg = m_pcbEditFrame->GetPcbNewSettings();
+    if( !m_job )
+    {
+        genDrillAndMapFiles( true, m_cbGenerateMap->GetValue() );
+    }
+    else
+    {
+        m_job->SetConfiguredOutputPath( m_outputDirectoryName->GetValue() );
+        m_job->m_format = m_rbExcellon->GetValue() ? JOB_EXPORT_PCB_DRILL::DRILL_FORMAT::EXCELLON
+												   : JOB_EXPORT_PCB_DRILL::DRILL_FORMAT::GERBER;
+        m_job->m_drillUnits = m_units->GetSelection() == 0
+                                                   ? JOB_EXPORT_PCB_DRILL::DRILL_UNITS::MILLIMETERS
+                                                   : JOB_EXPORT_PCB_DRILL::DRILL_UNITS::INCHES;
+        m_job->m_drillOrigin = static_cast<JOB_EXPORT_PCB_DRILL::DRILL_ORIGIN>( m_origin->GetSelection() );
+        m_job->m_excellonCombinePTHNPTH = m_Check_Merge_PTH_NPTH->IsChecked();
+        m_job->m_excellonMinimalHeader = m_Check_Minimal->IsChecked();
+        m_job->m_excellonMirrorY = m_Check_Mirror->IsChecked();
+        m_job->m_excellonOvalDrillRoute = !m_altDrillMode->GetValue();
+        m_job->m_mapFormat = static_cast<JOB_EXPORT_PCB_DRILL::MAP_FORMAT>( m_choiceDrillMap->GetSelection() );
+        m_job->m_zeroFormat = static_cast<JOB_EXPORT_PCB_DRILL::ZEROS_FORMAT>( m_zeros->GetSelection() );
+        m_job->m_generateMap = m_cbGenerateMap->IsChecked();
+    }
 
-    m_Merge_PTH_NPTH           = cfg->m_GenDrill.merge_pth_npth;
-    m_MinimalHeader            = cfg->m_GenDrill.minimal_header;
-    m_Mirror                   = cfg->m_GenDrill.mirror;
-    m_UnitDrillIsInch          = cfg->m_GenDrill.unit_drill_is_inch;
-    m_UseRouteModeForOvalHoles = cfg->m_GenDrill.use_route_for_oval_holes;
-    m_drillFileType            = cfg->m_GenDrill.drill_file_type;
-    m_mapFileType              = cfg->m_GenDrill.map_file_type;
-    m_ZerosFormat              = cfg->m_GenDrill.zeros_format;
-
-    m_drillOriginIsAuxAxis = m_plotOpts.GetUseAuxOrigin();
-
-    // Ensure validity of m_mapFileType
-    if( m_mapFileType < 0 || m_mapFileType >= (int)m_Choice_Drill_Map->GetCount() )
-        m_mapFileType = m_Choice_Drill_Map->GetCount() - 1;  // last item in list = default = PDF
-
-    InitDisplayParams();
+    return true;
 }
 
 
-void DIALOG_GENDRILL::InitDisplayParams()
+bool DIALOG_GENDRILL::TransferDataToWindow()
 {
-    m_browseButton->SetBitmap( KiBitmapBundle( BITMAPS::small_folder ) );
-
-    m_rbExcellon->SetValue( m_drillFileType == 0 );
-    m_rbGerberX2->SetValue( m_drillFileType == 1 );
-    m_Choice_Unit->SetSelection( m_UnitDrillIsInch ? 1 : 0 );
-    m_Choice_Zeros_Format->SetSelection( m_ZerosFormat );
-    UpdatePrecisionOptions();
-    m_Check_Minimal->SetValue( m_MinimalHeader );
-
-    m_Choice_Drill_Offset->SetSelection( m_drillOriginIsAuxAxis ? 1 : 0 );
-
-    m_Check_Mirror->SetValue( m_Mirror );
-    m_Check_Merge_PTH_NPTH->SetValue( m_Merge_PTH_NPTH );
-    m_Choice_Drill_Map->SetSelection( m_mapFileType );
-    m_radioBoxOvalHoleMode->SetSelection( m_UseRouteModeForOvalHoles ? 0 : 1 );
-
-    m_platedPadsHoleCount    = 0;
-    m_notplatedPadsHoleCount = 0;
-    m_throughViasCount = 0;
-    m_microViasCount   = 0;
-    m_blindOrBuriedViasCount = 0;
-
-    for( FOOTPRINT* footprint : m_board->Footprints() )
+    if( !m_job )
     {
-        for( PAD* pad : footprint->Pads() )
-        {
-            if( pad->GetDrillShape() == PAD_DRILL_SHAPE_CIRCLE )
-            {
-                if( pad->GetDrillSize().x != 0 )
-                {
-                    if( pad->GetAttribute() == PAD_ATTRIB::NPTH )
-                        m_notplatedPadsHoleCount++;
-                    else
-                        m_platedPadsHoleCount++;
-                }
-            }
-            else
-            {
-                if( pad->GetDrillSize().x != 0 && pad->GetDrillSize().y != 0 )
-                {
-                    if( pad->GetAttribute() == PAD_ATTRIB::NPTH )
-                        m_notplatedPadsHoleCount++;
-                    else
-                        m_platedPadsHoleCount++;
-                }
-            }
-        }
-    }
+        m_rbExcellon->SetValue( g_drillFileType == 0 );
+        m_rbGerberX2->SetValue( g_drillFileType == 1 );
+        m_units->SetSelection( g_unitDrillIsInch ? 1 : 0 );
+        m_zeros->SetSelection( g_zerosFormat );
+        updatePrecisionOptions();
+        m_Check_Minimal->SetValue( g_minimalHeader );
 
-    for( PCB_TRACK* track : m_board->Tracks() )
+        m_origin->SetSelection( m_drillOriginIsAuxAxis ? 1 : 0 );
+
+        m_Check_Mirror->SetValue( g_mirror );
+        m_Check_Merge_PTH_NPTH->SetValue( g_merge_PTH_NPTH );
+        m_choiceDrillMap->SetSelection( g_mapFileType );
+        m_altDrillMode->SetValue( !g_useRouteModeForOvalHoles );
+        m_cbGenerateMap->SetValue( g_generateMap );
+
+        // Output directory
+        m_outputDirectoryName->SetValue( m_plotOpts.GetOutputDirectory() );
+    }
+    else
     {
-        const PCB_VIA *via = dynamic_cast<const PCB_VIA*>( track );
+        m_browseButton->Hide();
+        m_outputDirectoryName->SetValue( m_job->GetConfiguredOutputPath() );
 
-        if( via )
-        {
-            switch( via->GetViaType() )
-            {
-            case VIATYPE::THROUGH:      m_throughViasCount++;       break;
-            case VIATYPE::MICROVIA:     m_microViasCount++;         break;
-            case VIATYPE::BLIND_BURIED: m_blindOrBuriedViasCount++; break;
-            default:                                                break;
-            }
-        }
+        m_rbExcellon->SetValue( m_job->m_format == JOB_EXPORT_PCB_DRILL::DRILL_FORMAT::EXCELLON );
+        m_rbGerberX2->SetValue( m_job->m_format == JOB_EXPORT_PCB_DRILL::DRILL_FORMAT::GERBER );
+        m_units->SetSelection( m_job->m_drillUnits == JOB_EXPORT_PCB_DRILL::DRILL_UNITS::INCHES );
+        m_zeros->SetSelection( static_cast<int>( m_job->m_zeroFormat ) );
+        updatePrecisionOptions();
+        m_Check_Minimal->SetValue( m_job->m_excellonMinimalHeader );
+
+        m_origin->SetSelection( m_job->m_drillOrigin == JOB_EXPORT_PCB_DRILL::DRILL_ORIGIN::PLOT );
+
+        m_Check_Mirror->SetValue( m_job->m_excellonMirrorY );
+        m_Check_Merge_PTH_NPTH->SetValue( m_job->m_excellonCombinePTHNPTH );
+        m_choiceDrillMap->SetSelection( static_cast<int>( m_job->m_mapFormat ) );
+        m_altDrillMode->SetValue( !m_job->m_excellonOvalDrillRoute );
+        m_cbGenerateMap->SetValue( m_job->m_generateMap );
     }
 
-    // Display hole counts:
-    m_PlatedPadsCountInfoMsg->SetLabel( wxString() << m_platedPadsHoleCount );
-    m_NotPlatedPadsCountInfoMsg->SetLabel( wxString() << m_notplatedPadsHoleCount );
-    m_ThroughViasInfoMsg->SetLabel( wxString() << m_throughViasCount );
-    m_MicroViasInfoMsg->SetLabel( wxString() << m_microViasCount );
-    m_BuriedViasInfoMsg->SetLabel( wxString() << m_blindOrBuriedViasCount );
-
-    // Output directory
-    m_outputDirectoryName->SetValue( m_plotOpts.GetOutputDirectory() );
-
     wxCommandEvent dummy;
     onFileFormatSelection( dummy );
+	return true;
+}
+
+
+void DIALOG_GENDRILL::initDialog()
+{
+    if( m_job )
+    {
+        SetTitle( m_job->GetSettingsDialogTitle() );
+    }
+    else
+    {
+        auto cfg = m_pcbEditFrame->GetPcbNewSettings();
+
+        g_merge_PTH_NPTH = cfg->m_GenDrill.merge_pth_npth;
+        g_minimalHeader = cfg->m_GenDrill.minimal_header;
+        g_mirror = cfg->m_GenDrill.mirror;
+        g_unitDrillIsInch = cfg->m_GenDrill.unit_drill_is_inch;
+        g_useRouteModeForOvalHoles = cfg->m_GenDrill.use_route_for_oval_holes;
+        g_drillFileType = cfg->m_GenDrill.drill_file_type;
+        g_mapFileType = cfg->m_GenDrill.map_file_type;
+        g_zerosFormat = cfg->m_GenDrill.zeros_format;
+        g_generateMap = cfg->m_GenDrill.generate_map;
+
+        // Ensure validity of g_mapFileType
+        if( g_mapFileType < 0 || g_mapFileType >= (int) m_choiceDrillMap->GetCount() )
+            g_mapFileType = m_choiceDrillMap->GetCount() - 1; // last item in list = default = PDF
+	}
+
+    // DIALOG_SHIM needs a unique hash_key because classname will be the same for both job and
+    // non-job versions (which have different sizes).
+    m_hash_key = TO_UTF8( GetTitle() );
+
+    m_drillOriginIsAuxAxis = m_plotOpts.GetUseAuxOrigin();
 }
 
 
@@ -209,21 +239,24 @@ void DIALOG_GENDRILL::onFileFormatSelection( wxCommandEvent& event )
 {
     bool enbl_Excellon = m_rbExcellon->GetValue();
 
-    m_drillFileType = enbl_Excellon ? 0 : 1;
+    g_drillFileType = enbl_Excellon ? 0 : 1;
 
-    m_Choice_Unit->Enable( enbl_Excellon );
-	m_Choice_Zeros_Format->Enable( enbl_Excellon );
+    m_unitsLabel->Enable( enbl_Excellon );
+    m_units->Enable( enbl_Excellon );
+    m_zerosLabel->Enable( enbl_Excellon );
+	m_zeros->Enable( enbl_Excellon );
     m_Check_Mirror->Enable( enbl_Excellon );
     m_Check_Minimal->Enable( enbl_Excellon );
     m_Check_Merge_PTH_NPTH->Enable( enbl_Excellon );
-    m_radioBoxOvalHoleMode->Enable( enbl_Excellon );
+    m_altDrillMode->Enable( enbl_Excellon );
 
     if( enbl_Excellon )
     {
-        UpdatePrecisionOptions();
+        updatePrecisionOptions();
     }
     else
     {
+        m_precisionLabel->Enable( true );
         m_staticTextPrecision->Enable( true );
         m_staticTextPrecision->SetLabel( m_plotOpts.GetGerberPrecision() == 6 ? wxT( "4.6" )
                                                                               : wxT( "4.5" ) );
@@ -231,50 +264,39 @@ void DIALOG_GENDRILL::onFileFormatSelection( wxCommandEvent& event )
 }
 
 
-void DIALOG_GENDRILL::UpdateConfig()
+void DIALOG_GENDRILL::updateConfig()
 {
     UpdateDrillParams();
 
-    auto cfg = m_pcbEditFrame->GetPcbNewSettings();
+    PCBNEW_SETTINGS* cfg = m_pcbEditFrame->GetPcbNewSettings();
 
-    cfg->m_GenDrill.merge_pth_npth           = m_Merge_PTH_NPTH;
-    cfg->m_GenDrill.minimal_header           = m_MinimalHeader;
-    cfg->m_GenDrill.mirror                   = m_Mirror;
-    cfg->m_GenDrill.unit_drill_is_inch       = m_UnitDrillIsInch;
-    cfg->m_GenDrill.use_route_for_oval_holes = m_UseRouteModeForOvalHoles;
-    cfg->m_GenDrill.drill_file_type          = m_drillFileType;
-    cfg->m_GenDrill.map_file_type            = m_mapFileType;
-    cfg->m_GenDrill.zeros_format             = m_ZerosFormat;
+    cfg->m_GenDrill.merge_pth_npth           = g_merge_PTH_NPTH;
+    cfg->m_GenDrill.minimal_header           = g_minimalHeader;
+    cfg->m_GenDrill.mirror                   = g_mirror;
+    cfg->m_GenDrill.unit_drill_is_inch       = g_unitDrillIsInch;
+    cfg->m_GenDrill.use_route_for_oval_holes = g_useRouteModeForOvalHoles;
+    cfg->m_GenDrill.drill_file_type          = g_drillFileType;
+    cfg->m_GenDrill.map_file_type            = g_mapFileType;
+    cfg->m_GenDrill.zeros_format             = g_zerosFormat;
+    cfg->m_GenDrill.generate_map             = g_generateMap;
 }
 
 
-void DIALOG_GENDRILL::OnSelDrillUnitsSelected( wxCommandEvent& event )
+void DIALOG_GENDRILL::onSelDrillUnitsSelected( wxCommandEvent& event )
 {
-    UpdatePrecisionOptions();
+    updatePrecisionOptions();
 }
 
 
-void DIALOG_GENDRILL::OnGenMapFile( wxCommandEvent& event )
+void DIALOG_GENDRILL::onSelZerosFmtSelected( wxCommandEvent& event )
 {
-    GenDrillAndMapFiles( false, true );
+    updatePrecisionOptions();
 }
 
 
-void DIALOG_GENDRILL::OnGenDrillFile( wxCommandEvent& event )
+void DIALOG_GENDRILL::updatePrecisionOptions()
 {
-    GenDrillAndMapFiles( true, false );
-}
-
-
-void DIALOG_GENDRILL::OnSelZerosFmtSelected( wxCommandEvent& event )
-{
-    UpdatePrecisionOptions();
-}
-
-
-void DIALOG_GENDRILL::UpdatePrecisionOptions()
-{
-    if( m_Choice_Unit->GetSelection()== 1 )
+    if( m_units->GetSelection() == 1 )
     {
         // Units = inches
         m_staticTextPrecision->SetLabel( precisionListForInches.GetPrecisionString() );
@@ -285,14 +307,20 @@ void DIALOG_GENDRILL::UpdatePrecisionOptions()
         m_staticTextPrecision->SetLabel( precisionListForMetric.GetPrecisionString() );
     }
 
-    if( m_Choice_Zeros_Format->GetSelection() == EXCELLON_WRITER::DECIMAL_FORMAT )
+    if( m_zeros->GetSelection() == EXCELLON_WRITER::DECIMAL_FORMAT )
+    {
+        m_precisionLabel->Enable( false );
         m_staticTextPrecision->Enable( false );
+    }
     else
+    {
+        m_precisionLabel->Enable( true );
         m_staticTextPrecision->Enable( true );
+    }
 }
 
 
-void DIALOG_GENDRILL::OnOutputDirectoryBrowseClicked( wxCommandEvent& event )
+void DIALOG_GENDRILL::onOutputDirectoryBrowseClicked( wxCommandEvent& event )
 {
     // Build the absolute path of current output directory to preselect it in the file browser.
     wxString path = ExpandEnvVarSubstitutions( m_outputDirectoryName->GetValue(), &Prj() );
@@ -333,27 +361,28 @@ void DIALOG_GENDRILL::UpdateDrillParams()
     dirStr = m_outputDirectoryName->GetValue();
     dirStr.Replace( wxT( "\\" ), wxT( "/" ) );
     m_plotOpts.SetOutputDirectory( dirStr );
-    m_drillOriginIsAuxAxis = m_Choice_Drill_Offset->GetSelection() == 1;
+    m_drillOriginIsAuxAxis = m_origin->GetSelection() == 1;
     m_plotOpts.SetUseAuxOrigin( m_drillOriginIsAuxAxis );
 
-    m_mapFileType = m_Choice_Drill_Map->GetSelection();
+    g_mapFileType = m_choiceDrillMap->GetSelection();
 
-    m_UnitDrillIsInch = (m_Choice_Unit->GetSelection() == 0) ? false : true;
-    m_MinimalHeader = m_Check_Minimal->IsChecked();
-    m_Mirror = m_Check_Mirror->IsChecked();
-    m_Merge_PTH_NPTH = m_Check_Merge_PTH_NPTH->IsChecked();
-    m_ZerosFormat = m_Choice_Zeros_Format->GetSelection();
-    m_UseRouteModeForOvalHoles = m_radioBoxOvalHoleMode->GetSelection() == 0;
+    g_unitDrillIsInch = ( m_units->GetSelection() == 0 ) ? false : true;
+    g_minimalHeader = m_Check_Minimal->IsChecked();
+    g_mirror = m_Check_Mirror->IsChecked();
+    g_merge_PTH_NPTH = m_Check_Merge_PTH_NPTH->IsChecked();
+    g_zerosFormat = m_zeros->GetSelection();
+    g_useRouteModeForOvalHoles = !m_altDrillMode->GetValue();
+    g_generateMap = m_cbGenerateMap->IsChecked();
 
-    if( m_Choice_Drill_Offset->GetSelection() == 0 )
-        m_DrillFileOffset = VECTOR2I( 0, 0 );
+    if( m_origin->GetSelection() == 0 )
+        g_drillFileOffset = VECTOR2I( 0, 0 );
     else
-        m_DrillFileOffset = m_board->GetDesignSettings().GetAuxOrigin();
+        g_drillFileOffset = m_board->GetDesignSettings().GetAuxOrigin();
 
-    if( m_UnitDrillIsInch )
-        m_Precision = precisionListForInches;
+    if( g_unitDrillIsInch )
+        g_precision = precisionListForInches;
     else
-        m_Precision = precisionListForMetric;
+        g_precision = precisionListForMetric;
 
     if( !m_plotOpts.IsSameAs( m_board->GetPlotOptions() ) )
     {
@@ -363,9 +392,9 @@ void DIALOG_GENDRILL::UpdateDrillParams()
 }
 
 
-void DIALOG_GENDRILL::GenDrillAndMapFiles( bool aGenDrill, bool aGenMap )
+void DIALOG_GENDRILL::genDrillAndMapFiles( bool aGenDrill, bool aGenMap )
 {
-    UpdateConfig();     // set params and Save drill options
+    updateConfig();     // set params and Save drill options
 
     m_pcbEditFrame->ClearMsgPanel();
     WX_TEXT_CTRL_REPORTER reporter( m_messagesBox );
@@ -379,7 +408,7 @@ void DIALOG_GENDRILL::GenDrillAndMapFiles( bool aGenDrill, bool aGenMap )
         PLOT_FORMAT::PDF
     };
 
-    unsigned choice = (unsigned) m_Choice_Drill_Map->GetSelection();
+    unsigned choice = (unsigned) m_choiceDrillMap->GetSelection();
 
     if( choice >= arrayDim( filefmt ) )
         choice = arrayDim( filefmt )-1;     // Last choice = PDF
@@ -410,13 +439,13 @@ void DIALOG_GENDRILL::GenDrillAndMapFiles( bool aGenDrill, bool aGenMap )
         return;
     }
 
-    if( m_drillFileType == 0 )
+    if( g_drillFileType == 0 )
     {
         EXCELLON_WRITER excellonWriter( m_board );
-        excellonWriter.SetFormat( !m_UnitDrillIsInch, (EXCELLON_WRITER::ZEROS_FMT) m_ZerosFormat,
-                                  m_Precision.m_Lhs, m_Precision.m_Rhs );
-        excellonWriter.SetOptions( m_Mirror, m_MinimalHeader, m_DrillFileOffset, m_Merge_PTH_NPTH );
-        excellonWriter.SetRouteModeForOvalHoles( m_UseRouteModeForOvalHoles );
+        excellonWriter.SetFormat( !g_unitDrillIsInch, (EXCELLON_WRITER::ZEROS_FMT) g_zerosFormat,
+                                  g_precision.m_Lhs, g_precision.m_Rhs );
+        excellonWriter.SetOptions( g_mirror, g_minimalHeader, g_drillFileOffset, g_merge_PTH_NPTH );
+        excellonWriter.SetRouteModeForOvalHoles( g_useRouteModeForOvalHoles );
         excellonWriter.SetMapFileFormat( filefmt[choice] );
 
         excellonWriter.CreateDrillandMapFilesSet( outputDir.GetFullPath(), aGenDrill, aGenMap,
@@ -429,7 +458,7 @@ void DIALOG_GENDRILL::GenDrillAndMapFiles( bool aGenDrill, bool aGenMap )
         // (SetFormat() accept 5 or 6, and any other value set the precision to 5)
         // the integer part precision is always 4, and units always mm
         gerberWriter.SetFormat( m_plotOpts.GetGerberPrecision() );
-        gerberWriter.SetOptions( m_DrillFileOffset );
+        gerberWriter.SetOptions( g_drillFileOffset );
         gerberWriter.SetMapFileFormat( filefmt[choice] );
 
         gerberWriter.CreateDrillandMapFilesSet( outputDir.GetFullPath(), aGenDrill, aGenMap,
@@ -438,9 +467,9 @@ void DIALOG_GENDRILL::GenDrillAndMapFiles( bool aGenDrill, bool aGenMap )
 }
 
 
-void DIALOG_GENDRILL::OnGenReportFile( wxCommandEvent& event )
+void DIALOG_GENDRILL::onGenReportFile( wxCommandEvent& event )
 {
-    UpdateConfig(); // set params and Save drill options
+    updateConfig(); // set params and Save drill options
 
     wxFileName fn = m_board->GetFileName();
 
@@ -463,10 +492,10 @@ void DIALOG_GENDRILL::OnGenReportFile( wxCommandEvent& event )
 
     // Info is slightly different between Excellon and Gerber
     // (file ext, Merge PTH/NPTH option)
-    if( m_drillFileType == 0 )
+    if( g_drillFileType == 0 )
     {
         EXCELLON_WRITER excellonWriter( m_board );
-        excellonWriter.SetMergeOption( m_Merge_PTH_NPTH );
+        excellonWriter.SetMergeOption( g_merge_PTH_NPTH );
         success = excellonWriter.GenDrillReportFile( dlg.GetPath() );
     }
     else
