@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2004-2018 Jean-Pierre Charras jp.charras at wanadoo.fr
- * Copyright (C) 2010-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,8 +30,6 @@
 #include <board.h>
 #include <footprint.h>
 #include <pcb_text.h>
-#include <fp_text.h>
-#include <pcbnew.h>
 #include <project.h>
 #include <pcb_edit_frame.h>
 #include <pcb_layer_box_selector.h>
@@ -39,13 +37,10 @@
 #include <scintilla_tricks.h>
 
 
-DIALOG_TEXT_PROPERTIES::DIALOG_TEXT_PROPERTIES( PCB_BASE_EDIT_FRAME* aParent, BOARD_ITEM* aItem ) :
+DIALOG_TEXT_PROPERTIES::DIALOG_TEXT_PROPERTIES( PCB_BASE_EDIT_FRAME* aParent, PCB_TEXT* aText ) :
         DIALOG_TEXT_PROPERTIES_BASE( aParent ),
         m_frame( aParent ),
-        m_item( aItem ),
-        m_edaText( nullptr ),
-        m_fpText( nullptr ),
-        m_pcbText( nullptr ),
+        m_item( aText ),
         m_textWidth( aParent, m_SizeXLabel, m_SizeXCtrl, m_SizeXUnits ),
         m_textHeight( aParent, m_SizeYLabel, m_SizeYCtrl, m_SizeYUnits ),
         m_thickness( aParent, m_ThicknessLabel, m_ThicknessCtrl, m_ThicknessUnits ),
@@ -56,15 +51,41 @@ DIALOG_TEXT_PROPERTIES::DIALOG_TEXT_PROPERTIES( PCB_BASE_EDIT_FRAME* aParent, BO
     wxString title;
 
     // Configure display origin transforms
-    m_posX.SetCoordType( ORIGIN_TRANSFORMS::ABS_X_COORD );
-    m_posY.SetCoordType( ORIGIN_TRANSFORMS::ABS_Y_COORD );
+    if( m_item->GetParentFootprint() )
+    {
+        m_posX.SetCoordType( ORIGIN_TRANSFORMS::REL_X_COORD );
+        m_posY.SetCoordType( ORIGIN_TRANSFORMS::REL_Y_COORD );
+    }
+    else
+    {
+        m_posX.SetCoordType( ORIGIN_TRANSFORMS::ABS_X_COORD );
+        m_posY.SetCoordType( ORIGIN_TRANSFORMS::ABS_Y_COORD );
+    }
 
     m_MultiLineText->SetEOLMode( wxSTC_EOL_LF );
 
+#ifdef _WIN32
+    // Without this setting, on Windows, some esoteric unicode chars create display issue
+    // in a wxStyledTextCtrl.
+    // for SetTechnology() info, see https://www.scintilla.org/ScintillaDoc.html#SCI_SETTECHNOLOGY
+    m_MultiLineText->SetTechnology(wxSTC_TECHNOLOGY_DIRECTWRITE);
+#endif
+
     m_scintillaTricks = new SCINTILLA_TRICKS( m_MultiLineText, wxT( "{}" ), false,
-            [this]()
+            // onAcceptFn
+            [this]( wxKeyEvent& aEvent )
             {
                 wxPostEvent( this, wxCommandEvent( wxEVT_COMMAND_BUTTON_CLICKED, wxID_OK ) );
+            },
+            // onCharFn
+            [this]( wxStyledTextEvent& aEvent )
+            {
+                m_scintillaTricks->DoTextVarAutocomplete(
+                        // getTokensFn
+                        [this]( const wxString& xRef, wxArrayString* tokens )
+                        {
+                            m_frame->GetContextualTextVars( m_item, xRef, tokens );
+                        } );
             } );
 
     // A hack which causes Scintilla to auto-size the text editor canvas
@@ -72,23 +93,35 @@ DIALOG_TEXT_PROPERTIES::DIALOG_TEXT_PROPERTIES( PCB_BASE_EDIT_FRAME* aParent, BO
     m_MultiLineText->SetScrollWidth( 1 );
     m_MultiLineText->SetScrollWidthTracking( true );
 
-    if( m_item->Type() == PCB_FP_TEXT_T )
+    if( m_item->GetParentFootprint() )
     {
-        m_fpText = (FP_TEXT*) m_item;
-        m_edaText = static_cast<EDA_TEXT*>( m_fpText );
+        m_PositionXLabel->SetLabel( _( "Offset X:" ) );
+        m_PositionYLabel->SetLabel( _( "Offset Y:" ) );
 
-        switch( m_fpText->GetType() )
+        if( m_item->Type() == PCB_FIELD_T )
         {
-        case FP_TEXT::TEXT_is_REFERENCE: title = _( "Footprint Reference Properties" ); break;
-        case FP_TEXT::TEXT_is_VALUE:     title = _( "Footprint Value Properties" );     break;
-        case FP_TEXT::TEXT_is_DIVERS:    title = _( "Footprint Text Properties" );      break;
+            PCB_FIELD* field = static_cast<PCB_FIELD*>( m_item );
+
+            if( field->IsReference() )
+            {
+                title = _( "Footprint Reference Properties" );
+                m_TextLabel->SetLabel( _( "Reference:" ) );
+            }
+            else if( field->IsValue() )
+            {
+                title = _( "Footprint Value Properties" );
+                m_TextLabel->SetLabel( _( "Value:" ) );
+            }
+            else
+            {
+                title = _( "Footprint Field Properties" );
+                m_TextLabel->SetLabel( _( "Text:" ) );
+            }
         }
-
-        switch( m_fpText->GetType() )
+        else
         {
-        case FP_TEXT::TEXT_is_REFERENCE: m_TextLabel->SetLabel( _( "Reference:" ) ); break;
-        case FP_TEXT::TEXT_is_VALUE:     m_TextLabel->SetLabel( _( "Value:" ) );     break;
-        case FP_TEXT::TEXT_is_DIVERS:    m_TextLabel->SetLabel( _( "Text:" ) );      break;
+            title = _( "Footprint Text Properties" );
+            m_TextLabel->SetLabel( _( "Text:" ) );
         }
 
         SetInitialFocus( m_SingleLineText );
@@ -117,9 +150,6 @@ DIALOG_TEXT_PROPERTIES::DIALOG_TEXT_PROPERTIES( PCB_BASE_EDIT_FRAME* aParent, BO
     {
         title = _( "Text Properties" );
 
-        m_pcbText = (PCB_TEXT*) aItem;
-        m_edaText = static_cast<EDA_TEXT*>( m_pcbText );
-
         SetInitialFocus( m_MultiLineText );
         m_SingleLineSizer->Show( false );
 
@@ -147,32 +177,32 @@ DIALOG_TEXT_PROPERTIES::DIALOG_TEXT_PROPERTIES( PCB_BASE_EDIT_FRAME* aParent, BO
     }
 
     m_bold->SetIsCheckButton();
-    m_bold->SetBitmap( KiBitmap( BITMAPS::text_bold ) );
+    m_bold->SetBitmap( KiBitmapBundle( BITMAPS::text_bold ) );
     m_italic->SetIsCheckButton();
-    m_italic->SetBitmap( KiBitmap( BITMAPS::text_italic ) );
+    m_italic->SetBitmap( KiBitmapBundle( BITMAPS::text_italic ) );
 
     m_separator1->SetIsSeparator();
 
     m_alignLeft->SetIsRadioButton();
-    m_alignLeft->SetBitmap( KiBitmap( BITMAPS::text_align_left ) );
+    m_alignLeft->SetBitmap( KiBitmapBundle( BITMAPS::text_align_left ) );
     m_alignCenter->SetIsRadioButton();
-    m_alignCenter->SetBitmap( KiBitmap( BITMAPS::text_align_center ) );
+    m_alignCenter->SetBitmap( KiBitmapBundle( BITMAPS::text_align_center ) );
     m_alignRight->SetIsRadioButton();
-    m_alignRight->SetBitmap( KiBitmap( BITMAPS::text_align_right ) );
+    m_alignRight->SetBitmap( KiBitmapBundle( BITMAPS::text_align_right ) );
 
     m_separator2->SetIsSeparator();
 
     m_valignBottom->SetIsRadioButton();
-    m_valignBottom->SetBitmap( KiBitmap( BITMAPS::text_valign_bottom ) );
+    m_valignBottom->SetBitmap( KiBitmapBundle( BITMAPS::text_valign_bottom ) );
     m_valignCenter->SetIsRadioButton();
-    m_valignCenter->SetBitmap( KiBitmap( BITMAPS::text_valign_center ) );
+    m_valignCenter->SetBitmap( KiBitmapBundle( BITMAPS::text_valign_center ) );
     m_valignTop->SetIsRadioButton();
-    m_valignTop->SetBitmap( KiBitmap( BITMAPS::text_valign_top ) );
+    m_valignTop->SetBitmap( KiBitmapBundle( BITMAPS::text_valign_top ) );
 
     m_separator3->SetIsSeparator();
 
     m_mirrored->SetIsCheckButton();
-    m_mirrored->SetBitmap( KiBitmap( BITMAPS::text_mirrored ) );
+    m_mirrored->SetBitmap( KiBitmapBundle( BITMAPS::text_mirrored ) );
 
     SetTitle( title );
     m_hash_key = title;
@@ -206,11 +236,6 @@ DIALOG_TEXT_PROPERTIES::DIALOG_TEXT_PROPERTIES( PCB_BASE_EDIT_FRAME* aParent, BO
     Connect( wxEVT_CHAR_HOOK, wxKeyEventHandler( DIALOG_TEXT_PROPERTIES::OnCharHook ),
              nullptr, this );
 
-    m_MultiLineText->Bind( wxEVT_STC_CHARADDED,
-                           &DIALOG_TEXT_PROPERTIES::onScintillaCharAdded, this );
-    m_MultiLineText->Bind( wxEVT_STC_AUTOCOMP_CHAR_DELETED,
-                           &DIALOG_TEXT_PROPERTIES::onScintillaCharAdded, this );
-
     finishDialogSettings();
 }
 
@@ -224,28 +249,30 @@ DIALOG_TEXT_PROPERTIES::~DIALOG_TEXT_PROPERTIES()
 }
 
 
-void PCB_BASE_EDIT_FRAME::ShowTextPropertiesDialog( BOARD_ITEM* aText )
+void PCB_BASE_EDIT_FRAME::ShowTextPropertiesDialog( PCB_TEXT* aText )
 {
     DIALOG_TEXT_PROPERTIES dlg( this, aText );
+
+    // QuasiModal required for Scintilla auto-complete
     dlg.ShowQuasiModal();
 }
 
 
 void DIALOG_TEXT_PROPERTIES::OnSetFocusText( wxFocusEvent& event )
 {
+    if( m_item->Type() == PCB_FIELD_T && static_cast<PCB_FIELD*>( m_item )->IsReference() )
+    {
 #ifdef __WXGTK__
-    // Force an update of the text control before setting the text selection
-    // This is needed because GTK seems to ignore the selection on first update
-    //
-    // Note that we can't do this on OSX as it tends to provoke Apple's
-    // "[NSAlert runModal] may not be invoked inside of transaction begin/commit pair"
-    // bug.  See: https://bugs.launchpad.net/kicad/+bug/1837225
-    if( m_fpText->GetType() == FP_TEXT::TEXT_is_REFERENCE )
+        // Force an update of the text control before setting the text selection
+        // This is needed because GTK seems to ignore the selection on first update
+        //
+        // Note that we can't do this on OSX as it tends to provoke Apple's
+        // "[NSAlert runModal] may not be invoked inside of transaction begin/commit pair"
+        // bug.  See: https://bugs.launchpad.net/kicad/+bug/1837225
         m_SingleLineText->Update();
 #endif
-
-    if( m_fpText->GetType() == FP_TEXT::TEXT_is_REFERENCE )
         KIUI::SelectReferenceNumber( static_cast<wxTextEntry*>( m_SingleLineText ) );
+    }
     else
         m_SingleLineText->SetSelection( -1, -1 );
 
@@ -253,102 +280,39 @@ void DIALOG_TEXT_PROPERTIES::OnSetFocusText( wxFocusEvent& event )
 }
 
 
-void DIALOG_TEXT_PROPERTIES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
-{
-    wxStyledTextCtrl* te = m_MultiLineText;
-    wxArrayString     autocompleteTokens;
-    int               text_pos = te->GetCurrentPos();
-    int               start = te->WordStartPosition( text_pos, true );
-    wxString          partial;
-
-    auto textVarRef =
-            [&]( int pos )
-            {
-                return pos >= 2 && te->GetCharAt( pos-2 ) == '$' && te->GetCharAt( pos-1 ) == '{';
-            };
-
-    // Check for cross-reference
-    if( start > 1 && te->GetCharAt( start-1 ) == ':' )
-    {
-        int refStart = te->WordStartPosition( start-1, true );
-
-        if( textVarRef( refStart ) )
-        {
-            partial = te->GetRange( start, text_pos );
-
-            wxString ref = te->GetRange( refStart, start-1 );
-            BOARD*   board = m_item->GetBoard();
-
-            for( FOOTPRINT* candidate : board->Footprints() )
-            {
-                if( candidate->GetReference() == ref )
-                {
-                    candidate->GetContextualTextVars( &autocompleteTokens );
-                    break;
-                }
-            }
-        }
-    }
-    else if( textVarRef( start ) )
-    {
-        partial = te->GetTextRange( start, text_pos );
-
-        BOARD* board = m_item->GetBoard();
-
-        board->GetContextualTextVars( &autocompleteTokens );
-
-        if( m_fpText )
-        {
-            if( FOOTPRINT* footprint = static_cast<FOOTPRINT*>( m_fpText->GetParentFootprint() ) )
-                footprint->GetContextualTextVars( &autocompleteTokens );
-        }
-
-        for( std::pair<wxString, wxString> entry : board->GetProject()->GetTextVars() )
-            autocompleteTokens.push_back( entry.first );
-    }
-
-    m_scintillaTricks->DoAutocomplete( partial, autocompleteTokens );
-    m_MultiLineText->SetFocus();
-}
-
-
 bool DIALOG_TEXT_PROPERTIES::TransferDataToWindow()
 {
+    BOARD*     board = m_frame->GetBoard();
+    FOOTPRINT* parentFP = m_item->GetParentFootprint();
+    wxString   text = m_item->GetText();
+
+    // show text variable cross-references in a human-readable format
+    text = board->ConvertKIIDsToCrossReferences( UnescapeString( text ) );
+
     if( m_SingleLineText->IsShown() )
     {
-        m_SingleLineText->SetValue( m_edaText->GetText() );
+        m_SingleLineText->SetValue( text );
 
-        if( m_fpText && m_fpText->GetType() == FP_TEXT::TEXT_is_REFERENCE )
+        if( m_item->Type() == PCB_FIELD_T && static_cast<PCB_FIELD*>( m_item )->IsReference() )
             KIUI::SelectReferenceNumber( static_cast<wxTextEntry*>( m_SingleLineText ) );
         else
             m_SingleLineText->SetSelection( -1, -1 );
     }
     else if( m_MultiLineText->IsShown() )
     {
-        BOARD*   board = m_frame->GetBoard();
-        wxString converted = board->ConvertKIIDsToCrossReferences(
-                UnescapeString( m_edaText->GetText() ) );
-
-        m_MultiLineText->SetValue( converted );
+        m_MultiLineText->SetValue( text );
         m_MultiLineText->SetSelection( -1, -1 );
         m_MultiLineText->EmptyUndoBuffer();
     }
 
-    if( m_item->Type() == PCB_FP_TEXT_T && m_fpText )
+    if( parentFP )
     {
-        FOOTPRINT* footprint = dynamic_cast<FOOTPRINT*>( m_fpText->GetParent() );
-        wxString   msg;
-
-        if( footprint )
-        {
-            msg.Printf( _( "Footprint %s (%s), %s, rotated %.1f deg"),
-                        footprint->GetReference(),
-                        footprint->GetValue(),
-                        footprint->IsFlipped() ? _( "back side (mirrored)" ) : _( "front side" ),
-                        footprint->GetOrientation().AsDegrees() );
-        }
-
-        m_statusLine->SetLabel( msg );
+        m_statusLine->SetLabel( wxString::Format( _( "Footprint %s (%s), %s, rotated %.1f deg"),
+                                                  parentFP->GetReference(),
+                                                  parentFP->GetValue(),
+                                                  parentFP->IsFlipped() ? _( "back side (mirrored)" )
+                                                                        : _( "front side" ),
+                                                  parentFP->GetOrientation().AsDegrees() ) );
     }
     else
     {
@@ -360,39 +324,41 @@ bool DIALOG_TEXT_PROPERTIES::TransferDataToWindow()
     m_LayerSelectionCtrl->SetLayerSelection( m_item->GetLayer() );
     m_cbKnockout->SetValue( m_item->IsKnockout() );
 
-    m_fontCtrl->SetFontSelection( m_edaText->GetFont() );
+    m_fontCtrl->SetFontSelection( m_item->GetFont() );
 
-    m_textWidth.SetValue( m_edaText->GetTextSize().x );
-    m_textHeight.SetValue( m_edaText->GetTextSize().y );
-    m_thickness.SetValue( m_edaText->GetTextThickness() );
-    m_posX.SetValue( m_edaText->GetTextPos().x );
-    m_posY.SetValue( m_edaText->GetTextPos().y );
+    m_textWidth.SetValue( m_item->GetTextSize().x );
+    m_textHeight.SetValue( m_item->GetTextSize().y );
+    m_thickness.SetValue( m_item->GetTextThickness() );
+    m_posX.SetValue( m_item->GetFPRelativePosition().x );
+    m_posY.SetValue( m_item->GetFPRelativePosition().y );
 
-    m_Visible->SetValue( m_edaText->IsVisible() );
+    m_Visible->SetValue( m_item->IsVisible() );
 
-    if( m_fpText )
-        m_KeepUpright->SetValue( m_fpText->IsKeepUpright() );
+    if( parentFP )
+        m_KeepUpright->SetValue( m_item->IsKeepUpright() );
 
-    m_bold->Check( m_edaText->IsBold() );
-    m_italic->Check( m_edaText->IsItalic() );
+    m_bold->Check( m_item->IsBold() );
+    m_italic->Check( m_item->IsItalic() );
 
-    switch ( m_edaText->GetHorizJustify() )
+    switch ( m_item->GetHorizJustify() )
     {
-    case GR_TEXT_H_ALIGN_LEFT:   m_alignLeft->Check( true );   break;
-    case GR_TEXT_H_ALIGN_CENTER: m_alignCenter->Check( true ); break;
-    case GR_TEXT_H_ALIGN_RIGHT:  m_alignRight->Check( true );  break;
+    case GR_TEXT_H_ALIGN_LEFT:          m_alignLeft->Check( true );    break;
+    case GR_TEXT_H_ALIGN_CENTER:        m_alignCenter->Check( true );  break;
+    case GR_TEXT_H_ALIGN_RIGHT:         m_alignRight->Check( true );   break;
+    case GR_TEXT_H_ALIGN_INDETERMINATE:                                break;
     }
 
-    switch ( m_edaText->GetVertJustify() )
+    switch ( m_item->GetVertJustify() )
     {
-    case GR_TEXT_V_ALIGN_BOTTOM: m_valignBottom->Check( true ); break;
-    case GR_TEXT_V_ALIGN_CENTER: m_valignCenter->Check( true ); break;
-    case GR_TEXT_V_ALIGN_TOP:    m_valignTop->Check( true );    break;
+    case GR_TEXT_V_ALIGN_BOTTOM:        m_valignBottom->Check( true ); break;
+    case GR_TEXT_V_ALIGN_CENTER:        m_valignCenter->Check( true ); break;
+    case GR_TEXT_V_ALIGN_TOP:           m_valignTop->Check( true );    break;
+    case GR_TEXT_V_ALIGN_INDETERMINATE:                                break;
     }
 
-    m_mirrored->Check( m_edaText->IsMirrored() );
+    m_mirrored->Check( m_item->IsMirrored() );
 
-    EDA_ANGLE orientation = m_edaText->GetTextAngle();
+    EDA_ANGLE orientation = m_item->GetTextAngle();
     m_orientation.SetAngleValue( orientation.Normalize180() );
 
     return DIALOG_TEXT_PROPERTIES_BASE::TransferDataToWindow();
@@ -404,6 +370,7 @@ void DIALOG_TEXT_PROPERTIES::onFontSelected( wxCommandEvent & aEvent )
     if( KIFONT::FONT::IsStroke( aEvent.GetString() ) )
     {
         m_thickness.Show( true );
+        m_buttonUpdateTh->Show( true );
 
         int textSize = std::min( m_textWidth.GetValue(), m_textHeight.GetValue() );
         int thickness = m_thickness.GetValue();
@@ -414,6 +381,7 @@ void DIALOG_TEXT_PROPERTIES::onFontSelected( wxCommandEvent & aEvent )
     else
     {
         m_thickness.Show( false );
+        m_buttonUpdateTh->Show( false );
     }
 }
 
@@ -461,17 +429,33 @@ void DIALOG_TEXT_PROPERTIES::onThickness( wxCommandEvent& event )
 }
 
 
+void DIALOG_TEXT_PROPERTIES::updateTextThickness( wxCommandEvent &aEvent )
+{
+    int textSize = std::min( m_textWidth.GetValue(), m_textHeight.GetValue() );
+    int thickness;
+
+    // Calculate the "best" thickness from text size and bold option:
+    if( m_bold->IsChecked() )
+        thickness = GetPenSizeForBold( textSize );
+    else
+        thickness = GetPenSizeForNormal( textSize );
+
+    m_thickness.SetValue( thickness );
+}
+
+
 bool DIALOG_TEXT_PROPERTIES::TransferDataFromWindow()
 {
     if( !DIALOG_TEXT_PROPERTIES_BASE::TransferDataFromWindow() )
         return false;
 
-    if( !m_textWidth.Validate( TEXTS_MIN_SIZE, TEXTS_MAX_SIZE )
-        || !m_textHeight.Validate( TEXTS_MIN_SIZE, TEXTS_MAX_SIZE ) )
-    {
-        return false;
-    }
+    int minSize = pcbIUScale.mmToIU( TEXT_MIN_SIZE_MM );
+    int maxSize = pcbIUScale.mmToIU( TEXT_MAX_SIZE_MM );
 
+    if( !m_textWidth.Validate( minSize, maxSize ) || !m_textHeight.Validate( minSize, maxSize ) )
+        return false;
+
+    BOARD*       board = m_frame->GetBoard();
     BOARD_COMMIT commit( m_frame );
     commit.Modify( m_item );
 
@@ -488,13 +472,18 @@ bool DIALOG_TEXT_PROPERTIES::TransferDataFromWindow()
     if( m_SingleLineText->IsShown() )
     {
         if( !m_SingleLineText->GetValue().IsEmpty() )
-            m_edaText->SetText( m_SingleLineText->GetValue() );
+        {
+            // convert any text variable cross-references to their UUIDs
+            wxString txt = board->ConvertCrossReferencesToKIIDs( m_SingleLineText->GetValue() );
+
+            m_item->SetText( txt );
+        }
     }
     else if( m_MultiLineText->IsShown() )
     {
         if( !m_MultiLineText->GetValue().IsEmpty() )
         {
-            BOARD*   board = m_frame->GetBoard();
+            // convert any text variable cross-references to their UUIDs
             wxString txt = board->ConvertCrossReferencesToKIIDs( m_MultiLineText->GetValue() );
 
 #ifdef __WXMAC__
@@ -506,7 +495,7 @@ bool DIALOG_TEXT_PROPERTIES::TransferDataFromWindow()
             // drawing routines so strip the \r char.
             txt.Replace( wxT( "\r" ), wxT( "" ) );
 #endif
-            m_edaText->SetText( EscapeString( txt, CTX_QUOTED_STR ) );
+            m_item->SetText( EscapeString( txt, CTX_QUOTED_STR ) );
         }
     }
 
@@ -517,55 +506,52 @@ bool DIALOG_TEXT_PROPERTIES::TransferDataFromWindow()
 
     if( m_fontCtrl->HaveFontSelection() )
     {
-        m_edaText->SetFont( m_fontCtrl->GetFontSelection( m_bold->IsChecked(),
-                                                          m_italic->IsChecked() ) );
+        m_item->SetFont( m_fontCtrl->GetFontSelection( m_bold->IsChecked(),
+                                                       m_italic->IsChecked() ) );
     }
 
-    m_edaText->SetTextSize( wxSize( m_textWidth.GetValue(), m_textHeight.GetValue() ) );
-    m_edaText->SetTextThickness( m_thickness.GetValue() );
-    m_edaText->SetTextPos( wxPoint( m_posX.GetValue(), m_posY.GetValue() ) );
-
-    if( m_fpText )
-        m_fpText->SetLocalCoord();
+    m_item->SetTextSize( VECTOR2I( m_textWidth.GetValue(), m_textHeight.GetValue() ) );
+    m_item->SetTextThickness( m_thickness.GetValue() );
+    m_item->SetFPRelativePosition( VECTOR2I( m_posX.GetValue(), m_posY.GetValue() ) );
 
     // Test for acceptable values for thickness and size and clamp if fails
-    int maxPenWidth = Clamp_Text_PenSize( m_edaText->GetTextThickness(), m_edaText->GetTextSize() );
+    int maxPenWidth = ClampTextPenSize( m_item->GetTextThickness(), m_item->GetTextSize() );
 
-    if( m_edaText->GetTextThickness() > maxPenWidth )
+    if( m_item->GetTextThickness() > maxPenWidth )
     {
         DisplayError( this, _( "The text thickness is too large for the text size.\n"
                                "It will be clamped." ) );
-        m_edaText->SetTextThickness( maxPenWidth );
+        m_item->SetTextThickness( maxPenWidth );
     }
 
-    m_edaText->SetTextAngle( m_orientation.GetAngleValue().Normalize() );
+    m_item->SetTextAngle( m_orientation.GetAngleValue().Normalize() );
 
-    m_edaText->SetVisible( m_Visible->GetValue() );
+    m_item->SetVisible( m_Visible->GetValue() );
 
-    if( m_fpText )
-        m_fpText->SetKeepUpright( m_KeepUpright->GetValue() );
+    if( m_KeepUpright->IsShown() )
+        m_item->SetKeepUpright( m_KeepUpright->GetValue() );
 
-    m_edaText->SetBold( m_bold->IsChecked() );
-    m_edaText->SetItalic( m_italic->IsChecked() );
+    m_item->SetBoldFlag( m_bold->IsChecked() );
+    m_item->SetItalicFlag( m_italic->IsChecked() );
 
     if( m_alignLeft->IsChecked() )
-        m_edaText->SetHorizJustify( GR_TEXT_H_ALIGN_LEFT );
+        m_item->SetHorizJustify( GR_TEXT_H_ALIGN_LEFT );
     else if( m_alignCenter->IsChecked() )
-        m_edaText->SetHorizJustify( GR_TEXT_H_ALIGN_CENTER );
+        m_item->SetHorizJustify( GR_TEXT_H_ALIGN_CENTER );
     else
-        m_edaText->SetHorizJustify( GR_TEXT_H_ALIGN_RIGHT );
+        m_item->SetHorizJustify( GR_TEXT_H_ALIGN_RIGHT );
 
     if( m_valignBottom->IsChecked() )
-        m_edaText->SetVertJustify ( GR_TEXT_V_ALIGN_BOTTOM );
+        m_item->SetVertJustify ( GR_TEXT_V_ALIGN_BOTTOM );
     else if( m_valignCenter->IsChecked() )
-        m_edaText->SetVertJustify( GR_TEXT_V_ALIGN_CENTER );
+        m_item->SetVertJustify( GR_TEXT_V_ALIGN_CENTER );
     else
-        m_edaText->SetVertJustify( GR_TEXT_V_ALIGN_TOP );
+        m_item->SetVertJustify( GR_TEXT_V_ALIGN_TOP );
 
-    m_edaText->SetMirrored( m_mirrored->IsChecked() );
+    m_item->SetMirrored( m_mirrored->IsChecked() );
 
     if( pushCommit )
-        commit.Push( _( "Change text properties" ) );
+        commit.Push( _( "Edit Text Properties" ) );
 
     return true;
 }
