@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2020-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,9 +23,10 @@
 
 #include <bitmaps.h>
 #include <confirm.h>
+#include <widgets/std_bitmap_button.h>
 #include <widgets/paged_dialog.h>
 #include <pcb_edit_frame.h>
-#include <pcb_expr_evaluator.h>
+#include <pcbexpr_evaluator.h>
 #include <board.h>
 #include <board_design_settings.h>
 #include <project.h>
@@ -33,24 +34,30 @@
 #include <tool/tool_manager.h>
 #include <panel_setup_rules.h>
 #include <widgets/wx_html_report_box.h>
-#include <wx/treebook.h>
 #include <dialogs/html_message_box.h>
 #include <scintilla_tricks.h>
 #include <drc/drc_rule_parser.h>
 #include <tools/drc_tool.h>
 #include <pgm_base.h>
+#include <wildcards_and_files_ext.h>
 
-PANEL_SETUP_RULES::PANEL_SETUP_RULES( PAGED_DIALOG* aParent, PCB_EDIT_FRAME* aFrame ) :
-        PANEL_SETUP_RULES_BASE( aParent->GetTreebook() ),
-        m_Parent( aParent ),
+PANEL_SETUP_RULES::PANEL_SETUP_RULES( wxWindow* aParentWindow, PCB_EDIT_FRAME* aFrame ) :
+        PANEL_SETUP_RULES_BASE( aParentWindow ),
         m_frame( aFrame ),
         m_scintillaTricks( nullptr ),
         m_helpWindow( nullptr )
 {
     m_scintillaTricks = new SCINTILLA_TRICKS( m_textEditor, wxT( "()" ), false,
-            [this]()
+            // onAcceptFn
+            [this]( wxKeyEvent& aEvent )
             {
-                wxPostEvent( m_Parent, wxCommandEvent( wxEVT_COMMAND_BUTTON_CLICKED, wxID_OK ) );
+                wxPostEvent( PAGED_DIALOG::GetDialog( this ),
+                             wxCommandEvent( wxEVT_COMMAND_BUTTON_CLICKED, wxID_OK ) );
+            },
+            // onCharFn
+            [this]( wxStyledTextEvent& aEvent )
+            {
+                onScintillaCharAdded( aEvent );
             } );
 
     m_textEditor->AutoCompSetSeparator( '|' );
@@ -63,20 +70,25 @@ PANEL_SETUP_RULES::PANEL_SETUP_RULES( PAGED_DIALOG* aParent, PCB_EDIT_FRAME* aFr
     m_pinTypeRegex.Compile( "^Pin_Type\\s*[!=]=\\s*$", wxRE_ADVANCED );
     m_fabPropRegex.Compile( "^Fabrication_Property\\s*[!=]=\\s*$", wxRE_ADVANCED );
     m_shapeRegex.Compile( "^Shape\\s*[!=]=\\s*$", wxRE_ADVANCED );
+    m_padShapeRegex.Compile( "^Pad_Shape\\s*[!=]=\\s*$", wxRE_ADVANCED );
+    m_padConnectionsRegex.Compile( "^Pad_Connections\\s*[!=]=\\s*$", wxRE_ADVANCED );
+    m_zoneConnStyleRegex.Compile( "^Zone_Connection_Style\\s*[!=]=\\s*$", wxRE_ADVANCED );
+    m_lineStyleRegex.Compile( "^Line_Style\\s*[!=]=\\s*$", wxRE_ADVANCED );
+    m_hJustRegex.Compile( "^Horizontal_Justification\\s*[!=]=\\s*$", wxRE_ADVANCED );
+    m_vJustRegex.Compile( "^Vertical_Justification\\s*[!=]=\\s*$", wxRE_ADVANCED );
 
-    m_compileButton->SetBitmap( KiBitmap( BITMAPS::drc ) );
+    m_compileButton->SetBitmap( KiBitmapBundle( BITMAPS::drc ) );
 
     m_textEditor->SetZoom( Pgm().GetCommonSettings()->m_Appearance.text_editor_zoom );
 
     m_textEditor->UsePopUp( 0 );
-    m_textEditor->Bind( wxEVT_STC_CHARADDED, &PANEL_SETUP_RULES::onScintillaCharAdded, this );
-    m_textEditor->Bind( wxEVT_STC_AUTOCOMP_CHAR_DELETED, &PANEL_SETUP_RULES::onScintillaCharAdded, this );
     m_textEditor->Bind( wxEVT_CHAR_HOOK, &PANEL_SETUP_RULES::onCharHook, this );
 }
 
 
 PANEL_SETUP_RULES::~PANEL_SETUP_RULES( )
 {
+    m_textEditor->Unbind( wxEVT_CHAR_HOOK, &PANEL_SETUP_RULES::onCharHook, this );
     Pgm().GetCommonSettings()->m_Appearance.text_editor_zoom = m_textEditor->GetZoom();
 
     delete m_scintillaTricks;
@@ -92,8 +104,13 @@ void PANEL_SETUP_RULES::onCharHook( wxKeyEvent& aEvent )
     {
         if( m_originalText != m_textEditor->GetText() )
         {
-            if( !IsOK( this, _( "Cancel Changes?" ) ) )
-                return;
+            if( IsOK( wxGetTopLevelParent( this ), _( "Cancel Changes?" ) ) )
+            {
+                m_textEditor->SetText( m_originalText );
+                m_textEditor->SelectAll();
+            }
+
+            return;
         }
     }
 
@@ -170,7 +187,15 @@ void PANEL_SETUP_RULES::OnContextMenu(wxMouseEvent &event)
 
 void PANEL_SETUP_RULES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
 {
-    m_Parent->SetModified();
+    if( aEvent.GetModifiers() == wxMOD_CONTROL && aEvent.GetKey() == ' ' )
+    {
+        // This is just a short-cut for do-auto-complete
+    }
+    else
+    {
+        PAGED_DIALOG::GetDialog( this )->SetModified();
+    }
+
     m_textEditor->SearchAnchor();
 
     wxString rules = m_textEditor->GetText();
@@ -213,9 +238,43 @@ void PANEL_SETUP_RULES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
                     || token == wxT( "zone" );
             };
 
+    auto isConstraintTypeToken =
+            []( const wxString& token ) -> bool
+            {
+                return token == wxT( "annular_width" )
+                    || token == wxT( "assertion" )
+                    || token == wxT( "clearance" )
+                    || token == wxT( "connection_width" )
+                    || token == wxT( "courtyard_clearance" )
+                    || token == wxT( "diff_pair_gap" )
+                    || token == wxT( "diff_pair_uncoupled" )
+                    || token == wxT( "disallow" )
+                    || token == wxT( "edge_clearance" )
+                    || token == wxT( "length" )
+                    || token == wxT( "hole_clearance" )
+                    || token == wxT( "hole_size" )
+                    || token == wxT( "hole_to_hole" )
+                    || token == wxT( "min_resolved_spokes" )
+                    || token == wxT( "physical_clearance" )
+                    || token == wxT( "physical_hole_clearance" )
+                    || token == wxT( "silk_clearance" )
+                    || token == wxT( "skew" )
+                    || token == wxT( "text_height" )
+                    || token == wxT( "text_thickness" )
+                    || token == wxT( "thermal_relief_gap" )
+                    || token == wxT( "thermal_spoke_width" )
+                    || token == wxT( "track_width" )
+                    || token == wxT( "track_angle" )
+                    || token == wxT( "track_segment_length" )
+                    || token == wxT( "via_count" )
+                    || token == wxT( "via_diameter" )
+                    || token == wxT( "zone_connection" );
+            };
+
     std::stack<wxString> sexprs;
     wxString             partial;
     wxString             last;
+    wxString             constraintType;
     int                  context = NONE;
     int                  expr_context = NONE;
 
@@ -288,22 +347,49 @@ void PANEL_SETUP_RULES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
             }
 
             if( !sexprs.empty() )
-                sexprs.pop();
+            {
+                // Ignore argument-less tokens
+                if( partial == wxT( "within_diff_pairs" ) )
+                {
+                    partial = wxEmptyString;
+                }
+                else
+                {
+                    if( sexprs.top() == wxT( "constraint" ) )
+                    {
+                        constraintType = wxEmptyString;
+                    }
+
+                    sexprs.pop();
+                }
+            }
 
             context = NONE;
         }
         else if( c == ' ' )
         {
-            if( context == SEXPR_OPEN && ( partial == wxT( "constraint" )
-                                        || partial == wxT( "disallow" )
-                                        || partial == wxT( "layer" )
-                                        || partial == wxT( "severity" ) ) )
+            if( context == SEXPR_OPEN && !partial.IsEmpty() )
             {
                 m_textEditor->AutoCompCancel();
                 sexprs.push( partial );
 
+                if(    partial == wxT( "constraint" )
+                    || partial == wxT( "layer" )
+                    || partial == wxT( "severity" ) )
+                {
+                    context = SEXPR_TOKEN;
+                }
+                else if( partial == wxT( "rule" )
+                      || partial == wxT( "condition" ) )
+                {
+                    context = SEXPR_STRING;
+                }
+                else
+                {
+                    context = NONE;
+                }
+
                 partial = wxEmptyString;
-                context = SEXPR_TOKEN;
                 continue;
             }
             else if( partial == wxT( "disallow" )
@@ -318,9 +404,7 @@ void PANEL_SETUP_RULES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
                 context = SEXPR_TOKEN;
                 continue;
             }
-            else if( partial == wxT( "rule" )
-                  || partial == wxT( "assertion" )
-                  || partial == wxT( "condition" ) )
+            else if( partial == wxT( "assertion" ) )
             {
                 m_textEditor->AutoCompCancel();
                 sexprs.push( partial );
@@ -329,6 +413,10 @@ void PANEL_SETUP_RULES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
                 context = SEXPR_STRING;
                 continue;
             }
+            else if( isConstraintTypeToken( partial ) )
+            {
+                constraintType = partial;
+            }
 
             context = NONE;
         }
@@ -356,7 +444,10 @@ void PANEL_SETUP_RULES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
         }
         else if( sexprs.top() == wxT( "constraint" ) )
         {
-            tokens = wxT( "max|min|opt" );
+            if( constraintType == wxT( "skew" ) )
+                tokens = wxT( "max|min|opt|within_diff_pairs" );
+            else
+                tokens = wxT( "max|min|opt" );
         }
     }
     else if( context == SEXPR_TOKEN )
@@ -372,6 +463,7 @@ void PANEL_SETUP_RULES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
                           "clearance|"
                           "connection_width|"
                           "courtyard_clearance|"
+                          "creepage|"
                           "diff_pair_gap|"
                           "diff_pair_uncoupled|"
                           "disallow|"
@@ -390,6 +482,8 @@ void PANEL_SETUP_RULES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
                           "thermal_relief_gap|"
                           "thermal_spoke_width|"
                           "track_width|"
+                          "track_angle|"
+                          "track_segment_length|"
                           "via_count|"
                           "via_diameter|"
                           "zone_connection" );
@@ -397,6 +491,7 @@ void PANEL_SETUP_RULES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
         else if( sexprs.top() == wxT( "disallow" ) || isDisallowToken( sexprs.top() ) )
         {
             tokens = wxT( "buried_via|"
+                          "footprint|"
                           "graphic|"
                           "hole|"
                           "micro_via|"
@@ -457,7 +552,7 @@ void PANEL_SETUP_RULES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
             for( const wxString& propName : propNames )
                 tokens += wxT( "|" ) + propName;
 
-            PCB_EXPR_BUILTIN_FUNCTIONS& functions = PCB_EXPR_BUILTIN_FUNCTIONS::Instance();
+            PCBEXPR_BUILTIN_FUNCTIONS& functions = PCBEXPR_BUILTIN_FUNCTIONS::Instance();
 
             for( const wxString& funcSig : functions.GetSignatures() )
             {
@@ -472,7 +567,7 @@ void PANEL_SETUP_RULES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
                 BOARD_DESIGN_SETTINGS&         bds = m_frame->GetBoard()->GetDesignSettings();
                 std::shared_ptr<NET_SETTINGS>& netSettings = bds.m_NetSettings;
 
-                for( const auto& [ name, netclass ] : netSettings->m_NetClasses )
+                for( const auto& [name, netclass] : netSettings->GetNetclasses() )
                     tokens += wxT( "|" ) + name;
             }
             else if( m_netNameRegex.Matches( last ) )
@@ -545,6 +640,52 @@ void PANEL_SETUP_RULES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
                               "Polygon|"
                               "Bezier" );
             }
+            else if( m_padShapeRegex.Matches( last ) )
+            {
+                tokens = wxT( "Circle|"
+                              "Rectangle|"
+                              "Oval|"
+                              "Trapezoid|"
+                              "Rounded rectangle|"
+                              "Chamfered rectangle|"
+                              "Custom" );
+            }
+            else if( m_padConnectionsRegex.Matches( last ) )
+            {
+                tokens = wxT( "Inherited|"
+                              "None|"
+                              "Solid|"
+                              "Thermal reliefs|"
+                              "Thermal reliefs for PTH" );
+            }
+            else if( m_zoneConnStyleRegex.Matches( last ) )
+            {
+                tokens = wxT( "Inherited|"
+                              "None|"
+                              "Solid|"
+                              "Thermal reliefs" );
+            }
+            else if( m_lineStyleRegex.Matches( last ) )
+            {
+                tokens = wxT( "Default|"
+                              "Solid|"
+                              "Dashed|"
+                              "Dotted|"
+                              "Dash-Dot|"
+                              "Dash-Dot-Dot" );
+            }
+            else if( m_hJustRegex.Matches( last ) )
+            {
+                tokens = wxT( "Left|"
+                              "Center|"
+                              "Right" );
+            }
+            else if( m_vJustRegex.Matches( last ) )
+            {
+                tokens = wxT( "Top|"
+                              "Center|"
+                              "Bottom" );
+            }
         }
     }
 
@@ -625,6 +766,10 @@ bool PANEL_SETUP_RULES::TransferDataToWindow()
             OnCompile( dummy );
         }
     }
+    else
+    {
+        m_textEditor->AddText( wxT( "(version 1)\n" ) );
+    }
 
     m_originalText = m_textEditor->GetText();
 
@@ -675,21 +820,101 @@ void PANEL_SETUP_RULES::OnSyntaxHelp( wxHyperlinkEvent& aEvent )
         m_helpWindow->ShowModeless();
         return;
     }
+    std::vector<wxString> msg;
+    msg.clear();
 
-    wxString msg =
-#include "dialogs/panel_setup_rules_help_md.h"
-    ;
+    wxString t =
+#include "dialogs/panel_setup_rules_help_1clauses.h"
+            ;
+    msg.emplace_back( t );
+    t =
+#include "dialogs/panel_setup_rules_help_2constraints.h"
+            ;
+    msg.emplace_back( t );
+    t =
+#include "dialogs/panel_setup_rules_help_3items.h"
+            ;
+    msg.emplace_back( t );
+    t =
+#include "dialogs/panel_setup_rules_help_4severity_names.h"
+            ;
+    msg.emplace_back( t );
+    t =
+#include "dialogs/panel_setup_rules_help_5examples.h"
+            ;
+    msg.emplace_back( t );
+    t =
+#include "dialogs/panel_setup_rules_help_6notes.h"
+            ;
+    msg.emplace_back( t );
+    t =
+#include "dialogs/panel_setup_rules_help_7properties.h"
+            ;
+    msg.emplace_back( t );
+    t =
+#include "dialogs/panel_setup_rules_help_8expression_functions.h"
+            ;
+    msg.emplace_back( t );
+    t =
+#include "dialogs/panel_setup_rules_help_9more_examples.h"
+            ;
+    msg.emplace_back( t );
+    t =
+#include "dialogs/panel_setup_rules_help_10documentation.h"
+            ;
+    msg.emplace_back( t );
+
+    wxString msg_txt = wxEmptyString;
+
+    for( wxString i : msg )
+        msg_txt << wxGetTranslation( i );
 
 #ifdef __WXMAC__
-    msg.Replace( wxT( "Ctrl+" ), wxT( "Cmd+" ) );
+    msg_txt.Replace( wxT( "Ctrl+" ), wxT( "Cmd+" ) );
 #endif
+    const wxString& msGg_txt = msg_txt;
 
     m_helpWindow = new HTML_MESSAGE_BOX( nullptr, _( "Syntax Help" ) );
-    m_helpWindow->SetDialogSizeInDU( 320, 320 );
+    m_helpWindow->SetDialogSizeInDU( 420, 320 );
 
-    wxString html_txt;
-    ConvertMarkdown2Html( wxGetTranslation( msg ), html_txt );
+    wxString html_txt = wxEmptyString;
+    ConvertMarkdown2Html( msGg_txt, html_txt );
+
+    html_txt.Replace( wxS( "<td" ), wxS( "<td valign=top" ) );
     m_helpWindow->AddHTML_Text( html_txt );
 
     m_helpWindow->ShowModeless();
 }
+
+
+void PANEL_SETUP_RULES::ImportSettingsFrom( BOARD* aBoard )
+{
+    if( !m_frame->Prj().IsNullProject() )
+    {
+        wxFileName relFile = aBoard->GetFileName();
+        relFile.SetExt( FILEEXT::DesignRulesFileExtension );
+
+        wxFileName absFile( aBoard->GetProject()->AbsolutePath( relFile.GetFullName() ) );
+
+        if( absFile.FileExists() )
+        {
+            wxTextFile file( absFile.GetFullPath() );
+
+            if( file.Open() )
+            {
+                m_textEditor->ClearAll();
+
+                for ( wxString str = file.GetFirstLine(); !file.Eof(); str = file.GetNextLine() )
+                {
+                    ConvertSmartQuotesAndDashes( &str );
+                    m_textEditor->AddText( str << '\n' );
+                }
+
+                m_textEditor->EmptyUndoBuffer();
+
+                wxCommandEvent dummy;
+                OnCompile( dummy );
+            }
+        }
+    }
+}
