@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2022 KiCad Developers.
+ * Copyright The KiCad Developers.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,27 +30,26 @@
 #include <drc/drc_engine.h>
 #include <drc/drc_rtree.h>
 #include <drc/drc_cache_generator.h>
+#include <mutex>
 
 bool DRC_CACHE_GENERATOR::Run()
 {
     m_board = m_drcEngine->GetBoard();
 
-    int&           m_largestClearance = m_board->m_DRCMaxClearance;
-    int&           m_largestPhysicalClearance = m_board->m_DRCMaxPhysicalClearance;
+    int&           largestClearance = m_board->m_DRCMaxClearance;
+    int&           largestPhysicalClearance = m_board->m_DRCMaxPhysicalClearance;
     DRC_CONSTRAINT worstConstraint;
     LSET           boardCopperLayers = LSET::AllCuMask( m_board->GetCopperLayerCount() );
+    thread_pool&   tp = GetKiCadThreadPool();
 
-    if( m_drcEngine->QueryWorstConstraint( CLEARANCE_CONSTRAINT, worstConstraint ) )
-        m_largestClearance = worstConstraint.GetValue().Min();
 
-    if( m_drcEngine->QueryWorstConstraint( HOLE_CLEARANCE_CONSTRAINT, worstConstraint ) )
-        m_largestClearance = std::max( m_largestClearance, worstConstraint.GetValue().Min() );
+    largestClearance = std::max( largestClearance, m_board->GetMaxClearanceValue() );
 
     if( m_drcEngine->QueryWorstConstraint( PHYSICAL_CLEARANCE_CONSTRAINT, worstConstraint ) )
-        m_largestPhysicalClearance = worstConstraint.GetValue().Min();
+        largestPhysicalClearance = worstConstraint.GetValue().Min();
 
     if( m_drcEngine->QueryWorstConstraint( PHYSICAL_HOLE_CLEARANCE_CONSTRAINT, worstConstraint ) )
-        m_largestPhysicalClearance = std::max( m_largestPhysicalClearance, worstConstraint.GetValue().Min() );
+        largestPhysicalClearance = std::max( largestPhysicalClearance, worstConstraint.GetValue().Min() );
 
     std::set<ZONE*> allZones;
 
@@ -65,16 +64,12 @@ bool DRC_CACHE_GENERATOR::Run()
             if( ( zone->GetLayerSet() & boardCopperLayers ).any() )
             {
                 m_board->m_DRCCopperZones.push_back( zone );
-                m_largestClearance = std::max( m_largestClearance, zone->GetLocalClearance() );
             }
         }
     }
 
     for( FOOTPRINT* footprint : m_board->Footprints() )
     {
-        for( PAD* pad : footprint->Pads() )
-            m_largestClearance = std::max( m_largestClearance, pad->GetLocalClearance() );
-
         for( ZONE* zone : footprint->Zones() )
         {
             allZones.insert( zone );
@@ -84,18 +79,13 @@ bool DRC_CACHE_GENERATOR::Run()
                 m_board->m_DRCZones.push_back( zone );
 
                 if( ( zone->GetLayerSet() & boardCopperLayers ).any() )
-                {
                     m_board->m_DRCCopperZones.push_back( zone );
-                    m_largestClearance = std::max( m_largestClearance, zone->GetLocalClearance() );
-                }
             }
         }
     }
 
-    // This is the number of tests between 2 calls to the progress bar
-    size_t progressDelta = 200;
-    size_t count = 0;
-    size_t ii = 0;
+    size_t              count = 0;
+    std::atomic<size_t> done( 1 );
 
     auto countItems =
             [&]( BOARD_ITEM* item ) -> bool
@@ -107,7 +97,7 @@ bool DRC_CACHE_GENERATOR::Run()
     auto addToCopperTree =
             [&]( BOARD_ITEM* item ) -> bool
             {
-                if( !reportProgress( ii++, count, progressDelta ) )
+                if( m_drcEngine->IsCancelled() )
                     return false;
 
                 LSET copperLayers = item->GetLayerSet() & boardCopperLayers;
@@ -121,12 +111,13 @@ bool DRC_CACHE_GENERATOR::Run()
                         copperLayers = boardCopperLayers;
                 }
 
-                for( PCB_LAYER_ID layer : copperLayers.Seq() )
-                {
-                    if( IsCopperLayer( layer ) )
-                        m_board->m_CopperItemRTreeCache->Insert( item, layer, m_largestClearance );
-                }
+                copperLayers.RunOnLayers(
+                        [&]( PCB_LAYER_ID layer )
+                        {
+                            m_board->m_CopperItemRTreeCache->Insert( item, layer, largestClearance );
+                        } );
 
+                done.fetch_add( 1 );
                 return true;
             };
 
@@ -136,13 +127,31 @@ bool DRC_CACHE_GENERATOR::Run()
     static const std::vector<KICAD_T> itemTypes = {
         PCB_TRACE_T, PCB_ARC_T, PCB_VIA_T,
         PCB_PAD_T,
-        PCB_SHAPE_T, PCB_FP_SHAPE_T,
-        PCB_TEXT_T, PCB_FP_TEXT_T, PCB_TEXTBOX_T, PCB_FP_TEXTBOX_T,
+        PCB_SHAPE_T,
+        PCB_FIELD_T, PCB_TEXT_T, PCB_TEXTBOX_T,
         PCB_DIMENSION_T
     };
 
     forEachGeometryItem( itemTypes, LSET::AllCuMask(), countItems );
-    forEachGeometryItem( itemTypes, LSET::AllCuMask(), addToCopperTree );
+
+    std::future<void> retn = tp.submit(
+            [&]()
+            {
+                std::unique_lock<std::shared_mutex> writeLock( m_board->m_CachesMutex );
+
+                if( !m_board->m_CopperItemRTreeCache )
+                    m_board->m_CopperItemRTreeCache = std::make_shared<DRC_RTREE>();
+
+                forEachGeometryItem( itemTypes, LSET::AllCuMask(), addToCopperTree );
+            } );
+
+    std::future_status status = retn.wait_for( std::chrono::milliseconds( 250 ) );
+
+    while( status != std::future_status::ready )
+    {
+        reportProgress( done, count );
+        status = retn.wait_for( std::chrono::milliseconds( 250 ) );
+    }
 
     if( !reportPhase( _( "Tessellating copper zones..." ) ) )
         return false;   // DRC cancelled
@@ -151,11 +160,12 @@ bool DRC_CACHE_GENERATOR::Run()
     // before we start.
 
     for( FOOTPRINT* footprint : m_board->Footprints() )
+    {
         footprint->BuildCourtyardCaches();
+        footprint->BuildNetTieCache();
+    }
 
-    thread_pool&                     tp = GetKiCadThreadPool();
     std::vector<std::future<size_t>> returns;
-    std::atomic<size_t>              done( 1 );
 
     returns.reserve( allZones.size() );
 
@@ -172,14 +182,17 @@ bool DRC_CACHE_GENERATOR::Run()
                 {
                    std::unique_ptr<DRC_RTREE> rtree = std::make_unique<DRC_RTREE>();
 
-                   for( PCB_LAYER_ID layer : aZone->GetLayerSet().Seq() )
-                   {
-                       if( IsCopperLayer( layer ) )
-                           rtree->Insert( aZone, layer );
-                   }
+                   aZone->GetLayerSet().RunOnLayers(
+                           [&]( PCB_LAYER_ID layer )
+                           {
+                               if( IsCopperLayer( layer ) )
+                                   rtree->Insert( aZone, layer );
+                           } );
 
-                   std::unique_lock<std::mutex> cacheLock( m_board->m_CachesMutex );
-                   m_board->m_CopperZoneRTreeCache[ aZone ] = std::move( rtree );
+                   {
+                       std::unique_lock<std::shared_mutex> writeLock( m_board->m_CachesMutex );
+                       m_board->m_CopperZoneRTreeCache[ aZone ] = std::move( rtree );
+                   }
 
                    done.fetch_add( 1 );
                 }
@@ -190,13 +203,15 @@ bool DRC_CACHE_GENERATOR::Run()
     for( ZONE* zone : allZones )
         returns.emplace_back( tp.submit( cache_zones, zone ) );
 
+    done.store( 1 );
+
     for( const std::future<size_t>& ret : returns )
     {
-        std::future_status status = ret.wait_for( std::chrono::milliseconds( 250 ) );
+        status = ret.wait_for( std::chrono::milliseconds( 250 ) );
 
         while( status != std::future_status::ready )
         {
-            m_drcEngine->ReportProgress( static_cast<double>( done ) / allZones.size() );
+            reportProgress( done, allZones.size() );
             status = ret.wait_for( std::chrono::milliseconds( 250 ) );
         }
     }
@@ -205,10 +220,13 @@ bool DRC_CACHE_GENERATOR::Run()
 
     for( ZONE* zone : m_board->Zones() )
     {
-        if( !zone->GetIsRuleArea() )
+        if( !zone->GetIsRuleArea() && !zone->IsTeardropArea() )
         {
-            for( PCB_LAYER_ID layer : zone->GetLayerSet().Seq() )
-                m_board->m_ZoneIsolatedIslandsMap[ zone ][ layer ] = ISOLATED_ISLANDS();
+            zone->GetLayerSet().RunOnLayers(
+                    [&]( PCB_LAYER_ID layer )
+                    {
+                        m_board->m_ZoneIsolatedIslandsMap[ zone ][ layer ] = ISOLATED_ISLANDS();
+                    } );
         }
     }
 
