@@ -4,7 +4,7 @@
  * Copyright (C) 2009-2013  Lorenzo Mercantonio
  * Copyright (C) 2014-2017  Cirilo Bernardo
  * Copyright (C) 2018 Jean-Pierre Charras jp.charras at wanadoo.fr
- * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -34,11 +34,11 @@
 #include "board.h"
 #include "board_design_settings.h"
 #include <fp_lib_table.h>
-#include "fp_shape.h"
 #include "footprint.h"
 #include "pad.h"
 #include "pcb_text.h"
 #include "pcb_track.h"
+#include <project_pcb.h>
 #include <core/arraydim.h>
 #include <filename_resolver.h>
 #include "plugins/3dapi/ifsg_all.h"
@@ -60,12 +60,14 @@ EXPORTER_VRML::EXPORTER_VRML( BOARD* aBoard )
 
 bool EXPORTER_VRML::ExportVRML_File( PROJECT* aProject, wxString *aMessages,
                               const wxString& aFullFileName, double aMMtoWRMLunit,
+                              bool aIncludeUnspecified, bool aIncludeDNP,
                               bool aExport3DFiles, bool aUseRelativePaths,
                               const wxString& a3D_Subdir,
                               double aXRef, double aYRef )
 {
     return pcb_exporter->ExportVRML_File( aProject, aMessages,
                                           aFullFileName, aMMtoWRMLunit,
+                                          aIncludeUnspecified, aIncludeDNP,
                                           aExport3DFiles, aUseRelativePaths,
                                           a3D_Subdir, aXRef, aYRef );
 }
@@ -104,6 +106,8 @@ EXPORTER_PCB_VRML::EXPORTER_PCB_VRML( BOARD* aBoard ) :
     m_precision = 6;
     m_WorldScale = 1.0;
     m_Cache3Dmodels = nullptr;
+    m_includeDNP = false;
+    m_includeUnspecified = false;
     m_UseInlineModelsInBrdfile = false;
     m_UseRelPathIn3DModelFilename = false;
     m_BoardToVrmlScale = pcbIUScale.MM_PER_IU;
@@ -111,9 +115,6 @@ EXPORTER_PCB_VRML::EXPORTER_PCB_VRML( BOARD* aBoard ) :
     for( int ii = 0; ii < VRML_COLOR_LAST; ++ii )
         m_sgmaterial[ii] = nullptr;
 
-    for( unsigned i = 0; i < arrayDim( m_layer_z );  ++i )
-        m_layer_z[i] = 0;
-
     // this default only makes sense if the output is in mm
     m_brd_thickness = pcbIUScale.IUTomm( m_board->GetDesignSettings().GetBoardThickness() );
 
@@ -130,8 +131,9 @@ EXPORTER_PCB_VRML::EXPORTER_PCB_VRML( BOARD* aBoard ) :
 
     const BOARD_STACKUP& stackup = m_board->GetDesignSettings().GetStackupDescriptor();
 
+    // Can't do a const KIGFX::COLOR4D& return type here because there are temporary variables
     auto findColor =
-            []( const wxString& aColorName, const CUSTOM_COLORS_LIST& aColorSet )
+            []( const wxString& aColorName, const CUSTOM_COLORS_LIST& aColorSet ) -> const KIGFX::COLOR4D
             {
                 if( aColorName.StartsWith( wxT( "#" ) ) )
                 {
@@ -395,8 +397,8 @@ void EXPORTER_PCB_VRML::ExportVrmlSolderMask()
         outlines = m_pcbOutlines;
         m_board->ConvertBrdLayerToPolygonalContours( pcb_layer, holes );
 
-        outlines.BooleanSubtract( holes, SHAPE_POLY_SET::PM_FAST );
-        outlines.Fracture( SHAPE_POLY_SET::PM_FAST );
+        outlines.BooleanSubtract( holes );
+        outlines.Fracture();
         ExportVrmlPolygonSet( vrmllayer, outlines );
 
         pcb_layer = B_Mask;
@@ -427,7 +429,8 @@ void EXPORTER_PCB_VRML::ExportStandardLayers()
 
         outlines.RemoveAllContours();
         m_board->ConvertBrdLayerToPolygonalContours( pcb_layer[lcnt], outlines );
-        outlines.Fracture( SHAPE_POLY_SET::PM_FAST );
+        outlines.BooleanIntersection( m_pcbOutlines );
+        outlines.Fracture();
 
         ExportVrmlPolygonSet( vrmllayer[lcnt], outlines );
     }
@@ -723,14 +726,12 @@ void EXPORTER_PCB_VRML::ComputeLayer3D_Zpos()
     double half_thickness = m_brd_thickness / 2;
 
     // Compute each layer's Z value, more or less like the 3d view
-    for( LSEQ seq = LSET::AllCuMask().Seq();  seq;  ++seq )
-    {
-        PCB_LAYER_ID i = *seq;
+    int orderFromTop = 0;
 
-        if( i < copper_layers )
-            SetLayerZ( i,  half_thickness - m_brd_thickness * i / (copper_layers - 1) );
-        else
-            SetLayerZ( i, - half_thickness );  // bottom layer
+    for( PCB_LAYER_ID layer : LSET::AllCuMask( copper_layers ).CuStack() )
+    {
+        SetLayerZ( layer, half_thickness - m_brd_thickness * orderFromTop / ( copper_layers - 1 ) );
+        orderFromTop++;
     }
 
     // To avoid rounding interference, we apply an epsilon to each successive layer
@@ -904,7 +905,7 @@ void EXPORTER_PCB_VRML::ExportVrmlPadHole( PAD* aPad )
         if( ( aPad->GetAttribute() != PAD_ATTRIB::NPTH ) )
             pth = true;
 
-        if( aPad->GetDrillShape() == PAD_DRILL_SHAPE_OBLONG )
+        if( aPad->GetDrillShape() == PAD_DRILL_SHAPE::OBLONG )
         {
             // Oblong hole (slot)
 
@@ -1003,7 +1004,7 @@ void EXPORTER_PCB_VRML::ExportVrmlFootprint( FOOTPRINT* aFootprint, std::ostream
 
         try
         {
-            fpRow = m_board->GetProject()->PcbFootprintLibs()->FindRow( libraryName, false );
+            fpRow = PROJECT_PCB::PcbFootprintLibs( m_board->GetProject() )->FindRow( libraryName, false );
         }
         catch( ... )
         {
@@ -1019,13 +1020,21 @@ void EXPORTER_PCB_VRML::ExportVrmlFootprint( FOOTPRINT* aFootprint, std::ostream
     for( PAD* pad : aFootprint->Pads() )
         ExportVrmlPadHole( pad );
 
+    if( !m_includeUnspecified
+        && ( !( aFootprint->GetAttributes() & ( FP_THROUGH_HOLE | FP_SMD ) ) ) )
+    {
+        return;
+    }
+
+    if( !m_includeDNP && aFootprint->IsDNP() )
+        return;
+
     bool isFlipped = aFootprint->GetLayer() == B_Cu;
 
     // Export the object VRML model(s)
     auto sM = aFootprint->Models().begin();
     auto eM = aFootprint->Models().end();
 
-    wxFileName subdir( m_Subdir3DFpModels, wxT( "" ) );
 
     while( sM != eM )
     {
@@ -1035,7 +1044,7 @@ void EXPORTER_PCB_VRML::ExportVrmlFootprint( FOOTPRINT* aFootprint, std::ostream
             continue;
         }
 
-        SGNODE* mod3d = (SGNODE*) m_Cache3Dmodels->Load( sM->m_Filename, footprintBasePath );
+        SGNODE* mod3d = (SGNODE*) m_Cache3Dmodels->Load( sM->m_Filename, footprintBasePath, aFootprint );
 
         if( nullptr == mod3d )
         {
@@ -1101,7 +1110,7 @@ void EXPORTER_PCB_VRML::ExportVrmlFootprint( FOOTPRINT* aFootprint, std::ostream
             aOutputFile->precision( m_precision );
 
             wxFileName srcFile =
-                    m_Cache3Dmodels->GetResolver()->ResolvePath( sM->m_Filename, wxEmptyString );
+                    m_Cache3Dmodels->GetResolver()->ResolvePath( sM->m_Filename, wxEmptyString, aFootprint );
             wxFileName dstFile;
             dstFile.SetPath( m_Subdir3DFpModels );
             dstFile.SetName( srcFile.GetName() );
@@ -1231,6 +1240,7 @@ void EXPORTER_PCB_VRML::ExportVrmlFootprint( FOOTPRINT* aFootprint, std::ostream
 
 bool EXPORTER_PCB_VRML::ExportVRML_File( PROJECT* aProject, wxString *aMessages,
                                          const wxString& aFullFileName, double aMMtoWRMLunit,
+                                         bool aIncludeUnspecified, bool aIncludeDNP,
                                          bool aExport3DFiles, bool aUseRelativePaths,
                                          const wxString& a3D_Subdir,
                                          double aXRef, double aYRef )
@@ -1245,9 +1255,15 @@ bool EXPORTER_PCB_VRML::ExportVRML_File( PROJECT* aProject, wxString *aMessages,
 
     SetScale( aMMtoWRMLunit );
     m_UseInlineModelsInBrdfile = aExport3DFiles;
-    m_Subdir3DFpModels = a3D_Subdir;
+
+    wxFileName subdir( a3D_Subdir, wxT( "" ) );
+    // convert the subdir path to a absolute full one with the output file as the cwd
+    m_Subdir3DFpModels = subdir.GetAbsolutePath( wxFileName( aFullFileName ).GetPath() );
+
     m_UseRelPathIn3DModelFilename = aUseRelativePaths;
-    m_Cache3Dmodels = aProject->Get3DCacheManager();
+    m_includeUnspecified = aIncludeUnspecified;
+    m_includeDNP = aIncludeDNP;
+    m_Cache3Dmodels = PROJECT_PCB::Get3DCacheManager( aProject );
 
     // When 3D models are separate files, for historical reasons the VRML unit
     // is expected to be 0.1 inch (2.54mm) instead of 1mm, so we adjust the m_BoardToVrmlScale
@@ -1298,7 +1314,7 @@ bool EXPORTER_PCB_VRML::ExportVRML_File( PROJECT* aProject, wxString *aMessages,
     catch( const std::exception& e )
     {
         if( aMessages )
-            *aMessages << _( "VRML Export Failed:\n" ) << FROM_UTF8( e.what() );
+            *aMessages << _( "VRML Export Failed:\n" ) << From_UTF8( e.what() );
 
         success = false;
     }
@@ -1307,6 +1323,7 @@ bool EXPORTER_PCB_VRML::ExportVRML_File( PROJECT* aProject, wxString *aMessages,
 }
 
 bool PCB_EDIT_FRAME::ExportVRML_File( const wxString& aFullFileName, double aMMtoWRMLunit,
+                                      bool aIncludeUnspecified, bool aIncludeDNP,
                                       bool aExport3DFiles, bool aUseRelativePaths,
                                       const wxString& a3D_Subdir,
                                       double aXRef, double aYRef )
@@ -1316,6 +1333,7 @@ bool PCB_EDIT_FRAME::ExportVRML_File( const wxString& aFullFileName, double aMMt
     EXPORTER_VRML model3d( GetBoard() );
 
     success = model3d.ExportVRML_File( &Prj(), &msgs, aFullFileName, aMMtoWRMLunit,
+                                       aIncludeUnspecified, aIncludeDNP,
                                        aExport3DFiles, aUseRelativePaths,
                                        a3D_Subdir, aXRef, aYRef );
 
@@ -1329,11 +1347,9 @@ bool PCB_EDIT_FRAME::ExportVRML_File( const wxString& aFullFileName, double aMMt
 void EXPORTER_PCB_VRML::ExportFp3DModelsAsLinkedFile( const wxString& aFullFileName )
 {
     // check if the 3D Subdir exists - create if not
-    wxFileName subdir( m_Subdir3DFpModels, wxT( "" ) );
-
-    if( ! subdir.DirExists() )
+    if( !wxDir::Exists( m_Subdir3DFpModels ) )
     {
-        if( !wxDir::Make( subdir.GetFullPath() ) )
+        if( !wxDir::Make( m_Subdir3DFpModels ) )
             throw( std::runtime_error( "Could not create 3D model subdirectory" ) );
     }
 
