@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 1992-2017 Jean_Pierre Charras <jp.charras at wanadoo.fr>
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,15 +26,17 @@
 #include <plotters/plotter_gerber.h>
 #include <plotters/plotters_pslike.h>
 #include <eda_item.h>
+#include <font/font.h>
 #include <confirm.h>
+#include <richio.h>
 #include <string_utils.h>
 #include <locale_io.h>
 #include <macros.h>
 #include <math/util.h>      // for KiROUND
 
 #include <board.h>
+#include <footprint.h>
 
-#include <pcbnew.h>
 #include <pcbplot.h>
 #include <gendrill_file_writer_base.h>
 #include <pcb_painter.h>
@@ -104,7 +106,7 @@ bool GENDRILL_WRITER_BASE::genDrillMapFile( const wxString& aFullFileName, PLOT_
     // Calculate dimensions and center of PCB. The Edge_Cuts layer must be visible
     // to calculate the board edges bounding box
     LSET visibleLayers = m_pcb->GetVisibleLayers();
-    m_pcb->SetVisibleLayers( visibleLayers | LSET( Edge_Cuts ) );
+    m_pcb->SetVisibleLayers( visibleLayers | LSET( { Edge_Cuts } ) );
     BOX2I bbbox = m_pcb->GetBoardEdgesBoundingBox();
     m_pcb->SetVisibleLayers( visibleLayers );
 
@@ -133,7 +135,7 @@ bool GENDRILL_WRITER_BASE::genDrillMapFile( const wxString& aFullFileName, PLOT_
     case PLOT_FORMAT::SVG:
     {
         PAGE_INFO pageA4( wxT( "A4" ) );
-        wxSize    pageSizeIU = pageA4.GetSizeIU( pcbIUScale.IU_PER_MILS );
+        VECTOR2I  pageSizeIU = pageA4.GetSizeIU( pcbIUScale.IU_PER_MILS );
 
         // Reserve a 10 mm margin around the page.
         int margin = pcbIUScale.mmToIU( 10 );
@@ -235,7 +237,7 @@ bool GENDRILL_WRITER_BASE::genDrillMapFile( const wxString& aFullFileName, PLOT_
     BRDITEMS_PLOTTER itemplotter( plotter, m_pcb, plot_opts );
 
     // Use attributes of a drawing layer (we are not really draw the Edge.Cuts layer)
-    itemplotter.SetLayerSet( Dwgs_User );
+    itemplotter.SetLayerSet( { Dwgs_User } );
 
     for( BOARD_ITEM* item : m_pcb->Drawings() )
     {
@@ -249,7 +251,7 @@ bool GENDRILL_WRITER_BASE::genDrillMapFile( const wxString& aFullFileName, PLOT_
             PCB_SHAPE dummy_shape( *static_cast<PCB_SHAPE*>( item ) );
             dummy_shape.SetLayer( Dwgs_User );
             dummy_shape.SetParentGroup( nullptr );      // Remove group association, not needed for plotting
-            itemplotter.PlotPcbShape( &dummy_shape );
+            itemplotter.PlotShape( &dummy_shape );
             }
             break;
 
@@ -258,6 +260,31 @@ bool GENDRILL_WRITER_BASE::genDrillMapFile( const wxString& aFullFileName, PLOT_
         }
     }
 
+    // Plot edge cuts in footprints
+    for( const FOOTPRINT* footprint : m_pcb->Footprints() )
+    {
+        for( BOARD_ITEM* item : footprint->GraphicalItems() )
+        {
+            if( item-> GetLayer() != Edge_Cuts )
+                continue;
+
+            switch( item->Type() )
+            {
+            case PCB_SHAPE_T:
+                {
+                PCB_SHAPE dummy_shape( *static_cast<PCB_SHAPE*>( item ) );
+                dummy_shape.SetLayer( Dwgs_User );
+                dummy_shape.SetParentGroup( nullptr );      // Remove group association, not needed for plotting
+                itemplotter.PlotShape( &dummy_shape );
+                }
+                break;
+
+            default:
+                break;
+            }
+        }
+    }
+
     int      plotX, plotY, TextWidth;
     int      intervalle = 0;
     char     line[1024];
@@ -298,7 +325,8 @@ bool GENDRILL_WRITER_BASE::genDrillMapFile( const wxString& aFullFileName, PLOT_
     attrs.m_Valign = GR_TEXT_V_ALIGN_CENTER;
     attrs.m_Multiline = false;
 
-    plotter->PlotText( VECTOR2I( plotX, plotY ), COLOR4D::UNSPECIFIED, Text, attrs, nullptr /* stroke font */ );
+    plotter->PlotText( VECTOR2I( plotX, plotY ), COLOR4D::UNSPECIFIED, Text, attrs,
+                       nullptr /* stroke font */, KIFONT::METRICS::Default() );
 
     // For some formats (PS, PDF SVG) we plot the drill size list on more than one column
     // because the list must be contained inside the printed page
@@ -337,7 +365,7 @@ bool GENDRILL_WRITER_BASE::genDrillMapFile( const wxString& aFullFileName, PLOT_
         snprintf( line, sizeof(line), "%3.3fmm / %2.4f\" ", diameter_in_mm( tool.m_Diameter ),
                  diameter_in_inches( tool.m_Diameter ) );
 
-        msg = FROM_UTF8( line );
+        msg = From_UTF8( line );
 
         // Now list how many holes and ovals are associated with each drill.
         if( ( tool.m_TotalCount == 1 ) && ( tool.m_OvalCount == 0 ) )
@@ -352,12 +380,13 @@ bool GENDRILL_WRITER_BASE::genDrillMapFile( const wxString& aFullFileName, PLOT_
             snprintf( line, sizeof(line), "(%d holes + %d slots)", tool.m_TotalCount - tool.m_OvalCount,
                      tool.m_OvalCount );
 
-        msg += FROM_UTF8( line );
+        msg += From_UTF8( line );
 
         if( tool.m_Hole_NotPlated )
             msg += wxT( " (not plated)" );
 
-        plotter->PlotText( VECTOR2I( plotX, y ), COLOR4D::UNSPECIFIED, msg, attrs, nullptr /* stroke font */ );
+        plotter->PlotText( VECTOR2I( plotX, y ), COLOR4D::UNSPECIFIED, msg, attrs,
+                           nullptr /* stroke font */, KIFONT::METRICS::Default() );
 
         intervalle = KiROUND( ( ( charSize * charScale ) + TextWidth ) * 1.2 );
 
@@ -392,7 +421,7 @@ bool GENDRILL_WRITER_BASE::GenDrillReportFile( const wxString& aFullFileName )
     std::vector<DRILL_LAYER_PAIR> hole_sets = getUniqueLayerPairs();
 
     out.Print( 0, "Drill report for %s\n", TO_UTF8( brdFilename.GetFullName() ) );
-    out.Print( 0, "Created on %s\n\n", TO_UTF8( DateAndTime() ) );
+    out.Print( 0, "Created on %s\n\n", TO_UTF8( GetISO8601CurrentDateTime() ) );
 
     // Output the cu layer stackup, so layer name references make sense.
     out.Print( 0, "Copper Layer Stackup:\n" );
@@ -402,12 +431,12 @@ bool GENDRILL_WRITER_BASE::GenDrillReportFile( const wxString& aFullFileName )
 
     int conventional_layer_num = 1;
 
-    for( LSEQ seq = cu.Seq();  seq;  ++seq, ++conventional_layer_num )
+    for( PCB_LAYER_ID layer : cu.Seq() )
     {
         out.Print( 0, "    L%-2d:  %-25s %s\n",
                    conventional_layer_num,
-                   TO_UTF8( m_pcb->GetLayerName( *seq ) ),
-                   layerName( *seq ).c_str() );             // generic layer name
+                   TO_UTF8( m_pcb->GetLayerName( layer ) ),
+                   layerName( layer ).c_str() );             // generic layer name
     }
 
     out.Print( 0, "\n\n" );
