@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,17 +27,23 @@
 
 #include <deque>
 
+#include <template_fieldnames.h>
+
 #include <board_item_container.h>
 #include <board_item.h>
 #include <collectors.h>
+#include <component_class_manager.h>
+#include <embedded_files.h>
 #include <layer_ids.h> // ALL_LAYERS definition.
+#include <lset.h>
 #include <lib_id.h>
 #include <list>
 
 #include <zones.h>
 #include <convert_shape_list_to_polygon.h>
 #include <pcb_item_containers.h>
-#include <fp_text.h>
+#include <pcb_text.h>
+#include <pcb_field.h>
 #include <functional>
 #include <math/vector3.h>
 
@@ -47,11 +53,16 @@ class PAD;
 class BOARD;
 class MSG_PANEL_ITEM;
 class SHAPE;
+class REPORTER;
 
 namespace KIGFX {
 class VIEW;
 }
 
+namespace KIFONT {
+class OUTLINE_FONT;
+}
+
 enum INCLUDE_NPTH_T
 {
     DO_NOT_INCLUDE_NPTH = false,
@@ -72,7 +83,8 @@ enum FOOTPRINT_ATTR_T
     FP_BOARD_ONLY               = 0x0010,   // Footprint has no corresponding symbol
     FP_JUST_ADDED               = 0x0020,   // Footprint just added by netlist update
     FP_ALLOW_SOLDERMASK_BRIDGES = 0x0040,
-    FP_ALLOW_MISSING_COURTYARD  = 0x0080
+    FP_ALLOW_MISSING_COURTYARD  = 0x0080,
+    FP_DNP                      = 0x0100
 };
 
 class FP_3DMODEL
@@ -94,10 +106,20 @@ public:
     double   m_Opacity;
     wxString m_Filename;    ///< The 3D shape filename in 3D library
     bool     m_Show;        ///< Include model in rendering
+
+    bool operator==( const FP_3DMODEL& aOther ) const
+    {
+        return m_Scale == aOther.m_Scale
+                && m_Rotation == aOther.m_Rotation
+                && m_Offset == aOther.m_Offset
+                && m_Opacity == aOther.m_Opacity
+                && m_Filename == aOther.m_Filename
+                && m_Show == aOther.m_Show;
+    }
 };
 
 
-class FOOTPRINT : public BOARD_ITEM_CONTAINER
+class FOOTPRINT : public BOARD_ITEM_CONTAINER, public EMBEDDED_FILES
 {
 public:
     FOOTPRINT( BOARD* parent );
@@ -112,6 +134,9 @@ public:
     FOOTPRINT& operator=( const FOOTPRINT& aOther );
     FOOTPRINT& operator=( FOOTPRINT&& aOther );
 
+    void Serialize( google::protobuf::Any &aContainer ) const override;
+    bool Deserialize( const google::protobuf::Any &aContainer ) override;
+
     static inline bool ClassOf( const EDA_ITEM* aItem )
     {
         return aItem && aItem->Type() == PCB_FOOTPRINT_T;
@@ -163,21 +188,30 @@ public:
      */
     SHAPE_POLY_SET GetBoundingHull() const;
 
+    bool TextOnly() const;
+
     // Virtual function
     const BOX2I GetBoundingBox() const override;
-    const BOX2I GetBoundingBox( bool aIncludeText, bool aIncludeInvisibleText ) const;
+    const BOX2I GetBoundingBox( bool aIncludeText ) const;
 
-    PADS& Pads()             { return m_pads; }
-    const PADS& Pads() const { return m_pads; }
+    /**
+     * Return the bounding box of the footprint on a given set of layers
+    */
+    const BOX2I GetLayerBoundingBox( LSET aLayers ) const;
+
+    VECTOR2I GetCenter() const override { return GetBoundingBox( false ).GetCenter(); }
+
+    std::deque<PAD*>& Pads()               { return m_pads; }
+    const std::deque<PAD*>& Pads() const   { return m_pads; }
 
     DRAWINGS& GraphicalItems()             { return m_drawings; }
     const DRAWINGS& GraphicalItems() const { return m_drawings; }
 
-    FP_ZONES& Zones()             { return m_fp_zones; }
-    const FP_ZONES& Zones() const { return m_fp_zones; }
+    ZONES& Zones()                         { return m_zones; }
+    const ZONES& Zones() const             { return m_zones; }
 
-    FP_GROUPS& Groups()             { return m_fp_groups; }
-    const FP_GROUPS& Groups() const { return m_fp_groups; }
+    GROUPS& Groups()                       { return m_groups; }
+    const GROUPS& Groups() const           { return m_groups; }
 
     bool HasThroughHolePads() const;
 
@@ -210,13 +244,16 @@ public:
     }
 
     const LIB_ID& GetFPID() const { return m_fpid; }
-    void SetFPID( const LIB_ID& aFPID ) { m_fpid = aFPID; }
+    void          SetFPID( const LIB_ID& aFPID )
+    {
+        m_fpid = aFPID;
+    }
 
     wxString GetFPIDAsString() const { return m_fpid.Format(); }
     void SetFPIDAsString( const wxString& aFPID ) { m_fpid.Parse( aFPID ); }
 
-    wxString GetDescription() const { return m_doc; }
-    void SetDescription( const wxString& aDoc ) { m_doc = aDoc; }
+    wxString GetLibDescription() const { return m_libDescription; }
+    void     SetLibDescription( const wxString& aDesc ) { m_libDescription = aDesc; }
 
     wxString GetKeywords() const { return m_keywords; }
     void SetKeywords( const wxString& aKeywords ) { m_keywords = aKeywords; }
@@ -224,28 +261,29 @@ public:
     const KIID_PATH& GetPath() const { return m_path; }
     void SetPath( const KIID_PATH& aPath ) { m_path = aPath; }
 
-    int GetLocalSolderMaskMargin() const { return m_localSolderMaskMargin; }
-    void SetLocalSolderMaskMargin( int aMargin ) { m_localSolderMaskMargin = aMargin; }
+    wxString GetSheetname() const { return m_sheetname; }
+    void SetSheetname( const wxString& aSheetname ) { m_sheetname = aSheetname; }
 
-    int GetLocalClearance() const { return m_localClearance; }
-    void SetLocalClearance( int aClearance ) { m_localClearance = aClearance; }
+    wxString GetSheetfile() const { return m_sheetfile; }
+    void SetSheetfile( const wxString& aSheetfile ) { m_sheetfile = aSheetfile; }
 
-    int GetLocalClearance( wxString* aSource ) const
-    {
-        if( aSource )
-            *aSource = wxString::Format( _( "footprint %s" ), GetReference() );
+    wxString GetFilters() const { return m_filters; }
+    void SetFilters( const wxString& aFilters ) { m_filters = aFilters; }
 
-        return m_localClearance;
-    }
+    std::optional<int> GetLocalClearance() const                 { return m_clearance; }
+    void SetLocalClearance( std::optional<int> aClearance )      { m_clearance = aClearance; }
 
-    int GetLocalSolderPasteMargin() const { return m_localSolderPasteMargin; }
-    void SetLocalSolderPasteMargin( int aMargin ) { m_localSolderPasteMargin = aMargin; }
+    std::optional<int> GetLocalSolderMaskMargin() const          { return m_solderMaskMargin; }
+    void SetLocalSolderMaskMargin( std::optional<int> aMargin )  { m_solderMaskMargin = aMargin; }
 
-    double GetLocalSolderPasteMarginRatio() const { return m_localSolderPasteMarginRatio; }
-    void SetLocalSolderPasteMarginRatio( double aRatio ) { m_localSolderPasteMarginRatio = aRatio; }
+    std::optional<int> GetLocalSolderPasteMargin() const         { return m_solderPasteMargin; }
+    void SetLocalSolderPasteMargin( std::optional<int> aMargin ) { m_solderPasteMargin = aMargin; }
 
-    void SetZoneConnection( ZONE_CONNECTION aType ) { m_zoneConnection = aType; }
-    ZONE_CONNECTION GetZoneConnection() const { return m_zoneConnection; }
+    std::optional<double> GetLocalSolderPasteMarginRatio() const { return m_solderPasteMarginRatio; }
+    void SetLocalSolderPasteMarginRatio( std::optional<double> aRatio ) { m_solderPasteMarginRatio = aRatio; }
+
+    void SetLocalZoneConnection( ZONE_CONNECTION aType )         { m_zoneConnection = aType; }
+    ZONE_CONNECTION GetLocalZoneConnection() const               { return m_zoneConnection; }
 
     int GetAttributes() const { return m_attributes; }
     void SetAttributes( int aAttributes ) { m_attributes = aAttributes; }
@@ -265,6 +303,33 @@ public:
         return false;
     }
 
+    std::optional<int> GetLocalClearance( wxString* aSource ) const
+    {
+        if( m_clearance.has_value() && aSource )
+            *aSource = wxString::Format( _( "footprint %s" ), GetReference() );
+
+        return m_clearance;
+    }
+
+    /**
+     * Return any local clearance overrides set in the "classic" (ie: pre-rule) system.
+     *
+     * @param aSource [out] optionally reports the source as a user-readable string.
+     * @return the clearance in internal units.
+     */
+    std::optional<int> GetClearanceOverrides( wxString* aSource ) const
+    {
+        return GetLocalClearance( aSource );
+    }
+
+    ZONE_CONNECTION GetZoneConnectionOverrides( wxString* aSource ) const
+    {
+        if( m_zoneConnection != ZONE_CONNECTION::INHERITED && aSource )
+            *aSource = wxString::Format( _( "footprint %s" ), GetReference() );
+
+        return m_zoneConnection;
+    }
+
     /**
      * @return a list of pad groups, each of which is allowed to short nets within their group.
      *         A pad group is a comma-separated list of pad numbers.
@@ -303,7 +368,7 @@ public:
 
     void Rotate( const VECTOR2I& aRotCentre, const EDA_ANGLE& aAngle ) override;
 
-    void Flip( const VECTOR2I& aCentre, bool aFlipLeftRight ) override;
+    void Flip( const VECTOR2I& aCentre, FLIP_DIRECTION aFlipDirection ) override;
 
     /**
      * Move the reference point of the footprint.
@@ -324,9 +389,13 @@ public:
     bool IsFlipped() const { return GetLayer() == B_Cu; }
 
     /**
-     * A special IsOnLayer for footprints: return true if the footprint contains only items on the
-     * given layer, even if that layer is not one of the valid footprint layers F_Cu and B_Cu.
-     * This allows selection of "graphic" footprints that contain only silkscreen, for example.
+     * Use instead of IsFlipped() when you also need to account for unsided footprints (those
+     * purely on user-layers, etc.).
+     */
+    PCB_LAYER_ID GetSide() const;
+
+    /**
+     * @copydoc BOARD_ITEM::IsOnLayer
      */
     bool IsOnLayer( PCB_LAYER_ID aLayer ) const override;
 
@@ -398,15 +467,15 @@ public:
      *
      * @param aErrorHandler callback to handle the error messages generated
      */
-    void CheckPads( const std::function<void( const PAD*, int, const wxString& )>& aErrorHandler );
+    void CheckPads( UNITS_PROVIDER* aUnitsProvider,
+                    const std::function<void( const PAD*, int, const wxString& )>& aErrorHandler );
 
     /**
      * Check for overlapping, different-numbered, non-net-tie pads.
      *
      * @param aErrorHandler callback to handle the error messages generated
      */
-    void CheckShortingPads( const std::function<void( const PAD*,
-                                                      const PAD*,
+    void CheckShortingPads( const std::function<void( const PAD*, const PAD*, int aErrorCode,
                                                       const VECTOR2I& )>& aErrorHandler );
 
     /**
@@ -428,6 +497,26 @@ public:
      */
     void CheckNetTiePadGroups( const std::function<void( const wxString& )>& aErrorHandler );
 
+    /**
+     * Cache the pads that are allowed to connect to each other in the footprint.
+     */
+    void BuildNetTieCache();
+
+    /**
+     * Get the set of net codes that are allowed to connect to a footprint item
+     */
+    const std::set<int>& GetNetTieCache( const BOARD_ITEM* aItem ) const
+    {
+        static const std::set<int> emptySet;
+
+        auto it = m_netTieCache.find( aItem );
+
+        if( it == m_netTieCache.end() )
+            return emptySet;
+
+        return it->second;
+    }
+
     /**
      * Generate pads shapes on layer \a aLayer as polygons and adds these polygons to
      * \a aBuffer.
@@ -495,7 +584,7 @@ public:
      */
     bool ResolveTextVar( wxString* token, int aDepth = 0 ) const;
 
-    ///< @copydoc EDA_ITEM::GetMsgPanelInfo
+    /// @copydoc EDA_ITEM::GetMsgPanelInfo
     void GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>& aList ) override;
 
     bool HitTest( const VECTOR2I& aPosition, int aAccuracy = 0 ) const override;
@@ -513,22 +602,28 @@ public:
 
     bool HitTest( const BOX2I& aRect, bool aContained, int aAccuracy = 0 ) const override;
 
+    /**
+     * Test if the point hits one or more of the footprint elements on a given layer.
+     *
+     * @param aPosition is the point to test
+     * @param aAccuracy is the hit test accuracy
+     * @param aLayer is the layer to test
+     * @return true if aPosition hits a footprint element on aLayer
+     */
+    bool HitTestOnLayer( const VECTOR2I& aPosition, PCB_LAYER_ID aLayer, int aAccuracy = 0 ) const;
+
+    bool HitTestOnLayer( const BOX2I& aRect, bool aContained, PCB_LAYER_ID aLayer, int aAccuracy = 0 ) const;
+
     /**
      * @return reference designator text.
      */
-    const wxString& GetReference() const
-    {
-        return m_reference->GetText();
-    }
+    const wxString& GetReference() const { return Reference().GetText(); }
 
     /**
      * @param aReference A reference to a wxString object containing the reference designator
      *                   text.
      */
-    void SetReference( const wxString& aReference )
-    {
-        m_reference->SetText( aReference );
-    }
+    void SetReference( const wxString& aReference ) { Reference().SetText( aReference ); }
 
     // Property system doesn't like const references
     wxString GetReferenceAsString() const
@@ -544,18 +639,12 @@ public:
     /**
      * @return the value text.
      */
-    const wxString& GetValue() const
-    {
-        return m_value->GetText();
-    }
+    const wxString& GetValue() const { return Value().GetText(); }
 
     /**
      * @param aValue A reference to a wxString object containing the value text.
      */
-    void SetValue( const wxString& aValue )
-    {
-        m_value->SetText( aValue );
-    }
+    void SetValue( const wxString& aValue ) { Value().SetText( aValue ); }
 
     // Property system doesn't like const references
     wxString GetValueAsString() const
@@ -564,21 +653,103 @@ public:
     }
 
     /// read/write accessors:
-    FP_TEXT& Value()           { return *m_value; }
-    FP_TEXT& Reference()       { return *m_reference; }
+    PCB_FIELD& Value()           { return *GetField( VALUE_FIELD ); }
+    PCB_FIELD& Reference()       { return *GetField( REFERENCE_FIELD ); }
 
     /// The const versions to keep the compiler happy.
-    FP_TEXT& Value() const     { return *m_value; }
-    FP_TEXT& Reference() const { return *m_reference; }
+    const PCB_FIELD& Value() const     { return *GetField( VALUE_FIELD ); }
+    const PCB_FIELD& Reference() const { return *GetField( REFERENCE_FIELD ); }
 
-    const std::map<wxString, wxString>& GetProperties() const        { return m_properties; }
-    void SetProperties( const std::map<wxString, wxString>& aProps ) { m_properties = aProps; }
-    const wxString& GetProperty( const wxString& aKey)               { return m_properties[ aKey ]; }
-    bool HasProperty( const wxString& aKey)
-    {
-        return m_properties.find( aKey ) != m_properties.end();
-    }
-    void SetProperty( const wxString& aKey, const wxString& aVal )   { m_properties[ aKey ] = aVal; }
+    //-----<Fields>-----------------------------------------------------------
+
+    /**
+     * Return a mandatory field in this symbol.
+     *
+     * @note If you need to fetch a user field, use GetFieldById.
+     *
+     * @param aFieldType is one of the mandatory field types (REFERENCE_FIELD, VALUE_FIELD, etc.).
+     * @return is the field at \a aFieldType or NULL if the field does not exist.
+     */
+    PCB_FIELD*       GetField( MANDATORY_FIELD_T aFieldType );
+    const PCB_FIELD* GetField( MANDATORY_FIELD_T aFieldNdx ) const;
+
+    /**
+     * Return a field in this symbol.
+     *
+     * @param aFieldId is the id of the field requested.  Note that this id ONLY SOMETIMES equates
+     *                 to the field's position in the vector.
+     * @return is the field at \a aFieldType or NULL if the field does not exist.
+     */
+    PCB_FIELD* GetFieldById( int aFieldId );
+
+    /**
+     * Return a field in this symbol.
+     *
+     * @param aFieldName is the name of the field
+     *
+     * @return is the field with \a aFieldName or NULL if the field does not exist.
+     */
+    PCB_FIELD* GetFieldByName( const wxString& aFieldName );
+
+    bool HasFieldByName( const wxString& aFieldName ) const;
+
+    /**
+     * Search for a field named \a aFieldName and returns text associated with this field.
+     *
+     * @param aFieldName is the name of the field
+     */
+    wxString GetFieldText( const wxString& aFieldName ) const;
+
+    /**
+     * Populate a std::vector with PCB_TEXTs.
+     *
+     * @param aVector is the vector to populate.
+     * @param aVisibleOnly is used to add only the fields that are visible and contain text.
+     */
+    void GetFields( std::vector<PCB_FIELD*>& aVector, bool aVisibleOnly ) const;
+
+    /**
+     * Return a vector of fields from the symbol
+     *
+     * @param aVisibleOnly is used to add only the fields that are visible and contain text.
+     */
+    std::vector<PCB_FIELD*> GetFields( bool aVisibleOnly = false ) const;
+
+    /**
+     * Clears all fields from the footprint
+     */
+    void ClearFields() { m_fields.clear(); }
+
+    /**
+     * Add a field to the symbol.
+     *
+     * @param aField is the field to add to this symbol.
+     *
+     * @return the newly inserted field.
+     */
+    PCB_FIELD* AddField( const PCB_FIELD& aField );
+
+    /**
+     * Remove a user field from the footprint.
+     * @param aFieldName is the user fieldName to remove.  Attempts to remove a mandatory
+     *                   field or a non-existant field are silently ignored.
+     */
+    void RemoveField( const wxString& aFieldName );
+
+    /**
+     * Return the next ID for a field for this footprint
+     */
+    int GetNextFieldId() const { return m_fields.size(); }
+
+    /**
+     * @brief Apply default board settings to the footprint field text properties.
+     *
+     * This is needed because the board settings are not available when the footprint is
+     * being created in the footprint library cache, and we want these fields to have
+     * the correct default text properties.
+     */
+    void ApplyDefaultSettings( const BOARD& board, bool aStyleFields, bool aStyleText,
+                               bool aStyleShapes );
 
     bool IsBoardOnly() const { return m_attributes & FP_BOARD_ONLY; }
     void SetBoardOnly( bool aIsBoardOnly = true )
@@ -616,6 +787,15 @@ public:
             m_attributes &= ~FP_ALLOW_MISSING_COURTYARD;
     }
 
+    bool IsDNP() const { return m_attributes & FP_DNP; }
+    void SetDNP( bool aDNP = true )
+    {
+        if( aDNP )
+            m_attributes |= FP_DNP;
+        else
+            m_attributes &= ~FP_DNP;
+    }
+
     void SetFileFormatVersionAtLoad( int aVersion ) { m_fileFormatVersionAtLoad = aVersion; }
     int GetFileFormatVersionAtLoad() const { return m_fileFormatVersionAtLoad; }
 
@@ -639,6 +819,9 @@ public:
      */
     PAD* GetPad( const VECTOR2I& aPosition, LSET aLayerMask = LSET::AllLayersMask() );
 
+    std::vector<const PAD*> GetPads( const wxString& aPadNumber,
+                                     const PAD* aIgnore = nullptr ) const;
+
     /**
      * Return the number of pads.
      *
@@ -660,6 +843,12 @@ public:
      */
     unsigned GetUniquePadCount( INCLUDE_NPTH_T aIncludeNPTH = INCLUDE_NPTH_T(INCLUDE_NPTH) ) const;
 
+    /**
+     * Return the names of the unique, non-blank pads.
+     */
+    std::set<wxString>
+    GetUniquePadNumbers( INCLUDE_NPTH_T aIncludeNPTH = INCLUDE_NPTH_T(INCLUDE_NPTH) ) const;
+
     /**
      * Return the next available pad number in the footprint.
      *
@@ -669,6 +858,11 @@ public:
      */
     wxString GetNextPadNumber( const wxString& aLastPadName ) const;
 
+    /**
+     * Position Reference and Value fields at the top and bottom of footprint's bounding box.
+     */
+    void AutoPositionFields();
+
     /**
      * Get the type of footprint
      * @return "SMD"/"Through hole"/"Other" based on attributes
@@ -704,24 +898,22 @@ public:
         return wxT( "FOOTPRINT" );
     }
 
-    wxString GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const override;
+    wxString GetItemDescription( UNITS_PROVIDER* aUnitsProvider, bool aFull ) const override;
 
     BITMAPS GetMenuImage() const override;
 
     EDA_ITEM* Clone() const override;
 
-    /**
-     * Invoke a function on all BOARD_ITEMs that belong to the footprint (pads, drawings, texts).
-     *
-     * @note This function should not add or remove items to the footprint.
-     *
-     * @param aFunction is the function to be invoked.
-     */
-    void RunOnChildren( const std::function<void (BOARD_ITEM*)>& aFunction ) const;
+    ///< @copydoc BOARD_ITEM::RunOnChildren
+    void RunOnChildren( const std::function<void (BOARD_ITEM*)>& aFunction ) const override;
 
-    virtual void ViewGetLayers( int aLayers[], int& aCount ) const override;
+    ///< @copydoc BOARD_ITEM::RunOnDescendants
+    void RunOnDescendants( const std::function<void( BOARD_ITEM* )>& aFunction,
+                           int aDepth = 0 ) const override;
 
-    double ViewGetLOD( int aLayer, KIGFX::VIEW* aView ) const override;
+    virtual std::vector<int> ViewGetLayers() const override;
+
+    double ViewGetLOD( int aLayer, const KIGFX::VIEW* aView ) const override;
 
     virtual const BOX2I ViewBBox() const override;
 
@@ -748,7 +940,8 @@ public:
     /**
      * Return true if a board footprint differs from the library version.
      */
-    bool FootprintNeedsUpdate( const FOOTPRINT* aLibFootprint );
+    bool FootprintNeedsUpdate( const FOOTPRINT* aLibFP, int aCompareFlags = 0,
+                               REPORTER* aReporter = nullptr );
 
     /**
      * Take ownership of caller's heap allocated aInitialComments block.
@@ -780,7 +973,7 @@ public:
 
     static double GetCoverageArea( const BOARD_ITEM* aItem, const GENERAL_COLLECTOR& aCollector );
 
-    ///< Return the initial comments block or NULL if none, without transfer of ownership.
+    /// Return the initial comments block or NULL if none, without transfer of ownership.
     const wxArrayString* GetInitialComments() const { return m_initial_comments; }
 
     /**
@@ -790,6 +983,13 @@ public:
      */
     const SHAPE_POLY_SET& GetCourtyard( PCB_LAYER_ID aLayer ) const;
 
+    /**
+     * Return the cached courtyard area. No checks are performed.
+     *
+     * @return the cached courtyard polygon.
+     */
+    const SHAPE_POLY_SET& GetCachedCourtyard( PCB_LAYER_ID aLayer ) const;
+
     /**
      * Build complex polygons of the courtyard areas from graphic items on the courtyard layers.
      *
@@ -802,6 +1002,62 @@ public:
     std::shared_ptr<SHAPE> GetEffectiveShape( PCB_LAYER_ID aLayer = UNDEFINED_LAYER,
                                               FLASHING aFlash = FLASHING::DEFAULT ) const override;
 
+    EMBEDDED_FILES* GetEmbeddedFiles() override
+    {
+        return static_cast<EMBEDDED_FILES*>( this );
+    }
+
+    const EMBEDDED_FILES* GetEmbeddedFiles() const
+    {
+        return static_cast<const EMBEDDED_FILES*>( this );
+    }
+
+    /**
+     * Get a list of outline fonts referenced in the footprint
+     */
+    std::set<KIFONT::OUTLINE_FONT*> GetFonts() const override;
+
+    void EmbedFonts() override;
+
+    double Similarity( const BOARD_ITEM& aOther ) const override;
+
+    /// Sets the component class object pointer for this footprint
+    void SetComponentClass( const COMPONENT_CLASS* aClass ) { m_componentClass = aClass; }
+
+    /**
+     * @brief Sets the transient component class names
+     *
+     * This is used during paste operations as we can't resolve the component classes immediately
+     * until we have the true board context once the pasted items have been placed
+     */
+    void SetTransientComponentClassNames( const std::unordered_set<wxString>& classNames )
+    {
+        m_transientComponentClassNames = classNames;
+    }
+
+    /// Gets the transient component class names
+    const std::unordered_set<wxString>& GetTransientComponentClassNames()
+    {
+        return m_transientComponentClassNames;
+    }
+
+    /// Remove the transient component class names
+    void ClearTransientComponentClassNames() { m_transientComponentClassNames.clear(); };
+
+    /// Resolves a set of component class names to this footprint's actual component class
+    void ResolveComponentClassNames( BOARD*                              aBoard,
+                                     const std::unordered_set<wxString>& aComponentClassNames );
+
+    /// Returns the component class for this footprint
+    const COMPONENT_CLASS* GetComponentClass() const { return m_componentClass; }
+
+    /// Used for display in the properties panel
+    wxString GetComponentClassAsString() const;
+
+
+    bool operator==( const BOARD_ITEM& aOther ) const override;
+    bool operator==( const FOOTPRINT& aOther ) const;
+
 #if defined(DEBUG)
     virtual void Show( int nestLevel, std::ostream& os ) const override { ShowDummy( os ); }
 #endif
@@ -815,25 +1071,32 @@ public:
     {
         bool operator()( const PAD* aFirst, const PAD* aSecond ) const;
     };
-
+  // TODO(JE) padstacks -- this code isn't used anywhere though
+#if 0
+    struct cmp_padstack
+    {
+        bool operator()( const PAD* aFirst, const PAD* aSecond ) const;
+    };
+#endif
     struct cmp_zones
     {
-        bool operator()( const FP_ZONE* aFirst, const FP_ZONE* aSecond ) const;
+        bool operator()( const ZONE* aFirst, const ZONE* aSecond ) const;
     };
 
 protected:
     virtual void swapData( BOARD_ITEM* aImage ) override;
 
+    void addMandatoryFields();
+
 private:
+    PCB_FIELDS      m_fields;
     DRAWINGS        m_drawings;          // BOARD_ITEMs for drawings on the board, owned by pointer.
-    PADS            m_pads;              // PAD items, owned by pointer
-    FP_ZONES        m_fp_zones;          // FP_ZONE items, owned by pointer
-    FP_GROUPS       m_fp_groups;         // PCB_GROUP items, owned by pointer
+    std::deque<PAD*> m_pads;              // PAD items, owned by pointer
+    ZONES           m_zones;             // PCB_ZONE items, owned by pointer
+    GROUPS          m_groups;            // PCB_GROUP items, owned by pointer
 
     EDA_ANGLE       m_orient;            // Orientation
     VECTOR2I        m_pos;               // Position of footprint on the board in internal units.
-    FP_TEXT*        m_reference;         // Component reference designator value (U34, R18..)
-    FP_TEXT*        m_value;             // Component value (74LS00, 22K..)
     LIB_ID          m_fpid;              // The #LIB_ID of the FOOTPRINT.
     int             m_attributes;        // Flag bits (see FOOTPRINT_ATTR_T)
     int             m_fpStatus;          // For autoplace: flags (LOCKED, FIELDS_AUTOPLACED)
@@ -851,8 +1114,6 @@ private:
     // fragile.
     mutable BOX2I          m_cachedBoundingBox;
     mutable int            m_boundingBoxCacheTimeStamp;
-    mutable BOX2I          m_cachedVisibleBBox;
-    mutable int            m_visibleBBoxCacheTimeStamp;
     mutable BOX2I          m_cachedTextExcludedBBox;
     mutable int            m_textExcludedBBoxCacheTimeStamp;
     mutable SHAPE_POLY_SET m_cachedHull;
@@ -862,28 +1123,40 @@ private:
     // A pad group is a comma-separated list of pad numbers.
     std::vector<wxString> m_netTiePadGroups;
 
-    ZONE_CONNECTION m_zoneConnection;
-    int             m_localClearance;
-    int             m_localSolderMaskMargin;       // Solder mask margin
-    int             m_localSolderPasteMargin;      // Solder paste margin absolute value
-    double          m_localSolderPasteMarginRatio; // Solder mask margin ratio value of pad size
+    // A list of 1:N footprint item to allowed net numbers
+    std::map<const BOARD_ITEM*, std::set<int>> m_netTieCache;
 
-    wxString        m_doc;               // File name and path for documentation file.
+    // Optional overrides
+    ZONE_CONNECTION       m_zoneConnection;
+    std::optional<int>    m_clearance;
+    std::optional<int>    m_solderMaskMargin;       // Solder mask margin
+    std::optional<int>    m_solderPasteMargin;      // Solder paste margin absolute value
+    std::optional<double> m_solderPasteMarginRatio; // Solder mask margin ratio of pad size
+                                                    // The final margin is the sum of these 2 values
+
+    wxString        m_libDescription;    // File name and path for documentation file.
     wxString        m_keywords;          // Search keywords to find footprint in library.
     KIID_PATH       m_path;              // Path to associated symbol ([sheetUUID, .., symbolUUID]).
+    wxString        m_sheetname;         // Name of the sheet containing the symbol for this footprint
+    wxString        m_sheetfile;         // File of the sheet containing the symbol for this footprint
+    wxString        m_filters;           // Footprint filters from symbol
     timestamp_t     m_lastEditTime;
     int             m_arflag;            // Use to trace ratsnest and auto routing.
     KIID            m_link;              // Temporary logical link used during editing
     LSET            m_privateLayers;     // Layers visible only in the footprint editor
 
     std::vector<FP_3DMODEL>       m_3D_Drawings;       // 3D models.
-    std::map<wxString, wxString>  m_properties;
     wxArrayString*                m_initial_comments;  // s-expression comments in the footprint,
                                                        // lazily allocated only if needed for speed
 
-    SHAPE_POLY_SET  m_courtyard_cache_front;  // Note that a footprint can have both front and back
-    SHAPE_POLY_SET  m_courtyard_cache_back;   // courtyards populated.
-    mutable int     m_courtyard_cache_timestamp;
+    SHAPE_POLY_SET   m_courtyard_cache_front; // Note that a footprint can have both front and back
+    SHAPE_POLY_SET   m_courtyard_cache_back;  // courtyards populated.
+    mutable HASH_128   m_courtyard_cache_front_hash;
+    mutable HASH_128   m_courtyard_cache_back_hash;
+    mutable std::mutex m_courtyard_cache_mutex;
+
+    const COMPONENT_CLASS* m_componentClass;
+    std::unordered_set<wxString> m_transientComponentClassNames;
 };
 
 #endif     // FOOTPRINT_H
