@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2004 Jean-Pierre Charras, jaen-pierre.charras@gipsa-lab.inpg.com
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,6 +43,20 @@ enum class DIM_UNITS_FORMAT
     PAREN_SUFFIX    // 1234.0 (mm)
 };
 
+enum class DIM_PRECISION
+{
+    X = 0,          // 0
+    X_X,            // 0.0
+    X_XX,           // 0.00
+    X_XXX,          // 0.000
+    X_XXXX,         // 0.0000
+    X_XXXXX,        // 0.00000
+    V_VV,           // 0.00 / 0 / 0.0
+    V_VVV,          // 0.000 / 0 / 0.00
+    V_VVVV,         // 0.0000 / 0.0 / 0.000
+    V_VVVVV         // 0.00000 / 0.00 / 0.0000
+};
+
 /// Where to place the text on a dimension
 enum class DIM_TEXT_POSITION
 {
@@ -62,6 +76,15 @@ enum class DIM_UNITS_MODE
     AUTOMATIC
 };
 
+/**
+* Used for dimension's arrow.
+*/
+enum class DIM_ARROW_DIRECTION
+{
+    INWARD,  ///< >-----<
+    OUTWARD, ///< <----->
+};
+
 /**
  * Frame to show around dimension text
  */
@@ -92,26 +115,13 @@ enum class DIM_TEXT_BORDER
  * - "crossbar" refers to the perpendicular line (usually with arrows at each end) between feature
  *   lines on linear dimensions
  */
-class PCB_DIMENSION_BASE : public BOARD_ITEM
+class PCB_DIMENSION_BASE : public PCB_TEXT
 {
 public:
     PCB_DIMENSION_BASE( BOARD_ITEM* aParent, KICAD_T aType = PCB_DIMENSION_T );
 
-    bool IsType( const std::vector<KICAD_T>& aScanTypes ) const override
-    {
-        if( BOARD_ITEM::IsType( aScanTypes ) )
-            return true;
-
-        for( KICAD_T scanType : aScanTypes )
-        {
-            if( scanType == PCB_LOCATE_GRAPHIC_T )
-                return true;
-        }
-
-        return false;
-    }
-
-    void SetParent( EDA_ITEM* aParent ) override;
+    void Serialize( google::protobuf::Any &aContainer ) const override;
+    bool Deserialize( const google::protobuf::Any &aContainer ) override;
 
     /**
      * The dimension's origin is the first feature point for the dimension.  Every dimension has
@@ -133,6 +143,13 @@ public:
     wxString GetOverrideText() const { return m_valueString; }
     void SetOverrideText( const wxString& aValue ) { m_valueString = aValue; }
 
+    void ChangeOverrideText( const wxString& aValue )
+    {
+        SetOverrideTextEnabled( true );
+        SetOverrideText( aValue );
+        Update();
+    }
+
     int GetMeasuredValue() const { return m_measuredValue; }
 
     // KiCad normally calculates the measured value but some importers need to set it.
@@ -145,39 +162,102 @@ public:
 
     /**
      * Update the dimension's cached text and geometry.
+     *
+     * Call this whenever you change something in the geometry
+     * definition, or the text (which can affect geometry, e.g. by
+     * a knockout of a crossbar line or similar)
      */
     void Update()
     {
+        // Calls updateText internally
         updateGeometry();
-        updateText();
+    }
+
+    void UpdateUnits()
+    {
+        SetUnitsMode( GetUnitsMode() );
+        Update();
     }
 
     wxString GetPrefix() const { return m_prefix; }
     void SetPrefix( const wxString& aPrefix );
 
+    void ChangePrefix( const wxString& aPrefix )
+    {
+        SetPrefix( aPrefix );
+        Update();
+    }
+
     wxString GetSuffix() const { return m_suffix; }
     void SetSuffix( const wxString& aSuffix );
 
+    void ChangeSuffix( const wxString& aSuffix )
+    {
+        SetSuffix( aSuffix );
+        Update();
+    }
+
+    DIM_ARROW_DIRECTION GetArrowDirection() const { return m_arrowDirection; }
+    void                SetArrowDirection( const DIM_ARROW_DIRECTION& aDirection )
+    {
+        m_arrowDirection = aDirection;
+    }
+
+    void ChangeArrowDirection( const DIM_ARROW_DIRECTION& aDirection )
+    {
+        SetArrowDirection( aDirection );
+        updateText();
+        updateGeometry();
+    }
+
     EDA_UNITS GetUnits() const { return m_units; }
     void SetUnits( EDA_UNITS aUnits );
 
     DIM_UNITS_MODE GetUnitsMode() const;
     void SetUnitsMode( DIM_UNITS_MODE aMode );
 
+    void ChangeUnitsMode( DIM_UNITS_MODE aMode )
+    {
+        SetUnitsMode( aMode );
+        Update();
+    }
+
     void SetAutoUnits( bool aAuto = true ) { m_autoUnits = aAuto; }
 
     DIM_UNITS_FORMAT GetUnitsFormat() const { return m_unitsFormat; }
     void SetUnitsFormat( const DIM_UNITS_FORMAT aFormat ) { m_unitsFormat = aFormat; }
 
-    int GetPrecision() const { return m_precision; }
-    void SetPrecision( int aPrecision ) { m_precision = aPrecision; }
+    void ChangeUnitsFormat( const DIM_UNITS_FORMAT aFormat )
+    {
+        SetUnitsFormat( aFormat );
+        Update();
+    }
+
+    DIM_PRECISION GetPrecision() const { return m_precision; }
+    void SetPrecision( DIM_PRECISION aPrecision ) { m_precision = aPrecision; }
+
+    void ChangePrecision( DIM_PRECISION aPrecision )
+    {
+        SetPrecision( aPrecision );
+        Update();
+    }
 
     bool GetSuppressZeroes() const { return m_suppressZeroes; }
     void SetSuppressZeroes( bool aSuppress ) { m_suppressZeroes = aSuppress; }
 
+    void ChangeSuppressZeroes( bool aSuppress )
+    {
+        SetSuppressZeroes( aSuppress );
+        Update();
+    }
+
     bool GetKeepTextAligned() const { return m_keepTextAligned; }
     void SetKeepTextAligned( bool aKeepAligned ) { m_keepTextAligned = aKeepAligned; }
 
+    double GetTextAngleDegreesProp() const { return GetTextAngleDegrees(); }
+    void ChangeTextAngleDegrees( double aDegrees );
+    void ChangeKeepTextAligned( bool aKeepAligned );
+
     void SetTextPositionMode( DIM_TEXT_POSITION aMode ) { m_textPosition = aMode; }
     DIM_TEXT_POSITION GetTextPositionMode() const { return m_textPosition; }
 
@@ -190,30 +270,6 @@ public:
     int GetLineThickness() const        { return m_lineThickness; }
     void SetLineThickness( int aWidth ) { m_lineThickness = aWidth; }
 
-    void SetLayer( PCB_LAYER_ID aLayer ) override;
-
-    void SetTextSize( const wxSize& aTextSize )
-    {
-        m_text.SetTextSize( aTextSize );
-    }
-
-    /**
-     * Set the override text - has no effect if m_overrideValue == false.
-     *
-     * @param aNewText is the text to use as the value.
-     */
-    void           SetText( const wxString& aNewText );
-
-    /**
-     * Retrieve the value text or override text, not including prefix or suffix.
-     *
-     * @return the value portion of the dimension text (either overridden or not).
-     */
-    const wxString GetText() const;
-
-    PCB_TEXT& Text() { return m_text; }
-    const PCB_TEXT& Text() const { return m_text; }
-
     /**
      * @return a list of line segments that make up this dimension (for drawing, plotting, etc).
      */
@@ -223,7 +279,7 @@ public:
 
     void Move( const VECTOR2I& offset ) override;
     void Rotate( const VECTOR2I& aRotCentre, const EDA_ANGLE& aAngle ) override;
-    void Flip( const VECTOR2I& aCentre, bool aFlipLeftRight ) override;
+    void Flip( const VECTOR2I& aCentre, FLIP_DIRECTION aFlipDirection ) override;
 
     /**
      * Mirror the dimension relative to a given horizontal axis.
@@ -233,7 +289,7 @@ public:
      *
      * @param axis_pos is the vertical axis position to mirror around.
      */
-    virtual void Mirror( const VECTOR2I& axis_pos, bool aMirrorLeftRight = false );
+    virtual void Mirror( const VECTOR2I& axis_pos, FLIP_DIRECTION aFlipDirection ) override;
 
     void GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>& aList ) override;
 
@@ -245,14 +301,21 @@ public:
     std::shared_ptr<SHAPE> GetEffectiveShape( PCB_LAYER_ID aLayer,
             FLASHING aFlash = FLASHING::DEFAULT ) const override;
 
-    wxString GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const override;
+    wxString GetItemDescription( UNITS_PROVIDER* aUnitsProvider, bool aFull ) const override;
 
     const BOX2I ViewBBox() const override;
 
+    void ClearRenderCache() override;
+
     void TransformShapeToPolygon( SHAPE_POLY_SET& aBuffer, PCB_LAYER_ID aLayer, int aClearance,
                                   int aError, ERROR_LOC aErrorLoc,
                                   bool aIgnoreLineWidth = false ) const override;
 
+    double Similarity( const BOARD_ITEM& aOther ) const override;
+
+    bool operator==( const PCB_DIMENSION_BASE& aOther ) const;
+    bool operator==( const BOARD_ITEM& aBoardItem ) const override;
+
 #if defined(DEBUG)
     virtual void Show( int nestLevel, std::ostream& os ) const override { ShowDummy( os ); }
 #endif
@@ -266,6 +329,9 @@ protected:
 
     /**
      * Update the text field value from the current geometry (called by updateGeometry normally).
+     *
+     * If you change the text, you should call updateGeometry which will call this,
+     * and also handle any text-dependent geoemtry handling (like a knockout)
      */
     virtual void updateText();
 
@@ -273,43 +339,43 @@ protected:
     void addShape( const ShapeType& aShape );
 
     /**
-     * Find the intersection between a given segment and polygon outline.
+     * Draws an arrow and updates the shape container.
+     * example arrow 0Deg tail:4  (---->)
      *
-     * @param aPoly is the polygon to collide.
-     * @param aSeg is the segment to collide.
-     * @param aStart if true will start from aSeg.A, otherwise aSeg.B.
-     * @return a point on aSeg that collides with aPoly closest to the start, if one exists.
+     * @param startPoint arrow point.
+     * @param anAngle arrow angle.
+     * @param aLength arrow tail length.
      */
-    static OPT_VECTOR2I segPolyIntersection( const SHAPE_POLY_SET& aPoly, const SEG& aSeg,
-                                             bool aStart = true );
-    static OPT_VECTOR2I segCircleIntersection( CIRCLE& aCircle, SEG& aSeg, bool aStart = true );
+    void drawAnArrow( VECTOR2I aStartPoint, EDA_ANGLE anAngle, int aLength );
 
     // Value format
-    bool              m_overrideTextEnabled;   ///< Manually specify the displayed measurement value
-    wxString          m_valueString;     ///< Displayed value when m_overrideValue = true
-    wxString          m_prefix;          ///< String prepended to the value
-    wxString          m_suffix;          ///< String appended to the value
-    EDA_UNITS         m_units;           ///< 0 = inches, 1 = mm
-    bool              m_autoUnits;       ///< If true, follow the currently selected UI units
-    DIM_UNITS_FORMAT  m_unitsFormat;     ///< How to render the units suffix
-    int               m_precision;       ///< Number of digits to display after decimal
-    bool              m_suppressZeroes;  ///< Suppress trailing zeroes
+    bool                    m_overrideTextEnabled;   ///< Manually specify the displayed measurement value
+    wxString                m_valueString;     ///< Displayed value when m_overrideValue = true
+    wxString                m_prefix;          ///< String prepended to the value
+    wxString                m_suffix;          ///< String appended to the value
+    EDA_UNITS               m_units;           ///< 0 = inches, 1 = mm
+    bool                    m_autoUnits;       ///< If true, follow the currently selected UI units
+    DIM_UNITS_FORMAT        m_unitsFormat;     ///< How to render the units suffix
+    DIM_ARROW_DIRECTION     m_arrowDirection;  ///< direction of dimension arrow.
+    DIM_PRECISION           m_precision;       ///< Number of digits to display after decimal
+    bool                    m_suppressZeroes;  ///< Suppress trailing zeroes
 
     // Geometry
-    int               m_lineThickness;    ///< Thickness used for all graphics in the dimension
-    int               m_arrowLength;      ///< Length of arrow shapes
-    int               m_extensionOffset;  ///< Distance from feature points to extension line start
-    DIM_TEXT_POSITION m_textPosition;     ///< How to position the text
-    bool              m_keepTextAligned;  ///< Calculate text orientation to match dimension
+    int                     m_lineThickness;    ///< Thickness used for all graphics in the dimension
+    int                     m_arrowLength;      ///< Length of arrow shapes
+    int                     m_extensionOffset;  ///< Distance from feature points to extension line start
+    DIM_TEXT_POSITION       m_textPosition;     ///< How to position the text
+    bool                    m_keepTextAligned;  ///< Calculate text orientation to match dimension
 
     // Internal
-    PCB_TEXT          m_text;             ///< The actual text object
-    int               m_measuredValue;    ///< value of PCB dimensions
-    VECTOR2I          m_start;
-    VECTOR2I          m_end;
+    int                     m_measuredValue;    ///< value of PCB dimensions
+    VECTOR2I                m_start;
+    VECTOR2I                m_end;
 
     ///< Internal cache of drawn shapes
     std::vector<std::shared_ptr<SHAPE>> m_shapes;
+
+    bool       m_inClearRenderCache;      ///< re-entrancy guard
 };
 
 
@@ -343,22 +409,24 @@ protected:
 class PCB_DIM_ALIGNED : public PCB_DIMENSION_BASE
 {
 public:
-    PCB_DIM_ALIGNED( BOARD_ITEM* aParent, KICAD_T aType );
+    PCB_DIM_ALIGNED( BOARD_ITEM* aParent, KICAD_T aType = PCB_DIM_ALIGNED_T );
 
     // Do not create a copy constructor & operator=.
     // The ones generated by the compiler are adequate.
 
     ~PCB_DIM_ALIGNED() = default;
 
+    void Serialize( google::protobuf::Any &aContainer ) const override;
+    bool Deserialize( const google::protobuf::Any &aContainer ) override;
+
     static inline bool ClassOf( const EDA_ITEM* aItem )
     {
-        return aItem && ( aItem->Type() == PCB_DIM_ALIGNED_T
-                            || aItem->Type() == PCB_FP_DIM_ALIGNED_T );
+        return aItem && aItem->Type() == PCB_DIM_ALIGNED_T;
     }
 
     EDA_ITEM* Clone() const override;
 
-    void Mirror( const VECTOR2I& axis_pos, bool aMirrorLeftRight = false ) override;
+    void Mirror( const VECTOR2I& axis_pos, FLIP_DIRECTION aFlipDirection ) override;
 
     BITMAPS GetMenuImage() const override;
 
@@ -374,6 +442,12 @@ public:
     void SetHeight( int aHeight ) { m_height = aHeight; }
     int GetHeight() const {  return m_height; }
 
+    void ChangeHeight( int aHeight )
+    {
+        SetHeight( aHeight );
+        Update();
+    }
+
     /**
      * Update the stored height basing on points coordinates.
      *
@@ -384,6 +458,12 @@ public:
     void SetExtensionHeight( int aHeight ) { m_extensionHeight = aHeight; }
     int GetExtensionHeight() const { return m_extensionHeight; }
 
+    void ChangeExtensionHeight( int aHeight )
+    {
+        SetExtensionHeight( aHeight );
+        Update();
+    }
+
     /**
      * Return the angle of the crossbar.
      *
@@ -433,18 +513,22 @@ public:
         VERTICAL    // Aligned with y-axis
     };
 
-    PCB_DIM_ORTHOGONAL( BOARD_ITEM* aParent, bool aInFP = false );
+    PCB_DIM_ORTHOGONAL( BOARD_ITEM* aParent );
 
     ~PCB_DIM_ORTHOGONAL() = default;
 
+    void Serialize( google::protobuf::Any &aContainer ) const override;
+    bool Deserialize( const google::protobuf::Any &aContainer ) override;
+
     static inline bool ClassOf( const EDA_ITEM* aItem )
     {
-        return aItem && ( aItem->Type() == PCB_DIM_ORTHOGONAL_T
-                            || aItem->Type() == PCB_FP_DIM_ORTHOGONAL_T );
+        return aItem && aItem->Type() == PCB_DIM_ORTHOGONAL_T;
     }
 
     EDA_ITEM* Clone() const override;
 
+    void Mirror( const VECTOR2I& axis_pos, FLIP_DIRECTION aFlipDirection ) override;
+
     BITMAPS GetMenuImage() const override;
 
     /**
@@ -498,12 +582,14 @@ private:
 class PCB_DIM_RADIAL : public PCB_DIMENSION_BASE
 {
 public:
-    PCB_DIM_RADIAL( BOARD_ITEM* aParent, bool aInFP = false );
+    PCB_DIM_RADIAL( BOARD_ITEM* aParent );
+
+    void Serialize( google::protobuf::Any &aContainer ) const override;
+    bool Deserialize( const google::protobuf::Any &aContainer ) override;
 
     static inline bool ClassOf( const EDA_ITEM* aItem )
     {
-        return aItem && ( aItem->Type() == PCB_DIM_RADIAL_T
-                            || aItem->Type() == PCB_FP_DIM_RADIAL_T );
+        return aItem && aItem->Type() == PCB_DIM_RADIAL_T;
     }
 
     EDA_ITEM* Clone() const override;
@@ -511,6 +597,12 @@ public:
     void SetLeaderLength( int aLength ) { m_leaderLength = aLength; }
     int GetLeaderLength() const { return m_leaderLength; }
 
+    void ChangeLeaderLength( int aLength )
+    {
+        SetLeaderLength( aLength );
+        Update();
+    }
+
     // Returns the point (c).
     VECTOR2I GetKnee() const;
 
@@ -528,7 +620,6 @@ protected:
     void updateGeometry() override;
 
 private:
-    bool m_isDiameter;
     int  m_leaderLength;
 };
 
@@ -549,12 +640,14 @@ private:
 class PCB_DIM_LEADER : public PCB_DIMENSION_BASE
 {
 public:
-    PCB_DIM_LEADER( BOARD_ITEM* aParent, bool aInFP = false );
+    PCB_DIM_LEADER( BOARD_ITEM* aParent );
+
+    void Serialize( google::protobuf::Any &aContainer ) const override;
+    bool Deserialize( const google::protobuf::Any &aContainer ) override;
 
     static inline bool ClassOf( const EDA_ITEM* aItem )
     {
-        return aItem && ( aItem->Type() == PCB_DIM_LEADER_T
-                            || aItem->Type() == PCB_FP_DIM_LEADER_T );
+        return aItem && aItem->Type() == PCB_DIM_LEADER_T;
     }
 
     EDA_ITEM* Clone() const override;
@@ -566,14 +659,21 @@ public:
         return wxT( "PCB_DIM_LEADER" );
     }
 
-    void SetTextBorder( DIM_TEXT_BORDER aFrame ) { m_textBorder = aFrame; }
+    void SetTextBorder( DIM_TEXT_BORDER aBorder ) { m_textBorder = aBorder; }
     DIM_TEXT_BORDER GetTextBorder() const { return m_textBorder; }
 
+    void ChangeTextBorder( DIM_TEXT_BORDER aBorder )
+    {
+        SetTextBorder( aBorder );
+        Update();
+    }
+
     void GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>& aList ) override;
 
 protected:
     virtual void swapData( BOARD_ITEM* aImage ) override;
 
+    void updateText() override;
     void updateGeometry() override;
 
 private:
@@ -590,12 +690,14 @@ private:
 class PCB_DIM_CENTER : public PCB_DIMENSION_BASE
 {
 public:
-    PCB_DIM_CENTER( BOARD_ITEM* aParent, bool aInFP = false );
+    PCB_DIM_CENTER( BOARD_ITEM* aParent );
+
+    void Serialize( google::protobuf::Any &aContainer ) const override;
+    bool Deserialize( const google::protobuf::Any &aContainer ) override;
 
     static inline bool ClassOf( const EDA_ITEM* aItem )
     {
-        return aItem && ( aItem->Type() == PCB_DIM_CENTER_T
-                            || aItem->Type() == PCB_FP_DIM_CENTER_T );
+        return aItem && aItem->Type() == PCB_DIM_CENTER_T;
     }
 
     EDA_ITEM* Clone() const override;
