@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2010 Jean-Pierre Charras, jp.charras@wanadoo.fr
- * Copyright (C) 2010-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -24,12 +24,15 @@
 #include "pcb_base_edit_frame.h"
 #include "zones.h"
 #include <mail_type.h>
+#include <settings/app_settings.h>
+#include <variant>
 
 class ACTION_PLUGIN;
 class PCB_SCREEN;
 class BOARD;
 class BOARD_COMMIT;
 class BOARD_ITEM_CONTAINER;
+class DIALOG_BOOK_REPORTER;
 class FOOTPRINT;
 class PCB_TRACK;
 class PCB_VIA;
@@ -55,6 +58,13 @@ class FP_LIB_TABLE;
 class BOARD_NETLIST_UPDATER;
 class ACTION_MENU;
 class TOOL_ACTION;
+class DIALOG_BOARD_SETUP;
+
+#ifdef KICAD_IPC_API
+class KICAD_API_SERVER;
+class API_HANDLER_PCB;
+class API_HANDLER_COMMON;
+#endif
 
 enum LAST_PATH_TYPE : unsigned int;
 
@@ -142,23 +152,19 @@ public:
     /**
      * Find the next item using our existing search parameters.
      */
-    void FindNext();
+    void FindNext( bool reverse = false );
 
     /**
      * Open a dialog frame to create plot and drill files relative to the current board.
      */
     void ToPlotter( int aID );
 
-    /**
-     * Show the Export to SVG file dialog.
-     */
-    void ExportSVG( wxCommandEvent& event );
-
     // User interface update command event handlers.
     void OnUpdateLayerSelectBox( wxUpdateUIEvent& aEvent );
 
     bool LayerManagerShown();
     bool PropertiesShown();
+    bool NetInspectorShown();
 
     void OnUpdateSelectViaSize( wxUpdateUIEvent& aEvent );
     void OnUpdateSelectTrackWidth( wxUpdateUIEvent& aEvent );
@@ -166,8 +172,9 @@ public:
 
     void RunEeschema();
 
-    void UpdateTrackWidthSelectBox( wxChoice* aTrackWidthSelectBox, bool aEdit = true );
-    void UpdateViaSizeSelectBox( wxChoice* aViaSizeSelectBox, bool aEdit = true );
+    void UpdateTrackWidthSelectBox( wxChoice* aTrackWidthSelectBox, bool aShowNetclass,
+                                    bool aShowEdit );
+    void UpdateViaSizeSelectBox( wxChoice* aViaSizeSelectBox, bool aShowNetclass, bool aShowEdit );
 
     /**
      * Return the angle used for rotate operations.
@@ -192,9 +199,9 @@ public:
 
     /**
      * Return ordered list of plugins in sequence in which they should appear on toolbar or
-     * in settings
+     * in settings.  Handles both legacy (SWIG) and API plugins, so returns a heterogenous list.
      */
-    static std::vector<ACTION_PLUGIN*> GetOrderedActionPlugins();
+    static std::vector<std::variant<ACTION_PLUGIN*, const PLUGIN_ACTION*>> GetOrderedActionPlugins();
 
     void SaveProjectLocalSettings() override;
 
@@ -210,6 +217,11 @@ public:
 
     void SaveSettings( APP_SETTINGS_BASE* aCfg ) override;
 
+    /**
+     * Load the drawing sheet file.
+     */
+    void LoadDrawingSheet();
+
     /**
      * Get the last path for a particular type.
      *
@@ -231,14 +243,10 @@ public:
     void SetLastPath( LAST_PATH_TYPE aType, const wxString& aLastPath );
 
     /**
-     * Scan existing markers and record data from any that are Excluded.
+     * If aCreateMarkers then create DRC exclusion markers from the serialized data.  If false,
+     * then use the serialized data to set exclusion flags on existing markers.
      */
-    void RecordDRCExclusions();
-
-    /**
-     * Update markers to match recorded exclusions.
-     */
-    void ResolveDRCExclusions();
+    void ResolveDRCExclusions( bool aCreateMarkers );
 
     void Process_Special_Functions( wxCommandEvent& event );
     void Tracks_and_Vias_Size_Event( wxCommandEvent& event );
@@ -301,7 +309,9 @@ public:
     void PrepareLayerIndicator( bool aForceRebuild = false );
 
     void ToggleLayersManager();
-    void ToggleProperties();
+
+    void ToggleNetInspector();
+
     void ToggleSearch();
 
     /**
@@ -312,6 +322,7 @@ public:
      * @param aOnlySMD true to force only footprints flagged smd to be in the list
      * @param aNoTHItems true to include only footprints with no TH pads no matter
      *                   the footprint flag
+     * @param aExcludeDNP true to exclude footprints flagged DNP
      * @param aTopSide true to list footprints on front (top) side.
      * @param aBottomSide true to list footprints on back (bottom) side, if \a aTopSide and
      *                    \a aTopSide are true, list footprints on both sides.
@@ -321,7 +332,7 @@ public:
      * @return the number of footprints found on aSide side or -1 if the file could not be created.
      */
     int DoGenFootprintsPositionFile( const wxString& aFullFileName, bool aUnitsMM, bool aOnlySMD,
-                                     bool aNoTHItems, bool aTopSide, bool aBottomSide,
+                                     bool aNoTHItems, bool aExcludeDNP, bool aTopSide, bool aBottomSide,
                                      bool aFormatCSV, bool aUseAuxOrigin, bool aNegateBottomX );
 
     /**
@@ -329,6 +340,16 @@ public:
      */
     void GenFootprintsReport( wxCommandEvent& event );
 
+    /**
+     * Create and IPC2581 output file
+    */
+    void GenIPC2581File( wxCommandEvent& event );
+
+    /**
+     * Create and Generate ODB++ output files
+    */
+    void GenODBPPFiles( wxCommandEvent& event );
+
     /**
      * Create an ASCII footprint report file giving some infos on footprints and board outlines.
      *
@@ -402,17 +423,20 @@ public:
      *
      * @param aFileName The file name to write.
      * @param aCreateProject will create an empty project alongside the board file
+     * @param aHeadless will suppress informational output (e.g. to be used from the API)
      * @return True if file was saved successfully.
      */
-    bool SavePcbCopy( const wxString& aFileName, bool aCreateProject = false );
+    bool SavePcbCopy( const wxString& aFileName, bool aCreateProject = false,
+                      bool aHeadless = false );
 
     /**
      * Delete all and reinitialize the current board.
      *
-     * @param aQuery true to prompt user for confirmation, false to initialize silently.
+     * @param doAskAboutUnsavedChanges true to prompt user if existing board contains unsaved
+     *                                 changes, false to re-initialize silently.
      * @param aFinal if true, we are clearing the board to exit, so don't run more events.
      */
-    bool Clear_Pcb( bool aQuery, bool aFinal = false );
+    bool Clear_Pcb( bool doAskAboutUnsavedChanges, bool aFinal = false );
 
     ///< @copydoc PCB_BASE_FRAME::SetBoard()
     void SetBoard( BOARD* aBoard, PROGRESS_REPORTER* aReporter = nullptr ) override
@@ -425,6 +449,8 @@ public:
     ///< @copydoc PCB_BASE_FRAME::GetModel()
     BOARD_ITEM_CONTAINER* GetModel() const override;
 
+    std::unique_ptr<GRID_HELPER> MakeGridHelper() override;
+
     ///< @copydoc PCB_BASE_FRAME::SetPageSettings()
     void SetPageSettings( const PAGE_INFO& aPageSettings ) override;
 
@@ -489,6 +515,7 @@ public:
      * @return true if Ok.
      */
     bool ExportVRML_File( const wxString& aFullFileName, double aMMtoWRMLunit,
+                          bool aIncludeUnspecified, bool aIncludeDNP,
                           bool aExport3DFiles, bool aUseRelativePaths,
                           const wxString& a3D_Subdir, double aXRef, double aYRef );
 
@@ -502,17 +529,6 @@ public:
      */
     void OnExportHyperlynx( wxCommandEvent& event );
 
-    /**
-     * run teardrop tool
-     */
-    void OnRunTeardropTool( wxCommandEvent& event );
-
-    /**
-     * Remove all teardrops
-     */
-    void OnRemoveTeardropTool( wxCommandEvent& event );
-
-
     /**
      * Create an IDF3 compliant BOARD (*.emn) and LIBRARY (*.emp) file.
      *
@@ -521,10 +537,13 @@ public:
      * @param aUseThou set to true if the desired IDF unit is thou (mil).
      * @param aXRef the board Reference Point in mm, X value.
      * @param aYRef the board Reference Point in mm, Y value.
+     * @param aIncludeUnspecified true to include unspecified-type footprint models
+     * @param aIncludeDNP true to include DNP footprint models
      * @return true if OK.
      */
     bool Export_IDF3( BOARD* aPcb, const wxString& aFullFileName,
-                      bool aUseThou, double aXRef, double aYRef );
+                      bool aUseThou, double aXRef, double aYRef,
+                      bool aIncludeUnspecified, bool aIncludeDNP );
 
     /**
      * Export the current BOARD to a STEP assembly.
@@ -566,9 +585,8 @@ public:
     void ExchangeFootprint( FOOTPRINT* aExisting, FOOTPRINT* aNew, BOARD_COMMIT& aCommit,
                             bool deleteExtraTexts = true, bool resetTextLayers = true,
                             bool resetTextEffects = true, bool resetFabricationAttrs = true,
-                            bool reset3DModels = true, bool* aUpdated = nullptr );
-
-    bool FootprintMatchesLibrary();
+                            bool resetTextContent = true, bool reset3DModels = true,
+                            bool* aUpdated = nullptr );
 
     /**
      * Install the corresponding dialog editor for the given item.
@@ -592,13 +610,13 @@ public:
      * Note that casting this to boolean will allow you to determine whether any action
      * happened.
      *
-     * @param aTrackItem the track segment or via to modify.
+     * @param aItem the track segment or via to modify.
      * @param aItemsListPicker the list picker to use for an undo command (can be NULL).
-     * @param aUseNetclassValue true to use NetClass value, false to use current designSettings
-     *                          value.
+     * @param aUseDesignRules true to use design rules value, false to use current designSettings
+     *                        value.
      */
-    void SetTrackSegmentWidth( PCB_TRACK* aTrackItem, PICKED_ITEMS_LIST* aItemsListPicker,
-                               bool aUseNetclassValue );
+    void SetTrackSegmentWidth( PCB_TRACK* aItem, PICKED_ITEMS_LIST* aItemsListPicker,
+                               bool aUseDesignRules );
 
 
     /**
@@ -625,7 +643,7 @@ public:
      * -2 if the schematic cannot be opened and 1 if OK.
      * If OK, opens Eeschema, and opens the schematic for this project
      */
-    int TestStandalone( void );
+    int TestStandalone();
 
     /**
      * Read a netlist from a file into a #NETLIST object.
@@ -688,7 +706,7 @@ public:
     /**
      * Called after the preferences dialog is run.
      */
-    void CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVarsChanged ) override;
+    void CommonSettingsChanged( int aFlags ) override;
 
     /**
      * Called when light/dark theme is changed.
@@ -697,12 +715,22 @@ public:
 
     void ProjectChanged() override;
 
+    bool CanAcceptApiCommands() override;
+
     wxString GetCurrentFileName() const override;
 
     SELECTION& GetCurrentSelection() override;
 
     TOOL_ACTION* GetExportNetlistAction() { return m_exportNetlistAction; }
 
+    DIALOG_BOOK_REPORTER* GetInspectDrcErrorDialog();
+
+    DIALOG_BOOK_REPORTER* GetInspectConstraintsDialog();
+
+    DIALOG_BOOK_REPORTER* GetInspectClearanceDialog();
+
+    DIALOG_BOOK_REPORTER* GetFootprintDiffDialog();
+
     DECLARE_EVENT_TABLE()
 
 protected:
@@ -771,6 +799,8 @@ protected:
      */
     void OnActionPluginButton( wxCommandEvent& aEvent );
 
+    PLUGIN_ACTION_SCOPE PluginActionScope() const override { return PLUGIN_ACTION_SCOPE::PCB; }
+
     /**
      * Update the state of the GUI after a new board is loaded or created.
      */
@@ -790,7 +820,8 @@ protected:
      * @param full file path of file to be imported.
      * @param aFileType PCB_FILE_T value for file type
      */
-    bool importFile( const wxString& aFileName, int aFileType );
+    bool importFile( const wxString& aFileName, int aFileType,
+                     const std::map<std::string, UTF8>* aProperties = nullptr );
 
     bool canCloseWindow( wxCloseEvent& aCloseEvent ) override;
     void doCloseWindow() override;
@@ -800,25 +831,31 @@ protected:
 
     void onSize( wxSizeEvent& aEvent );
 
-    int inferLegacyEdgeClearance( BOARD* aBoard );
+    int inferLegacyEdgeClearance( BOARD* aBoard, bool aShowUserMsg = true );
 
-    void redrawNetnames( wxTimerEvent& aEvent );
+    void redrawNetnames();
 
     void saveProjectSettings() override;
 
+    void onCloseModelessBookReporterDialogs( wxCommandEvent& aEvent );
+
+#ifdef KICAD_IPC_API
+    void onPluginAvailabilityChanged( wxCommandEvent& aEvt );
+#endif
+
 public:
-    PCB_LAYER_BOX_SELECTOR* m_SelLayerBox;  // a combo box to display and select active layer
+    PCB_LAYER_BOX_SELECTOR* m_SelLayerBox; // a combo box to display and select active layer
 
     wxChoice* m_SelTrackWidthBox;        // a choice box to display and select current track width
     wxChoice* m_SelViaSizeBox;           // a choice box to display and select current via diameter
 
     bool m_show_layer_manager_tools;
-    bool m_show_properties;
     bool m_show_search;
+    bool m_show_net_inspector;
 
-    bool m_ZoneFillsDirty;               // Board has been modified since last zone fill.
+    bool m_ZoneFillsDirty;          // Board has been modified since last zone fill.
 
-    bool m_probingSchToPcb;              // Recursion guard when synchronizing selection from schematic
+    bool m_probingSchToPcb;         // Recursion guard when synchronizing selection from schematic
 
 private:
     friend struct PCB::IFACE;
@@ -834,14 +871,25 @@ private:
     TOOL_ACTION* m_exportNetlistAction;
 
     DIALOG_FIND* m_findDialog;
+    DIALOG_BOOK_REPORTER* m_inspectDrcErrorDlg;
+    DIALOG_BOOK_REPORTER* m_inspectClearanceDlg;
+    DIALOG_BOOK_REPORTER* m_inspectConstraintsDlg;
+    DIALOG_BOOK_REPORTER* m_footprintDiffDlg;
+    DIALOG_BOARD_SETUP*   m_boardSetupDlg;
+
+    const std::map<std::string, UTF8>* m_importProperties; // Properties used for non-KiCad import.
 
     /**
      * Keep track of viewport so that track net labels can be adjusted when it changes.
      */
-    BOX2D        m_lastViewport;
-    wxTimer      m_redrawNetnamesTimer;
+    BOX2D        m_lastNetnamesViewport;
 
     wxTimer*     m_eventCounterTimer;
+
+#ifdef KICAD_IPC_API
+    std::unique_ptr<API_HANDLER_PCB> m_apiHandler;
+    std::unique_ptr<API_HANDLER_COMMON> m_apiHandlerCommon;
+#endif
 };
 
 #endif  // __PCB_EDIT_FRAME_H__
