@@ -1,7 +1,7 @@
 /*
 * This program source code file is part of KiCad, a free EDA CAD application.
 *
-* Copyright (C) 2020-2022 KiCad Developers, see AUTHORS.txt for contributors.
+* Copyright The KiCad Developers, see AUTHORS.txt for contributors.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
@@ -26,6 +26,7 @@
 #include <common.h>
 #include <footprint_editor_settings.h>
 #include <layer_ids.h>
+#include <lset.h>
 #include <pcbnew_settings.h>
 #include <pgm_base.h>
 #include <router/pns_routing_settings.h>
@@ -53,6 +54,7 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
           m_DrcDialog(),
           m_ExportIdf(),
           m_ExportStep(),
+          m_ExportODBPP(),
           m_ExportSvg(),
           m_ExportVrml(),
           m_FootprintWizardList(),
@@ -70,7 +72,7 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
           m_ArcEditMode( ARC_EDIT_MODE::KEEP_CENTER_ADJUST_ANGLE_RADIUS ),
           m_CtrlClickHighlight( false ),
           m_Use45DegreeLimit( false ),
-          m_FlipLeftRight( false ),
+          m_FlipDirection( FLIP_DIRECTION::TOP_BOTTOM ),
           m_ESCClearsNetHighlight( true ),
           m_PolarCoords( false ),
           m_RotationAngle( ANGLE_90 ),
@@ -84,9 +86,12 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
           m_FootprintViewerLibListWidth( 200 ),
           m_FootprintViewerFPListWidth( 300 )
 {
-    m_MagneticItems.pads     = MAGNETIC_OPTIONS::CAPTURE_CURSOR_IN_TRACK_TOOL;
-    m_MagneticItems.tracks   = MAGNETIC_OPTIONS::CAPTURE_CURSOR_IN_TRACK_TOOL;
-    m_MagneticItems.graphics = false;
+    m_MagneticItems.pads      = MAGNETIC_OPTIONS::CAPTURE_CURSOR_IN_TRACK_TOOL;
+    m_MagneticItems.tracks    = MAGNETIC_OPTIONS::CAPTURE_CURSOR_IN_TRACK_TOOL;
+    m_MagneticItems.graphics  = false;
+    m_MagneticItems.allLayers = false;
+
+    m_LockingOptions.m_sessionSkipPrompts = false;
 
     m_params.emplace_back( new PARAM<bool>( "aui.show_layer_manager",
             &m_AuiPanels.show_layer_manager, true ) );
@@ -94,15 +99,24 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
     m_params.emplace_back( new PARAM<int>( "aui.right_panel_width",
             &m_AuiPanels.right_panel_width, -1 ) );
 
+    m_params.emplace_back( new PARAM<int>( "aui.net_inspector_width",
+            &m_AuiPanels.net_inspector_width, -1 ) );
+
     m_params.emplace_back( new PARAM<int>( "aui.properties_panel_width",
             &m_AuiPanels.properties_panel_width, -1 ) );
 
     m_params.emplace_back( new PARAM<float>( "aui.properties_splitter_proportion",
-            &m_AuiPanels.properties_splitter_proportion, 0.5f ) );
+            &m_AuiPanels.properties_splitter, 0.5f ) );
 
     m_params.emplace_back( new PARAM<int>( "aui.search_panel_height",
             &m_AuiPanels.search_panel_height, -1 ) );
 
+    m_params.emplace_back( new PARAM<int>( "aui.search_panel_width",
+            &m_AuiPanels.search_panel_width, -1 ) );
+
+    m_params.emplace_back( new PARAM<int>( "aui.search_panel_dock_direction",
+            &m_AuiPanels.search_panel_dock_direction, 3 ) );
+
     m_params.emplace_back( new PARAM<int>( "aui.appearance_panel_tab",
             &m_AuiPanels.appearance_panel_tab, 0, 0, 2 ) );
 
@@ -118,6 +132,9 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
     m_params.emplace_back( new PARAM<bool>( "aui.show_search",
             &m_AuiPanels.show_search, false ) );
 
+    m_params.emplace_back( new PARAM<bool>( "aui.show_net_inspector",
+            &m_AuiPanels.show_net_inspector, false ) );
+
     m_params.emplace_back( new PARAM<int>( "footprint_chooser.width",
             &m_FootprintChooser.width, -1 ) );
 
@@ -130,8 +147,18 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
     m_params.emplace_back( new PARAM<int>( "footprint_chooser.sash_v",
             &m_FootprintChooser.sash_v, -1 ) );
 
+    m_params.emplace_back( new PARAM<int>( "footprint_chooser.sort_mode",
+            &m_FootprintChooser.sort_mode, 0 ) );
+
+    m_params.emplace_back( new PARAM<bool>( "footprint_chooser.filter_on_pin_count",
+            &m_FootprintChooser.filter_on_pin_count, false ) );
+
+    m_params.emplace_back( new PARAM<bool>( "footprint_chooser.use_fp_filters",
+            &m_FootprintChooser.use_fp_filters, false ) );
+
     m_params.emplace_back( new PARAM<bool>( "editing.flip_left_right",
-            &m_FlipLeftRight, true ) );
+            reinterpret_cast<bool*>( &m_FlipDirection ),
+            static_cast<bool>( FLIP_DIRECTION::LEFT_RIGHT ) ) );
 
     m_params.emplace_back( new PARAM<bool>( "editing.esc_clears_net_highlight",
             &m_ESCClearsNetHighlight, true ) );
@@ -150,6 +177,9 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
             reinterpret_cast<int*>( &m_MagneticItems.tracks ),
             static_cast<int>( MAGNETIC_OPTIONS::CAPTURE_CURSOR_IN_TRACK_TOOL ) ) );
 
+    m_params.emplace_back( new PARAM<bool>( "editing.magnetic_all_layers",
+            &m_MagneticItems.allLayers, false ) );
+
     m_params.emplace_back( new PARAM<bool>( "editing.polar_coords",
             &m_PolarCoords, false ) );
 
@@ -228,6 +258,9 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
     m_params.emplace_back( new PARAM<bool>( "pcb_display.ratsnest_curved",
             &m_Display.m_DisplayRatsnestLinesCurved, false ) );
 
+    m_params.emplace_back( new PARAM<double>( "pcb_display.ratsnest_thickness",
+            &m_Display.m_RatsnestThickness, 0.5 ) );
+
     m_params.emplace_back( new PARAM<int>( "pcb_display.track_clearance_mode",
             reinterpret_cast<int*>( &m_Display.m_TrackClearance ), SHOW_WITH_VIA_WHILE_ROUTING ) );
 
@@ -247,12 +280,18 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
     m_params.emplace_back( new PARAM<bool>( "pcb_display.origin_invert_y_axis",
             &m_Display.m_DisplayInvertYAxis, false ) );
 
+    m_params.emplace_back( new PARAM<bool>( "pcb_display.force_show_fields_when_fp_selected",
+            &m_Display.m_ForceShowFieldsWhenFPSelected, true ) );
+
     m_params.emplace_back( new PARAM<bool>( "pcb_display.live_3d_refresh",
             &m_Display.m_Live3DRefresh, false ) );
 
     m_params.emplace_back( new PARAM<bool>( "pcb_display.show_page_borders",
             &m_ShowPageLimits, true ) );
 
+    m_params.emplace_back( new PARAM<bool>( "cleanup.cleanup_refill_zones",
+            &m_Cleanup.cleanup_refill_zones, true ) );
+
     m_params.emplace_back( new PARAM<bool>( "cleanup.cleanup_vias",
             &m_Cleanup.cleanup_vias, true ) );
 
@@ -293,7 +332,7 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
             &m_GenDrill.mirror, false ) );
 
     m_params.emplace_back( new PARAM<bool>( "gen_drill.unit_drill_is_inch",
-            &m_GenDrill.unit_drill_is_inch, true ) );
+            &m_GenDrill.unit_drill_is_inch, false ) );
 
     m_params.emplace_back( new PARAM<bool>( "gen_drill.use_route_for_oval_holes",
             &m_GenDrill.use_route_for_oval_holes, true ) );
@@ -307,6 +346,30 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
     m_params.emplace_back( new PARAM<int>( "gen_drill.zeros_format",
             &m_GenDrill.zeros_format, 0, 0, 3 ) );
 
+    m_params.emplace_back( new PARAM<bool>( "gen_drill.generate_map",
+            &m_GenDrill.generate_map, false ) );
+
+    m_params.emplace_back( new PARAM<int>( "export_2581.units",
+            &m_Export2581.units, 0 ) );
+
+    m_params.emplace_back( new PARAM<int>( "export_2581.precision",
+            &m_Export2581.precision, 6 ) );
+
+    m_params.emplace_back( new PARAM<int>( "export_2581.version",
+            &m_Export2581.version, 1 ) );
+
+    m_params.emplace_back( new PARAM<bool>( "export_2581.compress",
+            &m_Export2581.compress, false ) );
+
+    m_params.emplace_back( new PARAM<int>( "export_odb.units",
+            &m_ExportODBPP.units, 0 ) );
+
+    m_params.emplace_back( new PARAM<int>( "export_odb.precision",
+            &m_ExportODBPP.precision, 6 ) );
+
+    m_params.emplace_back( new PARAM<int>( "export_odb.compress_format",
+            &m_ExportODBPP.compressFormat, 1 ) );
+
     m_params.emplace_back( new PARAM<bool>( "export_idf.auto_adjust",
             &m_ExportIdf.auto_adjust, false ) );
 
@@ -334,8 +397,11 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
     m_params.emplace_back( new PARAM<double>( "export_step.origin_y",
             &m_ExportStep.origin_y, 0 ) );
 
-    m_params.emplace_back( new PARAM<bool>( "export_step.no_virtual",
-            &m_ExportStep.no_virtual, false ) );
+    m_params.emplace_back( new PARAM<bool>( "export_step.no_unspecified",
+            &m_ExportStep.no_unspecified, false ) );
+
+    m_params.emplace_back( new PARAM<bool>( "export_step.no_dnp",
+            &m_ExportStep.no_dnp, false ) );
 
     m_params.emplace_back( new PARAM<bool>( "export_step.replace_models",
             &m_ExportStep.replace_models, true ) );
@@ -346,6 +412,12 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
     m_params.emplace_back( new PARAM<bool>( "export_svg.black_and_white",
             &m_ExportSvg.black_and_white, false ) );
 
+    m_params.emplace_back( new PARAM<bool>( "export_svg.use_theme",
+            &m_ExportSvg.use_selected_theme, true ) );
+
+    m_params.emplace_back( new PARAM<wxString>( "export_svg.color_theme",
+            &m_ExportSvg.color_theme, "" ) );
+
     m_params.emplace_back( new PARAM<bool>( "export_svg.mirror",
             &m_ExportSvg.mirror, false ) );
 
@@ -367,6 +439,12 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
     m_params.emplace_back( new PARAM<int>( "export_vrml.units",
             &m_ExportVrml.units, 1 ) );
 
+    m_params.emplace_back( new PARAM<bool>( "export_vrml.no_unspecified",
+            &m_ExportVrml.no_unspecified, false ) );
+
+    m_params.emplace_back( new PARAM<bool>( "export_vrml.no_dnp",
+            &m_ExportVrml.no_dnp, false ) );
+
     m_params.emplace_back( new PARAM<bool>( "export_vrml.copy_3d_models",
             &m_ExportVrml.copy_3d_models, false ) );
 
@@ -391,9 +469,21 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
     m_params.emplace_back( new PARAM<int>( "import_graphics.layer",
             &m_ImportGraphics.layer, Dwgs_User ) );
 
+    m_params.emplace_back( new PARAM<bool>( "import_graphics.use_dlg_layer_selection",
+            &m_ImportGraphics.use_dlg_layer_selection, true ) );
+
     m_params.emplace_back( new PARAM<bool>( "import_graphics.interactive_placement",
             &m_ImportGraphics.interactive_placement, true ) );
 
+    m_params.emplace_back( new PARAM<bool>( "import_graphics.group_items",
+            &m_ImportGraphics.group_items, true ) );
+
+    m_params.emplace_back( new PARAM<bool>( "import_graphics.fix_discontinuities",
+            &m_ImportGraphics.fix_discontinuities, true ) );
+
+    m_params.emplace_back( new PARAM<double>( "import_graphics.tolerance",
+            &m_ImportGraphics.tolerance, 1.0, 0.0, 10.0 ) );
+
     m_params.emplace_back( new PARAM<int>( "import_graphics.line_width_units",
             &m_ImportGraphics.dxf_line_width_units, 0 ) );
 
@@ -427,8 +517,12 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
     m_params.emplace_back( new PARAM<bool>( "netlist.associate_by_ref_sch",
             &m_NetlistDialog.associate_by_ref_sch, false ) );
 
-    m_params.emplace_back( new PARAM<wxString>( "place_file.output_directory",
-            &m_PlaceFile.output_directory, wxEmptyString ) );
+    /*
+     * place_file.output_directory is only used at run-time; actual data is in project file
+     *
+     * m_params.emplace_back( new PARAM<wxString>( "place_file.output_directory",
+     *                        &m_PlaceFile.output_directory, wxEmptyString ) );
+     */
 
     m_params.emplace_back( new PARAM<int>( "place_file.units",
             &m_PlaceFile.units, 1 ) );
@@ -478,8 +572,11 @@ PCBNEW_SETTINGS::PCBNEW_SETTINGS()
     m_params.emplace_back( new PARAM<bool>( "plot.mirror",
             &m_Plot.mirror, false ) );
 
+    m_params.emplace_back( new PARAM<bool>( "plot.as_item_checkboxes",
+            &m_Plot.as_item_checkboxes, false ) );
+
     m_params.emplace_back( new PARAM<wxString>( "window.footprint_text_shown_columns",
-            &m_FootprintTextShownColumns, "0 1 2 3 4 5 6" ) );
+            &m_FootprintTextShownColumns, "0 1 2 3 4 5 7" ) );
 
     m_params.emplace_back( new PARAM<int>( "footprint_wizard_list.width",
             &m_FootprintWizardList.width, -1 ) );
@@ -853,7 +950,8 @@ bool PCBNEW_SETTINGS::MigrateFromLegacy( wxConfigBase* aCfg )
 
     // Migrate color settings that were stored in the pcbnew config file
 
-    COLOR_SETTINGS* cs = Pgm().GetSettingsManager().GetMigratedColorSettings();
+    SETTINGS_MANAGER& mgr = Pgm().GetSettingsManager();
+    COLOR_SETTINGS*   cs = mgr.GetMigratedColorSettings();
 
     auto migrateLegacyColor =
             [&] ( const std::string& aKey, int aLayerId )
@@ -874,22 +972,20 @@ bool PCBNEW_SETTINGS::MigrateFromLegacy( wxConfigBase* aCfg )
     migrateLegacyColor( "Color4DAuxItems",           LAYER_AUX_ITEMS );
     migrateLegacyColor( "Color4DGrid",               LAYER_GRID );
     migrateLegacyColor( "Color4DNonPlatedEx",        LAYER_NON_PLATEDHOLES );
-    migrateLegacyColor( "Color4DPadThruHoleEx",      LAYER_PADS_TH );
     migrateLegacyColor( "Color4DPCBBackground",      LAYER_PCB_BACKGROUND );
     migrateLegacyColor( "Color4DPCBCursor",          LAYER_CURSOR );
     migrateLegacyColor( "Color4DRatsEx",             LAYER_RATSNEST );
-    migrateLegacyColor( "Color4DTxtInvisEx",         LAYER_MOD_TEXT_INVISIBLE );
     migrateLegacyColor( "Color4DViaBBlindEx",        LAYER_VIA_BBLIND );
     migrateLegacyColor( "Color4DViaMicroEx",         LAYER_VIA_MICROVIA );
     migrateLegacyColor( "Color4DViaThruEx",          LAYER_VIA_THROUGH );
     migrateLegacyColor( "Color4DWorksheet",          LAYER_DRAWINGSHEET );
     migrateLegacyColor( "Color4DGrid",               LAYER_PAGE_LIMITS );
 
-    Pgm().GetSettingsManager().SaveColorSettings( cs, "board" );
+    mgr.SaveColorSettings( cs, "board" );
 
     Set( "appearance.color_theme", cs->GetFilename() );
 
-    double x, y;
+    double x = 0, y = 0;
 
     if( aCfg->Read( f + "PcbUserGrid_X", &x ) && aCfg->Read( f + "PcbUserGrid_Y", &y ) )
     {
@@ -905,12 +1001,12 @@ bool PCBNEW_SETTINGS::MigrateFromLegacy( wxConfigBase* aCfg )
     }
 
     // Footprint editor settings were stored in pcbnew config file.  Migrate them here.
-    auto fpedit = Pgm().GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>( false );
+    FOOTPRINT_EDITOR_SETTINGS* fpedit = mgr.GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>( "fpedit" );
     fpedit->MigrateFromLegacy( aCfg );
     fpedit->Load();
 
     // Same with 3D viewer
-    auto viewer3d = Pgm().GetSettingsManager().GetAppSettings<EDA_3D_VIEWER_SETTINGS>( false );
+    EDA_3D_VIEWER_SETTINGS* viewer3d = mgr.GetAppSettings<EDA_3D_VIEWER_SETTINGS>( "3d_viewer" );
     viewer3d->MigrateFromLegacy( aCfg );
     viewer3d->Load();
 
@@ -923,7 +1019,7 @@ bool PCBNEW_SETTINGS::MigrateFromLegacy( wxConfigBase* aCfg )
 //{
 //    py::class_<PCBNEW_SETTINGS>( m, "settings" )
 //            .def_readwrite( "Use45DegreeGraphicSegments", &PCBNEW_SETTINGS::m_Use45DegreeGraphicSegments )
-//            .def_readwrite( "FlipLeftRight", &PCBNEW_SETTINGS::m_FlipLeftRight )
+//            .def_readwrite( "FlipLeftRight", &PCBNEW_SETTINGS::m_FlipDirection )
 //            .def_readwrite( "AddUnlockedPads", &PCBNEW_SETTINGS::m_AddUnlockedPads)
 //            .def_readwrite( "UsePolarCoords", &PCBNEW_SETTINGS::m_PolarCoords)
 //            .def_readwrite( "RotationAngle", &PCBNEW_SETTINGS::m_RotationAngle)
