@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2012 NBEE Embedded Systems, Miguel Angel Ajo <miguelangel@nbee.es>
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,15 +37,19 @@
 #include <board_design_settings.h>
 #include <pcb_marker.h>
 #include <cstdlib>
+#include <drawing_sheet/ds_data_model.h>
 #include <drc/drc_engine.h>
 #include <drc/drc_item.h>
 #include <fp_lib_table.h>
-#include <ignore.h>
-#include <io_mgr.h>
+#include <core/ignore.h>
+#include <pcb_io/pcb_io_mgr.h>
 #include <string_utils.h>
+#include <filename_resolver.h>
 #include <macros.h>
 #include <pcbnew_scripting_helpers.h>
+#include <pgm_base.h>
 #include <project.h>
+#include <project_pcb.h>
 #include <project/net_settings.h>
 #include <project/project_file.h>
 #include <settings/settings_manager.h>
@@ -54,7 +58,8 @@
 #include <wildcards_and_files_ext.h>
 #include <locale_io.h>
 #include <wx/app.h>
-
+#include <wx/crt.h>
+#include <wx/image.h>
 
 static PCB_EDIT_FRAME* s_PcbEditFrame = nullptr;
 static SETTINGS_MANAGER* s_SettingsManager = nullptr;
@@ -75,15 +80,28 @@ void ScriptingSetPcbEditFrame( PCB_EDIT_FRAME* aPcbEditFrame )
 }
 
 
-BOARD* LoadBoard( wxString& aFileName )
+void ScriptingOnDestructPcbEditFrame( PCB_EDIT_FRAME* aPcbEditFrame )
 {
-    if( aFileName.EndsWith( KiCadPcbFileExtension ) )
-        return LoadBoard( aFileName, IO_MGR::KICAD_SEXP );
-    else if( aFileName.EndsWith( LegacyPcbFileExtension ) )
-        return LoadBoard( aFileName, IO_MGR::LEGACY );
+    if( s_PcbEditFrame == aPcbEditFrame )
+        s_PcbEditFrame = nullptr;
+}
+
+
+BOARD* LoadBoard( const wxString& aFileName, bool aSetActive )
+{
+    if( aFileName.EndsWith( FILEEXT::KiCadPcbFileExtension ) )
+        return LoadBoard( aFileName, PCB_IO_MGR::KICAD_SEXP, aSetActive );
+    else if( aFileName.EndsWith( FILEEXT::LegacyPcbFileExtension ) )
+        return LoadBoard( aFileName, PCB_IO_MGR::LEGACY, aSetActive );
 
     // as fall back for any other kind use the legacy format
-    return LoadBoard( aFileName, IO_MGR::LEGACY );
+    return LoadBoard( aFileName, PCB_IO_MGR::LEGACY, aSetActive );
+}
+
+
+BOARD* LoadBoard( const wxString& aFileName )
+{
+    return LoadBoard( aFileName, false );
 }
 
 
@@ -124,14 +142,25 @@ PROJECT* GetDefaultProject()
     return project;
 }
 
+BOARD* LoadBoard( const wxString& aFileName, PCB_IO_MGR::PCB_FILE_T aFormat )
+{
+    return LoadBoard( aFileName, aFormat, false );
+}
 
-BOARD* LoadBoard( wxString& aFileName, IO_MGR::PCB_FILE_T aFormat )
+
+BOARD* LoadBoard( const wxString& aFileName, PCB_IO_MGR::PCB_FILE_T aFormat, bool aSetActive )
 {
     wxFileName pro = aFileName;
-    pro.SetExt( ProjectFileExtension );
+    pro.SetExt( FILEEXT::ProjectFileExtension );
     pro.MakeAbsolute();
     wxString projectPath = pro.GetFullPath();
 
+    // Ensure image handlers are loaded, because a board can include bitmap images
+    // using various formats.
+    // By default only the BMP handler is available.
+    wxInitAllImageHandlers();
+
+
     // Ensure the "C" locale is temporary set, before reading any file
     // It also avoid wxWidget alerts about locale issues, later, when using Python 3
     LOCALE_IO dummy;
@@ -142,7 +171,8 @@ BOARD* LoadBoard( wxString& aFileName, IO_MGR::PCB_FILE_T aFormat )
     {
         if( wxFileExists( projectPath ) )
         {
-            GetSettingsManager()->LoadProject( projectPath, false );
+            // cli
+            GetSettingsManager()->LoadProject( projectPath, aSetActive );
             project = GetSettingsManager()->GetProject( projectPath );
         }
     }
@@ -156,10 +186,56 @@ BOARD* LoadBoard( wxString& aFileName, IO_MGR::PCB_FILE_T aFormat )
     if( !project )
         project = GetDefaultProject();
 
-    BOARD* brd = IO_MGR::Load( aFormat, aFileName );
+    BASE_SCREEN::m_DrawingSheetFileName = project->GetProjectFile().m_BoardDrawingSheetFile;
+
+    BOARD* brd = nullptr;
+    try
+    {
+        brd = PCB_IO_MGR::Load( aFormat, aFileName );
+    }
+    catch( ... )
+    {
+        brd = nullptr;
+    }
 
     if( brd )
     {
+        // Load the drawing sheet from the filename stored in BASE_SCREEN::m_DrawingSheetFileName.
+        // If empty, or not existing, the default drawing sheet is loaded.
+        FILENAME_RESOLVER resolver;
+        resolver.SetProject( project );
+
+        // a PGM_BASE* process can be nullptr when running from a python script
+        // So use PgmOrNull() instead of &Pgm() to initialize the resolver
+        resolver.SetProgramBase( PgmOrNull() );
+
+        wxString filename = resolver.ResolvePath( BASE_SCREEN::m_DrawingSheetFileName,
+                                              project->GetProjectPath(),
+                                              brd->GetEmbeddedFiles() );
+
+        wxString msg;
+
+        if( !DS_DATA_MODEL::GetTheInstance().LoadDrawingSheet( filename, &msg ) )
+        {
+            wxFprintf( stderr, _( "Error loading drawing sheet '%s': %s" ),
+                       BASE_SCREEN::m_DrawingSheetFileName, msg );
+        }
+
+        // JEY TODO: move this global to the board
+        ENUM_MAP<PCB_LAYER_ID>& layerEnum = ENUM_MAP<PCB_LAYER_ID>::Instance();
+
+        layerEnum.Choices().Clear();
+        layerEnum.Undefined( UNDEFINED_LAYER );
+
+        for( PCB_LAYER_ID layer : LSET::AllLayersMask().Seq() )
+        {
+            // Canonical name
+            layerEnum.Map( layer, LSET::Name( layer ) );
+
+            // User name
+            layerEnum.Map( layer, brd->GetLayerName( layer ) );
+        }
+
         brd->SetProject( project );
 
         // Move legacy view settings to local project settings
@@ -175,7 +251,7 @@ BOARD* LoadBoard( wxString& aFileName, IO_MGR::PCB_FILE_T aFormat )
         try
         {
             wxFileName rules = pro;
-            rules.SetExt( DesignRulesFileExtension );
+            rules.SetExt( FILEEXT::DesignRulesFileExtension );
             bds.m_DRCEngine->InitEngine( rules );
         }
         catch( ... )
@@ -183,12 +259,12 @@ BOARD* LoadBoard( wxString& aFileName, IO_MGR::PCB_FILE_T aFormat )
             // Best efforts...
         }
 
-        for( PCB_MARKER* marker : brd->ResolveDRCExclusions() )
+        for( PCB_MARKER* marker : brd->ResolveDRCExclusions( true ) )
             brd->Add( marker );
 
         brd->BuildConnectivity();
         brd->BuildListOfNets();
-        brd->SynchronizeNetsAndNetClasses( false );
+        brd->SynchronizeNetsAndNetClasses( true );
         brd->UpdateUserUnits( brd, nullptr );
     }
 
@@ -200,7 +276,7 @@ BOARD* NewBoard( wxString& aFileName )
 {
     wxFileName boardFn = aFileName;
     wxFileName proFn   = aFileName;
-    proFn.SetExt( ProjectFileExtension );
+    proFn.SetExt( FILEEXT::ProjectFileExtension );
     proFn.MakeAbsolute();
 
     wxString projectPath = proFn.GetFullPath();
@@ -238,7 +314,7 @@ BOARD* CreateEmptyBoard()
 }
 
 
-bool SaveBoard( wxString& aFileName, BOARD* aBoard, IO_MGR::PCB_FILE_T aFormat, bool aSkipSettings )
+bool SaveBoard( wxString& aFileName, BOARD* aBoard, PCB_IO_MGR::PCB_FILE_T aFormat, bool aSkipSettings )
 {
     aBoard->BuildConnectivity();
     aBoard->SynchronizeNetsAndNetClasses( false );
@@ -249,7 +325,7 @@ bool SaveBoard( wxString& aFileName, BOARD* aBoard, IO_MGR::PCB_FILE_T aFormat,
 
     try
     {
-        IO_MGR::Save( aFormat, aFileName, aBoard, nullptr );
+        PCB_IO_MGR::Save( aFormat, aFileName, aBoard, nullptr );
     }
     catch( ... )
     {
@@ -259,7 +335,7 @@ bool SaveBoard( wxString& aFileName, BOARD* aBoard, IO_MGR::PCB_FILE_T aFormat,
     if( !aSkipSettings )
     {
         wxFileName pro = aFileName;
-        pro.SetExt( ProjectFileExtension );
+        pro.SetExt( FILEEXT::ProjectFileExtension );
         pro.MakeAbsolute();
 
         GetSettingsManager()->SaveProjectAs( pro.GetFullPath(), aBoard->GetProject() );
@@ -271,7 +347,7 @@ bool SaveBoard( wxString& aFileName, BOARD* aBoard, IO_MGR::PCB_FILE_T aFormat,
 
 bool SaveBoard( wxString& aFileName, BOARD* aBoard, bool aSkipSettings )
 {
-    return SaveBoard( aFileName, aBoard, IO_MGR::KICAD_SEXP, aSkipSettings );
+    return SaveBoard( aFileName, aBoard, PCB_IO_MGR::KICAD_SEXP, aSkipSettings );
 }
 
 
@@ -287,7 +363,7 @@ FP_LIB_TABLE* GetFootprintLibraryTable()
     if( !project )
         return nullptr;
 
-    return project->PcbFootprintLibs();
+    return PROJECT_PCB::PcbFootprintLibs( project );
 }
 
 
@@ -340,7 +416,7 @@ bool ExportSpecctraDSN( BOARD* aBoard, wxString& aFullFilename )
 {
     try
     {
-        ExportBoardToSpecctraFile( aBoard, aFullFilename );
+        DSN::ExportBoardToSpecctraFile( aBoard, aFullFilename );
     }
     catch( ... )
     {
@@ -351,12 +427,14 @@ bool ExportSpecctraDSN( BOARD* aBoard, wxString& aFullFilename )
 }
 
 
-bool ExportVRML( const wxString& aFullFileName, double aMMtoWRMLunit, bool aExport3DFiles,
+bool ExportVRML( const wxString& aFullFileName, double aMMtoWRMLunit, bool aIncludeUnspecified,
+                 bool aIncludeDNP, bool aExport3DFiles,
                  bool aUseRelativePaths, const wxString& a3D_Subdir, double aXRef, double aYRef )
 {
     if( s_PcbEditFrame )
     {
         bool ok = s_PcbEditFrame->ExportVRML_File( aFullFileName, aMMtoWRMLunit,
+                                                   aIncludeUnspecified, aIncludeDNP,
                                                    aExport3DFiles, aUseRelativePaths,
                                                    a3D_Subdir, aXRef, aYRef );
         return ok;
@@ -380,6 +458,20 @@ bool ImportSpecctraSES( wxString& aFullFilename )
     }
 }
 
+bool ImportSpecctraSES( BOARD* aBoard, wxString& aFullFilename )
+{
+    try
+    {
+        DSN::ImportSpecctraSession( aBoard, aFullFilename );
+    }
+    catch( ... )
+    {
+        return false;
+    }
+
+    return true;
+}
+
 
 bool ExportFootprintsToLibrary( bool aStoreInNewLib, const wxString& aLibName, wxString* aLibPath )
 {
@@ -437,6 +529,14 @@ std::deque<BOARD_ITEM*> GetCurrentSelection()
     return items;
 }
 
+void FocusOnItem( BOARD_ITEM* aItem, PCB_LAYER_ID aLayer )
+{
+    if( s_PcbEditFrame )
+    {
+        s_PcbEditFrame->FocusOnItem( aItem, aLayer );
+    }
+}
+
 
 bool IsActionRunning()
 {
@@ -462,7 +562,7 @@ bool WriteDRCReport( BOARD* aBoard, const wxString& aFileName, EDA_UNITS aUnits,
     wxCHECK( engine, false );
 
     wxFileName fn = aBoard->GetFileName();
-    fn.SetExt( DesignRulesFileExtension );
+    fn.SetExt( FILEEXT::DesignRulesFileExtension );
     PROJECT* prj = nullptr;
 
     if( aBoard->GetProject() )
@@ -474,6 +574,7 @@ bool WriteDRCReport( BOARD* aBoard, const wxString& aFileName, EDA_UNITS aUnits,
 
     // Load the global fp-lib-table otherwise we can't check the libs parity
     wxFileName  fn_flp = FP_LIB_TABLE::GetGlobalTableFileName();
+
     if( fn_flp.FileExists() )
         GFootprintTable.Load( fn_flp.GetFullPath() );
 
@@ -485,10 +586,10 @@ bool WriteDRCReport( BOARD* aBoard, const wxString& aFileName, EDA_UNITS aUnits,
     layerEnum.Choices().Clear();
     layerEnum.Undefined( UNDEFINED_LAYER );
 
-    for( LSEQ seq = LSET::AllLayersMask().Seq(); seq; ++seq )
+    for( PCB_LAYER_ID layer : LSET::AllLayersMask().Seq() )
     {
-        layerEnum.Map( *seq, LSET::Name( *seq ) );              // Add Canonical name
-        layerEnum.Map( *seq, aBoard->GetLayerName( *seq ) );    // Add User name
+        layerEnum.Map( layer, LSET::Name( layer ) );              // Add Canonical name
+        layerEnum.Map( layer, aBoard->GetLayerName( layer ) );    // Add User name
     }
 
     try
@@ -508,12 +609,15 @@ bool WriteDRCReport( BOARD* aBoard, const wxString& aFileName, EDA_UNITS aUnits,
     engine->SetProgressReporter( nullptr );
 
     engine->SetViolationHandler(
-            [&]( const std::shared_ptr<DRC_ITEM>& aItem, VECTOR2D aPos, int aLayer )
+            [&]( const std::shared_ptr<DRC_ITEM>& aItem, VECTOR2D aPos, int aLayer,
+                 DRC_CUSTOM_MARKER_HANDLER* aCustomHandler )
             {
                 if( aItem->GetErrorCode() == DRCE_MISSING_FOOTPRINT
                     || aItem->GetErrorCode() == DRCE_DUPLICATE_FOOTPRINT
                     || aItem->GetErrorCode() == DRCE_EXTRA_FOOTPRINT
-                    || aItem->GetErrorCode() == DRCE_NET_CONFLICT )
+                    || aItem->GetErrorCode() == DRCE_NET_CONFLICT
+                    || aItem->GetErrorCode() == DRCE_SCHEMATIC_PARITY
+                    || aItem->GetErrorCode() == DRCE_FOOTPRINT_FILTERS )
                 {
                     footprints.push_back( aItem );
                 }
@@ -527,9 +631,14 @@ bool WriteDRCReport( BOARD* aBoard, const wxString& aFileName, EDA_UNITS aUnits,
                 }
             } );
 
+    aBoard->RecordDRCExclusions();
+    aBoard->DeleteMARKERs( true, true );
     engine->RunTests( aUnits, aReportAllTrackErrors, false );
     engine->ClearViolationHandler();
 
+    // Update the exclusion status on any excluded markers that still exist.
+    aBoard->ResolveDRCExclusions( false );
+
     // TODO: Unify this with DIALOG_DRC::writeReport
 
     FILE* fp = wxFopen( aFileName, wxT( "w" ) );
@@ -576,3 +685,11 @@ bool WriteDRCReport( BOARD* aBoard, const wxString& aFileName, EDA_UNITS aUnits,
 
     return true;
 }
+
+wxString GetLanguage()
+{
+    if( s_PcbEditFrame )
+        return GetSettingsManager()->GetCommonSettings()->m_System.language;
+    else
+        return "";
+}
