@@ -2,7 +2,7 @@
  * KiRouter - a push-and-(sometimes-)shove PCB router
  *
  * Copyright (C) 2013-2015 CERN
- * Copyright (C) 2016-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * Author: Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
  * This program is free software: you can redistribute it and/or modify it
@@ -26,6 +26,8 @@
 #include "pns_solid.h"
 #include "pns_topology.h"
 #include "pns_debug_decorator.h"
+#include "pns_arc.h"
+#include "pns_utils.h"
 
 namespace PNS {
 
@@ -37,8 +39,8 @@ DIFF_PAIR_PLACER::DIFF_PAIR_PLACER( ROUTER* aRouter ) :
     m_initialDiagonal = false;
     m_startDiagonal = false;
     m_fitOk = false;
-    m_netP = 0;
-    m_netN = 0;
+    m_netP = nullptr;
+    m_netN = nullptr;
     m_iteration = 0;
     m_world = nullptr;
     m_shove = nullptr;
@@ -48,7 +50,6 @@ DIFF_PAIR_PLACER::DIFF_PAIR_PLACER( ROUTER* aRouter ) :
     m_viaDiameter = 0;
     m_viaDrill = 0;
     m_currentWidth = 0;
-    m_currentNet = 0;
     m_currentLayer = 0;
     m_startsOnVia = false;
     m_orthoMode = false;
@@ -68,12 +69,11 @@ void DIFF_PAIR_PLACER::setWorld( NODE* aWorld )
 }
 
 
-const VIA DIFF_PAIR_PLACER::makeVia( const VECTOR2I& aP, int aNet )
+const VIA DIFF_PAIR_PLACER::makeVia( const VECTOR2I& aP, NET_HANDLE aNet )
 {
-    const LAYER_RANGE layers( m_sizes.GetLayerTop(), m_sizes.GetLayerBottom() );
+    const PNS_LAYER_RANGE layers( m_sizes.GetLayerTop(), m_sizes.GetLayerBottom() );
 
-    VIA v( aP, layers, m_sizes.ViaDiameter(), m_sizes.ViaDrill(), -1, m_sizes.ViaType() );
-    v.SetNet( aNet );
+    VIA v( aP, layers, m_sizes.ViaDiameter(), m_sizes.ViaDrill(), aNet, m_sizes.ViaType() );
 
     return v;
 }
@@ -115,16 +115,16 @@ bool DIFF_PAIR_PLACER::rhMarkObstacles( const VECTOR2I& aP )
 
 bool DIFF_PAIR_PLACER::propagateDpHeadForces ( const VECTOR2I& aP, VECTOR2I& aNewP )
 {
-    VIA virtHead = makeVia( aP, -1 );
+    VIA virtHead = makeVia( aP, nullptr );
 
     if( m_placingVia )
     {
-        virtHead.SetDiameter( viaGap() + 2 * virtHead.Diameter() );
+        virtHead.SetDiameter( 0, viaGap() + 2 * virtHead.Diameter( 0 ) );
     }
     else
     {
         virtHead.SetLayer( m_currentLayer );
-        virtHead.SetDiameter( m_sizes.DiffPairGap() + 2 * m_sizes.DiffPairWidth() );
+        virtHead.SetDiameter( 0, m_sizes.DiffPairGap() + 2 * m_sizes.DiffPairWidth() );
     }
 
     bool solidsOnly = true;
@@ -145,11 +145,11 @@ bool DIFF_PAIR_PLACER::propagateDpHeadForces ( const VECTOR2I& aP, VECTOR2I& aNe
     // check proper clearances to the diff pair line.  It can be removed if some specialized
     // pushout for traces / diff pairs is implemented.  Just calling VIA::PushoutForce does not work
     // as the via may have different resolved clearance to items than the diff pair should.
-    int      maxIter  = 40;
-    int      iter     = 0;
-    bool     collided = false;
-    VECTOR2I force, totalForce;
-    std::set<ITEM*> handled;
+    int                   maxIter  = 40;
+    int                   iter     = 0;
+    bool                  collided = false;
+    VECTOR2I              force, totalForce;
+    std::set<const ITEM*> handled;
 
     while( iter < maxIter )
     {
@@ -159,11 +159,21 @@ bool DIFF_PAIR_PLACER::propagateDpHeadForces ( const VECTOR2I& aP, VECTOR2I& aNe
         if( !obs || handled.count( obs->m_item ) )
             break;
 
-        int clearance = m_currentNode->GetClearance( obs->m_item, &m_currentTrace.PLine() );
+        int clearance = m_currentNode->GetClearance( obs->m_item, &m_currentTrace.PLine(), false );
+        VECTOR2I layerForce;
+        collided = false;
 
-        if( obs->m_item->Shape()->Collide( virtHead.Shape(), clearance, &force ) )
+        for( int viaLayer : virtHead.RelevantShapeLayers( obs->m_item ) )
+        {
+            collided |= obs->m_item->Shape( viaLayer )->Collide( virtHead.Shape( viaLayer ),
+                                                                 clearance, &layerForce );
+
+            if( layerForce.SquaredEuclideanNorm() > force.SquaredEuclideanNorm() )
+                force = layerForce;
+        }
+
+        if( collided )
         {
-            collided = true;
             totalForce += force;
             virtHead.SetPos( virtHead.Pos() + force );
         }
@@ -189,10 +199,11 @@ bool DIFF_PAIR_PLACER::attemptWalk( NODE* aNode, DIFF_PAIR* aCurrent, DIFF_PAIR&
                                     bool aPFirst, bool aWindCw, bool aSolidsOnly )
 {
     WALKAROUND walkaround( aNode, Router() );
-    WALKAROUND::WALKAROUND_STATUS wf1;
+    WALKAROUND::STATUS wf1;
 
     walkaround.SetSolidsOnly( aSolidsOnly );
     walkaround.SetIterationLimit( Settings().WalkaroundIterationLimit() );
+    walkaround.SetAllowedPolicies( { WALKAROUND::WP_SHORTEST } );
 
     SHOVE shove( aNode, Router() );
     LINE walkP, walkN;
@@ -223,20 +234,22 @@ bool DIFF_PAIR_PLACER::attemptWalk( NODE* aNode, DIFF_PAIR* aCurrent, DIFF_PAIR&
                 continue;
         }
 
-        wf1 = walkaround.Route( preWalk, postWalk, false );
+        auto wf1 = walkaround.Route( preWalk );
 
-        if( wf1 != WALKAROUND::DONE )
+        if( wf1.status[ WALKAROUND::WP_SHORTEST ] != WALKAROUND::ST_DONE )
             return false;
 
+        postWalk = wf1.lines[ WALKAROUND::WP_SHORTEST ];
+
         LINE postShove( preShove );
 
         shove.ForceClearance( true, cur.Gap() - 2 * PNS_HULL_MARGIN );
 
-        SHOVE::SHOVE_STATUS sh1;
+        bool sh1;
 
         sh1 = shove.ShoveObstacleLine( postWalk, preShove, postShove );
 
-        if( sh1 != SHOVE::SH_OK )
+        if( !sh1 )
             return false;
 
         postWalk.Line().Simplify();
@@ -354,10 +367,11 @@ bool DIFF_PAIR_PLACER::rhShoveOnly( const VECTOR2I& aP )
     LINE nLine( m_currentTrace.NLine() );
     ITEM_SET head;
 
-    head.Add( &pLine );
-    head.Add( &nLine );
+    m_shove->ClearHeads();
+    m_shove->AddHeads( pLine );
+    m_shove->AddHeads( nLine );
 
-    SHOVE::SHOVE_STATUS status = m_shove->ShoveMultiLines( head );
+    SHOVE::SHOVE_STATUS status = m_shove->Run();
 
     m_currentNode = m_shove->CurrentNode();
 
@@ -365,12 +379,24 @@ bool DIFF_PAIR_PLACER::rhShoveOnly( const VECTOR2I& aP )
     {
         m_currentNode = m_shove->CurrentNode();
 
+        if( m_shove->HeadsModified( 0 ))
+            pLine = m_shove->GetModifiedHead(0);
+
+        if( m_shove->HeadsModified( 1 ))
+            nLine = m_shove->GetModifiedHead(1);
+
         if( !m_currentNode->CheckColliding( &m_currentTrace.PLine() ) &&
             !m_currentNode->CheckColliding( &m_currentTrace.NLine() ) )
         {
             m_fitOk = true;
         }
     }
+    else
+    {
+        // bring back previous state
+        m_currentTrace.SetShape( pLine.CLine(), nLine.CLine() );
+    }
+    
 
     return m_fitOk;
 }
@@ -434,17 +460,39 @@ OPT_VECTOR2I getDanglingAnchor( NODE* aNode, ITEM* aItem )
 {
     switch( aItem->Kind() )
     {
+    case ITEM::LINE_T:
+    {
+        LINE* l = static_cast<LINE*>( aItem );
+
+        if( !l->PointCount() )
+            return OPT_VECTOR2I();
+        else
+            return l->CPoint( 0 );
+    }
     case ITEM::VIA_T:
     case ITEM::SOLID_T:
         return aItem->Anchor( 0 );
 
     case ITEM::ARC_T:
+    {
+        ARC* a = static_cast<ARC*>( aItem );
+
+        const JOINT* jA = aNode->FindJoint( aItem->Anchor( 0 ), aItem );
+        const JOINT* jB = aNode->FindJoint( aItem->Anchor( 1 ), aItem );
+
+        if( jA && jA->LinkCount() == 1 )
+            return a->Arc().GetP0();
+        else if( jB && jB->LinkCount() == 1 )
+            return a->Arc().GetP1();
+        else
+            return OPT_VECTOR2I();
+    }
     case ITEM::SEGMENT_T:
     {
         SEGMENT* s = static_cast<SEGMENT*>( aItem );
 
-        JOINT* jA = aNode->FindJoint( aItem->Anchor( 0 ), aItem );
-        JOINT* jB = aNode->FindJoint( aItem->Anchor( 1 ), aItem );
+        const JOINT* jA = aNode->FindJoint( aItem->Anchor( 0 ), aItem );
+        const JOINT* jB = aNode->FindJoint( aItem->Anchor( 1 ), aItem );
 
         if( jA && jA->LinkCount() == 1 )
             return s->Seg().A;
@@ -464,7 +512,7 @@ OPT_VECTOR2I getDanglingAnchor( NODE* aNode, ITEM* aItem )
 bool DIFF_PAIR_PLACER::FindDpPrimitivePair( NODE* aWorld, const VECTOR2I& aP, ITEM* aItem,
                                             DP_PRIMITIVE_PAIR& aPair, wxString* aErrorMsg )
 {
-    int netP, netN;
+    NET_HANDLE netP, netN;
 
     bool result = aWorld->GetRuleResolver()->DpNetPair( aItem, netP, netN );
 
@@ -479,8 +527,8 @@ bool DIFF_PAIR_PLACER::FindDpPrimitivePair( NODE* aWorld, const VECTOR2I& aP, IT
         return false;
     }
 
-    int refNet = aItem->Net();
-    int coupledNet = ( refNet == netP ) ? netN : netP;
+    NET_HANDLE refNet = aItem->Net();
+    NET_HANDLE coupledNet = ( refNet == netP ) ? netN : netP;
 
     OPT_VECTOR2I refAnchor = getDanglingAnchor( aWorld, aItem );
     ITEM* primRef = aItem;
@@ -835,7 +883,7 @@ bool DIFF_PAIR_PLACER::CommitPlacement()
 }
 
 
-void DIFF_PAIR_PLACER::GetModifiedNets( std::vector<int> &aNets ) const
+void DIFF_PAIR_PLACER::GetModifiedNets( std::vector<NET_HANDLE> &aNets ) const
 {
     aNets.push_back( m_netP );
     aNets.push_back( m_netN );
@@ -855,9 +903,9 @@ void DIFF_PAIR_PLACER::updateLeadingRatLine()
 }
 
 
-const std::vector<int> DIFF_PAIR_PLACER::CurrentNets() const
+const std::vector<NET_HANDLE> DIFF_PAIR_PLACER::CurrentNets() const
 {
-    std::vector<int> rv;
+    std::vector<NET_HANDLE> rv;
     rv.push_back( m_netP );
     rv.push_back( m_netN );
     return rv;
