@@ -2,7 +2,7 @@
  * KiRouter - a push-and-(sometimes-)shove PCB router
  *
  * Copyright (C) 2013-2017 CERN
- * Copyright (C) 2016-2020 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * Author: Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
  * This program is free software: you can redistribute it and/or modify it
@@ -28,6 +28,8 @@
 #include "pns_node.h"
 #include "pns_via.h"
 #include "pns_utils.h"
+#include "pns_router.h"
+#include "pns_debug_decorator.h"
 
 #include <geometry/shape_rect.h>
 
@@ -42,8 +44,23 @@ LINE::LINE( const LINE& aOther )
     m_net = aOther.m_net;
     m_movable = aOther.m_movable;
     m_layers = aOther.m_layers;
-    m_via = aOther.m_via;
-    m_hasVia = aOther.m_hasVia;
+
+    m_via = nullptr;
+
+    if( aOther.m_via )
+    {
+        if( aOther.m_via->BelongsTo( &aOther ) )
+        {
+            m_via = aOther.m_via->Clone();
+            m_via->SetOwner( this );
+            m_via->SetNet( m_net );
+        }
+        else
+        {
+            m_via = aOther.m_via;
+        }
+    }
+
     m_marker = aOther.m_marker;
     m_rank = aOther.m_rank;
     m_blockingObstacle = aOther.m_blockingObstacle;
@@ -54,6 +71,8 @@ LINE::LINE( const LINE& aOther )
 
 LINE::~LINE()
 {
+    if( m_via && m_via->BelongsTo( this ) )
+        delete m_via;
 }
 
 
@@ -64,8 +83,23 @@ LINE& LINE::operator=( const LINE& aOther )
     m_net = aOther.m_net;
     m_movable = aOther.m_movable;
     m_layers = aOther.m_layers;
-    m_via = aOther.m_via;
-    m_hasVia = aOther.m_hasVia;
+
+    m_via = nullptr;
+
+    if( aOther.m_via )
+    {
+        if( aOther.m_via->BelongsTo( &aOther ) )
+        {
+            m_via = aOther.m_via->Clone();
+            m_via->SetOwner( this );
+            m_via->SetNet( m_net );
+        }
+        else
+        {
+            m_via = aOther.m_via;
+        }
+    }
+
     m_marker = aOther.m_marker;
     m_rank = aOther.m_rank;
     m_owner = aOther.m_owner;
@@ -109,10 +143,8 @@ int LINE::Marker() const
 {
     int marker = m_marker;
 
-    for( auto s : m_links )
-    {
+    for( LINKED_ITEM* s : m_links )
         marker |= s->Marker();
-    }
 
     return marker;
 }
@@ -120,7 +152,7 @@ int LINE::Marker() const
 
 SEGMENT* SEGMENT::Clone() const
 {
-    SEGMENT* s = new SEGMENT;
+    SEGMENT* s = new SEGMENT( *this );
 
     s->m_seg = m_seg;
     s->m_net = m_net;
@@ -177,7 +209,7 @@ bool LINE::Walkaround( const SHAPE_LINE_CHAIN& aObstacle, SHAPE_LINE_CHAIN& aPat
         return false;
     }
 
-    const auto pFirst = line.CPoint(0);
+    const VECTOR2I pFirst = line.CPoint(0);
 
     bool inFirst = aObstacle.PointInside( pFirst ) && !aObstacle.PointOnEdge( pFirst );
 
@@ -218,45 +250,39 @@ bool LINE::Walkaround( const SHAPE_LINE_CHAIN& aObstacle, SHAPE_LINE_CHAIN& aPat
 
     std::vector<VERTEX> vts;
 
-    auto findVertex = [&]( VECTOR2I pos) -> VERTEX*
-    {
-        for( VERTEX& v : vts )
-        {
-            if(v.pos == pos )
-                return &v;
-        }
-
-        return nullptr;
-    };
+    auto findVertex =
+            [&]( const VECTOR2I& pos ) -> VERTEX*
+            {
+                for( VERTEX& v : vts )
+                {
+                    if( v.pos == pos )
+                        return &v;
+                }
 
+                return nullptr;
+            };
 
     // corner case for loopy tracks: insert the end loop point back into the hull
-    if( auto isect = pnew.SelfIntersecting() )
+    if( const std::optional<SHAPE_LINE_CHAIN::INTERSECTION> isect = pnew.SelfIntersecting() )
     {
         if( isect->p != pnew.CPoint( -1 ) )
-        {
             pnew.Split( isect->p );
-        }
     }
 
     // insert all intersections found into the new hull/path SLCs
-    for( auto& ip : ips )
+    for( SHAPE_LINE_CHAIN::INTERSECTION& ip : ips )
     {
         if( pnew.Find( ip.p, 1 ) < 0)
-        {
             pnew.Split(ip.p);
-        }
 
         if( hnew.Find( ip.p, 1 ) < 0 )
-        {
             hnew.Split(ip.p);
-        }
     }
 
     for( int i = 0; i < pnew.PointCount(); i++ )
     {
-        auto p = pnew.CPoint(i);
-        bool onEdge = hnew.PointOnEdge( p );
+        const VECTOR2I& p = pnew.CPoint( i );
+        bool            onEdge = hnew.PointOnEdge( p );
 
         if ( !onEdge )
             continue;
@@ -264,9 +290,7 @@ bool LINE::Walkaround( const SHAPE_LINE_CHAIN& aObstacle, SHAPE_LINE_CHAIN& aPat
         int idx = hnew.Find( p );
 
         if(idx < 0 )
-        {
-            hnew.Split(p);
-        }
+            hnew.Split( p );
     }
 
     #ifdef TOM_EXTRA_DEBUG
@@ -286,9 +310,9 @@ bool LINE::Walkaround( const SHAPE_LINE_CHAIN& aObstacle, SHAPE_LINE_CHAIN& aPat
     // create a graph of hull/path vertices and classify them (inside/on edge/outside the hull)
     for( int i = 0; i < pnew.PointCount(); i++ )
     {
-        auto p = pnew.CPoint(i);
-        bool onEdge = hnew.PointOnEdge( p );
-        bool inside = hnew.PointInside( p );
+        const VECTOR2I& p = pnew.CPoint(i);
+        bool            onEdge = hnew.PointOnEdge( p );
+        bool            inside = hnew.PointInside( p );
 
         #ifdef TOM_EXTRA_DEBUG
         printf("pnew %d inside %d onedge %d\n", i, !!inside, !!onEdge );
@@ -323,8 +347,8 @@ bool LINE::Walkaround( const SHAPE_LINE_CHAIN& aObstacle, SHAPE_LINE_CHAIN& aPat
     // insert hull vertices into the graph
     for( int i = 0; i < hnew.PointCount(); i++ )
     {
-        auto hp = hnew.CPoint( i );
-        auto vn = findVertex( hp );
+        const VECTOR2I& hp = hnew.CPoint( i );
+        VERTEX*         vn = findVertex( hp );
 
         // if vertex already present (it's very likely that in recursive shoving hull and path vertices will overlap)
         // just mark it as a path vertex that also belongs to the hull
@@ -347,11 +371,11 @@ bool LINE::Walkaround( const SHAPE_LINE_CHAIN& aObstacle, SHAPE_LINE_CHAIN& aPat
     // go around the hull and fix up the neighbour link lists
     for( int i = 0; i < hnew.PointCount(); i++ )
     {
-        auto vc = findVertex( hnew.CPoint(i ) );
-        auto vnext = findVertex( hnew.CPoint( i+1 ) );
+        VERTEX* vc = findVertex( hnew.CPoint( i ) );
+        VERTEX* vnext = findVertex( hnew.CPoint( i+1 ) );
 
-        if(vc && vnext)
-            vc->neighbours.push_back(vnext);
+        if( vc && vnext )
+            vc->neighbours.push_back( vnext );
     }
 
     // In the case that the initial path ends *inside* the current obstacle (i.e. the mouse cursor
@@ -363,17 +387,17 @@ bool LINE::Walkaround( const SHAPE_LINE_CHAIN& aObstacle, SHAPE_LINE_CHAIN& aPat
 
     int i = 0;
 #ifdef TOM_EXTRA_DEBUG
-    for(auto &v: vts)
+    for( VERTEX* &v: vts )
     {
         if( v.indexh < 0 && v.type == ON_EDGE )
-        {
             v.type = OUTSIDE; // hack
-        }
+
         printf("V %d pos %d %d ip %d ih %d type %d\n", i++, v.pos.x, v.pos.y, v.indexp, v.indexh, v.type );
     }
 #endif
     // vts[0] = start point
-    VERTEX* v = &vts[0], *v_prev = nullptr;
+    VERTEX*          v = &vts[0];
+    VERTEX*          v_prev = nullptr;
     SHAPE_LINE_CHAIN out;
 
     int iterLimit = 1000;
@@ -386,15 +410,14 @@ bool LINE::Walkaround( const SHAPE_LINE_CHAIN& aObstacle, SHAPE_LINE_CHAIN& aPat
         // I'm not 100% sure this algorithm doesn't have bugs that may cause it to freeze,
         // so here's a temporary iteration limit
         if( iterLimit == 0 )
-        {
             return false;
-        }
 
         if( v->visited )
         {
             // loop found? stop walking
             break;
         }
+
 #ifdef TOM_EXTRA_DEBUG
         printf("---\nvisit ip %d ih %d type %d outs %d neig %d\n", v->indexp, v->indexh, v->type, out.PointCount(), v->neighbours.size() );
 #endif
@@ -408,10 +431,11 @@ bool LINE::Walkaround( const SHAPE_LINE_CHAIN& aObstacle, SHAPE_LINE_CHAIN& aPat
             // that is not inside the hull
             out.Append( v->pos );
             VERTEX* v_next_fallback = nullptr;
-            for( auto vn : v->neighbours )
+
+            for( VERTEX* vn : v->neighbours )
             {
-                if( areNeighbours( vn->indexp , v->indexp, pnew.PointCount() ) &&
-                    vn->type != INSIDE )
+                if( areNeighbours( vn->indexp , v->indexp, pnew.PointCount() )
+                        && vn->type != INSIDE )
                 {
                     if( !vn->visited )
                     {
@@ -419,7 +443,9 @@ bool LINE::Walkaround( const SHAPE_LINE_CHAIN& aObstacle, SHAPE_LINE_CHAIN& aPat
                         break;
                     }
                     else if( vn != v_prev )
+                    {
                         v_next_fallback = vn;
+                    }
                 }
             }
 
@@ -434,7 +460,6 @@ bool LINE::Walkaround( const SHAPE_LINE_CHAIN& aObstacle, SHAPE_LINE_CHAIN& aPat
                 #endif
                 return false;
             }
-
         }
         else if( v->type == ON_EDGE )
         {
@@ -491,10 +516,10 @@ bool LINE::Walkaround( const SHAPE_LINE_CHAIN& aObstacle, SHAPE_LINE_CHAIN& aPat
 
                 if( v_next )
                 {
-                    for( auto &v : vts )
+                    for( VERTEX &vt : vts )
                     {
-                        if( v.isHull )
-                            v.visited = false;
+                        if( vt.isHull )
+                            vt.visited = false;
                     }
                 }
 
@@ -529,9 +554,7 @@ bool LINE::Walkaround( const SHAPE_LINE_CHAIN& aObstacle, SHAPE_LINE_CHAIN& aPat
         v = v_next;
 
         if( !v )
-        {
             return false;
-        }
     }
 
     if( appendV )
@@ -545,6 +568,12 @@ bool LINE::Walkaround( const SHAPE_LINE_CHAIN& aObstacle, SHAPE_LINE_CHAIN& aPat
 
 const SHAPE_LINE_CHAIN SEGMENT::Hull( int aClearance, int aWalkaroundThickness, int aLayer ) const
 {
+    /*DEBUG_DECORATOR* debugDecorator = ROUTER::GetInstance()->GetInterface()->GetDebugDecorator();
+
+    PNS_DBG( debugDecorator, Message, wxString::Format( wxT( "seghull %d %d" ), aWalkaroundThickness, aClearance ) );
+    PNS_DBG(debugDecorator, AddShape, &m_seg, RED, 0, wxT("theseg") );
+        */
+
    return SegmentHull( m_seg, aClearance, aWalkaroundThickness );
 }
 
@@ -591,7 +620,7 @@ const LINE LINE::ClipToNearestObstacle( NODE* aNode ) const
 
 
 
-SHAPE_LINE_CHAIN dragCornerInternal( const SHAPE_LINE_CHAIN& aOrigin, const VECTOR2I& aP )
+SHAPE_LINE_CHAIN dragCornerInternal( const SHAPE_LINE_CHAIN& aOrigin, const VECTOR2I& aP, DIRECTION_45 aPreferredEndingDirection = DIRECTION_45() )
 {
     std::optional<SHAPE_LINE_CHAIN> picked;
     int i;
@@ -610,17 +639,19 @@ SHAPE_LINE_CHAIN dragCornerInternal( const SHAPE_LINE_CHAIN& aOrigin, const VECT
         return DIRECTION_45().BuildInitialTrace( aOrigin.CPoint( 0 ), aP, dir.IsDiagonal() );
     }
 
-    if( aOrigin.CSegment( -1 ).Length() > 100000 * 30 ) // fixme: constant/parameter?
+
+    //if( aOrigin.CSegment( -1 ).Length() > 100000 * 30 ) // fixme: constant/parameter?
         d = 1;
 
     for( i = aOrigin.SegmentCount() - d; i >= 0; i-- )
     {
-        DIRECTION_45 d_start( aOrigin.CSegment( i ) );
-        VECTOR2I p_start = aOrigin.CPoint( i );
+        DIRECTION_45     d_start( aOrigin.CSegment( i ) );
+        const VECTOR2I&  p_start = aOrigin.CPoint( i );
         SHAPE_LINE_CHAIN paths[2];
-        DIRECTION_45 dirs[2];
-        DIRECTION_45 d_prev = ( i > 0 ? DIRECTION_45( aOrigin.CSegment( i-1 ) ) : DIRECTION_45() );
-        int dirCount = 0;
+        DIRECTION_45     dirs[2];
+        DIRECTION_45     d_prev = ( i > 0 ? DIRECTION_45( aOrigin.CSegment( i-1 ) )
+                                          : DIRECTION_45() );
+        int              dirCount = 0;
 
         for( int j = 0; j < 2; j++ )
         {
@@ -635,12 +666,28 @@ SHAPE_LINE_CHAIN dragCornerInternal( const SHAPE_LINE_CHAIN& aOrigin, const VECT
             ++dirCount;
         }
 
-        for( int j = 0; j < dirCount; j++ )
+        if( aPreferredEndingDirection != DIRECTION_45::UNDEFINED )
         {
-            if( dirs[j] == d_start )
+            for( int j = 0; j < dirCount; j++ )
             {
-                picked = paths[j];
-                break;
+                DIRECTION_45 endingDir( paths[j].CSegment(-1) );
+                if( endingDir == aPreferredEndingDirection )
+                {
+                    picked = paths[j];
+                    break;
+                }
+            }
+        }
+
+        if( !picked )
+        {
+            for( int j = 0; j < dirCount; j++ )
+            {
+                if( dirs[j] == d_start )
+                {
+                    picked = paths[j];
+                    break;
+                }
             }
         }
 
@@ -674,7 +721,7 @@ SHAPE_LINE_CHAIN dragCornerInternal( const SHAPE_LINE_CHAIN& aOrigin, const VECT
 }
 
 
-void LINE::dragCorner45( const VECTOR2I& aP, int aIndex )
+void LINE::dragCorner45( const VECTOR2I& aP, int aIndex, DIRECTION_45 aPreferredEndingDirection )
 {
     SHAPE_LINE_CHAIN path;
 
@@ -682,9 +729,13 @@ void LINE::dragCorner45( const VECTOR2I& aP, int aIndex )
     VECTOR2I snapped = snapDraggedCorner( m_line, aP, aIndex );
 
     if( aIndex == 0 )
-        path = dragCornerInternal( m_line.Reverse(), snapped ).Reverse();
+    {
+        path = dragCornerInternal( m_line.Reverse(), snapped, aPreferredEndingDirection ).Reverse();
+    }
     else if( aIndex == m_line.SegmentCount() )
-        path = dragCornerInternal( m_line, snapped );
+    {
+        path = dragCornerInternal( m_line, snapped, aPreferredEndingDirection );
+    }
     else
     {
         // Are we next to an arc? Insert a new point so we slice correctly
@@ -692,9 +743,9 @@ void LINE::dragCorner45( const VECTOR2I& aP, int aIndex )
             m_line.Insert( aIndex + 1, m_line.CPoint( aIndex + 1 ) );
 
         // fixme: awkward behaviour for "outwards" drags
-        path = dragCornerInternal( m_line.Slice( 0, aIndex ), snapped );
+        path = dragCornerInternal( m_line.Slice( 0, aIndex ), snapped, aPreferredEndingDirection );
         SHAPE_LINE_CHAIN path_rev =
-                dragCornerInternal( m_line.Slice( aIndex, -1 ).Reverse(), snapped ).Reverse();
+                dragCornerInternal( m_line.Slice( aIndex, -1 ).Reverse(), snapped, aPreferredEndingDirection ).Reverse();
         path.Append( path_rev );
     }
 
@@ -731,7 +782,7 @@ void LINE::dragCornerFree( const VECTOR2I& aP, int aIndex )
     m_line.Simplify();
 }
 
-void LINE::DragCorner( const VECTOR2I& aP, int aIndex, bool aFreeAngle )
+void LINE::DragCorner( const VECTOR2I& aP, int aIndex, bool aFreeAngle, DIRECTION_45 aPreferredEndingDirection )
 {
     wxCHECK_RET( aIndex >= 0, wxT( "Negative index passed to LINE::DragCorner" ) );
 
@@ -741,7 +792,7 @@ void LINE::DragCorner( const VECTOR2I& aP, int aIndex, bool aFreeAngle )
     }
     else
     {
-        dragCorner45( aP, aIndex );
+        dragCorner45( aP, aIndex, aPreferredEndingDirection );
     }
 }
 
@@ -1035,9 +1086,21 @@ void LINE::AppendVia( const VIA& aVia )
         Reverse();
     }
 
-    m_hasVia = true;
+    m_via = aVia.Clone();
+    m_via->SetOwner( this );
+    m_via->SetNet( m_net );
+}
+
+
+void LINE::LinkVia( VIA* aVia )
+{
+    if( m_line.PointCount() > 1 && aVia->Pos() == m_line.CPoint( 0 ) )
+    {
+        Reverse();
+    }
+
     m_via = aVia;
-    m_via.SetNet( m_net );
+    Link( aVia );
 }
 
 
@@ -1243,11 +1306,26 @@ bool LINE::HasLockedSegments() const
 
 void LINE::Clear()
 {
-    m_hasVia = false;
+    ClearLinks();
+    RemoveVia();
     m_line.Clear();
 }
 
 
+void LINE::RemoveVia()
+{
+    if( m_via )
+    {
+        if( ContainsLink( m_via ) )
+            Unlink( m_via );
+        if( m_via->BelongsTo( this ) )
+            delete m_via;
+    }
+
+    m_via = nullptr;
+}
+
+
 const std::string SEGMENT::Format( ) const
 {
     std::stringstream ss;
@@ -1256,5 +1334,19 @@ const std::string SEGMENT::Format( ) const
     return ss.str();
 }
 
+
+int LINE::FindSegment( const SEGMENT* aSeg ) const
+{
+    for( int i = 0; i < m_line.SegmentCount(); i++)
+    {
+        const SEG&s = m_line.CSegment(i);
+        if( s == aSeg->Seg() )
+            return i;
+    }
+
+    return -1;
 }
 
+}
+
+
