@@ -2,7 +2,7 @@
  * KiRouter - a push-and-(sometimes-)shove PCB router
  *
  * Copyright (C) 2013-2017 CERN
- * Copyright (C) 2016-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * Author: Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
  * This program is free software: you can redistribute it and/or modify it
@@ -32,7 +32,6 @@
 #include "pns_topology.h"
 #include "pns_walkaround.h"
 #include "pns_mouse_trail_tracer.h"
-#include "pns_utils.h"
 
 #include <wx/log.h>
 
@@ -50,7 +49,7 @@ LINE_PLACER::LINE_PLACER( ROUTER* aRouter ) :
     // Init temporary variables (do not leave uninitialized members)
     m_lastNode = nullptr;
     m_placingVia = false;
-    m_currentNet = 0;
+    m_currentNet = nullptr;
     m_currentLayer = 0;
     m_startItem = nullptr;
     m_endItem = nullptr;
@@ -73,10 +72,18 @@ void LINE_PLACER::setWorld( NODE* aWorld )
 
 const VIA LINE_PLACER::makeVia( const VECTOR2I& aP )
 {
-    const LAYER_RANGE layers( m_sizes.ViaType() == VIATYPE::THROUGH ? F_Cu : m_sizes.GetLayerTop(),
-                              m_sizes.ViaType() == VIATYPE::THROUGH ? B_Cu : m_sizes.GetLayerBottom() );
+    // fixme: should belong to KICAD_IFACE
+    auto iface = Router()->GetInterface();
 
-    return VIA( aP, layers, m_sizes.ViaDiameter(), m_sizes.ViaDrill(), -1, m_sizes.ViaType() );
+    int start = m_sizes.ViaType() == VIATYPE::THROUGH ?iface->GetPNSLayerFromBoardLayer( F_Cu ) : m_sizes.GetLayerTop();
+    int end = m_sizes.ViaType() == VIATYPE::THROUGH ? iface->GetPNSLayerFromBoardLayer( B_Cu ) : m_sizes.GetLayerBottom();
+
+    const PNS_LAYER_RANGE layers(
+        start ,
+        end
+    );
+
+    return VIA( aP, layers, m_sizes.ViaDiameter(), m_sizes.ViaDrill(), nullptr, m_sizes.ViaType() );
 }
 
 
@@ -218,9 +225,7 @@ bool LINE_PLACER::handlePullback()
 
     if( pullback_1 || pullback_2 )
     {
-        lastSegIdx = tail.PrevShape( -1 );
-
-        if( !tail.IsPtOnArc( lastSegIdx ) )
+        if( !tail.IsArcSegment( lastSegIdx ) )
         {
             const SEG& seg = tail.CSegment( lastSegIdx );
             m_direction    = DIRECTION_45( seg );
@@ -392,29 +397,8 @@ bool LINE_PLACER::mergeHead()
 }
 
 
-VECTOR2I closestProjectedPoint( const SHAPE_LINE_CHAIN& line, const VECTOR2I& p )
-{
-    // Keep distances squared for performance
-    SEG::ecoord min_dist_sq = VECTOR2I::ECOORD_MAX;
-    VECTOR2I    closest;
-
-    for( int i = 0; i < line.SegmentCount(); i++ )
-    {
-        const SEG& s = line.CSegment( i );
-        VECTOR2I   a = s.NearestPoint( p );
-        int        d_sq = (a - p).SquaredEuclideanNorm();
-
-        if( d_sq < min_dist_sq )
-        {
-            min_dist_sq = d_sq;
-            closest = a;
-        }
-    }
-
-    return closest;
-}
-
-bool LINE_PLACER::clipAndCheckCollisions( VECTOR2I aP, SHAPE_LINE_CHAIN aL, SHAPE_LINE_CHAIN& aOut, int &thresholdDist )
+bool LINE_PLACER::clipAndCheckCollisions( const VECTOR2I& aP, const SHAPE_LINE_CHAIN& aL,
+                                          SHAPE_LINE_CHAIN& aOut, int &thresholdDist )
 {
     SHAPE_LINE_CHAIN l( aL );
     int idx = l.Split( aP );
@@ -448,7 +432,8 @@ bool LINE_PLACER::clipAndCheckCollisions( VECTOR2I aP, SHAPE_LINE_CHAIN aL, SHAP
 }
 
 
-bool LINE_PLACER::cursorDistMinimum( const SHAPE_LINE_CHAIN& aL, const VECTOR2I& aCursor,  double lengthThreshold, SHAPE_LINE_CHAIN &aOut )
+bool LINE_PLACER::cursorDistMinimum( const SHAPE_LINE_CHAIN& aL, const VECTOR2I& aCursor,
+                                     double lengthThreshold, SHAPE_LINE_CHAIN &aOut )
 {
     std::vector<int>      dists;
     std::vector<VECTOR2I> pts;
@@ -568,7 +553,7 @@ bool LINE_PLACER::cursorDistMinimum( const SHAPE_LINE_CHAIN& aL, const VECTOR2I&
 }
 
 
-bool LINE_PLACER::rhWalkBase( const VECTOR2I& aP, LINE& aWalkLine, int aCollisionMask,
+bool LINE_PLACER::rhWalkBase( const VECTOR2I& aP, LINE& aWalkLine, int aCollisionMask, PNS::PNS_MODE aMode,
                               bool& aViaOk )
 {
     LINE walkFull( m_head );
@@ -586,6 +571,7 @@ bool LINE_PLACER::rhWalkBase( const VECTOR2I& aP, LINE& aWalkLine, int aCollisio
     walkaround.SetLogger( Logger() );
     walkaround.SetIterationLimit( Settings().WalkaroundIterationLimit() );
     walkaround.SetItemMask( aCollisionMask );
+    walkaround.SetAllowedPolicies( { WALKAROUND::WP_CCW, WALKAROUND::WP_CW } );
 
     int round = 0;
 
@@ -596,9 +582,11 @@ bool LINE_PLACER::rhWalkBase( const VECTOR2I& aP, LINE& aWalkLine, int aCollisio
         PNS_DBG( Dbg(), BeginGroup, wxString::Format( "walk-round-%d", round ), 0 );
         round++;
 
-        aViaOk = buildInitialLine( walkP, l1, round == 0 );
+        aViaOk = buildInitialLine( walkP, l1, aMode, round == 0 );
         PNS_DBG( Dbg(), AddItem, &l1, BLUE, 20000, wxT( "walk-base-l1" ) );
-        PNS_DBG( Dbg(), AddPoint, l1.Via().Pos(), BLUE, 100000, wxT( "walk-base-l1-via" ) );
+
+        if( l1.EndsWithVia() )
+            PNS_DBG( Dbg(), AddPoint, l1.Via().Pos(), BLUE, 100000, wxT( "walk-base-l1-via" ) );
 
         LINE initTrack( m_tail );
         initTrack.Line().Append( l1.CLine() );
@@ -618,53 +606,61 @@ bool LINE_PLACER::rhWalkBase( const VECTOR2I& aP, LINE& aWalkLine, int aCollisio
         optimizer.SetEffortLevel( OPTIMIZER::MERGE_SEGMENTS );
         optimizer.SetCollisionMask( aCollisionMask );
 
-        int len_cw = wr.statusCw != WALKAROUND::STUCK ? wr.lineCw.CLine().Length()
+        using WALKAROUND::WP_CW;
+        using WALKAROUND::WP_CCW;
+
+        int len_cw = wr.status[WP_CW] != WALKAROUND::ST_STUCK ? wr.lines[WP_CW].CLine().Length()
                                                       : std::numeric_limits<int>::max();
-        int len_ccw = wr.statusCcw != WALKAROUND::STUCK ? wr.lineCcw.CLine().Length()
+        int len_ccw = wr.status[WP_CCW] != WALKAROUND::ST_STUCK ? wr.lines[WP_CCW].CLine().Length()
                                                         : std::numeric_limits<int>::max();
 
-        if( wr.statusCw == WALKAROUND::DONE )
+
+        if( wr.status[ WP_CW ] == WALKAROUND::ST_DONE )
         {
-            PNS_DBG( Dbg(), AddItem, &wr.lineCw, BLUE, 20000, wxT( "wf-result-cw-preopt" ) );
+            PNS_DBG( Dbg(), AddItem, &wr.lines[WP_CW], BLUE, 20000, wxT( "wf-result-cw-preopt" ) );
             LINE tmpHead, tmpTail;
 
-            if( splitHeadTail( wr.lineCw, m_tail, tmpHead, tmpTail ) )
+
+            OPTIMIZER::Optimize( &wr.lines[WP_CW], OPTIMIZER::MERGE_SEGMENTS, m_currentNode );
+
+            if( splitHeadTail( wr.lines[WP_CW], m_tail, tmpHead, tmpTail ) )
             {
                 optimizer.Optimize( &tmpHead );
-                wr.lineCw.SetShape( tmpTail.CLine () );
-                wr.lineCw.Line().Append( tmpHead.CLine( ) );
+                wr.lines[WP_CW].SetShape( tmpTail.CLine () );
+                wr.lines[WP_CW].Line().Append( tmpHead.CLine( ) );
             }
 
-            PNS_DBG( Dbg(), AddItem, &wr.lineCw, RED, 20000, wxT( "wf-result-cw-postopt" ) );
-            len_cw = wr.lineCw.CLine().Length();
-            bestLine = wr.lineCw;
+            PNS_DBG( Dbg(), AddItem, &wr.lines[WP_CW], RED, 20000, wxT( "wf-result-cw-postopt" ) );
+            len_cw = wr.lines[WP_CW].CLine().Length();
+            bestLine = wr.lines[WP_CW];
         }
 
-        if( wr.statusCcw == WALKAROUND::DONE )
+        if( wr.status[WP_CCW] == WALKAROUND::ST_DONE )
         {
-            PNS_DBG( Dbg(), AddItem, &wr.lineCcw, BLUE, 20000, wxT( "wf-result-ccw-preopt" ) );
+            PNS_DBG( Dbg(), AddItem, &wr.lines[WP_CCW], BLUE, 20000, wxT( "wf-result-ccw-preopt" ) );
 
             LINE tmpHead, tmpTail;
 
-            if( splitHeadTail( wr.lineCw, m_tail, tmpHead, tmpTail ) )
+            OPTIMIZER::Optimize( &wr.lines[WP_CCW], OPTIMIZER::MERGE_SEGMENTS, m_currentNode );
+
+            if( splitHeadTail( wr.lines[WP_CCW], m_tail, tmpHead, tmpTail ) )
             {
                 optimizer.Optimize( &tmpHead );
-                wr.lineCw.SetShape( tmpTail.CLine () );
-                wr.lineCw.Line().Append( tmpHead.CLine( ) );
+                wr.lines[WP_CCW].SetShape( tmpTail.CLine () );
+                wr.lines[WP_CCW].Line().Append( tmpHead.CLine( ) );
             }
 
-            PNS_DBG( Dbg(), AddItem, &wr.lineCcw, RED, 20000, wxT( "wf-result-ccw-postopt" ) );
-            len_ccw = wr.lineCcw.CLine().Length();
+            PNS_DBG( Dbg(), AddItem, &wr.lines[WP_CCW], RED, 20000, wxT( "wf-result-ccw-postopt" ) );
+            len_ccw = wr.lines[WP_CCW].CLine().Length();
+
             if( len_ccw < len_cw )
-                {
-                bestLine = wr.lineCcw;
-                }
-            }
+                bestLine = wr.lines[WP_CCW];
+        }
 
         int bestLength = len_cw < len_ccw ? len_cw : len_ccw;
 
         if( bestLength < hugThresholdLengthComplete && bestLine.has_value() )
-            {
+        {
             walkFull.SetShape( bestLine->CLine() );
             walkP = walkFull.CLine().CPoint(-1);
             PNS_DBGN( Dbg(), EndGroup );
@@ -678,42 +674,43 @@ bool LINE_PLACER::rhWalkBase( const VECTOR2I& aP, LINE& aWalkLine, int aCollisio
 
         SHAPE_LINE_CHAIN l_cw, l_ccw;
 
-        if( wr.statusCw != WALKAROUND::STUCK )
-                {
-            validCw = cursorDistMinimum( wr.lineCw.CLine(), aP, hugThresholdLength, l_cw );
-            if( validCw )
-            {
-                distCw = ( aP - l_cw.CPoint( -1 ) ).EuclideanNorm();
-                }
-            PNS_DBG( Dbg(), AddShape, &l_cw, MAGENTA, 200000,
-                     wxString::Format( "wh-result-cw %s",
-                                       validCw ? "non-colliding" : "colliding" ) );
-            }
 
-        if( wr.statusCcw != WALKAROUND::STUCK )
+        if( wr.status[WP_CW] != WALKAROUND::ST_STUCK )
         {
-            validCcw = cursorDistMinimum( wr.lineCcw.CLine(), aP, hugThresholdLength, l_ccw );
+            validCw = cursorDistMinimum( wr.lines[WP_CW].CLine(), aP, hugThresholdLength, l_cw );
+
+            if( validCw )
+                distCw = ( aP - l_cw.CPoint( -1 ) ).EuclideanNorm();
+
+            PNS_DBG( Dbg(), AddShape, &l_cw, MAGENTA, 200000, wxString::Format( "wh-result-cw %s",
+                                                                                 validCw ? "non-colliding"
+                                                                                         : "colliding" ) );
+        }
+
+        if( wr.status[WP_CCW] != WALKAROUND::ST_STUCK )
+        {
+            validCcw = cursorDistMinimum( wr.lines[WP_CCW].CLine(), aP, hugThresholdLength, l_ccw );
+
             if( validCcw )
-            {
                 distCcw = ( aP - l_ccw.CPoint( -1 ) ).EuclideanNorm();
-            }
-            PNS_DBG( Dbg(), AddShape, &l_ccw, MAGENTA, 200000,
-                     wxString::Format( "wh-result-ccw %s",
-                                       validCcw ? "non-colliding" : "colliding" ) );
+
+            PNS_DBG( Dbg(), AddShape, &l_ccw, MAGENTA, 200000, wxString::Format( "wh-result-ccw %s",
+                                                                                 validCcw ? "non-colliding"
+                                                                                          : "colliding" ) );
         }
 
 
         if( distCw < distCcw && validCw )
-            {
-                walkFull.SetShape( l_cw );
+        {
+            walkFull.SetShape( l_cw );
             walkP = l_cw.CPoint(-1);
-            }
+        }
         else if( validCcw )
-            {
-                walkFull.SetShape( l_ccw );
+        {
+            walkFull.SetShape( l_ccw );
             walkP = l_ccw.CPoint(-1);
-            }
-            else
+        }
+        else
         {
             PNS_DBGN( Dbg(), EndGroup );
             return false;
@@ -731,11 +728,16 @@ bool LINE_PLACER::rhWalkBase( const VECTOR2I& aP, LINE& aWalkLine, int aCollisio
     }
 
     PNS_DBG( Dbg(), AddItem, &walkFull, GREEN, 200000, wxT( "walk-full" ) );
-    PNS_DBG( Dbg(), AddPoint, walkFull.Via().Pos(), GREEN, 200000, wxString::Format( "walk-via ok %d", aViaOk?1:0 ) );
+
+    if( walkFull.EndsWithVia() )
+    {
+        PNS_DBG( Dbg(), AddPoint, walkFull.Via().Pos(), GREEN, 200000,
+                 wxString::Format( "walk-via ok %d", aViaOk ? 1 : 0 ) );
+    }
 
     aWalkLine = walkFull;
 
-    return true;
+    return !walkFull.EndsWithVia() || aViaOk;
 }
 
 
@@ -746,7 +748,7 @@ bool LINE_PLACER::rhWalkOnly( const VECTOR2I& aP, LINE& aNewHead, LINE& aNewTail
     int effort = 0;
     bool viaOk = false;
 
-    if( ! rhWalkBase( aP, walkFull, ITEM::ANY_T, viaOk ) )
+    if( ! rhWalkBase( aP, walkFull, ITEM::ANY_T, RM_Walkaround, viaOk ) )
         return false;
 
     switch( Settings().OptimizerEffort() )
@@ -782,7 +784,7 @@ bool LINE_PLACER::rhWalkOnly( const VECTOR2I& aP, LINE& aNewHead, LINE& aNewTail
     // (now a hull or a walk/shove line cannot collide with the 'owner' of the hull under any circumstances).
     // This, however, introduced a subtle bug. For a row/column/any other 'regular' arrangement
     // of overlapping hulls (think of pads of a SOP/SOIC chip or a regular via grid), walking around may
-    // produce a new 'head' that is not considered colliding (due to the clearance epsilon), but with 
+    // produce a new 'head' that is not considered colliding (due to the clearance epsilon), but with
     // its start point inside one of the subsequent hulls to process.
     // We can't have head[0] inside any hull for the algorithm to work - therefore, we now consider the entire
     // 'tail+head' trace when walking around and in case of success, reconstruct the
@@ -812,7 +814,7 @@ bool LINE_PLACER::rhWalkOnly( const VECTOR2I& aP, LINE& aNewHead, LINE& aNewTail
 
 bool LINE_PLACER::rhMarkObstacles( const VECTOR2I& aP, LINE& aNewHead, LINE& aNewTail )
 {
-    buildInitialLine( aP, m_head );
+    buildInitialLine( aP, m_head, RM_MarkObstacles );
     m_head.SetBlockingObstacle( nullptr );
 
     auto obs = m_currentNode->NearestObstacle( &m_head );
@@ -822,15 +824,19 @@ bool LINE_PLACER::rhMarkObstacles( const VECTOR2I& aP, LINE& aNewHead, LINE& aNe
     // the shove/walk mode that certain users find too intrusive.
     if( obs )
     {
-        int cl = m_currentNode->GetClearance( obs->m_item, &m_head, false );
-        auto hull = obs->m_item->Hull( cl, m_head.Width() );
+        int              clearance = m_currentNode->GetClearance( obs->m_item, &m_head, false );
+        SHAPE_LINE_CHAIN hull = obs->m_item->Hull( clearance, m_head.Width(), m_head.Layer() );
+        VECTOR2I         nearest;
 
-        auto nearest = hull.NearestPoint( aP );
+        DIRECTION_45::CORNER_MODE cornerMode = Settings().GetCornerMode();
+
+        if( cornerMode == DIRECTION_45::MITERED_90 || cornerMode == DIRECTION_45::ROUNDED_90 )
+            nearest = hull.BBox().NearestPoint( aP );
+        else
+            nearest = hull.NearestPoint( aP );
 
         if( ( nearest - aP ).EuclideanNorm() < m_head.Width() / 2 )
-        {
-            buildInitialLine( nearest, m_head );
-        }
+            buildInitialLine( nearest, m_head, RM_MarkObstacles );
     }
 
     // Note: Something like the below could be used to implement a "stop at first obstacle" mode,
@@ -890,16 +896,16 @@ bool LINE_PLACER::splitHeadTail( const LINE& aNewLine, const LINE& aOldTail, LIN
         if( !found )
             i--;
 
+        // If the old tail doesn't have any points of the new line, we can't split it.
+        if( i >= l2.PointCount() )
+            i = l2.PointCount() - 1;
+
         newHead.Clear();
 
         if( i == 0 )
-        {
             newTail.Clear();
-        }
-    else
-    {
+        else
             newTail.SetShape( l2.CLine().Slice( 0, i ) );
-    }
 
         newHead.SetShape( l2.CLine().Slice( i, -1 ) );
     }
@@ -915,16 +921,16 @@ bool LINE_PLACER::splitHeadTail( const LINE& aNewLine, const LINE& aOldTail, LIN
     aNewTail = newTail;
 
     return true;
-    }
+}
 
 
 bool LINE_PLACER::rhShoveOnly( const VECTOR2I& aP, LINE& aNewHead, LINE& aNewTail )
-    {
+{
     LINE walkSolids;
 
     bool viaOk = false;
 
-    if( ! rhWalkBase( aP, walkSolids, ITEM::SOLID_T, viaOk ) )
+    if( ! rhWalkBase( aP, walkSolids, ITEM::SOLID_T, RM_Shove, viaOk ) )
         return false;
 
     m_currentNode = m_shove->CurrentNode();
@@ -935,24 +941,24 @@ bool LINE_PLACER::rhShoveOnly( const VECTOR2I& aP, LINE& aNewHead, LINE& aNewTai
     if( m_endItem )
     {
         // Make sure the springback algorithm won't erase the NODE that owns m_endItem.
-        m_shove->SetSpringbackDoNotTouchNode( m_endItem->Owner() );
+        m_shove->SetSpringbackDoNotTouchNode( static_cast<const NODE*>( m_endItem->Owner() ) );
     }
 
     LINE newHead( walkSolids );
 
     if( walkSolids.EndsWithVia() )
-    {
         PNS_DBG( Dbg(), AddPoint, newHead.Via().Pos(), RED, 1000000, wxString::Format( "SVIA [%d]", viaOk?1:0 ) );
-    }
 
     if( m_placingVia && viaOk )
     {
         newHead.AppendVia( makeVia( newHead.CPoint( -1 ) ) );
         PNS_DBG( Dbg(), AddPoint, newHead.Via().Pos(), GREEN, 1000000, "shove-new-via" );
-        
+
     }
 
-    SHOVE::SHOVE_STATUS status = m_shove->ShoveLines( newHead );
+    m_shove->ClearHeads();
+    m_shove->AddHeads( newHead, SHOVE::SHP_SHOVE );
+    bool shoveOk = m_shove->Run() == SHOVE::SH_OK;
 
     m_currentNode = m_shove->CurrentNode();
 
@@ -980,27 +986,25 @@ bool LINE_PLACER::rhShoveOnly( const VECTOR2I& aP, LINE& aNewHead, LINE& aNewTai
         effort |= OPTIMIZER::SMART_PADS;
     }
 
-    if( status == SHOVE::SH_OK  || status == SHOVE::SH_HEAD_MODIFIED )
+    if( shoveOk )
     {
-        if( status == SHOVE::SH_HEAD_MODIFIED )
-            newHead = m_shove->NewHead();
+        if( m_shove->HeadsModified() )
+            newHead = m_shove->GetModifiedHead( 0 );
 
-        OPTIMIZER optimizer( m_currentNode );
-
-        PNS_DBG( Dbg(), AddPoint, newHead.Via().Pos(), GREEN, 1000000, "shove-via-preopt" );
-        PNS_DBG( Dbg(), AddPoint, newHead.Via().Pos(), GREEN, 1000000, "shove-via-postopt" );
+        if( newHead.EndsWithVia() )
+        {
+            PNS_DBG( Dbg(), AddPoint, newHead.Via().Pos(), GREEN, 1000000, "shove-via-preopt" );
+            PNS_DBG( Dbg(), AddPoint, newHead.Via().Pos(), GREEN, 1000000, "shove-via-postopt" );
+        }
 
         if( ! splitHeadTail( newHead, m_tail, aNewHead, aNewTail ) )
             return false;
 
         if( newHead.EndsWithVia() )
-        {
             aNewHead.AppendVia( newHead.Via() );
-        }
 
-        optimizer.SetEffortLevel( effort );
-        optimizer.SetCollisionMask( ITEM::ANY_T );
-        optimizer.Optimize( &aNewHead );
+        OPTIMIZER::Optimize( &aNewHead, effort, m_currentNode );
+        PNS_DBG( Dbg(), AddItem, aNewHead.Clone(), GREEN, 1000000, "head-sh-postopt" );
 
         return true;
     }
@@ -1077,7 +1081,6 @@ bool LINE_PLACER::optimizeTailHeadTransition()
     // If so, replace the (threshold) last tail points and the head with
     // the optimized line
 
-
     PNS_DBG( Dbg(), AddItem, &new_head, LIGHTCYAN, 10000, wxT( "ht-newline" ) );
 
     if( OPTIMIZER::Optimize( &new_head, OPTIMIZER::MERGE_SEGMENTS, m_currentNode ) )
@@ -1099,13 +1102,9 @@ bool LINE_PLACER::optimizeTailHeadTransition()
 void LINE_PLACER::updatePStart( const LINE& tail )
 {
     if( tail.CLine().PointCount() )
-        {
-            m_p_start = tail.CLine().CPoint(-1);
-        }
-        else
-        {
-            m_p_start = m_currentStart;
-        }
+        m_p_start = tail.CLine().CPoint(-1);
+    else
+        m_p_start = m_currentStart;
 }
 
 void LINE_PLACER::routeStep( const VECTOR2I& aP )
@@ -1117,9 +1116,9 @@ void LINE_PLACER::routeStep( const VECTOR2I& aP )
 
 
     PNS_DBG( Dbg(), Message, wxString::Format( "routeStep: direction: %s head: %d, tail: %d shapes" ,
-                m_direction.Format(),
-                m_head.ShapeCount(),
-                m_tail.ShapeCount() ) );
+                                               m_direction.Format(),
+                                               m_head.ShapeCount(),
+                                               m_tail.ShapeCount() ) );
 
     PNS_DBG( Dbg(), BeginGroup, wxT( "route-step" ), 0 );
 
@@ -1146,6 +1145,16 @@ void LINE_PLACER::routeStep( const VECTOR2I& aP )
         {
             m_tail = prevTail;
             m_head = prevHead;
+
+            // If we fail to walk out of the initial point (no tail), instead of returning an empty
+            // line, return a zero-length line so that the user gets some feedback that routing is
+            // happening.  This will get pruned later.
+            if( m_tail.PointCount() == 0 )
+            {
+                m_tail.Line().Append( m_p_start );
+                m_tail.Line().Append( m_p_start, true );
+            }
+
             fail = true;
         }
 
@@ -1214,10 +1223,10 @@ bool LINE_PLACER::route( const VECTOR2I& aP )
 {
     routeStep( aP );
 
-    if (!m_head.PointCount() )
+    if( !m_head.PointCount() )
         return false;
 
-    return m_head.CPoint(-1) == aP;
+    return m_head.CPoint( -1 ) == aP;
 }
 
 
@@ -1225,7 +1234,12 @@ const LINE LINE_PLACER::Trace() const
 {
     SHAPE_LINE_CHAIN l( m_tail.CLine() );
     l.Append( m_head.CLine() );
-    l.Simplify();
+
+    // Only simplify if we have more than two points, because if we have a zero-length seg as the
+    // only part of the trace, we don't want it to be removed at this stage (will be the case if
+    // the routing start point violates DRC due to track width in shove/walk mode, for example).
+    if( l.PointCount() > 2 )
+        l.Simplify();
 
     LINE tmp( m_head );
 
@@ -1247,6 +1261,16 @@ const ITEM_SET LINE_PLACER::Traces()
 
 void LINE_PLACER::FlipPosture()
 {
+    // In order to fix issue 12369 get the current line placer first direction
+    // and copy it to the mouse trail tracer, as the current placer may have
+    // changed the route.
+    if( m_mouseTrailTracer.IsManuallyForced() == false && m_currentTrace.SegmentCount() > 0 )
+    {
+        DIRECTION_45 firstDirection( m_currentTrace.CSegment( 0 ) );
+
+        m_mouseTrailTracer.SetDefaultDirections( firstDirection, DIRECTION_45::UNDEFINED );
+    }
+
     m_mouseTrailTracer.FlipPosture();
 }
 
@@ -1268,7 +1292,7 @@ bool LINE_PLACER::SplitAdjacentSegments( NODE* aNode, ITEM* aSeg, const VECTOR2I
     if( !aSeg->OfKind( ITEM::SEGMENT_T ) )
         return false;
 
-    JOINT* jt = aNode->FindJoint( aP, aSeg );
+    const JOINT* jt = aNode->FindJoint( aP, aSeg );
 
     if( jt && jt->LinkCount() >= 1 )
         return false;
@@ -1288,6 +1312,38 @@ bool LINE_PLACER::SplitAdjacentSegments( NODE* aNode, ITEM* aSeg, const VECTOR2I
 }
 
 
+bool LINE_PLACER::SplitAdjacentArcs( NODE* aNode, ITEM* aArc, const VECTOR2I& aP )
+{
+    if( !aArc )
+        return false;
+
+    if( !aArc->OfKind( ITEM::ARC_T ) )
+        return false;
+
+    const JOINT* jt = aNode->FindJoint( aP, aArc );
+
+    if( jt && jt->LinkCount() >= 1 )
+        return false;
+
+    ARC*             a_old = static_cast<ARC*>( aArc );
+    const SHAPE_ARC& o_arc = a_old->Arc();
+
+    std::unique_ptr<ARC> a_new[2] = { Clone( *a_old ), Clone( *a_old ) };
+
+    a_new[0]->Arc().ConstructFromStartEndCenter( o_arc.GetP0(), aP, o_arc.GetCenter(),
+                                                 o_arc.IsClockwise(), o_arc.GetWidth() );
+
+    a_new[1]->Arc().ConstructFromStartEndCenter( aP, o_arc.GetP1(), o_arc.GetCenter(),
+                                                 o_arc.IsClockwise(), o_arc.GetWidth() );
+
+    aNode->Remove( a_old );
+    aNode->Add( std::move( a_new[0] ), true );
+    aNode->Add( std::move( a_new[1] ), true );
+
+    return true;
+}
+
+
 bool LINE_PLACER::SetLayer( int aLayer )
 {
     if( m_idle )
@@ -1327,7 +1383,7 @@ bool LINE_PLACER::Start( const VECTOR2I& aP, ITEM* aStartItem )
     m_currentStart = VECTOR2I( aP );
     m_fixStart =  VECTOR2I( aP );
     m_currentEnd = VECTOR2I( aP );
-    m_currentNet = std::max( 0, aStartItem ? aStartItem->Net() : 0 );
+    m_currentNet = aStartItem ? aStartItem->Net() : Router()->GetInterface()->GetOrphanedNetHandle();
     m_startItem = aStartItem;
     m_placingVia = false;
     m_chainedPlacement = false;
@@ -1426,11 +1482,10 @@ void LINE_PLACER::initPlacement()
 bool LINE_PLACER::Move( const VECTOR2I& aP, ITEM* aEndItem )
 {
     LINE current;
-    VECTOR2I p = aP;
-    int eiDepth = -1;
+    int  eiDepth = -1;
 
     if( aEndItem && aEndItem->Owner() )
-        eiDepth = static_cast<NODE*>( aEndItem->Owner() )->Depth();
+        eiDepth = static_cast<const NODE*>( aEndItem->Owner() )->Depth();
 
     if( m_lastNode )
     {
@@ -1440,7 +1495,7 @@ bool LINE_PLACER::Move( const VECTOR2I& aP, ITEM* aEndItem )
 
     m_endItem = aEndItem;
 
-    bool reachesEnd = route( p );
+    bool reachesEnd = route( aP );
 
     current = Trace();
 
@@ -1457,7 +1512,8 @@ bool LINE_PLACER::Move( const VECTOR2I& aP, ITEM* aEndItem )
             && aEndItem && latestNode->Depth() >= eiDepth
             && current.SegmentCount() )
     {
-        SplitAdjacentSegments( m_lastNode, aEndItem, current.CPoint( -1 ) );
+        if ( aEndItem->Net() == m_currentNet )
+            SplitAdjacentSegments( m_lastNode, aEndItem, current.CPoint( -1 ) );
 
         if( Settings().RemoveLoops() )
             removeLoops( m_lastNode, current );
@@ -1485,20 +1541,34 @@ bool LINE_PLACER::FixRoute( const VECTOR2I& aP, ITEM* aEndItem, bool aForceFinis
         {
             // The user has indicated a connection should be made.  If either the trace or
             // endItem is net-less, then allow the connection by adopting the net of the other.
-            if( m_currentNet <= 0 )
+            if( m_router->GetInterface()->GetNetCode( m_currentNet ) <= 0 )
             {
                 m_currentNet = aEndItem->Net();
                 pl.SetNet( m_currentNet );
             }
-            else if (aEndItem->Net() <= 0 )
+            else if( m_router->GetInterface()->GetNetCode( aEndItem->Net() ) <= 0 )
             {
                 aEndItem->SetNet( m_currentNet );
             }
         }
+    }
 
-        // Collisions still prevent fixing unless "Allow DRC violations" is checked
-        if( !Settings().AllowDRCViolations() && m_world->CheckColliding( &pl ) )
-            return false;
+    // Collisions still prevent fixing unless "Allow DRC violations" is checked
+    // Note that collisions can occur even in walk/shove modes if the beginning of the trace
+    // collides (for example if the starting track width is too high).
+
+    if( !Settings().AllowDRCViolations() )
+    {
+        NODE* checkNode = ( Settings().Mode() == RM_Shove ) ? m_shove->CurrentNode() : m_world;
+        std::optional<OBSTACLE> obs = checkNode->CheckColliding( &pl );
+
+        if( obs )
+        {
+            // TODO: Determine why the shove node sometimes reports collisions against shoved objects.
+            // For now, to work around this issue, we consider only solids in shove mode.
+            if( Settings().Mode() != RM_Shove || obs->m_item->OfKind( ITEM::SOLID_T ) )
+                return false;
+        }
     }
 
     const SHAPE_LINE_CHAIN& l = pl.CLine();
@@ -1540,7 +1610,7 @@ bool LINE_PLACER::FixRoute( const VECTOR2I& aP, ITEM* aEndItem, bool aForceFinis
     if( l.PointCount() > 2 )
         p_pre_last = l.CPoint( -2 );
 
-    if( aEndItem && m_currentNet >= 0 && m_currentNet == aEndItem->Net() )
+    if( aEndItem && m_currentNet && m_currentNet == aEndItem->Net() )
         realEnd = true;
 
     if( aForceFinish )
@@ -1652,14 +1722,16 @@ bool LINE_PLACER::FixRoute( const VECTOR2I& aP, ITEM* aEndItem, bool aForceFinis
 }
 
 
-bool LINE_PLACER::UnfixRoute()
+std::optional<VECTOR2I> LINE_PLACER::UnfixRoute()
 {
-    FIXED_TAIL::STAGE st;
+    FIXED_TAIL::STAGE       st;
+    std::optional<VECTOR2I> ret;
 
     if ( !m_fixedTail.PopStage( st ) )
-    {
-        return false;
-    }
+        return ret;
+
+    if( m_head.Line().PointCount() )
+        ret = m_head.Line().CPoint( 0 );
 
     m_head.Line().Clear();
     m_tail.Line().Clear();
@@ -1691,7 +1763,7 @@ bool LINE_PLACER::UnfixRoute()
 
     m_lastNode = m_currentNode->Branch();
 
-    return true;
+    return ret;
 }
 
 
@@ -1728,6 +1800,7 @@ void LINE_PLACER::removeLoops( NODE* aNode, LINE& aLatest )
         return;
 
     std::set<LINKED_ITEM *> toErase;
+    aLatest.ClearLinks();
     aNode->Add( aLatest, true );
 
     for( int s = 0; s < aLatest.LinkCount(); s++ )
@@ -1783,7 +1856,7 @@ void LINE_PLACER::simplifyNewLine( NODE* aNode, LINKED_ITEM* aLatest )
     std::set<ITEM*> cleanup;
 
     auto processJoint =
-            [&]( JOINT* aJoint, ITEM* aItem )
+            [&]( const JOINT* aJoint, ITEM* aItem )
             {
                 if( !aJoint || aJoint->IsLineCorner() )
                     return;
@@ -1792,7 +1865,7 @@ void LINE_PLACER::simplifyNewLine( NODE* aNode, LINKED_ITEM* aLatest )
 
                 NODE::ITEM_VECTOR toRemove;
 
-                for( ITEM* neighbor : aJoint->Links() )
+                for( ITEM* neighbor : aJoint->CLinks().CItems() )
                 {
                     if( neighbor == aItem
                         || !neighbor->OfKind( ITEM::SEGMENT_T | ITEM::ARC_T )
@@ -1801,18 +1874,18 @@ void LINE_PLACER::simplifyNewLine( NODE* aNode, LINKED_ITEM* aLatest )
                         continue;
                     }
 
-                    if( static_cast<SEGMENT*>( neighbor )->Width()
-                            != static_cast<SEGMENT*>( aItem )->Width() )
+                    if( static_cast<const SEGMENT*>( neighbor )->Width()
+                            != static_cast<const SEGMENT*>( aItem )->Width() )
                     {
                         continue;
                     }
 
-                    const SEG& testSeg = static_cast<SEGMENT*>( neighbor )->Seg();
+                    const SEG& testSeg = static_cast<const SEGMENT*>( neighbor )->Seg();
 
                     if( refSeg.Contains( testSeg ) )
                     {
-                        JOINT* nA = aNode->FindJoint( neighbor->Anchor( 0 ), neighbor );
-                        JOINT* nB = aNode->FindJoint( neighbor->Anchor( 1 ), neighbor );
+                        const JOINT* nA = aNode->FindJoint( neighbor->Anchor( 0 ), neighbor );
+                        const JOINT* nB = aNode->FindJoint( neighbor->Anchor( 1 ), neighbor );
 
                         if( ( nA == aJoint && nB->LinkCount() == 1 ) ||
                             ( nB == aJoint && nA->LinkCount() == 1 ) )
@@ -1828,8 +1901,8 @@ void LINE_PLACER::simplifyNewLine( NODE* aNode, LINKED_ITEM* aLatest )
         if( !item->OfKind( ITEM::SEGMENT_T ) || cleanup.count( item ) )
             continue;
 
-        JOINT* jA = aNode->FindJoint( item->Anchor( 0 ), item );
-        JOINT* jB = aNode->FindJoint( item->Anchor( 1 ), item );
+        const JOINT* jA = aNode->FindJoint( item->Anchor( 0 ), item );
+        const JOINT* jB = aNode->FindJoint( item->Anchor( 1 ), item );
 
         processJoint( jA, item );
         processJoint( jB, item );
@@ -1840,7 +1913,8 @@ void LINE_PLACER::simplifyNewLine( NODE* aNode, LINKED_ITEM* aLatest )
 
     // And now we can proceed with assembling the final line and optimizing it.
 
-    LINE l = aNode->AssembleLine( aLatest );
+    LINE l_orig = aNode->AssembleLine( aLatest );
+    LINE l( l_orig );
 
     bool optimized = OPTIMIZER::Optimize( &l, OPTIMIZER::MERGE_COLINEAR, aNode );
 
@@ -1850,9 +1924,10 @@ void LINE_PLACER::simplifyNewLine( NODE* aNode, LINKED_ITEM* aLatest )
 
     if( optimized || simplified.PointCount() != l.PointCount() )
     {
-        aNode->Remove( l );
+        aNode->Remove( l_orig );
         l.SetShape( simplified );
         aNode->Add( l );
+        PNS_DBG( Dbg(), AddItem, &l, RED, 100000, wxT("simplified"));
     }
 }
 
@@ -1866,8 +1941,8 @@ void LINE_PLACER::UpdateSizes( const SIZES_SETTINGS& aSizes )
         // If the track width continues from an existing track, we don't want to change the width.
         // Disallow changing width after the first segment has been fixed because we don't want to
         // go back and rip up tracks or allow DRC errors
-        if( m_sizes.TrackWidthIsExplicit() || ( !HasPlacedAnything()
-                        && ( !m_startItem || m_startItem->Kind() != ITEM::SEGMENT_T ) ) )
+        if( m_sizes.TrackWidthIsExplicit()
+            || ( !HasPlacedAnything() && ( !m_startItem || m_startItem->Kind() != ITEM::SEGMENT_T ) ) )
         {
             m_head.SetWidth( m_sizes.TrackWidth() );
             m_tail.SetWidth( m_sizes.TrackWidth() );
@@ -1900,14 +1975,13 @@ void LINE_PLACER::SetOrthoMode( bool aOrthoMode )
 }
 
 
-bool LINE_PLACER::buildInitialLine( const VECTOR2I& aP, LINE& aHead, bool aForceNoVia )
+bool LINE_PLACER::buildInitialLine( const VECTOR2I& aP, LINE& aHead, PNS::PNS_MODE aMode, bool aForceNoVia )
 {
     SHAPE_LINE_CHAIN l;
     DIRECTION_45 guessedDir = m_mouseTrailTracer.GetPosture( aP );
 
-    PNS_DBG( Dbg(), Message,
-                wxString::Format( "buildInitialLine: m_direction %s, guessedDir %s, tail points %d",
-                m_direction.Format(), guessedDir.Format(), m_tail.PointCount() ) );
+    PNS_DBG( Dbg(), Message, wxString::Format( wxT( "buildInitialLine: m_direction %s, guessedDir %s, tail points %d" ),
+                                               m_direction.Format(), guessedDir.Format(), m_tail.PointCount() ) );
 
     DIRECTION_45::CORNER_MODE cornerMode = Settings().GetCornerMode();
     // Rounded corners don't make sense when routing orthogonally (single track at a time)
@@ -1956,38 +2030,35 @@ bool LINE_PLACER::buildInitialLine( const VECTOR2I& aP, LINE& aHead, bool aForce
     VIA v( makeVia( aP ) );
     v.SetNet( aHead.Net() );
 
-    if( Settings().Mode() == RM_MarkObstacles )
+    if( aMode == RM_MarkObstacles )
     {
         aHead.AppendVia( v );
         return true;
     }
 
-    const int collMask = ( Settings().Mode() == RM_Walkaround ) ? ITEM::ANY_T : ITEM::SOLID_T;
+    const int collMask = ( aMode == RM_Walkaround ) ? ITEM::ANY_T : ITEM::SOLID_T;
     const int iterLimit = Settings().ViaForcePropIterationLimit();
 
     for( int attempt = 0; attempt < 2; attempt++)
     {
         VECTOR2I lead = aP - m_p_start;
-    VECTOR2I force;
+        VECTOR2I force;
 
         if( attempt == 1 && m_last_p_end.has_value() )
-        {
             lead = aP - m_last_p_end.value();
-        }
 
         if( v.PushoutForce( m_currentNode, lead, force, collMask, iterLimit ) )
-    {
-        SHAPE_LINE_CHAIN line =
-                guessedDir.BuildInitialTrace( m_p_start, aP + force, false, cornerMode );
-        aHead = LINE( aHead, line );
+        {
+            SHAPE_LINE_CHAIN line = guessedDir.BuildInitialTrace( m_p_start, aP + force, false, cornerMode );
+            aHead = LINE( aHead, line );
 
-        v.SetPos( v.Pos() + force );
+            v.SetPos( v.Pos() + force );
 
             aHead.AppendVia( v );
 
-	PNS_DBG( Dbg(), AddPoint, v.Pos(), GREEN, 1000000, "via-force-coll-2" );
+            PNS_DBG( Dbg(), AddPoint, v.Pos(), GREEN, 1000000, "via-force-coll-2" );
 
-        return true;
+            return true;
         }
     }
 
@@ -1995,7 +2066,7 @@ bool LINE_PLACER::buildInitialLine( const VECTOR2I& aP, LINE& aHead, bool aForce
 }
 
 
-void LINE_PLACER::GetModifiedNets( std::vector<int>& aNets ) const
+void LINE_PLACER::GetModifiedNets( std::vector<NET_HANDLE>& aNets ) const
 {
     aNets.push_back( m_currentNet );
 }
@@ -2004,6 +2075,7 @@ void LINE_PLACER::GetModifiedNets( std::vector<int>& aNets ) const
 bool LINE_PLACER::AbortPlacement()
 {
     m_world->KillChildren();
+    m_lastNode = nullptr;
     return true;
 }
 
