@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2007-2015 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
- * Copyright (C) 2015-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,7 +32,7 @@
 */
 
 #include <pcb_edit_frame.h>
-#include <confirm.h>            // DisplayError()
+#include <confirm.h>            // DisplayErrorMessage()
 #include <gestfich.h>           // EDA_FileSelector()
 #include <locale_io.h>
 #include <macros.h>
@@ -44,7 +44,7 @@
 #include <board.h>
 #include <board_design_settings.h>
 #include <footprint.h>
-#include <fp_shape.h>
+#include <pcb_shape.h>
 #include <pcb_track.h>
 #include <pad.h>
 #include <zone.h>
@@ -69,14 +69,6 @@ using namespace DSN;
 // I am guessing non convex polygons with holes linked could create issues with any Router.
 #define EXPORT_CUSTOM_PADS_CONVEX_HULL
 
-// Add .1 mil to the requested clearances as a safety margin.  There has been disagreement about
-// interpretation of clearance in the past between KiCad and Freerouter, so keep this safetyMargin
-// until the disagreement is resolved and stable.  Freerouter seems to be moving (protected)
-// traces upon loading the DSN file, and even though it seems to sometimes add its own 0.1 to the
-// clearances, I believe this is happening after the load process (and moving traces) so I am of
-// the opinion this is still needed.
-static const double safetyMargin = 0.1;
-
 
 bool PCB_EDIT_FRAME::ExportSpecctraFile( const wxString& aFullFilename )
 {
@@ -87,7 +79,7 @@ bool PCB_EDIT_FRAME::ExportSpecctraFile( const wxString& aFullFilename )
 
     try
     {
-        ExportBoardToSpecctraFile( GetBoard(), aFullFilename );
+        DSN::ExportBoardToSpecctraFile( GetBoard(), aFullFilename );
     }
     catch( const IO_ERROR& ioe )
     {
@@ -110,6 +102,9 @@ bool PCB_EDIT_FRAME::ExportSpecctraFile( const wxString& aFullFilename )
     return ok;
 }
 
+namespace DSN
+{
+
 
 void ExportBoardToSpecctraFile( BOARD* aBoard, const wxString& aFullFilename )
 {
@@ -146,8 +141,6 @@ void ExportBoardToSpecctraFile( BOARD* aBoard, const wxString& aFullFilename )
 }
 
 
-namespace DSN {
-
 // "specctra reported units" are what we tell the external router that our exported lengths are in
 
 /**
@@ -197,14 +190,23 @@ static POINT mapPt( const VECTOR2I& pt )
 }
 
 
+static POINT mapPt( const VECTOR2I& pt, FOOTPRINT* aFootprint )
+{
+    VECTOR2I fpRelative = pt - aFootprint->GetPosition();
+    RotatePoint( fpRelative, -aFootprint->GetOrientation() );
+    return mapPt( fpRelative );
+}
+
+
 /**
  * Decide if the pad is a copper-less through hole which needs to be made into a round keepout.
  */
 static bool isRoundKeepout( PAD* aPad )
 {
-    if( aPad->GetShape() == PAD_SHAPE::CIRCLE )
+    // TODO(JE) padstacks
+    if( aPad->GetShape( ::PADSTACK::ALL_LAYERS ) == PAD_SHAPE::CIRCLE )
     {
-        if( aPad->GetDrillSize().x >= aPad->GetSize().x )
+        if( aPad->GetDrillSize().x >= aPad->GetSize( ::PADSTACK::ALL_LAYERS ).x )
             return true;
 
         if( !( aPad->GetLayerSet() & LSET::AllCuMask() ).any() )
@@ -224,7 +226,7 @@ static PATH* makePath( const POINT& aStart, const POINT& aEnd, const std::string
 
     path->AppendPoint( aStart );
     path->AppendPoint( aEnd );
-    path->SetLayerId( aLayerName.c_str() );
+    path->SetLayerId( aLayerName );
     return path;
 }
 
@@ -237,7 +239,6 @@ bool SPECCTRA_DB::BuiltBoardOutlines( BOARD* aBoard  )
 
 PADSTACK* SPECCTRA_DB::makePADSTACK( BOARD* aBoard, PAD* aPad )
 {
-    char        name[256];                  // padstack name builder
     std::string uniqifier;
 
     // caller must do these checks before calling here.
@@ -245,27 +246,25 @@ PADSTACK* SPECCTRA_DB::makePADSTACK( BOARD* aBoard, PAD* aPad )
 
     PADSTACK*   padstack = new PADSTACK();
 
-    int         reportedLayers = 0;         // how many in reported padstack
-    const char* layerName[MAX_CU_LAYERS];
-
     uniqifier = '[';
 
-    static const LSET all_cu = LSET::AllCuMask();
+    const int   copperCount = aBoard->GetCopperLayerCount();
+    static const LSET all_cu = LSET::AllCuMask( copperCount );
+    int         reportedLayers = 0;
+    std::vector<std::string> layerName( copperCount );
 
     bool onAllCopperLayers = ( (aPad->GetLayerSet() & all_cu) == all_cu );
 
     if( onAllCopperLayers )
         uniqifier += 'A'; // A for all layers
 
-    const int copperCount = aBoard->GetCopperLayerCount();
-
     for( int layer=0; layer<copperCount; ++layer )
     {
         PCB_LAYER_ID kilayer = m_pcbLayer2kicad[layer];
 
         if( onAllCopperLayers || aPad->IsOnLayer( kilayer ) )
         {
-            layerName[reportedLayers++] = m_layerIds[layer].c_str();
+            layerName[reportedLayers++] = m_layerIds[layer];
 
             if( !onAllCopperLayers )
             {
@@ -283,25 +282,26 @@ PADSTACK* SPECCTRA_DB::makePADSTACK( BOARD* aBoard, PAD* aPad )
 
     POINT   dsnOffset;
 
-    if( aPad->GetOffset().x || aPad->GetOffset().y )
+    // TODO(JE) padstacks
+    const VECTOR2I& padSize = aPad->GetSize( ::PADSTACK::ALL_LAYERS );
+    const VECTOR2I& offset = aPad->GetOffset( ::PADSTACK::ALL_LAYERS );
+
+    if( offset.x || offset.y )
     {
-        char offsetTxt[64];
-
-        VECTOR2I offset( aPad->GetOffset().x, aPad->GetOffset().y );
-
         dsnOffset = mapPt( offset );
-
-        // using () would cause padstack name to be quoted, and {} locks freerouter, so use [].
-        sprintf( offsetTxt, "[%.6g,%.6g]", dsnOffset.x, dsnOffset.y );
-
-        uniqifier += offsetTxt;
+        // Using () would cause padstack name to be quoted, and {} locks freerouter, so use [].
+        std::ostringstream oss;
+        oss.imbue( std::locale::classic() );
+        oss << std::fixed << std::setprecision( 6 )
+            << '[' << dsnOffset.x << ',' << dsnOffset.y << ']';
+        uniqifier += oss.str();
     }
 
-    switch( aPad->GetShape() )
+    switch( aPad->GetShape( ::PADSTACK::ALL_LAYERS ) )
     {
     case PAD_SHAPE::CIRCLE:
     {
-        double diameter = scale( aPad->GetSize().x );
+        double diameter = scale( padSize.x );
 
         for( int ndx = 0; ndx < reportedLayers; ++ndx )
         {
@@ -318,19 +318,18 @@ PADSTACK* SPECCTRA_DB::makePADSTACK( BOARD* aBoard, PAD* aPad )
             circle->SetVertex( dsnOffset );
         }
 
-        snprintf( name, sizeof(name), "Round%sPad_%.6g_um",
-                  uniqifier.c_str(), IU2um( aPad->GetSize().x ) );
+        std::ostringstream oss;
+        oss << "Round" << uniqifier << "Pad_" << std::fixed << std::setprecision(6)
+            << IU2um( padSize.x ) << "_um";
 
-        name[ sizeof(name) - 1 ] = 0;
-
-        padstack->SetPadstackId( name );
+        padstack->SetPadstackId( oss.str().c_str() );
         break;
     }
 
-    case PAD_SHAPE::RECT:
+    case PAD_SHAPE::RECTANGLE:
     {
-        double dx = scale( aPad->GetSize().x ) / 2.0;
-        double dy = scale( aPad->GetSize().y ) / 2.0;
+        double dx = scale( padSize.x ) / 2.0;
+        double dy = scale( padSize.y ) / 2.0;
 
         POINT lowerLeft( -dx, -dy );
         POINT upperRight( dx, dy );
@@ -352,19 +351,18 @@ PADSTACK* SPECCTRA_DB::makePADSTACK( BOARD* aBoard, PAD* aPad )
             rect->SetCorners( lowerLeft, upperRight );
         }
 
-        snprintf( name, sizeof( name ), "Rect%sPad_%.6gx%.6g_um", uniqifier.c_str(),
-                  IU2um( aPad->GetSize().x ), IU2um( aPad->GetSize().y ) );
+        std::ostringstream oss;
+        oss << "Rect" << uniqifier << "Pad_" << std::fixed << std::setprecision(6)
+            << IU2um( padSize.x ) << "x" << IU2um( padSize.y ) << "_um";
 
-        name[sizeof( name ) - 1] = 0;
-
-        padstack->SetPadstackId( name );
+        padstack->SetPadstackId( oss.str().c_str() );
         break;
     }
 
     case PAD_SHAPE::OVAL:
     {
-        double dx = scale( aPad->GetSize().x ) / 2.0;
-        double dy = scale( aPad->GetSize().y ) / 2.0;
+        double dx = scale( padSize.x ) / 2.0;
+        double dy = scale( padSize.y ) / 2.0;
         double dr = dx - dy;
         double radius;
         POINT  pstart;
@@ -403,21 +401,23 @@ PADSTACK* SPECCTRA_DB::makePADSTACK( BOARD* aBoard, PAD* aPad )
             path->aperture_width = 2.0 * radius;
         }
 
-        snprintf( name, sizeof( name ), "Oval%sPad_%.6gx%.6g_um", uniqifier.c_str(),
-                  IU2um( aPad->GetSize().x ), IU2um( aPad->GetSize().y ) );
-        name[sizeof( name ) - 1] = 0;
+        std::ostringstream oss;
+        oss << "Oval" << uniqifier << "Pad_" << std::fixed << std::setprecision(6)
+            << IU2um( padSize.x ) << "x" << IU2um( padSize.y ) << "_um";
 
-        padstack->SetPadstackId( name );
+        padstack->SetPadstackId( oss.str().c_str() );
         break;
     }
 
     case PAD_SHAPE::TRAPEZOID:
     {
-        double dx = scale( aPad->GetSize().x ) / 2.0;
-        double dy = scale( aPad->GetSize().y ) / 2.0;
+        double dx = scale( padSize.x ) / 2.0;
+        double dy = scale( padSize.y ) / 2.0;
 
-        double ddx = scale( aPad->GetDelta().x ) / 2.0;
-        double ddy = scale( aPad->GetDelta().y ) / 2.0;
+        const VECTOR2I& delta = aPad->GetDelta( ::PADSTACK::ALL_LAYERS );
+
+        double ddx = scale( delta.x ) / 2.0;
+        double ddy = scale( delta.y ) / 2.0;
 
         // see class_pad_draw_functions.cpp which draws the trapezoid pad
         POINT lowerLeft( -dx - ddy, -dy - ddx );
@@ -450,13 +450,13 @@ PADSTACK* SPECCTRA_DB::makePADSTACK( BOARD* aBoard, PAD* aPad )
         }
 
         // this string _must_ be unique for a given physical shape
-        snprintf( name, sizeof( name ), "Trapz%sPad_%.6gx%.6g_%c%.6gx%c%.6g_um", uniqifier.c_str(),
-                  IU2um( aPad->GetSize().x ), IU2um( aPad->GetSize().y ),
-                  aPad->GetDelta().x < 0 ? 'n' : 'p', std::abs( IU2um( aPad->GetDelta().x ) ),
-                  aPad->GetDelta().y < 0 ? 'n' : 'p', std::abs( IU2um( aPad->GetDelta().y ) ) );
-        name[sizeof( name ) - 1] = 0;
+        std::ostringstream oss;
+        oss << "Trapz" << uniqifier << "Pad_" << std::fixed << std::setprecision(6)
+            << IU2um( padSize.x ) << "x" << IU2um( padSize.y ) << "_"
+            << ( delta.x < 0 ? "n" : "p") << std::abs( IU2um( delta.x ) ) << "x"
+            << ( delta.y < 0 ? "n" : "p") << std::abs( IU2um( delta.y ) ) << "_um";
 
-        padstack->SetPadstackId( name );
+        padstack->SetPadstackId( oss.str().c_str() );
         break;
     }
 
@@ -465,7 +465,7 @@ PADSTACK* SPECCTRA_DB::makePADSTACK( BOARD* aBoard, PAD* aPad )
     {
         // Export the shape as as polygon, round rect does not exist as primitive
         const int      circleToSegmentsCount = 36;
-        int            rradius = aPad->GetRoundRectCornerRadius();
+        int            rradius = aPad->GetRoundRectCornerRadius( ::PADSTACK::ALL_LAYERS );
         SHAPE_POLY_SET cornerBuffer;
 
         // Use a slightly bigger shape because the round corners are approximated by
@@ -478,17 +478,16 @@ PADSTACK* SPECCTRA_DB::makePADSTACK( BOARD* aBoard, PAD* aPad )
          */
         double correctionFactor = cos( M_PI / (double) circleToSegmentsCount );
         int    extra_clearance = KiROUND( rradius * ( 1.0 - correctionFactor ) );
-        VECTOR2I psize = aPad->GetSize();
+        VECTOR2I psize = padSize;
         psize.x += extra_clearance * 2;
         psize.y += extra_clearance * 2;
         rradius += extra_clearance;
-        bool doChamfer = aPad->GetShape() == PAD_SHAPE::CHAMFERED_RECT;
+        bool doChamfer = aPad->GetShape( ::PADSTACK::ALL_LAYERS ) == PAD_SHAPE::CHAMFERED_RECT;
 
         TransformRoundChamferedRectToPolygon( cornerBuffer, VECTOR2I( 0, 0 ), psize, ANGLE_0,
-                                              rradius, aPad->GetChamferRectRatio(),
-                                              doChamfer ? aPad->GetChamferPositions() : 0,
-                                              0, aBoard->GetDesignSettings().m_MaxError,
-                                              ERROR_INSIDE );
+                rradius, aPad->GetChamferRectRatio( ::PADSTACK::ALL_LAYERS ),
+                doChamfer ? aPad->GetChamferPositions( ::PADSTACK::ALL_LAYERS ) : 0,
+                0, aBoard->GetDesignSettings().m_MaxError, ERROR_INSIDE );
 
         SHAPE_LINE_CHAIN& polygonal_shape = cornerBuffer.Outline( 0 );
 
@@ -523,14 +522,17 @@ PADSTACK* SPECCTRA_DB::makePADSTACK( BOARD* aBoard, PAD* aPad )
         }
 
         // this string _must_ be unique for a given physical shape
-        snprintf( name, sizeof( name ), "RoundRect%sPad_%.6gx%.6g_%.6g_um_%f_%X", uniqifier.c_str(),
-                  IU2um( aPad->GetSize().x ), IU2um( aPad->GetSize().y ), IU2um( rradius ),
-                  doChamfer ? aPad->GetChamferRectRatio() : 0.0,
-                  doChamfer ? aPad->GetChamferPositions() : 0 );
+        std::ostringstream oss;
+        oss << "RoundRect" << uniqifier << "Pad_"
+            << std::fixed << std::setprecision(6)
+            << IU2um( padSize.x ) << 'x'
+            << IU2um( padSize.y ) << '_'
+            << IU2um( rradius ) << "_um_"
+            << ( doChamfer ? aPad->GetChamferRectRatio( ::PADSTACK::ALL_LAYERS ) : 0.0 ) << '_'
+            << std::hex << std::uppercase
+            << ( doChamfer ? aPad->GetChamferPositions( ::PADSTACK::ALL_LAYERS ) : 0 );
 
-        name[sizeof( name ) - 1] = 0;
-
-        padstack->SetPadstackId( name );
+        padstack->SetPadstackId( oss.str().c_str() );
         break;
     }
 
@@ -538,7 +540,7 @@ PADSTACK* SPECCTRA_DB::makePADSTACK( BOARD* aBoard, PAD* aPad )
     {
         std::vector<VECTOR2I> polygonal_shape;
         SHAPE_POLY_SET       pad_shape;
-        aPad->MergePrimitivesAsPolygon( &pad_shape );
+        aPad->MergePrimitivesAsPolygon( ::PADSTACK::ALL_LAYERS, &pad_shape );
 
 #ifdef EXPORT_CUSTOM_PADS_CONVEX_HULL
         BuildConvexHull( polygonal_shape, pad_shape );
@@ -575,15 +577,18 @@ PADSTACK* SPECCTRA_DB::makePADSTACK( BOARD* aBoard, PAD* aPad )
         }
 
         // this string _must_ be unique for a given physical shape, so try to make it unique
-        MD5_HASH hash = pad_shape.GetHash();
-        BOX2I    rect = aPad->GetBoundingBox();
-        snprintf( name, sizeof( name ), "Cust%sPad_%.6gx%.6g_%.6gx_%.6g_%d_um_%s",
-                  uniqifier.c_str(), IU2um( aPad->GetSize().x ), IU2um( aPad->GetSize().y ),
-                  IU2um( rect.GetWidth() ), IU2um( rect.GetHeight() ), (int) polygonal_shape.size(),
-                  hash.Format( true ).c_str() );
-        name[sizeof( name ) - 1] = 0;
+        const HASH_128 hash = pad_shape.GetHash();
+        const BOX2I rect = aPad->GetBoundingBox();
 
-        padstack->SetPadstackId( name );
+        std::ostringstream oss;
+        oss << "Cust" << uniqifier << "Pad_"
+            << std::fixed << std::setprecision(6)
+            << IU2um( padSize.x ) << 'x' << IU2um( padSize.y ) << '_'
+            << IU2um( rect.GetWidth() ) << 'x' << IU2um( rect.GetHeight() ) << '_'
+            << polygonal_shape.size() << "_um_"
+            << hash.ToString();
+
+        padstack->SetPadstackId( oss.str().c_str() );
         break;
     }
     }
@@ -619,13 +624,13 @@ IMAGE* SPECCTRA_DB::makeIMAGE( BOARD* aBoard, FOOTPRINT* aFootprint )
         if( isRoundKeepout( pad ) )
         {
             double  diameter = scale( pad->GetDrillSize().x );
-            POINT   vertex   = mapPt( pad->GetPos0() );
+            POINT   vertex   = mapPt( pad->GetFPRelativePosition() );
 
-            diameter += aBoard->GetDesignSettings().m_HoleClearance * 2;
+            diameter += scale( aBoard->GetDesignSettings().m_HoleClearance * 2 );
 
             int layerCount = aBoard->GetCopperLayerCount();
 
-            for( int layer=0;  layer<layerCount;  ++layer )
+            for( int layer=0; layer<layerCount; ++layer )
             {
                 KEEPOUT* keepout = new KEEPOUT( image, T_keepout );
 
@@ -637,7 +642,7 @@ IMAGE* SPECCTRA_DB::makeIMAGE( BOARD* aBoard, FOOTPRINT* aFootprint )
 
                 circle->SetDiameter( diameter );
                 circle->SetVertex( vertex );
-                circle->SetLayerId( m_layerIds[layer].c_str() );
+                circle->SetLayerId( m_layerIds[layer] );
             }
         }
         else        // else if() could there be a square keepout here?
@@ -674,12 +679,10 @@ IMAGE* SPECCTRA_DB::makeIMAGE( BOARD* aBoard, FOOTPRINT* aFootprint )
             }
             else    // pad name is a duplicate within this footprint
             {
-                char    buf[32];
                 int     duplicates = ++pinmap[ padNumber ];
 
-                sprintf( buf, "@%d", duplicates );
-
-                pin->m_pin_id += buf;      // append "@1" or "@2", etc. to pin name
+                pin->m_pin_id +=
+                        "@" + std::to_string( duplicates ); // append "@1" or "@2", etc. to pin name
             }
 
             pin->m_kiNetCode = pad->GetNetCode();
@@ -691,20 +694,20 @@ IMAGE* SPECCTRA_DB::makeIMAGE( BOARD* aBoard, FOOTPRINT* aFootprint )
             EDA_ANGLE angle = pad->GetOrientation() - aFootprint->GetOrientation();
             pin->SetRotation( angle.Normalize().AsDegrees() );
 
-            wxPoint pos( pad->GetPos0() );
+            VECTOR2I pos( pad->GetFPRelativePosition() );
 
             pin->SetVertex( mapPt( pos ) );
         }
     }
 
-    // get all the FOOTPRINT's FP_SHAPEs and convert those to DSN outlines.
-    fpItems.Collect( aFootprint, { PCB_FP_SHAPE_T } );
+    // get all the FOOTPRINT's SHAPEs and convert those to DSN outlines.
+    fpItems.Collect( aFootprint, { PCB_SHAPE_T } );
 
     for( int i = 0; i < fpItems.GetCount(); ++i )
     {
-        FP_SHAPE* graphic = (FP_SHAPE*) fpItems[i];
-        SHAPE*    outline;
-        PATH*     path;
+        PCB_SHAPE* graphic = static_cast<PCB_SHAPE*>( fpItems[i] );
+        SHAPE*     outline;
+        PATH*      path;
 
         switch( graphic->GetShape() )
         {
@@ -717,8 +720,8 @@ IMAGE* SPECCTRA_DB::makeIMAGE( BOARD* aBoard, FOOTPRINT* aFootprint )
             outline->SetShape( path );
             path->SetAperture( scale( graphic->GetWidth() ) );
             path->SetLayerId( "signal" );
-            path->AppendPoint( mapPt( graphic->GetStart0() ) );
-            path->AppendPoint( mapPt( graphic->GetEnd0() ) );
+            path->AppendPoint( mapPt( graphic->GetStart(), aFootprint ) );
+            path->AppendPoint( mapPt( graphic->GetEnd(), aFootprint ) );
             break;
 
         case SHAPE_T::CIRCLE:
@@ -735,7 +738,7 @@ IMAGE* SPECCTRA_DB::makeIMAGE( BOARD* aBoard, FOOTPRINT* aFootprint )
             path->SetLayerId( "signal" );
 
             double radius = graphic->GetRadius();
-            VECTOR2I circle_centre = graphic->GetStart0();
+            VECTOR2I circle_centre = graphic->GetStart();
 
             SHAPE_LINE_CHAIN polyline;
             ConvertArcToPolyline( polyline, VECTOR2I( circle_centre ), radius, ANGLE_0, ANGLE_360,
@@ -744,13 +747,13 @@ IMAGE* SPECCTRA_DB::makeIMAGE( BOARD* aBoard, FOOTPRINT* aFootprint )
             for( int ii = 0; ii < polyline.PointCount(); ++ii )
             {
                 VECTOR2I corner( polyline.CPoint( ii ).x, polyline.CPoint( ii ).y );
-                path->AppendPoint( mapPt( corner ) );
+                path->AppendPoint( mapPt( corner, aFootprint ) );
             }
 
             break;
         }
 
-        case SHAPE_T::RECT:
+        case SHAPE_T::RECTANGLE:
         {
             outline = new SHAPE( image, T_outline );
 
@@ -760,17 +763,17 @@ IMAGE* SPECCTRA_DB::makeIMAGE( BOARD* aBoard, FOOTPRINT* aFootprint )
             outline->SetShape( path );
             path->SetAperture( scale( graphic->GetWidth() ) );
             path->SetLayerId( "signal" );
-            VECTOR2I corner = graphic->GetStart0();
-            path->AppendPoint( mapPt( corner ) );
+            VECTOR2I corner = graphic->GetStart();
+            path->AppendPoint( mapPt( corner, aFootprint ) );
 
-            corner.x = graphic->GetEnd0().x;
-            path->AppendPoint( mapPt( corner ) );
+            corner.x = graphic->GetEnd().x;
+            path->AppendPoint( mapPt( corner, aFootprint ) );
 
-            corner.y = graphic->GetEnd0().y;
-            path->AppendPoint( mapPt( corner ) );
+            corner.y = graphic->GetEnd().y;
+            path->AppendPoint( mapPt( corner, aFootprint ) );
 
-            corner.x = graphic->GetStart0().x;
-            path->AppendPoint( mapPt( corner ) );
+            corner.x = graphic->GetStart().x;
+            path->AppendPoint( mapPt( corner, aFootprint ) );
             break;
         }
 
@@ -788,7 +791,7 @@ IMAGE* SPECCTRA_DB::makeIMAGE( BOARD* aBoard, FOOTPRINT* aFootprint )
             path->SetAperture( 0 );//scale( graphic->GetWidth() ) );
             path->SetLayerId( "signal" );
 
-            VECTOR2I  arc_centre = graphic->GetCenter0();
+            VECTOR2I  arc_centre = graphic->GetCenter();
             double    radius = graphic->GetRadius() + graphic->GetWidth()/2;
             EDA_ANGLE arcAngle = graphic->GetArcAngle();
 
@@ -840,13 +843,13 @@ IMAGE* SPECCTRA_DB::makeIMAGE( BOARD* aBoard, FOOTPRINT* aFootprint )
             TransformCircleToPolygon( polyBuffer, graphic->GetEnd() - move,
                                       graphic->GetWidth() / 2, ARC_HIGH_DEF, ERROR_INSIDE );
 
-            polyBuffer.Simplify( SHAPE_POLY_SET::PM_FAST );
+            polyBuffer.Simplify();
             SHAPE_LINE_CHAIN& poly = polyBuffer.Outline( 0 );
 
             for( int ii = 0; ii < poly.PointCount(); ++ii )
             {
                 VECTOR2I corner( poly.CPoint( ii ).x, poly.CPoint( ii ).y );
-                path->AppendPoint( mapPt( corner ) );
+                path->AppendPoint( mapPt( corner, aFootprint ) );
             }
 
             break;
@@ -857,7 +860,7 @@ IMAGE* SPECCTRA_DB::makeIMAGE( BOARD* aBoard, FOOTPRINT* aFootprint )
         }
     }
 
-    for( FP_ZONE* zone : aFootprint->Zones() )
+    for( ZONE* zone : aFootprint->Zones() )
     {
         if( !zone->GetIsRuleArea() )
             continue;
@@ -886,16 +889,10 @@ IMAGE* SPECCTRA_DB::makeIMAGE( BOARD* aBoard, FOOTPRINT* aFootprint )
 
         // Now, build keepout polygon on each copper layer where the zone
         // keepout is living (keepout zones can live on many copper layers)
-        const int copperCount = aBoard->GetCopperLayerCount();
+        LSET layerset = aBoard->GetEnabledLayers() & zone->GetLayerSet() & LSET::AllCuMask();
 
-        for( int layer = 0; layer < copperCount; layer++ )
+        for( PCB_LAYER_ID layer : layerset.CuStack() )
         {
-            if( layer == copperCount-1 )
-                layer = B_Cu;
-
-            if( !zone->IsOnLayer( PCB_LAYER_ID( layer ) ) )
-                continue;
-
             KEEPOUT* keepout = new KEEPOUT( m_pcb->m_structure, keepout_type );
             image->m_keepouts.push_back( keepout );
 
@@ -997,7 +994,7 @@ PADSTACK* SPECCTRA_DB::makeVia( int aCopperDiameter, int aDrillDiameter,
         shape->SetShape( circle );
 
         circle->SetDiameter( dsnDiameter );
-        circle->SetLayerId( m_layerIds[layer].c_str() );
+        circle->SetLayerId( m_layerIds[layer] );
     }
 
     snprintf( name, sizeof( name ), "Via[%d-%d]_%.6g:%.6g_um",
@@ -1025,7 +1022,9 @@ PADSTACK* SPECCTRA_DB::makeVia( const PCB_VIA* aVia )
     if( topLayer > botLayer )
         std::swap( topLayer, botLayer );
 
-    return makeVia( aVia->GetWidth(), aVia->GetDrillValue(), topLayer, botLayer );
+    // TODO(JE) padstacks
+    return makeVia( aVia->GetWidth( ::PADSTACK::ALL_LAYERS ), aVia->GetDrillValue(),
+                    topLayer, botLayer );
 }
 
 
@@ -1041,12 +1040,12 @@ void SPECCTRA_DB::fillBOUNDARY( BOARD* aBoard, BOUNDARY* boundary )
 
         for( int ii = 0; ii < outline.PointCount(); ii++ )
         {
-            wxPoint pos( outline.CPoint( ii ).x, outline.CPoint( ii ).y );
+            VECTOR2I pos( outline.CPoint( ii ).x, outline.CPoint( ii ).y );
             path->AppendPoint( mapPt( pos ) );
         }
 
         // Close polygon:
-        wxPoint pos0( outline.CPoint( 0 ).x, outline.CPoint( 0 ).y );
+        VECTOR2I pos0( outline.CPoint( 0 ).x, outline.CPoint( 0 ).y );
         path->AppendPoint( mapPt( pos0 ) );
 
         // Generate holes as keepout:
@@ -1064,12 +1063,12 @@ void SPECCTRA_DB::fillBOUNDARY( BOARD* aBoard, BOUNDARY* boundary )
 
             for( int jj = 0; jj < hole.PointCount(); jj++ )
             {
-                wxPoint pos( hole.CPoint( jj ).x, hole.CPoint( jj ).y );
+                VECTOR2I pos( hole.CPoint( jj ).x, hole.CPoint( jj ).y );
                 poly_ko->AppendPoint( mapPt( pos ) );
             }
 
             // Close polygon:
-            wxPoint pos( hole.CPoint( 0 ).x, hole.CPoint( 0 ).y );
+            VECTOR2I pos( hole.CPoint( 0 ).x, hole.CPoint( 0 ).y );
             poly_ko->AppendPoint( mapPt( pos ) );
         }
     }
@@ -1149,9 +1148,7 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
             layer->properties.push_back( PROPERTY() );
             PROPERTY*   property = &layer->properties.back();
             property->name = "index";
-            char        temp[32];
-            sprintf( temp, "%d", pcbNdx );
-            property->value = temp;
+            property->value = std::to_string( pcbNdx );
         }
     }
 
@@ -1181,31 +1178,16 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
     //-----<rules>--------------------------------------------------------
     {
         char      rule[80];
-        int       defaultTrackWidth = netSettings->m_DefaultNetClass->GetTrackWidth();
-        int       defaultClearance  = netSettings->m_DefaultNetClass->GetClearance();
+        int       defaultTrackWidth = netSettings->GetDefaultNetclass()->GetTrackWidth();
+        int       defaultClearance = netSettings->GetDefaultNetclass()->GetClearance();
         double    clearance         = scale( defaultClearance );
 
         STRINGS&  rules = m_pcb->m_structure->m_rules->m_rules;
 
-        sprintf( rule, "(width %.6g)", scale( defaultTrackWidth ) );
+        std::snprintf( rule, sizeof( rule ), "(width %.6g)", scale( defaultTrackWidth ) );
         rules.push_back( rule );
 
-        sprintf( rule, "(clearance %.6g)", clearance + safetyMargin );
-        rules.push_back( rule );
-
-        // On a high density board (4 mil tracks, 4 mil spacing) a typical solder mask clearance
-        // will be 2-3 mils.  This exposes 2 to 3 mils of bare board around each pad, and would
-        // leave only 1 to 2 mils of solder mask between the solder mask's boundary and the edge of
-        // any trace within "clearance" of the pad.  So we need at least 2 mils *extra* clearance
-        // for traces which would come near a pad on a different net.  So if the baseline trace to
-        // trace clearance was 4 mils, then the SMD to trace clearance should be at least 6 mils.
-        double default_smd = clearance + safetyMargin;
-
-        if( default_smd <= 6.0 )
-            default_smd = 6.0;
-
-        sprintf( rule, "(clearance %.6g (type default_smd))", default_smd );
-
+        std::snprintf( rule, sizeof( rule ), "(clearance %.6g)", clearance );
         rules.push_back( rule );
 
         // Pad to pad spacing on a single SMT part can be closer than our clearance. We don't want
@@ -1213,7 +1195,7 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
         // clearance to freerouter.
         clearance = scale( defaultClearance ) / 4;
 
-        sprintf( rule, "(clearance %.6g (type smd_smd))", clearance );
+        std::snprintf( rule, sizeof( rule ), "(clearance %.6g (type smd_smd))", clearance );
         rules.push_back( rule );
     }
 
@@ -1228,20 +1210,15 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
                 continue;
 
             // Currently, we export only copper layers
-            if( !  zone->IsOnCopperLayer() )
+            if( ! zone->IsOnCopperLayer() )
                 continue;
 
             // Now, build zone polygon on each copper layer where the zone
             // is living (zones can live on many copper layers)
-            const int copperCount = aBoard->GetCopperLayerCount();
+            LSET layerset = aBoard->GetEnabledLayers() & zone->GetLayerSet() & LSET::AllCuMask();
 
-            for( int layer = 0; layer < copperCount; layer++ )
+            for( PCB_LAYER_ID layer : layerset )
             {
-                if( layer == copperCount-1 )
-                    layer = B_Cu;
-
-                if( !zone->IsOnLayer( PCB_LAYER_ID( layer ) ) )
-                    continue;
                 COPPER_PLANE*   plane = new COPPER_PLANE( m_pcb->m_structure );
 
                 m_pcb->m_structure->m_planes.push_back( plane );
@@ -1253,14 +1230,12 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
 
                 if( plane->m_name.size() == 0 )
                 {
-                    char name[32];
-
                     // This is one of those no connection zones, netcode=0, and it has no name.
                     // Create a unique, bogus netname.
                     NET* no_net = new NET( m_pcb->m_network );
 
-                    sprintf( name, "@:no_net_%d", netlessZones++ );
-                    no_net->m_net_id = name;
+
+                    no_net->m_net_id = "@:no_net_" + std::to_string( netlessZones++ );
 
                     // add the bogus net name to network->nets.
                     m_pcb->m_network->m_nets.push_back( no_net );
@@ -1273,12 +1248,12 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
 
                 // Handle the main outlines
                 SHAPE_POLY_SET::ITERATOR iterator;
-                wxPoint startpoint;
+                VECTOR2I                 startpoint;
                 bool is_first_point = true;
 
                 for( iterator = zone->IterateWithHoles(); iterator; iterator++ )
                 {
-                    wxPoint point( iterator->x, iterator->y );
+                    VECTOR2I point( iterator->x, iterator->y );
 
                     if( is_first_point )
                     {
@@ -1323,7 +1298,7 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
                     wxASSERT( window );
                     wxASSERT( cutout );
 
-                    wxPoint point(iterator->x, iterator->y );
+                    VECTOR2I point( iterator->x, iterator->y );
 
                     if( is_first_point )
                     {
@@ -1364,16 +1339,10 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
 
             // Now, build keepout polygon on each copper layer where the zone
             // keepout is living (keepout zones can live on many copper layers)
-            const int copperCount = aBoard->GetCopperLayerCount();
+            LSET layerset = aBoard->GetEnabledLayers() & zone->GetLayerSet() & LSET::AllCuMask();
 
-            for( int layer = 0; layer < copperCount; layer++ )
+            for( PCB_LAYER_ID layer : layerset )
             {
-                if( layer == copperCount - 1 )
-                    layer = B_Cu;
-
-                if( !zone->IsOnLayer( PCB_LAYER_ID( layer ) ) )
-                    continue;
-
                 KEEPOUT*   keepout = new KEEPOUT( m_pcb->m_structure, keepout_type );
                 m_pcb->m_structure->m_keepouts.push_back( keepout );
 
@@ -1384,12 +1353,12 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
 
                 // Handle the main outlines
                 SHAPE_POLY_SET::ITERATOR iterator;
-                bool is_first_point = true;
-                wxPoint startpoint;
+                bool                     is_first_point = true;
+                VECTOR2I                 startpoint;
 
                 for( iterator = zone->IterateWithHoles(); iterator; iterator++ )
                 {
-                    wxPoint point( iterator->x, iterator->y );
+                    VECTOR2I point( iterator->x, iterator->y );
 
                     if( is_first_point )
                     {
@@ -1431,7 +1400,7 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
                     wxASSERT( window );
                     wxASSERT( cutout );
 
-                    wxPoint point(iterator->x, iterator->y );
+                    VECTOR2I point( iterator->x, iterator->y );
 
                     if( is_first_point )
                     {
@@ -1451,10 +1420,10 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
 
     //-----<build the images, components, and netlist>-----------------------
     {
-        PIN_REF       empty( m_pcb->m_network );
-        std::string   componentId;
-        int           highestNetCode = 0;
-        NETINFO_LIST& netInfo = aBoard->GetNetInfo();
+        PIN_REF             empty( m_pcb->m_network );
+        std::string         componentId;
+        int                 highestNetCode = 0;
+        const NETINFO_LIST& netInfo = aBoard->GetNetInfo();
 
         // find the highest numbered netCode within the board.
         for( NETINFO_LIST::iterator i = netInfo.begin(); i != netInfo.end(); ++i )
@@ -1573,9 +1542,9 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
         // Add the via from the Default netclass first.  The via container
         // in pcb->library preserves the sequence of addition.
 
-        PADSTACK*   via = makeVia( netSettings->m_DefaultNetClass->GetViaDiameter(),
-                                   netSettings->m_DefaultNetClass->GetViaDrill(),
-                                   m_top_via_layer, m_bot_via_layer );
+        PADSTACK* via = makeVia( netSettings->GetDefaultNetclass()->GetViaDiameter(),
+                                 netSettings->GetDefaultNetclass()->GetViaDrill(), m_top_via_layer,
+                                 m_bot_via_layer );
 
         // we AppendVia() this first one, there is no way it can be a duplicate,
         // the pcb->library via container is empty at this point.  After this,
@@ -1587,7 +1556,7 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
         // pcb->library->spareViaIndex = pcb->library->vias.size();
 
         // output the non-Default netclass vias
-        for( const auto& [ name, netclass ] : netSettings->m_NetClasses )
+        for( const auto& [name, netclass] : netSettings->GetNetclasses() )
         {
             via = makeVia( netclass->GetViaDiameter(), netclass->GetViaDrill(),
                            m_top_via_layer, m_bot_via_layer );
@@ -1647,8 +1616,8 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
                 else
                     wire->m_wire_type = T_route;  // could be T_protect
 
-                int kiLayer  = track->GetLayer();
-                int pcbLayer = m_kicadLayer2pcb[kiLayer];
+                PCB_LAYER_ID kiLayer = track->GetLayer();
+                int          pcbLayer = m_kicadLayer2pcb[kiLayer];
 
                 path = new PATH( wire );
                 wire->SetShape( path );
@@ -1719,9 +1688,9 @@ void SPECCTRA_DB::FromBOARD( BOARD* aBoard )
 
     //-----<output NETCLASSs>----------------------------------------------------
 
-    exportNETCLASS( netSettings->m_DefaultNetClass, aBoard );
+    exportNETCLASS( netSettings->GetDefaultNetclass(), aBoard );
 
-    for( const auto& [ name, netclass ] : netSettings->m_NetClasses )
+    for( const auto& [name, netclass] : netSettings->GetNetclasses() )
         exportNETCLASS( netclass, aBoard );
 }
 
@@ -1776,12 +1745,12 @@ void SPECCTRA_DB::exportNETCLASS( const std::shared_ptr<NETCLASS>& aNetClass, BO
 
     // output the track width.
     int trackWidth = aNetClass->GetTrackWidth();
-    sprintf( text, "(width %.6g)", scale( trackWidth ) );
+    std::snprintf( text, sizeof( text ), "(width %.6g)", scale( trackWidth ) );
     clazz->m_rules->m_rules.push_back( text );
 
     // output the clearance.
     int clearance = aNetClass->GetClearance();
-    sprintf( text, "(clearance %.6g)", scale( clearance ) + safetyMargin );
+    std::snprintf( text, sizeof( text ), "(clearance %.6g)", scale( clearance ) );
     clazz->m_rules->m_rules.push_back( text );
 
     if( aNetClass->GetName() == NETCLASS::Default )
@@ -1794,7 +1763,7 @@ void SPECCTRA_DB::exportNETCLASS( const std::shared_ptr<NETCLASS>& aNetClass, BO
     PADSTACK* via = makeVia( aNetClass->GetViaDiameter(), aNetClass->GetViaDrill(),
                              m_top_via_layer, m_bot_via_layer );
 
-    snprintf( text, sizeof(text), "(use_via %s)", via->GetPadstackId().c_str() );
+    snprintf( text, sizeof( text ), "(use_via \"%s\")", via->GetPadstackId().c_str() );
     clazz->m_circuit.push_back( text );
 
     delete via;
@@ -1812,7 +1781,7 @@ void SPECCTRA_DB::FlipFOOTPRINTs( BOARD* aBoard )
 
         if( footprint->GetLayer() == B_Cu )
         {
-            footprint->Flip( footprint->GetPosition(), false );
+            footprint->Flip( footprint->GetPosition(), FLIP_DIRECTION::TOP_BOTTOM );
             footprint->SetFlag( 1 );
         }
     }
@@ -1833,7 +1802,7 @@ void SPECCTRA_DB::RevertFOOTPRINTs( BOARD* aBoard )
     {
         if( footprint->GetFlag() )
         {
-            footprint->Flip( footprint->GetPosition(), false );
+            footprint->Flip( footprint->GetPosition(), FLIP_DIRECTION::TOP_BOTTOM );
             footprint->SetFlag( 0 );
         }
     }
@@ -1841,4 +1810,4 @@ void SPECCTRA_DB::RevertFOOTPRINTs( BOARD* aBoard )
     m_footprintsAreFlipped = false;
 }
 
-}       // namespace DSN
+} // namespace DSN
