@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2013-2017 CERN
- * Copyright (C) 2017-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
@@ -24,44 +24,32 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-#include <advanced_config.h>
+#include <functional>
 #include <limits>
+#include <kiplatform/ui.h>
 #include <board.h>
-#include <board_design_settings.h>
-#include <footprint.h>
-#include <fp_shape.h>
-#include <collectors.h>
+#include <board_commit.h>
+#include <gal/graphics_abstraction_layer.h>
+#include <geometry/geometry_utils.h>
+#include <pad.h>
+#include <pcb_group.h>
+#include <pcb_generator.h>
 #include <pcb_edit_frame.h>
-#include <drawing_sheet/ds_proxy_view_item.h>
-#include <kiway.h>
-#include <pcbnew_settings.h>
 #include <spread_footprints.h>
 #include <tools/pcb_actions.h>
 #include <tools/pcb_selection_tool.h>
 #include <tools/edit_tool.h>
 #include <tools/pcb_grid_helper.h>
 #include <tools/drc_tool.h>
-#include <tools/drawing_tool.h>
 #include <tools/zone_filler_tool.h>
-#include <view/view_controls.h>
-#include <connectivity/connectivity_algo.h>
-#include <connectivity/connectivity_items.h>
-#include <cassert>
-#include <functional>
-#include <wx/hyperlink.h>
 #include <router/router_tool.h>
 #include <dialogs/dialog_move_exact.h>
-#include <dialogs/dialog_unit_entry.h>
-#include <board_commit.h>
-#include <pcb_group.h>
-#include <pcb_target.h>
 #include <zone_filler.h>
 #include <drc/drc_engine.h>
 #include <drc/drc_item.h>
 #include <drc/drc_rule.h>
-#include <pad.h>
-#include <geometry/shape_segment.h>
 #include <drc/drc_interactive_courtyard_clearance.h>
+#include <view/view_controls.h>
 
 
 int EDIT_TOOL::Swap( const TOOL_EVENT& aEvent )
@@ -84,12 +72,8 @@ int EDIT_TOOL::Swap( const TOOL_EVENT& aEvent )
                 {
                     BOARD_ITEM* item = aCollector[i];
 
-                    switch( item->Type() )
-                    {
-                    case PCB_TRACE_T: aCollector.Remove( item ); break;
-
-                    default: break;
-                    }
+                    if( item->Type() == PCB_TRACE_T )
+                        aCollector.Remove( item );
                 }
             },
             true /* prompt user regarding locked items */ );
@@ -97,42 +81,31 @@ int EDIT_TOOL::Swap( const TOOL_EVENT& aEvent )
     if( selection.Size() < 2 )
         return 0;
 
+    BOARD_COMMIT  localCommit( this );
+    BOARD_COMMIT* commit = dynamic_cast<BOARD_COMMIT*>( aEvent.Commit() );
+
+    if( !commit )
+        commit = &localCommit;
+
     std::vector<EDA_ITEM*> sorted = selection.GetItemsSortedBySelectionOrder();
 
-    // When editing footprints, all items have the same parent
-    if( IsFootprintEditor() )
+    // Save items, so changes can be undone
+    for( EDA_ITEM* item : selection )
     {
-        m_commit->Modify( selection.Front() );
-    }
-    else
-    {
-        // Save items, so changes can be undone
-        for( EDA_ITEM* item : selection )
-        {
-            // Don't double move footprint pads, fields, etc.
-            //
-            // For PCB_GROUP_T, the parent is the board.
-            if( item->GetParent() && item->GetParent()->IsSelected() )
-                continue;
-
-            m_commit->Modify( item );
-
-            // If moving a group, record position of all the descendants for undo
-            if( item->Type() == PCB_GROUP_T )
-            {
-                PCB_GROUP* group = static_cast<PCB_GROUP*>( item );
-                group->RunOnDescendants( [&]( BOARD_ITEM* bItem )
-                                         {
-                                             m_commit->Modify( bItem );
-                                         });
-            }
-        }
+        if( !item->IsNew() && !item->IsMoving() )
+            commit->Modify( item );
     }
 
     for( size_t i = 0; i < sorted.size() - 1; i++ )
     {
-        BOARD_ITEM* a = static_cast<BOARD_ITEM*>( sorted[i] );
-        BOARD_ITEM* b = static_cast<BOARD_ITEM*>( sorted[( i + 1 ) % sorted.size()] );
+        EDA_ITEM* edaItemA = sorted[i];
+        EDA_ITEM* edaItemB = sorted[( i + 1 ) % sorted.size()];
+
+        if( !edaItemA->IsBOARD_ITEM() || !edaItemB->IsBOARD_ITEM() )
+            continue;
+
+        BOARD_ITEM* a = static_cast<BOARD_ITEM*>( edaItemA );
+        BOARD_ITEM* b = static_cast<BOARD_ITEM*>( edaItemB );
 
         // Swap X,Y position
         VECTOR2I aPos = a->GetPosition(), bPos = b->GetPosition();
@@ -140,13 +113,6 @@ int EDIT_TOOL::Swap( const TOOL_EVENT& aEvent )
         a->SetPosition( aPos );
         b->SetPosition( bPos );
 
-        // Pads need special handling to keep their offset from their parent
-        if( a->Type() == PCB_PAD_T )
-            static_cast<PAD*>( a )->SetLocalCoord();
-
-        if( b->Type() == PCB_PAD_T )
-            static_cast<PAD*>( b )->SetLocalCoord();
-
         // Handle footprints specially. They can be flipped to the back of the board which
         // requires a special transformation.
         if( a->Type() == PCB_FOOTPRINT_T && b->Type() == PCB_FOOTPRINT_T )
@@ -154,15 +120,18 @@ int EDIT_TOOL::Swap( const TOOL_EVENT& aEvent )
             FOOTPRINT* aFP = static_cast<FOOTPRINT*>( a );
             FOOTPRINT* bFP = static_cast<FOOTPRINT*>( b );
 
+            // Store initial orientation of footprints, before flipping them.
+            EDA_ANGLE aAngle = aFP->GetOrientation();
+            EDA_ANGLE bAngle = bFP->GetOrientation();
+
             // Flip both if needed
             if( aFP->IsFlipped() != bFP->IsFlipped() )
             {
-                aFP->Flip( aPos, false );
-                bFP->Flip( bPos, false );
+                aFP->Flip( aPos, FLIP_DIRECTION::TOP_BOTTOM );
+                bFP->Flip( bPos, FLIP_DIRECTION::TOP_BOTTOM );
             }
 
             // Set orientation
-            EDA_ANGLE aAngle = aFP->GetOrientation(), bAngle = bFP->GetOrientation();
             std::swap( aAngle, bAngle );
             aFP->SetOrientation( aAngle );
             bFP->SetOrientation( bAngle );
@@ -179,8 +148,8 @@ int EDIT_TOOL::Swap( const TOOL_EVENT& aEvent )
         }
     }
 
-    if( !m_dragging )
-        m_commit->Push( _( "Swap" ) );
+    if( !localCommit.Empty() )
+        localCommit.Push( _( "Swap" ) );
 
     m_toolMgr->ProcessEvent( EVENTS::SelectedItemsModified );
 
@@ -190,12 +159,19 @@ int EDIT_TOOL::Swap( const TOOL_EVENT& aEvent )
 
 int EDIT_TOOL::PackAndMoveFootprints( const TOOL_EVENT& aEvent )
 {
+    if( isRouterActive() || m_dragging )
+    {
+        wxBell();
+        return 0;
+    }
+
+    BOARD_COMMIT   commit( this );
     PCB_SELECTION& selection = m_selectionTool->RequestSelection(
             []( const VECTOR2I& aPt, GENERAL_COLLECTOR& aCollector, PCB_SELECTION_TOOL* sTool )
             {
                 sTool->FilterCollectorForMarkers( aCollector );
                 sTool->FilterCollectorForHierarchy( aCollector, true );
-                sTool->FilterCollectorForFreePads( aCollector );
+                sTool->FilterCollectorForFreePads( aCollector, true );
 
                 // Iterate from the back so we don't have to worry about removals.
                 for( int i = aCollector.GetCount() - 1; i >= 0; --i )
@@ -220,25 +196,56 @@ int EDIT_TOOL::PackAndMoveFootprints( const TOOL_EVENT& aEvent )
 
     for( FOOTPRINT* item : footprintsToPack )
     {
-        m_commit->Modify( item );
-        footprintsBbox.Merge( item->GetBoundingBox( false, false ) );
+        commit.Modify( item );
+        item->SetFlags( IS_MOVING );
+        footprintsBbox.Merge( item->GetBoundingBox( false ) );
     }
 
     SpreadFootprints( &footprintsToPack, footprintsBbox.Normalize().GetOrigin(), false );
 
-    return doMoveSelection( aEvent, _( "Pack footprints" ) );
+    if( doMoveSelection( aEvent, &commit, true ) )
+        commit.Push( _( "Pack Footprints" ) );
+    else
+        commit.Revert();
+
+    return 0;
 }
 
 
 int EDIT_TOOL::Move( const TOOL_EVENT& aEvent )
 {
-    if( isRouterActive() )
+    if( isRouterActive() || m_dragging )
     {
         wxBell();
         return 0;
     }
 
-    return doMoveSelection( aEvent, _( "Move" ) );
+    if( BOARD_COMMIT* commit = dynamic_cast<BOARD_COMMIT*>( aEvent.Commit() ) )
+    {
+        wxCHECK( aEvent.SynchronousState(), 0 );
+        aEvent.SynchronousState()->store( STS_RUNNING );
+
+        if( doMoveSelection( aEvent, commit, true ) )
+            aEvent.SynchronousState()->store( STS_FINISHED );
+        else
+            aEvent.SynchronousState()->store( STS_CANCELLED );
+    }
+    else
+    {
+        BOARD_COMMIT localCommit( this );
+
+        if( doMoveSelection( aEvent, &localCommit, false ) )
+            localCommit.Push( _( "Move" ) );
+        else
+            localCommit.Revert();
+    }
+
+    // Notify point editor.  (While doMoveSelection() will re-select the items and post this
+    // event, it's done before the edit flags are cleared in BOARD_COMMIT::Push() so the point
+    // editor doesn't fire up.)
+    m_toolMgr->ProcessEvent( EVENTS::SelectedEvent );
+
+    return 0;
 }
 
 
@@ -247,50 +254,50 @@ VECTOR2I EDIT_TOOL::getSafeMovement( const VECTOR2I& aMovement, const BOX2I& aSo
 {
     typedef std::numeric_limits<int> coord_limits;
 
-    int max = coord_limits::max();
-    int min = -max;
+    static const double max = coord_limits::max() - (int) COORDS_PADDING;
+    static const double min = -max;
 
-    double left = aBBoxOffset.x + aSourceBBox.GetPosition().x;
-    double top = aBBoxOffset.y + aSourceBBox.GetPosition().y;
-
-    double right = left + aSourceBBox.GetSize().x;
-    double bottom = top + aSourceBBox.GetSize().y;
+    BOX2D testBox( aSourceBBox.GetPosition(), aSourceBBox.GetSize() );
+    testBox.Offset( aBBoxOffset );
 
     // Do not restrict movement if bounding box is already out of bounds
-    if( left < min || top < min || right > max || bottom > max )
+    if( testBox.GetLeft() < min || testBox.GetTop() < min || testBox.GetRight() > max
+        || testBox.GetBottom() > max )
+    {
         return aMovement;
+    }
 
-    // Constrain moving bounding box to coordinates limits
-    VECTOR2D tryMovement( aMovement );
-    VECTOR2D bBoxOrigin( aSourceBBox.GetPosition() + aBBoxOffset );
-    VECTOR2D clampedBBoxOrigin = GetClampedCoords( bBoxOrigin + tryMovement, COORDS_PADDING );
+    testBox.Offset( aMovement );
 
-    tryMovement = clampedBBoxOrigin - bBoxOrigin;
+    if( testBox.GetLeft() < min )
+        testBox.Offset( min - testBox.GetLeft(), 0 );
 
-    VECTOR2D bBoxEnd( aSourceBBox.GetEnd() + aBBoxOffset );
-    VECTOR2D clampedBBoxEnd = GetClampedCoords( bBoxEnd + tryMovement, COORDS_PADDING );
+    if( max < testBox.GetRight() )
+        testBox.Offset( -( testBox.GetRight() - max ), 0 );
 
-    tryMovement = clampedBBoxEnd - bBoxEnd;
+    if( testBox.GetTop() < min )
+        testBox.Offset( 0, min - testBox.GetTop() );
 
-    return GetClampedCoords<double, int>( tryMovement );
+    if( max < testBox.GetBottom() )
+        testBox.Offset( 0, -( testBox.GetBottom() - max ) );
+
+    return KiROUND( testBox.GetPosition() - aBBoxOffset - aSourceBBox.GetPosition() );
 }
 
 
-int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommitMessage )
+bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit, bool aAutoStart )
 {
-    bool moveWithReference = aEvent.IsAction( &PCB_ACTIONS::moveWithReference );
-    bool moveIndividually = aEvent.IsAction( &PCB_ACTIONS::moveIndividually );
+    const bool moveWithReference = aEvent.IsAction( &PCB_ACTIONS::moveWithReference );
+    const bool moveIndividually = aEvent.IsAction( &PCB_ACTIONS::moveIndividually );
 
-    PCB_BASE_EDIT_FRAME*  editFrame = getEditFrame<PCB_BASE_EDIT_FRAME>();
-    PCBNEW_SETTINGS*      cfg = editFrame->GetPcbNewSettings();
-    BOARD*                board = editFrame->GetBoard();
-    KIGFX::VIEW_CONTROLS* controls  = getViewControls();
-    VECTOR2I              originalCursorPos = controls->GetCursorPosition();
-    STATUS_TEXT_POPUP     statusPopup( frame() );
-    wxString              status;
-    size_t                itemIdx = 0;
-
-    editFrame->PushTool( aEvent );
+    PCB_BASE_EDIT_FRAME*               editFrame = getEditFrame<PCB_BASE_EDIT_FRAME>();
+    PCBNEW_SETTINGS*                   cfg = editFrame->GetPcbNewSettings();
+    BOARD*                             board = editFrame->GetBoard();
+    KIGFX::VIEW_CONTROLS*              controls = getViewControls();
+    VECTOR2I                           originalCursorPos = controls->GetCursorPosition();
+    std::unique_ptr<STATUS_TEXT_POPUP> statusPopup;
+    wxString                           status;
+    size_t                             itemIdx = 0;
 
     // Be sure that there is at least one item that we can modify. If nothing was selected before,
     // try looking for the stuff under mouse cursor (i.e. KiCad old-style hover selection)
@@ -299,43 +306,17 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
             {
                 sTool->FilterCollectorForMarkers( aCollector );
                 sTool->FilterCollectorForHierarchy( aCollector, true );
+                sTool->FilterCollectorForFreePads( aCollector );
+                sTool->FilterCollectorForTableCells( aCollector );
             },
-            // Prompt user regarding locked items if in board editor and in free-pad-mode (if
-            // we're not in free-pad mode we delay this until the second RequestSelection()).
-            !m_isFootprintEditor && cfg->m_AllowFreePads );
+            true /* prompt user regarding locked items */ );
 
     if( m_dragging || selection.Empty() )
-    {
-        editFrame->PopTool( aEvent );
-        return 0;
-    }
-
-    LSET     item_layers = selection.GetSelectionLayers();
-    bool     is_hover    = selection.IsHover(); // N.B. This must be saved before the second call
-                                                // to RequestSelection() below
-    VECTOR2I pickedReferencePoint;
-
-    // Now filter out pads if not in free pads mode.  We cannot do this in the first
-    // RequestSelection() as we need the item_layers when a pad is the selection front.
-    if( !m_isFootprintEditor && !cfg->m_AllowFreePads )
-    {
-        selection = m_selectionTool->RequestSelection(
-                []( const VECTOR2I& aPt, GENERAL_COLLECTOR& aCollector, PCB_SELECTION_TOOL* sTool )
-                {
-                    sTool->FilterCollectorForMarkers( aCollector );
-                    sTool->FilterCollectorForHierarchy( aCollector, true );
-                    sTool->FilterCollectorForFreePads( aCollector );
-                },
-                true /* prompt user regarding locked items */ );
-    }
-
-    if( selection.Empty() )
-    {
-        editFrame->PopTool( aEvent );
-        return 0;
-    }
+        return false;
 
+    editFrame->PushTool( aEvent );
     Activate();
+
     // Must be done after Activate() so that it gets set into the correct context
     controls->ShowCursor( true );
     controls->SetAutoPan( true );
@@ -345,7 +326,7 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
             [editFrame]( bool constrained )
             {
                 editFrame->DisplayConstraintsMsg( constrained ? _( "Constrain to H, V, 45" )
-                                                              : wxT( "" ) );
+                                                              : wxString( wxT( "" ) ) );
             };
 
     auto updateStatusPopup =
@@ -363,7 +344,7 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
                 else if( item->Type() == PCB_PAD_T )
                 {
                     PAD*       pad = static_cast<PAD*>( item );
-                    FOOTPRINT* fp = static_cast<FOOTPRINT*>( pad->GetParentFootprint() );
+                    FOOTPRINT* fp = pad->GetParentFootprint();
                     msg = wxString::Format( _( "%s pad %s" ), fp->GetReference(), pad->GetNumber() );
                 }
                 else
@@ -371,7 +352,10 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
                     msg = item->GetTypeDesc().Lower();
                 }
 
-                statusPopup.SetText( wxString::Format( popuptext, msg, ii, count ) );
+                if( !statusPopup )
+                    statusPopup = std::make_unique<STATUS_TEXT_POPUP>( frame() );
+
+                statusPopup->SetText( wxString::Format( popuptext, msg, ii, count ) );
             };
 
     std::vector<BOARD_ITEM*> sel_items;         // All the items operated on by the move below
@@ -379,19 +363,20 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
 
     for( EDA_ITEM* item : selection )
     {
-        BOARD_ITEM* boardItem = dynamic_cast<BOARD_ITEM*>( item );
-        FOOTPRINT*  footprint = dynamic_cast<FOOTPRINT*>( item );
-
-        if( boardItem )
+        if( item->IsBOARD_ITEM() )
         {
-            if( !is_hover )
+            BOARD_ITEM* boardItem = static_cast<BOARD_ITEM*>( item );
+
+            if( !selection.IsHover() )
                 orig_items.push_back( boardItem );
 
             sel_items.push_back( boardItem );
         }
 
-        if( footprint )
+        if( item->Type() == PCB_FOOTPRINT_T )
         {
+            FOOTPRINT* footprint = static_cast<FOOTPRINT*>( item );
+
             for( PAD* pad : footprint->Pads() )
                 sel_items.push_back( pad );
 
@@ -401,14 +386,16 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
         }
     }
 
+    VECTOR2I pickedReferencePoint;
+
     if( moveWithReference && !pickReferencePoint( _( "Select reference point for move..." ), "", "",
                                                   pickedReferencePoint ) )
     {
-        if( is_hover )
-            m_toolMgr->RunAction( PCB_ACTIONS::selectionClear, true );
+        if( selection.IsHover() )
+            m_toolMgr->RunAction( PCB_ACTIONS::selectionClear );
 
         editFrame->PopTool( aEvent );
-        return 0;
+        return false;
     }
 
     if( moveIndividually )
@@ -416,12 +403,15 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
         orig_items.clear();
 
         for( EDA_ITEM* item : selection.GetItemsSortedBySelectionOrder() )
-            orig_items.push_back( static_cast<BOARD_ITEM*>( item ) );
+        {
+            if( item->IsBOARD_ITEM() )
+                orig_items.push_back( static_cast<BOARD_ITEM*>( item ) );
+        }
 
         updateStatusPopup( orig_items[ itemIdx ], itemIdx + 1, orig_items.size() );
-        statusPopup.Popup();
-        statusPopup.Move( wxGetMousePosition() + wxPoint( 20, 20 ) );
-        canvas()->SetStatusPopup( statusPopup.GetPanel() );
+        statusPopup->Popup();
+        statusPopup->Move( KIPLATFORM::UI::GetMousePosition() + wxPoint( 20, 20 ) );
+        canvas()->SetStatusPopup( statusPopup->GetPanel() );
 
         m_selectionTool->ClearSelection();
         m_selectionTool->AddItemToSel( orig_items[ itemIdx ] );
@@ -432,18 +422,18 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
 
     bool            restore_state = false;
     VECTOR2I        originalPos;
-    VECTOR2I        totalMovement;
     VECTOR2D        bboxMovement;
     BOX2I           originalBBox;
     bool            updateBBox = true;
+    LSET            layers( { editFrame->GetActiveLayer() } );
     PCB_GRID_HELPER grid( m_toolMgr, editFrame->GetMagneticItemsSettings() );
     TOOL_EVENT      copy = aEvent;
     TOOL_EVENT*     evt = &copy;
     VECTOR2I        prevPos;
+    bool            enableLocalRatsnest = true;
 
     bool hv45Mode        = false;
     bool eatFirstMouseUp = true;
-    bool hasRedrawn3D    = false;
     bool allowRedraw3D   = cfg->m_Display.m_Live3DRefresh;
     bool showCourtyardConflicts = !m_isFootprintEditor && cfg->m_ShowCourtyardCollisions;
 
@@ -460,7 +450,7 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
     displayConstraintsMessage( hv45Mode );
 
     // Prime the pump
-    m_toolMgr->RunAction( ACTIONS::refreshPreview );
+    m_toolMgr->PostAction( ACTIONS::refreshPreview );
 
     // Main loop: keep receiving events
     do
@@ -486,7 +476,8 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
 
                 VECTOR2I mousePos( controls->GetMousePosition() );
 
-                m_cursor = grid.BestSnapAnchor( mousePos, item_layers, sel_items );
+                m_cursor = grid.BestSnapAnchor( mousePos, layers,
+                                                grid.GetSelectionGrid( selection ), sel_items );
 
                 if( controls->GetSettings().m_lastKeyboardCursorPositionValid )
                 {
@@ -516,12 +507,7 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
 
                     for( EDA_ITEM* item : sel_items )
                     {
-                        BOX2I viewBBOX = item->ViewBBox();
-
-                        if( originalBBox.GetWidth() == 0 && originalBBox.GetHeight() == 0 )
-                            originalBBox = viewBBOX;
-                        else
-                            originalBBox.Merge( viewBBOX );
+                        originalBBox.Merge( item->ViewBBox() );
                     }
 
                     updateBBox = false;
@@ -537,28 +523,27 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
                 selection.SetReferencePoint( m_cursor );
 
                 prevPos = m_cursor;
-                totalMovement += movement;
                 bboxMovement += movement;
 
                 // Drag items to the current cursor position
                 for( EDA_ITEM* item : sel_items )
                 {
-                    // Don't double move footprint pads, fields, etc.
-                    //
-                    // For PCB_GROUP_T, we make sure the selection includes only the top level
-                    // group and not its descendants.
+                    // Don't double move child items.
                     if( !item->GetParent() || !item->GetParent()->IsSelected() )
                         static_cast<BOARD_ITEM*>( item )->Move( movement );
 
-                    if( !redraw3D && item->Type() == PCB_FOOTPRINT_T )
+                    if( item->Type() == PCB_GENERATOR_T && sel_items.size() == 1 )
+                    {
+                        m_toolMgr->RunSynchronousAction( PCB_ACTIONS::genUpdateEdit, aCommit,
+                                                         static_cast<PCB_GENERATOR*>( item ) );
+                    }
+
+                    if( item->Type() == PCB_FOOTPRINT_T )
                         redraw3D = true;
                 }
 
                 if( redraw3D && allowRedraw3D )
-                {
                     editFrame->Update3DView( false, true );
-                    hasRedrawn3D = true;
-                }
 
                 if( showCourtyardConflicts && drc_on_move->m_FpInMove.size() )
                 {
@@ -568,44 +553,42 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
 
                 m_toolMgr->PostEvent( EVENTS::SelectedItemsMoved );
             }
-            else if( !m_dragging && !evt->IsAction( &ACTIONS::refreshPreview ) )
+            else if( !m_dragging && ( aAutoStart || !evt->IsAction( &ACTIONS::refreshPreview ) ) )
             {
                 // Prepare to start dragging
                 editFrame->HideSolderMask();
 
                 m_dragging = true;
 
-                // When editing footprints, all items have the same parent
-                if( IsFootprintEditor() )
+                for( EDA_ITEM* item : selection )
                 {
-                    m_commit->Modify( selection.Front() );
-                }
-                else
-                {
-                    // Save items, so changes can be undone
-                    for( EDA_ITEM* item : selection )
+                    if( item->GetParent() && item->GetParent()->IsSelected() )
+                        continue;
+
+                    if( !item->IsNew() && !item->IsMoving() )
                     {
-                        // Don't double move footprint pads, fields, etc.
-                        //
-                        // For PCB_GROUP_T, the parent is the board.
-                        if( item->GetParent() && item->GetParent()->IsSelected() )
-                            continue;
-
-                        m_commit->Modify( item );
-
-                        // If moving a group, record position of all the descendants for undo
-                        if( item->Type() == PCB_GROUP_T )
+                        if( item->Type() == PCB_GENERATOR_T && sel_items.size() == 1 )
                         {
-                            PCB_GROUP* group = static_cast<PCB_GROUP*>( item );
-                            group->RunOnDescendants( [&]( BOARD_ITEM* bItem )
-                                                     {
-                                                         m_commit->Modify( bItem );
-                                                     });
+                            enableLocalRatsnest = false;
+
+                            m_toolMgr->RunSynchronousAction( PCB_ACTIONS::genStartEdit, aCommit,
+                                                             static_cast<PCB_GENERATOR*>( item ) );
                         }
+                        else
+                        {
+                            aCommit->Modify( item );
+                        }
+
+                        item->SetFlags( IS_MOVING );
+
+                        static_cast<BOARD_ITEM*>( item )->RunOnDescendants(
+                                [&]( BOARD_ITEM* bItem )
+                                {
+                                    item->SetFlags( IS_MOVING );
+                                } );
                     }
                 }
 
-                editFrame->UndoRedoBlock( true );
                 m_cursor = controls->GetCursorPosition();
 
                 if( selection.HasReferencePoint() )
@@ -635,13 +618,26 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
                 }
                 else
                 {
-                    for( BOARD_ITEM* item : sel_items )
+                    if( showCourtyardConflicts )
                     {
-                        if( showCourtyardConflicts && item->Type() == PCB_FOOTPRINT_T )
-                            drc_on_move->m_FpInMove.push_back( static_cast<FOOTPRINT*>( item ) );
+                        std::vector<FOOTPRINT*>& FPs = drc_on_move->m_FpInMove;
+
+                        for( BOARD_ITEM* item : sel_items )
+                        {
+                            if( item->Type() == PCB_FOOTPRINT_T )
+                                FPs.push_back( static_cast<FOOTPRINT*>( item ) );
+
+                            item->RunOnDescendants(
+                                    [&]( BOARD_ITEM* descendent )
+                                    {
+                                        if( descendent->Type() == PCB_FOOTPRINT_T )
+                                            FPs.push_back( static_cast<FOOTPRINT*>( descendent ) );
+                                    } );
+                        }
                     }
 
                     m_cursor = grid.BestDragOrigin( originalCursorPos, sel_items,
+                                                    grid.GetSelectionGrid( selection ),
                                                     &m_selectionTool->GetFilter() );
 
                     // Set the current cursor position to the first dragged item origin, so the
@@ -675,9 +671,11 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
                 m_toolMgr->PostEvent( EVENTS::SelectedItemsModified );
             }
 
-            statusPopup.Move( wxGetMousePosition() + wxPoint( 20, 20 ) );
+            if( statusPopup )
+                statusPopup->Move( KIPLATFORM::UI::GetMousePosition() + wxPoint( 20, 20 ) );
 
-            m_toolMgr->RunAction( PCB_ACTIONS::updateLocalRatsnest, false, new VECTOR2I( movement ) );
+            if( enableLocalRatsnest )
+                m_toolMgr->PostAction( PCB_ACTIONS::updateLocalRatsnest, movement );
         }
         else if( evt->IsCancelInteractive() || evt->IsActivate() )
         {
@@ -687,21 +685,17 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
             restore_state = true; // Canceling the tool means that items have to be restored
             break;                // Finish
         }
-        else if( evt->IsAction( &ACTIONS::undo ) )
+        else if( evt->IsClick( BUT_RIGHT ) )
+        {
+            m_selectionTool->GetToolMenu().ShowContextMenu( selection );
+        }
+        else if( evt->IsAction( &ACTIONS::undo ) || evt->IsAction( &ACTIONS::doDelete ) )
         {
             restore_state = true; // Perform undo locally
             break;                // Finish
         }
-        else if( evt->IsAction( &ACTIONS::doDelete ) || evt->IsAction( &ACTIONS::cut ) )
+        else if( evt->IsAction( &ACTIONS::duplicate ) || evt->IsAction( &ACTIONS::cut ) )
         {
-            // Dispatch TOOL_ACTIONs
-            evt->SetPassEvent();
-            break; // finish -- there is no further processing for removed items
-        }
-        else if( evt->IsAction( &ACTIONS::duplicate ) )
-        {
-            evt->SetPassEvent();
-            break; // finish -- Duplicate tool will start a new Move with the dup'ed items
         }
         else if( evt->IsAction( &PCB_ACTIONS::rotateCw )
                 || evt->IsAction( &PCB_ACTIONS::rotateCcw )
@@ -713,21 +707,10 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
             eatFirstMouseUp = false;
             evt->SetPassEvent();
         }
-        else if( evt->IsAction( &PCB_ACTIONS::moveExact ) )
-        {
-            // Reset positions so the Move Exactly is from the start.
-            for( EDA_ITEM* item : selection )
-            {
-                BOARD_ITEM* i = static_cast<BOARD_ITEM*>( item );
-                i->Move( -totalMovement );
-            }
-
-            break; // finish -- we moved exactly, so we are finished
-        }
         else if( evt->IsMouseUp( BUT_LEFT ) || evt->IsClick( BUT_LEFT ) || isSkip )
         {
             // Eat mouse-up/-click events that leaked through from the lock dialog
-            if( eatFirstMouseUp && evt->Parameter<intptr_t>() != ACTIONS::CURSOR_CLICK )
+            if( eatFirstMouseUp && !evt->IsAction( &ACTIONS::cursorClick ) )
             {
                 eatFirstMouseUp = false;
                 continue;
@@ -738,6 +721,7 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
                 if( isSkip )
                     orig_items[itemIdx]->SetPosition( originalPos );
 
+                view()->Update( orig_items[itemIdx] );
                 rebuildConnectivity();
 
                 if( ++itemIdx < orig_items.size() )
@@ -755,8 +739,8 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
                     updateStatusPopup( nextItem, itemIdx + 1, orig_items.size() );
 
                     // Pick up new item
-                    m_commit->Modify( nextItem );
-                    nextItem->SetPosition( controls->GetMousePosition( true ) );
+                    aCommit->Modify( nextItem );
+                    nextItem->Move( controls->GetCursorPosition( true ) - nextItem->GetPosition() );
 
                     continue;
                 }
@@ -778,7 +762,18 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
             displayConstraintsMessage( hv45Mode );
             evt->SetPassEvent( false );
         }
-        else if( ZONE_FILLER_TOOL::IsZoneFillAction( evt ) )
+        else if( evt->IsAction( &ACTIONS::increment ) )
+        {
+            m_toolMgr->RunSynchronousAction( ACTIONS::increment, aCommit,
+                                             evt->Parameter<ACTIONS::INCREMENT>() );
+        }
+        else if( ZONE_FILLER_TOOL::IsZoneFillAction( evt )
+                 || evt->IsAction( &PCB_ACTIONS::moveExact )
+                 || evt->IsAction( &PCB_ACTIONS::moveWithReference )
+                 || evt->IsAction( &PCB_ACTIONS::copyWithReference )
+                 || evt->IsAction( &PCB_ACTIONS::positionRelative )
+                 || evt->IsAction( &PCB_ACTIONS::positionRelativeInteractively )
+                 || evt->IsAction( &ACTIONS::redo ) )
         {
             wxBell();
         }
@@ -798,45 +793,40 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
     controls->SetAutoPan( false );
 
     m_dragging = false;
-    editFrame->UndoRedoBlock( false );
 
     // Discard reference point when selection is "dropped" onto the board
     selection.ClearReferencePoint();
 
     // Unselect all items to clear selection flags and then re-select the originally selected
-    // items (after the potential Revert()).
-    m_toolMgr->RunAction( PCB_ACTIONS::selectionClear, true );
+    // items.
+    m_toolMgr->RunAction( PCB_ACTIONS::selectionClear );
 
-    // TODO: there's an encapsulation leak here: this commit often has more than just the move
-    // in it; for instance it might have a paste, append board, etc. as well.
     if( restore_state )
     {
-        m_commit->Revert();
-        m_selectionTool->RebuildSelection();
-
-        // Mainly for point editor, but there might be other clients that need to adjust to
-        // reverted state.
-        m_toolMgr->PostEvent( EVENTS::SelectedItemsMoved );
-
-        // Property panel needs to know about the reselect
-        m_toolMgr->PostEvent( EVENTS::SelectedItemsModified );
-
-        if( hasRedrawn3D )
-            editFrame->Update3DView( false, true );
+        if( sel_items.size() == 1 && sel_items.back()->Type() == PCB_GENERATOR_T )
+        {
+            m_toolMgr->RunSynchronousAction( PCB_ACTIONS::genRevertEdit, aCommit,
+                                             static_cast<PCB_GENERATOR*>( sel_items.back() ) );
+        }
     }
     else
     {
-        m_commit->Push( aCommitMessage );
-        m_toolMgr->RunAction( PCB_ACTIONS::selectItems, true, &orig_items );
+        if( sel_items.size() == 1 && sel_items.back()->Type() == PCB_GENERATOR_T )
+        {
+            m_toolMgr->RunSynchronousAction( PCB_ACTIONS::genPushEdit, aCommit,
+                                             static_cast<PCB_GENERATOR*>( sel_items.back() ) );
+        }
+
+        EDA_ITEMS oItems( orig_items.begin(), orig_items.end() );
+        m_toolMgr->RunAction<EDA_ITEMS*>( PCB_ACTIONS::selectItems, &oItems );
     }
 
-    m_toolMgr->GetTool<DRAWING_TOOL>()->UpdateStatusBar();
     // Remove the dynamic ratsnest from the screen
-    m_toolMgr->RunAction( PCB_ACTIONS::hideLocalRatsnest, true );
+    m_toolMgr->RunAction( PCB_ACTIONS::hideLocalRatsnest );
 
     editFrame->PopTool( aEvent );
     editFrame->GetCanvas()->SetCurrentCursor( KICURSOR::ARROW );
 
-    return restore_state ? -1 : 0;
+    return !restore_state;
 }
 
