@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2014-2019 CERN
- * Copyright (C) 2019-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  *
  * This program is free software; you can redistribute it and/or
@@ -23,15 +23,22 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-#include "footprint_editor_control.h"
-#include <wx/generic/textdlgg.h>
+#include <advanced_config.h>
 #include <string_utils.h>
 #include <pgm_base.h>
 #include <tool/tool_manager.h>
+#include <tool/library_editor_control.h>
 #include <tools/pcb_actions.h>
+#include <eda_doc.h>
 #include <footprint_edit_frame.h>
+#include <generate_footprint_info.h>
 #include <pcbnew_id.h>
 #include <confirm.h>
+#include <kidialog.h>
+#include <wx/filename.h>
+#include <wildcards_and_files_ext.h>
+#include <launch_ext.h> // To default when file manager setting is empty
+#include <gestfich.h> // To open with a text editor
 #include <widgets/wx_infobar.h>
 #include <footprint.h>
 #include <pad.h>
@@ -42,10 +49,13 @@
 #include <dialogs/dialog_footprint_checker.h>
 #include <footprint_wizard_frame.h>
 #include <kiway.h>
-#include <drc/drc_item.h>
+#include <project_pcb.h>
+#include <view/view_controls.h>
 
 #include <memory>
 
+#include "footprint_editor_control.h"
+
 
 FOOTPRINT_EDITOR_CONTROL::FOOTPRINT_EDITOR_CONTROL() :
     PCB_TOOL_BASE( "pcbnew.ModuleEditor" ),
@@ -55,11 +65,6 @@ FOOTPRINT_EDITOR_CONTROL::FOOTPRINT_EDITOR_CONTROL() :
 }
 
 
-FOOTPRINT_EDITOR_CONTROL::~FOOTPRINT_EDITOR_CONTROL()
-{
-}
-
-
 void FOOTPRINT_EDITOR_CONTROL::Reset( RESET_REASON aReason )
 {
     m_frame = getEditFrame<FOOTPRINT_EDIT_FRAME>();
@@ -71,14 +76,16 @@ void FOOTPRINT_EDITOR_CONTROL::Reset( RESET_REASON aReason )
 
 bool FOOTPRINT_EDITOR_CONTROL::Init()
 {
+    LIBRARY_EDITOR_CONTROL* libraryTreeTool = m_toolMgr->GetTool<LIBRARY_EDITOR_CONTROL>();
+
     // Build a context menu for the footprint tree
     //
-    CONDITIONAL_MENU& ctxMenu = m_menu.GetMenu();
+    CONDITIONAL_MENU& ctxMenu = m_menu->GetMenu();
 
     auto libSelectedCondition =
             [ this ]( const SELECTION& aSel )
             {
-                LIB_ID sel = m_frame->GetTreeFPID();
+                LIB_ID sel = m_frame->GetLibTree()->GetSelectedLibId();
                 return !sel.GetLibNickname().empty() && sel.GetLibItemName().empty();
             };
 
@@ -88,63 +95,118 @@ bool FOOTPRINT_EDITOR_CONTROL::Init()
     auto libInferredCondition =
             [ this ]( const SELECTION& aSel )
             {
-                LIB_ID sel = m_frame->GetTreeFPID();
+                LIB_ID sel = m_frame->GetLibTree()->GetSelectedLibId();
                 return !sel.GetLibNickname().empty();
             };
-    auto pinnedLibSelectedCondition =
-            [ this ]( const SELECTION& aSel )
-            {
-                LIB_TREE_NODE* current = m_frame->GetCurrentTreeNode();
-                return current && current->m_Type == LIB_TREE_NODE::LIB && current->m_Pinned;
-            };
-    auto unpinnedLibSelectedCondition =
-            [ this ](const SELECTION& aSel )
-            {
-                LIB_TREE_NODE* current = m_frame->GetCurrentTreeNode();
-                return current && current->m_Type == LIB_TREE_NODE::LIB && !current->m_Pinned;
-            };
+
     auto fpSelectedCondition =
             [ this ]( const SELECTION& aSel )
             {
-                LIB_ID sel = m_frame->GetTreeFPID();
+                LIB_ID sel = m_frame->GetLibTree()->GetSelectedLibId();
                 return !sel.GetLibNickname().empty() && !sel.GetLibItemName().empty();
             };
 
-    ctxMenu.AddItem( ACTIONS::pinLibrary,             unpinnedLibSelectedCondition );
-    ctxMenu.AddItem( ACTIONS::unpinLibrary,           pinnedLibSelectedCondition );
+    auto fpExportCondition =
+            [ this ]( const SELECTION& aSel )
+            {
+                FOOTPRINT* fp = m_frame->GetBoard()->GetFirstFootprint();
+                return fp != nullptr;
+            };
 
-    ctxMenu.AddSeparator();
-    ctxMenu.AddItem( PCB_ACTIONS::newFootprint,       libSelectedCondition );
-    ctxMenu.AddItem( PCB_ACTIONS::createFootprint,    libSelectedCondition );
+    auto canOpenExternally =
+            [ this ]( const SELECTION& aSel )
+            {
+                // The option is shown if the editor has no current edits,
+                // dumb/simple guard against opening a new file that does not exist on disk
+                bool ret = !m_frame->IsContentModified();
+                return ret;
+            };
 
-    ctxMenu.AddSeparator();
-    ctxMenu.AddItem( ACTIONS::save,                   SELECTION_CONDITIONS::ShowAlways );
-    ctxMenu.AddItem( ACTIONS::saveAs,                 libSelectedCondition || fpSelectedCondition );
-    ctxMenu.AddItem( ACTIONS::revert,                 libSelectedCondition || libInferredCondition );
+// clang-format off
+    ctxMenu.AddItem( PCB_ACTIONS::newFootprint,       libSelectedCondition, 10 );
+    ctxMenu.AddItem( PCB_ACTIONS::createFootprint,    libSelectedCondition, 10 );
 
-    ctxMenu.AddSeparator();
-    ctxMenu.AddItem( PCB_ACTIONS::cutFootprint,       fpSelectedCondition );
-    ctxMenu.AddItem( PCB_ACTIONS::copyFootprint,      fpSelectedCondition );
-    ctxMenu.AddItem( PCB_ACTIONS::pasteFootprint,     libInferredCondition );
-    ctxMenu.AddItem( PCB_ACTIONS::duplicateFootprint, fpSelectedCondition );
-    ctxMenu.AddItem( PCB_ACTIONS::renameFootprint,    fpSelectedCondition );
-    ctxMenu.AddItem( PCB_ACTIONS::deleteFootprint,    fpSelectedCondition );
+    ctxMenu.AddSeparator( 10 );
+    ctxMenu.AddItem( ACTIONS::save,                   SELECTION_CONDITIONS::ShowAlways, 10 );
+    ctxMenu.AddItem( ACTIONS::saveAs,                 libSelectedCondition || fpSelectedCondition, 10 );
+    ctxMenu.AddItem( ACTIONS::revert,                 libSelectedCondition || libInferredCondition, 10 );
 
-    ctxMenu.AddSeparator();
-    ctxMenu.AddItem( PCB_ACTIONS::importFootprint,    libInferredCondition );
-    ctxMenu.AddItem( PCB_ACTIONS::exportFootprint,    fpSelectedCondition );
+    ctxMenu.AddSeparator( 10 );
+    ctxMenu.AddItem( PCB_ACTIONS::cutFootprint,       fpSelectedCondition, 10 );
+    ctxMenu.AddItem( PCB_ACTIONS::copyFootprint,      fpSelectedCondition, 10 );
+    ctxMenu.AddItem( PCB_ACTIONS::pasteFootprint,     libInferredCondition, 10 );
+    ctxMenu.AddItem( PCB_ACTIONS::duplicateFootprint, fpSelectedCondition, 10 );
+    ctxMenu.AddItem( PCB_ACTIONS::renameFootprint,    fpSelectedCondition, 10 );
+    ctxMenu.AddItem( PCB_ACTIONS::deleteFootprint,    fpSelectedCondition, 10 );
 
-    // If we've got nothing else to show, at least show a hide tree option
-    ctxMenu.AddItem( PCB_ACTIONS::hideFootprintTree,  !libInferredCondition );
+    ctxMenu.AddSeparator( 100 );
+    ctxMenu.AddItem( PCB_ACTIONS::importFootprint,    libInferredCondition, 100 );
+    ctxMenu.AddItem( PCB_ACTIONS::exportFootprint,    fpExportCondition, 100 );
+
+    if( ADVANCED_CFG::GetCfg().m_EnableLibWithText )
+    {
+        ctxMenu.AddSeparator( 200 );
+        ctxMenu.AddItem( ACTIONS::openWithTextEditor, canOpenExternally && fpSelectedCondition, 200 );
+    }
+
+    if( ADVANCED_CFG::GetCfg().m_EnableLibDir )
+    {
+        ctxMenu.AddSeparator( 200 );
+        ctxMenu.AddItem( ACTIONS::openDirectory,  canOpenExternally && ( libSelectedCondition || fpSelectedCondition ), 200 );
+    }
+// clang-format on
+
+    libraryTreeTool->AddContextMenuItems( &ctxMenu );
 
     return true;
 }
 
 
+void FOOTPRINT_EDITOR_CONTROL::tryToSaveFootprintInLibrary( FOOTPRINT&    aFootprint,
+                                                            const LIB_ID& aTargetLib )
+{
+    const wxString libraryName = aTargetLib.GetUniStringLibNickname();
+
+    if( aTargetLib.GetLibNickname().empty() )
+    {
+        // Do nothing - the footprint will need to be saved manually to assign
+        // to a library.
+    }
+    else
+    {
+        FP_LIB_TABLE& libTable = *PROJECT_PCB::PcbFootprintLibs( &m_frame->Prj() );
+
+        if( !libTable.IsFootprintLibWritable( libraryName ) )
+        {
+            // If the library is not writeable, we'll give the user a
+            // footprint not in a library. But add a warning to let them know
+            // they didn't quite get what they wanted.
+            m_frame->ShowInfoBarWarning(
+                    wxString::Format(
+                            _( "The footprint could not be added to the selected library ('%s'). "
+                               "This library is read-only." ),
+                            libraryName ),
+                    false );
+            // And the footprint will need to be saved manually
+        }
+        else
+        {
+            // Go ahead and save it to the library
+            LIB_ID fpid = aFootprint.GetFPID();
+            fpid.SetLibNickname( aTargetLib.GetLibNickname() );
+            aFootprint.SetFPID( fpid );
+            m_frame->SaveFootprint( &aFootprint );
+            m_frame->ClearModify();
+        }
+    }
+}
+
+
 int FOOTPRINT_EDITOR_CONTROL::NewFootprint( const TOOL_EVENT& aEvent )
 {
-    LIB_ID     selected = m_frame->GetTreeFPID();
-    FOOTPRINT* newFootprint = m_frame->CreateNewFootprint( wxEmptyString );
+    const LIB_ID   selected = m_frame->GetTargetFPID();
+    const wxString libraryName = selected.GetUniStringLibNickname();
+    FOOTPRINT*     newFootprint = m_frame->CreateNewFootprint( wxEmptyString, libraryName );
 
     if( !newFootprint )
         return 0;
@@ -158,21 +220,13 @@ int FOOTPRINT_EDITOR_CONTROL::NewFootprint( const TOOL_EVENT& aEvent )
     // Initialize data relative to nets and netclasses (for a new footprint the defaults are
     // used).  This is mandatory to handle and draw pads.
     board()->BuildListOfNets();
-    newFootprint->SetPosition( wxPoint( 0, 0 ) );
+    newFootprint->SetPosition( VECTOR2I( 0, 0 ) );
     newFootprint->ClearFlags();
 
     m_frame->Zoom_Automatique( false );
     m_frame->GetScreen()->SetContentModified();
 
-    // If selected from the library tree then go ahead and save it there
-    if( !selected.GetLibNickname().empty() )
-    {
-        LIB_ID fpid = newFootprint->GetFPID();
-        fpid.SetLibNickname( selected.GetLibNickname() );
-        newFootprint->SetFPID( fpid );
-        m_frame->SaveFootprint( newFootprint );
-        m_frame->ClearModify();
-    }
+    tryToSaveFootprintInLibrary( *newFootprint, selected );
 
     m_frame->UpdateView();
     m_frame->Update3DView( true, true );
@@ -184,7 +238,8 @@ int FOOTPRINT_EDITOR_CONTROL::NewFootprint( const TOOL_EVENT& aEvent )
 
 int FOOTPRINT_EDITOR_CONTROL::CreateFootprint( const TOOL_EVENT& aEvent )
 {
-    LIB_ID selected = m_frame->GetTreeFPID();
+    LIB_ID selected = m_frame->GetLibTree()->GetSelectedLibId();
+    wxString libraryName = selected.GetUniStringLibNickname();
 
     if( m_frame->IsContentModified() )
     {
@@ -199,51 +254,46 @@ int FOOTPRINT_EDITOR_CONTROL::CreateFootprint( const TOOL_EVENT& aEvent )
         }
     }
 
-    auto* wizard = (FOOTPRINT_WIZARD_FRAME*) m_frame->Kiway().Player( FRAME_FOOTPRINT_WIZARD,
-                                                                      true, m_frame );
-
-    if( wizard->ShowModal( nullptr, m_frame ) )
+    if( KIWAY_PLAYER* frame = m_frame->Kiway().Player( FRAME_FOOTPRINT_WIZARD, true, m_frame ) )
     {
-        // Creates the new footprint from python script wizard
-        FOOTPRINT* newFootprint = wizard->GetBuiltFootprint();
+        FOOTPRINT_WIZARD_FRAME* wizard = static_cast<FOOTPRINT_WIZARD_FRAME*>( frame );
 
-        if( newFootprint )    // i.e. if create footprint command is OK
+        if( wizard->ShowModal( nullptr, m_frame ) )
         {
-            m_frame->Clear_Pcb( false );
+            // Creates the new footprint from python script wizard
+            FOOTPRINT* newFootprint = wizard->GetBuiltFootprint();
 
-            canvas()->GetViewControls()->SetCrossHairCursorPosition( VECTOR2D( 0, 0 ), false );
-            //  Add the new object to board
-            m_frame->AddFootprintToBoard( newFootprint );
-
-            // Initialize data relative to nets and netclasses (for a new footprint the defaults
-            // are used).  This is mandatory to handle and draw pads.
-            board()->BuildListOfNets();
-            newFootprint->SetPosition( wxPoint( 0, 0 ) );
-            newFootprint->ClearFlags();
-
-            m_frame->Zoom_Automatique( false );
-            m_frame->GetScreen()->SetContentModified();
-            m_frame->OnModify();
-
-            // If selected from the library tree then go ahead and save it there
-            if( !selected.GetLibNickname().empty() )
+            if( newFootprint )    // i.e. if create footprint command is OK
             {
-                LIB_ID fpid = newFootprint->GetFPID();
-                fpid.SetLibNickname( selected.GetLibNickname() );
-                newFootprint->SetFPID( fpid );
-                m_frame->SaveFootprint( newFootprint );
-                m_frame->ClearModify();
+                m_frame->Clear_Pcb( false );
+
+                canvas()->GetViewControls()->SetCrossHairCursorPosition( VECTOR2D( 0, 0 ), false );
+                //  Add the new object to board
+                m_frame->AddFootprintToBoard( newFootprint );
+
+                // Initialize data relative to nets and netclasses (for a new footprint the
+                // defaults are used).  This is mandatory to handle and draw pads.
+                board()->BuildListOfNets();
+                newFootprint->SetPosition( VECTOR2I( 0, 0 ) );
+                newFootprint->ClearFlags();
+
+                m_frame->Zoom_Automatique( false );
+                m_frame->GetScreen()->SetContentModified();
+                m_frame->OnModify();
+
+                tryToSaveFootprintInLibrary( *newFootprint, selected );
+
+                m_frame->UpdateView();
+                canvas()->Refresh();
+                m_frame->Update3DView( true, true );
+
+                m_frame->SyncLibraryTree( false );
             }
-
-            m_frame->UpdateView();
-            canvas()->Refresh();
-            m_frame->Update3DView( true, true );
-
-            m_frame->SyncLibraryTree( false );
         }
+
+        wizard->Destroy();
     }
 
-    wizard->Destroy();
     return 0;
 }
 
@@ -276,7 +326,7 @@ int FOOTPRINT_EDITOR_CONTROL::SaveAs( const TOOL_EVENT& aEvent )
     {
         // Save Library As
         const wxString& src_libNickname = m_frame->GetTargetFPID().GetLibNickname();
-        wxString src_libFullName = m_frame->Prj().PcbFootprintLibs()->GetFullURI( src_libNickname );
+        wxString src_libFullName = PROJECT_PCB::PcbFootprintLibs( &m_frame->Prj() )->GetFullURI( src_libNickname );
 
         if( m_frame->SaveLibraryAs( src_libFullName ) )
             m_frame->SyncLibraryTree( true );
@@ -292,7 +342,7 @@ int FOOTPRINT_EDITOR_CONTROL::SaveAs( const TOOL_EVENT& aEvent )
             // Get rid of the save-will-update-board-only (or any other dismissable warning)
             WX_INFOBAR* infobar = m_frame->GetInfoBar();
 
-            if( infobar->IsShown() && infobar->HasCloseButton() )
+            if( infobar->IsShownOnScreen() && infobar->HasCloseButton() )
                 infobar->Dismiss();
 
             canvas()->ForceRefresh();
@@ -325,7 +375,7 @@ int FOOTPRINT_EDITOR_CONTROL::Revert( const TOOL_EVENT& aEvent )
 
 int FOOTPRINT_EDITOR_CONTROL::CutCopyFootprint( const TOOL_EVENT& aEvent )
 {
-    LIB_ID fpID = m_frame->GetTreeFPID();
+    LIB_ID fpID = m_frame->GetLibTree()->GetSelectedLibId();
 
     if( fpID == m_frame->GetLoadedFPID() )
     {
@@ -346,12 +396,12 @@ int FOOTPRINT_EDITOR_CONTROL::CutCopyFootprint( const TOOL_EVENT& aEvent )
 
 int FOOTPRINT_EDITOR_CONTROL::PasteFootprint( const TOOL_EVENT& aEvent )
 {
-    if( m_copiedFootprint && !m_frame->GetTreeFPID().GetLibNickname().empty() )
+    if( m_copiedFootprint && !m_frame->GetLibTree()->GetSelectedLibId().GetLibNickname().empty() )
     {
-        wxString newLib = m_frame->GetTreeFPID().GetLibNickname();
+        wxString newLib = m_frame->GetLibTree()->GetSelectedLibId().GetLibNickname();
         wxString newName = m_copiedFootprint->GetFPID().GetLibItemName();
 
-        while( m_frame->Prj().PcbFootprintLibs()->FootprintExists( newLib, newName ) )
+        while( PROJECT_PCB::PcbFootprintLibs( &m_frame->Prj() )->FootprintExists( newLib, newName ) )
             newName += _( "_copy" );
 
         m_copiedFootprint->SetFPID( LIB_ID( newLib, newName ) );
@@ -369,7 +419,7 @@ int FOOTPRINT_EDITOR_CONTROL::PasteFootprint( const TOOL_EVENT& aEvent )
 
 int FOOTPRINT_EDITOR_CONTROL::DuplicateFootprint( const TOOL_EVENT& aEvent )
 {
-    LIB_ID     fpID = m_frame->GetTreeFPID();
+    LIB_ID     fpID = m_frame->GetLibTree()->GetSelectedLibId();
     FOOTPRINT* footprint;
 
     if( fpID == m_frame->GetLoadedFPID() )
@@ -391,41 +441,48 @@ int FOOTPRINT_EDITOR_CONTROL::DuplicateFootprint( const TOOL_EVENT& aEvent )
 
 int FOOTPRINT_EDITOR_CONTROL::RenameFootprint( const TOOL_EVENT& aEvent )
 {
-    FP_LIB_TABLE* tbl = m_frame->Prj().PcbFootprintLibs();
-    LIB_ID        fpID = m_frame->GetTreeFPID();
-    wxString      libraryName = fpID.GetLibNickname();
-    wxString      oldName = fpID.GetLibItemName();
-    wxString      newName = oldName;
-    bool          done = false;
+    LIBRARY_EDITOR_CONTROL* libTool   = m_toolMgr->GetTool<LIBRARY_EDITOR_CONTROL>();
+    FP_LIB_TABLE*           tbl = PROJECT_PCB::PcbFootprintLibs( &m_frame->Prj() );
 
-    while( !done )
+    LIB_ID   fpID = m_frame->GetLibTree()->GetSelectedLibId();
+    wxString libraryName = fpID.GetLibNickname();
+    wxString oldName = fpID.GetLibItemName();
+    wxString newName;
+    wxString msg;
+
+    if( !libTool->RenameLibrary( _( "Change Footprint Name" ), oldName,
+            [&]( const wxString& aNewName )
+            {
+                newName = aNewName;
+
+                if( newName.IsEmpty() )
+                {
+                    wxMessageBox( _( "Footprint must have a name." ) );
+                    return false;
+                }
+
+                // If no change, accept it without prompting
+                if( oldName != newName && tbl->FootprintExists( libraryName, newName ) )
+                {
+                    msg = wxString::Format( _( "Footprint '%s' already exists in library '%s'." ),
+                                            newName, libraryName );
+
+                    KIDIALOG errorDlg( m_frame, msg, _( "Confirmation" ),
+                                       wxOK | wxCANCEL | wxICON_WARNING );
+                    errorDlg.SetOKLabel( _( "Overwrite" ) );
+
+                    return errorDlg.ShowModal() == wxID_OK;
+                }
+
+                return true;
+            } ) )
     {
-        wxTextEntryDialog dlg( m_frame, _( "New name:" ), _( "Change Footprint Name" ), newName );
-
-        if( dlg.ShowModal() != wxID_OK )
-            return 0;   // canceled by user
-
-        newName = dlg.GetValue();
-        newName.Trim( true ).Trim( false );
-
-        if( newName.IsEmpty() )
-        {
-            DisplayErrorMessage( m_frame, _( "Footprint name cannot be empty." ) );
-        }
-        else if( tbl->FootprintExists( libraryName, newName ) )
-        {
-            DisplayErrorMessage( m_frame, wxString::Format( _( "Footprint name '%s' already "
-                                                               "in use in library '%s'." ),
-                                                            UnescapeString( newName ),
-                                                            libraryName ) );
-            newName = oldName;
-        }
-        else
-        {
-            done = true;
-        }
+        return 0;   // cancelled by user
     }
 
+    if( newName == oldName )
+        return 0;
+
     FOOTPRINT* footprint = nullptr;
 
     if( fpID == m_frame->GetLoadedFPID() )
@@ -458,11 +515,11 @@ int FOOTPRINT_EDITOR_CONTROL::RenameFootprint( const TOOL_EVENT& aEvent )
 
                 m_frame->SaveFootprintInLibrary( footprint, libraryName );
 
-                m_frame->Prj().PcbFootprintLibs()->FootprintDelete( libraryName, oldName );
+                PROJECT_PCB::PcbFootprintLibs( &m_frame->Prj() )->FootprintDelete( libraryName, oldName );
             }
             catch( const IO_ERROR& ioe )
             {
-                DisplayError( m_frame, ioe.What() );
+                DisplayErrorMessage( m_frame, _( "Error renaming footprint" ), ioe.What() );
             }
             catch( ... )
             {
@@ -517,7 +574,7 @@ int FOOTPRINT_EDITOR_CONTROL::ImportFootprint( const TOOL_EVENT& aEvent )
         m_frame->ReCreateHToolbar();
     }
 
-    m_toolMgr->RunAction( ACTIONS::zoomFitScreen, true );
+    m_toolMgr->RunAction( ACTIONS::zoomFitScreen );
     m_frame->OnModify();
     return 0;
 }
@@ -525,61 +582,176 @@ int FOOTPRINT_EDITOR_CONTROL::ImportFootprint( const TOOL_EVENT& aEvent )
 
 int FOOTPRINT_EDITOR_CONTROL::ExportFootprint( const TOOL_EVENT& aEvent )
 {
-    LIB_ID     fpID = m_frame->GetTreeFPID();
-    FOOTPRINT* fp;
+    if( FOOTPRINT* fp = m_frame->GetBoard()->GetFirstFootprint() )
+        m_frame->ExportFootprint( fp );
 
-    if( !fpID.IsValid() )
-        fp = m_frame->GetBoard()->GetFirstFootprint();
-    else
-        fp = m_frame->LoadFootprint( fpID );
+    return 0;
+}
 
-    m_frame->ExportFootprint( fp );
+
+int FOOTPRINT_EDITOR_CONTROL::OpenDirectory( const TOOL_EVENT& aEvent )
+{
+    // No check for multi selection since the context menu option must be hidden in that case
+    FP_LIB_TABLE* globalTable = dynamic_cast<FP_LIB_TABLE*>( &GFootprintTable );
+    FP_LIB_TABLE* projectTable = PROJECT_PCB::PcbFootprintLibs( &m_frame->Prj() );
+    LIB_ID        libId = m_frame->GetTargetFPID();
+
+    wxString    libName = libId.GetLibNickname();
+    wxString    libItemName = libId.GetLibItemName();
+    wxString    path = wxEmptyString;
+
+    for( FP_LIB_TABLE* table : { globalTable, projectTable } )
+    {
+        if( !table )
+            break;
+
+        try
+        {
+            path = table->FindRow( libName, true )->GetFullURI( true );
+        }
+        catch( IO_ERROR& )
+        {
+            // Do nothing: libName can be not found in globalTable if libName is in projectTable
+        }
+
+        if( !path.IsEmpty() )
+            break;
+    }
+
+    wxString fileExt = wxEmptyString;
+
+    // If selection is footprint
+    if( !libItemName.IsEmpty() )
+        fileExt = FILEEXT::KiCadFootprintFileExtension;
+
+    wxFileName fileName( path, libItemName, fileExt );
+
+    COMMON_SETTINGS* cfg = Pgm().GetCommonSettings();
+
+    wxString explCommand = cfg->m_System.file_explorer;
+
+    if( explCommand.IsEmpty() )
+    {
+        path = fileName.GetFullPath().BeforeLast( wxFileName::GetPathSeparator() );
+
+        if( !path.IsEmpty() && wxDirExists( path ) )
+            LaunchExternal( path );
+        return 0;
+    }
+
+    if( !explCommand.EndsWith( "%F" ) )
+    {
+        wxMessageBox( _( "Missing/malformed file explorer argument '%F' in common settings." ) );
+        return 0;
+    }
+
+    wxString escapedFilePath = fileName.GetFullPath();
+    escapedFilePath.Replace( wxS( "\"" ), wxS( "_" ) );
+
+    wxString fileArg = wxEmptyString;
+    fileArg << '"' << escapedFilePath << '"';
+
+    explCommand.Replace( wxT( "%F" ), fileArg );
+
+    if( !explCommand.IsEmpty() )
+        wxExecute( explCommand );
+
+    return 0;
+}
+
+
+int FOOTPRINT_EDITOR_CONTROL::OpenWithTextEditor( const TOOL_EVENT& aEvent )
+{
+    wxString fullEditorName = Pgm().GetTextEditor();
+
+    if( fullEditorName.IsEmpty() )
+    {
+        wxMessageBox( _( "No text editor selected in KiCad. Please choose one." ) );
+        return 0;
+    }
+
+    // No check for multi selection since the context menu option must be hidden in that case
+    FP_LIB_TABLE* globalTable = dynamic_cast<FP_LIB_TABLE*>( &GFootprintTable );
+    FP_LIB_TABLE* projectTable = PROJECT_PCB::PcbFootprintLibs( &m_frame->Prj() );
+    LIB_ID        libId = m_frame->GetLibTree()->GetSelectedLibId();
+
+    wxString    libName = libId.GetLibNickname();
+    wxString    libItemName = wxEmptyString;
+
+    for( FP_LIB_TABLE* table : { globalTable, projectTable } )
+    {
+        if( !table )
+            break;
+
+        try
+        {
+            libItemName = table->FindRow( libName, true )->GetFullURI( true );
+        }
+        catch( IO_ERROR& )
+        {
+            // Do nothing: libName can be not found in globalTable if libName is in projectTable
+        }
+
+        if( !libItemName.IsEmpty() )
+            break;
+    }
+
+    libItemName << wxFileName::GetPathSeparator();
+    libItemName << libId.GetLibItemName();
+    libItemName << '.' + FILEEXT::KiCadFootprintFileExtension;
+
+    if( !wxFileName::FileExists( libItemName ) )
+        return 0;
+
+    ExecuteFile( fullEditorName, libItemName.wc_str(), nullptr, false );
+
+    return 0;
+}
+
+
+int FOOTPRINT_EDITOR_CONTROL::ShowDatasheet( const TOOL_EVENT& aEvent )
+{
+    if( FOOTPRINT* footprint = m_frame->GetBoard()->GetFirstFootprint() )
+    {
+        std::optional<wxString> url = GetFootprintDocumentationURL( *footprint );
+
+        if( !url.has_value() )
+        {
+            frame()->ShowInfoBarMsg( _( "No datasheet found in the footprint." ) );
+        }
+        else
+        {
+            // Only absolute URLs are supported
+            SEARCH_STACK* searchStack = nullptr;
+            GetAssociatedDocument( m_frame, *url, &m_frame->Prj(), searchStack, footprint );
+        }
+    }
     return 0;
 }
 
 
 int FOOTPRINT_EDITOR_CONTROL::EditFootprint( const TOOL_EVENT& aEvent )
 {
-    m_frame->LoadFootprintFromLibrary( m_frame->GetTreeFPID() );
+    m_frame->LoadFootprintFromLibrary( m_frame->GetLibTree()->GetSelectedLibId() );
     return 0;
 }
 
 
-int FOOTPRINT_EDITOR_CONTROL::PinLibrary( const TOOL_EVENT& aEvent )
+int FOOTPRINT_EDITOR_CONTROL::EditLibraryFootprint( const TOOL_EVENT& aEvent )
 {
-    LIB_TREE_NODE* currentNode = m_frame->GetCurrentTreeNode();
+    FOOTPRINT* footprint = m_frame->GetBoard()->GetFirstFootprint();
 
-    if( currentNode && !currentNode->m_Pinned )
+    if( !footprint || !m_frame->IsCurrentFPFromBoard() )
     {
-        m_frame->Prj().PinLibrary( currentNode->m_LibId.GetLibNickname(), false );
-
-        currentNode->m_Pinned = true;
-        m_frame->RegenerateLibraryTree();
+        wxBell();
+        return 0;
     }
 
-    return 0;
-}
+    m_frame->LoadFootprintFromLibrary( footprint->GetFPID() );
 
+    if( !m_frame->IsLibraryTreeShown() )
+        m_frame->ToggleLibraryTree();
 
-int FOOTPRINT_EDITOR_CONTROL::UnpinLibrary( const TOOL_EVENT& aEvent )
-{
-    LIB_TREE_NODE* currentNode = m_frame->GetCurrentTreeNode();
-
-    if( currentNode && currentNode->m_Pinned )
-    {
-        m_frame->Prj().UnpinLibrary( currentNode->m_LibId.GetLibNickname(), false );
-
-        currentNode->m_Pinned = false;
-        m_frame->RegenerateLibraryTree();
-    }
-
-    return 0;
-}
-
-
-int FOOTPRINT_EDITOR_CONTROL::ToggleFootprintTree( const TOOL_EVENT& aEvent )
-{
-    m_frame->ToggleSearchTree();
     return 0;
 }
 
@@ -591,15 +763,21 @@ int FOOTPRINT_EDITOR_CONTROL::ToggleLayersManager( const TOOL_EVENT& aEvent )
 }
 
 
+int FOOTPRINT_EDITOR_CONTROL::ToggleProperties( const TOOL_EVENT& aEvent )
+{
+    m_frame->ToggleProperties();
+    return 0;
+}
+
+
 int FOOTPRINT_EDITOR_CONTROL::Properties( const TOOL_EVENT& aEvent )
 {
-    FOOTPRINT* footprint = m_frame->GetBoard()->GetFirstFootprint();
-
-    if( footprint )
+    if( FOOTPRINT* footprint = m_frame->GetBoard()->GetFirstFootprint() )
     {
         getEditFrame<FOOTPRINT_EDIT_FRAME>()->OnEditItemRequest( footprint );
         m_frame->GetCanvas()->Refresh();
     }
+
     return 0;
 }
 
@@ -626,7 +804,6 @@ int FOOTPRINT_EDITOR_CONTROL::CheckFootprint( const TOOL_EVENT& aEvent )
     if( !m_checkerDialog )
     {
         m_checkerDialog = new DIALOG_FOOTPRINT_CHECKER( m_frame );
-
         m_checkerDialog->Show( true );
     }
     else // The dialog is just not visible (because the user has double clicked on an error item)
@@ -643,7 +820,7 @@ void FOOTPRINT_EDITOR_CONTROL::CrossProbe( const PCB_MARKER* aMarker )
     if( !m_checkerDialog )
         m_checkerDialog = new DIALOG_FOOTPRINT_CHECKER( m_frame );
 
-    if( !m_checkerDialog->IsShown() )
+    if( !m_checkerDialog->IsShownOnScreen() )
         m_checkerDialog->Show( true );
 
     m_checkerDialog->SelectMarker( aMarker );
@@ -712,14 +889,6 @@ int FOOTPRINT_EDITOR_CONTROL::RepairFootprint( const TOOL_EVENT& aEvent )
         details += wxString::Format( _( "%d duplicate IDs replaced.\n" ), duplicates );
     }
 
-    /*******************************
-     * Your test here
-     */
-
-    /*******************************
-     * Inform the user
-     */
-
     if( errors )
     {
         m_frame->OnModify();
@@ -738,6 +907,7 @@ int FOOTPRINT_EDITOR_CONTROL::RepairFootprint( const TOOL_EVENT& aEvent )
 
 void FOOTPRINT_EDITOR_CONTROL::setTransitions()
 {
+    // clang-format off
     Go( &FOOTPRINT_EDITOR_CONTROL::NewFootprint,         PCB_ACTIONS::newFootprint.MakeEvent() );
     Go( &FOOTPRINT_EDITOR_CONTROL::CreateFootprint,      PCB_ACTIONS::createFootprint.MakeEvent() );
     Go( &FOOTPRINT_EDITOR_CONTROL::Save,                 ACTIONS::save.MakeEvent() );
@@ -748,6 +918,7 @@ void FOOTPRINT_EDITOR_CONTROL::setTransitions()
     Go( &FOOTPRINT_EDITOR_CONTROL::DeleteFootprint,      PCB_ACTIONS::deleteFootprint.MakeEvent() );
 
     Go( &FOOTPRINT_EDITOR_CONTROL::EditFootprint,        PCB_ACTIONS::editFootprint.MakeEvent() );
+    Go( &FOOTPRINT_EDITOR_CONTROL::EditLibraryFootprint, PCB_ACTIONS::editLibFpInFpEditor.MakeEvent() );
     Go( &FOOTPRINT_EDITOR_CONTROL::CutCopyFootprint,     PCB_ACTIONS::cutFootprint.MakeEvent() );
     Go( &FOOTPRINT_EDITOR_CONTROL::CutCopyFootprint,     PCB_ACTIONS::copyFootprint.MakeEvent() );
     Go( &FOOTPRINT_EDITOR_CONTROL::PasteFootprint,       PCB_ACTIONS::pasteFootprint.MakeEvent() );
@@ -755,17 +926,19 @@ void FOOTPRINT_EDITOR_CONTROL::setTransitions()
     Go( &FOOTPRINT_EDITOR_CONTROL::ImportFootprint,      PCB_ACTIONS::importFootprint.MakeEvent() );
     Go( &FOOTPRINT_EDITOR_CONTROL::ExportFootprint,      PCB_ACTIONS::exportFootprint.MakeEvent() );
 
+    Go( &FOOTPRINT_EDITOR_CONTROL::OpenWithTextEditor,   ACTIONS::openWithTextEditor.MakeEvent() );
+    Go( &FOOTPRINT_EDITOR_CONTROL::OpenDirectory,        ACTIONS::openDirectory.MakeEvent() );
+    Go( &FOOTPRINT_EDITOR_CONTROL::ShowDatasheet,        ACTIONS::showDatasheet.MakeEvent() );
+
     Go( &FOOTPRINT_EDITOR_CONTROL::EditTextAndGraphics,  PCB_ACTIONS::editTextAndGraphics.MakeEvent() );
     Go( &FOOTPRINT_EDITOR_CONTROL::CleanupGraphics,      PCB_ACTIONS::cleanupGraphics.MakeEvent() );
 
     Go( &FOOTPRINT_EDITOR_CONTROL::CheckFootprint,       PCB_ACTIONS::checkFootprint.MakeEvent() );
     Go( &FOOTPRINT_EDITOR_CONTROL::RepairFootprint,      PCB_ACTIONS::repairFootprint.MakeEvent() );
 
-    Go( &FOOTPRINT_EDITOR_CONTROL::PinLibrary,           ACTIONS::pinLibrary.MakeEvent() );
-    Go( &FOOTPRINT_EDITOR_CONTROL::UnpinLibrary,         ACTIONS::unpinLibrary.MakeEvent() );
-    Go( &FOOTPRINT_EDITOR_CONTROL::ToggleFootprintTree,  PCB_ACTIONS::showFootprintTree.MakeEvent() );
-    Go( &FOOTPRINT_EDITOR_CONTROL::ToggleFootprintTree,  PCB_ACTIONS::hideFootprintTree.MakeEvent() );
     Go( &FOOTPRINT_EDITOR_CONTROL::Properties,           PCB_ACTIONS::footprintProperties.MakeEvent() );
     Go( &FOOTPRINT_EDITOR_CONTROL::DefaultPadProperties, PCB_ACTIONS::defaultPadProperties.MakeEvent() );
     Go( &FOOTPRINT_EDITOR_CONTROL::ToggleLayersManager,  PCB_ACTIONS::showLayersManager.MakeEvent() );
+    Go( &FOOTPRINT_EDITOR_CONTROL::ToggleProperties,     PCB_ACTIONS::showProperties.MakeEvent() );
+    // clang-format on
 }
