@@ -1,8 +1,8 @@
 /*
  * This program source code file is part of KICAD, a free EDA CAD application.
  *
- * Copyright (C) 2020 CERN
- * Copyright (C) 2021-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2020-2023 CERN
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  *
  * This program is free software; you can redistribute it and/or
@@ -21,8 +21,12 @@
 
 #include "pcb_properties_panel.h"
 
-#include <pcb_edit_frame.h>
+#include <font/fontconfig.h>
+#include <font/kicad_font_name.h>
+#include <pgm_base.h>
+#include <pcb_base_edit_frame.h>
 #include <tool/tool_manager.h>
+#include <tools/pcb_actions.h>
 #include <tools/pcb_selection_tool.h>
 #include <properties/property_mgr.h>
 #include <properties/pg_editors.h>
@@ -32,11 +36,13 @@
 #include <pcb_shape.h>
 #include <pcb_text.h>
 #include <pcb_track.h>
+#include <pcb_generator.h>
+#include <pad.h>
 #include <settings/color_settings.h>
 #include <string_utils.h>
 
 
-PCB_PROPERTIES_PANEL::PCB_PROPERTIES_PANEL( wxWindow* aParent, PCB_EDIT_FRAME* aFrame ) :
+PCB_PROPERTIES_PANEL::PCB_PROPERTIES_PANEL( wxWindow* aParent, PCB_BASE_EDIT_FRAME* aFrame ) :
         PROPERTIES_PANEL( aParent, aFrame ),
         m_frame( aFrame ),
         m_propMgr( PROPERTY_MANAGER::Instance() )
@@ -46,7 +52,9 @@ PCB_PROPERTIES_PANEL::PCB_PROPERTIES_PANEL( wxWindow* aParent, PCB_EDIT_FRAME* a
 
     wxASSERT( wxPGGlobalVars );
 
-    auto it = wxPGGlobalVars->m_mapEditorClasses.find( PG_UNIT_EDITOR::EDITOR_NAME );
+    wxString editorKey = PG_UNIT_EDITOR::BuildEditorName( m_frame );
+
+    auto it = wxPGGlobalVars->m_mapEditorClasses.find( editorKey );
 
     if( it != wxPGGlobalVars->m_mapEditorClasses.end() )
     {
@@ -72,8 +80,19 @@ PCB_PROPERTIES_PANEL::PCB_PROPERTIES_PANEL( wxWindow* aParent, PCB_EDIT_FRAME* a
     {
         m_checkboxEditorInstance = static_cast<PG_CHECKBOX_EDITOR*>( it->second );
     }
-}
 
+    it = wxPGGlobalVars->m_mapEditorClasses.find( PG_RATIO_EDITOR::EDITOR_NAME );
+
+    if( it == wxPGGlobalVars->m_mapEditorClasses.end() )
+    {
+        PG_RATIO_EDITOR* ratioEditor = new PG_RATIO_EDITOR();
+        m_ratioEditorInstance = static_cast<PG_RATIO_EDITOR*>( wxPropertyGrid::RegisterEditorClass( ratioEditor ) );
+    }
+    else
+    {
+        m_ratioEditorInstance = static_cast<PG_RATIO_EDITOR*>( it->second );
+    }
+}
 
 
 PCB_PROPERTIES_PANEL::~PCB_PROPERTIES_PANEL()
@@ -101,11 +120,6 @@ void PCB_PROPERTIES_PANEL::AfterCommit()
     const SELECTION& selection = selectionTool->GetSelection();
 
     rebuildProperties( selection );
-
-    CallAfter( [&]()
-               {
-                   static_cast<PCB_EDIT_FRAME*>( m_frame )->GetCanvas()->SetFocus();
-               } );
 }
 
 
@@ -127,8 +141,7 @@ wxPGProperty* PCB_PROPERTIES_PANEL::createPGProperty( const PROPERTY_BASE* aProp
             boardLayerIDs.push_back( canonicalLayers.GetValue( ii ) );
         }
 
-        auto ret = new PGPROPERTY_COLORENUM( wxPG_LABEL, wxPG_LABEL,
-                                             new wxPGChoices( boardLayerNames, boardLayerIDs ) );
+        auto ret = new PGPROPERTY_COLORENUM( new wxPGChoices( boardLayerNames, boardLayerIDs ) );
 
         ret->SetColorFunc(
                 [&]( int aValue ) -> wxColour
@@ -144,7 +157,7 @@ wxPGProperty* PCB_PROPERTIES_PANEL::createPGProperty( const PROPERTY_BASE* aProp
         return ret;
     }
 
-    return PGPropertyFactory( aProperty );
+    return PGPropertyFactory( aProperty, m_frame );
 }
 
 
@@ -200,6 +213,8 @@ void PCB_PROPERTIES_PANEL::valueChanged( wxPropertyGridEvent& aEvent )
     wxVariant newValue = aEvent.GetPropertyValue();
     BOARD_COMMIT changes( m_frame );
 
+    PROPERTY_COMMIT_HANDLER handler( &changes );
+
     for( EDA_ITEM* edaItem : selection )
     {
         BOARD_ITEM* item = static_cast<BOARD_ITEM*>( edaItem );
@@ -207,7 +222,8 @@ void PCB_PROPERTIES_PANEL::valueChanged( wxPropertyGridEvent& aEvent )
         item->Set( property, newValue );
     }
 
-    changes.Push( _( "Change property" ) );
+    changes.Push( _( "Edit Properties" ) );
+
     m_frame->Refresh();
 
     // Perform grid updates as necessary based on value change
@@ -217,14 +233,17 @@ void PCB_PROPERTIES_PANEL::valueChanged( wxPropertyGridEvent& aEvent )
 
 void PCB_PROPERTIES_PANEL::updateLists( const BOARD* aBoard )
 {
-    wxPGChoices layersAll, layersCu, nets;
+    wxPGChoices layersAll;
+    wxPGChoices layersCu;
+    wxPGChoices nets;
+    wxPGChoices fonts;
 
     // Regenerate all layers
-    for( LSEQ seq = aBoard->GetEnabledLayers().UIOrder(); seq; ++seq )
-        layersAll.Add( LSET::Name( *seq ), *seq );
+    for( PCB_LAYER_ID layer : aBoard->GetEnabledLayers().UIOrder() )
+        layersAll.Add( LSET::Name( layer ), layer );
 
-    for( LSEQ seq = LSET( aBoard->GetEnabledLayers() & LSET::AllCuMask() ).UIOrder(); seq; ++seq )
-        layersCu.Add( LSET::Name( *seq ), *seq );
+    for( PCB_LAYER_ID layer : LSET( aBoard->GetEnabledLayers() & LSET::AllCuMask() ).UIOrder() )
+        layersCu.Add( LSET::Name( layer ), layer );
 
     m_propMgr.GetProperty( TYPE_HASH( BOARD_ITEM ), _HKI( "Layer" ) )->SetChoices( layersAll );
     m_propMgr.GetProperty( TYPE_HASH( PCB_SHAPE ), _HKI( "Layer" ) )->SetChoices( layersAll );
@@ -236,9 +255,34 @@ void PCB_PROPERTIES_PANEL::updateLists( const BOARD* aBoard )
     m_propMgr.GetProperty( TYPE_HASH( PCB_VIA ), _HKI( "Layer Bottom" ) )->SetChoices( layersCu );
 
     // Regenerate nets
+
+    std::vector<std::pair<wxString, int>> netNames;
+    netNames.reserve( aBoard->GetNetInfo().NetsByNetcode().size() );
+
     for( const auto& [ netCode, netInfo ] : aBoard->GetNetInfo().NetsByNetcode() )
-        nets.Add( UnescapeString( netInfo->GetNetname() ), netCode );
+        netNames.emplace_back( UnescapeString( netInfo->GetNetname() ), netCode );
+
+    std::sort( netNames.begin(), netNames.end(), []( const auto& a, const auto& b )
+    {
+        return a.first.CmpNoCase( b.first ) < 0;
+    } );
+
+    for( const auto& [ netName, netCode ] : netNames )
+        nets.Add( netName, netCode );
 
     auto netProperty = m_propMgr.GetProperty( TYPE_HASH( BOARD_CONNECTED_ITEM ), _HKI( "Net" ) );
     netProperty->SetChoices( nets );
+
+    // Regenerate font names
+    std::vector<std::string> fontNames;
+    Fontconfig()->ListFonts( fontNames, std::string( Pgm().GetLanguageTag().utf8_str() ),
+                             aBoard->GetFontFiles() );
+
+    fonts.Add( KICAD_FONT_NAME, -1 );
+
+    for( int ii = 0; ii < (int) fontNames.size(); ++ii )
+        fonts.Add( wxString( fontNames[ii] ), ii );
+
+    auto fontProperty = m_propMgr.GetProperty( TYPE_HASH( EDA_TEXT ), _HKI( "Font" ) );
+    fontProperty->SetChoices( fonts );
 }
