@@ -28,6 +28,10 @@
 
 // Author: Alfred Peng
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>  // Must come first
+#endif
+
 #include <fcntl.h>
 #include <sys/frame.h>
 #include <sys/stat.h>
